/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2015-2020 Philippe Proulx <pproulx@efficios.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 * The following code was generated by barectf v3.1.1
 * on 2024-11-24T14:51:03.572448.
 *
 * For more details, see <https://barectf.org/>.
 */

#include <stdint.h>
#include <string.h>
#include <assert.h>

#include "barectf.h"
#include "barectf-bitfield.h"

#define _ALIGN(_at_var, _align)						\
	do {								\
		(_at_var) = ((_at_var) + ((_align) - 1)) & -(_align);	\
	} while (0)

#ifdef __cplusplus
# define _TO_VOID_PTR(_value)		static_cast<void *>(_value)
# define _FROM_VOID_PTR(_type, _value)	static_cast<_type *>(_value)
#else
# define _TO_VOID_PTR(_value)		((void *) (_value))
# define _FROM_VOID_PTR(_type, _value)	((_type *) (_value))
#endif

#define _BITS_TO_BYTES(_x)	((_x) >> 3)
#define _BYTES_TO_BITS(_x)	((_x) << 3)

union _f2u {
	float f;
	uint32_t u;
};

union _d2u {
	double f;
	uint64_t u;
};

uint32_t barectf_packet_size(const void * const vctx)
{
	return _FROM_VOID_PTR(const struct barectf_ctx, vctx)->packet_size;
}

int barectf_packet_is_full(const void * const vctx)
{
	const struct barectf_ctx * const ctx = _FROM_VOID_PTR(const struct barectf_ctx, vctx);

	return ctx->at == ctx->packet_size;
}

int barectf_packet_is_empty(const void * const vctx)
{
	const struct barectf_ctx * const ctx = _FROM_VOID_PTR(const struct barectf_ctx, vctx);

	return ctx->at <= ctx->off_content;
}

uint32_t barectf_packet_events_discarded(const void * const vctx)
{
	return _FROM_VOID_PTR(const struct barectf_ctx, vctx)->events_discarded;
}

uint32_t barectf_discarded_event_records_count(const void * const vctx)
{
	return barectf_packet_events_discarded(vctx);
}

uint32_t barectf_packet_sequence_number(const void * const vctx)
{
	return _FROM_VOID_PTR(const struct barectf_ctx, vctx)->sequence_number;
}

uint8_t *barectf_packet_buf(const void * const vctx)
{
	return _FROM_VOID_PTR(const struct barectf_ctx, vctx)->buf;
}

uint8_t *barectf_packet_buf_addr(const void * const vctx)
{
	return barectf_packet_buf(vctx);
}

uint32_t barectf_packet_buf_size(const void * const vctx)
{
	const struct barectf_ctx * const ctx = _FROM_VOID_PTR(const struct barectf_ctx, vctx);

	return _BITS_TO_BYTES(ctx->packet_size);
}

void barectf_packet_set_buf(void * const vctx, uint8_t * const buf,
		const uint32_t buf_size)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	ctx->buf = buf;

	if (ctx->at == ctx->packet_size) {
		/* Keep full packet state */
		ctx->at = _BYTES_TO_BITS(buf_size);
	}

	ctx->packet_size = _BYTES_TO_BITS(buf_size);
}

int barectf_packet_is_open(const void * const vctx)
{
	return _FROM_VOID_PTR(const struct barectf_ctx, vctx)->packet_is_open;
}

int barectf_is_in_tracing_section(const void * const vctx)
{
	return _FROM_VOID_PTR(const struct barectf_ctx, vctx)->in_tracing_section;
}

volatile const int *barectf_is_in_tracing_section_ptr(const void * const vctx)
{
	return &_FROM_VOID_PTR(const struct barectf_ctx, vctx)->in_tracing_section;
}

int barectf_is_tracing_enabled(const void * const vctx)
{
	return _FROM_VOID_PTR(const struct barectf_ctx, vctx)->is_tracing_enabled;
}

void barectf_enable_tracing(void * const vctx, const int enable)
{
	_FROM_VOID_PTR(struct barectf_ctx, vctx)->is_tracing_enabled = enable;
}

static
void _write_c_str(struct barectf_ctx * const ctx, const char * const src)
{
	const uint32_t sz = strlen(src) + 1;

	memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], src, sz);
	ctx->at += _BYTES_TO_BITS(sz);
}

static
int _reserve_er_space(void * const vctx, const uint32_t er_size)
{
	int ret;
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Event _cannot_ fit? */
	if (er_size > (ctx->packet_size - ctx->off_content)) {
		goto no_space;
	}

	/* Packet is full? */
	if (barectf_packet_is_full(ctx)) {
		/* Yes: is the back end full? */
		if (ctx->cbs.is_backend_full(ctx->data)) {
			/* Yes: discard event record */
			goto no_space;
		}

		/* Back-end is _not_ full: open new packet */
		ctx->use_cur_last_event_ts = 1;
		ctx->cbs.open_packet(ctx->data);
		ctx->use_cur_last_event_ts = 0;
	}

	/* Event fits the current packet? */
	if (er_size > (ctx->packet_size - ctx->at)) {
		/* No: close packet now */
		ctx->use_cur_last_event_ts = 1;
		ctx->cbs.close_packet(ctx->data);
		ctx->use_cur_last_event_ts = 0;

		/* Is the back end full? */
		if (ctx->cbs.is_backend_full(ctx->data)) {
			/* Yes: discard event record */
			goto no_space;
		}

		/* Back-end is _not_ full: open new packet */
		ctx->use_cur_last_event_ts = 1;
		ctx->cbs.open_packet(ctx->data);
		ctx->use_cur_last_event_ts = 0;
		assert(er_size <= (ctx->packet_size - ctx->at));
	}

	ret = 1;
	goto end;

no_space:
	ctx->events_discarded++;
	ret = 0;

end:
	return ret;
}

static
void _commit_er(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Is the packet full? */
	if (barectf_packet_is_full(ctx)) {
		/* Yes: close it now */
		ctx->cbs.close_packet(ctx->data);
	}
}

/* Initialize context */
void barectf_init(void *vctx,
	uint8_t * const buf, const uint32_t buf_size,
	const struct barectf_platform_callbacks cbs, void * const data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	ctx->cbs = cbs;
	ctx->data = data;
	ctx->buf = buf;
	ctx->packet_size = _BYTES_TO_BITS(buf_size);
	ctx->at = 0;
	ctx->events_discarded = 0;
	ctx->sequence_number = 0;
	ctx->packet_is_open = 0;
	ctx->in_tracing_section = 0;
	ctx->is_tracing_enabled = 1;
	ctx->use_cur_last_event_ts = 0;
}

/* Open packet for data stream type `api_ops` */
void barectf_api_ops_open_packet(
	struct barectf_api_ops_ctx * const sctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	const int saved_in_tracing_section = ctx->in_tracing_section;

	/*
	 * This function is either called by a tracing function, or
	 * directly by the platform.
	 *
	 * If it's called by a tracing function, then
	 * `ctx->in_tracing_section` is 1, so it's safe to open
	 * the packet here (alter the packet), even if tracing was
	 * disabled in the meantime because we're already in a tracing
	 * section (which finishes at the end of the tracing function
	 * call).
	 *
	 * If it's called directly by the platform, then if tracing is
	 * disabled, we don't want to alter the packet, and return
	 * immediately.
	 */
	if (!ctx->is_tracing_enabled && !saved_in_tracing_section) {
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Do not open a packet that is already open */
	if (ctx->packet_is_open) {
		ctx->in_tracing_section = saved_in_tracing_section;
		goto end;
	}

	ctx->at = 0;

	/* Write packet header structure */
	{
		/* Align for packet header structure */
		_ALIGN(ctx->at, 8);

		/* Align for `magic` field */
		_ALIGN(ctx->at, 8);

		/* Write magic number field */
		{
			const uint32_t tmp_val = (uint32_t) 0xc1fc1fc1UL;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `stream_id` field */
		_ALIGN(ctx->at, 8);

		/* Write data stream type ID field */
		{
			const uint64_t tmp_val = (uint64_t) 0;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}

	/* Write packet context structure */
	{
		/* Align for packet context structure */
		_ALIGN(ctx->at, 8);

		/* Align for `packet_size` field */
		_ALIGN(ctx->at, 8);

		/* Write packet total size field */
		{
			const uint64_t tmp_val = (uint64_t) ctx->packet_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `content_size` field */
		_ALIGN(ctx->at, 8);

		/* Do not write `content_size` field; save its offset */
		sctx->off_pc_content_size = ctx->at;
		ctx->at += 64;
	}

	/* Save content beginning's offset */
	ctx->off_content = ctx->at;

	/* Mark current packet as open */
	ctx->packet_is_open = 1;

	/* Not tracing anymore */
	ctx->in_tracing_section = saved_in_tracing_section;

end:
	return;
}

/* Close packet for data stream type `api_ops` */
void barectf_api_ops_close_packet(struct barectf_api_ops_ctx * const sctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	const int saved_in_tracing_section = ctx->in_tracing_section;

	/*
	 * This function is either called by a tracing function, or
	 * directly by the platform.
	 *
	 * If it's called by a tracing function, then
	 * `ctx->in_tracing_section` is 1, so it's safe to close
	 * the packet here (alter the packet), even if tracing was
	 * disabled in the meantime, because we're already in a tracing
	 * section (which finishes at the end of the tracing function
	 * call).
	 *
	 * If it's called directly by the platform, then if tracing is
	 * disabled, we don't want to alter the packet, and return
	 * immediately.
	 */
	if (!ctx->is_tracing_enabled && !saved_in_tracing_section) {
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Do not close a packet that is not open */
	if (!ctx->packet_is_open) {
		ctx->in_tracing_section = saved_in_tracing_section;
		goto end;
	}

	/* Save content size */
	ctx->content_size = ctx->at;

	/* Go back to `content_size` field offset */
	ctx->at = sctx->off_pc_content_size;

	/* Write `content_size` field */
	{
		const uint64_t tmp_val = (uint64_t) ctx->content_size;

		memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
		ctx->at += 64;
	}

	/* Go back to end of packet */
	ctx->at = ctx->packet_size;

	/* Mark packet as closed */
	ctx->packet_is_open = 0;

	/* Not tracing anymore */
	ctx->in_tracing_section = saved_in_tracing_section;

end:
	return;
}

static void _serialize_er_header_api_ops(void * const vctx,
	const uint32_t ert_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	struct barectf_api_ops_ctx * const sctx = _FROM_VOID_PTR(struct barectf_api_ops_ctx, vctx);
	const uint64_t ts = sctx->cur_last_event_ts;

	/* Write header structure */
	{
		/* Align for header structure */
		_ALIGN(ctx->at, 8);

		/* Align for `id` field */
		_ALIGN(ctx->at, 8);

		/* Write event record type ID field */
		{
			const uint64_t tmp_val = (uint64_t) ert_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `timestamp` field */
		_ALIGN(ctx->at, 8);

		/* Write timestamp field */
		{
			const uint64_t tmp_val = (uint64_t) ts;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_common_ctx_api_ops(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Write common context structure */
	{
		/* Align for common context structure */
		_ALIGN(ctx->at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(ctx->at, 32);

		/* Write `_thread_id` field */
		{
			const uint32_t tmp_val = (uint32_t) cc__thread_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_queue_id` field */
		_ALIGN(ctx->at, 32);

		/* Write `_queue_id` field */
		{
			const uint32_t tmp_val = (uint32_t) cc__queue_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_agent_id` field */
		_ALIGN(ctx->at, 32);

		/* Write `_agent_id` field */
		{
			const uint32_t tmp_val = (uint32_t) cc__agent_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_correlation_id` field */
		_ALIGN(ctx->at, 64);

		/* Write `_correlation_id` field */
		{
			const uint64_t tmp_val = (uint64_t) cc__correlation_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_api_ops_hip_op_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const p__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_api_ops(ctx, 0);

	/* Serialize common context */
	_serialize_er_common_ctx_api_ops(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_kernel_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kernel_name` field */
		_write_c_str(ctx, p__kernel_name);
	}
}

static void _serialize_er_api_ops_hip_op_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_api_ops(ctx, 1);

	/* Serialize common context */
	_serialize_er_common_ctx_api_ops(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_api_ops_hsa_op_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_api_ops(ctx, 2);

	/* Serialize common context */
	_serialize_er_common_ctx_api_ops(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_api_ops_hsa_op_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_api_ops(ctx, 3);

	/* Serialize common context */
	_serialize_er_common_ctx_api_ops(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static uint32_t _er_size_api_ops_hip_op_begin(void * const vctx,
	const char * const p__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__kernel_name) + 1);
	}

	return at - ctx->at;
}

static uint32_t _er_size_api_ops_hip_op_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_api_ops_hsa_op_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_api_ops_hsa_op_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

/* Trace (data stream type `api_ops`, event record type `hip_op_begin`) */
void barectf_api_ops_trace_hip_op_begin(struct barectf_api_ops_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const p__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_api_ops_hip_op_begin(_TO_VOID_PTR(ctx), p__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_api_ops_hip_op_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `api_ops`, event record type `hip_op_end`) */
void barectf_api_ops_trace_hip_op_end(struct barectf_api_ops_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_api_ops_hip_op_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_api_ops_hip_op_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `api_ops`, event record type `hsa_op_begin`) */
void barectf_api_ops_trace_hsa_op_begin(struct barectf_api_ops_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_api_ops_hsa_op_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_api_ops_hsa_op_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `api_ops`, event record type `hsa_op_end`) */
void barectf_api_ops_trace_hsa_op_end(struct barectf_api_ops_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_api_ops_hsa_op_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_api_ops_hsa_op_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}
/* Open packet for data stream type `hip_api` */
void barectf_hip_api_open_packet(
	struct barectf_hip_api_ctx * const sctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	const int saved_in_tracing_section = ctx->in_tracing_section;

	/*
	 * This function is either called by a tracing function, or
	 * directly by the platform.
	 *
	 * If it's called by a tracing function, then
	 * `ctx->in_tracing_section` is 1, so it's safe to open
	 * the packet here (alter the packet), even if tracing was
	 * disabled in the meantime because we're already in a tracing
	 * section (which finishes at the end of the tracing function
	 * call).
	 *
	 * If it's called directly by the platform, then if tracing is
	 * disabled, we don't want to alter the packet, and return
	 * immediately.
	 */
	if (!ctx->is_tracing_enabled && !saved_in_tracing_section) {
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Do not open a packet that is already open */
	if (ctx->packet_is_open) {
		ctx->in_tracing_section = saved_in_tracing_section;
		goto end;
	}

	ctx->at = 0;

	/* Write packet header structure */
	{
		/* Align for packet header structure */
		_ALIGN(ctx->at, 8);

		/* Align for `magic` field */
		_ALIGN(ctx->at, 8);

		/* Write magic number field */
		{
			const uint32_t tmp_val = (uint32_t) 0xc1fc1fc1UL;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `stream_id` field */
		_ALIGN(ctx->at, 8);

		/* Write data stream type ID field */
		{
			const uint64_t tmp_val = (uint64_t) 1;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}

	/* Write packet context structure */
	{
		/* Align for packet context structure */
		_ALIGN(ctx->at, 8);

		/* Align for `packet_size` field */
		_ALIGN(ctx->at, 8);

		/* Write packet total size field */
		{
			const uint64_t tmp_val = (uint64_t) ctx->packet_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `content_size` field */
		_ALIGN(ctx->at, 8);

		/* Do not write `content_size` field; save its offset */
		sctx->off_pc_content_size = ctx->at;
		ctx->at += 64;
	}

	/* Save content beginning's offset */
	ctx->off_content = ctx->at;

	/* Mark current packet as open */
	ctx->packet_is_open = 1;

	/* Not tracing anymore */
	ctx->in_tracing_section = saved_in_tracing_section;

end:
	return;
}

/* Close packet for data stream type `hip_api` */
void barectf_hip_api_close_packet(struct barectf_hip_api_ctx * const sctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	const int saved_in_tracing_section = ctx->in_tracing_section;

	/*
	 * This function is either called by a tracing function, or
	 * directly by the platform.
	 *
	 * If it's called by a tracing function, then
	 * `ctx->in_tracing_section` is 1, so it's safe to close
	 * the packet here (alter the packet), even if tracing was
	 * disabled in the meantime, because we're already in a tracing
	 * section (which finishes at the end of the tracing function
	 * call).
	 *
	 * If it's called directly by the platform, then if tracing is
	 * disabled, we don't want to alter the packet, and return
	 * immediately.
	 */
	if (!ctx->is_tracing_enabled && !saved_in_tracing_section) {
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Do not close a packet that is not open */
	if (!ctx->packet_is_open) {
		ctx->in_tracing_section = saved_in_tracing_section;
		goto end;
	}

	/* Save content size */
	ctx->content_size = ctx->at;

	/* Go back to `content_size` field offset */
	ctx->at = sctx->off_pc_content_size;

	/* Write `content_size` field */
	{
		const uint64_t tmp_val = (uint64_t) ctx->content_size;

		memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
		ctx->at += 64;
	}

	/* Go back to end of packet */
	ctx->at = ctx->packet_size;

	/* Mark packet as closed */
	ctx->packet_is_open = 0;

	/* Not tracing anymore */
	ctx->in_tracing_section = saved_in_tracing_section;

end:
	return;
}

static void _serialize_er_header_hip_api(void * const vctx,
	const uint32_t ert_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	struct barectf_hip_api_ctx * const sctx = _FROM_VOID_PTR(struct barectf_hip_api_ctx, vctx);
	const uint64_t ts = sctx->cur_last_event_ts;

	/* Write header structure */
	{
		/* Align for header structure */
		_ALIGN(ctx->at, 8);

		/* Align for `id` field */
		_ALIGN(ctx->at, 8);

		/* Write event record type ID field */
		{
			const uint64_t tmp_val = (uint64_t) ert_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `timestamp` field */
		_ALIGN(ctx->at, 8);

		/* Write timestamp field */
		{
			const uint64_t tmp_val = (uint64_t) ts;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_common_ctx_hip_api(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Write common context structure */
	{
		/* Align for common context structure */
		_ALIGN(ctx->at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(ctx->at, 32);

		/* Write `_thread_id` field */
		{
			const uint32_t tmp_val = (uint32_t) cc__thread_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_queue_id` field */
		_ALIGN(ctx->at, 32);

		/* Write `_queue_id` field */
		{
			const uint32_t tmp_val = (uint32_t) cc__queue_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_agent_id` field */
		_ALIGN(ctx->at, 32);

		/* Write `_agent_id` field */
		{
			const uint32_t tmp_val = (uint32_t) cc__agent_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_correlation_id` field */
		_ALIGN(ctx->at, 64);

		/* Write `_correlation_id` field */
		{
			const uint64_t tmp_val = (uint64_t) cc__correlation_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kernel_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kernel_name` field */
		_write_c_str(ctx, cc__kernel_name);
	}
}

static void _serialize_er_hip_api___hipPopCallConfigurationBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__gridDim,
	const uint32_t p__gridDim__val__x,
	const uint32_t p__gridDim__val__y,
	const uint32_t p__gridDim__val__z,
	const uint64_t p__blockDim,
	const uint32_t p__blockDim__val__x,
	const uint32_t p__blockDim__val__y,
	const uint32_t p__blockDim__val__z,
	const uint64_t p__sharedMem,
	const uint64_t p__sharedMem__val,
	const uint64_t p__stream,
	const uint64_t p__stream__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 0);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_gridDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDim` field */
		{
			const uint64_t tmp_val = (uint64_t) p__gridDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_gridDim__val__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDim__val__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDim__val__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_gridDim__val__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDim__val__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDim__val__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_gridDim__val__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDim__val__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDim__val__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDim` field */
		{
			const uint64_t tmp_val = (uint64_t) p__blockDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_blockDim__val__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDim__val__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDim__val__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDim__val__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDim__val__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDim__val__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDim__val__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDim__val__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDim__val__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_sharedMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sharedMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sharedMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sharedMem__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sharedMem__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sharedMem__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api___hipPopCallConfigurationEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 1);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api___hipPushCallConfigurationBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint32_t p__gridDim__x,
	const uint32_t p__gridDim__y,
	const uint32_t p__gridDim__z,
	const uint32_t p__blockDim__x,
	const uint32_t p__blockDim__y,
	const uint32_t p__blockDim__z,
	const uint64_t p__sharedMem,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 2);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_gridDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_gridDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_gridDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_sharedMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sharedMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sharedMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api___hipPushCallConfigurationEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 3);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipArray3DCreateBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__array,
	const uint64_t p__array__val,
	const uint64_t p__pAllocateArray,
	const uint64_t p__pAllocateArray__val__Width,
	const uint64_t p__pAllocateArray__val__Height,
	const uint64_t p__pAllocateArray__val__Depth,
	const int32_t p__pAllocateArray__val__Format,
	const uint32_t p__pAllocateArray__val__NumChannels,
	const uint32_t p__pAllocateArray__val__Flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 4);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_array` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_array__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pAllocateArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pAllocateArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pAllocateArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pAllocateArray__val__Width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pAllocateArray__val__Width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pAllocateArray__val__Width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pAllocateArray__val__Height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pAllocateArray__val__Height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pAllocateArray__val__Height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pAllocateArray__val__Depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pAllocateArray__val__Depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pAllocateArray__val__Depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pAllocateArray__val__Format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pAllocateArray__val__Format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pAllocateArray__val__Format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pAllocateArray__val__NumChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pAllocateArray__val__NumChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pAllocateArray__val__NumChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pAllocateArray__val__Flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pAllocateArray__val__Flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pAllocateArray__val__Flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipArray3DCreateEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 5);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipArray3DGetDescriptorBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pArrayDescriptor,
	const uint64_t p__pArrayDescriptor__val__Width,
	const uint64_t p__pArrayDescriptor__val__Height,
	const uint64_t p__pArrayDescriptor__val__Depth,
	const int32_t p__pArrayDescriptor__val__Format,
	const uint32_t p__pArrayDescriptor__val__NumChannels,
	const uint32_t p__pArrayDescriptor__val__Flags,
	const uint64_t p__array)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 6);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pArrayDescriptor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArrayDescriptor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pArrayDescriptor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pArrayDescriptor__val__Width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArrayDescriptor__val__Width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pArrayDescriptor__val__Width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pArrayDescriptor__val__Height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArrayDescriptor__val__Height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pArrayDescriptor__val__Height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pArrayDescriptor__val__Depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArrayDescriptor__val__Depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pArrayDescriptor__val__Depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pArrayDescriptor__val__Format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArrayDescriptor__val__Format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pArrayDescriptor__val__Format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pArrayDescriptor__val__NumChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArrayDescriptor__val__NumChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pArrayDescriptor__val__NumChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pArrayDescriptor__val__Flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArrayDescriptor__val__Flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pArrayDescriptor__val__Flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_array` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipArray3DGetDescriptorEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 7);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipArrayCreateBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pHandle,
	const uint64_t p__pHandle__val,
	const uint64_t p__pAllocateArray,
	const uint64_t p__pAllocateArray__val__Width,
	const uint64_t p__pAllocateArray__val__Height,
	const int32_t p__pAllocateArray__val__Format,
	const uint32_t p__pAllocateArray__val__NumChannels)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 8);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pHandle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pHandle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pHandle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pHandle__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pHandle__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pHandle__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pAllocateArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pAllocateArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pAllocateArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pAllocateArray__val__Width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pAllocateArray__val__Width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pAllocateArray__val__Width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pAllocateArray__val__Height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pAllocateArray__val__Height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pAllocateArray__val__Height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pAllocateArray__val__Format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pAllocateArray__val__Format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pAllocateArray__val__Format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pAllocateArray__val__NumChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pAllocateArray__val__NumChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pAllocateArray__val__NumChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipArrayCreateEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 9);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipArrayDestroyBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__array)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 10);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_array` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipArrayDestroyEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 11);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipArrayGetDescriptorBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pArrayDescriptor,
	const uint64_t p__pArrayDescriptor__val__Width,
	const uint64_t p__pArrayDescriptor__val__Height,
	const int32_t p__pArrayDescriptor__val__Format,
	const uint32_t p__pArrayDescriptor__val__NumChannels,
	const uint64_t p__array)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 12);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pArrayDescriptor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArrayDescriptor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pArrayDescriptor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pArrayDescriptor__val__Width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArrayDescriptor__val__Width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pArrayDescriptor__val__Width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pArrayDescriptor__val__Height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArrayDescriptor__val__Height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pArrayDescriptor__val__Height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pArrayDescriptor__val__Format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArrayDescriptor__val__Format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pArrayDescriptor__val__Format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pArrayDescriptor__val__NumChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArrayDescriptor__val__NumChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pArrayDescriptor__val__NumChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_array` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipArrayGetDescriptorEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 13);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipArrayGetInfoBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__desc,
	const int32_t p__desc__val__x,
	const int32_t p__desc__val__y,
	const int32_t p__desc__val__z,
	const int32_t p__desc__val__w,
	const int32_t p__desc__val__f,
	const uint64_t p__extent,
	const uint64_t p__extent__val__width,
	const uint64_t p__extent__val__height,
	const uint64_t p__extent__val__depth,
	const uint64_t p__flags,
	const uint32_t p__flags__val,
	const uint64_t p__array)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 14);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_desc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__desc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc__val__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_extent` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__val__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__val__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__val__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__val__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__val__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__val__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__val__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__val__depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__val__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_array` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipArrayGetInfoEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 15);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipChooseDeviceBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 16);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipChooseDeviceEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 17);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipChooseDeviceR0000Begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__device,
	const int32_t p__device__val,
	const uint64_t p__prop,
	const uint64_t p__prop__val__name,
	const uint64_t p__prop__val__totalGlobalMem,
	const uint64_t p__prop__val__sharedMemPerBlock,
	const int32_t p__prop__val__regsPerBlock,
	const int32_t p__prop__val__warpSize,
	const int32_t p__prop__val__maxThreadsPerBlock,
	const uint64_t p__prop__val__maxThreadsDim,
	const uint64_t p__prop__val__maxGridSize,
	const int32_t p__prop__val__clockRate,
	const int32_t p__prop__val__memoryClockRate,
	const int32_t p__prop__val__memoryBusWidth,
	const uint64_t p__prop__val__totalConstMem,
	const int32_t p__prop__val__major,
	const int32_t p__prop__val__minor,
	const int32_t p__prop__val__multiProcessorCount,
	const int32_t p__prop__val__l2CacheSize,
	const int32_t p__prop__val__maxThreadsPerMultiProcessor,
	const int32_t p__prop__val__computeMode,
	const int32_t p__prop__val__clockInstructionRate,
	const uint32_t p__prop__val__arch__hasGlobalInt32Atomics,
	const uint32_t p__prop__val__arch__hasGlobalFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasSharedInt32Atomics,
	const uint32_t p__prop__val__arch__hasSharedFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasFloatAtomicAdd,
	const uint32_t p__prop__val__arch__hasGlobalInt64Atomics,
	const uint32_t p__prop__val__arch__hasSharedInt64Atomics,
	const uint32_t p__prop__val__arch__hasDoubles,
	const uint32_t p__prop__val__arch__hasWarpVote,
	const uint32_t p__prop__val__arch__hasWarpBallot,
	const uint32_t p__prop__val__arch__hasWarpShuffle,
	const uint32_t p__prop__val__arch__hasFunnelShift,
	const uint32_t p__prop__val__arch__hasThreadFenceSystem,
	const uint32_t p__prop__val__arch__hasSyncThreadsExt,
	const uint32_t p__prop__val__arch__hasSurfaceFuncs,
	const uint32_t p__prop__val__arch__has3dGrid,
	const uint32_t p__prop__val__arch__hasDynamicParallelism,
	const int32_t p__prop__val__concurrentKernels,
	const int32_t p__prop__val__pciDomainID,
	const int32_t p__prop__val__pciBusID,
	const int32_t p__prop__val__pciDeviceID,
	const uint64_t p__prop__val__maxSharedMemoryPerMultiProcessor,
	const int32_t p__prop__val__isMultiGpuBoard,
	const int32_t p__prop__val__canMapHostMemory,
	const int32_t p__prop__val__gcnArch,
	const uint64_t p__prop__val__gcnArchName,
	const int32_t p__prop__val__integrated,
	const int32_t p__prop__val__cooperativeLaunch,
	const int32_t p__prop__val__cooperativeMultiDeviceLaunch,
	const int32_t p__prop__val__maxTexture1DLinear,
	const int32_t p__prop__val__maxTexture1D,
	const uint64_t p__prop__val__maxTexture2D,
	const uint64_t p__prop__val__maxTexture3D,
	const uint64_t p__prop__val__hdpMemFlushCntl,
	const uint64_t p__prop__val__hdpRegFlushCntl,
	const uint64_t p__prop__val__memPitch,
	const uint64_t p__prop__val__textureAlignment,
	const uint64_t p__prop__val__texturePitchAlignment,
	const int32_t p__prop__val__kernelExecTimeoutEnabled,
	const int32_t p__prop__val__ECCEnabled,
	const int32_t p__prop__val__tccDriver,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedFunc,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedGridDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem,
	const int32_t p__prop__val__isLargeBar,
	const int32_t p__prop__val__asicRevision,
	const int32_t p__prop__val__managedMemory,
	const int32_t p__prop__val__directManagedMemAccessFromHost,
	const int32_t p__prop__val__concurrentManagedAccess,
	const int32_t p__prop__val__pageableMemoryAccess,
	const int32_t p__prop__val__pageableMemoryAccessUsesHostPageTables)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 18);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint64_t tmp_val = (uint64_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_device__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__name` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__name;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__totalGlobalMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__totalGlobalMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__totalGlobalMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__sharedMemPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__sharedMemPerBlock` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__sharedMemPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__regsPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__regsPerBlock` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__regsPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__warpSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__warpSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__warpSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxThreadsPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxThreadsPerBlock` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxThreadsPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxThreadsDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxThreadsDim` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxThreadsDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxGridSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxGridSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxGridSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__clockRate` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__clockRate` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__clockRate;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memoryClockRate` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memoryClockRate` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__memoryClockRate;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memoryBusWidth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memoryBusWidth` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__memoryBusWidth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__totalConstMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__totalConstMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__totalConstMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__major` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__major` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__major;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__minor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__minor` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__minor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__multiProcessorCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__multiProcessorCount` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__multiProcessorCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__l2CacheSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__l2CacheSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__l2CacheSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxThreadsPerMultiProcessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxThreadsPerMultiProcessor` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxThreadsPerMultiProcessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__computeMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__computeMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__computeMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__clockInstructionRate` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__clockInstructionRate` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__clockInstructionRate;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasGlobalInt32Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasGlobalInt32Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasGlobalInt32Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasGlobalFloatAtomicExch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasGlobalFloatAtomicExch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasGlobalFloatAtomicExch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSharedInt32Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSharedInt32Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSharedInt32Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSharedFloatAtomicExch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSharedFloatAtomicExch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSharedFloatAtomicExch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasFloatAtomicAdd` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasFloatAtomicAdd` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasFloatAtomicAdd;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasGlobalInt64Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasGlobalInt64Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasGlobalInt64Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSharedInt64Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSharedInt64Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSharedInt64Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasDoubles` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasDoubles` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasDoubles;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasWarpVote` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasWarpVote` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasWarpVote;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasWarpBallot` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasWarpBallot` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasWarpBallot;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasWarpShuffle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasWarpShuffle` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasWarpShuffle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasFunnelShift` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasFunnelShift` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasFunnelShift;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasThreadFenceSystem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasThreadFenceSystem` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasThreadFenceSystem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSyncThreadsExt` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSyncThreadsExt` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSyncThreadsExt;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSurfaceFuncs` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSurfaceFuncs` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSurfaceFuncs;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__has3dGrid` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__has3dGrid` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__has3dGrid;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasDynamicParallelism` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasDynamicParallelism` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasDynamicParallelism;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__concurrentKernels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__concurrentKernels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__concurrentKernels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pciDomainID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pciDomainID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pciDomainID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pciBusID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pciBusID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pciBusID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pciDeviceID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pciDeviceID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pciDeviceID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxSharedMemoryPerMultiProcessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSharedMemoryPerMultiProcessor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSharedMemoryPerMultiProcessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__isMultiGpuBoard` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__isMultiGpuBoard` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__isMultiGpuBoard;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__canMapHostMemory` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__canMapHostMemory` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__canMapHostMemory;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__gcnArch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__gcnArch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__gcnArch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__gcnArchName` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__gcnArchName` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__gcnArchName;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__integrated` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__integrated` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__integrated;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeLaunch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeLaunch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeLaunch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceLaunch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceLaunch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceLaunch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture1DLinear` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture1DLinear` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxTexture1DLinear;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture1D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture1D` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxTexture1D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture2D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture2D` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture2D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture3D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture3D` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture3D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__hdpMemFlushCntl` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hdpMemFlushCntl` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__hdpMemFlushCntl;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__hdpRegFlushCntl` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hdpRegFlushCntl` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__hdpRegFlushCntl;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__memPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__memPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__textureAlignment` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__textureAlignment` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__textureAlignment;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__texturePitchAlignment` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__texturePitchAlignment` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__texturePitchAlignment;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__kernelExecTimeoutEnabled` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__kernelExecTimeoutEnabled` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__kernelExecTimeoutEnabled;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__ECCEnabled` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__ECCEnabled` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__ECCEnabled;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__tccDriver` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__tccDriver` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__tccDriver;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedFunc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedFunc` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedFunc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedGridDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__isLargeBar` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__isLargeBar` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__isLargeBar;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__asicRevision` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__asicRevision` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__asicRevision;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__managedMemory` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__managedMemory` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__managedMemory;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__directManagedMemAccessFromHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__directManagedMemAccessFromHost` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__directManagedMemAccessFromHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__concurrentManagedAccess` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__concurrentManagedAccess` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__concurrentManagedAccess;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pageableMemoryAccess` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pageableMemoryAccess` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pageableMemoryAccess;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pageableMemoryAccessUsesHostPageTables` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pageableMemoryAccessUsesHostPageTables` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pageableMemoryAccessUsesHostPageTables;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipChooseDeviceR0000End(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 19);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipChooseDeviceR0600Begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__device,
	const int32_t p__device__val,
	const uint64_t p__prop,
	const uint64_t p__prop__val__name,
	const uint64_t p__prop__val__uuid__bytes,
	const uint64_t p__prop__val__luid,
	const uint32_t p__prop__val__luidDeviceNodeMask,
	const uint64_t p__prop__val__totalGlobalMem,
	const uint64_t p__prop__val__sharedMemPerBlock,
	const int32_t p__prop__val__regsPerBlock,
	const int32_t p__prop__val__warpSize,
	const uint64_t p__prop__val__memPitch,
	const int32_t p__prop__val__maxThreadsPerBlock,
	const uint64_t p__prop__val__maxThreadsDim,
	const uint64_t p__prop__val__maxGridSize,
	const int32_t p__prop__val__clockRate,
	const uint64_t p__prop__val__totalConstMem,
	const int32_t p__prop__val__major,
	const int32_t p__prop__val__minor,
	const uint64_t p__prop__val__textureAlignment,
	const uint64_t p__prop__val__texturePitchAlignment,
	const int32_t p__prop__val__deviceOverlap,
	const int32_t p__prop__val__multiProcessorCount,
	const int32_t p__prop__val__kernelExecTimeoutEnabled,
	const int32_t p__prop__val__integrated,
	const int32_t p__prop__val__canMapHostMemory,
	const int32_t p__prop__val__computeMode,
	const int32_t p__prop__val__maxTexture1D,
	const int32_t p__prop__val__maxTexture1DMipmap,
	const int32_t p__prop__val__maxTexture1DLinear,
	const uint64_t p__prop__val__maxTexture2D,
	const uint64_t p__prop__val__maxTexture2DMipmap,
	const uint64_t p__prop__val__maxTexture2DLinear,
	const uint64_t p__prop__val__maxTexture2DGather,
	const uint64_t p__prop__val__maxTexture3D,
	const uint64_t p__prop__val__maxTexture3DAlt,
	const int32_t p__prop__val__maxTextureCubemap,
	const uint64_t p__prop__val__maxTexture1DLayered,
	const uint64_t p__prop__val__maxTexture2DLayered,
	const uint64_t p__prop__val__maxTextureCubemapLayered,
	const int32_t p__prop__val__maxSurface1D,
	const uint64_t p__prop__val__maxSurface2D,
	const uint64_t p__prop__val__maxSurface3D,
	const uint64_t p__prop__val__maxSurface1DLayered,
	const uint64_t p__prop__val__maxSurface2DLayered,
	const int32_t p__prop__val__maxSurfaceCubemap,
	const uint64_t p__prop__val__maxSurfaceCubemapLayered,
	const uint64_t p__prop__val__surfaceAlignment,
	const int32_t p__prop__val__concurrentKernels,
	const int32_t p__prop__val__ECCEnabled,
	const int32_t p__prop__val__pciBusID,
	const int32_t p__prop__val__pciDeviceID,
	const int32_t p__prop__val__pciDomainID,
	const int32_t p__prop__val__tccDriver,
	const int32_t p__prop__val__asyncEngineCount,
	const int32_t p__prop__val__unifiedAddressing,
	const int32_t p__prop__val__memoryClockRate,
	const int32_t p__prop__val__memoryBusWidth,
	const int32_t p__prop__val__l2CacheSize,
	const int32_t p__prop__val__persistingL2CacheMaxSize,
	const int32_t p__prop__val__maxThreadsPerMultiProcessor,
	const int32_t p__prop__val__streamPrioritiesSupported,
	const int32_t p__prop__val__globalL1CacheSupported,
	const int32_t p__prop__val__localL1CacheSupported,
	const uint64_t p__prop__val__sharedMemPerMultiprocessor,
	const int32_t p__prop__val__regsPerMultiprocessor,
	const int32_t p__prop__val__managedMemory,
	const int32_t p__prop__val__isMultiGpuBoard,
	const int32_t p__prop__val__multiGpuBoardGroupID,
	const int32_t p__prop__val__hostNativeAtomicSupported,
	const int32_t p__prop__val__singleToDoublePrecisionPerfRatio,
	const int32_t p__prop__val__pageableMemoryAccess,
	const int32_t p__prop__val__concurrentManagedAccess,
	const int32_t p__prop__val__computePreemptionSupported,
	const int32_t p__prop__val__canUseHostPointerForRegisteredMem,
	const int32_t p__prop__val__cooperativeLaunch,
	const int32_t p__prop__val__cooperativeMultiDeviceLaunch,
	const uint64_t p__prop__val__sharedMemPerBlockOptin,
	const int32_t p__prop__val__pageableMemoryAccessUsesHostPageTables,
	const int32_t p__prop__val__directManagedMemAccessFromHost,
	const int32_t p__prop__val__maxBlocksPerMultiProcessor,
	const int32_t p__prop__val__accessPolicyMaxWindowSize,
	const uint64_t p__prop__val__reservedSharedMemPerBlock,
	const int32_t p__prop__val__hostRegisterSupported,
	const int32_t p__prop__val__sparseHipArraySupported,
	const int32_t p__prop__val__hostRegisterReadOnlySupported,
	const int32_t p__prop__val__timelineSemaphoreInteropSupported,
	const int32_t p__prop__val__memoryPoolsSupported,
	const int32_t p__prop__val__gpuDirectRDMASupported,
	const uint32_t p__prop__val__gpuDirectRDMAFlushWritesOptions,
	const int32_t p__prop__val__gpuDirectRDMAWritesOrdering,
	const uint32_t p__prop__val__memoryPoolSupportedHandleTypes,
	const int32_t p__prop__val__deferredMappingHipArraySupported,
	const int32_t p__prop__val__ipcEventSupported,
	const int32_t p__prop__val__clusterLaunch,
	const int32_t p__prop__val__unifiedFunctionPointers,
	const uint64_t p__prop__val__reserved,
	const uint64_t p__prop__val__hipReserved,
	const uint64_t p__prop__val__gcnArchName,
	const uint64_t p__prop__val__maxSharedMemoryPerMultiProcessor,
	const int32_t p__prop__val__clockInstructionRate,
	const uint32_t p__prop__val__arch__hasGlobalInt32Atomics,
	const uint32_t p__prop__val__arch__hasGlobalFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasSharedInt32Atomics,
	const uint32_t p__prop__val__arch__hasSharedFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasFloatAtomicAdd,
	const uint32_t p__prop__val__arch__hasGlobalInt64Atomics,
	const uint32_t p__prop__val__arch__hasSharedInt64Atomics,
	const uint32_t p__prop__val__arch__hasDoubles,
	const uint32_t p__prop__val__arch__hasWarpVote,
	const uint32_t p__prop__val__arch__hasWarpBallot,
	const uint32_t p__prop__val__arch__hasWarpShuffle,
	const uint32_t p__prop__val__arch__hasFunnelShift,
	const uint32_t p__prop__val__arch__hasThreadFenceSystem,
	const uint32_t p__prop__val__arch__hasSyncThreadsExt,
	const uint32_t p__prop__val__arch__hasSurfaceFuncs,
	const uint32_t p__prop__val__arch__has3dGrid,
	const uint32_t p__prop__val__arch__hasDynamicParallelism,
	const uint64_t p__prop__val__hdpMemFlushCntl,
	const uint64_t p__prop__val__hdpRegFlushCntl,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedFunc,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedGridDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem,
	const int32_t p__prop__val__isLargeBar,
	const int32_t p__prop__val__asicRevision)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 20);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint64_t tmp_val = (uint64_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_device__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__name` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__name;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__uuid__bytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__uuid__bytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__uuid__bytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__luid` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__luid` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__luid;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__luidDeviceNodeMask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__luidDeviceNodeMask` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__luidDeviceNodeMask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__totalGlobalMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__totalGlobalMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__totalGlobalMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__sharedMemPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__sharedMemPerBlock` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__sharedMemPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__regsPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__regsPerBlock` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__regsPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__warpSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__warpSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__warpSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__memPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxThreadsPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxThreadsPerBlock` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxThreadsPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxThreadsDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxThreadsDim` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxThreadsDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxGridSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxGridSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxGridSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__clockRate` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__clockRate` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__clockRate;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__totalConstMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__totalConstMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__totalConstMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__major` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__major` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__major;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__minor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__minor` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__minor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__textureAlignment` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__textureAlignment` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__textureAlignment;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__texturePitchAlignment` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__texturePitchAlignment` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__texturePitchAlignment;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__deviceOverlap` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__deviceOverlap` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__deviceOverlap;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__multiProcessorCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__multiProcessorCount` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__multiProcessorCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__kernelExecTimeoutEnabled` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__kernelExecTimeoutEnabled` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__kernelExecTimeoutEnabled;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__integrated` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__integrated` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__integrated;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__canMapHostMemory` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__canMapHostMemory` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__canMapHostMemory;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__computeMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__computeMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__computeMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture1D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture1D` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxTexture1D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture1DMipmap` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture1DMipmap` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxTexture1DMipmap;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture1DLinear` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture1DLinear` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxTexture1DLinear;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture2D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture2D` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture2D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture2DMipmap` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture2DMipmap` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture2DMipmap;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture2DLinear` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture2DLinear` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture2DLinear;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture2DGather` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture2DGather` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture2DGather;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture3D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture3D` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture3D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture3DAlt` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture3DAlt` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture3DAlt;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTextureCubemap` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTextureCubemap` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxTextureCubemap;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture1DLayered` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture1DLayered` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture1DLayered;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture2DLayered` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture2DLayered` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture2DLayered;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTextureCubemapLayered` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTextureCubemapLayered` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTextureCubemapLayered;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxSurface1D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurface1D` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxSurface1D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxSurface2D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurface2D` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSurface2D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxSurface3D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurface3D` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSurface3D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxSurface1DLayered` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurface1DLayered` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSurface1DLayered;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxSurface2DLayered` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurface2DLayered` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSurface2DLayered;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxSurfaceCubemap` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurfaceCubemap` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxSurfaceCubemap;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxSurfaceCubemapLayered` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurfaceCubemapLayered` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSurfaceCubemapLayered;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__surfaceAlignment` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__surfaceAlignment` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__surfaceAlignment;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__concurrentKernels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__concurrentKernels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__concurrentKernels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__ECCEnabled` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__ECCEnabled` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__ECCEnabled;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pciBusID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pciBusID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pciBusID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pciDeviceID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pciDeviceID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pciDeviceID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pciDomainID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pciDomainID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pciDomainID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__tccDriver` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__tccDriver` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__tccDriver;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__asyncEngineCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__asyncEngineCount` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__asyncEngineCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__unifiedAddressing` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__unifiedAddressing` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__unifiedAddressing;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memoryClockRate` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memoryClockRate` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__memoryClockRate;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memoryBusWidth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memoryBusWidth` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__memoryBusWidth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__l2CacheSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__l2CacheSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__l2CacheSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__persistingL2CacheMaxSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__persistingL2CacheMaxSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__persistingL2CacheMaxSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxThreadsPerMultiProcessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxThreadsPerMultiProcessor` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxThreadsPerMultiProcessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__streamPrioritiesSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__streamPrioritiesSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__streamPrioritiesSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__globalL1CacheSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__globalL1CacheSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__globalL1CacheSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__localL1CacheSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__localL1CacheSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__localL1CacheSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__sharedMemPerMultiprocessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__sharedMemPerMultiprocessor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__sharedMemPerMultiprocessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__regsPerMultiprocessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__regsPerMultiprocessor` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__regsPerMultiprocessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__managedMemory` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__managedMemory` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__managedMemory;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__isMultiGpuBoard` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__isMultiGpuBoard` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__isMultiGpuBoard;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__multiGpuBoardGroupID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__multiGpuBoardGroupID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__multiGpuBoardGroupID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__hostNativeAtomicSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hostNativeAtomicSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__hostNativeAtomicSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__singleToDoublePrecisionPerfRatio` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__singleToDoublePrecisionPerfRatio` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__singleToDoublePrecisionPerfRatio;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pageableMemoryAccess` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pageableMemoryAccess` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pageableMemoryAccess;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__concurrentManagedAccess` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__concurrentManagedAccess` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__concurrentManagedAccess;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__computePreemptionSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__computePreemptionSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__computePreemptionSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__canUseHostPointerForRegisteredMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__canUseHostPointerForRegisteredMem` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__canUseHostPointerForRegisteredMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeLaunch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeLaunch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeLaunch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceLaunch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceLaunch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceLaunch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__sharedMemPerBlockOptin` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__sharedMemPerBlockOptin` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__sharedMemPerBlockOptin;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__pageableMemoryAccessUsesHostPageTables` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pageableMemoryAccessUsesHostPageTables` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pageableMemoryAccessUsesHostPageTables;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__directManagedMemAccessFromHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__directManagedMemAccessFromHost` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__directManagedMemAccessFromHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxBlocksPerMultiProcessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxBlocksPerMultiProcessor` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxBlocksPerMultiProcessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__accessPolicyMaxWindowSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__accessPolicyMaxWindowSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__accessPolicyMaxWindowSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__reservedSharedMemPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__reservedSharedMemPerBlock` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__reservedSharedMemPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__hostRegisterSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hostRegisterSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__hostRegisterSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__sparseHipArraySupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__sparseHipArraySupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__sparseHipArraySupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__hostRegisterReadOnlySupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hostRegisterReadOnlySupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__hostRegisterReadOnlySupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__timelineSemaphoreInteropSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__timelineSemaphoreInteropSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__timelineSemaphoreInteropSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memoryPoolsSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memoryPoolsSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__memoryPoolsSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__gpuDirectRDMASupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__gpuDirectRDMASupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__gpuDirectRDMASupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__gpuDirectRDMAFlushWritesOptions` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__gpuDirectRDMAFlushWritesOptions` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__gpuDirectRDMAFlushWritesOptions;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__gpuDirectRDMAWritesOrdering` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__gpuDirectRDMAWritesOrdering` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__gpuDirectRDMAWritesOrdering;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memoryPoolSupportedHandleTypes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memoryPoolSupportedHandleTypes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__memoryPoolSupportedHandleTypes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__deferredMappingHipArraySupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__deferredMappingHipArraySupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__deferredMappingHipArraySupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__ipcEventSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__ipcEventSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__ipcEventSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__clusterLaunch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__clusterLaunch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__clusterLaunch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__unifiedFunctionPointers` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__unifiedFunctionPointers` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__unifiedFunctionPointers;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__hipReserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hipReserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__hipReserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__gcnArchName` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__gcnArchName` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__gcnArchName;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxSharedMemoryPerMultiProcessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSharedMemoryPerMultiProcessor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSharedMemoryPerMultiProcessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__clockInstructionRate` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__clockInstructionRate` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__clockInstructionRate;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasGlobalInt32Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasGlobalInt32Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasGlobalInt32Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasGlobalFloatAtomicExch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasGlobalFloatAtomicExch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasGlobalFloatAtomicExch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSharedInt32Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSharedInt32Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSharedInt32Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSharedFloatAtomicExch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSharedFloatAtomicExch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSharedFloatAtomicExch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasFloatAtomicAdd` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasFloatAtomicAdd` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasFloatAtomicAdd;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasGlobalInt64Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasGlobalInt64Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasGlobalInt64Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSharedInt64Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSharedInt64Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSharedInt64Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasDoubles` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasDoubles` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasDoubles;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasWarpVote` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasWarpVote` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasWarpVote;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasWarpBallot` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasWarpBallot` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasWarpBallot;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasWarpShuffle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasWarpShuffle` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasWarpShuffle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasFunnelShift` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasFunnelShift` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasFunnelShift;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasThreadFenceSystem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasThreadFenceSystem` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasThreadFenceSystem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSyncThreadsExt` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSyncThreadsExt` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSyncThreadsExt;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSurfaceFuncs` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSurfaceFuncs` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSurfaceFuncs;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__has3dGrid` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__has3dGrid` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__has3dGrid;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasDynamicParallelism` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasDynamicParallelism` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasDynamicParallelism;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__hdpMemFlushCntl` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hdpMemFlushCntl` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__hdpMemFlushCntl;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__hdpRegFlushCntl` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hdpRegFlushCntl` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__hdpRegFlushCntl;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedFunc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedFunc` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedFunc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedGridDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__isLargeBar` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__isLargeBar` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__isLargeBar;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__asicRevision` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__asicRevision` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__asicRevision;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipChooseDeviceR0600End(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 21);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipConfigureCallBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint32_t p__gridDim__x,
	const uint32_t p__gridDim__y,
	const uint32_t p__gridDim__z,
	const uint32_t p__blockDim__x,
	const uint32_t p__blockDim__y,
	const uint32_t p__blockDim__z,
	const uint64_t p__sharedMem,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 22);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_gridDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_gridDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_gridDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_sharedMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sharedMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sharedMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipConfigureCallEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 23);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCreateSurfaceObjectBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pSurfObject,
	const uint64_t p__pSurfObject__val,
	const uint64_t p__pResDesc,
	const int32_t p__pResDesc__val__resType)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 24);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pSurfObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pSurfObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pSurfObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pSurfObject__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pSurfObject__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pSurfObject__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pResDesc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pResDesc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pResDesc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pResDesc__val__resType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pResDesc__val__resType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pResDesc__val__resType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipCreateSurfaceObjectEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 25);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxCreateBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx,
	const uint64_t p__ctx__val,
	const uint32_t p__flags,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 26);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ctx` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ctx` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ctx;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ctx__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ctx__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ctx__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipCtxCreateEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 27);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxDestroyBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 28);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ctx` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ctx` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ctx;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipCtxDestroyEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 29);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxDisablePeerAccessBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__peerCtx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 30);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_peerCtx` field */
		_ALIGN(ctx->at, 8);

		/* Write `_peerCtx` field */
		{
			const uint64_t tmp_val = (uint64_t) p__peerCtx;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipCtxDisablePeerAccessEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 31);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxEnablePeerAccessBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__peerCtx,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 32);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_peerCtx` field */
		_ALIGN(ctx->at, 8);

		/* Write `_peerCtx` field */
		{
			const uint64_t tmp_val = (uint64_t) p__peerCtx;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipCtxEnablePeerAccessEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 33);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxGetApiVersionBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx,
	const uint64_t p__apiVersion,
	const int32_t p__apiVersion__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 34);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ctx` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ctx` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ctx;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_apiVersion` field */
		_ALIGN(ctx->at, 8);

		/* Write `_apiVersion` field */
		{
			const uint64_t tmp_val = (uint64_t) p__apiVersion;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_apiVersion__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_apiVersion__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__apiVersion__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipCtxGetApiVersionEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 35);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxGetCacheConfigBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__cacheConfig,
	const int32_t p__cacheConfig__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 36);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_cacheConfig` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cacheConfig` field */
		{
			const uint64_t tmp_val = (uint64_t) p__cacheConfig;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_cacheConfig__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cacheConfig__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__cacheConfig__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipCtxGetCacheConfigEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 37);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxGetCurrentBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx,
	const uint64_t p__ctx__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 38);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ctx` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ctx` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ctx;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ctx__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ctx__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ctx__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipCtxGetCurrentEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 39);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxGetDeviceBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__device,
	const int32_t p__device__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 40);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint64_t tmp_val = (uint64_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_device__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipCtxGetDeviceEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 41);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxGetFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__flags,
	const uint32_t p__flags__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 42);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipCtxGetFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 43);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxGetSharedMemConfigBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pConfig,
	const int32_t p__pConfig__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 44);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pConfig` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pConfig` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pConfig;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pConfig__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pConfig__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pConfig__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipCtxGetSharedMemConfigEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 45);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxPopCurrentBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx,
	const uint64_t p__ctx__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 46);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ctx` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ctx` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ctx;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ctx__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ctx__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ctx__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipCtxPopCurrentEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 47);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxPushCurrentBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 48);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ctx` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ctx` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ctx;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipCtxPushCurrentEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 49);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxSetCacheConfigBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__cacheConfig)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 50);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_cacheConfig` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cacheConfig` field */
		{
			const uint32_t tmp_val = (uint32_t) p__cacheConfig;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipCtxSetCacheConfigEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 51);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxSetCurrentBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 52);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ctx` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ctx` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ctx;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipCtxSetCurrentEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 53);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxSetSharedMemConfigBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__config)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 54);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_config` field */
		_ALIGN(ctx->at, 8);

		/* Write `_config` field */
		{
			const uint32_t tmp_val = (uint32_t) p__config;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipCtxSetSharedMemConfigEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 55);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxSynchronizeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 56);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipCtxSynchronizeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 57);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDestroyExternalMemoryBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__extMem)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 58);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_extMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipDestroyExternalMemoryEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 59);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDestroyExternalSemaphoreBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__extSem)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 60);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_extSem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extSem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extSem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipDestroyExternalSemaphoreEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 61);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDestroySurfaceObjectBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__surfaceObject)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 62);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_surfaceObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_surfaceObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__surfaceObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipDestroySurfaceObjectEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 63);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceCanAccessPeerBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__canAccessPeer,
	const int32_t p__canAccessPeer__val,
	const int32_t p__deviceId,
	const int32_t p__peerDeviceId)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 64);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_canAccessPeer` field */
		_ALIGN(ctx->at, 8);

		/* Write `_canAccessPeer` field */
		{
			const uint64_t tmp_val = (uint64_t) p__canAccessPeer;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_canAccessPeer__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_canAccessPeer__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__canAccessPeer__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_deviceId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_deviceId` field */
		{
			const uint32_t tmp_val = (uint32_t) p__deviceId;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_peerDeviceId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_peerDeviceId` field */
		{
			const uint32_t tmp_val = (uint32_t) p__peerDeviceId;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceCanAccessPeerEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 65);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceComputeCapabilityBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__major,
	const int32_t p__major__val,
	const uint64_t p__minor,
	const int32_t p__minor__val,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 66);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_major` field */
		_ALIGN(ctx->at, 8);

		/* Write `_major` field */
		{
			const uint64_t tmp_val = (uint64_t) p__major;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_major__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_major__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__major__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_minor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_minor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__minor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_minor__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_minor__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__minor__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceComputeCapabilityEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 67);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceDisablePeerAccessBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__peerDeviceId)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 68);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_peerDeviceId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_peerDeviceId` field */
		{
			const uint32_t tmp_val = (uint32_t) p__peerDeviceId;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceDisablePeerAccessEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 69);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceEnablePeerAccessBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__peerDeviceId,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 70);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_peerDeviceId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_peerDeviceId` field */
		{
			const uint32_t tmp_val = (uint32_t) p__peerDeviceId;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceEnablePeerAccessEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 71);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetAttributeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pi,
	const int32_t p__pi__val,
	const int32_t p__attr,
	const int32_t p__deviceId)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 72);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pi` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pi` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pi;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pi__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pi__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pi__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_deviceId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_deviceId` field */
		{
			const uint32_t tmp_val = (uint32_t) p__deviceId;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetAttributeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 73);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__device,
	const int32_t p__device__val,
	const int32_t p__ordinal)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 74);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint64_t tmp_val = (uint64_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_device__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_ordinal` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ordinal` field */
		{
			const uint32_t tmp_val = (uint32_t) p__ordinal;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetByPCIBusIdBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__device,
	const int32_t p__device__val,
	const char * const p__pciBusId,
	const int8_t p__pciBusId__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 75);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint64_t tmp_val = (uint64_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_device__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pciBusId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pciBusId` field */
		_write_c_str(ctx, p__pciBusId);

		/* Align for `_pciBusId__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pciBusId__val` field */
		{
			const uint8_t tmp_val = (uint8_t) p__pciBusId__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetByPCIBusIdEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 76);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetCacheConfigBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__cacheConfig,
	const int32_t p__cacheConfig__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 77);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_cacheConfig` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cacheConfig` field */
		{
			const uint64_t tmp_val = (uint64_t) p__cacheConfig;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_cacheConfig__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cacheConfig__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__cacheConfig__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetCacheConfigEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 78);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetDefaultMemPoolBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const uint64_t p__mem_pool__val,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 79);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mem_pool__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetDefaultMemPoolEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 80);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 81);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetGraphMemAttributeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__device,
	const int32_t p__attr,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 82);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetGraphMemAttributeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 83);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetLimitBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pValue,
	const uint64_t p__pValue__val,
	const int32_t p__limit)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 84);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pValue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pValue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pValue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pValue__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pValue__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pValue__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_limit` field */
		_ALIGN(ctx->at, 8);

		/* Write `_limit` field */
		{
			const uint32_t tmp_val = (uint32_t) p__limit;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetLimitEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 85);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetMemPoolBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const uint64_t p__mem_pool__val,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 86);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mem_pool__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetMemPoolEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 87);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetNameBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const char * const p__name,
	const int8_t p__name__val,
	const int32_t p__len,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 88);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_name` field */
		_write_c_str(ctx, p__name);

		/* Align for `_name__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_name__val` field */
		{
			const uint8_t tmp_val = (uint8_t) p__name__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}

		/* Align for `_len` field */
		_ALIGN(ctx->at, 8);

		/* Write `_len` field */
		{
			const uint32_t tmp_val = (uint32_t) p__len;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetNameEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 89);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetP2PAttributeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__value,
	const int32_t p__value__val,
	const int32_t p__attr,
	const int32_t p__srcDevice,
	const int32_t p__dstDevice)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 90);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_srcDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcDevice` field */
		{
			const uint32_t tmp_val = (uint32_t) p__srcDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dstDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dstDevice` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dstDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetP2PAttributeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 91);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetPCIBusIdBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const char * const p__pciBusId,
	const int8_t p__pciBusId__val,
	const int32_t p__len,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 92);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pciBusId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pciBusId` field */
		_write_c_str(ctx, p__pciBusId);

		/* Align for `_pciBusId__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pciBusId__val` field */
		{
			const uint8_t tmp_val = (uint8_t) p__pciBusId__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}

		/* Align for `_len` field */
		_ALIGN(ctx->at, 8);

		/* Write `_len` field */
		{
			const uint32_t tmp_val = (uint32_t) p__len;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetPCIBusIdEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 93);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetSharedMemConfigBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pConfig,
	const int32_t p__pConfig__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 94);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pConfig` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pConfig` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pConfig;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pConfig__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pConfig__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pConfig__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetSharedMemConfigEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 95);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetStreamPriorityRangeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__leastPriority,
	const int32_t p__leastPriority__val,
	const uint64_t p__greatestPriority,
	const int32_t p__greatestPriority__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 96);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_leastPriority` field */
		_ALIGN(ctx->at, 8);

		/* Write `_leastPriority` field */
		{
			const uint64_t tmp_val = (uint64_t) p__leastPriority;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_leastPriority__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_leastPriority__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__leastPriority__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_greatestPriority` field */
		_ALIGN(ctx->at, 8);

		/* Write `_greatestPriority` field */
		{
			const uint64_t tmp_val = (uint64_t) p__greatestPriority;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_greatestPriority__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_greatestPriority__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__greatestPriority__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetStreamPriorityRangeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 97);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGetUuidBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__uuid,
	const uint64_t p__uuid__val__bytes,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 98);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_uuid` field */
		_ALIGN(ctx->at, 8);

		/* Write `_uuid` field */
		{
			const uint64_t tmp_val = (uint64_t) p__uuid;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_uuid__val__bytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_uuid__val__bytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__uuid__val__bytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGetUuidEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 99);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceGraphMemTrimBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 100);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceGraphMemTrimEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 101);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDevicePrimaryCtxGetStateBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__dev,
	const uint64_t p__flags,
	const uint32_t p__flags__val,
	const uint64_t p__active,
	const int32_t p__active__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 102);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dev` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dev;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_active` field */
		_ALIGN(ctx->at, 8);

		/* Write `_active` field */
		{
			const uint64_t tmp_val = (uint64_t) p__active;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_active__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_active__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__active__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDevicePrimaryCtxGetStateEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 103);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDevicePrimaryCtxReleaseBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__dev)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 104);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dev` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dev;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDevicePrimaryCtxReleaseEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 105);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDevicePrimaryCtxResetBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__dev)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 106);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dev` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dev;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDevicePrimaryCtxResetEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 107);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDevicePrimaryCtxRetainBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pctx,
	const uint64_t p__pctx__val,
	const int32_t p__dev)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 108);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pctx` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pctx` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pctx;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pctx__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pctx__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pctx__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dev` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dev;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDevicePrimaryCtxRetainEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 109);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDevicePrimaryCtxSetFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__dev,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 110);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dev` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dev;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDevicePrimaryCtxSetFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 111);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceResetBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 112);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceResetEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 113);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceSetCacheConfigBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__cacheConfig)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 114);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_cacheConfig` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cacheConfig` field */
		{
			const uint32_t tmp_val = (uint32_t) p__cacheConfig;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceSetCacheConfigEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 115);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceSetGraphMemAttributeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__device,
	const int32_t p__attr,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 116);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceSetGraphMemAttributeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 117);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceSetLimitBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__limit,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 118);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_limit` field */
		_ALIGN(ctx->at, 8);

		/* Write `_limit` field */
		{
			const uint32_t tmp_val = (uint32_t) p__limit;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceSetLimitEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 119);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceSetMemPoolBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__device,
	const uint64_t p__mem_pool)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 120);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceSetMemPoolEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 121);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceSetSharedMemConfigBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__config)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 122);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_config` field */
		_ALIGN(ctx->at, 8);

		/* Write `_config` field */
		{
			const uint32_t tmp_val = (uint32_t) p__config;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceSetSharedMemConfigEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 123);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceSynchronizeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 124);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceSynchronizeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 125);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDeviceTotalMemBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__bytes,
	const uint64_t p__bytes__val,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 126);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_bytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_bytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__bytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_bytes__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_bytes__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__bytes__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDeviceTotalMemEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 127);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDriverGetVersionBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__driverVersion,
	const int32_t p__driverVersion__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 128);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_driverVersion` field */
		_ALIGN(ctx->at, 8);

		/* Write `_driverVersion` field */
		{
			const uint64_t tmp_val = (uint64_t) p__driverVersion;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_driverVersion__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_driverVersion__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__driverVersion__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipDriverGetVersionEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 129);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDrvGraphAddMemcpyNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__phGraphNode,
	const uint64_t p__phGraphNode__val,
	const uint64_t p__hGraph,
	const uint64_t p__dependencies,
	const uint64_t p__dependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__copyParams,
	const uint64_t p__copyParams__val__srcXInBytes,
	const uint64_t p__copyParams__val__srcY,
	const uint64_t p__copyParams__val__srcZ,
	const uint64_t p__copyParams__val__srcLOD,
	const int32_t p__copyParams__val__srcMemoryType,
	const uint64_t p__copyParams__val__srcHost,
	const uint64_t p__copyParams__val__srcDevice,
	const uint64_t p__copyParams__val__srcArray,
	const uint64_t p__copyParams__val__srcPitch,
	const uint64_t p__copyParams__val__srcHeight,
	const uint64_t p__copyParams__val__dstXInBytes,
	const uint64_t p__copyParams__val__dstY,
	const uint64_t p__copyParams__val__dstZ,
	const uint64_t p__copyParams__val__dstLOD,
	const int32_t p__copyParams__val__dstMemoryType,
	const uint64_t p__copyParams__val__dstHost,
	const uint64_t p__copyParams__val__dstDevice,
	const uint64_t p__copyParams__val__dstArray,
	const uint64_t p__copyParams__val__dstPitch,
	const uint64_t p__copyParams__val__dstHeight,
	const uint64_t p__copyParams__val__WidthInBytes,
	const uint64_t p__copyParams__val__Height,
	const uint64_t p__copyParams__val__Depth,
	const uint64_t p__ctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 130);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_phGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_phGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__phGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_phGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_phGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__phGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hGraph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__srcXInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__srcXInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__srcXInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__srcY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__srcY` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__srcY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__srcZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__srcZ` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__srcZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__srcLOD` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__srcLOD` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__srcLOD;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__srcMemoryType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__srcMemoryType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__copyParams__val__srcMemoryType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_copyParams__val__srcHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__srcHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__srcHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__srcDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__srcDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__srcDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__srcPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__srcPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__srcPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__srcHeight` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__srcHeight` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__srcHeight;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__dstXInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__dstXInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__dstXInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__dstY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__dstY` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__dstY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__dstZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__dstZ` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__dstZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__dstLOD` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__dstLOD` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__dstLOD;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__dstMemoryType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__dstMemoryType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__copyParams__val__dstMemoryType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_copyParams__val__dstHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__dstHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__dstHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__dstDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__dstDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__dstDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__dstPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__dstPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__dstPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__dstHeight` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__dstHeight` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__dstHeight;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__WidthInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__WidthInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__WidthInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__Height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__Height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__Height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_copyParams__val__Depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copyParams__val__Depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copyParams__val__Depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ctx` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ctx` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ctx;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipDrvGraphAddMemcpyNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 131);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDrvGraphAddMemsetNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__phGraphNode,
	const uint64_t p__phGraphNode__val,
	const uint64_t p__hGraph,
	const uint64_t p__dependencies,
	const uint64_t p__dependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__memsetParams,
	const uint64_t p__memsetParams__val__dst,
	const uint64_t p__memsetParams__val__pitch,
	const uint32_t p__memsetParams__val__value,
	const uint32_t p__memsetParams__val__elementSize,
	const uint64_t p__memsetParams__val__width,
	const uint64_t p__memsetParams__val__height,
	const uint64_t p__ctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 132);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_phGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_phGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__phGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_phGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_phGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__phGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hGraph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_memsetParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memsetParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memsetParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_memsetParams__val__dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memsetParams__val__dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memsetParams__val__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_memsetParams__val__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memsetParams__val__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memsetParams__val__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_memsetParams__val__value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memsetParams__val__value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__memsetParams__val__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_memsetParams__val__elementSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memsetParams__val__elementSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__memsetParams__val__elementSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_memsetParams__val__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memsetParams__val__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memsetParams__val__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_memsetParams__val__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memsetParams__val__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memsetParams__val__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ctx` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ctx` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ctx;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipDrvGraphAddMemsetNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 133);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDrvMemcpy2DUnalignedBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pCopy,
	const uint64_t p__pCopy__val__srcXInBytes,
	const uint64_t p__pCopy__val__srcY,
	const int32_t p__pCopy__val__srcMemoryType,
	const uint64_t p__pCopy__val__srcHost,
	const uint64_t p__pCopy__val__srcDevice,
	const uint64_t p__pCopy__val__srcArray,
	const uint64_t p__pCopy__val__srcPitch,
	const uint64_t p__pCopy__val__dstXInBytes,
	const uint64_t p__pCopy__val__dstY,
	const int32_t p__pCopy__val__dstMemoryType,
	const uint64_t p__pCopy__val__dstHost,
	const uint64_t p__pCopy__val__dstDevice,
	const uint64_t p__pCopy__val__dstArray,
	const uint64_t p__pCopy__val__dstPitch,
	const uint64_t p__pCopy__val__WidthInBytes,
	const uint64_t p__pCopy__val__Height)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 134);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pCopy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcXInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcXInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcXInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcY` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcMemoryType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcMemoryType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pCopy__val__srcMemoryType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pCopy__val__srcHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstXInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstXInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstXInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstY` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstMemoryType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstMemoryType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pCopy__val__dstMemoryType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pCopy__val__dstHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__WidthInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__WidthInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__WidthInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__Height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__Height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__Height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipDrvMemcpy2DUnalignedEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 135);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDrvMemcpy3DAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pCopy,
	const uint64_t p__pCopy__val__srcXInBytes,
	const uint64_t p__pCopy__val__srcY,
	const uint64_t p__pCopy__val__srcZ,
	const uint64_t p__pCopy__val__srcLOD,
	const int32_t p__pCopy__val__srcMemoryType,
	const uint64_t p__pCopy__val__srcHost,
	const uint64_t p__pCopy__val__srcDevice,
	const uint64_t p__pCopy__val__srcArray,
	const uint64_t p__pCopy__val__srcPitch,
	const uint64_t p__pCopy__val__srcHeight,
	const uint64_t p__pCopy__val__dstXInBytes,
	const uint64_t p__pCopy__val__dstY,
	const uint64_t p__pCopy__val__dstZ,
	const uint64_t p__pCopy__val__dstLOD,
	const int32_t p__pCopy__val__dstMemoryType,
	const uint64_t p__pCopy__val__dstHost,
	const uint64_t p__pCopy__val__dstDevice,
	const uint64_t p__pCopy__val__dstArray,
	const uint64_t p__pCopy__val__dstPitch,
	const uint64_t p__pCopy__val__dstHeight,
	const uint64_t p__pCopy__val__WidthInBytes,
	const uint64_t p__pCopy__val__Height,
	const uint64_t p__pCopy__val__Depth,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 136);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pCopy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcXInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcXInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcXInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcY` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcZ` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcLOD` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcLOD` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcLOD;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcMemoryType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcMemoryType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pCopy__val__srcMemoryType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pCopy__val__srcHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcHeight` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcHeight` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcHeight;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstXInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstXInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstXInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstY` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstZ` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstLOD` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstLOD` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstLOD;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstMemoryType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstMemoryType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pCopy__val__dstMemoryType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pCopy__val__dstHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstHeight` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstHeight` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstHeight;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__WidthInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__WidthInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__WidthInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__Height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__Height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__Height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__Depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__Depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__Depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipDrvMemcpy3DAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 137);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDrvMemcpy3DBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pCopy,
	const uint64_t p__pCopy__val__srcXInBytes,
	const uint64_t p__pCopy__val__srcY,
	const uint64_t p__pCopy__val__srcZ,
	const uint64_t p__pCopy__val__srcLOD,
	const int32_t p__pCopy__val__srcMemoryType,
	const uint64_t p__pCopy__val__srcHost,
	const uint64_t p__pCopy__val__srcDevice,
	const uint64_t p__pCopy__val__srcArray,
	const uint64_t p__pCopy__val__srcPitch,
	const uint64_t p__pCopy__val__srcHeight,
	const uint64_t p__pCopy__val__dstXInBytes,
	const uint64_t p__pCopy__val__dstY,
	const uint64_t p__pCopy__val__dstZ,
	const uint64_t p__pCopy__val__dstLOD,
	const int32_t p__pCopy__val__dstMemoryType,
	const uint64_t p__pCopy__val__dstHost,
	const uint64_t p__pCopy__val__dstDevice,
	const uint64_t p__pCopy__val__dstArray,
	const uint64_t p__pCopy__val__dstPitch,
	const uint64_t p__pCopy__val__dstHeight,
	const uint64_t p__pCopy__val__WidthInBytes,
	const uint64_t p__pCopy__val__Height,
	const uint64_t p__pCopy__val__Depth)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 138);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pCopy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcXInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcXInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcXInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcY` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcZ` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcLOD` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcLOD` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcLOD;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcMemoryType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcMemoryType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pCopy__val__srcMemoryType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pCopy__val__srcHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcHeight` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcHeight` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcHeight;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstXInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstXInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstXInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstY` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstZ` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstLOD` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstLOD` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstLOD;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstMemoryType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstMemoryType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pCopy__val__dstMemoryType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pCopy__val__dstHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstHeight` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstHeight` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstHeight;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__WidthInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__WidthInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__WidthInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__Height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__Height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__Height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__Depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__Depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__Depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipDrvMemcpy3DEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 139);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipDrvPointerGetAttributesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint32_t p__numAttributes,
	const uint64_t p__attributes,
	const int32_t p__attributes__val,
	const uint64_t p__data,
	const uint64_t p__data__val,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 140);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_numAttributes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numAttributes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numAttributes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attributes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attributes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attributes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attributes__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attributes__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attributes__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipDrvPointerGetAttributesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 141);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipEventCreateBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event,
	const uint64_t p__event__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 142);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipEventCreateEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 143);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipEventCreateWithFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event,
	const uint64_t p__event__val,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 144);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipEventCreateWithFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 145);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipEventDestroyBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 146);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipEventDestroyEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 147);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipEventElapsedTimeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ms,
	const uint64_t p__ms__val,
	const uint64_t p__start,
	const uint64_t p__stop)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 148);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ms` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ms` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ms;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ms__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ms__val` field */
		{
			union _f2u f2u;

			f2u.f = p__ms__val;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_start` field */
		_ALIGN(ctx->at, 8);

		/* Write `_start` field */
		{
			const uint64_t tmp_val = (uint64_t) p__start;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stop` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stop` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stop;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipEventElapsedTimeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 149);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipEventQueryBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 150);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipEventQueryEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 151);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipEventRecordBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 152);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipEventRecordEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 153);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipEventSynchronizeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 154);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipEventSynchronizeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 155);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipExtGetLastErrorBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 156);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipExtGetLastErrorEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 157);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipExtGetLinkTypeAndHopCountBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__device1,
	const int32_t p__device2,
	const uint64_t p__linktype,
	const uint32_t p__linktype__val,
	const uint64_t p__hopcount,
	const uint32_t p__hopcount__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 158);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_device1` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device1` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device1;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_device2` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device2` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device2;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_linktype` field */
		_ALIGN(ctx->at, 8);

		/* Write `_linktype` field */
		{
			const uint64_t tmp_val = (uint64_t) p__linktype;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_linktype__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_linktype__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__linktype__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_hopcount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hopcount` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hopcount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hopcount__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hopcount__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__hopcount__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipExtGetLinkTypeAndHopCountEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 159);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipExtLaunchKernelBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__function_address,
	const uint32_t p__numBlocks__x,
	const uint32_t p__numBlocks__y,
	const uint32_t p__numBlocks__z,
	const uint32_t p__dimBlocks__x,
	const uint32_t p__dimBlocks__y,
	const uint32_t p__dimBlocks__z,
	const uint64_t p__args,
	const uint64_t p__args__val,
	const uint64_t p__sharedMemBytes,
	const uint64_t p__stream,
	const uint64_t p__startEvent,
	const uint64_t p__stopEvent,
	const int32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 160);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_function_address` field */
		_ALIGN(ctx->at, 8);

		/* Write `_function_address` field */
		{
			const uint64_t tmp_val = (uint64_t) p__function_address;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numBlocks__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numBlocks__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_numBlocks__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numBlocks__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_numBlocks__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numBlocks__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dimBlocks__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dimBlocks__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dimBlocks__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dimBlocks__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dimBlocks__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dimBlocks__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dimBlocks__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dimBlocks__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dimBlocks__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_args` field */
		_ALIGN(ctx->at, 8);

		/* Write `_args` field */
		{
			const uint64_t tmp_val = (uint64_t) p__args;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_args__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_args__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__args__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sharedMemBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sharedMemBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sharedMemBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_startEvent` field */
		_ALIGN(ctx->at, 8);

		/* Write `_startEvent` field */
		{
			const uint64_t tmp_val = (uint64_t) p__startEvent;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stopEvent` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stopEvent` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stopEvent;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipExtLaunchKernelEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 161);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipExtLaunchMultiKernelMultiDeviceBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__launchParamsList,
	const uint64_t p__launchParamsList__val__func,
	const uint32_t p__launchParamsList__val__gridDim__x,
	const uint32_t p__launchParamsList__val__gridDim__y,
	const uint32_t p__launchParamsList__val__gridDim__z,
	const uint32_t p__launchParamsList__val__blockDim__x,
	const uint32_t p__launchParamsList__val__blockDim__y,
	const uint32_t p__launchParamsList__val__blockDim__z,
	const uint64_t p__launchParamsList__val__args,
	const uint64_t p__launchParamsList__val__sharedMem,
	const uint64_t p__launchParamsList__val__stream,
	const int32_t p__numDevices,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 162);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_launchParamsList` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_launchParamsList__val__func` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__func` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList__val__func;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_launchParamsList__val__gridDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__gridDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__gridDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__gridDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__gridDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__gridDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__gridDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__gridDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__gridDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__blockDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__blockDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__blockDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__blockDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__blockDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__blockDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__blockDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__blockDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__blockDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__args` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__args` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList__val__args;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_launchParamsList__val__sharedMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__sharedMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList__val__sharedMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_launchParamsList__val__stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList__val__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDevices` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDevices` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numDevices;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipExtLaunchMultiKernelMultiDeviceEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 163);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipExtMallocWithFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__sizeBytes,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 164);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipExtMallocWithFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 165);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipExtModuleLaunchKernelBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__f,
	const uint32_t p__globalWorkSizeX,
	const uint32_t p__globalWorkSizeY,
	const uint32_t p__globalWorkSizeZ,
	const uint32_t p__localWorkSizeX,
	const uint32_t p__localWorkSizeY,
	const uint32_t p__localWorkSizeZ,
	const uint64_t p__sharedMemBytes,
	const uint64_t p__hStream,
	const uint64_t p__kernelParams,
	const uint64_t p__kernelParams__val,
	const uint64_t p__extra,
	const uint64_t p__extra__val,
	const uint64_t p__startEvent,
	const uint64_t p__stopEvent,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 166);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_f` field */
		{
			const uint64_t tmp_val = (uint64_t) p__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_globalWorkSizeX` field */
		_ALIGN(ctx->at, 8);

		/* Write `_globalWorkSizeX` field */
		{
			const uint32_t tmp_val = (uint32_t) p__globalWorkSizeX;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_globalWorkSizeY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_globalWorkSizeY` field */
		{
			const uint32_t tmp_val = (uint32_t) p__globalWorkSizeY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_globalWorkSizeZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_globalWorkSizeZ` field */
		{
			const uint32_t tmp_val = (uint32_t) p__globalWorkSizeZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_localWorkSizeX` field */
		_ALIGN(ctx->at, 8);

		/* Write `_localWorkSizeX` field */
		{
			const uint32_t tmp_val = (uint32_t) p__localWorkSizeX;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_localWorkSizeY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_localWorkSizeY` field */
		{
			const uint32_t tmp_val = (uint32_t) p__localWorkSizeY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_localWorkSizeZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_localWorkSizeZ` field */
		{
			const uint32_t tmp_val = (uint32_t) p__localWorkSizeZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_sharedMemBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sharedMemBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sharedMemBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hStream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hStream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hStream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kernelParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kernelParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__kernelParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kernelParams__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kernelParams__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__kernelParams__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extra` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extra` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extra;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extra__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extra__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extra__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_startEvent` field */
		_ALIGN(ctx->at, 8);

		/* Write `_startEvent` field */
		{
			const uint64_t tmp_val = (uint64_t) p__startEvent;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stopEvent` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stopEvent` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stopEvent;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipExtModuleLaunchKernelEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 167);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipExtStreamCreateWithCUMaskBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__stream__val,
	const uint32_t p__cuMaskSize,
	const uint64_t p__cuMask,
	const uint32_t p__cuMask__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 168);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_cuMaskSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cuMaskSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__cuMaskSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_cuMask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cuMask` field */
		{
			const uint64_t tmp_val = (uint64_t) p__cuMask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_cuMask__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cuMask__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__cuMask__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipExtStreamCreateWithCUMaskEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 169);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipExtStreamGetCUMaskBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint32_t p__cuMaskSize,
	const uint64_t p__cuMask,
	const uint32_t p__cuMask__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 170);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_cuMaskSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cuMaskSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__cuMaskSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_cuMask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cuMask` field */
		{
			const uint64_t tmp_val = (uint64_t) p__cuMask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_cuMask__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cuMask__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__cuMask__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipExtStreamGetCUMaskEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 171);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipExternalMemoryGetMappedBufferBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr,
	const uint64_t p__devPtr__val,
	const uint64_t p__extMem,
	const uint64_t p__bufferDesc,
	const uint64_t p__bufferDesc__val__offset,
	const uint64_t p__bufferDesc__val__size,
	const uint32_t p__bufferDesc__val__flags,
	const uint64_t p__bufferDesc__val__reserved)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 172);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_devPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__devPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_devPtr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_devPtr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__devPtr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_bufferDesc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_bufferDesc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__bufferDesc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_bufferDesc__val__offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_bufferDesc__val__offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__bufferDesc__val__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_bufferDesc__val__size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_bufferDesc__val__size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__bufferDesc__val__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_bufferDesc__val__flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_bufferDesc__val__flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__bufferDesc__val__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_bufferDesc__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_bufferDesc__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__bufferDesc__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipExternalMemoryGetMappedBufferEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 173);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipExternalMemoryGetMappedMipmappedArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mipmap,
	const uint64_t p__mipmap__val,
	const uint64_t p__extMem,
	const uint64_t p__mipmapDesc,
	const uint64_t p__mipmapDesc__val__offset,
	const int32_t p__mipmapDesc__val__formatDesc__x,
	const int32_t p__mipmapDesc__val__formatDesc__y,
	const int32_t p__mipmapDesc__val__formatDesc__z,
	const int32_t p__mipmapDesc__val__formatDesc__w,
	const int32_t p__mipmapDesc__val__formatDesc__f,
	const uint64_t p__mipmapDesc__val__extent__width,
	const uint64_t p__mipmapDesc__val__extent__height,
	const uint64_t p__mipmapDesc__val__extent__depth,
	const uint32_t p__mipmapDesc__val__flags,
	const uint32_t p__mipmapDesc__val__numLevels)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 174);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mipmap` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmap` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mipmap;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mipmap__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmap__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mipmap__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mipmapDesc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmapDesc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mipmapDesc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mipmapDesc__val__offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmapDesc__val__offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mipmapDesc__val__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mipmapDesc__val__formatDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmapDesc__val__formatDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmapDesc__val__formatDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmapDesc__val__formatDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmapDesc__val__formatDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmapDesc__val__formatDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmapDesc__val__formatDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmapDesc__val__formatDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmapDesc__val__formatDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmapDesc__val__formatDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmapDesc__val__formatDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmapDesc__val__formatDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmapDesc__val__formatDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmapDesc__val__formatDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmapDesc__val__formatDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmapDesc__val__extent__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmapDesc__val__extent__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mipmapDesc__val__extent__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mipmapDesc__val__extent__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmapDesc__val__extent__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mipmapDesc__val__extent__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mipmapDesc__val__extent__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmapDesc__val__extent__depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mipmapDesc__val__extent__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mipmapDesc__val__flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmapDesc__val__flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmapDesc__val__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmapDesc__val__numLevels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmapDesc__val__numLevels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmapDesc__val__numLevels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipExternalMemoryGetMappedMipmappedArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 175);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipFreeArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__array)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 176);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_array` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipFreeArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 177);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipFreeAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 178);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipFreeAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 179);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipFreeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 180);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipFreeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 181);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipFreeHostBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 182);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipFreeHostEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 183);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipFreeMipmappedArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mipmappedArray)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 184);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mipmappedArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mipmappedArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipFreeMipmappedArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 185);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipFuncGetAttributeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__value,
	const int32_t p__value__val,
	const int32_t p__attrib,
	const uint64_t p__hfunc)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 186);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attrib` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attrib` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attrib;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_hfunc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hfunc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hfunc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipFuncGetAttributeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 187);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipFuncGetAttributesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__attr,
	const int32_t p__attr__val__binaryVersion,
	const int32_t p__attr__val__cacheModeCA,
	const uint64_t p__attr__val__constSizeBytes,
	const uint64_t p__attr__val__localSizeBytes,
	const int32_t p__attr__val__maxDynamicSharedSizeBytes,
	const int32_t p__attr__val__maxThreadsPerBlock,
	const int32_t p__attr__val__numRegs,
	const int32_t p__attr__val__preferredShmemCarveout,
	const int32_t p__attr__val__ptxVersion,
	const uint64_t p__attr__val__sharedSizeBytes,
	const uint64_t p__func)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 188);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_attr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attr__val__binaryVersion` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr__val__binaryVersion` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr__val__binaryVersion;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attr__val__cacheModeCA` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr__val__cacheModeCA` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr__val__cacheModeCA;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attr__val__constSizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr__val__constSizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attr__val__constSizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attr__val__localSizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr__val__localSizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attr__val__localSizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attr__val__maxDynamicSharedSizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr__val__maxDynamicSharedSizeBytes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr__val__maxDynamicSharedSizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attr__val__maxThreadsPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr__val__maxThreadsPerBlock` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr__val__maxThreadsPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attr__val__numRegs` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr__val__numRegs` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr__val__numRegs;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attr__val__preferredShmemCarveout` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr__val__preferredShmemCarveout` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr__val__preferredShmemCarveout;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attr__val__ptxVersion` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr__val__ptxVersion` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr__val__ptxVersion;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attr__val__sharedSizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr__val__sharedSizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attr__val__sharedSizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_func` field */
		_ALIGN(ctx->at, 8);

		/* Write `_func` field */
		{
			const uint64_t tmp_val = (uint64_t) p__func;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipFuncGetAttributesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 189);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipFuncSetAttributeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__func,
	const int32_t p__attr,
	const int32_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 190);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_func` field */
		_ALIGN(ctx->at, 8);

		/* Write `_func` field */
		{
			const uint64_t tmp_val = (uint64_t) p__func;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipFuncSetAttributeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 191);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipFuncSetCacheConfigBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__func,
	const int32_t p__config)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 192);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_func` field */
		_ALIGN(ctx->at, 8);

		/* Write `_func` field */
		{
			const uint64_t tmp_val = (uint64_t) p__func;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_config` field */
		_ALIGN(ctx->at, 8);

		/* Write `_config` field */
		{
			const uint32_t tmp_val = (uint32_t) p__config;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipFuncSetCacheConfigEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 193);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipFuncSetSharedMemConfigBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__func,
	const int32_t p__config)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 194);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_func` field */
		_ALIGN(ctx->at, 8);

		/* Write `_func` field */
		{
			const uint64_t tmp_val = (uint64_t) p__func;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_config` field */
		_ALIGN(ctx->at, 8);

		/* Write `_config` field */
		{
			const uint32_t tmp_val = (uint32_t) p__config;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipFuncSetSharedMemConfigEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 195);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGLGetDevicesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pHipDeviceCount,
	const uint32_t p__pHipDeviceCount__val,
	const uint64_t p__pHipDevices,
	const int32_t p__pHipDevices__val,
	const uint32_t p__hipDeviceCount,
	const int32_t p__deviceList)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 196);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pHipDeviceCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pHipDeviceCount` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pHipDeviceCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pHipDeviceCount__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pHipDeviceCount__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pHipDeviceCount__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pHipDevices` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pHipDevices` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pHipDevices;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pHipDevices__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pHipDevices__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pHipDevices__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_hipDeviceCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hipDeviceCount` field */
		{
			const uint32_t tmp_val = (uint32_t) p__hipDeviceCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_deviceList` field */
		_ALIGN(ctx->at, 8);

		/* Write `_deviceList` field */
		{
			const uint32_t tmp_val = (uint32_t) p__deviceList;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGLGetDevicesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 197);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetChannelDescBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__desc,
	const int32_t p__desc__val__x,
	const int32_t p__desc__val__y,
	const int32_t p__desc__val__z,
	const int32_t p__desc__val__w,
	const int32_t p__desc__val__f,
	const uint64_t p__array)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 198);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_desc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__desc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc__val__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_array` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGetChannelDescEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 199);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetDeviceBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__deviceId,
	const int32_t p__deviceId__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 200);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_deviceId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_deviceId` field */
		{
			const uint64_t tmp_val = (uint64_t) p__deviceId;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_deviceId__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_deviceId__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__deviceId__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGetDeviceCountBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__count,
	const int32_t p__count__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 201);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__count__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGetDeviceCountEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 202);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetDeviceEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 203);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetDeviceFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__flags,
	const uint32_t p__flags__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 204);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGetDeviceFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 205);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetDevicePropertiesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 206);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetDevicePropertiesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 207);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetDevicePropertiesR0000Begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__prop,
	const uint64_t p__prop__val__name,
	const uint64_t p__prop__val__totalGlobalMem,
	const uint64_t p__prop__val__sharedMemPerBlock,
	const int32_t p__prop__val__regsPerBlock,
	const int32_t p__prop__val__warpSize,
	const int32_t p__prop__val__maxThreadsPerBlock,
	const uint64_t p__prop__val__maxThreadsDim,
	const uint64_t p__prop__val__maxGridSize,
	const int32_t p__prop__val__clockRate,
	const int32_t p__prop__val__memoryClockRate,
	const int32_t p__prop__val__memoryBusWidth,
	const uint64_t p__prop__val__totalConstMem,
	const int32_t p__prop__val__major,
	const int32_t p__prop__val__minor,
	const int32_t p__prop__val__multiProcessorCount,
	const int32_t p__prop__val__l2CacheSize,
	const int32_t p__prop__val__maxThreadsPerMultiProcessor,
	const int32_t p__prop__val__computeMode,
	const int32_t p__prop__val__clockInstructionRate,
	const uint32_t p__prop__val__arch__hasGlobalInt32Atomics,
	const uint32_t p__prop__val__arch__hasGlobalFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasSharedInt32Atomics,
	const uint32_t p__prop__val__arch__hasSharedFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasFloatAtomicAdd,
	const uint32_t p__prop__val__arch__hasGlobalInt64Atomics,
	const uint32_t p__prop__val__arch__hasSharedInt64Atomics,
	const uint32_t p__prop__val__arch__hasDoubles,
	const uint32_t p__prop__val__arch__hasWarpVote,
	const uint32_t p__prop__val__arch__hasWarpBallot,
	const uint32_t p__prop__val__arch__hasWarpShuffle,
	const uint32_t p__prop__val__arch__hasFunnelShift,
	const uint32_t p__prop__val__arch__hasThreadFenceSystem,
	const uint32_t p__prop__val__arch__hasSyncThreadsExt,
	const uint32_t p__prop__val__arch__hasSurfaceFuncs,
	const uint32_t p__prop__val__arch__has3dGrid,
	const uint32_t p__prop__val__arch__hasDynamicParallelism,
	const int32_t p__prop__val__concurrentKernels,
	const int32_t p__prop__val__pciDomainID,
	const int32_t p__prop__val__pciBusID,
	const int32_t p__prop__val__pciDeviceID,
	const uint64_t p__prop__val__maxSharedMemoryPerMultiProcessor,
	const int32_t p__prop__val__isMultiGpuBoard,
	const int32_t p__prop__val__canMapHostMemory,
	const int32_t p__prop__val__gcnArch,
	const uint64_t p__prop__val__gcnArchName,
	const int32_t p__prop__val__integrated,
	const int32_t p__prop__val__cooperativeLaunch,
	const int32_t p__prop__val__cooperativeMultiDeviceLaunch,
	const int32_t p__prop__val__maxTexture1DLinear,
	const int32_t p__prop__val__maxTexture1D,
	const uint64_t p__prop__val__maxTexture2D,
	const uint64_t p__prop__val__maxTexture3D,
	const uint64_t p__prop__val__hdpMemFlushCntl,
	const uint64_t p__prop__val__hdpRegFlushCntl,
	const uint64_t p__prop__val__memPitch,
	const uint64_t p__prop__val__textureAlignment,
	const uint64_t p__prop__val__texturePitchAlignment,
	const int32_t p__prop__val__kernelExecTimeoutEnabled,
	const int32_t p__prop__val__ECCEnabled,
	const int32_t p__prop__val__tccDriver,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedFunc,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedGridDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem,
	const int32_t p__prop__val__isLargeBar,
	const int32_t p__prop__val__asicRevision,
	const int32_t p__prop__val__managedMemory,
	const int32_t p__prop__val__directManagedMemAccessFromHost,
	const int32_t p__prop__val__concurrentManagedAccess,
	const int32_t p__prop__val__pageableMemoryAccess,
	const int32_t p__prop__val__pageableMemoryAccessUsesHostPageTables,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 208);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_prop` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__name` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__name;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__totalGlobalMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__totalGlobalMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__totalGlobalMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__sharedMemPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__sharedMemPerBlock` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__sharedMemPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__regsPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__regsPerBlock` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__regsPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__warpSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__warpSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__warpSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxThreadsPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxThreadsPerBlock` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxThreadsPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxThreadsDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxThreadsDim` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxThreadsDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxGridSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxGridSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxGridSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__clockRate` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__clockRate` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__clockRate;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memoryClockRate` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memoryClockRate` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__memoryClockRate;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memoryBusWidth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memoryBusWidth` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__memoryBusWidth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__totalConstMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__totalConstMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__totalConstMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__major` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__major` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__major;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__minor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__minor` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__minor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__multiProcessorCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__multiProcessorCount` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__multiProcessorCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__l2CacheSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__l2CacheSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__l2CacheSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxThreadsPerMultiProcessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxThreadsPerMultiProcessor` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxThreadsPerMultiProcessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__computeMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__computeMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__computeMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__clockInstructionRate` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__clockInstructionRate` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__clockInstructionRate;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasGlobalInt32Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasGlobalInt32Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasGlobalInt32Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasGlobalFloatAtomicExch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasGlobalFloatAtomicExch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasGlobalFloatAtomicExch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSharedInt32Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSharedInt32Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSharedInt32Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSharedFloatAtomicExch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSharedFloatAtomicExch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSharedFloatAtomicExch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasFloatAtomicAdd` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasFloatAtomicAdd` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasFloatAtomicAdd;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasGlobalInt64Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasGlobalInt64Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasGlobalInt64Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSharedInt64Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSharedInt64Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSharedInt64Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasDoubles` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasDoubles` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasDoubles;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasWarpVote` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasWarpVote` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasWarpVote;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasWarpBallot` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasWarpBallot` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasWarpBallot;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasWarpShuffle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasWarpShuffle` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasWarpShuffle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasFunnelShift` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasFunnelShift` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasFunnelShift;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasThreadFenceSystem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasThreadFenceSystem` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasThreadFenceSystem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSyncThreadsExt` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSyncThreadsExt` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSyncThreadsExt;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSurfaceFuncs` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSurfaceFuncs` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSurfaceFuncs;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__has3dGrid` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__has3dGrid` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__has3dGrid;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasDynamicParallelism` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasDynamicParallelism` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasDynamicParallelism;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__concurrentKernels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__concurrentKernels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__concurrentKernels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pciDomainID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pciDomainID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pciDomainID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pciBusID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pciBusID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pciBusID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pciDeviceID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pciDeviceID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pciDeviceID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxSharedMemoryPerMultiProcessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSharedMemoryPerMultiProcessor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSharedMemoryPerMultiProcessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__isMultiGpuBoard` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__isMultiGpuBoard` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__isMultiGpuBoard;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__canMapHostMemory` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__canMapHostMemory` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__canMapHostMemory;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__gcnArch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__gcnArch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__gcnArch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__gcnArchName` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__gcnArchName` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__gcnArchName;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__integrated` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__integrated` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__integrated;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeLaunch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeLaunch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeLaunch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceLaunch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceLaunch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceLaunch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture1DLinear` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture1DLinear` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxTexture1DLinear;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture1D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture1D` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxTexture1D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture2D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture2D` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture2D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture3D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture3D` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture3D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__hdpMemFlushCntl` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hdpMemFlushCntl` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__hdpMemFlushCntl;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__hdpRegFlushCntl` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hdpRegFlushCntl` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__hdpRegFlushCntl;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__memPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__memPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__textureAlignment` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__textureAlignment` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__textureAlignment;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__texturePitchAlignment` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__texturePitchAlignment` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__texturePitchAlignment;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__kernelExecTimeoutEnabled` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__kernelExecTimeoutEnabled` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__kernelExecTimeoutEnabled;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__ECCEnabled` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__ECCEnabled` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__ECCEnabled;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__tccDriver` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__tccDriver` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__tccDriver;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedFunc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedFunc` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedFunc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedGridDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__isLargeBar` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__isLargeBar` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__isLargeBar;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__asicRevision` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__asicRevision` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__asicRevision;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__managedMemory` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__managedMemory` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__managedMemory;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__directManagedMemAccessFromHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__directManagedMemAccessFromHost` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__directManagedMemAccessFromHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__concurrentManagedAccess` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__concurrentManagedAccess` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__concurrentManagedAccess;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pageableMemoryAccess` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pageableMemoryAccess` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pageableMemoryAccess;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pageableMemoryAccessUsesHostPageTables` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pageableMemoryAccessUsesHostPageTables` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pageableMemoryAccessUsesHostPageTables;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGetDevicePropertiesR0000End(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 209);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetDevicePropertiesR0600Begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__prop,
	const uint64_t p__prop__val__name,
	const uint64_t p__prop__val__uuid__bytes,
	const uint64_t p__prop__val__luid,
	const uint32_t p__prop__val__luidDeviceNodeMask,
	const uint64_t p__prop__val__totalGlobalMem,
	const uint64_t p__prop__val__sharedMemPerBlock,
	const int32_t p__prop__val__regsPerBlock,
	const int32_t p__prop__val__warpSize,
	const uint64_t p__prop__val__memPitch,
	const int32_t p__prop__val__maxThreadsPerBlock,
	const uint64_t p__prop__val__maxThreadsDim,
	const uint64_t p__prop__val__maxGridSize,
	const int32_t p__prop__val__clockRate,
	const uint64_t p__prop__val__totalConstMem,
	const int32_t p__prop__val__major,
	const int32_t p__prop__val__minor,
	const uint64_t p__prop__val__textureAlignment,
	const uint64_t p__prop__val__texturePitchAlignment,
	const int32_t p__prop__val__deviceOverlap,
	const int32_t p__prop__val__multiProcessorCount,
	const int32_t p__prop__val__kernelExecTimeoutEnabled,
	const int32_t p__prop__val__integrated,
	const int32_t p__prop__val__canMapHostMemory,
	const int32_t p__prop__val__computeMode,
	const int32_t p__prop__val__maxTexture1D,
	const int32_t p__prop__val__maxTexture1DMipmap,
	const int32_t p__prop__val__maxTexture1DLinear,
	const uint64_t p__prop__val__maxTexture2D,
	const uint64_t p__prop__val__maxTexture2DMipmap,
	const uint64_t p__prop__val__maxTexture2DLinear,
	const uint64_t p__prop__val__maxTexture2DGather,
	const uint64_t p__prop__val__maxTexture3D,
	const uint64_t p__prop__val__maxTexture3DAlt,
	const int32_t p__prop__val__maxTextureCubemap,
	const uint64_t p__prop__val__maxTexture1DLayered,
	const uint64_t p__prop__val__maxTexture2DLayered,
	const uint64_t p__prop__val__maxTextureCubemapLayered,
	const int32_t p__prop__val__maxSurface1D,
	const uint64_t p__prop__val__maxSurface2D,
	const uint64_t p__prop__val__maxSurface3D,
	const uint64_t p__prop__val__maxSurface1DLayered,
	const uint64_t p__prop__val__maxSurface2DLayered,
	const int32_t p__prop__val__maxSurfaceCubemap,
	const uint64_t p__prop__val__maxSurfaceCubemapLayered,
	const uint64_t p__prop__val__surfaceAlignment,
	const int32_t p__prop__val__concurrentKernels,
	const int32_t p__prop__val__ECCEnabled,
	const int32_t p__prop__val__pciBusID,
	const int32_t p__prop__val__pciDeviceID,
	const int32_t p__prop__val__pciDomainID,
	const int32_t p__prop__val__tccDriver,
	const int32_t p__prop__val__asyncEngineCount,
	const int32_t p__prop__val__unifiedAddressing,
	const int32_t p__prop__val__memoryClockRate,
	const int32_t p__prop__val__memoryBusWidth,
	const int32_t p__prop__val__l2CacheSize,
	const int32_t p__prop__val__persistingL2CacheMaxSize,
	const int32_t p__prop__val__maxThreadsPerMultiProcessor,
	const int32_t p__prop__val__streamPrioritiesSupported,
	const int32_t p__prop__val__globalL1CacheSupported,
	const int32_t p__prop__val__localL1CacheSupported,
	const uint64_t p__prop__val__sharedMemPerMultiprocessor,
	const int32_t p__prop__val__regsPerMultiprocessor,
	const int32_t p__prop__val__managedMemory,
	const int32_t p__prop__val__isMultiGpuBoard,
	const int32_t p__prop__val__multiGpuBoardGroupID,
	const int32_t p__prop__val__hostNativeAtomicSupported,
	const int32_t p__prop__val__singleToDoublePrecisionPerfRatio,
	const int32_t p__prop__val__pageableMemoryAccess,
	const int32_t p__prop__val__concurrentManagedAccess,
	const int32_t p__prop__val__computePreemptionSupported,
	const int32_t p__prop__val__canUseHostPointerForRegisteredMem,
	const int32_t p__prop__val__cooperativeLaunch,
	const int32_t p__prop__val__cooperativeMultiDeviceLaunch,
	const uint64_t p__prop__val__sharedMemPerBlockOptin,
	const int32_t p__prop__val__pageableMemoryAccessUsesHostPageTables,
	const int32_t p__prop__val__directManagedMemAccessFromHost,
	const int32_t p__prop__val__maxBlocksPerMultiProcessor,
	const int32_t p__prop__val__accessPolicyMaxWindowSize,
	const uint64_t p__prop__val__reservedSharedMemPerBlock,
	const int32_t p__prop__val__hostRegisterSupported,
	const int32_t p__prop__val__sparseHipArraySupported,
	const int32_t p__prop__val__hostRegisterReadOnlySupported,
	const int32_t p__prop__val__timelineSemaphoreInteropSupported,
	const int32_t p__prop__val__memoryPoolsSupported,
	const int32_t p__prop__val__gpuDirectRDMASupported,
	const uint32_t p__prop__val__gpuDirectRDMAFlushWritesOptions,
	const int32_t p__prop__val__gpuDirectRDMAWritesOrdering,
	const uint32_t p__prop__val__memoryPoolSupportedHandleTypes,
	const int32_t p__prop__val__deferredMappingHipArraySupported,
	const int32_t p__prop__val__ipcEventSupported,
	const int32_t p__prop__val__clusterLaunch,
	const int32_t p__prop__val__unifiedFunctionPointers,
	const uint64_t p__prop__val__reserved,
	const uint64_t p__prop__val__hipReserved,
	const uint64_t p__prop__val__gcnArchName,
	const uint64_t p__prop__val__maxSharedMemoryPerMultiProcessor,
	const int32_t p__prop__val__clockInstructionRate,
	const uint32_t p__prop__val__arch__hasGlobalInt32Atomics,
	const uint32_t p__prop__val__arch__hasGlobalFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasSharedInt32Atomics,
	const uint32_t p__prop__val__arch__hasSharedFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasFloatAtomicAdd,
	const uint32_t p__prop__val__arch__hasGlobalInt64Atomics,
	const uint32_t p__prop__val__arch__hasSharedInt64Atomics,
	const uint32_t p__prop__val__arch__hasDoubles,
	const uint32_t p__prop__val__arch__hasWarpVote,
	const uint32_t p__prop__val__arch__hasWarpBallot,
	const uint32_t p__prop__val__arch__hasWarpShuffle,
	const uint32_t p__prop__val__arch__hasFunnelShift,
	const uint32_t p__prop__val__arch__hasThreadFenceSystem,
	const uint32_t p__prop__val__arch__hasSyncThreadsExt,
	const uint32_t p__prop__val__arch__hasSurfaceFuncs,
	const uint32_t p__prop__val__arch__has3dGrid,
	const uint32_t p__prop__val__arch__hasDynamicParallelism,
	const uint64_t p__prop__val__hdpMemFlushCntl,
	const uint64_t p__prop__val__hdpRegFlushCntl,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedFunc,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedGridDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem,
	const int32_t p__prop__val__isLargeBar,
	const int32_t p__prop__val__asicRevision,
	const int32_t p__deviceId)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 210);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_prop` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__name` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__name;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__uuid__bytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__uuid__bytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__uuid__bytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__luid` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__luid` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__luid;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__luidDeviceNodeMask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__luidDeviceNodeMask` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__luidDeviceNodeMask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__totalGlobalMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__totalGlobalMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__totalGlobalMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__sharedMemPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__sharedMemPerBlock` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__sharedMemPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__regsPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__regsPerBlock` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__regsPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__warpSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__warpSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__warpSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__memPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxThreadsPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxThreadsPerBlock` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxThreadsPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxThreadsDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxThreadsDim` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxThreadsDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxGridSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxGridSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxGridSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__clockRate` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__clockRate` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__clockRate;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__totalConstMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__totalConstMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__totalConstMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__major` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__major` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__major;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__minor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__minor` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__minor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__textureAlignment` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__textureAlignment` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__textureAlignment;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__texturePitchAlignment` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__texturePitchAlignment` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__texturePitchAlignment;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__deviceOverlap` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__deviceOverlap` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__deviceOverlap;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__multiProcessorCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__multiProcessorCount` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__multiProcessorCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__kernelExecTimeoutEnabled` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__kernelExecTimeoutEnabled` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__kernelExecTimeoutEnabled;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__integrated` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__integrated` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__integrated;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__canMapHostMemory` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__canMapHostMemory` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__canMapHostMemory;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__computeMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__computeMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__computeMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture1D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture1D` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxTexture1D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture1DMipmap` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture1DMipmap` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxTexture1DMipmap;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture1DLinear` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture1DLinear` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxTexture1DLinear;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture2D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture2D` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture2D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture2DMipmap` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture2DMipmap` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture2DMipmap;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture2DLinear` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture2DLinear` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture2DLinear;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture2DGather` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture2DGather` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture2DGather;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture3D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture3D` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture3D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture3DAlt` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture3DAlt` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture3DAlt;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTextureCubemap` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTextureCubemap` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxTextureCubemap;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxTexture1DLayered` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture1DLayered` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture1DLayered;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTexture2DLayered` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTexture2DLayered` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTexture2DLayered;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxTextureCubemapLayered` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxTextureCubemapLayered` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxTextureCubemapLayered;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxSurface1D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurface1D` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxSurface1D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxSurface2D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurface2D` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSurface2D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxSurface3D` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurface3D` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSurface3D;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxSurface1DLayered` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurface1DLayered` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSurface1DLayered;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxSurface2DLayered` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurface2DLayered` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSurface2DLayered;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxSurfaceCubemap` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurfaceCubemap` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxSurfaceCubemap;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxSurfaceCubemapLayered` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSurfaceCubemapLayered` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSurfaceCubemapLayered;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__surfaceAlignment` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__surfaceAlignment` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__surfaceAlignment;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__concurrentKernels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__concurrentKernels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__concurrentKernels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__ECCEnabled` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__ECCEnabled` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__ECCEnabled;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pciBusID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pciBusID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pciBusID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pciDeviceID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pciDeviceID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pciDeviceID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pciDomainID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pciDomainID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pciDomainID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__tccDriver` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__tccDriver` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__tccDriver;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__asyncEngineCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__asyncEngineCount` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__asyncEngineCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__unifiedAddressing` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__unifiedAddressing` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__unifiedAddressing;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memoryClockRate` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memoryClockRate` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__memoryClockRate;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memoryBusWidth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memoryBusWidth` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__memoryBusWidth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__l2CacheSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__l2CacheSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__l2CacheSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__persistingL2CacheMaxSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__persistingL2CacheMaxSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__persistingL2CacheMaxSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxThreadsPerMultiProcessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxThreadsPerMultiProcessor` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxThreadsPerMultiProcessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__streamPrioritiesSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__streamPrioritiesSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__streamPrioritiesSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__globalL1CacheSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__globalL1CacheSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__globalL1CacheSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__localL1CacheSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__localL1CacheSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__localL1CacheSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__sharedMemPerMultiprocessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__sharedMemPerMultiprocessor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__sharedMemPerMultiprocessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__regsPerMultiprocessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__regsPerMultiprocessor` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__regsPerMultiprocessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__managedMemory` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__managedMemory` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__managedMemory;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__isMultiGpuBoard` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__isMultiGpuBoard` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__isMultiGpuBoard;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__multiGpuBoardGroupID` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__multiGpuBoardGroupID` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__multiGpuBoardGroupID;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__hostNativeAtomicSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hostNativeAtomicSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__hostNativeAtomicSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__singleToDoublePrecisionPerfRatio` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__singleToDoublePrecisionPerfRatio` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__singleToDoublePrecisionPerfRatio;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__pageableMemoryAccess` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pageableMemoryAccess` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pageableMemoryAccess;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__concurrentManagedAccess` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__concurrentManagedAccess` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__concurrentManagedAccess;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__computePreemptionSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__computePreemptionSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__computePreemptionSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__canUseHostPointerForRegisteredMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__canUseHostPointerForRegisteredMem` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__canUseHostPointerForRegisteredMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeLaunch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeLaunch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeLaunch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceLaunch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceLaunch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceLaunch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__sharedMemPerBlockOptin` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__sharedMemPerBlockOptin` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__sharedMemPerBlockOptin;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__pageableMemoryAccessUsesHostPageTables` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__pageableMemoryAccessUsesHostPageTables` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__pageableMemoryAccessUsesHostPageTables;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__directManagedMemAccessFromHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__directManagedMemAccessFromHost` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__directManagedMemAccessFromHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__maxBlocksPerMultiProcessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxBlocksPerMultiProcessor` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__maxBlocksPerMultiProcessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__accessPolicyMaxWindowSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__accessPolicyMaxWindowSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__accessPolicyMaxWindowSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__reservedSharedMemPerBlock` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__reservedSharedMemPerBlock` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__reservedSharedMemPerBlock;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__hostRegisterSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hostRegisterSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__hostRegisterSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__sparseHipArraySupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__sparseHipArraySupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__sparseHipArraySupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__hostRegisterReadOnlySupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hostRegisterReadOnlySupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__hostRegisterReadOnlySupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__timelineSemaphoreInteropSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__timelineSemaphoreInteropSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__timelineSemaphoreInteropSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memoryPoolsSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memoryPoolsSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__memoryPoolsSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__gpuDirectRDMASupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__gpuDirectRDMASupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__gpuDirectRDMASupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__gpuDirectRDMAFlushWritesOptions` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__gpuDirectRDMAFlushWritesOptions` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__gpuDirectRDMAFlushWritesOptions;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__gpuDirectRDMAWritesOrdering` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__gpuDirectRDMAWritesOrdering` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__gpuDirectRDMAWritesOrdering;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__memoryPoolSupportedHandleTypes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__memoryPoolSupportedHandleTypes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__memoryPoolSupportedHandleTypes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__deferredMappingHipArraySupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__deferredMappingHipArraySupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__deferredMappingHipArraySupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__ipcEventSupported` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__ipcEventSupported` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__ipcEventSupported;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__clusterLaunch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__clusterLaunch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__clusterLaunch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__unifiedFunctionPointers` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__unifiedFunctionPointers` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__unifiedFunctionPointers;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__hipReserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hipReserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__hipReserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__gcnArchName` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__gcnArchName` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__gcnArchName;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__maxSharedMemoryPerMultiProcessor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__maxSharedMemoryPerMultiProcessor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__maxSharedMemoryPerMultiProcessor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__clockInstructionRate` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__clockInstructionRate` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__clockInstructionRate;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasGlobalInt32Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasGlobalInt32Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasGlobalInt32Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasGlobalFloatAtomicExch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasGlobalFloatAtomicExch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasGlobalFloatAtomicExch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSharedInt32Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSharedInt32Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSharedInt32Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSharedFloatAtomicExch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSharedFloatAtomicExch` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSharedFloatAtomicExch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasFloatAtomicAdd` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasFloatAtomicAdd` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasFloatAtomicAdd;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasGlobalInt64Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasGlobalInt64Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasGlobalInt64Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSharedInt64Atomics` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSharedInt64Atomics` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSharedInt64Atomics;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasDoubles` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasDoubles` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasDoubles;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasWarpVote` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasWarpVote` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasWarpVote;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasWarpBallot` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasWarpBallot` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasWarpBallot;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasWarpShuffle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasWarpShuffle` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasWarpShuffle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasFunnelShift` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasFunnelShift` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasFunnelShift;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasThreadFenceSystem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasThreadFenceSystem` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasThreadFenceSystem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSyncThreadsExt` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSyncThreadsExt` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSyncThreadsExt;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasSurfaceFuncs` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasSurfaceFuncs` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasSurfaceFuncs;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__has3dGrid` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__has3dGrid` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__has3dGrid;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__arch__hasDynamicParallelism` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__arch__hasDynamicParallelism` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__arch__hasDynamicParallelism;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__hdpMemFlushCntl` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hdpMemFlushCntl` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__hdpMemFlushCntl;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__hdpRegFlushCntl` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__hdpRegFlushCntl` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__hdpRegFlushCntl;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedFunc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedFunc` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedFunc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedGridDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__isLargeBar` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__isLargeBar` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__isLargeBar;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__asicRevision` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__asicRevision` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__asicRevision;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_deviceId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_deviceId` field */
		{
			const uint32_t tmp_val = (uint32_t) p__deviceId;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGetDevicePropertiesR0600End(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 211);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetErrorStringBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 212);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetErrorStringEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 213);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetFuncBySymbolBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__functionPtr,
	const uint64_t p__functionPtr__val,
	const uint64_t p__symbolPtr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 214);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_functionPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_functionPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__functionPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_functionPtr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_functionPtr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__functionPtr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbolPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbolPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbolPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGetFuncBySymbolEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 215);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetLastErrorBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 216);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetLastErrorEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 217);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetMipmappedArrayLevelBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__levelArray,
	const uint64_t p__levelArray__val,
	const uint64_t p__mipmappedArray,
	const uint32_t p__level)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 218);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_levelArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_levelArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__levelArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_levelArray__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_levelArray__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__levelArray__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mipmappedArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mipmappedArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_level` field */
		_ALIGN(ctx->at, 8);

		/* Write `_level` field */
		{
			const uint32_t tmp_val = (uint32_t) p__level;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGetMipmappedArrayLevelEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 219);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetProcAddressBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const char * const p__symbol,
	const int8_t p__symbol__val,
	const uint64_t p__pfn,
	const uint64_t p__pfn__val,
	const int32_t p__hipVersion,
	const uint64_t p__flags,
	const uint64_t p__symbolStatus,
	const int32_t p__symbolStatus__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 220);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		_write_c_str(ctx, p__symbol);

		/* Align for `_symbol__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol__val` field */
		{
			const uint8_t tmp_val = (uint8_t) p__symbol__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}

		/* Align for `_pfn` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pfn` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pfn;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pfn__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pfn__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pfn__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hipVersion` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hipVersion` field */
		{
			const uint32_t tmp_val = (uint32_t) p__hipVersion;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbolStatus` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbolStatus` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbolStatus;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbolStatus__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbolStatus__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__symbolStatus__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGetProcAddressEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 221);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetSymbolAddressBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr,
	const uint64_t p__devPtr__val,
	const uint64_t p__symbol)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 222);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_devPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__devPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_devPtr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_devPtr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__devPtr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGetSymbolAddressEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 223);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGetSymbolSizeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__size,
	const uint64_t p__size__val,
	const uint64_t p__symbol)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 224);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGetSymbolSizeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 225);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddChildGraphNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__childGraph)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 226);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_childGraph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_childGraph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__childGraph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddChildGraphNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 227);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddDependenciesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__from,
	const uint64_t p__from__val,
	const uint64_t p__to,
	const uint64_t p__to__val,
	const uint64_t p__numDependencies)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 228);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_from` field */
		_ALIGN(ctx->at, 8);

		/* Write `_from` field */
		{
			const uint64_t tmp_val = (uint64_t) p__from;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_from__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_from__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__from__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_to` field */
		_ALIGN(ctx->at, 8);

		/* Write `_to` field */
		{
			const uint64_t tmp_val = (uint64_t) p__to;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_to__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_to__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__to__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddDependenciesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 229);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddEmptyNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 230);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddEmptyNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 231);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddEventRecordNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 232);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddEventRecordNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 233);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddEventWaitNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 234);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddEventWaitNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 235);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddExternalSemaphoresSignalNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__nodeParams,
	const uint64_t p__nodeParams__val__extSemArray,
	const uint64_t p__nodeParams__val__paramsArray,
	const uint32_t p__nodeParams__val__numExtSems)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 236);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__extSemArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__extSemArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__extSemArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__paramsArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__paramsArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__paramsArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__numExtSems` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__numExtSems` field */
		{
			const uint32_t tmp_val = (uint32_t) p__nodeParams__val__numExtSems;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddExternalSemaphoresSignalNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 237);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddExternalSemaphoresWaitNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__nodeParams,
	const uint64_t p__nodeParams__val__extSemArray,
	const uint64_t p__nodeParams__val__paramsArray,
	const uint32_t p__nodeParams__val__numExtSems)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 238);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__extSemArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__extSemArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__extSemArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__paramsArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__paramsArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__paramsArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__numExtSems` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__numExtSems` field */
		{
			const uint32_t tmp_val = (uint32_t) p__nodeParams__val__numExtSems;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddExternalSemaphoresWaitNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 239);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddHostNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__fn,
	const uint64_t p__pNodeParams__val__userData)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 240);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__fn` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__fn` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__fn;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__userData` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__userData` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__userData;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddHostNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 241);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddKernelNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__pNodeParams,
	const uint32_t p__pNodeParams__val__blockDim__x,
	const uint32_t p__pNodeParams__val__blockDim__y,
	const uint32_t p__pNodeParams__val__blockDim__z,
	const uint64_t p__pNodeParams__val__extra,
	const uint64_t p__pNodeParams__val__func,
	const uint32_t p__pNodeParams__val__gridDim__x,
	const uint32_t p__pNodeParams__val__gridDim__y,
	const uint32_t p__pNodeParams__val__gridDim__z,
	const uint64_t p__pNodeParams__val__kernelParams,
	const uint32_t p__pNodeParams__val__sharedMemBytes)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 242);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__blockDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__blockDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__blockDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__blockDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__blockDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__blockDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__blockDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__blockDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__blockDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__extra` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__extra` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__extra;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__func` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__func` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__func;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__gridDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__gridDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__gridDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__gridDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__gridDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__gridDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__gridDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__gridDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__gridDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__kernelParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__kernelParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__kernelParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__sharedMemBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__sharedMemBytes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__sharedMemBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddKernelNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 243);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddMemAllocNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__pNodeParams,
	const int32_t p__pNodeParams__val__poolProps__allocType,
	const int32_t p__pNodeParams__val__poolProps__handleTypes,
	const int32_t p__pNodeParams__val__poolProps__location__type,
	const int32_t p__pNodeParams__val__poolProps__location__id,
	const uint64_t p__pNodeParams__val__poolProps__win32SecurityAttributes,
	const uint64_t p__pNodeParams__val__poolProps__maxSize,
	const uint64_t p__pNodeParams__val__poolProps__reserved,
	const uint64_t p__pNodeParams__val__accessDescs,
	const uint64_t p__pNodeParams__val__accessDescCount,
	const uint64_t p__pNodeParams__val__bytesize,
	const uint64_t p__pNodeParams__val__dptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 244);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__poolProps__allocType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__allocType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__poolProps__allocType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__poolProps__handleTypes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__handleTypes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__poolProps__handleTypes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__poolProps__location__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__location__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__poolProps__location__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__poolProps__location__id` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__location__id` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__poolProps__location__id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__poolProps__win32SecurityAttributes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__win32SecurityAttributes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__poolProps__win32SecurityAttributes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__poolProps__maxSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__maxSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__poolProps__maxSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__poolProps__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__poolProps__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__accessDescs` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__accessDescs` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__accessDescs;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__accessDescCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__accessDescCount` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__accessDescCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__bytesize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__bytesize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__bytesize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddMemAllocNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 245);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddMemFreeNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__dev_ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 246);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddMemFreeNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 247);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddMemcpyNode1DBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__count,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 248);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddMemcpyNode1DEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 249);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddMemcpyNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__pCopyParams,
	const uint64_t p__pCopyParams__val__srcArray,
	const uint64_t p__pCopyParams__val__srcPos__x,
	const uint64_t p__pCopyParams__val__srcPos__y,
	const uint64_t p__pCopyParams__val__srcPos__z,
	const uint64_t p__pCopyParams__val__srcPtr__ptr,
	const uint64_t p__pCopyParams__val__srcPtr__pitch,
	const uint64_t p__pCopyParams__val__srcPtr__xsize,
	const uint64_t p__pCopyParams__val__srcPtr__ysize,
	const uint64_t p__pCopyParams__val__dstArray,
	const uint64_t p__pCopyParams__val__dstPos__x,
	const uint64_t p__pCopyParams__val__dstPos__y,
	const uint64_t p__pCopyParams__val__dstPos__z,
	const uint64_t p__pCopyParams__val__dstPtr__ptr,
	const uint64_t p__pCopyParams__val__dstPtr__pitch,
	const uint64_t p__pCopyParams__val__dstPtr__xsize,
	const uint64_t p__pCopyParams__val__dstPtr__ysize,
	const uint64_t p__pCopyParams__val__extent__width,
	const uint64_t p__pCopyParams__val__extent__height,
	const uint64_t p__pCopyParams__val__extent__depth,
	const int32_t p__pCopyParams__val__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 250);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__srcPos__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__srcPos__x` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__srcPos__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__srcPos__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__srcPos__y` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__srcPos__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__srcPos__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__srcPos__z` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__srcPos__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__srcPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__srcPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__srcPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__srcPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__srcPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__srcPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__srcPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__srcPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__srcPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__srcPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__srcPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__srcPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__dstPos__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__dstPos__x` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__dstPos__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__dstPos__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__dstPos__y` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__dstPos__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__dstPos__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__dstPos__z` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__dstPos__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__dstPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__dstPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__dstPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__dstPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__dstPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__dstPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__dstPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__dstPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__dstPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__dstPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__dstPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__dstPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__extent__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__extent__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__extent__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__extent__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__extent__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__extent__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__extent__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__extent__depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopyParams__val__extent__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopyParams__val__kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopyParams__val__kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pCopyParams__val__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddMemcpyNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 251);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddMemcpyNodeFromSymbolBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__dst,
	const uint64_t p__symbol,
	const uint64_t p__count,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 252);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddMemcpyNodeFromSymbolEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 253);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddMemcpyNodeToSymbolBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__symbol,
	const uint64_t p__src,
	const uint64_t p__count,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 254);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddMemcpyNodeToSymbolEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 255);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddMemsetNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__pMemsetParams,
	const uint64_t p__pMemsetParams__val__dst,
	const uint32_t p__pMemsetParams__val__elementSize,
	const uint64_t p__pMemsetParams__val__height,
	const uint64_t p__pMemsetParams__val__pitch,
	const uint32_t p__pMemsetParams__val__value,
	const uint64_t p__pMemsetParams__val__width)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 256);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pMemsetParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMemsetParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pMemsetParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pMemsetParams__val__dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMemsetParams__val__dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pMemsetParams__val__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pMemsetParams__val__elementSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMemsetParams__val__elementSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pMemsetParams__val__elementSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pMemsetParams__val__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMemsetParams__val__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pMemsetParams__val__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pMemsetParams__val__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMemsetParams__val__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pMemsetParams__val__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pMemsetParams__val__value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMemsetParams__val__value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pMemsetParams__val__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pMemsetParams__val__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMemsetParams__val__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pMemsetParams__val__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddMemsetNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 257);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphAddNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__nodeParams,
	const int32_t p__nodeParams__val__type,
	const uint64_t p__nodeParams__val__reserved0,
	const int64_t p__nodeParams__val__reserved2)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 258);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__nodeParams__val__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_nodeParams__val__reserved0` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__reserved0` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__reserved0;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__reserved2` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__reserved2` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__reserved2;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphAddNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 259);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphChildGraphNodeGetGraphBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pGraph,
	const uint64_t p__pGraph__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 260);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraph__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraph__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraph__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphChildGraphNodeGetGraphEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 261);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphCloneBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphClone,
	const uint64_t p__pGraphClone__val,
	const uint64_t p__originalGraph)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 262);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphClone` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphClone` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphClone;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphClone__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphClone__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphClone__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_originalGraph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_originalGraph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__originalGraph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphCloneEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 263);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphCreateBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraph,
	const uint64_t p__pGraph__val,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 264);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraph__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraph__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraph__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphCreateEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 265);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphDebugDotPrintBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const char * const p__path,
	const int8_t p__path__val,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 266);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_path` field */
		_ALIGN(ctx->at, 8);

		/* Write `_path` field */
		_write_c_str(ctx, p__path);

		/* Align for `_path__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_path__val` field */
		{
			const uint8_t tmp_val = (uint8_t) p__path__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphDebugDotPrintEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 267);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphDestroyBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 268);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphDestroyEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 269);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphDestroyNodeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 270);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphDestroyNodeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 271);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphEventRecordNodeGetEventBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__event_out,
	const uint64_t p__event_out__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 272);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event_out__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event_out__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event_out__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphEventRecordNodeGetEventEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 273);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphEventRecordNodeSetEventBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 274);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphEventRecordNodeSetEventEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 275);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphEventWaitNodeGetEventBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__event_out,
	const uint64_t p__event_out__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 276);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event_out__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event_out__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event_out__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphEventWaitNodeGetEventEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 277);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphEventWaitNodeSetEventBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 278);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphEventWaitNodeSetEventEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 279);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecChildGraphNodeSetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__childGraph)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 280);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_childGraph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_childGraph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__childGraph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecChildGraphNodeSetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 281);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecDestroyBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graphExec)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 282);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_graphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecDestroyEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 283);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecEventRecordNodeSetEventBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hNode,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 284);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecEventRecordNodeSetEventEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 285);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecEventWaitNodeSetEventBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hNode,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 286);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecEventWaitNodeSetEventEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 287);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecExternalSemaphoresSignalNodeSetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hNode,
	const uint64_t p__nodeParams,
	const uint64_t p__nodeParams__val__extSemArray,
	const uint64_t p__nodeParams__val__paramsArray,
	const uint32_t p__nodeParams__val__numExtSems)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 288);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__extSemArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__extSemArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__extSemArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__paramsArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__paramsArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__paramsArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__numExtSems` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__numExtSems` field */
		{
			const uint32_t tmp_val = (uint32_t) p__nodeParams__val__numExtSems;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecExternalSemaphoresSignalNodeSetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 289);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecExternalSemaphoresWaitNodeSetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hNode,
	const uint64_t p__nodeParams,
	const uint64_t p__nodeParams__val__extSemArray,
	const uint64_t p__nodeParams__val__paramsArray,
	const uint32_t p__nodeParams__val__numExtSems)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 290);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__extSemArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__extSemArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__extSemArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__paramsArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__paramsArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__paramsArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__numExtSems` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__numExtSems` field */
		{
			const uint32_t tmp_val = (uint32_t) p__nodeParams__val__numExtSems;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecExternalSemaphoresWaitNodeSetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 291);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecHostNodeSetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__fn,
	const uint64_t p__pNodeParams__val__userData)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 292);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__fn` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__fn` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__fn;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__userData` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__userData` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__userData;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecHostNodeSetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 293);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecKernelNodeSetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint32_t p__pNodeParams__val__blockDim__x,
	const uint32_t p__pNodeParams__val__blockDim__y,
	const uint32_t p__pNodeParams__val__blockDim__z,
	const uint64_t p__pNodeParams__val__extra,
	const uint64_t p__pNodeParams__val__func,
	const uint32_t p__pNodeParams__val__gridDim__x,
	const uint32_t p__pNodeParams__val__gridDim__y,
	const uint32_t p__pNodeParams__val__gridDim__z,
	const uint64_t p__pNodeParams__val__kernelParams,
	const uint32_t p__pNodeParams__val__sharedMemBytes)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 294);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__blockDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__blockDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__blockDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__blockDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__blockDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__blockDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__blockDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__blockDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__blockDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__extra` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__extra` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__extra;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__func` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__func` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__func;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__gridDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__gridDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__gridDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__gridDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__gridDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__gridDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__gridDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__gridDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__gridDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__kernelParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__kernelParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__kernelParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__sharedMemBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__sharedMemBytes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__sharedMemBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecKernelNodeSetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 295);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecMemcpyNodeSetParams1DBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__count,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 296);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecMemcpyNodeSetParams1DEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 297);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecMemcpyNodeSetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__srcArray,
	const uint64_t p__pNodeParams__val__srcPos__x,
	const uint64_t p__pNodeParams__val__srcPos__y,
	const uint64_t p__pNodeParams__val__srcPos__z,
	const uint64_t p__pNodeParams__val__srcPtr__ptr,
	const uint64_t p__pNodeParams__val__srcPtr__pitch,
	const uint64_t p__pNodeParams__val__srcPtr__xsize,
	const uint64_t p__pNodeParams__val__srcPtr__ysize,
	const uint64_t p__pNodeParams__val__dstArray,
	const uint64_t p__pNodeParams__val__dstPos__x,
	const uint64_t p__pNodeParams__val__dstPos__y,
	const uint64_t p__pNodeParams__val__dstPos__z,
	const uint64_t p__pNodeParams__val__dstPtr__ptr,
	const uint64_t p__pNodeParams__val__dstPtr__pitch,
	const uint64_t p__pNodeParams__val__dstPtr__xsize,
	const uint64_t p__pNodeParams__val__dstPtr__ysize,
	const uint64_t p__pNodeParams__val__extent__width,
	const uint64_t p__pNodeParams__val__extent__height,
	const uint64_t p__pNodeParams__val__extent__depth,
	const int32_t p__pNodeParams__val__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 298);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPos__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPos__x` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPos__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPos__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPos__y` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPos__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPos__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPos__z` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPos__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPos__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPos__x` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPos__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPos__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPos__y` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPos__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPos__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPos__z` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPos__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__extent__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__extent__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__extent__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__extent__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__extent__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__extent__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__extent__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__extent__depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__extent__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecMemcpyNodeSetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 299);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecMemcpyNodeSetParamsFromSymbolBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__dst,
	const uint64_t p__symbol,
	const uint64_t p__count,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 300);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecMemcpyNodeSetParamsFromSymbolEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 301);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecMemcpyNodeSetParamsToSymbolBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__symbol,
	const uint64_t p__src,
	const uint64_t p__count,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 302);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecMemcpyNodeSetParamsToSymbolEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 303);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecMemsetNodeSetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__dst,
	const uint32_t p__pNodeParams__val__elementSize,
	const uint64_t p__pNodeParams__val__height,
	const uint64_t p__pNodeParams__val__pitch,
	const uint32_t p__pNodeParams__val__value,
	const uint64_t p__pNodeParams__val__width)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 304);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__elementSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__elementSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__elementSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecMemsetNodeSetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 305);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExecUpdateBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hGraph,
	const uint64_t p__hErrorNode_out,
	const uint64_t p__hErrorNode_out__val,
	const uint64_t p__updateResult_out,
	const int32_t p__updateResult_out__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 306);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hGraph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hErrorNode_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hErrorNode_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hErrorNode_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hErrorNode_out__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hErrorNode_out__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hErrorNode_out__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_updateResult_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_updateResult_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__updateResult_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_updateResult_out__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_updateResult_out__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__updateResult_out__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExecUpdateEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 307);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExternalSemaphoresSignalNodeGetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hNode,
	const uint64_t p__params_out,
	const uint64_t p__params_out__val__extSemArray,
	const uint64_t p__params_out__val__paramsArray,
	const uint32_t p__params_out__val__numExtSems)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 308);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_params_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_params_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__params_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_params_out__val__extSemArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_params_out__val__extSemArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__params_out__val__extSemArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_params_out__val__paramsArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_params_out__val__paramsArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__params_out__val__paramsArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_params_out__val__numExtSems` field */
		_ALIGN(ctx->at, 8);

		/* Write `_params_out__val__numExtSems` field */
		{
			const uint32_t tmp_val = (uint32_t) p__params_out__val__numExtSems;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExternalSemaphoresSignalNodeGetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 309);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExternalSemaphoresSignalNodeSetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hNode,
	const uint64_t p__nodeParams,
	const uint64_t p__nodeParams__val__extSemArray,
	const uint64_t p__nodeParams__val__paramsArray,
	const uint32_t p__nodeParams__val__numExtSems)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 310);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__extSemArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__extSemArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__extSemArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__paramsArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__paramsArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__paramsArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__numExtSems` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__numExtSems` field */
		{
			const uint32_t tmp_val = (uint32_t) p__nodeParams__val__numExtSems;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExternalSemaphoresSignalNodeSetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 311);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExternalSemaphoresWaitNodeGetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hNode,
	const uint64_t p__params_out,
	const uint64_t p__params_out__val__extSemArray,
	const uint64_t p__params_out__val__paramsArray,
	const uint32_t p__params_out__val__numExtSems)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 312);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_params_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_params_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__params_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_params_out__val__extSemArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_params_out__val__extSemArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__params_out__val__extSemArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_params_out__val__paramsArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_params_out__val__paramsArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__params_out__val__paramsArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_params_out__val__numExtSems` field */
		_ALIGN(ctx->at, 8);

		/* Write `_params_out__val__numExtSems` field */
		{
			const uint32_t tmp_val = (uint32_t) p__params_out__val__numExtSems;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExternalSemaphoresWaitNodeGetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 313);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphExternalSemaphoresWaitNodeSetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hNode,
	const uint64_t p__nodeParams,
	const uint64_t p__nodeParams__val__extSemArray,
	const uint64_t p__nodeParams__val__paramsArray,
	const uint32_t p__nodeParams__val__numExtSems)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 314);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__extSemArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__extSemArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__extSemArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__paramsArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__paramsArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodeParams__val__paramsArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodeParams__val__numExtSems` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodeParams__val__numExtSems` field */
		{
			const uint32_t tmp_val = (uint32_t) p__nodeParams__val__numExtSems;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphExternalSemaphoresWaitNodeSetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 315);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphGetEdgesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__from,
	const uint64_t p__from__val,
	const uint64_t p__to,
	const uint64_t p__to__val,
	const uint64_t p__numEdges,
	const uint64_t p__numEdges__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 316);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_from` field */
		_ALIGN(ctx->at, 8);

		/* Write `_from` field */
		{
			const uint64_t tmp_val = (uint64_t) p__from;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_from__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_from__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__from__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_to` field */
		_ALIGN(ctx->at, 8);

		/* Write `_to` field */
		{
			const uint64_t tmp_val = (uint64_t) p__to;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_to__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_to__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__to__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numEdges` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numEdges` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numEdges;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numEdges__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numEdges__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numEdges__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphGetEdgesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 317);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphGetNodesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__nodes,
	const uint64_t p__nodes__val,
	const uint64_t p__numNodes,
	const uint64_t p__numNodes__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 318);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_nodes__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_nodes__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__nodes__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numNodes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numNodes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numNodes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numNodes__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numNodes__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numNodes__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphGetNodesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 319);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphGetRootNodesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__pRootNodes,
	const uint64_t p__pRootNodes__val,
	const uint64_t p__pNumRootNodes,
	const uint64_t p__pNumRootNodes__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 320);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pRootNodes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pRootNodes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pRootNodes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pRootNodes__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pRootNodes__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pRootNodes__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNumRootNodes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNumRootNodes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNumRootNodes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNumRootNodes__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNumRootNodes__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNumRootNodes__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphGetRootNodesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 321);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphHostNodeGetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__fn,
	const uint64_t p__pNodeParams__val__userData)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 322);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__fn` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__fn` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__fn;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__userData` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__userData` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__userData;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphHostNodeGetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 323);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphHostNodeSetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__fn,
	const uint64_t p__pNodeParams__val__userData)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 324);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__fn` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__fn` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__fn;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__userData` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__userData` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__userData;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphHostNodeSetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 325);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphInstantiateBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphExec,
	const uint64_t p__pGraphExec__val,
	const uint64_t p__graph,
	const uint64_t p__pErrorNode,
	const uint64_t p__pErrorNode__val,
	const char * const p__pLogBuffer,
	const int8_t p__pLogBuffer__val,
	const uint64_t p__bufferSize)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 326);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphExec__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphExec__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphExec__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pErrorNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pErrorNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pErrorNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pErrorNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pErrorNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pErrorNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pLogBuffer` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pLogBuffer` field */
		_write_c_str(ctx, p__pLogBuffer);

		/* Align for `_pLogBuffer__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pLogBuffer__val` field */
		{
			const uint8_t tmp_val = (uint8_t) p__pLogBuffer__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}

		/* Align for `_bufferSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_bufferSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__bufferSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphInstantiateEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 327);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphInstantiateWithFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphExec,
	const uint64_t p__pGraphExec__val,
	const uint64_t p__graph,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 328);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphExec__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphExec__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphExec__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphInstantiateWithFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 329);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphInstantiateWithParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphExec,
	const uint64_t p__pGraphExec__val,
	const uint64_t p__graph,
	const uint64_t p__instantiateParams,
	const uint64_t p__instantiateParams__val__errNode_out,
	const uint64_t p__instantiateParams__val__flags,
	const int32_t p__instantiateParams__val__result_out,
	const uint64_t p__instantiateParams__val__uploadStream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 330);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraphExec__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraphExec__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraphExec__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_instantiateParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_instantiateParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__instantiateParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_instantiateParams__val__errNode_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_instantiateParams__val__errNode_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__instantiateParams__val__errNode_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_instantiateParams__val__flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_instantiateParams__val__flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__instantiateParams__val__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_instantiateParams__val__result_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_instantiateParams__val__result_out` field */
		{
			const uint32_t tmp_val = (uint32_t) p__instantiateParams__val__result_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_instantiateParams__val__uploadStream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_instantiateParams__val__uploadStream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__instantiateParams__val__uploadStream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphInstantiateWithParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 331);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphKernelNodeCopyAttributesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hSrc,
	const uint64_t p__hDst)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 332);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hSrc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hSrc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hSrc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hDst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hDst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hDst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphKernelNodeCopyAttributesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 333);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphKernelNodeGetAttributeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hNode,
	const int32_t p__attr,
	const uint64_t p__value,
	const uint64_t p__value__val__accessPolicyWindow__base_ptr,
	const int32_t p__value__val__accessPolicyWindow__hitProp,
	const uint64_t p__value__val__accessPolicyWindow__hitRatio,
	const int32_t p__value__val__accessPolicyWindow__missProp,
	const uint64_t p__value__val__accessPolicyWindow__num_bytes,
	const int32_t p__value__val__cooperative,
	const int32_t p__value__val__priority)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 334);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value__val__accessPolicyWindow__base_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__accessPolicyWindow__base_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value__val__accessPolicyWindow__base_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value__val__accessPolicyWindow__hitProp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__accessPolicyWindow__hitProp` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value__val__accessPolicyWindow__hitProp;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value__val__accessPolicyWindow__hitRatio` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__accessPolicyWindow__hitRatio` field */
		{
			union _f2u f2u;

			f2u.f = p__value__val__accessPolicyWindow__hitRatio;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_value__val__accessPolicyWindow__missProp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__accessPolicyWindow__missProp` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value__val__accessPolicyWindow__missProp;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value__val__accessPolicyWindow__num_bytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__accessPolicyWindow__num_bytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value__val__accessPolicyWindow__num_bytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value__val__cooperative` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__cooperative` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value__val__cooperative;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value__val__priority` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__priority` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value__val__priority;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphKernelNodeGetAttributeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 335);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphKernelNodeGetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint32_t p__pNodeParams__val__blockDim__x,
	const uint32_t p__pNodeParams__val__blockDim__y,
	const uint32_t p__pNodeParams__val__blockDim__z,
	const uint64_t p__pNodeParams__val__extra,
	const uint64_t p__pNodeParams__val__func,
	const uint32_t p__pNodeParams__val__gridDim__x,
	const uint32_t p__pNodeParams__val__gridDim__y,
	const uint32_t p__pNodeParams__val__gridDim__z,
	const uint64_t p__pNodeParams__val__kernelParams,
	const uint32_t p__pNodeParams__val__sharedMemBytes)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 336);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__blockDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__blockDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__blockDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__blockDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__blockDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__blockDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__blockDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__blockDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__blockDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__extra` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__extra` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__extra;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__func` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__func` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__func;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__gridDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__gridDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__gridDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__gridDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__gridDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__gridDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__gridDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__gridDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__gridDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__kernelParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__kernelParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__kernelParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__sharedMemBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__sharedMemBytes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__sharedMemBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphKernelNodeGetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 337);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphKernelNodeSetAttributeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hNode,
	const int32_t p__attr,
	const uint64_t p__value,
	const uint64_t p__value__val__accessPolicyWindow__base_ptr,
	const int32_t p__value__val__accessPolicyWindow__hitProp,
	const uint64_t p__value__val__accessPolicyWindow__hitRatio,
	const int32_t p__value__val__accessPolicyWindow__missProp,
	const uint64_t p__value__val__accessPolicyWindow__num_bytes,
	const int32_t p__value__val__cooperative,
	const int32_t p__value__val__priority)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 338);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value__val__accessPolicyWindow__base_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__accessPolicyWindow__base_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value__val__accessPolicyWindow__base_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value__val__accessPolicyWindow__hitProp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__accessPolicyWindow__hitProp` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value__val__accessPolicyWindow__hitProp;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value__val__accessPolicyWindow__hitRatio` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__accessPolicyWindow__hitRatio` field */
		{
			union _f2u f2u;

			f2u.f = p__value__val__accessPolicyWindow__hitRatio;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_value__val__accessPolicyWindow__missProp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__accessPolicyWindow__missProp` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value__val__accessPolicyWindow__missProp;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value__val__accessPolicyWindow__num_bytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__accessPolicyWindow__num_bytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value__val__accessPolicyWindow__num_bytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value__val__cooperative` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__cooperative` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value__val__cooperative;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value__val__priority` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value__val__priority` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value__val__priority;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphKernelNodeSetAttributeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 339);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphKernelNodeSetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint32_t p__pNodeParams__val__blockDim__x,
	const uint32_t p__pNodeParams__val__blockDim__y,
	const uint32_t p__pNodeParams__val__blockDim__z,
	const uint64_t p__pNodeParams__val__extra,
	const uint64_t p__pNodeParams__val__func,
	const uint32_t p__pNodeParams__val__gridDim__x,
	const uint32_t p__pNodeParams__val__gridDim__y,
	const uint32_t p__pNodeParams__val__gridDim__z,
	const uint64_t p__pNodeParams__val__kernelParams,
	const uint32_t p__pNodeParams__val__sharedMemBytes)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 340);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__blockDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__blockDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__blockDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__blockDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__blockDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__blockDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__blockDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__blockDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__blockDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__extra` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__extra` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__extra;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__func` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__func` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__func;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__gridDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__gridDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__gridDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__gridDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__gridDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__gridDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__gridDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__gridDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__gridDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__kernelParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__kernelParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__kernelParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__sharedMemBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__sharedMemBytes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__sharedMemBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphKernelNodeSetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 341);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphLaunchBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graphExec,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 342);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_graphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphLaunchEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 343);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphMemAllocNodeGetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const int32_t p__pNodeParams__val__poolProps__allocType,
	const int32_t p__pNodeParams__val__poolProps__handleTypes,
	const int32_t p__pNodeParams__val__poolProps__location__type,
	const int32_t p__pNodeParams__val__poolProps__location__id,
	const uint64_t p__pNodeParams__val__poolProps__win32SecurityAttributes,
	const uint64_t p__pNodeParams__val__poolProps__maxSize,
	const uint64_t p__pNodeParams__val__poolProps__reserved,
	const uint64_t p__pNodeParams__val__accessDescs,
	const uint64_t p__pNodeParams__val__accessDescCount,
	const uint64_t p__pNodeParams__val__bytesize,
	const uint64_t p__pNodeParams__val__dptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 344);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__poolProps__allocType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__allocType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__poolProps__allocType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__poolProps__handleTypes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__handleTypes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__poolProps__handleTypes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__poolProps__location__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__location__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__poolProps__location__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__poolProps__location__id` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__location__id` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__poolProps__location__id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__poolProps__win32SecurityAttributes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__win32SecurityAttributes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__poolProps__win32SecurityAttributes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__poolProps__maxSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__maxSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__poolProps__maxSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__poolProps__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__poolProps__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__poolProps__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__accessDescs` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__accessDescs` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__accessDescs;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__accessDescCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__accessDescCount` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__accessDescCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__bytesize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__bytesize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__bytesize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphMemAllocNodeGetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 345);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphMemFreeNodeGetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__dev_ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 346);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphMemFreeNodeGetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 347);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphMemcpyNodeGetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__srcArray,
	const uint64_t p__pNodeParams__val__srcPos__x,
	const uint64_t p__pNodeParams__val__srcPos__y,
	const uint64_t p__pNodeParams__val__srcPos__z,
	const uint64_t p__pNodeParams__val__srcPtr__ptr,
	const uint64_t p__pNodeParams__val__srcPtr__pitch,
	const uint64_t p__pNodeParams__val__srcPtr__xsize,
	const uint64_t p__pNodeParams__val__srcPtr__ysize,
	const uint64_t p__pNodeParams__val__dstArray,
	const uint64_t p__pNodeParams__val__dstPos__x,
	const uint64_t p__pNodeParams__val__dstPos__y,
	const uint64_t p__pNodeParams__val__dstPos__z,
	const uint64_t p__pNodeParams__val__dstPtr__ptr,
	const uint64_t p__pNodeParams__val__dstPtr__pitch,
	const uint64_t p__pNodeParams__val__dstPtr__xsize,
	const uint64_t p__pNodeParams__val__dstPtr__ysize,
	const uint64_t p__pNodeParams__val__extent__width,
	const uint64_t p__pNodeParams__val__extent__height,
	const uint64_t p__pNodeParams__val__extent__depth,
	const int32_t p__pNodeParams__val__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 348);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPos__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPos__x` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPos__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPos__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPos__y` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPos__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPos__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPos__z` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPos__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPos__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPos__x` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPos__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPos__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPos__y` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPos__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPos__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPos__z` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPos__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__extent__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__extent__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__extent__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__extent__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__extent__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__extent__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__extent__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__extent__depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__extent__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphMemcpyNodeGetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 349);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphMemcpyNodeSetParams1DBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__count,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 350);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphMemcpyNodeSetParams1DEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 351);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphMemcpyNodeSetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__srcArray,
	const uint64_t p__pNodeParams__val__srcPos__x,
	const uint64_t p__pNodeParams__val__srcPos__y,
	const uint64_t p__pNodeParams__val__srcPos__z,
	const uint64_t p__pNodeParams__val__srcPtr__ptr,
	const uint64_t p__pNodeParams__val__srcPtr__pitch,
	const uint64_t p__pNodeParams__val__srcPtr__xsize,
	const uint64_t p__pNodeParams__val__srcPtr__ysize,
	const uint64_t p__pNodeParams__val__dstArray,
	const uint64_t p__pNodeParams__val__dstPos__x,
	const uint64_t p__pNodeParams__val__dstPos__y,
	const uint64_t p__pNodeParams__val__dstPos__z,
	const uint64_t p__pNodeParams__val__dstPtr__ptr,
	const uint64_t p__pNodeParams__val__dstPtr__pitch,
	const uint64_t p__pNodeParams__val__dstPtr__xsize,
	const uint64_t p__pNodeParams__val__dstPtr__ysize,
	const uint64_t p__pNodeParams__val__extent__width,
	const uint64_t p__pNodeParams__val__extent__height,
	const uint64_t p__pNodeParams__val__extent__depth,
	const int32_t p__pNodeParams__val__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 352);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPos__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPos__x` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPos__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPos__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPos__y` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPos__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPos__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPos__z` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPos__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__srcPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__srcPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__srcPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPos__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPos__x` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPos__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPos__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPos__y` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPos__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPos__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPos__z` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPos__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dstPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dstPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dstPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__extent__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__extent__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__extent__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__extent__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__extent__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__extent__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__extent__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__extent__depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__extent__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphMemcpyNodeSetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 353);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphMemcpyNodeSetParamsFromSymbolBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__dst,
	const uint64_t p__symbol,
	const uint64_t p__count,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 354);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphMemcpyNodeSetParamsFromSymbolEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 355);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphMemcpyNodeSetParamsToSymbolBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__symbol,
	const uint64_t p__src,
	const uint64_t p__count,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 356);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphMemcpyNodeSetParamsToSymbolEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 357);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphMemsetNodeGetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__dst,
	const uint32_t p__pNodeParams__val__elementSize,
	const uint64_t p__pNodeParams__val__height,
	const uint64_t p__pNodeParams__val__pitch,
	const uint32_t p__pNodeParams__val__value,
	const uint64_t p__pNodeParams__val__width)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 358);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__elementSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__elementSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__elementSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphMemsetNodeGetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 359);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphMemsetNodeSetParamsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__dst,
	const uint32_t p__pNodeParams__val__elementSize,
	const uint64_t p__pNodeParams__val__height,
	const uint64_t p__pNodeParams__val__pitch,
	const uint32_t p__pNodeParams__val__value,
	const uint64_t p__pNodeParams__val__width)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 360);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__elementSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__elementSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__elementSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNodeParams__val__value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNodeParams__val__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNodeParams__val__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNodeParams__val__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNodeParams__val__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphMemsetNodeSetParamsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 361);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphNodeFindInCloneBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pNode,
	const uint64_t p__pNode__val,
	const uint64_t p__originalNode,
	const uint64_t p__clonedGraph)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 362);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNode__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_originalNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_originalNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__originalNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_clonedGraph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_clonedGraph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__clonedGraph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphNodeFindInCloneEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 363);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphNodeGetDependenciesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__pNumDependencies,
	const uint64_t p__pNumDependencies__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 364);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNumDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNumDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNumDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNumDependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNumDependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNumDependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphNodeGetDependenciesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 365);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphNodeGetDependentNodesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pDependentNodes,
	const uint64_t p__pDependentNodes__val,
	const uint64_t p__pNumDependentNodes,
	const uint64_t p__pNumDependentNodes__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 366);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependentNodes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependentNodes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependentNodes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pDependentNodes__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pDependentNodes__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pDependentNodes__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNumDependentNodes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNumDependentNodes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNumDependentNodes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNumDependentNodes__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNumDependentNodes__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNumDependentNodes__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphNodeGetDependentNodesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 367);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphNodeGetEnabledBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hNode,
	const uint64_t p__isEnabled,
	const uint32_t p__isEnabled__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 368);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_isEnabled` field */
		_ALIGN(ctx->at, 8);

		/* Write `_isEnabled` field */
		{
			const uint64_t tmp_val = (uint64_t) p__isEnabled;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_isEnabled__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_isEnabled__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__isEnabled__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphNodeGetEnabledEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 369);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphNodeGetTypeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pType,
	const int32_t p__pType__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 370);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_node` field */
		_ALIGN(ctx->at, 8);

		/* Write `_node` field */
		{
			const uint64_t tmp_val = (uint64_t) p__node;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pType` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pType__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pType__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pType__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphNodeGetTypeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 371);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphNodeSetEnabledBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hNode,
	const uint32_t p__isEnabled)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 372);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hGraphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hGraphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hNode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hNode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hNode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_isEnabled` field */
		_ALIGN(ctx->at, 8);

		/* Write `_isEnabled` field */
		{
			const uint32_t tmp_val = (uint32_t) p__isEnabled;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphNodeSetEnabledEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 373);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphReleaseUserObjectBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__object,
	const uint32_t p__count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 374);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_object` field */
		_ALIGN(ctx->at, 8);

		/* Write `_object` field */
		{
			const uint64_t tmp_val = (uint64_t) p__object;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint32_t tmp_val = (uint32_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphReleaseUserObjectEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 375);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphRemoveDependenciesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__from,
	const uint64_t p__from__val,
	const uint64_t p__to,
	const uint64_t p__to__val,
	const uint64_t p__numDependencies)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 376);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_from` field */
		_ALIGN(ctx->at, 8);

		/* Write `_from` field */
		{
			const uint64_t tmp_val = (uint64_t) p__from;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_from__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_from__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__from__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_to` field */
		_ALIGN(ctx->at, 8);

		/* Write `_to` field */
		{
			const uint64_t tmp_val = (uint64_t) p__to;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_to__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_to__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__to__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphRemoveDependenciesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 377);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphRetainUserObjectBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__object,
	const uint32_t p__count,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 378);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_object` field */
		_ALIGN(ctx->at, 8);

		/* Write `_object` field */
		{
			const uint64_t tmp_val = (uint64_t) p__object;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint32_t tmp_val = (uint32_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphRetainUserObjectEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 379);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphUploadBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graphExec,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 380);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_graphExec` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graphExec` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graphExec;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphUploadEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 381);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphicsGLRegisterBufferBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__resource,
	const uint64_t p__resource__val,
	const int32_t p__buffer,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 382);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_resource` field */
		_ALIGN(ctx->at, 8);

		/* Write `_resource` field */
		{
			const uint64_t tmp_val = (uint64_t) p__resource;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_resource__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_resource__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__resource__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_buffer` field */
		_ALIGN(ctx->at, 8);

		/* Write `_buffer` field */
		{
			const uint32_t tmp_val = (uint32_t) p__buffer;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphicsGLRegisterBufferEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 383);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphicsGLRegisterImageBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__resource,
	const uint64_t p__resource__val,
	const int32_t p__image,
	const int32_t p__target,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 384);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_resource` field */
		_ALIGN(ctx->at, 8);

		/* Write `_resource` field */
		{
			const uint64_t tmp_val = (uint64_t) p__resource;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_resource__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_resource__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__resource__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image` field */
		{
			const uint32_t tmp_val = (uint32_t) p__image;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_target` field */
		_ALIGN(ctx->at, 8);

		/* Write `_target` field */
		{
			const uint32_t tmp_val = (uint32_t) p__target;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphicsGLRegisterImageEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 385);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphicsMapResourcesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__count,
	const uint64_t p__resources,
	const uint64_t p__resources__val,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 386);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint32_t tmp_val = (uint32_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_resources` field */
		_ALIGN(ctx->at, 8);

		/* Write `_resources` field */
		{
			const uint64_t tmp_val = (uint64_t) p__resources;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_resources__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_resources__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__resources__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphicsMapResourcesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 387);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphicsResourceGetMappedPointerBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr,
	const uint64_t p__devPtr__val,
	const uint64_t p__size,
	const uint64_t p__size__val,
	const uint64_t p__resource)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 388);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_devPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__devPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_devPtr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_devPtr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__devPtr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_resource` field */
		_ALIGN(ctx->at, 8);

		/* Write `_resource` field */
		{
			const uint64_t tmp_val = (uint64_t) p__resource;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphicsResourceGetMappedPointerEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 389);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphicsSubResourceGetMappedArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__array,
	const uint64_t p__array__val,
	const uint64_t p__resource,
	const uint32_t p__arrayIndex,
	const uint32_t p__mipLevel)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 390);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_array` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_array__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_resource` field */
		_ALIGN(ctx->at, 8);

		/* Write `_resource` field */
		{
			const uint64_t tmp_val = (uint64_t) p__resource;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_arrayIndex` field */
		_ALIGN(ctx->at, 8);

		/* Write `_arrayIndex` field */
		{
			const uint32_t tmp_val = (uint32_t) p__arrayIndex;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipLevel` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipLevel` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipLevel;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipGraphicsSubResourceGetMappedArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 391);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphicsUnmapResourcesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__count,
	const uint64_t p__resources,
	const uint64_t p__resources__val,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 392);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint32_t tmp_val = (uint32_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_resources` field */
		_ALIGN(ctx->at, 8);

		/* Write `_resources` field */
		{
			const uint64_t tmp_val = (uint64_t) p__resources;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_resources__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_resources__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__resources__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphicsUnmapResourcesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 393);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipGraphicsUnregisterResourceBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__resource)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 394);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_resource` field */
		_ALIGN(ctx->at, 8);

		/* Write `_resource` field */
		{
			const uint64_t tmp_val = (uint64_t) p__resource;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipGraphicsUnregisterResourceEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 395);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipHccModuleLaunchKernelBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__f,
	const uint32_t p__globalWorkSizeX,
	const uint32_t p__globalWorkSizeY,
	const uint32_t p__globalWorkSizeZ,
	const uint32_t p__blockDimX,
	const uint32_t p__blockDimY,
	const uint32_t p__blockDimZ,
	const uint64_t p__sharedMemBytes,
	const uint64_t p__hStream,
	const uint64_t p__kernelParams,
	const uint64_t p__kernelParams__val,
	const uint64_t p__extra,
	const uint64_t p__extra__val,
	const uint64_t p__startEvent,
	const uint64_t p__stopEvent)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 396);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_f` field */
		{
			const uint64_t tmp_val = (uint64_t) p__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_globalWorkSizeX` field */
		_ALIGN(ctx->at, 8);

		/* Write `_globalWorkSizeX` field */
		{
			const uint32_t tmp_val = (uint32_t) p__globalWorkSizeX;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_globalWorkSizeY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_globalWorkSizeY` field */
		{
			const uint32_t tmp_val = (uint32_t) p__globalWorkSizeY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_globalWorkSizeZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_globalWorkSizeZ` field */
		{
			const uint32_t tmp_val = (uint32_t) p__globalWorkSizeZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDimX` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDimX` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDimX;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDimY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDimY` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDimY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDimZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDimZ` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDimZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_sharedMemBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sharedMemBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sharedMemBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hStream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hStream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hStream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kernelParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kernelParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__kernelParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kernelParams__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kernelParams__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__kernelParams__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extra` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extra` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extra;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extra__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extra__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extra__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_startEvent` field */
		_ALIGN(ctx->at, 8);

		/* Write `_startEvent` field */
		{
			const uint64_t tmp_val = (uint64_t) p__startEvent;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stopEvent` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stopEvent` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stopEvent;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipHccModuleLaunchKernelEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 397);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipHostAllocBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__size,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 398);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipHostAllocEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 399);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipHostFreeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 400);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipHostFreeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 401);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipHostGetDevicePointerBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr,
	const uint64_t p__devPtr__val,
	const uint64_t p__hstPtr,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 402);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_devPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__devPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_devPtr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_devPtr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__devPtr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hstPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hstPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hstPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipHostGetDevicePointerEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 403);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipHostGetFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__flagsPtr,
	const uint32_t p__flagsPtr__val,
	const uint64_t p__hostPtr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 404);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_flagsPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flagsPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flagsPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flagsPtr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flagsPtr__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flagsPtr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_hostPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hostPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hostPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipHostGetFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 405);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipHostMallocBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__size,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 406);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipHostMallocEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 407);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipHostRegisterBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hostPtr,
	const uint64_t p__sizeBytes,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 408);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hostPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hostPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hostPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipHostRegisterEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 409);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipHostUnregisterBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hostPtr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 410);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hostPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hostPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hostPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipHostUnregisterEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 411);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipImportExternalMemoryBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__extMem_out,
	const uint64_t p__extMem_out__val,
	const uint64_t p__memHandleDesc,
	const int32_t p__memHandleDesc__val__type,
	const uint64_t p__memHandleDesc__val__size,
	const uint32_t p__memHandleDesc__val__flags,
	const uint64_t p__memHandleDesc__val__reserved)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 412);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_extMem_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extMem_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extMem_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extMem_out__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extMem_out__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extMem_out__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_memHandleDesc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memHandleDesc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memHandleDesc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_memHandleDesc__val__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memHandleDesc__val__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__memHandleDesc__val__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_memHandleDesc__val__size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memHandleDesc__val__size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memHandleDesc__val__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_memHandleDesc__val__flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memHandleDesc__val__flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__memHandleDesc__val__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_memHandleDesc__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memHandleDesc__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memHandleDesc__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipImportExternalMemoryEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 413);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipImportExternalSemaphoreBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__extSem_out,
	const uint64_t p__extSem_out__val,
	const uint64_t p__semHandleDesc,
	const int32_t p__semHandleDesc__val__type,
	const uint32_t p__semHandleDesc__val__flags,
	const uint64_t p__semHandleDesc__val__reserved)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 414);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_extSem_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extSem_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extSem_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extSem_out__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extSem_out__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extSem_out__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_semHandleDesc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_semHandleDesc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__semHandleDesc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_semHandleDesc__val__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_semHandleDesc__val__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__semHandleDesc__val__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_semHandleDesc__val__flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_semHandleDesc__val__flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__semHandleDesc__val__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_semHandleDesc__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_semHandleDesc__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__semHandleDesc__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipImportExternalSemaphoreEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 415);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipInitBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 416);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipInitEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 417);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipIpcCloseMemHandleBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 418);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_devPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__devPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipIpcCloseMemHandleEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 419);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipIpcGetEventHandleBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__handle,
	const uint64_t p__handle__val__reserved,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 420);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipIpcGetEventHandleEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 421);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipIpcGetMemHandleBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__handle,
	const uint64_t p__handle__val__reserved,
	const uint64_t p__devPtr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 422);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_devPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_devPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__devPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipIpcGetMemHandleEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 423);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipIpcOpenEventHandleBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event,
	const uint64_t p__event__val,
	const uint64_t p__handle__reserved)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 424);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipIpcOpenEventHandleEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 425);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipIpcOpenMemHandleBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr,
	const uint64_t p__devPtr__val,
	const uint64_t p__handle__reserved,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 426);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_devPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__devPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_devPtr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_devPtr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__devPtr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipIpcOpenMemHandleEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 427);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipLaunchByPtrBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hostFunction)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 428);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hostFunction` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hostFunction` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hostFunction;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipLaunchByPtrEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 429);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipLaunchCooperativeKernelBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__f,
	const uint32_t p__gridDim__x,
	const uint32_t p__gridDim__y,
	const uint32_t p__gridDim__z,
	const uint32_t p__blockDimX__x,
	const uint32_t p__blockDimX__y,
	const uint32_t p__blockDimX__z,
	const uint64_t p__kernelParams,
	const uint64_t p__kernelParams__val,
	const uint32_t p__sharedMemBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 430);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_f` field */
		{
			const uint64_t tmp_val = (uint64_t) p__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_gridDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_gridDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_gridDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDimX__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDimX__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDimX__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDimX__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDimX__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDimX__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDimX__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDimX__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDimX__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_kernelParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kernelParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__kernelParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kernelParams__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kernelParams__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__kernelParams__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sharedMemBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sharedMemBytes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__sharedMemBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipLaunchCooperativeKernelEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 431);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipLaunchCooperativeKernelMultiDeviceBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__launchParamsList,
	const uint64_t p__launchParamsList__val__func,
	const uint32_t p__launchParamsList__val__gridDim__x,
	const uint32_t p__launchParamsList__val__gridDim__y,
	const uint32_t p__launchParamsList__val__gridDim__z,
	const uint32_t p__launchParamsList__val__blockDim__x,
	const uint32_t p__launchParamsList__val__blockDim__y,
	const uint32_t p__launchParamsList__val__blockDim__z,
	const uint64_t p__launchParamsList__val__args,
	const uint64_t p__launchParamsList__val__sharedMem,
	const uint64_t p__launchParamsList__val__stream,
	const int32_t p__numDevices,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 432);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_launchParamsList` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_launchParamsList__val__func` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__func` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList__val__func;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_launchParamsList__val__gridDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__gridDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__gridDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__gridDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__gridDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__gridDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__gridDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__gridDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__gridDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__blockDim__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__blockDim__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__blockDim__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__blockDim__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__blockDim__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__blockDim__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__blockDim__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__blockDim__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__blockDim__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__args` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__args` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList__val__args;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_launchParamsList__val__sharedMem` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__sharedMem` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList__val__sharedMem;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_launchParamsList__val__stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList__val__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDevices` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDevices` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numDevices;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipLaunchCooperativeKernelMultiDeviceEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 433);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipLaunchHostFuncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__fn,
	const uint64_t p__userData)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 434);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_fn` field */
		_ALIGN(ctx->at, 8);

		/* Write `_fn` field */
		{
			const uint64_t tmp_val = (uint64_t) p__fn;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_userData` field */
		_ALIGN(ctx->at, 8);

		/* Write `_userData` field */
		{
			const uint64_t tmp_val = (uint64_t) p__userData;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipLaunchHostFuncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 435);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipLaunchKernelBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__function_address,
	const uint32_t p__numBlocks__x,
	const uint32_t p__numBlocks__y,
	const uint32_t p__numBlocks__z,
	const uint32_t p__dimBlocks__x,
	const uint32_t p__dimBlocks__y,
	const uint32_t p__dimBlocks__z,
	const uint64_t p__args,
	const uint64_t p__args__val,
	const uint64_t p__sharedMemBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 436);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_function_address` field */
		_ALIGN(ctx->at, 8);

		/* Write `_function_address` field */
		{
			const uint64_t tmp_val = (uint64_t) p__function_address;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numBlocks__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numBlocks__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_numBlocks__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numBlocks__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_numBlocks__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numBlocks__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dimBlocks__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dimBlocks__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dimBlocks__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dimBlocks__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dimBlocks__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dimBlocks__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dimBlocks__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dimBlocks__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dimBlocks__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_args` field */
		_ALIGN(ctx->at, 8);

		/* Write `_args` field */
		{
			const uint64_t tmp_val = (uint64_t) p__args;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_args__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_args__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__args__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sharedMemBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sharedMemBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sharedMemBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipLaunchKernelEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 437);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMalloc3DArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__array,
	const uint64_t p__array__val,
	const uint64_t p__desc,
	const int32_t p__desc__val__x,
	const int32_t p__desc__val__y,
	const int32_t p__desc__val__z,
	const int32_t p__desc__val__w,
	const int32_t p__desc__val__f,
	const uint64_t p__extent__width,
	const uint64_t p__extent__height,
	const uint64_t p__extent__depth,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 438);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_array` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_array__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__desc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc__val__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_extent__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMalloc3DArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 439);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMalloc3DBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pitchedDevPtr,
	const uint64_t p__pitchedDevPtr__val__ptr,
	const uint64_t p__pitchedDevPtr__val__pitch,
	const uint64_t p__pitchedDevPtr__val__xsize,
	const uint64_t p__pitchedDevPtr__val__ysize,
	const uint64_t p__extent__width,
	const uint64_t p__extent__height,
	const uint64_t p__extent__depth)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 440);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pitchedDevPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitchedDevPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitchedDevPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitchedDevPtr__val__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitchedDevPtr__val__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitchedDevPtr__val__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitchedDevPtr__val__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitchedDevPtr__val__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitchedDevPtr__val__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitchedDevPtr__val__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitchedDevPtr__val__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitchedDevPtr__val__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitchedDevPtr__val__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitchedDevPtr__val__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitchedDevPtr__val__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMalloc3DEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 441);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMallocArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__array,
	const uint64_t p__array__val,
	const uint64_t p__desc,
	const int32_t p__desc__val__x,
	const int32_t p__desc__val__y,
	const int32_t p__desc__val__z,
	const int32_t p__desc__val__w,
	const int32_t p__desc__val__f,
	const uint64_t p__width,
	const uint64_t p__height,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 442);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_array` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_array__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__desc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc__val__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMallocArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 443);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMallocAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__dev_ptr__val,
	const uint64_t p__size,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 444);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dev_ptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMallocAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 445);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMallocBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 446);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMallocEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 447);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMallocFromPoolAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__dev_ptr__val,
	const uint64_t p__size,
	const uint64_t p__mem_pool,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 448);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dev_ptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMallocFromPoolAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 449);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMallocHostBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 450);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMallocHostEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 451);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMallocManagedBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__dev_ptr__val,
	const uint64_t p__size,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 452);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dev_ptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMallocManagedEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 453);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMallocMipmappedArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mipmappedArray,
	const uint64_t p__mipmappedArray__val,
	const uint64_t p__desc,
	const int32_t p__desc__val__x,
	const int32_t p__desc__val__y,
	const int32_t p__desc__val__z,
	const int32_t p__desc__val__w,
	const int32_t p__desc__val__f,
	const uint64_t p__extent__width,
	const uint64_t p__extent__height,
	const uint64_t p__extent__depth,
	const uint32_t p__numLevels,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 454);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mipmappedArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mipmappedArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mipmappedArray__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mipmappedArray__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__desc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc__val__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_extent__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numLevels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numLevels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numLevels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMallocMipmappedArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 455);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMallocPitchBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__pitch,
	const uint64_t p__pitch__val,
	const uint64_t p__width,
	const uint64_t p__height)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 456);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitch__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitch__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitch__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMallocPitchEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 457);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemAddressFreeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 458);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_devPtr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__devPtr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemAddressFreeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 459);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemAddressReserveBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__size,
	const uint64_t p__alignment,
	const uint64_t p__addr,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 460);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_alignment` field */
		_ALIGN(ctx->at, 8);

		/* Write `_alignment` field */
		{
			const uint64_t tmp_val = (uint64_t) p__alignment;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_addr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_addr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__addr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemAddressReserveEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 461);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemAdviseBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__count,
	const int32_t p__advice,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 462);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_advice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_advice` field */
		{
			const uint32_t tmp_val = (uint32_t) p__advice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemAdviseEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 463);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemAllocHostBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 464);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemAllocHostEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 465);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemAllocPitchBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dptr,
	const uint64_t p__dptr__val,
	const uint64_t p__pitch,
	const uint64_t p__pitch__val,
	const uint64_t p__widthInBytes,
	const uint64_t p__height,
	const uint32_t p__elementSizeBytes)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 466);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitch__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitch__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitch__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_widthInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_widthInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__widthInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_elementSizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_elementSizeBytes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__elementSizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemAllocPitchEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 467);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemCreateBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__handle,
	const uint64_t p__handle__val,
	const uint64_t p__size,
	const uint64_t p__prop,
	const int32_t p__prop__val__type,
	const int32_t p__prop__val__requestedHandleType,
	const int32_t p__prop__val__location__type,
	const int32_t p__prop__val__location__id,
	const uint64_t p__prop__val__win32HandleMetaData,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 468);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__requestedHandleType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__requestedHandleType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__requestedHandleType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__location__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__location__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__location__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__location__id` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__location__id` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__location__id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__win32HandleMetaData` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__win32HandleMetaData` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__win32HandleMetaData;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemCreateEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 469);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemExportToShareableHandleBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__shareableHandle,
	const uint64_t p__handle,
	const int32_t p__handleType,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 470);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_shareableHandle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_shareableHandle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__shareableHandle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handleType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handleType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__handleType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemExportToShareableHandleEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 471);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemGetAccessBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__flags,
	const uint64_t p__flags__val,
	const uint64_t p__location,
	const int32_t p__location__val__type,
	const int32_t p__location__val__id,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 472);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_location` field */
		_ALIGN(ctx->at, 8);

		/* Write `_location` field */
		{
			const uint64_t tmp_val = (uint64_t) p__location;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_location__val__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_location__val__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__location__val__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_location__val__id` field */
		_ALIGN(ctx->at, 8);

		/* Write `_location__val__id` field */
		{
			const uint32_t tmp_val = (uint32_t) p__location__val__id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemGetAccessEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 473);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemGetAddressRangeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pbase,
	const uint64_t p__pbase__val,
	const uint64_t p__psize,
	const uint64_t p__psize__val,
	const uint64_t p__dptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 474);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pbase` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pbase` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pbase;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pbase__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pbase__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pbase__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_psize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_psize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__psize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_psize__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_psize__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__psize__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemGetAddressRangeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 475);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemGetAllocationGranularityBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__granularity,
	const uint64_t p__granularity__val,
	const uint64_t p__prop,
	const int32_t p__prop__val__type,
	const int32_t p__prop__val__requestedHandleType,
	const int32_t p__prop__val__location__type,
	const int32_t p__prop__val__location__id,
	const uint64_t p__prop__val__win32HandleMetaData,
	const int32_t p__option)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 476);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_granularity` field */
		_ALIGN(ctx->at, 8);

		/* Write `_granularity` field */
		{
			const uint64_t tmp_val = (uint64_t) p__granularity;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_granularity__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_granularity__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__granularity__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__requestedHandleType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__requestedHandleType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__requestedHandleType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__location__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__location__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__location__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__location__id` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__location__id` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__location__id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__win32HandleMetaData` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__win32HandleMetaData` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__win32HandleMetaData;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_option` field */
		_ALIGN(ctx->at, 8);

		/* Write `_option` field */
		{
			const uint32_t tmp_val = (uint32_t) p__option;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemGetAllocationGranularityEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 477);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemGetAllocationPropertiesFromHandleBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__prop,
	const int32_t p__prop__val__type,
	const int32_t p__prop__val__requestedHandleType,
	const int32_t p__prop__val__location__type,
	const int32_t p__prop__val__location__id,
	const uint64_t p__prop__val__win32HandleMetaData,
	const uint64_t p__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 478);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_prop` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_prop__val__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__requestedHandleType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__requestedHandleType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__requestedHandleType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__location__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__location__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__location__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__location__id` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__location__id` field */
		{
			const uint32_t tmp_val = (uint32_t) p__prop__val__location__id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_prop__val__win32HandleMetaData` field */
		_ALIGN(ctx->at, 8);

		/* Write `_prop__val__win32HandleMetaData` field */
		{
			const uint64_t tmp_val = (uint64_t) p__prop__val__win32HandleMetaData;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemGetAllocationPropertiesFromHandleEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 479);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemGetInfoBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__free,
	const uint64_t p__free__val,
	const uint64_t p__total,
	const uint64_t p__total__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 480);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_free` field */
		_ALIGN(ctx->at, 8);

		/* Write `_free` field */
		{
			const uint64_t tmp_val = (uint64_t) p__free;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_free__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_free__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__free__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_total` field */
		_ALIGN(ctx->at, 8);

		/* Write `_total` field */
		{
			const uint64_t tmp_val = (uint64_t) p__total;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_total__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_total__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__total__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemGetInfoEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 481);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemImportFromShareableHandleBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__handle,
	const uint64_t p__handle__val,
	const uint64_t p__osHandle,
	const int32_t p__shHandleType)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 482);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_osHandle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_osHandle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__osHandle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_shHandleType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_shHandleType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__shHandleType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemImportFromShareableHandleEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 483);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemMapArrayAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mapInfoList,
	const int32_t p__mapInfoList__val__resourceType,
	const int32_t p__mapInfoList__val__subresourceType,
	const int32_t p__mapInfoList__val__memOperationType,
	const int32_t p__mapInfoList__val__memHandleType,
	const uint64_t p__mapInfoList__val__offset,
	const uint32_t p__mapInfoList__val__deviceBitMask,
	const uint32_t p__mapInfoList__val__flags,
	const uint64_t p__mapInfoList__val__reserved,
	const uint32_t p__count,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 484);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mapInfoList` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mapInfoList` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mapInfoList;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mapInfoList__val__resourceType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mapInfoList__val__resourceType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mapInfoList__val__resourceType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mapInfoList__val__subresourceType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mapInfoList__val__subresourceType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mapInfoList__val__subresourceType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mapInfoList__val__memOperationType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mapInfoList__val__memOperationType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mapInfoList__val__memOperationType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mapInfoList__val__memHandleType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mapInfoList__val__memHandleType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mapInfoList__val__memHandleType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mapInfoList__val__offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mapInfoList__val__offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mapInfoList__val__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mapInfoList__val__deviceBitMask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mapInfoList__val__deviceBitMask` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mapInfoList__val__deviceBitMask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mapInfoList__val__flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mapInfoList__val__flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mapInfoList__val__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mapInfoList__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mapInfoList__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mapInfoList__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint32_t tmp_val = (uint32_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemMapArrayAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 485);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemMapBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__offset,
	const uint64_t p__handle,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 486);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemMapEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 487);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemPoolCreateBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const uint64_t p__mem_pool__val,
	const uint64_t p__pool_props,
	const int32_t p__pool_props__val__allocType,
	const int32_t p__pool_props__val__handleTypes,
	const int32_t p__pool_props__val__location__type,
	const int32_t p__pool_props__val__location__id,
	const uint64_t p__pool_props__val__win32SecurityAttributes,
	const uint64_t p__pool_props__val__maxSize,
	const uint64_t p__pool_props__val__reserved)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 488);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mem_pool__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pool_props` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pool_props` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pool_props;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pool_props__val__allocType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pool_props__val__allocType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pool_props__val__allocType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pool_props__val__handleTypes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pool_props__val__handleTypes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pool_props__val__handleTypes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pool_props__val__location__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pool_props__val__location__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pool_props__val__location__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pool_props__val__location__id` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pool_props__val__location__id` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pool_props__val__location__id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pool_props__val__win32SecurityAttributes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pool_props__val__win32SecurityAttributes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pool_props__val__win32SecurityAttributes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pool_props__val__maxSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pool_props__val__maxSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pool_props__val__maxSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pool_props__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pool_props__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pool_props__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemPoolCreateEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 489);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemPoolDestroyBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 490);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemPoolDestroyEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 491);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemPoolExportPointerBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__export_data,
	const uint64_t p__export_data__val__reserved,
	const uint64_t p__dev_ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 492);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_export_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_export_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__export_data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_export_data__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_export_data__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__export_data__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemPoolExportPointerEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 493);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemPoolExportToShareableHandleBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__shared_handle,
	const uint64_t p__mem_pool,
	const int32_t p__handle_type,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 494);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_shared_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_shared_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__shared_handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle_type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle_type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__handle_type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemPoolExportToShareableHandleEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 495);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemPoolGetAccessBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__flags,
	const int32_t p__flags__val,
	const uint64_t p__mem_pool,
	const uint64_t p__location,
	const int32_t p__location__val__type,
	const int32_t p__location__val__id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 496);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_location` field */
		_ALIGN(ctx->at, 8);

		/* Write `_location` field */
		{
			const uint64_t tmp_val = (uint64_t) p__location;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_location__val__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_location__val__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__location__val__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_location__val__id` field */
		_ALIGN(ctx->at, 8);

		/* Write `_location__val__id` field */
		{
			const uint32_t tmp_val = (uint32_t) p__location__val__id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemPoolGetAccessEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 497);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemPoolGetAttributeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const int32_t p__attr,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 498);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemPoolGetAttributeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 499);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemPoolImportFromShareableHandleBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const uint64_t p__mem_pool__val,
	const uint64_t p__shared_handle,
	const int32_t p__handle_type,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 500);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mem_pool__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_shared_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_shared_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__shared_handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle_type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle_type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__handle_type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemPoolImportFromShareableHandleEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 501);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemPoolImportPointerBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__dev_ptr__val,
	const uint64_t p__mem_pool,
	const uint64_t p__export_data,
	const uint64_t p__export_data__val__reserved)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 502);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dev_ptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_export_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_export_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__export_data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_export_data__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_export_data__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__export_data__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemPoolImportPointerEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 503);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemPoolSetAccessBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const uint64_t p__desc_list,
	const int32_t p__desc_list__val__location__type,
	const int32_t p__desc_list__val__location__id,
	const int32_t p__desc_list__val__flags,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 504);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc_list` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc_list` field */
		{
			const uint64_t tmp_val = (uint64_t) p__desc_list;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc_list__val__location__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc_list__val__location__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc_list__val__location__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc_list__val__location__id` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc_list__val__location__id` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc_list__val__location__id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc_list__val__flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc_list__val__flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc_list__val__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemPoolSetAccessEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 505);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemPoolSetAttributeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const int32_t p__attr,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 506);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attr` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemPoolSetAttributeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 507);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemPoolTrimToBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const uint64_t p__min_bytes_to_hold)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 508);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mem_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mem_pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_min_bytes_to_hold` field */
		_ALIGN(ctx->at, 8);

		/* Write `_min_bytes_to_hold` field */
		{
			const uint64_t tmp_val = (uint64_t) p__min_bytes_to_hold;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemPoolTrimToEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 509);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemPrefetchAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__count,
	const int32_t p__device,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 510);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemPrefetchAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 511);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemPtrGetInfoBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__size__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 512);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemPtrGetInfoEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 513);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemRangeGetAttributeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__data,
	const uint64_t p__data_size,
	const int32_t p__attribute,
	const uint64_t p__dev_ptr,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 514);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemRangeGetAttributeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 515);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemRangeGetAttributesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__data,
	const uint64_t p__data__val,
	const uint64_t p__data_sizes,
	const uint64_t p__data_sizes__val,
	const uint64_t p__attributes,
	const int32_t p__attributes__val,
	const uint64_t p__num_attributes,
	const uint64_t p__dev_ptr,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 516);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data_sizes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data_sizes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data_sizes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data_sizes__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data_sizes__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data_sizes__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attributes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attributes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attributes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attributes__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attributes__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attributes__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_num_attributes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_attributes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__num_attributes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemRangeGetAttributesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 517);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemReleaseBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 518);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemReleaseEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 519);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemRetainAllocationHandleBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__handle,
	const uint64_t p__handle__val,
	const uint64_t p__addr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 520);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_addr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_addr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__addr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemRetainAllocationHandleEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 521);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemSetAccessBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__desc,
	const int32_t p__desc__val__location__type,
	const int32_t p__desc__val__location__id,
	const int32_t p__desc__val__flags,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 522);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__desc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc__val__location__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__location__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__location__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__location__id` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__location__id` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__location__id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemSetAccessEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 523);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemUnmapBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 524);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemUnmapEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 525);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpy2DArrayToArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__wOffsetDst,
	const uint64_t p__hOffsetDst,
	const uint64_t p__src,
	const uint64_t p__wOffsetSrc,
	const uint64_t p__hOffsetSrc,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 526);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wOffsetDst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wOffsetDst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__wOffsetDst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hOffsetDst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hOffsetDst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hOffsetDst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wOffsetSrc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wOffsetSrc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__wOffsetSrc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hOffsetSrc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hOffsetSrc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hOffsetSrc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpy2DArrayToArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 527);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpy2DAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__dpitch,
	const uint64_t p__src,
	const uint64_t p__spitch,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 528);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dpitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dpitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dpitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_spitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_spitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__spitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpy2DAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 529);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpy2DBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__dpitch,
	const uint64_t p__src,
	const uint64_t p__spitch,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 530);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dpitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dpitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dpitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_spitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_spitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__spitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpy2DEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 531);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpy2DFromArrayAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__dpitch,
	const uint64_t p__src,
	const uint64_t p__wOffset,
	const uint64_t p__hOffset,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 532);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dpitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dpitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dpitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__wOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpy2DFromArrayAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 533);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpy2DFromArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__dpitch,
	const uint64_t p__src,
	const uint64_t p__wOffset,
	const uint64_t p__hOffset,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 534);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dpitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dpitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dpitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__wOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpy2DFromArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 535);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpy2DToArrayAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__wOffset,
	const uint64_t p__hOffset,
	const uint64_t p__src,
	const uint64_t p__spitch,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 536);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__wOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_spitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_spitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__spitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpy2DToArrayAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 537);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpy2DToArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__wOffset,
	const uint64_t p__hOffset,
	const uint64_t p__src,
	const uint64_t p__spitch,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 538);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__wOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_spitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_spitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__spitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpy2DToArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 539);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpy3DAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__p,
	const uint64_t p__p__val__srcArray,
	const uint64_t p__p__val__srcPos__x,
	const uint64_t p__p__val__srcPos__y,
	const uint64_t p__p__val__srcPos__z,
	const uint64_t p__p__val__srcPtr__ptr,
	const uint64_t p__p__val__srcPtr__pitch,
	const uint64_t p__p__val__srcPtr__xsize,
	const uint64_t p__p__val__srcPtr__ysize,
	const uint64_t p__p__val__dstArray,
	const uint64_t p__p__val__dstPos__x,
	const uint64_t p__p__val__dstPos__y,
	const uint64_t p__p__val__dstPos__z,
	const uint64_t p__p__val__dstPtr__ptr,
	const uint64_t p__p__val__dstPtr__pitch,
	const uint64_t p__p__val__dstPtr__xsize,
	const uint64_t p__p__val__dstPtr__ysize,
	const uint64_t p__p__val__extent__width,
	const uint64_t p__p__val__extent__height,
	const uint64_t p__p__val__extent__depth,
	const int32_t p__p__val__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 540);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_p` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPos__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPos__x` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPos__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPos__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPos__y` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPos__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPos__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPos__z` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPos__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPos__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPos__x` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPos__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPos__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPos__y` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPos__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPos__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPos__z` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPos__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__extent__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__extent__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__extent__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__extent__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__extent__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__extent__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__extent__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__extent__depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__extent__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__p__val__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpy3DAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 541);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpy3DBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__p,
	const uint64_t p__p__val__srcArray,
	const uint64_t p__p__val__srcPos__x,
	const uint64_t p__p__val__srcPos__y,
	const uint64_t p__p__val__srcPos__z,
	const uint64_t p__p__val__srcPtr__ptr,
	const uint64_t p__p__val__srcPtr__pitch,
	const uint64_t p__p__val__srcPtr__xsize,
	const uint64_t p__p__val__srcPtr__ysize,
	const uint64_t p__p__val__dstArray,
	const uint64_t p__p__val__dstPos__x,
	const uint64_t p__p__val__dstPos__y,
	const uint64_t p__p__val__dstPos__z,
	const uint64_t p__p__val__dstPtr__ptr,
	const uint64_t p__p__val__dstPtr__pitch,
	const uint64_t p__p__val__dstPtr__xsize,
	const uint64_t p__p__val__dstPtr__ysize,
	const uint64_t p__p__val__extent__width,
	const uint64_t p__p__val__extent__height,
	const uint64_t p__p__val__extent__depth,
	const int32_t p__p__val__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 542);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_p` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPos__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPos__x` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPos__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPos__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPos__y` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPos__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPos__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPos__z` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPos__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__srcPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__srcPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__srcPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPos__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPos__x` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPos__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPos__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPos__y` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPos__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPos__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPos__z` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPos__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__dstPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__dstPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__dstPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__extent__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__extent__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__extent__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__extent__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__extent__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__extent__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__extent__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__extent__depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__p__val__extent__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_p__val__kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_p__val__kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__p__val__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpy3DEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 543);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 544);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 545);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyAtoABegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dstArray,
	const uint64_t p__dstOffset,
	const uint64_t p__srcArray,
	const uint64_t p__srcOffset,
	const uint64_t p__ByteCount)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 546);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dstOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dstOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dstOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__srcOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ByteCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ByteCount` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ByteCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyAtoAEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 547);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyAtoDBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dstDevice,
	const uint64_t p__srcArray,
	const uint64_t p__srcOffset,
	const uint64_t p__ByteCount)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 548);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dstDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dstDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dstDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__srcOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ByteCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ByteCount` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ByteCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyAtoDEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 549);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyAtoHAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dstHost,
	const uint64_t p__srcArray,
	const uint64_t p__srcOffset,
	const uint64_t p__ByteCount,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 550);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dstHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dstHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dstHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__srcOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ByteCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ByteCount` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ByteCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyAtoHAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 551);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyAtoHBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__srcArray,
	const uint64_t p__srcOffset,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 552);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__srcOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyAtoHEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 553);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 554);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyDtoABegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dstArray,
	const uint64_t p__dstOffset,
	const uint64_t p__srcDevice,
	const uint64_t p__ByteCount)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 555);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dstOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dstOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dstOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__srcDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ByteCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ByteCount` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ByteCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyDtoAEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 556);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyDtoDAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 557);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyDtoDAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 558);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyDtoDBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 559);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyDtoDEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 560);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyDtoHAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 561);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyDtoHAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 562);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyDtoHBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 563);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyDtoHEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 564);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 565);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyFromArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__srcArray,
	const uint64_t p__wOffset,
	const uint64_t p__hOffset,
	const uint64_t p__count,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 566);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__wOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyFromArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 567);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyFromSymbolAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__symbol,
	const uint64_t p__sizeBytes,
	const uint64_t p__offset,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 568);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyFromSymbolAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 569);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyFromSymbolBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__symbol,
	const uint64_t p__sizeBytes,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 570);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyFromSymbolEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 571);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyHtoAAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dstArray,
	const uint64_t p__dstOffset,
	const uint64_t p__srcHost,
	const uint64_t p__ByteCount,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 572);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dstOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dstOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dstOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__srcHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ByteCount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ByteCount` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ByteCount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyHtoAAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 573);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyHtoABegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dstArray,
	const uint64_t p__dstOffset,
	const uint64_t p__srcHost,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 574);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dstOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dstOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dstOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__srcHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyHtoAEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 575);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyHtoDAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 576);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyHtoDAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 577);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyHtoDBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 578);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyHtoDEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 579);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyParam2DAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pCopy,
	const uint64_t p__pCopy__val__srcXInBytes,
	const uint64_t p__pCopy__val__srcY,
	const int32_t p__pCopy__val__srcMemoryType,
	const uint64_t p__pCopy__val__srcHost,
	const uint64_t p__pCopy__val__srcDevice,
	const uint64_t p__pCopy__val__srcArray,
	const uint64_t p__pCopy__val__srcPitch,
	const uint64_t p__pCopy__val__dstXInBytes,
	const uint64_t p__pCopy__val__dstY,
	const int32_t p__pCopy__val__dstMemoryType,
	const uint64_t p__pCopy__val__dstHost,
	const uint64_t p__pCopy__val__dstDevice,
	const uint64_t p__pCopy__val__dstArray,
	const uint64_t p__pCopy__val__dstPitch,
	const uint64_t p__pCopy__val__WidthInBytes,
	const uint64_t p__pCopy__val__Height,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 580);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pCopy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcXInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcXInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcXInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcY` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcMemoryType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcMemoryType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pCopy__val__srcMemoryType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pCopy__val__srcHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstXInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstXInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstXInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstY` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstMemoryType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstMemoryType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pCopy__val__dstMemoryType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pCopy__val__dstHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__WidthInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__WidthInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__WidthInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__Height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__Height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__Height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyParam2DAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 581);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyParam2DBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pCopy,
	const uint64_t p__pCopy__val__srcXInBytes,
	const uint64_t p__pCopy__val__srcY,
	const int32_t p__pCopy__val__srcMemoryType,
	const uint64_t p__pCopy__val__srcHost,
	const uint64_t p__pCopy__val__srcDevice,
	const uint64_t p__pCopy__val__srcArray,
	const uint64_t p__pCopy__val__srcPitch,
	const uint64_t p__pCopy__val__dstXInBytes,
	const uint64_t p__pCopy__val__dstY,
	const int32_t p__pCopy__val__dstMemoryType,
	const uint64_t p__pCopy__val__dstHost,
	const uint64_t p__pCopy__val__dstDevice,
	const uint64_t p__pCopy__val__dstArray,
	const uint64_t p__pCopy__val__dstPitch,
	const uint64_t p__pCopy__val__WidthInBytes,
	const uint64_t p__pCopy__val__Height)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 582);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pCopy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcXInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcXInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcXInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcY` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcMemoryType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcMemoryType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pCopy__val__srcMemoryType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pCopy__val__srcHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__srcPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__srcPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__srcPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstXInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstXInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstXInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstY` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstMemoryType` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstMemoryType` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pCopy__val__dstMemoryType;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pCopy__val__dstHost` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstHost` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstHost;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstDevice` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__dstPitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__dstPitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__dstPitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__WidthInBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__WidthInBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__WidthInBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCopy__val__Height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCopy__val__Height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCopy__val__Height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyParam2DEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 583);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyPeerAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const int32_t p__dstDeviceId,
	const uint64_t p__src,
	const int32_t p__srcDevice,
	const uint64_t p__sizeBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 584);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dstDeviceId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dstDeviceId` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dstDeviceId;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcDevice` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcDevice` field */
		{
			const uint32_t tmp_val = (uint32_t) p__srcDevice;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyPeerAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 585);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyPeerBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const int32_t p__dstDeviceId,
	const uint64_t p__src,
	const int32_t p__srcDeviceId,
	const uint64_t p__sizeBytes)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 586);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dstDeviceId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dstDeviceId` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dstDeviceId;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_srcDeviceId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_srcDeviceId` field */
		{
			const uint32_t tmp_val = (uint32_t) p__srcDeviceId;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyPeerEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 587);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyToArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__wOffset,
	const uint64_t p__hOffset,
	const uint64_t p__src,
	const uint64_t p__count,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 588);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__wOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyToArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 589);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyToSymbolAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__symbol,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const uint64_t p__offset,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 590);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyToSymbolAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 591);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyToSymbolBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__symbol,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 592);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyToSymbolEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 593);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemcpyWithStreamBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 594);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kind` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kind` field */
		{
			const uint32_t tmp_val = (uint32_t) p__kind;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemcpyWithStreamEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 595);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemset2DAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__pitch,
	const int32_t p__value,
	const uint64_t p__width,
	const uint64_t p__height,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 596);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemset2DAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 597);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemset2DBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__pitch,
	const int32_t p__value,
	const uint64_t p__width,
	const uint64_t p__height)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 598);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemset2DEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 599);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemset3DAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pitchedDevPtr__ptr,
	const uint64_t p__pitchedDevPtr__pitch,
	const uint64_t p__pitchedDevPtr__xsize,
	const uint64_t p__pitchedDevPtr__ysize,
	const int32_t p__value,
	const uint64_t p__extent__width,
	const uint64_t p__extent__height,
	const uint64_t p__extent__depth,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 600);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pitchedDevPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitchedDevPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitchedDevPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitchedDevPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitchedDevPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitchedDevPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitchedDevPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitchedDevPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitchedDevPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitchedDevPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitchedDevPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitchedDevPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_extent__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemset3DAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 601);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemset3DBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pitchedDevPtr__ptr,
	const uint64_t p__pitchedDevPtr__pitch,
	const uint64_t p__pitchedDevPtr__xsize,
	const uint64_t p__pitchedDevPtr__ysize,
	const int32_t p__value,
	const uint64_t p__extent__width,
	const uint64_t p__extent__height,
	const uint64_t p__extent__depth)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 602);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pitchedDevPtr__ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitchedDevPtr__ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitchedDevPtr__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitchedDevPtr__pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitchedDevPtr__pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitchedDevPtr__pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitchedDevPtr__xsize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitchedDevPtr__xsize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitchedDevPtr__xsize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pitchedDevPtr__ysize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pitchedDevPtr__ysize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pitchedDevPtr__ysize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_extent__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extent__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extent__depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extent__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemset3DEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 603);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemsetAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const int32_t p__value,
	const uint64_t p__sizeBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 604);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemsetAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 605);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemsetBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const int32_t p__value,
	const uint64_t p__sizeBytes)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 606);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_sizeBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sizeBytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sizeBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemsetD16AsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dest,
	const uint16_t p__value,
	const uint64_t p__count,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 607);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dest` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dest` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dest;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint16_t tmp_val = (uint16_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemsetD16AsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 608);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemsetD16Begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dest,
	const uint16_t p__value,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 609);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dest` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dest` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dest;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint16_t tmp_val = (uint16_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemsetD16End(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 610);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemsetD32AsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const int32_t p__value,
	const uint64_t p__count,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 611);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemsetD32AsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 612);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemsetD32Begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dest,
	const int32_t p__value,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 613);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dest` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dest` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dest;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemsetD32End(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 614);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemsetD8AsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dest,
	const uint8_t p__value,
	const uint64_t p__count,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 615);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dest` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dest` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dest;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint8_t tmp_val = (uint8_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemsetD8AsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 616);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemsetD8Begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dest,
	const uint8_t p__value,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 617);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dest` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dest` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dest;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint8_t tmp_val = (uint8_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMemsetD8End(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 618);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMemsetEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 619);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMipmappedArrayCreateBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pHandle,
	const uint64_t p__pHandle__val,
	const uint64_t p__pMipmappedArrayDesc,
	const uint64_t p__pMipmappedArrayDesc__val__Width,
	const uint64_t p__pMipmappedArrayDesc__val__Height,
	const uint64_t p__pMipmappedArrayDesc__val__Depth,
	const int32_t p__pMipmappedArrayDesc__val__Format,
	const uint32_t p__pMipmappedArrayDesc__val__NumChannels,
	const uint32_t p__pMipmappedArrayDesc__val__Flags,
	const uint32_t p__numMipmapLevels)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 620);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pHandle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pHandle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pHandle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pHandle__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pHandle__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pHandle__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pMipmappedArrayDesc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMipmappedArrayDesc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pMipmappedArrayDesc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pMipmappedArrayDesc__val__Width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMipmappedArrayDesc__val__Width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pMipmappedArrayDesc__val__Width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pMipmappedArrayDesc__val__Height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMipmappedArrayDesc__val__Height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pMipmappedArrayDesc__val__Height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pMipmappedArrayDesc__val__Depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMipmappedArrayDesc__val__Depth` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pMipmappedArrayDesc__val__Depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pMipmappedArrayDesc__val__Format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMipmappedArrayDesc__val__Format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pMipmappedArrayDesc__val__Format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pMipmappedArrayDesc__val__NumChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMipmappedArrayDesc__val__NumChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pMipmappedArrayDesc__val__NumChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pMipmappedArrayDesc__val__Flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pMipmappedArrayDesc__val__Flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pMipmappedArrayDesc__val__Flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_numMipmapLevels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numMipmapLevels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numMipmapLevels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMipmappedArrayCreateEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 621);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMipmappedArrayDestroyBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hMipmappedArray)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 622);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_hMipmappedArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hMipmappedArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hMipmappedArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipMipmappedArrayDestroyEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 623);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipMipmappedArrayGetLevelBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pLevelArray,
	const uint64_t p__pLevelArray__val,
	const uint64_t p__hMipMappedArray,
	const uint32_t p__level)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 624);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pLevelArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pLevelArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pLevelArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pLevelArray__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pLevelArray__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pLevelArray__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hMipMappedArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hMipMappedArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hMipMappedArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_level` field */
		_ALIGN(ctx->at, 8);

		/* Write `_level` field */
		{
			const uint32_t tmp_val = (uint32_t) p__level;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipMipmappedArrayGetLevelEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 625);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleGetFunctionBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__function,
	const uint64_t p__function__val,
	const uint64_t p__module,
	const char * const p__kname,
	const int8_t p__kname__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 626);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_function` field */
		_ALIGN(ctx->at, 8);

		/* Write `_function` field */
		{
			const uint64_t tmp_val = (uint64_t) p__function;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_function__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_function__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__function__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_module` field */
		_ALIGN(ctx->at, 8);

		/* Write `_module` field */
		{
			const uint64_t tmp_val = (uint64_t) p__module;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kname` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kname` field */
		_write_c_str(ctx, p__kname);

		/* Align for `_kname__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kname__val` field */
		{
			const uint8_t tmp_val = (uint8_t) p__kname__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}
	}
}

static void _serialize_er_hip_api_hipModuleGetFunctionEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 627);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleGetGlobalBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dptr,
	const uint64_t p__dptr__val,
	const uint64_t p__bytes,
	const uint64_t p__bytes__val,
	const uint64_t p__hmod,
	const char * const p__name,
	const int8_t p__name__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 628);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_bytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_bytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__bytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_bytes__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_bytes__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__bytes__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hmod` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hmod` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hmod;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_name` field */
		_write_c_str(ctx, p__name);

		/* Align for `_name__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_name__val` field */
		{
			const uint8_t tmp_val = (uint8_t) p__name__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}
	}
}

static void _serialize_er_hip_api_hipModuleGetGlobalEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 629);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleGetTexRefBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const uint64_t p__texRef__val,
	const uint64_t p__hmod,
	const char * const p__name,
	const int8_t p__name__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 630);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_hmod` field */
		_ALIGN(ctx->at, 8);

		/* Write `_hmod` field */
		{
			const uint64_t tmp_val = (uint64_t) p__hmod;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_name` field */
		_write_c_str(ctx, p__name);

		/* Align for `_name__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_name__val` field */
		{
			const uint8_t tmp_val = (uint8_t) p__name__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}
	}
}

static void _serialize_er_hip_api_hipModuleGetTexRefEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 631);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleLaunchCooperativeKernelBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__f,
	const uint32_t p__gridDimX,
	const uint32_t p__gridDimY,
	const uint32_t p__gridDimZ,
	const uint32_t p__blockDimX,
	const uint32_t p__blockDimY,
	const uint32_t p__blockDimZ,
	const uint32_t p__sharedMemBytes,
	const uint64_t p__stream,
	const uint64_t p__kernelParams,
	const uint64_t p__kernelParams__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 632);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_f` field */
		{
			const uint64_t tmp_val = (uint64_t) p__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_gridDimX` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDimX` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDimX;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_gridDimY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDimY` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDimY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_gridDimZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDimZ` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDimZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDimX` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDimX` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDimX;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDimY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDimY` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDimY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDimZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDimZ` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDimZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_sharedMemBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sharedMemBytes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__sharedMemBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kernelParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kernelParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__kernelParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kernelParams__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kernelParams__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__kernelParams__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipModuleLaunchCooperativeKernelEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 633);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleLaunchCooperativeKernelMultiDeviceBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__launchParamsList,
	const uint64_t p__launchParamsList__val__function,
	const uint32_t p__launchParamsList__val__gridDimX,
	const uint32_t p__launchParamsList__val__gridDimY,
	const uint32_t p__launchParamsList__val__gridDimZ,
	const uint32_t p__launchParamsList__val__blockDimX,
	const uint32_t p__launchParamsList__val__blockDimY,
	const uint32_t p__launchParamsList__val__blockDimZ,
	const uint32_t p__launchParamsList__val__sharedMemBytes,
	const uint64_t p__launchParamsList__val__hStream,
	const uint64_t p__launchParamsList__val__kernelParams,
	const uint32_t p__numDevices,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 634);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_launchParamsList` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_launchParamsList__val__function` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__function` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList__val__function;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_launchParamsList__val__gridDimX` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__gridDimX` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__gridDimX;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__gridDimY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__gridDimY` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__gridDimY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__gridDimZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__gridDimZ` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__gridDimZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__blockDimX` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__blockDimX` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__blockDimX;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__blockDimY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__blockDimY` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__blockDimY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__blockDimZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__blockDimZ` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__blockDimZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__sharedMemBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__sharedMemBytes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__launchParamsList__val__sharedMemBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_launchParamsList__val__hStream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__hStream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList__val__hStream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_launchParamsList__val__kernelParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_launchParamsList__val__kernelParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__launchParamsList__val__kernelParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDevices` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDevices` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numDevices;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipModuleLaunchCooperativeKernelMultiDeviceEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 635);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleLaunchKernelBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__f,
	const uint32_t p__gridDimX,
	const uint32_t p__gridDimY,
	const uint32_t p__gridDimZ,
	const uint32_t p__blockDimX,
	const uint32_t p__blockDimY,
	const uint32_t p__blockDimZ,
	const uint32_t p__sharedMemBytes,
	const uint64_t p__stream,
	const uint64_t p__kernelParams,
	const uint64_t p__kernelParams__val,
	const uint64_t p__extra,
	const uint64_t p__extra__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 636);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_f` field */
		{
			const uint64_t tmp_val = (uint64_t) p__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_gridDimX` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDimX` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDimX;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_gridDimY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDimY` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDimY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_gridDimZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridDimZ` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridDimZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDimX` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDimX` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDimX;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDimY` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDimY` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDimY;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockDimZ` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockDimZ` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockDimZ;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_sharedMemBytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sharedMemBytes` field */
		{
			const uint32_t tmp_val = (uint32_t) p__sharedMemBytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kernelParams` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kernelParams` field */
		{
			const uint64_t tmp_val = (uint64_t) p__kernelParams;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kernelParams__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kernelParams__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__kernelParams__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extra` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extra` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extra;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extra__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extra__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extra__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipModuleLaunchKernelEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 637);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleLoadBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__module,
	const uint64_t p__module__val,
	const char * const p__fname,
	const int8_t p__fname__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 638);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_module` field */
		_ALIGN(ctx->at, 8);

		/* Write `_module` field */
		{
			const uint64_t tmp_val = (uint64_t) p__module;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_module__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_module__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__module__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_fname` field */
		_ALIGN(ctx->at, 8);

		/* Write `_fname` field */
		_write_c_str(ctx, p__fname);

		/* Align for `_fname__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_fname__val` field */
		{
			const uint8_t tmp_val = (uint8_t) p__fname__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}
	}
}

static void _serialize_er_hip_api_hipModuleLoadDataBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__module,
	const uint64_t p__module__val,
	const uint64_t p__image)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 639);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_module` field */
		_ALIGN(ctx->at, 8);

		/* Write `_module` field */
		{
			const uint64_t tmp_val = (uint64_t) p__module;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_module__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_module__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__module__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipModuleLoadDataEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 640);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleLoadDataExBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__module,
	const uint64_t p__module__val,
	const uint64_t p__image,
	const uint32_t p__numOptions,
	const uint64_t p__options,
	const int32_t p__options__val,
	const uint64_t p__optionsValues,
	const uint64_t p__optionsValues__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 641);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_module` field */
		_ALIGN(ctx->at, 8);

		/* Write `_module` field */
		{
			const uint64_t tmp_val = (uint64_t) p__module;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_module__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_module__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__module__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numOptions` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numOptions` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numOptions;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_options` field */
		_ALIGN(ctx->at, 8);

		/* Write `_options` field */
		{
			const uint64_t tmp_val = (uint64_t) p__options;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_options__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_options__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__options__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_optionsValues` field */
		_ALIGN(ctx->at, 8);

		/* Write `_optionsValues` field */
		{
			const uint64_t tmp_val = (uint64_t) p__optionsValues;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_optionsValues__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_optionsValues__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__optionsValues__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipModuleLoadDataExEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 642);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleLoadEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 643);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__numBlocks,
	const int32_t p__numBlocks__val,
	const uint64_t p__f,
	const int32_t p__blockSize,
	const uint64_t p__dynSharedMemPerBlk)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 644);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_numBlocks` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numBlocks;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numBlocks__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numBlocks__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_f` field */
		{
			const uint64_t tmp_val = (uint64_t) p__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_blockSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dynSharedMemPerBlk` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dynSharedMemPerBlk` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dynSharedMemPerBlk;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 645);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__numBlocks,
	const int32_t p__numBlocks__val,
	const uint64_t p__f,
	const int32_t p__blockSize,
	const uint64_t p__dynSharedMemPerBlk,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 646);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_numBlocks` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numBlocks;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numBlocks__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numBlocks__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_f` field */
		{
			const uint64_t tmp_val = (uint64_t) p__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_blockSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dynSharedMemPerBlk` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dynSharedMemPerBlk` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dynSharedMemPerBlk;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 647);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleOccupancyMaxPotentialBlockSizeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__gridSize,
	const int32_t p__gridSize__val,
	const uint64_t p__blockSize,
	const int32_t p__blockSize__val,
	const uint64_t p__f,
	const uint64_t p__dynSharedMemPerBlk,
	const int32_t p__blockSizeLimit)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 648);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_gridSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__gridSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_gridSize__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridSize__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridSize__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__blockSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_blockSize__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockSize__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockSize__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_f` field */
		{
			const uint64_t tmp_val = (uint64_t) p__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dynSharedMemPerBlk` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dynSharedMemPerBlk` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dynSharedMemPerBlk;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_blockSizeLimit` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockSizeLimit` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockSizeLimit;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipModuleOccupancyMaxPotentialBlockSizeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 649);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleOccupancyMaxPotentialBlockSizeWithFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__gridSize,
	const int32_t p__gridSize__val,
	const uint64_t p__blockSize,
	const int32_t p__blockSize__val,
	const uint64_t p__f,
	const uint64_t p__dynSharedMemPerBlk,
	const int32_t p__blockSizeLimit,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 650);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_gridSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__gridSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_gridSize__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridSize__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridSize__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__blockSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_blockSize__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockSize__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockSize__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_f` field */
		{
			const uint64_t tmp_val = (uint64_t) p__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dynSharedMemPerBlk` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dynSharedMemPerBlk` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dynSharedMemPerBlk;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_blockSizeLimit` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockSizeLimit` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockSizeLimit;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipModuleOccupancyMaxPotentialBlockSizeWithFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 651);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipModuleUnloadBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__module)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 652);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_module` field */
		_ALIGN(ctx->at, 8);

		/* Write `_module` field */
		{
			const uint64_t tmp_val = (uint64_t) p__module;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipModuleUnloadEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 653);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__numBlocks,
	const int32_t p__numBlocks__val,
	const uint64_t p__f,
	const int32_t p__blockSize,
	const uint64_t p__dynamicSMemSize)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 654);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_numBlocks` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numBlocks;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numBlocks__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numBlocks__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_f` field */
		{
			const uint64_t tmp_val = (uint64_t) p__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_blockSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dynamicSMemSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dynamicSMemSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dynamicSMemSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 655);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__numBlocks,
	const int32_t p__numBlocks__val,
	const uint64_t p__f,
	const int32_t p__blockSize,
	const uint64_t p__dynamicSMemSize,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 656);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_numBlocks` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numBlocks;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numBlocks__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numBlocks__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numBlocks__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_f` field */
		{
			const uint64_t tmp_val = (uint64_t) p__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_blockSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockSize` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dynamicSMemSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dynamicSMemSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dynamicSMemSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 657);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipOccupancyMaxPotentialBlockSizeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__gridSize,
	const int32_t p__gridSize__val,
	const uint64_t p__blockSize,
	const int32_t p__blockSize__val,
	const uint64_t p__f,
	const uint64_t p__dynSharedMemPerBlk,
	const int32_t p__blockSizeLimit)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 658);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_gridSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__gridSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_gridSize__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_gridSize__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__gridSize__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_blockSize` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockSize` field */
		{
			const uint64_t tmp_val = (uint64_t) p__blockSize;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_blockSize__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockSize__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockSize__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_f` field */
		{
			const uint64_t tmp_val = (uint64_t) p__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dynSharedMemPerBlk` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dynSharedMemPerBlk` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dynSharedMemPerBlk;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_blockSizeLimit` field */
		_ALIGN(ctx->at, 8);

		/* Write `_blockSizeLimit` field */
		{
			const uint32_t tmp_val = (uint32_t) p__blockSizeLimit;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipOccupancyMaxPotentialBlockSizeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 659);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipPeekAtLastErrorBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 660);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipPeekAtLastErrorEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 661);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipPointerGetAttributeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__data,
	const int32_t p__attribute,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 662);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipPointerGetAttributeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 663);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipPointerGetAttributesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__attributes,
	const int32_t p__attributes__val__type,
	const int32_t p__attributes__val__device,
	const uint64_t p__attributes__val__devicePointer,
	const uint64_t p__attributes__val__hostPointer,
	const int32_t p__attributes__val__isManaged,
	const uint32_t p__attributes__val__allocationFlags,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 664);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_attributes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attributes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attributes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attributes__val__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attributes__val__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attributes__val__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attributes__val__device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attributes__val__device` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attributes__val__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attributes__val__devicePointer` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attributes__val__devicePointer` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attributes__val__devicePointer;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attributes__val__hostPointer` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attributes__val__hostPointer` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attributes__val__hostPointer;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attributes__val__isManaged` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attributes__val__isManaged` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attributes__val__isManaged;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_attributes__val__allocationFlags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attributes__val__allocationFlags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attributes__val__allocationFlags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipPointerGetAttributesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 665);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipPointerSetAttributeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__value,
	const int32_t p__attribute,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 666);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipPointerSetAttributeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 667);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipProfilerStartBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 668);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipProfilerStartEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 669);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipProfilerStopBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 670);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipProfilerStopEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 671);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipRuntimeGetVersionBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__runtimeVersion,
	const int32_t p__runtimeVersion__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 672);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_runtimeVersion` field */
		_ALIGN(ctx->at, 8);

		/* Write `_runtimeVersion` field */
		{
			const uint64_t tmp_val = (uint64_t) p__runtimeVersion;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_runtimeVersion__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_runtimeVersion__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__runtimeVersion__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipRuntimeGetVersionEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 673);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipSetDeviceBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__deviceId)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 674);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_deviceId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_deviceId` field */
		{
			const uint32_t tmp_val = (uint32_t) p__deviceId;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipSetDeviceEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 675);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipSetDeviceFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 676);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipSetDeviceFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 677);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipSetValidDevicesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__device_arr,
	const int32_t p__device_arr__val,
	const int32_t p__len)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 678);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_device_arr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device_arr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__device_arr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_device_arr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device_arr__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device_arr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_len` field */
		_ALIGN(ctx->at, 8);

		/* Write `_len` field */
		{
			const uint32_t tmp_val = (uint32_t) p__len;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipSetValidDevicesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 679);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipSetupArgumentBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__arg,
	const uint64_t p__size,
	const uint64_t p__offset)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 680);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_arg` field */
		_ALIGN(ctx->at, 8);

		/* Write `_arg` field */
		{
			const uint64_t tmp_val = (uint64_t) p__arg;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipSetupArgumentEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 681);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipSignalExternalSemaphoresAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__extSemArray,
	const uint64_t p__extSemArray__val,
	const uint64_t p__paramsArray,
	const uint32_t p__paramsArray__val__flags,
	const uint64_t p__paramsArray__val__reserved,
	const uint32_t p__numExtSems,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 682);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_extSemArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extSemArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extSemArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extSemArray__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extSemArray__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extSemArray__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_paramsArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_paramsArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__paramsArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_paramsArray__val__flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_paramsArray__val__flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__paramsArray__val__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_paramsArray__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_paramsArray__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__paramsArray__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numExtSems` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numExtSems` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numExtSems;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipSignalExternalSemaphoresAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 683);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamAddCallbackBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__callback,
	const uint64_t p__userData,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 684);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_userData` field */
		_ALIGN(ctx->at, 8);

		/* Write `_userData` field */
		{
			const uint64_t tmp_val = (uint64_t) p__userData;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamAddCallbackEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 685);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamAttachMemAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__dev_ptr,
	const uint64_t p__length,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 686);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_length` field */
		_ALIGN(ctx->at, 8);

		/* Write `_length` field */
		{
			const uint64_t tmp_val = (uint64_t) p__length;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamAttachMemAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 687);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamBeginCaptureBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const int32_t p__mode)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 688);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamBeginCaptureEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 689);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamBeginCaptureToGraphBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__graph,
	const uint64_t p__dependencies,
	const uint64_t p__dependencies__val,
	const uint64_t p__dependencyData,
	const uint8_t p__dependencyData__val__from_port,
	const uint64_t p__dependencyData__val__reserved,
	const uint8_t p__dependencyData__val__to_port,
	const uint8_t p__dependencyData__val__type,
	const uint64_t p__numDependencies,
	const int32_t p__mode)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 690);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dependencyData` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencyData` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dependencyData;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dependencyData__val__from_port` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencyData__val__from_port` field */
		{
			const uint8_t tmp_val = (uint8_t) p__dependencyData__val__from_port;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}

		/* Align for `_dependencyData__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencyData__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dependencyData__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dependencyData__val__to_port` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencyData__val__to_port` field */
		{
			const uint8_t tmp_val = (uint8_t) p__dependencyData__val__to_port;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}

		/* Align for `_dependencyData__val__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencyData__val__type` field */
		{
			const uint8_t tmp_val = (uint8_t) p__dependencyData__val__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamBeginCaptureToGraphEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 691);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamCreateBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__stream__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 692);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipStreamCreateEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 693);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamCreateWithFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__stream__val,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 694);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamCreateWithFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 695);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamCreateWithPriorityBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__stream__val,
	const uint32_t p__flags,
	const int32_t p__priority)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 696);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_stream__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_priority` field */
		_ALIGN(ctx->at, 8);

		/* Write `_priority` field */
		{
			const uint32_t tmp_val = (uint32_t) p__priority;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamCreateWithPriorityEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 697);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamDestroyBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 698);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipStreamDestroyEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 699);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamEndCaptureBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__pGraph,
	const uint64_t p__pGraph__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 700);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraph` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraph` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraph;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pGraph__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pGraph__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pGraph__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipStreamEndCaptureEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 701);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamGetCaptureInfoBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__pCaptureStatus,
	const int32_t p__pCaptureStatus__val,
	const uint64_t p__pId,
	const uint64_t p__pId__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 702);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCaptureStatus` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCaptureStatus` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCaptureStatus;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCaptureStatus__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCaptureStatus__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pCaptureStatus__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pId` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pId` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pId;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pId__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pId__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pId__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipStreamGetCaptureInfoEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 703);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamGetCaptureInfo_v2Begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__captureStatus_out,
	const int32_t p__captureStatus_out__val,
	const uint64_t p__id_out,
	const uint64_t p__id_out__val,
	const uint64_t p__graph_out,
	const uint64_t p__graph_out__val,
	const uint64_t p__dependencies_out,
	const uint64_t p__dependencies_out__val,
	const uint64_t p__numDependencies_out,
	const uint64_t p__numDependencies_out__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 704);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_captureStatus_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_captureStatus_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__captureStatus_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_captureStatus_out__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_captureStatus_out__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__captureStatus_out__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_id_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_id_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__id_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_id_out__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_id_out__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__id_out__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_graph_out__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_graph_out__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__graph_out__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dependencies_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencies_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dependencies_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dependencies_out__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencies_out__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dependencies_out__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies_out__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies_out__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies_out__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipStreamGetCaptureInfo_v2End(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 705);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamGetDeviceBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__device,
	const int32_t p__device__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 706);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_device` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device` field */
		{
			const uint64_t tmp_val = (uint64_t) p__device;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_device__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_device__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__device__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamGetDeviceEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 707);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamGetFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__flags,
	const uint32_t p__flags__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 708);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamGetFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 709);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamGetPriorityBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__priority,
	const int32_t p__priority__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 710);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_priority` field */
		_ALIGN(ctx->at, 8);

		/* Write `_priority` field */
		{
			const uint64_t tmp_val = (uint64_t) p__priority;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_priority__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_priority__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__priority__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamGetPriorityEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 711);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamIsCapturingBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__pCaptureStatus,
	const int32_t p__pCaptureStatus__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 712);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCaptureStatus` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCaptureStatus` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pCaptureStatus;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pCaptureStatus__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pCaptureStatus__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pCaptureStatus__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamIsCapturingEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 713);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamQueryBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 714);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipStreamQueryEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 715);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamSynchronizeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 716);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipStreamSynchronizeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 717);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamUpdateCaptureDependenciesBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__dependencies,
	const uint64_t p__dependencies__val,
	const uint64_t p__numDependencies,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 718);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dependencies__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dependencies__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dependencies__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numDependencies` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numDependencies` field */
		{
			const uint64_t tmp_val = (uint64_t) p__numDependencies;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamUpdateCaptureDependenciesEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 719);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamWaitEventBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__event,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 720);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_event` field */
		_ALIGN(ctx->at, 8);

		/* Write `_event` field */
		{
			const uint64_t tmp_val = (uint64_t) p__event;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamWaitEventEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 721);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamWaitValue32Begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__ptr,
	const uint32_t p__value,
	const uint32_t p__flags,
	const uint32_t p__mask)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 722);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mask` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamWaitValue32End(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 723);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamWaitValue64Begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__ptr,
	const uint64_t p__value,
	const uint32_t p__flags,
	const uint64_t p__mask)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 724);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mask` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipStreamWaitValue64End(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 725);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamWriteValue32Begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__ptr,
	const uint32_t p__value,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 726);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamWriteValue32End(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 727);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipStreamWriteValue64Begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__ptr,
	const uint64_t p__value,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 728);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipStreamWriteValue64End(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 729);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefGetAddressBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__dev_ptr__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 730);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dev_ptr__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dev_ptr__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dev_ptr__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefGetAddressEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 731);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefGetArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pArray,
	const uint64_t p__pArray__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 732);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pArray__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArray__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pArray__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefGetArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 733);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefGetBorderColorBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pBorderColor,
	const uint64_t p__pBorderColor__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 734);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pBorderColor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pBorderColor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pBorderColor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pBorderColor__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pBorderColor__val` field */
		{
			union _f2u f2u;

			f2u.f = p__pBorderColor__val;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefGetBorderColorEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 735);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefGetFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pFlags,
	const uint32_t p__pFlags__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 736);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pFlags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pFlags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pFlags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pFlags__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pFlags__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pFlags__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefGetFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 737);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefGetFormatBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pFormat,
	const int32_t p__pFormat__val,
	const uint64_t p__pNumChannels,
	const int32_t p__pNumChannels__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 738);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pFormat` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pFormat` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pFormat;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pFormat__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pFormat__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pFormat__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pNumChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNumChannels` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pNumChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pNumChannels__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pNumChannels__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pNumChannels__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefGetFormatEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 739);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefGetMaxAnisotropyBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pmaxAnsio,
	const int32_t p__pmaxAnsio__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 740);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pmaxAnsio` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pmaxAnsio` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pmaxAnsio;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pmaxAnsio__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pmaxAnsio__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__pmaxAnsio__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefGetMaxAnisotropyEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 741);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefGetMipMappedArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pArray,
	const uint64_t p__pArray__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 742);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pArray__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pArray__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pArray__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefGetMipMappedArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 743);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefGetMipmapLevelBiasBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pbias,
	const uint64_t p__pbias__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 744);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pbias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pbias` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pbias;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pbias__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pbias__val` field */
		{
			union _f2u f2u;

			f2u.f = p__pbias__val;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefGetMipmapLevelBiasEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 745);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefGetMipmapLevelClampBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pminMipmapLevelClamp,
	const uint64_t p__pminMipmapLevelClamp__val,
	const uint64_t p__pmaxMipmapLevelClamp,
	const uint64_t p__pmaxMipmapLevelClamp__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 746);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pminMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pminMipmapLevelClamp` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pminMipmapLevelClamp;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pminMipmapLevelClamp__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pminMipmapLevelClamp__val` field */
		{
			union _f2u f2u;

			f2u.f = p__pminMipmapLevelClamp__val;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_pmaxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pmaxMipmapLevelClamp` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pmaxMipmapLevelClamp;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pmaxMipmapLevelClamp__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pmaxMipmapLevelClamp__val` field */
		{
			union _f2u f2u;

			f2u.f = p__pmaxMipmapLevelClamp__val;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefGetMipmapLevelClampEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 747);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefSetAddress2DBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint64_t p__desc,
	const uint64_t p__desc__val__Width,
	const uint64_t p__desc__val__Height,
	const int32_t p__desc__val__Format,
	const uint32_t p__desc__val__NumChannels,
	const uint64_t p__dptr,
	const uint64_t p__Pitch)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 748);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__desc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc__val__Width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__Width` field */
		{
			const uint64_t tmp_val = (uint64_t) p__desc__val__Width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc__val__Height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__Height` field */
		{
			const uint64_t tmp_val = (uint64_t) p__desc__val__Height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc__val__Format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__Format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__Format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_desc__val__NumChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc__val__NumChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__desc__val__NumChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_Pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_Pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__Pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefSetAddress2DEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 749);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefSetAddressBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ByteOffset,
	const uint64_t p__ByteOffset__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint64_t p__dptr,
	const uint64_t p__bytes)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 750);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ByteOffset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ByteOffset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ByteOffset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ByteOffset__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ByteOffset__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ByteOffset__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_bytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_bytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__bytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefSetAddressEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 751);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefSetArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__tex,
	const int32_t p__tex__val__normalized,
	const int32_t p__tex__val__readMode,
	const int32_t p__tex__val__filterMode,
	const int32_t p__tex__val__channelDesc__x,
	const int32_t p__tex__val__channelDesc__y,
	const int32_t p__tex__val__channelDesc__z,
	const int32_t p__tex__val__channelDesc__w,
	const int32_t p__tex__val__channelDesc__f,
	const int32_t p__tex__val__sRGB,
	const uint32_t p__tex__val__maxAnisotropy,
	const int32_t p__tex__val__mipmapFilterMode,
	const uint64_t p__tex__val__mipmapLevelBias,
	const uint64_t p__tex__val__minMipmapLevelClamp,
	const uint64_t p__tex__val__maxMipmapLevelClamp,
	const uint64_t p__tex__val__textureObject,
	const int32_t p__tex__val__numChannels,
	const int32_t p__tex__val__format,
	const uint64_t p__array,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 752);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_tex` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex` field */
		{
			const uint64_t tmp_val = (uint64_t) p__tex;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_tex__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__tex__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_tex__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__tex__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_tex__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__tex__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_tex__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__tex__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_tex__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__tex__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_tex__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__tex__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_tex__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__tex__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_tex__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__tex__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_tex__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__tex__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_tex__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__tex__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_tex__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__tex__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_tex__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__tex__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_tex__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__tex__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_tex__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__tex__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_tex__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__tex__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_tex__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__tex__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_tex__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_tex__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__tex__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_array` field */
		_ALIGN(ctx->at, 8);

		/* Write `_array` field */
		{
			const uint64_t tmp_val = (uint64_t) p__array;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefSetArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 753);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefSetBorderColorBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint64_t p__pBorderColor,
	const uint64_t p__pBorderColor__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 754);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pBorderColor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pBorderColor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pBorderColor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pBorderColor__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pBorderColor__val` field */
		{
			union _f2u f2u;

			f2u.f = p__pBorderColor__val;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}
	}
}

static void _serialize_er_hip_api_hipTexRefSetBorderColorEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 755);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefSetFlagsBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint32_t p__Flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 756);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_Flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_Flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__Flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefSetFlagsEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 757);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefSetFormatBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const int32_t p__fmt,
	const int32_t p__NumPackedComponents)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 758);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_fmt` field */
		_ALIGN(ctx->at, 8);

		/* Write `_fmt` field */
		{
			const uint32_t tmp_val = (uint32_t) p__fmt;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_NumPackedComponents` field */
		_ALIGN(ctx->at, 8);

		/* Write `_NumPackedComponents` field */
		{
			const uint32_t tmp_val = (uint32_t) p__NumPackedComponents;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefSetFormatEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 759);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefSetMaxAnisotropyBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint32_t p__maxAniso)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 760);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_maxAniso` field */
		_ALIGN(ctx->at, 8);

		/* Write `_maxAniso` field */
		{
			const uint32_t tmp_val = (uint32_t) p__maxAniso;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefSetMaxAnisotropyEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 761);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefSetMipmapLevelBiasBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint64_t p__bias)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 762);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_bias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_bias` field */
		{
			union _f2u f2u;

			f2u.f = p__bias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}
	}
}

static void _serialize_er_hip_api_hipTexRefSetMipmapLevelBiasEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 763);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefSetMipmapLevelClampBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint64_t p__minMipMapLevelClamp,
	const uint64_t p__maxMipMapLevelClamp)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 764);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_minMipMapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_minMipMapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__minMipMapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_maxMipMapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_maxMipMapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__maxMipMapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}
	}
}

static void _serialize_er_hip_api_hipTexRefSetMipmapLevelClampEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 765);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipTexRefSetMipmappedArrayBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint64_t p__mipmappedArray,
	const uint64_t p__mipmappedArray__val__data,
	const int32_t p__mipmappedArray__val__desc__x,
	const int32_t p__mipmappedArray__val__desc__y,
	const int32_t p__mipmappedArray__val__desc__z,
	const int32_t p__mipmappedArray__val__desc__w,
	const int32_t p__mipmappedArray__val__desc__f,
	const uint32_t p__mipmappedArray__val__type,
	const uint32_t p__mipmappedArray__val__width,
	const uint32_t p__mipmappedArray__val__height,
	const uint32_t p__mipmappedArray__val__depth,
	const uint32_t p__mipmappedArray__val__min_mipmap_level,
	const uint32_t p__mipmappedArray__val__max_mipmap_level,
	const uint32_t p__mipmappedArray__val__flags,
	const int32_t p__mipmappedArray__val__format,
	const uint32_t p__mipmappedArray__val__num_channels,
	const uint32_t p__Flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 766);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_texRef` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__normalized` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__normalized;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__readMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__readMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__filterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__filterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__channelDesc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__channelDesc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__sRGB` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__sRGB;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxAnisotropy` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__maxAnisotropy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapFilterMode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__mipmapFilterMode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__mipmapLevelBias` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__mipmapLevelBias;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__minMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__minMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__maxMipmapLevelClamp` field */
		{
			union _f2u f2u;

			f2u.f = p__texRef__val__maxMipmapLevelClamp;
			{
				const uint32_t tmp_val = (uint32_t) f2u.u;

				memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
				ctx->at += 32;
			}
		}

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__textureObject` field */
		{
			const uint64_t tmp_val = (uint64_t) p__texRef__val__textureObject;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__numChannels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__numChannels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_texRef__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_texRef__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__texRef__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mipmappedArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mipmappedArray__val__data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mipmappedArray__val__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mipmappedArray__val__desc__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__desc__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__desc__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray__val__desc__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__desc__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__desc__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray__val__desc__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__desc__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__desc__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray__val__desc__w` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__desc__w` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__desc__w;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray__val__desc__f` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__desc__f` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__desc__f;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray__val__type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray__val__width` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__width` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__width;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray__val__height` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__height` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__height;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray__val__depth` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__depth` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__depth;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray__val__min_mipmap_level` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__min_mipmap_level` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__min_mipmap_level;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray__val__max_mipmap_level` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__max_mipmap_level` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__max_mipmap_level;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray__val__flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray__val__format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__format` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mipmappedArray__val__num_channels` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mipmappedArray__val__num_channels` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mipmappedArray__val__num_channels;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_Flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_Flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__Flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipTexRefSetMipmappedArrayEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 767);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipThreadExchangeStreamCaptureModeBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mode,
	const int32_t p__mode__val)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 768);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mode` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mode__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mode__val` field */
		{
			const uint32_t tmp_val = (uint32_t) p__mode__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipThreadExchangeStreamCaptureModeEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 769);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipUserObjectCreateBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__object_out,
	const uint64_t p__object_out__val,
	const uint64_t p__ptr,
	const uint64_t p__destroy,
	const uint32_t p__initialRefcount,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 770);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_object_out` field */
		_ALIGN(ctx->at, 8);

		/* Write `_object_out` field */
		{
			const uint64_t tmp_val = (uint64_t) p__object_out;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_object_out__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_object_out__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__object_out__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_destroy` field */
		_ALIGN(ctx->at, 8);

		/* Write `_destroy` field */
		{
			const uint64_t tmp_val = (uint64_t) p__destroy;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_initialRefcount` field */
		_ALIGN(ctx->at, 8);

		/* Write `_initialRefcount` field */
		{
			const uint32_t tmp_val = (uint32_t) p__initialRefcount;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipUserObjectCreateEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 771);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipUserObjectReleaseBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__object,
	const uint32_t p__count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 772);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_object` field */
		_ALIGN(ctx->at, 8);

		/* Write `_object` field */
		{
			const uint64_t tmp_val = (uint64_t) p__object;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint32_t tmp_val = (uint32_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipUserObjectReleaseEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 773);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipUserObjectRetainBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__object,
	const uint32_t p__count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 774);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_object` field */
		_ALIGN(ctx->at, 8);

		/* Write `_object` field */
		{
			const uint64_t tmp_val = (uint64_t) p__object;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint32_t tmp_val = (uint32_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hip_api_hipUserObjectRetainEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 775);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hip_api_hipWaitExternalSemaphoresAsyncBegin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__extSemArray,
	const uint64_t p__extSemArray__val,
	const uint64_t p__paramsArray,
	const uint32_t p__paramsArray__val__flags,
	const uint64_t p__paramsArray__val__reserved,
	const uint32_t p__numExtSems,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 776);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_extSemArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extSemArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extSemArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_extSemArray__val` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extSemArray__val` field */
		{
			const uint64_t tmp_val = (uint64_t) p__extSemArray__val;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_paramsArray` field */
		_ALIGN(ctx->at, 8);

		/* Write `_paramsArray` field */
		{
			const uint64_t tmp_val = (uint64_t) p__paramsArray;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_paramsArray__val__flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_paramsArray__val__flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__paramsArray__val__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_paramsArray__val__reserved` field */
		_ALIGN(ctx->at, 8);

		/* Write `_paramsArray__val__reserved` field */
		{
			const uint64_t tmp_val = (uint64_t) p__paramsArray__val__reserved;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_numExtSems` field */
		_ALIGN(ctx->at, 8);

		/* Write `_numExtSems` field */
		{
			const uint32_t tmp_val = (uint32_t) p__numExtSems;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_stream` field */
		_ALIGN(ctx->at, 8);

		/* Write `_stream` field */
		{
			const uint64_t tmp_val = (uint64_t) p__stream;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hip_api_hipWaitExternalSemaphoresAsyncEnd(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hip_api(ctx, 777);

	/* Serialize common context */
	_serialize_er_common_ctx_hip_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Write payload structure */
	{
	}
}

static uint32_t _er_size_hip_api___hipPopCallConfigurationBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_gridDim` field */
		_ALIGN(at, 8);

		/* Add `_gridDim` bit array field's size */
		at += 64;

		/* Align for `_gridDim__val__x` field */
		_ALIGN(at, 8);

		/* Add `_gridDim__val__x` bit array field's size */
		at += 32;

		/* Align for `_gridDim__val__y` field */
		_ALIGN(at, 8);

		/* Add `_gridDim__val__y` bit array field's size */
		at += 32;

		/* Align for `_gridDim__val__z` field */
		_ALIGN(at, 8);

		/* Add `_gridDim__val__z` bit array field's size */
		at += 32;

		/* Align for `_blockDim` field */
		_ALIGN(at, 8);

		/* Add `_blockDim` bit array field's size */
		at += 64;

		/* Align for `_blockDim__val__x` field */
		_ALIGN(at, 8);

		/* Add `_blockDim__val__x` bit array field's size */
		at += 32;

		/* Align for `_blockDim__val__y` field */
		_ALIGN(at, 8);

		/* Add `_blockDim__val__y` bit array field's size */
		at += 32;

		/* Align for `_blockDim__val__z` field */
		_ALIGN(at, 8);

		/* Add `_blockDim__val__z` bit array field's size */
		at += 32;

		/* Align for `_sharedMem` field */
		_ALIGN(at, 8);

		/* Add `_sharedMem` bit array field's size */
		at += 64;

		/* Align for `_sharedMem__val` field */
		_ALIGN(at, 8);

		/* Add `_sharedMem__val` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_stream__val` field */
		_ALIGN(at, 8);

		/* Add `_stream__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api___hipPopCallConfigurationEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api___hipPushCallConfigurationBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_gridDim__x` field */
		_ALIGN(at, 8);

		/* Add `_gridDim__x` bit array field's size */
		at += 32;

		/* Align for `_gridDim__y` field */
		_ALIGN(at, 8);

		/* Add `_gridDim__y` bit array field's size */
		at += 32;

		/* Align for `_gridDim__z` field */
		_ALIGN(at, 8);

		/* Add `_gridDim__z` bit array field's size */
		at += 32;

		/* Align for `_blockDim__x` field */
		_ALIGN(at, 8);

		/* Add `_blockDim__x` bit array field's size */
		at += 32;

		/* Align for `_blockDim__y` field */
		_ALIGN(at, 8);

		/* Add `_blockDim__y` bit array field's size */
		at += 32;

		/* Align for `_blockDim__z` field */
		_ALIGN(at, 8);

		/* Add `_blockDim__z` bit array field's size */
		at += 32;

		/* Align for `_sharedMem` field */
		_ALIGN(at, 8);

		/* Add `_sharedMem` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api___hipPushCallConfigurationEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipArray3DCreateBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_array` field */
		_ALIGN(at, 8);

		/* Add `_array` bit array field's size */
		at += 64;

		/* Align for `_array__val` field */
		_ALIGN(at, 8);

		/* Add `_array__val` bit array field's size */
		at += 64;

		/* Align for `_pAllocateArray` field */
		_ALIGN(at, 8);

		/* Add `_pAllocateArray` bit array field's size */
		at += 64;

		/* Align for `_pAllocateArray__val__Width` field */
		_ALIGN(at, 8);

		/* Add `_pAllocateArray__val__Width` bit array field's size */
		at += 64;

		/* Align for `_pAllocateArray__val__Height` field */
		_ALIGN(at, 8);

		/* Add `_pAllocateArray__val__Height` bit array field's size */
		at += 64;

		/* Align for `_pAllocateArray__val__Depth` field */
		_ALIGN(at, 8);

		/* Add `_pAllocateArray__val__Depth` bit array field's size */
		at += 64;

		/* Align for `_pAllocateArray__val__Format` field */
		_ALIGN(at, 8);

		/* Add `_pAllocateArray__val__Format` bit array field's size */
		at += 32;

		/* Align for `_pAllocateArray__val__NumChannels` field */
		_ALIGN(at, 8);

		/* Add `_pAllocateArray__val__NumChannels` bit array field's size */
		at += 32;

		/* Align for `_pAllocateArray__val__Flags` field */
		_ALIGN(at, 8);

		/* Add `_pAllocateArray__val__Flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipArray3DCreateEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipArray3DGetDescriptorBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pArrayDescriptor` field */
		_ALIGN(at, 8);

		/* Add `_pArrayDescriptor` bit array field's size */
		at += 64;

		/* Align for `_pArrayDescriptor__val__Width` field */
		_ALIGN(at, 8);

		/* Add `_pArrayDescriptor__val__Width` bit array field's size */
		at += 64;

		/* Align for `_pArrayDescriptor__val__Height` field */
		_ALIGN(at, 8);

		/* Add `_pArrayDescriptor__val__Height` bit array field's size */
		at += 64;

		/* Align for `_pArrayDescriptor__val__Depth` field */
		_ALIGN(at, 8);

		/* Add `_pArrayDescriptor__val__Depth` bit array field's size */
		at += 64;

		/* Align for `_pArrayDescriptor__val__Format` field */
		_ALIGN(at, 8);

		/* Add `_pArrayDescriptor__val__Format` bit array field's size */
		at += 32;

		/* Align for `_pArrayDescriptor__val__NumChannels` field */
		_ALIGN(at, 8);

		/* Add `_pArrayDescriptor__val__NumChannels` bit array field's size */
		at += 32;

		/* Align for `_pArrayDescriptor__val__Flags` field */
		_ALIGN(at, 8);

		/* Add `_pArrayDescriptor__val__Flags` bit array field's size */
		at += 32;

		/* Align for `_array` field */
		_ALIGN(at, 8);

		/* Add `_array` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipArray3DGetDescriptorEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipArrayCreateBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pHandle` field */
		_ALIGN(at, 8);

		/* Add `_pHandle` bit array field's size */
		at += 64;

		/* Align for `_pHandle__val` field */
		_ALIGN(at, 8);

		/* Add `_pHandle__val` bit array field's size */
		at += 64;

		/* Align for `_pAllocateArray` field */
		_ALIGN(at, 8);

		/* Add `_pAllocateArray` bit array field's size */
		at += 64;

		/* Align for `_pAllocateArray__val__Width` field */
		_ALIGN(at, 8);

		/* Add `_pAllocateArray__val__Width` bit array field's size */
		at += 64;

		/* Align for `_pAllocateArray__val__Height` field */
		_ALIGN(at, 8);

		/* Add `_pAllocateArray__val__Height` bit array field's size */
		at += 64;

		/* Align for `_pAllocateArray__val__Format` field */
		_ALIGN(at, 8);

		/* Add `_pAllocateArray__val__Format` bit array field's size */
		at += 32;

		/* Align for `_pAllocateArray__val__NumChannels` field */
		_ALIGN(at, 8);

		/* Add `_pAllocateArray__val__NumChannels` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipArrayCreateEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipArrayDestroyBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_array` field */
		_ALIGN(at, 8);

		/* Add `_array` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipArrayDestroyEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipArrayGetDescriptorBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pArrayDescriptor` field */
		_ALIGN(at, 8);

		/* Add `_pArrayDescriptor` bit array field's size */
		at += 64;

		/* Align for `_pArrayDescriptor__val__Width` field */
		_ALIGN(at, 8);

		/* Add `_pArrayDescriptor__val__Width` bit array field's size */
		at += 64;

		/* Align for `_pArrayDescriptor__val__Height` field */
		_ALIGN(at, 8);

		/* Add `_pArrayDescriptor__val__Height` bit array field's size */
		at += 64;

		/* Align for `_pArrayDescriptor__val__Format` field */
		_ALIGN(at, 8);

		/* Add `_pArrayDescriptor__val__Format` bit array field's size */
		at += 32;

		/* Align for `_pArrayDescriptor__val__NumChannels` field */
		_ALIGN(at, 8);

		/* Add `_pArrayDescriptor__val__NumChannels` bit array field's size */
		at += 32;

		/* Align for `_array` field */
		_ALIGN(at, 8);

		/* Add `_array` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipArrayGetDescriptorEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipArrayGetInfoBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_desc` field */
		_ALIGN(at, 8);

		/* Add `_desc` bit array field's size */
		at += 64;

		/* Align for `_desc__val__x` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__x` bit array field's size */
		at += 32;

		/* Align for `_desc__val__y` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__y` bit array field's size */
		at += 32;

		/* Align for `_desc__val__z` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__z` bit array field's size */
		at += 32;

		/* Align for `_desc__val__w` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__w` bit array field's size */
		at += 32;

		/* Align for `_desc__val__f` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__f` bit array field's size */
		at += 32;

		/* Align for `_extent` field */
		_ALIGN(at, 8);

		/* Add `_extent` bit array field's size */
		at += 64;

		/* Align for `_extent__val__width` field */
		_ALIGN(at, 8);

		/* Add `_extent__val__width` bit array field's size */
		at += 64;

		/* Align for `_extent__val__height` field */
		_ALIGN(at, 8);

		/* Add `_extent__val__height` bit array field's size */
		at += 64;

		/* Align for `_extent__val__depth` field */
		_ALIGN(at, 8);

		/* Add `_extent__val__depth` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;

		/* Align for `_flags__val` field */
		_ALIGN(at, 8);

		/* Add `_flags__val` bit array field's size */
		at += 32;

		/* Align for `_array` field */
		_ALIGN(at, 8);

		/* Add `_array` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipArrayGetInfoEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipChooseDeviceBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipChooseDeviceEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipChooseDeviceR0000Begin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 64;

		/* Align for `_device__val` field */
		_ALIGN(at, 8);

		/* Add `_device__val` bit array field's size */
		at += 32;

		/* Align for `_prop` field */
		_ALIGN(at, 8);

		/* Add `_prop` bit array field's size */
		at += 64;

		/* Align for `_prop__val__name` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__name` bit array field's size */
		at += 64;

		/* Align for `_prop__val__totalGlobalMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__totalGlobalMem` bit array field's size */
		at += 64;

		/* Align for `_prop__val__sharedMemPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__sharedMemPerBlock` bit array field's size */
		at += 64;

		/* Align for `_prop__val__regsPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__regsPerBlock` bit array field's size */
		at += 32;

		/* Align for `_prop__val__warpSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__warpSize` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxThreadsPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxThreadsPerBlock` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxThreadsDim` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxThreadsDim` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxGridSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxGridSize` bit array field's size */
		at += 64;

		/* Align for `_prop__val__clockRate` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__clockRate` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memoryClockRate` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memoryClockRate` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memoryBusWidth` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memoryBusWidth` bit array field's size */
		at += 32;

		/* Align for `_prop__val__totalConstMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__totalConstMem` bit array field's size */
		at += 64;

		/* Align for `_prop__val__major` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__major` bit array field's size */
		at += 32;

		/* Align for `_prop__val__minor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__minor` bit array field's size */
		at += 32;

		/* Align for `_prop__val__multiProcessorCount` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__multiProcessorCount` bit array field's size */
		at += 32;

		/* Align for `_prop__val__l2CacheSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__l2CacheSize` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxThreadsPerMultiProcessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxThreadsPerMultiProcessor` bit array field's size */
		at += 32;

		/* Align for `_prop__val__computeMode` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__computeMode` bit array field's size */
		at += 32;

		/* Align for `_prop__val__clockInstructionRate` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__clockInstructionRate` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasGlobalInt32Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasGlobalInt32Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasGlobalFloatAtomicExch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasGlobalFloatAtomicExch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSharedInt32Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSharedInt32Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSharedFloatAtomicExch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSharedFloatAtomicExch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasFloatAtomicAdd` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasFloatAtomicAdd` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasGlobalInt64Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasGlobalInt64Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSharedInt64Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSharedInt64Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasDoubles` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasDoubles` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasWarpVote` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasWarpVote` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasWarpBallot` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasWarpBallot` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasWarpShuffle` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasWarpShuffle` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasFunnelShift` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasFunnelShift` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasThreadFenceSystem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasThreadFenceSystem` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSyncThreadsExt` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSyncThreadsExt` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSurfaceFuncs` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSurfaceFuncs` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__has3dGrid` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__has3dGrid` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasDynamicParallelism` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasDynamicParallelism` bit array field's size */
		at += 32;

		/* Align for `_prop__val__concurrentKernels` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__concurrentKernels` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pciDomainID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pciDomainID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pciBusID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pciBusID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pciDeviceID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pciDeviceID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxSharedMemoryPerMultiProcessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSharedMemoryPerMultiProcessor` bit array field's size */
		at += 64;

		/* Align for `_prop__val__isMultiGpuBoard` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__isMultiGpuBoard` bit array field's size */
		at += 32;

		/* Align for `_prop__val__canMapHostMemory` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__canMapHostMemory` bit array field's size */
		at += 32;

		/* Align for `_prop__val__gcnArch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__gcnArch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__gcnArchName` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__gcnArchName` bit array field's size */
		at += 64;

		/* Align for `_prop__val__integrated` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__integrated` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeLaunch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeLaunch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceLaunch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceLaunch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture1DLinear` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture1DLinear` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture1D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture1D` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture2D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture2D` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture3D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture3D` bit array field's size */
		at += 64;

		/* Align for `_prop__val__hdpMemFlushCntl` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hdpMemFlushCntl` bit array field's size */
		at += 64;

		/* Align for `_prop__val__hdpRegFlushCntl` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hdpRegFlushCntl` bit array field's size */
		at += 64;

		/* Align for `_prop__val__memPitch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memPitch` bit array field's size */
		at += 64;

		/* Align for `_prop__val__textureAlignment` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__textureAlignment` bit array field's size */
		at += 64;

		/* Align for `_prop__val__texturePitchAlignment` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__texturePitchAlignment` bit array field's size */
		at += 64;

		/* Align for `_prop__val__kernelExecTimeoutEnabled` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__kernelExecTimeoutEnabled` bit array field's size */
		at += 32;

		/* Align for `_prop__val__ECCEnabled` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__ECCEnabled` bit array field's size */
		at += 32;

		/* Align for `_prop__val__tccDriver` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__tccDriver` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedFunc` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedFunc` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` bit array field's size */
		at += 32;

		/* Align for `_prop__val__isLargeBar` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__isLargeBar` bit array field's size */
		at += 32;

		/* Align for `_prop__val__asicRevision` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__asicRevision` bit array field's size */
		at += 32;

		/* Align for `_prop__val__managedMemory` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__managedMemory` bit array field's size */
		at += 32;

		/* Align for `_prop__val__directManagedMemAccessFromHost` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__directManagedMemAccessFromHost` bit array field's size */
		at += 32;

		/* Align for `_prop__val__concurrentManagedAccess` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__concurrentManagedAccess` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pageableMemoryAccess` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pageableMemoryAccess` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pageableMemoryAccessUsesHostPageTables` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pageableMemoryAccessUsesHostPageTables` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipChooseDeviceR0000End(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipChooseDeviceR0600Begin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 64;

		/* Align for `_device__val` field */
		_ALIGN(at, 8);

		/* Add `_device__val` bit array field's size */
		at += 32;

		/* Align for `_prop` field */
		_ALIGN(at, 8);

		/* Add `_prop` bit array field's size */
		at += 64;

		/* Align for `_prop__val__name` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__name` bit array field's size */
		at += 64;

		/* Align for `_prop__val__uuid__bytes` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__uuid__bytes` bit array field's size */
		at += 64;

		/* Align for `_prop__val__luid` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__luid` bit array field's size */
		at += 64;

		/* Align for `_prop__val__luidDeviceNodeMask` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__luidDeviceNodeMask` bit array field's size */
		at += 32;

		/* Align for `_prop__val__totalGlobalMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__totalGlobalMem` bit array field's size */
		at += 64;

		/* Align for `_prop__val__sharedMemPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__sharedMemPerBlock` bit array field's size */
		at += 64;

		/* Align for `_prop__val__regsPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__regsPerBlock` bit array field's size */
		at += 32;

		/* Align for `_prop__val__warpSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__warpSize` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memPitch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memPitch` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxThreadsPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxThreadsPerBlock` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxThreadsDim` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxThreadsDim` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxGridSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxGridSize` bit array field's size */
		at += 64;

		/* Align for `_prop__val__clockRate` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__clockRate` bit array field's size */
		at += 32;

		/* Align for `_prop__val__totalConstMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__totalConstMem` bit array field's size */
		at += 64;

		/* Align for `_prop__val__major` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__major` bit array field's size */
		at += 32;

		/* Align for `_prop__val__minor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__minor` bit array field's size */
		at += 32;

		/* Align for `_prop__val__textureAlignment` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__textureAlignment` bit array field's size */
		at += 64;

		/* Align for `_prop__val__texturePitchAlignment` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__texturePitchAlignment` bit array field's size */
		at += 64;

		/* Align for `_prop__val__deviceOverlap` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__deviceOverlap` bit array field's size */
		at += 32;

		/* Align for `_prop__val__multiProcessorCount` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__multiProcessorCount` bit array field's size */
		at += 32;

		/* Align for `_prop__val__kernelExecTimeoutEnabled` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__kernelExecTimeoutEnabled` bit array field's size */
		at += 32;

		/* Align for `_prop__val__integrated` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__integrated` bit array field's size */
		at += 32;

		/* Align for `_prop__val__canMapHostMemory` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__canMapHostMemory` bit array field's size */
		at += 32;

		/* Align for `_prop__val__computeMode` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__computeMode` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture1D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture1D` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture1DMipmap` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture1DMipmap` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture1DLinear` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture1DLinear` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture2D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture2D` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture2DMipmap` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture2DMipmap` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture2DLinear` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture2DLinear` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture2DGather` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture2DGather` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture3D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture3D` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture3DAlt` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture3DAlt` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTextureCubemap` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTextureCubemap` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture1DLayered` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture1DLayered` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture2DLayered` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture2DLayered` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTextureCubemapLayered` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTextureCubemapLayered` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxSurface1D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurface1D` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxSurface2D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurface2D` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxSurface3D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurface3D` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxSurface1DLayered` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurface1DLayered` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxSurface2DLayered` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurface2DLayered` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxSurfaceCubemap` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurfaceCubemap` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxSurfaceCubemapLayered` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurfaceCubemapLayered` bit array field's size */
		at += 64;

		/* Align for `_prop__val__surfaceAlignment` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__surfaceAlignment` bit array field's size */
		at += 64;

		/* Align for `_prop__val__concurrentKernels` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__concurrentKernels` bit array field's size */
		at += 32;

		/* Align for `_prop__val__ECCEnabled` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__ECCEnabled` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pciBusID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pciBusID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pciDeviceID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pciDeviceID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pciDomainID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pciDomainID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__tccDriver` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__tccDriver` bit array field's size */
		at += 32;

		/* Align for `_prop__val__asyncEngineCount` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__asyncEngineCount` bit array field's size */
		at += 32;

		/* Align for `_prop__val__unifiedAddressing` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__unifiedAddressing` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memoryClockRate` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memoryClockRate` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memoryBusWidth` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memoryBusWidth` bit array field's size */
		at += 32;

		/* Align for `_prop__val__l2CacheSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__l2CacheSize` bit array field's size */
		at += 32;

		/* Align for `_prop__val__persistingL2CacheMaxSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__persistingL2CacheMaxSize` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxThreadsPerMultiProcessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxThreadsPerMultiProcessor` bit array field's size */
		at += 32;

		/* Align for `_prop__val__streamPrioritiesSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__streamPrioritiesSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__globalL1CacheSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__globalL1CacheSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__localL1CacheSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__localL1CacheSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__sharedMemPerMultiprocessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__sharedMemPerMultiprocessor` bit array field's size */
		at += 64;

		/* Align for `_prop__val__regsPerMultiprocessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__regsPerMultiprocessor` bit array field's size */
		at += 32;

		/* Align for `_prop__val__managedMemory` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__managedMemory` bit array field's size */
		at += 32;

		/* Align for `_prop__val__isMultiGpuBoard` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__isMultiGpuBoard` bit array field's size */
		at += 32;

		/* Align for `_prop__val__multiGpuBoardGroupID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__multiGpuBoardGroupID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__hostNativeAtomicSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hostNativeAtomicSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__singleToDoublePrecisionPerfRatio` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__singleToDoublePrecisionPerfRatio` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pageableMemoryAccess` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pageableMemoryAccess` bit array field's size */
		at += 32;

		/* Align for `_prop__val__concurrentManagedAccess` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__concurrentManagedAccess` bit array field's size */
		at += 32;

		/* Align for `_prop__val__computePreemptionSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__computePreemptionSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__canUseHostPointerForRegisteredMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__canUseHostPointerForRegisteredMem` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeLaunch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeLaunch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceLaunch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceLaunch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__sharedMemPerBlockOptin` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__sharedMemPerBlockOptin` bit array field's size */
		at += 64;

		/* Align for `_prop__val__pageableMemoryAccessUsesHostPageTables` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pageableMemoryAccessUsesHostPageTables` bit array field's size */
		at += 32;

		/* Align for `_prop__val__directManagedMemAccessFromHost` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__directManagedMemAccessFromHost` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxBlocksPerMultiProcessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxBlocksPerMultiProcessor` bit array field's size */
		at += 32;

		/* Align for `_prop__val__accessPolicyMaxWindowSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__accessPolicyMaxWindowSize` bit array field's size */
		at += 32;

		/* Align for `_prop__val__reservedSharedMemPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__reservedSharedMemPerBlock` bit array field's size */
		at += 64;

		/* Align for `_prop__val__hostRegisterSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hostRegisterSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__sparseHipArraySupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__sparseHipArraySupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__hostRegisterReadOnlySupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hostRegisterReadOnlySupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__timelineSemaphoreInteropSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__timelineSemaphoreInteropSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memoryPoolsSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memoryPoolsSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__gpuDirectRDMASupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__gpuDirectRDMASupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__gpuDirectRDMAFlushWritesOptions` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__gpuDirectRDMAFlushWritesOptions` bit array field's size */
		at += 32;

		/* Align for `_prop__val__gpuDirectRDMAWritesOrdering` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__gpuDirectRDMAWritesOrdering` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memoryPoolSupportedHandleTypes` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memoryPoolSupportedHandleTypes` bit array field's size */
		at += 32;

		/* Align for `_prop__val__deferredMappingHipArraySupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__deferredMappingHipArraySupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__ipcEventSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__ipcEventSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__clusterLaunch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__clusterLaunch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__unifiedFunctionPointers` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__unifiedFunctionPointers` bit array field's size */
		at += 32;

		/* Align for `_prop__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__reserved` bit array field's size */
		at += 64;

		/* Align for `_prop__val__hipReserved` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hipReserved` bit array field's size */
		at += 64;

		/* Align for `_prop__val__gcnArchName` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__gcnArchName` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxSharedMemoryPerMultiProcessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSharedMemoryPerMultiProcessor` bit array field's size */
		at += 64;

		/* Align for `_prop__val__clockInstructionRate` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__clockInstructionRate` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasGlobalInt32Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasGlobalInt32Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasGlobalFloatAtomicExch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasGlobalFloatAtomicExch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSharedInt32Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSharedInt32Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSharedFloatAtomicExch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSharedFloatAtomicExch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasFloatAtomicAdd` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasFloatAtomicAdd` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasGlobalInt64Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasGlobalInt64Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSharedInt64Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSharedInt64Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasDoubles` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasDoubles` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasWarpVote` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasWarpVote` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasWarpBallot` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasWarpBallot` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasWarpShuffle` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasWarpShuffle` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasFunnelShift` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasFunnelShift` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasThreadFenceSystem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasThreadFenceSystem` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSyncThreadsExt` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSyncThreadsExt` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSurfaceFuncs` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSurfaceFuncs` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__has3dGrid` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__has3dGrid` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasDynamicParallelism` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasDynamicParallelism` bit array field's size */
		at += 32;

		/* Align for `_prop__val__hdpMemFlushCntl` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hdpMemFlushCntl` bit array field's size */
		at += 64;

		/* Align for `_prop__val__hdpRegFlushCntl` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hdpRegFlushCntl` bit array field's size */
		at += 64;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedFunc` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedFunc` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` bit array field's size */
		at += 32;

		/* Align for `_prop__val__isLargeBar` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__isLargeBar` bit array field's size */
		at += 32;

		/* Align for `_prop__val__asicRevision` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__asicRevision` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipChooseDeviceR0600End(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipConfigureCallBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_gridDim__x` field */
		_ALIGN(at, 8);

		/* Add `_gridDim__x` bit array field's size */
		at += 32;

		/* Align for `_gridDim__y` field */
		_ALIGN(at, 8);

		/* Add `_gridDim__y` bit array field's size */
		at += 32;

		/* Align for `_gridDim__z` field */
		_ALIGN(at, 8);

		/* Add `_gridDim__z` bit array field's size */
		at += 32;

		/* Align for `_blockDim__x` field */
		_ALIGN(at, 8);

		/* Add `_blockDim__x` bit array field's size */
		at += 32;

		/* Align for `_blockDim__y` field */
		_ALIGN(at, 8);

		/* Add `_blockDim__y` bit array field's size */
		at += 32;

		/* Align for `_blockDim__z` field */
		_ALIGN(at, 8);

		/* Add `_blockDim__z` bit array field's size */
		at += 32;

		/* Align for `_sharedMem` field */
		_ALIGN(at, 8);

		/* Add `_sharedMem` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipConfigureCallEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCreateSurfaceObjectBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pSurfObject` field */
		_ALIGN(at, 8);

		/* Add `_pSurfObject` bit array field's size */
		at += 64;

		/* Align for `_pSurfObject__val` field */
		_ALIGN(at, 8);

		/* Add `_pSurfObject__val` bit array field's size */
		at += 64;

		/* Align for `_pResDesc` field */
		_ALIGN(at, 8);

		/* Add `_pResDesc` bit array field's size */
		at += 64;

		/* Align for `_pResDesc__val__resType` field */
		_ALIGN(at, 8);

		/* Add `_pResDesc__val__resType` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCreateSurfaceObjectEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxCreateBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ctx` field */
		_ALIGN(at, 8);

		/* Add `_ctx` bit array field's size */
		at += 64;

		/* Align for `_ctx__val` field */
		_ALIGN(at, 8);

		/* Add `_ctx__val` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxCreateEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxDestroyBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ctx` field */
		_ALIGN(at, 8);

		/* Add `_ctx` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxDestroyEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxDisablePeerAccessBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_peerCtx` field */
		_ALIGN(at, 8);

		/* Add `_peerCtx` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxDisablePeerAccessEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxEnablePeerAccessBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_peerCtx` field */
		_ALIGN(at, 8);

		/* Add `_peerCtx` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxEnablePeerAccessEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxGetApiVersionBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ctx` field */
		_ALIGN(at, 8);

		/* Add `_ctx` bit array field's size */
		at += 64;

		/* Align for `_apiVersion` field */
		_ALIGN(at, 8);

		/* Add `_apiVersion` bit array field's size */
		at += 64;

		/* Align for `_apiVersion__val` field */
		_ALIGN(at, 8);

		/* Add `_apiVersion__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxGetApiVersionEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxGetCacheConfigBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_cacheConfig` field */
		_ALIGN(at, 8);

		/* Add `_cacheConfig` bit array field's size */
		at += 64;

		/* Align for `_cacheConfig__val` field */
		_ALIGN(at, 8);

		/* Add `_cacheConfig__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxGetCacheConfigEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxGetCurrentBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ctx` field */
		_ALIGN(at, 8);

		/* Add `_ctx` bit array field's size */
		at += 64;

		/* Align for `_ctx__val` field */
		_ALIGN(at, 8);

		/* Add `_ctx__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxGetCurrentEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxGetDeviceBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 64;

		/* Align for `_device__val` field */
		_ALIGN(at, 8);

		/* Add `_device__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxGetDeviceEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxGetFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;

		/* Align for `_flags__val` field */
		_ALIGN(at, 8);

		/* Add `_flags__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxGetFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxGetSharedMemConfigBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pConfig` field */
		_ALIGN(at, 8);

		/* Add `_pConfig` bit array field's size */
		at += 64;

		/* Align for `_pConfig__val` field */
		_ALIGN(at, 8);

		/* Add `_pConfig__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxGetSharedMemConfigEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxPopCurrentBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ctx` field */
		_ALIGN(at, 8);

		/* Add `_ctx` bit array field's size */
		at += 64;

		/* Align for `_ctx__val` field */
		_ALIGN(at, 8);

		/* Add `_ctx__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxPopCurrentEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxPushCurrentBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ctx` field */
		_ALIGN(at, 8);

		/* Add `_ctx` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxPushCurrentEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxSetCacheConfigBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_cacheConfig` field */
		_ALIGN(at, 8);

		/* Add `_cacheConfig` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxSetCacheConfigEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxSetCurrentBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ctx` field */
		_ALIGN(at, 8);

		/* Add `_ctx` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxSetCurrentEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxSetSharedMemConfigBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_config` field */
		_ALIGN(at, 8);

		/* Add `_config` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxSetSharedMemConfigEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxSynchronizeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipCtxSynchronizeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDestroyExternalMemoryBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_extMem` field */
		_ALIGN(at, 8);

		/* Add `_extMem` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDestroyExternalMemoryEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDestroyExternalSemaphoreBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_extSem` field */
		_ALIGN(at, 8);

		/* Add `_extSem` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDestroyExternalSemaphoreEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDestroySurfaceObjectBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_surfaceObject` field */
		_ALIGN(at, 8);

		/* Add `_surfaceObject` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDestroySurfaceObjectEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceCanAccessPeerBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_canAccessPeer` field */
		_ALIGN(at, 8);

		/* Add `_canAccessPeer` bit array field's size */
		at += 64;

		/* Align for `_canAccessPeer__val` field */
		_ALIGN(at, 8);

		/* Add `_canAccessPeer__val` bit array field's size */
		at += 32;

		/* Align for `_deviceId` field */
		_ALIGN(at, 8);

		/* Add `_deviceId` bit array field's size */
		at += 32;

		/* Align for `_peerDeviceId` field */
		_ALIGN(at, 8);

		/* Add `_peerDeviceId` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceCanAccessPeerEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceComputeCapabilityBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_major` field */
		_ALIGN(at, 8);

		/* Add `_major` bit array field's size */
		at += 64;

		/* Align for `_major__val` field */
		_ALIGN(at, 8);

		/* Add `_major__val` bit array field's size */
		at += 32;

		/* Align for `_minor` field */
		_ALIGN(at, 8);

		/* Add `_minor` bit array field's size */
		at += 64;

		/* Align for `_minor__val` field */
		_ALIGN(at, 8);

		/* Add `_minor__val` bit array field's size */
		at += 32;

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceComputeCapabilityEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceDisablePeerAccessBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_peerDeviceId` field */
		_ALIGN(at, 8);

		/* Add `_peerDeviceId` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceDisablePeerAccessEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceEnablePeerAccessBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_peerDeviceId` field */
		_ALIGN(at, 8);

		/* Add `_peerDeviceId` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceEnablePeerAccessEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetAttributeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pi` field */
		_ALIGN(at, 8);

		/* Add `_pi` bit array field's size */
		at += 64;

		/* Align for `_pi__val` field */
		_ALIGN(at, 8);

		/* Add `_pi__val` bit array field's size */
		at += 32;

		/* Align for `_attr` field */
		_ALIGN(at, 8);

		/* Add `_attr` bit array field's size */
		at += 32;

		/* Align for `_deviceId` field */
		_ALIGN(at, 8);

		/* Add `_deviceId` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetAttributeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 64;

		/* Align for `_device__val` field */
		_ALIGN(at, 8);

		/* Add `_device__val` bit array field's size */
		at += 32;

		/* Align for `_ordinal` field */
		_ALIGN(at, 8);

		/* Add `_ordinal` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetByPCIBusIdBegin(void * const vctx,
	const char * const cc__kernel_name,
	const char * const p__pciBusId)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 64;

		/* Align for `_device__val` field */
		_ALIGN(at, 8);

		/* Add `_device__val` bit array field's size */
		at += 32;

		/* Align for `_pciBusId` field */
		_ALIGN(at, 8);

		/* Add `_pciBusId` string field's size */
		at += _BYTES_TO_BITS(strlen(p__pciBusId) + 1);

		/* Align for `_pciBusId__val` field */
		_ALIGN(at, 8);

		/* Add `_pciBusId__val` bit array field's size */
		at += 8;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetByPCIBusIdEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetCacheConfigBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_cacheConfig` field */
		_ALIGN(at, 8);

		/* Add `_cacheConfig` bit array field's size */
		at += 64;

		/* Align for `_cacheConfig__val` field */
		_ALIGN(at, 8);

		/* Add `_cacheConfig__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetCacheConfigEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetDefaultMemPoolBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;

		/* Align for `_mem_pool__val` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool__val` bit array field's size */
		at += 64;

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetDefaultMemPoolEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetGraphMemAttributeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;

		/* Align for `_attr` field */
		_ALIGN(at, 8);

		/* Add `_attr` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetGraphMemAttributeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetLimitBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pValue` field */
		_ALIGN(at, 8);

		/* Add `_pValue` bit array field's size */
		at += 64;

		/* Align for `_pValue__val` field */
		_ALIGN(at, 8);

		/* Add `_pValue__val` bit array field's size */
		at += 64;

		/* Align for `_limit` field */
		_ALIGN(at, 8);

		/* Add `_limit` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetLimitEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetMemPoolBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;

		/* Align for `_mem_pool__val` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool__val` bit array field's size */
		at += 64;

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetMemPoolEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetNameBegin(void * const vctx,
	const char * const cc__kernel_name,
	const char * const p__name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_name` field */
		_ALIGN(at, 8);

		/* Add `_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__name) + 1);

		/* Align for `_name__val` field */
		_ALIGN(at, 8);

		/* Add `_name__val` bit array field's size */
		at += 8;

		/* Align for `_len` field */
		_ALIGN(at, 8);

		/* Add `_len` bit array field's size */
		at += 32;

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetNameEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetP2PAttributeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;

		/* Align for `_value__val` field */
		_ALIGN(at, 8);

		/* Add `_value__val` bit array field's size */
		at += 32;

		/* Align for `_attr` field */
		_ALIGN(at, 8);

		/* Add `_attr` bit array field's size */
		at += 32;

		/* Align for `_srcDevice` field */
		_ALIGN(at, 8);

		/* Add `_srcDevice` bit array field's size */
		at += 32;

		/* Align for `_dstDevice` field */
		_ALIGN(at, 8);

		/* Add `_dstDevice` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetP2PAttributeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetPCIBusIdBegin(void * const vctx,
	const char * const cc__kernel_name,
	const char * const p__pciBusId)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pciBusId` field */
		_ALIGN(at, 8);

		/* Add `_pciBusId` string field's size */
		at += _BYTES_TO_BITS(strlen(p__pciBusId) + 1);

		/* Align for `_pciBusId__val` field */
		_ALIGN(at, 8);

		/* Add `_pciBusId__val` bit array field's size */
		at += 8;

		/* Align for `_len` field */
		_ALIGN(at, 8);

		/* Add `_len` bit array field's size */
		at += 32;

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetPCIBusIdEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetSharedMemConfigBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pConfig` field */
		_ALIGN(at, 8);

		/* Add `_pConfig` bit array field's size */
		at += 64;

		/* Align for `_pConfig__val` field */
		_ALIGN(at, 8);

		/* Add `_pConfig__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetSharedMemConfigEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetStreamPriorityRangeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_leastPriority` field */
		_ALIGN(at, 8);

		/* Add `_leastPriority` bit array field's size */
		at += 64;

		/* Align for `_leastPriority__val` field */
		_ALIGN(at, 8);

		/* Add `_leastPriority__val` bit array field's size */
		at += 32;

		/* Align for `_greatestPriority` field */
		_ALIGN(at, 8);

		/* Add `_greatestPriority` bit array field's size */
		at += 64;

		/* Align for `_greatestPriority__val` field */
		_ALIGN(at, 8);

		/* Add `_greatestPriority__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetStreamPriorityRangeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetUuidBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_uuid` field */
		_ALIGN(at, 8);

		/* Add `_uuid` bit array field's size */
		at += 64;

		/* Align for `_uuid__val__bytes` field */
		_ALIGN(at, 8);

		/* Add `_uuid__val__bytes` bit array field's size */
		at += 64;

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGetUuidEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGraphMemTrimBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceGraphMemTrimEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDevicePrimaryCtxGetStateBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dev` field */
		_ALIGN(at, 8);

		/* Add `_dev` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;

		/* Align for `_flags__val` field */
		_ALIGN(at, 8);

		/* Add `_flags__val` bit array field's size */
		at += 32;

		/* Align for `_active` field */
		_ALIGN(at, 8);

		/* Add `_active` bit array field's size */
		at += 64;

		/* Align for `_active__val` field */
		_ALIGN(at, 8);

		/* Add `_active__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDevicePrimaryCtxGetStateEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDevicePrimaryCtxReleaseBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dev` field */
		_ALIGN(at, 8);

		/* Add `_dev` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDevicePrimaryCtxReleaseEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDevicePrimaryCtxResetBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dev` field */
		_ALIGN(at, 8);

		/* Add `_dev` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDevicePrimaryCtxResetEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDevicePrimaryCtxRetainBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pctx` field */
		_ALIGN(at, 8);

		/* Add `_pctx` bit array field's size */
		at += 64;

		/* Align for `_pctx__val` field */
		_ALIGN(at, 8);

		/* Add `_pctx__val` bit array field's size */
		at += 64;

		/* Align for `_dev` field */
		_ALIGN(at, 8);

		/* Add `_dev` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDevicePrimaryCtxRetainEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDevicePrimaryCtxSetFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dev` field */
		_ALIGN(at, 8);

		/* Add `_dev` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDevicePrimaryCtxSetFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceResetBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceResetEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceSetCacheConfigBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_cacheConfig` field */
		_ALIGN(at, 8);

		/* Add `_cacheConfig` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceSetCacheConfigEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceSetGraphMemAttributeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;

		/* Align for `_attr` field */
		_ALIGN(at, 8);

		/* Add `_attr` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceSetGraphMemAttributeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceSetLimitBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_limit` field */
		_ALIGN(at, 8);

		/* Add `_limit` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceSetLimitEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceSetMemPoolBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceSetMemPoolEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceSetSharedMemConfigBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_config` field */
		_ALIGN(at, 8);

		/* Add `_config` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceSetSharedMemConfigEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceSynchronizeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceSynchronizeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceTotalMemBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_bytes` field */
		_ALIGN(at, 8);

		/* Add `_bytes` bit array field's size */
		at += 64;

		/* Align for `_bytes__val` field */
		_ALIGN(at, 8);

		/* Add `_bytes__val` bit array field's size */
		at += 64;

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDeviceTotalMemEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDriverGetVersionBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_driverVersion` field */
		_ALIGN(at, 8);

		/* Add `_driverVersion` bit array field's size */
		at += 64;

		/* Align for `_driverVersion__val` field */
		_ALIGN(at, 8);

		/* Add `_driverVersion__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDriverGetVersionEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDrvGraphAddMemcpyNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_phGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_phGraphNode` bit array field's size */
		at += 64;

		/* Align for `_phGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_phGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_hGraph` field */
		_ALIGN(at, 8);

		/* Add `_hGraph` bit array field's size */
		at += 64;

		/* Align for `_dependencies` field */
		_ALIGN(at, 8);

		/* Add `_dependencies` bit array field's size */
		at += 64;

		/* Align for `_dependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_dependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_copyParams` field */
		_ALIGN(at, 8);

		/* Add `_copyParams` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__srcXInBytes` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__srcXInBytes` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__srcY` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__srcY` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__srcZ` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__srcZ` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__srcLOD` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__srcLOD` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__srcMemoryType` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__srcMemoryType` bit array field's size */
		at += 32;

		/* Align for `_copyParams__val__srcHost` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__srcHost` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__srcDevice` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__srcDevice` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__srcArray` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__srcArray` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__srcPitch` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__srcPitch` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__srcHeight` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__srcHeight` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__dstXInBytes` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__dstXInBytes` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__dstY` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__dstY` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__dstZ` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__dstZ` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__dstLOD` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__dstLOD` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__dstMemoryType` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__dstMemoryType` bit array field's size */
		at += 32;

		/* Align for `_copyParams__val__dstHost` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__dstHost` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__dstDevice` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__dstDevice` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__dstArray` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__dstArray` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__dstPitch` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__dstPitch` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__dstHeight` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__dstHeight` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__WidthInBytes` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__WidthInBytes` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__Height` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__Height` bit array field's size */
		at += 64;

		/* Align for `_copyParams__val__Depth` field */
		_ALIGN(at, 8);

		/* Add `_copyParams__val__Depth` bit array field's size */
		at += 64;

		/* Align for `_ctx` field */
		_ALIGN(at, 8);

		/* Add `_ctx` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDrvGraphAddMemcpyNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDrvGraphAddMemsetNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_phGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_phGraphNode` bit array field's size */
		at += 64;

		/* Align for `_phGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_phGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_hGraph` field */
		_ALIGN(at, 8);

		/* Add `_hGraph` bit array field's size */
		at += 64;

		/* Align for `_dependencies` field */
		_ALIGN(at, 8);

		/* Add `_dependencies` bit array field's size */
		at += 64;

		/* Align for `_dependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_dependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_memsetParams` field */
		_ALIGN(at, 8);

		/* Add `_memsetParams` bit array field's size */
		at += 64;

		/* Align for `_memsetParams__val__dst` field */
		_ALIGN(at, 8);

		/* Add `_memsetParams__val__dst` bit array field's size */
		at += 64;

		/* Align for `_memsetParams__val__pitch` field */
		_ALIGN(at, 8);

		/* Add `_memsetParams__val__pitch` bit array field's size */
		at += 64;

		/* Align for `_memsetParams__val__value` field */
		_ALIGN(at, 8);

		/* Add `_memsetParams__val__value` bit array field's size */
		at += 32;

		/* Align for `_memsetParams__val__elementSize` field */
		_ALIGN(at, 8);

		/* Add `_memsetParams__val__elementSize` bit array field's size */
		at += 32;

		/* Align for `_memsetParams__val__width` field */
		_ALIGN(at, 8);

		/* Add `_memsetParams__val__width` bit array field's size */
		at += 64;

		/* Align for `_memsetParams__val__height` field */
		_ALIGN(at, 8);

		/* Add `_memsetParams__val__height` bit array field's size */
		at += 64;

		/* Align for `_ctx` field */
		_ALIGN(at, 8);

		/* Add `_ctx` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDrvGraphAddMemsetNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDrvMemcpy2DUnalignedBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pCopy` field */
		_ALIGN(at, 8);

		/* Add `_pCopy` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcXInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcXInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcY` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcY` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcMemoryType` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcMemoryType` bit array field's size */
		at += 32;

		/* Align for `_pCopy__val__srcHost` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcHost` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcDevice` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcDevice` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcArray` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcArray` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcPitch` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcPitch` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstXInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstXInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstY` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstY` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstMemoryType` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstMemoryType` bit array field's size */
		at += 32;

		/* Align for `_pCopy__val__dstHost` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstHost` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstDevice` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstDevice` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstArray` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstArray` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstPitch` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstPitch` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__WidthInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__WidthInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__Height` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__Height` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDrvMemcpy2DUnalignedEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDrvMemcpy3DAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pCopy` field */
		_ALIGN(at, 8);

		/* Add `_pCopy` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcXInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcXInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcY` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcY` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcZ` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcZ` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcLOD` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcLOD` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcMemoryType` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcMemoryType` bit array field's size */
		at += 32;

		/* Align for `_pCopy__val__srcHost` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcHost` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcDevice` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcDevice` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcArray` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcArray` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcPitch` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcPitch` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcHeight` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcHeight` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstXInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstXInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstY` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstY` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstZ` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstZ` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstLOD` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstLOD` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstMemoryType` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstMemoryType` bit array field's size */
		at += 32;

		/* Align for `_pCopy__val__dstHost` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstHost` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstDevice` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstDevice` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstArray` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstArray` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstPitch` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstPitch` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstHeight` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstHeight` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__WidthInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__WidthInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__Height` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__Height` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__Depth` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__Depth` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDrvMemcpy3DAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDrvMemcpy3DBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pCopy` field */
		_ALIGN(at, 8);

		/* Add `_pCopy` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcXInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcXInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcY` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcY` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcZ` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcZ` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcLOD` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcLOD` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcMemoryType` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcMemoryType` bit array field's size */
		at += 32;

		/* Align for `_pCopy__val__srcHost` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcHost` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcDevice` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcDevice` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcArray` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcArray` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcPitch` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcPitch` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcHeight` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcHeight` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstXInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstXInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstY` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstY` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstZ` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstZ` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstLOD` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstLOD` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstMemoryType` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstMemoryType` bit array field's size */
		at += 32;

		/* Align for `_pCopy__val__dstHost` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstHost` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstDevice` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstDevice` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstArray` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstArray` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstPitch` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstPitch` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstHeight` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstHeight` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__WidthInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__WidthInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__Height` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__Height` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__Depth` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__Depth` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDrvMemcpy3DEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDrvPointerGetAttributesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_numAttributes` field */
		_ALIGN(at, 8);

		/* Add `_numAttributes` bit array field's size */
		at += 32;

		/* Align for `_attributes` field */
		_ALIGN(at, 8);

		/* Add `_attributes` bit array field's size */
		at += 64;

		/* Align for `_attributes__val` field */
		_ALIGN(at, 8);

		/* Add `_attributes__val` bit array field's size */
		at += 32;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;

		/* Align for `_data__val` field */
		_ALIGN(at, 8);

		/* Add `_data__val` bit array field's size */
		at += 64;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipDrvPointerGetAttributesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventCreateBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;

		/* Align for `_event__val` field */
		_ALIGN(at, 8);

		/* Add `_event__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventCreateEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventCreateWithFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;

		/* Align for `_event__val` field */
		_ALIGN(at, 8);

		/* Add `_event__val` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventCreateWithFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventDestroyBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventDestroyEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventElapsedTimeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ms` field */
		_ALIGN(at, 8);

		/* Add `_ms` bit array field's size */
		at += 64;

		/* Align for `_ms__val` field */
		_ALIGN(at, 8);

		/* Add `_ms__val` bit array field's size */
		at += 32;

		/* Align for `_start` field */
		_ALIGN(at, 8);

		/* Add `_start` bit array field's size */
		at += 64;

		/* Align for `_stop` field */
		_ALIGN(at, 8);

		/* Add `_stop` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventElapsedTimeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventQueryBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventQueryEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventRecordBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventRecordEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventSynchronizeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipEventSynchronizeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtGetLastErrorBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtGetLastErrorEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtGetLinkTypeAndHopCountBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_device1` field */
		_ALIGN(at, 8);

		/* Add `_device1` bit array field's size */
		at += 32;

		/* Align for `_device2` field */
		_ALIGN(at, 8);

		/* Add `_device2` bit array field's size */
		at += 32;

		/* Align for `_linktype` field */
		_ALIGN(at, 8);

		/* Add `_linktype` bit array field's size */
		at += 64;

		/* Align for `_linktype__val` field */
		_ALIGN(at, 8);

		/* Add `_linktype__val` bit array field's size */
		at += 32;

		/* Align for `_hopcount` field */
		_ALIGN(at, 8);

		/* Add `_hopcount` bit array field's size */
		at += 64;

		/* Align for `_hopcount__val` field */
		_ALIGN(at, 8);

		/* Add `_hopcount__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtGetLinkTypeAndHopCountEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtLaunchKernelBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_function_address` field */
		_ALIGN(at, 8);

		/* Add `_function_address` bit array field's size */
		at += 64;

		/* Align for `_numBlocks__x` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks__x` bit array field's size */
		at += 32;

		/* Align for `_numBlocks__y` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks__y` bit array field's size */
		at += 32;

		/* Align for `_numBlocks__z` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks__z` bit array field's size */
		at += 32;

		/* Align for `_dimBlocks__x` field */
		_ALIGN(at, 8);

		/* Add `_dimBlocks__x` bit array field's size */
		at += 32;

		/* Align for `_dimBlocks__y` field */
		_ALIGN(at, 8);

		/* Add `_dimBlocks__y` bit array field's size */
		at += 32;

		/* Align for `_dimBlocks__z` field */
		_ALIGN(at, 8);

		/* Add `_dimBlocks__z` bit array field's size */
		at += 32;

		/* Align for `_args` field */
		_ALIGN(at, 8);

		/* Add `_args` bit array field's size */
		at += 64;

		/* Align for `_args__val` field */
		_ALIGN(at, 8);

		/* Add `_args__val` bit array field's size */
		at += 64;

		/* Align for `_sharedMemBytes` field */
		_ALIGN(at, 8);

		/* Add `_sharedMemBytes` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_startEvent` field */
		_ALIGN(at, 8);

		/* Add `_startEvent` bit array field's size */
		at += 64;

		/* Align for `_stopEvent` field */
		_ALIGN(at, 8);

		/* Add `_stopEvent` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtLaunchKernelEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtLaunchMultiKernelMultiDeviceBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_launchParamsList` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList` bit array field's size */
		at += 64;

		/* Align for `_launchParamsList__val__func` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__func` bit array field's size */
		at += 64;

		/* Align for `_launchParamsList__val__gridDim__x` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__gridDim__x` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__gridDim__y` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__gridDim__y` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__gridDim__z` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__gridDim__z` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__blockDim__x` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__blockDim__x` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__blockDim__y` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__blockDim__y` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__blockDim__z` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__blockDim__z` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__args` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__args` bit array field's size */
		at += 64;

		/* Align for `_launchParamsList__val__sharedMem` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__sharedMem` bit array field's size */
		at += 64;

		/* Align for `_launchParamsList__val__stream` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__stream` bit array field's size */
		at += 64;

		/* Align for `_numDevices` field */
		_ALIGN(at, 8);

		/* Add `_numDevices` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtLaunchMultiKernelMultiDeviceEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtMallocWithFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_ptr__val` field */
		_ALIGN(at, 8);

		/* Add `_ptr__val` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtMallocWithFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtModuleLaunchKernelBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_f` field */
		_ALIGN(at, 8);

		/* Add `_f` bit array field's size */
		at += 64;

		/* Align for `_globalWorkSizeX` field */
		_ALIGN(at, 8);

		/* Add `_globalWorkSizeX` bit array field's size */
		at += 32;

		/* Align for `_globalWorkSizeY` field */
		_ALIGN(at, 8);

		/* Add `_globalWorkSizeY` bit array field's size */
		at += 32;

		/* Align for `_globalWorkSizeZ` field */
		_ALIGN(at, 8);

		/* Add `_globalWorkSizeZ` bit array field's size */
		at += 32;

		/* Align for `_localWorkSizeX` field */
		_ALIGN(at, 8);

		/* Add `_localWorkSizeX` bit array field's size */
		at += 32;

		/* Align for `_localWorkSizeY` field */
		_ALIGN(at, 8);

		/* Add `_localWorkSizeY` bit array field's size */
		at += 32;

		/* Align for `_localWorkSizeZ` field */
		_ALIGN(at, 8);

		/* Add `_localWorkSizeZ` bit array field's size */
		at += 32;

		/* Align for `_sharedMemBytes` field */
		_ALIGN(at, 8);

		/* Add `_sharedMemBytes` bit array field's size */
		at += 64;

		/* Align for `_hStream` field */
		_ALIGN(at, 8);

		/* Add `_hStream` bit array field's size */
		at += 64;

		/* Align for `_kernelParams` field */
		_ALIGN(at, 8);

		/* Add `_kernelParams` bit array field's size */
		at += 64;

		/* Align for `_kernelParams__val` field */
		_ALIGN(at, 8);

		/* Add `_kernelParams__val` bit array field's size */
		at += 64;

		/* Align for `_extra` field */
		_ALIGN(at, 8);

		/* Add `_extra` bit array field's size */
		at += 64;

		/* Align for `_extra__val` field */
		_ALIGN(at, 8);

		/* Add `_extra__val` bit array field's size */
		at += 64;

		/* Align for `_startEvent` field */
		_ALIGN(at, 8);

		/* Add `_startEvent` bit array field's size */
		at += 64;

		/* Align for `_stopEvent` field */
		_ALIGN(at, 8);

		/* Add `_stopEvent` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtModuleLaunchKernelEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtStreamCreateWithCUMaskBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_stream__val` field */
		_ALIGN(at, 8);

		/* Add `_stream__val` bit array field's size */
		at += 64;

		/* Align for `_cuMaskSize` field */
		_ALIGN(at, 8);

		/* Add `_cuMaskSize` bit array field's size */
		at += 32;

		/* Align for `_cuMask` field */
		_ALIGN(at, 8);

		/* Add `_cuMask` bit array field's size */
		at += 64;

		/* Align for `_cuMask__val` field */
		_ALIGN(at, 8);

		/* Add `_cuMask__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtStreamCreateWithCUMaskEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtStreamGetCUMaskBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_cuMaskSize` field */
		_ALIGN(at, 8);

		/* Add `_cuMaskSize` bit array field's size */
		at += 32;

		/* Align for `_cuMask` field */
		_ALIGN(at, 8);

		/* Add `_cuMask` bit array field's size */
		at += 64;

		/* Align for `_cuMask__val` field */
		_ALIGN(at, 8);

		/* Add `_cuMask__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExtStreamGetCUMaskEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExternalMemoryGetMappedBufferBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(at, 8);

		/* Add `_devPtr` bit array field's size */
		at += 64;

		/* Align for `_devPtr__val` field */
		_ALIGN(at, 8);

		/* Add `_devPtr__val` bit array field's size */
		at += 64;

		/* Align for `_extMem` field */
		_ALIGN(at, 8);

		/* Add `_extMem` bit array field's size */
		at += 64;

		/* Align for `_bufferDesc` field */
		_ALIGN(at, 8);

		/* Add `_bufferDesc` bit array field's size */
		at += 64;

		/* Align for `_bufferDesc__val__offset` field */
		_ALIGN(at, 8);

		/* Add `_bufferDesc__val__offset` bit array field's size */
		at += 64;

		/* Align for `_bufferDesc__val__size` field */
		_ALIGN(at, 8);

		/* Add `_bufferDesc__val__size` bit array field's size */
		at += 64;

		/* Align for `_bufferDesc__val__flags` field */
		_ALIGN(at, 8);

		/* Add `_bufferDesc__val__flags` bit array field's size */
		at += 32;

		/* Align for `_bufferDesc__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_bufferDesc__val__reserved` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExternalMemoryGetMappedBufferEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExternalMemoryGetMappedMipmappedArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mipmap` field */
		_ALIGN(at, 8);

		/* Add `_mipmap` bit array field's size */
		at += 64;

		/* Align for `_mipmap__val` field */
		_ALIGN(at, 8);

		/* Add `_mipmap__val` bit array field's size */
		at += 64;

		/* Align for `_extMem` field */
		_ALIGN(at, 8);

		/* Add `_extMem` bit array field's size */
		at += 64;

		/* Align for `_mipmapDesc` field */
		_ALIGN(at, 8);

		/* Add `_mipmapDesc` bit array field's size */
		at += 64;

		/* Align for `_mipmapDesc__val__offset` field */
		_ALIGN(at, 8);

		/* Add `_mipmapDesc__val__offset` bit array field's size */
		at += 64;

		/* Align for `_mipmapDesc__val__formatDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_mipmapDesc__val__formatDesc__x` bit array field's size */
		at += 32;

		/* Align for `_mipmapDesc__val__formatDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_mipmapDesc__val__formatDesc__y` bit array field's size */
		at += 32;

		/* Align for `_mipmapDesc__val__formatDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_mipmapDesc__val__formatDesc__z` bit array field's size */
		at += 32;

		/* Align for `_mipmapDesc__val__formatDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_mipmapDesc__val__formatDesc__w` bit array field's size */
		at += 32;

		/* Align for `_mipmapDesc__val__formatDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_mipmapDesc__val__formatDesc__f` bit array field's size */
		at += 32;

		/* Align for `_mipmapDesc__val__extent__width` field */
		_ALIGN(at, 8);

		/* Add `_mipmapDesc__val__extent__width` bit array field's size */
		at += 64;

		/* Align for `_mipmapDesc__val__extent__height` field */
		_ALIGN(at, 8);

		/* Add `_mipmapDesc__val__extent__height` bit array field's size */
		at += 64;

		/* Align for `_mipmapDesc__val__extent__depth` field */
		_ALIGN(at, 8);

		/* Add `_mipmapDesc__val__extent__depth` bit array field's size */
		at += 64;

		/* Align for `_mipmapDesc__val__flags` field */
		_ALIGN(at, 8);

		/* Add `_mipmapDesc__val__flags` bit array field's size */
		at += 32;

		/* Align for `_mipmapDesc__val__numLevels` field */
		_ALIGN(at, 8);

		/* Add `_mipmapDesc__val__numLevels` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipExternalMemoryGetMappedMipmappedArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFreeArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_array` field */
		_ALIGN(at, 8);

		/* Add `_array` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFreeArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFreeAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFreeAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFreeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFreeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFreeHostBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFreeHostEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFreeMipmappedArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mipmappedArray` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFreeMipmappedArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFuncGetAttributeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;

		/* Align for `_value__val` field */
		_ALIGN(at, 8);

		/* Add `_value__val` bit array field's size */
		at += 32;

		/* Align for `_attrib` field */
		_ALIGN(at, 8);

		/* Add `_attrib` bit array field's size */
		at += 32;

		/* Align for `_hfunc` field */
		_ALIGN(at, 8);

		/* Add `_hfunc` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFuncGetAttributeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFuncGetAttributesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_attr` field */
		_ALIGN(at, 8);

		/* Add `_attr` bit array field's size */
		at += 64;

		/* Align for `_attr__val__binaryVersion` field */
		_ALIGN(at, 8);

		/* Add `_attr__val__binaryVersion` bit array field's size */
		at += 32;

		/* Align for `_attr__val__cacheModeCA` field */
		_ALIGN(at, 8);

		/* Add `_attr__val__cacheModeCA` bit array field's size */
		at += 32;

		/* Align for `_attr__val__constSizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_attr__val__constSizeBytes` bit array field's size */
		at += 64;

		/* Align for `_attr__val__localSizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_attr__val__localSizeBytes` bit array field's size */
		at += 64;

		/* Align for `_attr__val__maxDynamicSharedSizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_attr__val__maxDynamicSharedSizeBytes` bit array field's size */
		at += 32;

		/* Align for `_attr__val__maxThreadsPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_attr__val__maxThreadsPerBlock` bit array field's size */
		at += 32;

		/* Align for `_attr__val__numRegs` field */
		_ALIGN(at, 8);

		/* Add `_attr__val__numRegs` bit array field's size */
		at += 32;

		/* Align for `_attr__val__preferredShmemCarveout` field */
		_ALIGN(at, 8);

		/* Add `_attr__val__preferredShmemCarveout` bit array field's size */
		at += 32;

		/* Align for `_attr__val__ptxVersion` field */
		_ALIGN(at, 8);

		/* Add `_attr__val__ptxVersion` bit array field's size */
		at += 32;

		/* Align for `_attr__val__sharedSizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_attr__val__sharedSizeBytes` bit array field's size */
		at += 64;

		/* Align for `_func` field */
		_ALIGN(at, 8);

		/* Add `_func` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFuncGetAttributesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFuncSetAttributeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_func` field */
		_ALIGN(at, 8);

		/* Add `_func` bit array field's size */
		at += 64;

		/* Align for `_attr` field */
		_ALIGN(at, 8);

		/* Add `_attr` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFuncSetAttributeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFuncSetCacheConfigBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_func` field */
		_ALIGN(at, 8);

		/* Add `_func` bit array field's size */
		at += 64;

		/* Align for `_config` field */
		_ALIGN(at, 8);

		/* Add `_config` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFuncSetCacheConfigEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFuncSetSharedMemConfigBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_func` field */
		_ALIGN(at, 8);

		/* Add `_func` bit array field's size */
		at += 64;

		/* Align for `_config` field */
		_ALIGN(at, 8);

		/* Add `_config` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipFuncSetSharedMemConfigEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGLGetDevicesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pHipDeviceCount` field */
		_ALIGN(at, 8);

		/* Add `_pHipDeviceCount` bit array field's size */
		at += 64;

		/* Align for `_pHipDeviceCount__val` field */
		_ALIGN(at, 8);

		/* Add `_pHipDeviceCount__val` bit array field's size */
		at += 32;

		/* Align for `_pHipDevices` field */
		_ALIGN(at, 8);

		/* Add `_pHipDevices` bit array field's size */
		at += 64;

		/* Align for `_pHipDevices__val` field */
		_ALIGN(at, 8);

		/* Add `_pHipDevices__val` bit array field's size */
		at += 32;

		/* Align for `_hipDeviceCount` field */
		_ALIGN(at, 8);

		/* Add `_hipDeviceCount` bit array field's size */
		at += 32;

		/* Align for `_deviceList` field */
		_ALIGN(at, 8);

		/* Add `_deviceList` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGLGetDevicesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetChannelDescBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_desc` field */
		_ALIGN(at, 8);

		/* Add `_desc` bit array field's size */
		at += 64;

		/* Align for `_desc__val__x` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__x` bit array field's size */
		at += 32;

		/* Align for `_desc__val__y` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__y` bit array field's size */
		at += 32;

		/* Align for `_desc__val__z` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__z` bit array field's size */
		at += 32;

		/* Align for `_desc__val__w` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__w` bit array field's size */
		at += 32;

		/* Align for `_desc__val__f` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__f` bit array field's size */
		at += 32;

		/* Align for `_array` field */
		_ALIGN(at, 8);

		/* Add `_array` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetChannelDescEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetDeviceBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_deviceId` field */
		_ALIGN(at, 8);

		/* Add `_deviceId` bit array field's size */
		at += 64;

		/* Align for `_deviceId__val` field */
		_ALIGN(at, 8);

		/* Add `_deviceId__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetDeviceCountBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_count__val` field */
		_ALIGN(at, 8);

		/* Add `_count__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetDeviceCountEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetDeviceEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetDeviceFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;

		/* Align for `_flags__val` field */
		_ALIGN(at, 8);

		/* Add `_flags__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetDeviceFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetDevicePropertiesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetDevicePropertiesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetDevicePropertiesR0000Begin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_prop` field */
		_ALIGN(at, 8);

		/* Add `_prop` bit array field's size */
		at += 64;

		/* Align for `_prop__val__name` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__name` bit array field's size */
		at += 64;

		/* Align for `_prop__val__totalGlobalMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__totalGlobalMem` bit array field's size */
		at += 64;

		/* Align for `_prop__val__sharedMemPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__sharedMemPerBlock` bit array field's size */
		at += 64;

		/* Align for `_prop__val__regsPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__regsPerBlock` bit array field's size */
		at += 32;

		/* Align for `_prop__val__warpSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__warpSize` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxThreadsPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxThreadsPerBlock` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxThreadsDim` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxThreadsDim` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxGridSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxGridSize` bit array field's size */
		at += 64;

		/* Align for `_prop__val__clockRate` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__clockRate` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memoryClockRate` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memoryClockRate` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memoryBusWidth` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memoryBusWidth` bit array field's size */
		at += 32;

		/* Align for `_prop__val__totalConstMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__totalConstMem` bit array field's size */
		at += 64;

		/* Align for `_prop__val__major` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__major` bit array field's size */
		at += 32;

		/* Align for `_prop__val__minor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__minor` bit array field's size */
		at += 32;

		/* Align for `_prop__val__multiProcessorCount` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__multiProcessorCount` bit array field's size */
		at += 32;

		/* Align for `_prop__val__l2CacheSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__l2CacheSize` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxThreadsPerMultiProcessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxThreadsPerMultiProcessor` bit array field's size */
		at += 32;

		/* Align for `_prop__val__computeMode` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__computeMode` bit array field's size */
		at += 32;

		/* Align for `_prop__val__clockInstructionRate` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__clockInstructionRate` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasGlobalInt32Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasGlobalInt32Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasGlobalFloatAtomicExch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasGlobalFloatAtomicExch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSharedInt32Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSharedInt32Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSharedFloatAtomicExch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSharedFloatAtomicExch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasFloatAtomicAdd` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasFloatAtomicAdd` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasGlobalInt64Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasGlobalInt64Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSharedInt64Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSharedInt64Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasDoubles` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasDoubles` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasWarpVote` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasWarpVote` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasWarpBallot` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasWarpBallot` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasWarpShuffle` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasWarpShuffle` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasFunnelShift` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasFunnelShift` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasThreadFenceSystem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasThreadFenceSystem` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSyncThreadsExt` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSyncThreadsExt` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSurfaceFuncs` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSurfaceFuncs` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__has3dGrid` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__has3dGrid` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasDynamicParallelism` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasDynamicParallelism` bit array field's size */
		at += 32;

		/* Align for `_prop__val__concurrentKernels` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__concurrentKernels` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pciDomainID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pciDomainID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pciBusID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pciBusID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pciDeviceID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pciDeviceID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxSharedMemoryPerMultiProcessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSharedMemoryPerMultiProcessor` bit array field's size */
		at += 64;

		/* Align for `_prop__val__isMultiGpuBoard` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__isMultiGpuBoard` bit array field's size */
		at += 32;

		/* Align for `_prop__val__canMapHostMemory` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__canMapHostMemory` bit array field's size */
		at += 32;

		/* Align for `_prop__val__gcnArch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__gcnArch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__gcnArchName` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__gcnArchName` bit array field's size */
		at += 64;

		/* Align for `_prop__val__integrated` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__integrated` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeLaunch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeLaunch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceLaunch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceLaunch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture1DLinear` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture1DLinear` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture1D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture1D` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture2D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture2D` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture3D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture3D` bit array field's size */
		at += 64;

		/* Align for `_prop__val__hdpMemFlushCntl` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hdpMemFlushCntl` bit array field's size */
		at += 64;

		/* Align for `_prop__val__hdpRegFlushCntl` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hdpRegFlushCntl` bit array field's size */
		at += 64;

		/* Align for `_prop__val__memPitch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memPitch` bit array field's size */
		at += 64;

		/* Align for `_prop__val__textureAlignment` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__textureAlignment` bit array field's size */
		at += 64;

		/* Align for `_prop__val__texturePitchAlignment` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__texturePitchAlignment` bit array field's size */
		at += 64;

		/* Align for `_prop__val__kernelExecTimeoutEnabled` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__kernelExecTimeoutEnabled` bit array field's size */
		at += 32;

		/* Align for `_prop__val__ECCEnabled` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__ECCEnabled` bit array field's size */
		at += 32;

		/* Align for `_prop__val__tccDriver` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__tccDriver` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedFunc` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedFunc` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` bit array field's size */
		at += 32;

		/* Align for `_prop__val__isLargeBar` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__isLargeBar` bit array field's size */
		at += 32;

		/* Align for `_prop__val__asicRevision` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__asicRevision` bit array field's size */
		at += 32;

		/* Align for `_prop__val__managedMemory` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__managedMemory` bit array field's size */
		at += 32;

		/* Align for `_prop__val__directManagedMemAccessFromHost` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__directManagedMemAccessFromHost` bit array field's size */
		at += 32;

		/* Align for `_prop__val__concurrentManagedAccess` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__concurrentManagedAccess` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pageableMemoryAccess` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pageableMemoryAccess` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pageableMemoryAccessUsesHostPageTables` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pageableMemoryAccessUsesHostPageTables` bit array field's size */
		at += 32;

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetDevicePropertiesR0000End(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetDevicePropertiesR0600Begin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_prop` field */
		_ALIGN(at, 8);

		/* Add `_prop` bit array field's size */
		at += 64;

		/* Align for `_prop__val__name` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__name` bit array field's size */
		at += 64;

		/* Align for `_prop__val__uuid__bytes` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__uuid__bytes` bit array field's size */
		at += 64;

		/* Align for `_prop__val__luid` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__luid` bit array field's size */
		at += 64;

		/* Align for `_prop__val__luidDeviceNodeMask` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__luidDeviceNodeMask` bit array field's size */
		at += 32;

		/* Align for `_prop__val__totalGlobalMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__totalGlobalMem` bit array field's size */
		at += 64;

		/* Align for `_prop__val__sharedMemPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__sharedMemPerBlock` bit array field's size */
		at += 64;

		/* Align for `_prop__val__regsPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__regsPerBlock` bit array field's size */
		at += 32;

		/* Align for `_prop__val__warpSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__warpSize` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memPitch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memPitch` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxThreadsPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxThreadsPerBlock` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxThreadsDim` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxThreadsDim` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxGridSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxGridSize` bit array field's size */
		at += 64;

		/* Align for `_prop__val__clockRate` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__clockRate` bit array field's size */
		at += 32;

		/* Align for `_prop__val__totalConstMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__totalConstMem` bit array field's size */
		at += 64;

		/* Align for `_prop__val__major` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__major` bit array field's size */
		at += 32;

		/* Align for `_prop__val__minor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__minor` bit array field's size */
		at += 32;

		/* Align for `_prop__val__textureAlignment` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__textureAlignment` bit array field's size */
		at += 64;

		/* Align for `_prop__val__texturePitchAlignment` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__texturePitchAlignment` bit array field's size */
		at += 64;

		/* Align for `_prop__val__deviceOverlap` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__deviceOverlap` bit array field's size */
		at += 32;

		/* Align for `_prop__val__multiProcessorCount` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__multiProcessorCount` bit array field's size */
		at += 32;

		/* Align for `_prop__val__kernelExecTimeoutEnabled` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__kernelExecTimeoutEnabled` bit array field's size */
		at += 32;

		/* Align for `_prop__val__integrated` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__integrated` bit array field's size */
		at += 32;

		/* Align for `_prop__val__canMapHostMemory` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__canMapHostMemory` bit array field's size */
		at += 32;

		/* Align for `_prop__val__computeMode` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__computeMode` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture1D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture1D` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture1DMipmap` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture1DMipmap` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture1DLinear` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture1DLinear` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture2D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture2D` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture2DMipmap` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture2DMipmap` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture2DLinear` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture2DLinear` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture2DGather` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture2DGather` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture3D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture3D` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture3DAlt` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture3DAlt` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTextureCubemap` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTextureCubemap` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxTexture1DLayered` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture1DLayered` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTexture2DLayered` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTexture2DLayered` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxTextureCubemapLayered` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxTextureCubemapLayered` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxSurface1D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurface1D` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxSurface2D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurface2D` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxSurface3D` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurface3D` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxSurface1DLayered` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurface1DLayered` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxSurface2DLayered` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurface2DLayered` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxSurfaceCubemap` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurfaceCubemap` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxSurfaceCubemapLayered` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSurfaceCubemapLayered` bit array field's size */
		at += 64;

		/* Align for `_prop__val__surfaceAlignment` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__surfaceAlignment` bit array field's size */
		at += 64;

		/* Align for `_prop__val__concurrentKernels` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__concurrentKernels` bit array field's size */
		at += 32;

		/* Align for `_prop__val__ECCEnabled` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__ECCEnabled` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pciBusID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pciBusID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pciDeviceID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pciDeviceID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pciDomainID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pciDomainID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__tccDriver` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__tccDriver` bit array field's size */
		at += 32;

		/* Align for `_prop__val__asyncEngineCount` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__asyncEngineCount` bit array field's size */
		at += 32;

		/* Align for `_prop__val__unifiedAddressing` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__unifiedAddressing` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memoryClockRate` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memoryClockRate` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memoryBusWidth` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memoryBusWidth` bit array field's size */
		at += 32;

		/* Align for `_prop__val__l2CacheSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__l2CacheSize` bit array field's size */
		at += 32;

		/* Align for `_prop__val__persistingL2CacheMaxSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__persistingL2CacheMaxSize` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxThreadsPerMultiProcessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxThreadsPerMultiProcessor` bit array field's size */
		at += 32;

		/* Align for `_prop__val__streamPrioritiesSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__streamPrioritiesSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__globalL1CacheSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__globalL1CacheSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__localL1CacheSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__localL1CacheSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__sharedMemPerMultiprocessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__sharedMemPerMultiprocessor` bit array field's size */
		at += 64;

		/* Align for `_prop__val__regsPerMultiprocessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__regsPerMultiprocessor` bit array field's size */
		at += 32;

		/* Align for `_prop__val__managedMemory` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__managedMemory` bit array field's size */
		at += 32;

		/* Align for `_prop__val__isMultiGpuBoard` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__isMultiGpuBoard` bit array field's size */
		at += 32;

		/* Align for `_prop__val__multiGpuBoardGroupID` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__multiGpuBoardGroupID` bit array field's size */
		at += 32;

		/* Align for `_prop__val__hostNativeAtomicSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hostNativeAtomicSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__singleToDoublePrecisionPerfRatio` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__singleToDoublePrecisionPerfRatio` bit array field's size */
		at += 32;

		/* Align for `_prop__val__pageableMemoryAccess` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pageableMemoryAccess` bit array field's size */
		at += 32;

		/* Align for `_prop__val__concurrentManagedAccess` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__concurrentManagedAccess` bit array field's size */
		at += 32;

		/* Align for `_prop__val__computePreemptionSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__computePreemptionSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__canUseHostPointerForRegisteredMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__canUseHostPointerForRegisteredMem` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeLaunch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeLaunch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceLaunch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceLaunch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__sharedMemPerBlockOptin` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__sharedMemPerBlockOptin` bit array field's size */
		at += 64;

		/* Align for `_prop__val__pageableMemoryAccessUsesHostPageTables` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__pageableMemoryAccessUsesHostPageTables` bit array field's size */
		at += 32;

		/* Align for `_prop__val__directManagedMemAccessFromHost` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__directManagedMemAccessFromHost` bit array field's size */
		at += 32;

		/* Align for `_prop__val__maxBlocksPerMultiProcessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxBlocksPerMultiProcessor` bit array field's size */
		at += 32;

		/* Align for `_prop__val__accessPolicyMaxWindowSize` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__accessPolicyMaxWindowSize` bit array field's size */
		at += 32;

		/* Align for `_prop__val__reservedSharedMemPerBlock` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__reservedSharedMemPerBlock` bit array field's size */
		at += 64;

		/* Align for `_prop__val__hostRegisterSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hostRegisterSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__sparseHipArraySupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__sparseHipArraySupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__hostRegisterReadOnlySupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hostRegisterReadOnlySupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__timelineSemaphoreInteropSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__timelineSemaphoreInteropSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memoryPoolsSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memoryPoolsSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__gpuDirectRDMASupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__gpuDirectRDMASupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__gpuDirectRDMAFlushWritesOptions` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__gpuDirectRDMAFlushWritesOptions` bit array field's size */
		at += 32;

		/* Align for `_prop__val__gpuDirectRDMAWritesOrdering` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__gpuDirectRDMAWritesOrdering` bit array field's size */
		at += 32;

		/* Align for `_prop__val__memoryPoolSupportedHandleTypes` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__memoryPoolSupportedHandleTypes` bit array field's size */
		at += 32;

		/* Align for `_prop__val__deferredMappingHipArraySupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__deferredMappingHipArraySupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__ipcEventSupported` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__ipcEventSupported` bit array field's size */
		at += 32;

		/* Align for `_prop__val__clusterLaunch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__clusterLaunch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__unifiedFunctionPointers` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__unifiedFunctionPointers` bit array field's size */
		at += 32;

		/* Align for `_prop__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__reserved` bit array field's size */
		at += 64;

		/* Align for `_prop__val__hipReserved` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hipReserved` bit array field's size */
		at += 64;

		/* Align for `_prop__val__gcnArchName` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__gcnArchName` bit array field's size */
		at += 64;

		/* Align for `_prop__val__maxSharedMemoryPerMultiProcessor` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__maxSharedMemoryPerMultiProcessor` bit array field's size */
		at += 64;

		/* Align for `_prop__val__clockInstructionRate` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__clockInstructionRate` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasGlobalInt32Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasGlobalInt32Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasGlobalFloatAtomicExch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasGlobalFloatAtomicExch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSharedInt32Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSharedInt32Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSharedFloatAtomicExch` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSharedFloatAtomicExch` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasFloatAtomicAdd` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasFloatAtomicAdd` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasGlobalInt64Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasGlobalInt64Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSharedInt64Atomics` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSharedInt64Atomics` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasDoubles` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasDoubles` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasWarpVote` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasWarpVote` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasWarpBallot` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasWarpBallot` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasWarpShuffle` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasWarpShuffle` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasFunnelShift` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasFunnelShift` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasThreadFenceSystem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasThreadFenceSystem` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSyncThreadsExt` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSyncThreadsExt` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasSurfaceFuncs` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasSurfaceFuncs` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__has3dGrid` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__has3dGrid` bit array field's size */
		at += 32;

		/* Align for `_prop__val__arch__hasDynamicParallelism` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__arch__hasDynamicParallelism` bit array field's size */
		at += 32;

		/* Align for `_prop__val__hdpMemFlushCntl` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hdpMemFlushCntl` bit array field's size */
		at += 64;

		/* Align for `_prop__val__hdpRegFlushCntl` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__hdpRegFlushCntl` bit array field's size */
		at += 64;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedFunc` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedFunc` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedGridDim` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedBlockDim` bit array field's size */
		at += 32;

		/* Align for `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__cooperativeMultiDeviceUnmatchedSharedMem` bit array field's size */
		at += 32;

		/* Align for `_prop__val__isLargeBar` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__isLargeBar` bit array field's size */
		at += 32;

		/* Align for `_prop__val__asicRevision` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__asicRevision` bit array field's size */
		at += 32;

		/* Align for `_deviceId` field */
		_ALIGN(at, 8);

		/* Add `_deviceId` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetDevicePropertiesR0600End(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetErrorStringBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetErrorStringEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetFuncBySymbolBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_functionPtr` field */
		_ALIGN(at, 8);

		/* Add `_functionPtr` bit array field's size */
		at += 64;

		/* Align for `_functionPtr__val` field */
		_ALIGN(at, 8);

		/* Add `_functionPtr__val` bit array field's size */
		at += 64;

		/* Align for `_symbolPtr` field */
		_ALIGN(at, 8);

		/* Add `_symbolPtr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetFuncBySymbolEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetLastErrorBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetLastErrorEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetMipmappedArrayLevelBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_levelArray` field */
		_ALIGN(at, 8);

		/* Add `_levelArray` bit array field's size */
		at += 64;

		/* Align for `_levelArray__val` field */
		_ALIGN(at, 8);

		/* Add `_levelArray__val` bit array field's size */
		at += 64;

		/* Align for `_mipmappedArray` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray` bit array field's size */
		at += 64;

		/* Align for `_level` field */
		_ALIGN(at, 8);

		/* Add `_level` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetMipmappedArrayLevelEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetProcAddressBegin(void * const vctx,
	const char * const cc__kernel_name,
	const char * const p__symbol)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` string field's size */
		at += _BYTES_TO_BITS(strlen(p__symbol) + 1);

		/* Align for `_symbol__val` field */
		_ALIGN(at, 8);

		/* Add `_symbol__val` bit array field's size */
		at += 8;

		/* Align for `_pfn` field */
		_ALIGN(at, 8);

		/* Add `_pfn` bit array field's size */
		at += 64;

		/* Align for `_pfn__val` field */
		_ALIGN(at, 8);

		/* Add `_pfn__val` bit array field's size */
		at += 64;

		/* Align for `_hipVersion` field */
		_ALIGN(at, 8);

		/* Add `_hipVersion` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;

		/* Align for `_symbolStatus` field */
		_ALIGN(at, 8);

		/* Add `_symbolStatus` bit array field's size */
		at += 64;

		/* Align for `_symbolStatus__val` field */
		_ALIGN(at, 8);

		/* Add `_symbolStatus__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetProcAddressEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetSymbolAddressBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(at, 8);

		/* Add `_devPtr` bit array field's size */
		at += 64;

		/* Align for `_devPtr__val` field */
		_ALIGN(at, 8);

		/* Add `_devPtr__val` bit array field's size */
		at += 64;

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetSymbolAddressEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetSymbolSizeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_size__val` field */
		_ALIGN(at, 8);

		/* Add `_size__val` bit array field's size */
		at += 64;

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGetSymbolSizeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddChildGraphNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_childGraph` field */
		_ALIGN(at, 8);

		/* Add `_childGraph` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddChildGraphNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddDependenciesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_from` field */
		_ALIGN(at, 8);

		/* Add `_from` bit array field's size */
		at += 64;

		/* Align for `_from__val` field */
		_ALIGN(at, 8);

		/* Add `_from__val` bit array field's size */
		at += 64;

		/* Align for `_to` field */
		_ALIGN(at, 8);

		/* Add `_to` bit array field's size */
		at += 64;

		/* Align for `_to__val` field */
		_ALIGN(at, 8);

		/* Add `_to__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddDependenciesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddEmptyNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddEmptyNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddEventRecordNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddEventRecordNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddEventWaitNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddEventWaitNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddExternalSemaphoresSignalNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_nodeParams` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__extSemArray` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__extSemArray` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__paramsArray` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__paramsArray` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__numExtSems` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__numExtSems` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddExternalSemaphoresSignalNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddExternalSemaphoresWaitNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_nodeParams` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__extSemArray` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__extSemArray` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__paramsArray` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__paramsArray` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__numExtSems` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__numExtSems` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddExternalSemaphoresWaitNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddHostNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__fn` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__fn` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__userData` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__userData` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddHostNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddKernelNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__blockDim__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__blockDim__x` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__blockDim__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__blockDim__y` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__blockDim__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__blockDim__z` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__extra` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__extra` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__func` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__func` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__gridDim__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__gridDim__x` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__gridDim__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__gridDim__y` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__gridDim__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__gridDim__z` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__kernelParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__kernelParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__sharedMemBytes` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__sharedMemBytes` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddKernelNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemAllocNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__poolProps__allocType` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__allocType` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__poolProps__handleTypes` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__handleTypes` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__poolProps__location__type` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__location__type` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__poolProps__location__id` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__location__id` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__poolProps__win32SecurityAttributes` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__win32SecurityAttributes` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__poolProps__maxSize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__maxSize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__poolProps__reserved` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__reserved` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__accessDescs` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__accessDescs` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__accessDescCount` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__accessDescCount` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__bytesize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__bytesize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dptr` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemAllocNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemFreeNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemFreeNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemcpyNode1DBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemcpyNode1DEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemcpyNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__srcArray` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__srcArray` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__srcPos__x` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__srcPos__x` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__srcPos__y` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__srcPos__y` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__srcPos__z` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__srcPos__z` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__srcPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__srcPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__srcPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__srcPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__srcPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__srcPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__srcPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__srcPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__dstArray` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__dstArray` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__dstPos__x` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__dstPos__x` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__dstPos__y` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__dstPos__y` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__dstPos__z` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__dstPos__z` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__dstPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__dstPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__dstPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__dstPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__dstPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__dstPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__dstPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__dstPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__extent__width` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__extent__width` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__extent__height` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__extent__height` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__extent__depth` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__extent__depth` bit array field's size */
		at += 64;

		/* Align for `_pCopyParams__val__kind` field */
		_ALIGN(at, 8);

		/* Add `_pCopyParams__val__kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemcpyNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemcpyNodeFromSymbolBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_offset` field */
		_ALIGN(at, 8);

		/* Add `_offset` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemcpyNodeFromSymbolEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemcpyNodeToSymbolBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_offset` field */
		_ALIGN(at, 8);

		/* Add `_offset` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemcpyNodeToSymbolEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemsetNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_pMemsetParams` field */
		_ALIGN(at, 8);

		/* Add `_pMemsetParams` bit array field's size */
		at += 64;

		/* Align for `_pMemsetParams__val__dst` field */
		_ALIGN(at, 8);

		/* Add `_pMemsetParams__val__dst` bit array field's size */
		at += 64;

		/* Align for `_pMemsetParams__val__elementSize` field */
		_ALIGN(at, 8);

		/* Add `_pMemsetParams__val__elementSize` bit array field's size */
		at += 32;

		/* Align for `_pMemsetParams__val__height` field */
		_ALIGN(at, 8);

		/* Add `_pMemsetParams__val__height` bit array field's size */
		at += 64;

		/* Align for `_pMemsetParams__val__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pMemsetParams__val__pitch` bit array field's size */
		at += 64;

		/* Align for `_pMemsetParams__val__value` field */
		_ALIGN(at, 8);

		/* Add `_pMemsetParams__val__value` bit array field's size */
		at += 32;

		/* Align for `_pMemsetParams__val__width` field */
		_ALIGN(at, 8);

		/* Add `_pMemsetParams__val__width` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddMemsetNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphNode` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode` bit array field's size */
		at += 64;

		/* Align for `_pGraphNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphNode__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_nodeParams` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__type` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__type` bit array field's size */
		at += 32;

		/* Align for `_nodeParams__val__reserved0` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__reserved0` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__reserved2` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__reserved2` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphAddNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphChildGraphNodeGetGraphBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pGraph` field */
		_ALIGN(at, 8);

		/* Add `_pGraph` bit array field's size */
		at += 64;

		/* Align for `_pGraph__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraph__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphChildGraphNodeGetGraphEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphCloneBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphClone` field */
		_ALIGN(at, 8);

		/* Add `_pGraphClone` bit array field's size */
		at += 64;

		/* Align for `_pGraphClone__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphClone__val` bit array field's size */
		at += 64;

		/* Align for `_originalGraph` field */
		_ALIGN(at, 8);

		/* Add `_originalGraph` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphCloneEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphCreateBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraph` field */
		_ALIGN(at, 8);

		/* Add `_pGraph` bit array field's size */
		at += 64;

		/* Align for `_pGraph__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraph__val` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphCreateEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphDebugDotPrintBegin(void * const vctx,
	const char * const cc__kernel_name,
	const char * const p__path)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_path` field */
		_ALIGN(at, 8);

		/* Add `_path` string field's size */
		at += _BYTES_TO_BITS(strlen(p__path) + 1);

		/* Align for `_path__val` field */
		_ALIGN(at, 8);

		/* Add `_path__val` bit array field's size */
		at += 8;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphDebugDotPrintEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphDestroyBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphDestroyEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphDestroyNodeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphDestroyNodeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphEventRecordNodeGetEventBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_event_out` field */
		_ALIGN(at, 8);

		/* Add `_event_out` bit array field's size */
		at += 64;

		/* Align for `_event_out__val` field */
		_ALIGN(at, 8);

		/* Add `_event_out__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphEventRecordNodeGetEventEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphEventRecordNodeSetEventBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphEventRecordNodeSetEventEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphEventWaitNodeGetEventBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_event_out` field */
		_ALIGN(at, 8);

		/* Add `_event_out` bit array field's size */
		at += 64;

		/* Align for `_event_out__val` field */
		_ALIGN(at, 8);

		/* Add `_event_out__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphEventWaitNodeGetEventEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphEventWaitNodeSetEventBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphEventWaitNodeSetEventEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecChildGraphNodeSetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_childGraph` field */
		_ALIGN(at, 8);

		/* Add `_childGraph` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecChildGraphNodeSetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecDestroyBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_graphExec` field */
		_ALIGN(at, 8);

		/* Add `_graphExec` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecDestroyEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecEventRecordNodeSetEventBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_hNode` field */
		_ALIGN(at, 8);

		/* Add `_hNode` bit array field's size */
		at += 64;

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecEventRecordNodeSetEventEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecEventWaitNodeSetEventBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_hNode` field */
		_ALIGN(at, 8);

		/* Add `_hNode` bit array field's size */
		at += 64;

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecEventWaitNodeSetEventEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecExternalSemaphoresSignalNodeSetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_hNode` field */
		_ALIGN(at, 8);

		/* Add `_hNode` bit array field's size */
		at += 64;

		/* Align for `_nodeParams` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__extSemArray` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__extSemArray` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__paramsArray` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__paramsArray` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__numExtSems` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__numExtSems` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecExternalSemaphoresSignalNodeSetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecExternalSemaphoresWaitNodeSetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_hNode` field */
		_ALIGN(at, 8);

		/* Add `_hNode` bit array field's size */
		at += 64;

		/* Align for `_nodeParams` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__extSemArray` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__extSemArray` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__paramsArray` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__paramsArray` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__numExtSems` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__numExtSems` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecExternalSemaphoresWaitNodeSetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecHostNodeSetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__fn` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__fn` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__userData` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__userData` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecHostNodeSetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecKernelNodeSetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__blockDim__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__blockDim__x` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__blockDim__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__blockDim__y` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__blockDim__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__blockDim__z` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__extra` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__extra` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__func` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__func` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__gridDim__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__gridDim__x` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__gridDim__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__gridDim__y` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__gridDim__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__gridDim__z` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__kernelParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__kernelParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__sharedMemBytes` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__sharedMemBytes` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecKernelNodeSetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecMemcpyNodeSetParams1DBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecMemcpyNodeSetParams1DEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecMemcpyNodeSetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcArray` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcArray` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPos__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPos__x` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPos__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPos__y` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPos__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPos__z` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstArray` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstArray` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPos__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPos__x` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPos__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPos__y` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPos__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPos__z` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__extent__width` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__extent__width` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__extent__height` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__extent__height` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__extent__depth` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__extent__depth` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__kind` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecMemcpyNodeSetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecMemcpyNodeSetParamsFromSymbolBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_offset` field */
		_ALIGN(at, 8);

		/* Add `_offset` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecMemcpyNodeSetParamsFromSymbolEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecMemcpyNodeSetParamsToSymbolBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_offset` field */
		_ALIGN(at, 8);

		/* Add `_offset` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecMemcpyNodeSetParamsToSymbolEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecMemsetNodeSetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dst` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dst` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__elementSize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__elementSize` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__height` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__height` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__pitch` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__value` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__value` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__width` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__width` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecMemsetNodeSetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecUpdateBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_hGraph` field */
		_ALIGN(at, 8);

		/* Add `_hGraph` bit array field's size */
		at += 64;

		/* Align for `_hErrorNode_out` field */
		_ALIGN(at, 8);

		/* Add `_hErrorNode_out` bit array field's size */
		at += 64;

		/* Align for `_hErrorNode_out__val` field */
		_ALIGN(at, 8);

		/* Add `_hErrorNode_out__val` bit array field's size */
		at += 64;

		/* Align for `_updateResult_out` field */
		_ALIGN(at, 8);

		/* Add `_updateResult_out` bit array field's size */
		at += 64;

		/* Align for `_updateResult_out__val` field */
		_ALIGN(at, 8);

		/* Add `_updateResult_out__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExecUpdateEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExternalSemaphoresSignalNodeGetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hNode` field */
		_ALIGN(at, 8);

		/* Add `_hNode` bit array field's size */
		at += 64;

		/* Align for `_params_out` field */
		_ALIGN(at, 8);

		/* Add `_params_out` bit array field's size */
		at += 64;

		/* Align for `_params_out__val__extSemArray` field */
		_ALIGN(at, 8);

		/* Add `_params_out__val__extSemArray` bit array field's size */
		at += 64;

		/* Align for `_params_out__val__paramsArray` field */
		_ALIGN(at, 8);

		/* Add `_params_out__val__paramsArray` bit array field's size */
		at += 64;

		/* Align for `_params_out__val__numExtSems` field */
		_ALIGN(at, 8);

		/* Add `_params_out__val__numExtSems` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExternalSemaphoresSignalNodeGetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExternalSemaphoresSignalNodeSetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hNode` field */
		_ALIGN(at, 8);

		/* Add `_hNode` bit array field's size */
		at += 64;

		/* Align for `_nodeParams` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__extSemArray` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__extSemArray` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__paramsArray` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__paramsArray` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__numExtSems` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__numExtSems` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExternalSemaphoresSignalNodeSetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExternalSemaphoresWaitNodeGetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hNode` field */
		_ALIGN(at, 8);

		/* Add `_hNode` bit array field's size */
		at += 64;

		/* Align for `_params_out` field */
		_ALIGN(at, 8);

		/* Add `_params_out` bit array field's size */
		at += 64;

		/* Align for `_params_out__val__extSemArray` field */
		_ALIGN(at, 8);

		/* Add `_params_out__val__extSemArray` bit array field's size */
		at += 64;

		/* Align for `_params_out__val__paramsArray` field */
		_ALIGN(at, 8);

		/* Add `_params_out__val__paramsArray` bit array field's size */
		at += 64;

		/* Align for `_params_out__val__numExtSems` field */
		_ALIGN(at, 8);

		/* Add `_params_out__val__numExtSems` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExternalSemaphoresWaitNodeGetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExternalSemaphoresWaitNodeSetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hNode` field */
		_ALIGN(at, 8);

		/* Add `_hNode` bit array field's size */
		at += 64;

		/* Align for `_nodeParams` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__extSemArray` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__extSemArray` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__paramsArray` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__paramsArray` bit array field's size */
		at += 64;

		/* Align for `_nodeParams__val__numExtSems` field */
		_ALIGN(at, 8);

		/* Add `_nodeParams__val__numExtSems` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphExternalSemaphoresWaitNodeSetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphGetEdgesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_from` field */
		_ALIGN(at, 8);

		/* Add `_from` bit array field's size */
		at += 64;

		/* Align for `_from__val` field */
		_ALIGN(at, 8);

		/* Add `_from__val` bit array field's size */
		at += 64;

		/* Align for `_to` field */
		_ALIGN(at, 8);

		/* Add `_to` bit array field's size */
		at += 64;

		/* Align for `_to__val` field */
		_ALIGN(at, 8);

		/* Add `_to__val` bit array field's size */
		at += 64;

		/* Align for `_numEdges` field */
		_ALIGN(at, 8);

		/* Add `_numEdges` bit array field's size */
		at += 64;

		/* Align for `_numEdges__val` field */
		_ALIGN(at, 8);

		/* Add `_numEdges__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphGetEdgesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphGetNodesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_nodes` field */
		_ALIGN(at, 8);

		/* Add `_nodes` bit array field's size */
		at += 64;

		/* Align for `_nodes__val` field */
		_ALIGN(at, 8);

		/* Add `_nodes__val` bit array field's size */
		at += 64;

		/* Align for `_numNodes` field */
		_ALIGN(at, 8);

		/* Add `_numNodes` bit array field's size */
		at += 64;

		/* Align for `_numNodes__val` field */
		_ALIGN(at, 8);

		/* Add `_numNodes__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphGetNodesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphGetRootNodesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pRootNodes` field */
		_ALIGN(at, 8);

		/* Add `_pRootNodes` bit array field's size */
		at += 64;

		/* Align for `_pRootNodes__val` field */
		_ALIGN(at, 8);

		/* Add `_pRootNodes__val` bit array field's size */
		at += 64;

		/* Align for `_pNumRootNodes` field */
		_ALIGN(at, 8);

		/* Add `_pNumRootNodes` bit array field's size */
		at += 64;

		/* Align for `_pNumRootNodes__val` field */
		_ALIGN(at, 8);

		/* Add `_pNumRootNodes__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphGetRootNodesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphHostNodeGetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__fn` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__fn` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__userData` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__userData` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphHostNodeGetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphHostNodeSetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__fn` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__fn` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__userData` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__userData` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphHostNodeSetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphInstantiateBegin(void * const vctx,
	const char * const cc__kernel_name,
	const char * const p__pLogBuffer)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_pGraphExec` bit array field's size */
		at += 64;

		/* Align for `_pGraphExec__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphExec__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_pErrorNode` field */
		_ALIGN(at, 8);

		/* Add `_pErrorNode` bit array field's size */
		at += 64;

		/* Align for `_pErrorNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pErrorNode__val` bit array field's size */
		at += 64;

		/* Align for `_pLogBuffer` field */
		_ALIGN(at, 8);

		/* Add `_pLogBuffer` string field's size */
		at += _BYTES_TO_BITS(strlen(p__pLogBuffer) + 1);

		/* Align for `_pLogBuffer__val` field */
		_ALIGN(at, 8);

		/* Add `_pLogBuffer__val` bit array field's size */
		at += 8;

		/* Align for `_bufferSize` field */
		_ALIGN(at, 8);

		/* Add `_bufferSize` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphInstantiateEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphInstantiateWithFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_pGraphExec` bit array field's size */
		at += 64;

		/* Align for `_pGraphExec__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphExec__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphInstantiateWithFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphInstantiateWithParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_pGraphExec` bit array field's size */
		at += 64;

		/* Align for `_pGraphExec__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraphExec__val` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_instantiateParams` field */
		_ALIGN(at, 8);

		/* Add `_instantiateParams` bit array field's size */
		at += 64;

		/* Align for `_instantiateParams__val__errNode_out` field */
		_ALIGN(at, 8);

		/* Add `_instantiateParams__val__errNode_out` bit array field's size */
		at += 64;

		/* Align for `_instantiateParams__val__flags` field */
		_ALIGN(at, 8);

		/* Add `_instantiateParams__val__flags` bit array field's size */
		at += 64;

		/* Align for `_instantiateParams__val__result_out` field */
		_ALIGN(at, 8);

		/* Add `_instantiateParams__val__result_out` bit array field's size */
		at += 32;

		/* Align for `_instantiateParams__val__uploadStream` field */
		_ALIGN(at, 8);

		/* Add `_instantiateParams__val__uploadStream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphInstantiateWithParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphKernelNodeCopyAttributesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hSrc` field */
		_ALIGN(at, 8);

		/* Add `_hSrc` bit array field's size */
		at += 64;

		/* Align for `_hDst` field */
		_ALIGN(at, 8);

		/* Add `_hDst` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphKernelNodeCopyAttributesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphKernelNodeGetAttributeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hNode` field */
		_ALIGN(at, 8);

		/* Add `_hNode` bit array field's size */
		at += 64;

		/* Align for `_attr` field */
		_ALIGN(at, 8);

		/* Add `_attr` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;

		/* Align for `_value__val__accessPolicyWindow__base_ptr` field */
		_ALIGN(at, 8);

		/* Add `_value__val__accessPolicyWindow__base_ptr` bit array field's size */
		at += 64;

		/* Align for `_value__val__accessPolicyWindow__hitProp` field */
		_ALIGN(at, 8);

		/* Add `_value__val__accessPolicyWindow__hitProp` bit array field's size */
		at += 32;

		/* Align for `_value__val__accessPolicyWindow__hitRatio` field */
		_ALIGN(at, 8);

		/* Add `_value__val__accessPolicyWindow__hitRatio` bit array field's size */
		at += 32;

		/* Align for `_value__val__accessPolicyWindow__missProp` field */
		_ALIGN(at, 8);

		/* Add `_value__val__accessPolicyWindow__missProp` bit array field's size */
		at += 32;

		/* Align for `_value__val__accessPolicyWindow__num_bytes` field */
		_ALIGN(at, 8);

		/* Add `_value__val__accessPolicyWindow__num_bytes` bit array field's size */
		at += 64;

		/* Align for `_value__val__cooperative` field */
		_ALIGN(at, 8);

		/* Add `_value__val__cooperative` bit array field's size */
		at += 32;

		/* Align for `_value__val__priority` field */
		_ALIGN(at, 8);

		/* Add `_value__val__priority` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphKernelNodeGetAttributeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphKernelNodeGetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__blockDim__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__blockDim__x` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__blockDim__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__blockDim__y` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__blockDim__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__blockDim__z` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__extra` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__extra` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__func` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__func` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__gridDim__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__gridDim__x` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__gridDim__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__gridDim__y` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__gridDim__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__gridDim__z` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__kernelParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__kernelParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__sharedMemBytes` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__sharedMemBytes` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphKernelNodeGetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphKernelNodeSetAttributeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hNode` field */
		_ALIGN(at, 8);

		/* Add `_hNode` bit array field's size */
		at += 64;

		/* Align for `_attr` field */
		_ALIGN(at, 8);

		/* Add `_attr` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;

		/* Align for `_value__val__accessPolicyWindow__base_ptr` field */
		_ALIGN(at, 8);

		/* Add `_value__val__accessPolicyWindow__base_ptr` bit array field's size */
		at += 64;

		/* Align for `_value__val__accessPolicyWindow__hitProp` field */
		_ALIGN(at, 8);

		/* Add `_value__val__accessPolicyWindow__hitProp` bit array field's size */
		at += 32;

		/* Align for `_value__val__accessPolicyWindow__hitRatio` field */
		_ALIGN(at, 8);

		/* Add `_value__val__accessPolicyWindow__hitRatio` bit array field's size */
		at += 32;

		/* Align for `_value__val__accessPolicyWindow__missProp` field */
		_ALIGN(at, 8);

		/* Add `_value__val__accessPolicyWindow__missProp` bit array field's size */
		at += 32;

		/* Align for `_value__val__accessPolicyWindow__num_bytes` field */
		_ALIGN(at, 8);

		/* Add `_value__val__accessPolicyWindow__num_bytes` bit array field's size */
		at += 64;

		/* Align for `_value__val__cooperative` field */
		_ALIGN(at, 8);

		/* Add `_value__val__cooperative` bit array field's size */
		at += 32;

		/* Align for `_value__val__priority` field */
		_ALIGN(at, 8);

		/* Add `_value__val__priority` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphKernelNodeSetAttributeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphKernelNodeSetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__blockDim__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__blockDim__x` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__blockDim__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__blockDim__y` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__blockDim__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__blockDim__z` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__extra` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__extra` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__func` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__func` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__gridDim__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__gridDim__x` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__gridDim__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__gridDim__y` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__gridDim__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__gridDim__z` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__kernelParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__kernelParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__sharedMemBytes` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__sharedMemBytes` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphKernelNodeSetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphLaunchBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_graphExec` field */
		_ALIGN(at, 8);

		/* Add `_graphExec` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphLaunchEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemAllocNodeGetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__poolProps__allocType` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__allocType` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__poolProps__handleTypes` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__handleTypes` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__poolProps__location__type` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__location__type` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__poolProps__location__id` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__location__id` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__poolProps__win32SecurityAttributes` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__win32SecurityAttributes` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__poolProps__maxSize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__maxSize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__poolProps__reserved` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__poolProps__reserved` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__accessDescs` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__accessDescs` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__accessDescCount` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__accessDescCount` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__bytesize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__bytesize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dptr` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemAllocNodeGetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemFreeNodeGetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemFreeNodeGetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemcpyNodeGetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcArray` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcArray` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPos__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPos__x` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPos__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPos__y` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPos__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPos__z` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstArray` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstArray` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPos__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPos__x` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPos__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPos__y` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPos__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPos__z` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__extent__width` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__extent__width` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__extent__height` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__extent__height` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__extent__depth` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__extent__depth` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__kind` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemcpyNodeGetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemcpyNodeSetParams1DBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemcpyNodeSetParams1DEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemcpyNodeSetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcArray` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcArray` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPos__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPos__x` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPos__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPos__y` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPos__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPos__z` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__srcPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__srcPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstArray` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstArray` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPos__x` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPos__x` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPos__y` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPos__y` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPos__z` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPos__z` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dstPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dstPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__extent__width` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__extent__width` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__extent__height` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__extent__height` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__extent__depth` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__extent__depth` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__kind` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemcpyNodeSetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemcpyNodeSetParamsFromSymbolBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_offset` field */
		_ALIGN(at, 8);

		/* Add `_offset` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemcpyNodeSetParamsFromSymbolEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemcpyNodeSetParamsToSymbolBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_offset` field */
		_ALIGN(at, 8);

		/* Add `_offset` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemcpyNodeSetParamsToSymbolEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemsetNodeGetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dst` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dst` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__elementSize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__elementSize` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__height` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__height` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__pitch` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__value` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__value` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__width` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__width` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemsetNodeGetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemsetNodeSetParamsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__dst` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__dst` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__elementSize` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__elementSize` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__height` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__height` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__pitch` bit array field's size */
		at += 64;

		/* Align for `_pNodeParams__val__value` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__value` bit array field's size */
		at += 32;

		/* Align for `_pNodeParams__val__width` field */
		_ALIGN(at, 8);

		/* Add `_pNodeParams__val__width` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphMemsetNodeSetParamsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphNodeFindInCloneBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pNode` field */
		_ALIGN(at, 8);

		/* Add `_pNode` bit array field's size */
		at += 64;

		/* Align for `_pNode__val` field */
		_ALIGN(at, 8);

		/* Add `_pNode__val` bit array field's size */
		at += 64;

		/* Align for `_originalNode` field */
		_ALIGN(at, 8);

		/* Add `_originalNode` bit array field's size */
		at += 64;

		/* Align for `_clonedGraph` field */
		_ALIGN(at, 8);

		/* Add `_clonedGraph` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphNodeFindInCloneEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphNodeGetDependenciesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies` bit array field's size */
		at += 64;

		/* Align for `_pDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependencies__val` bit array field's size */
		at += 64;

		/* Align for `_pNumDependencies` field */
		_ALIGN(at, 8);

		/* Add `_pNumDependencies` bit array field's size */
		at += 64;

		/* Align for `_pNumDependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_pNumDependencies__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphNodeGetDependenciesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphNodeGetDependentNodesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pDependentNodes` field */
		_ALIGN(at, 8);

		/* Add `_pDependentNodes` bit array field's size */
		at += 64;

		/* Align for `_pDependentNodes__val` field */
		_ALIGN(at, 8);

		/* Add `_pDependentNodes__val` bit array field's size */
		at += 64;

		/* Align for `_pNumDependentNodes` field */
		_ALIGN(at, 8);

		/* Add `_pNumDependentNodes` bit array field's size */
		at += 64;

		/* Align for `_pNumDependentNodes__val` field */
		_ALIGN(at, 8);

		/* Add `_pNumDependentNodes__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphNodeGetDependentNodesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphNodeGetEnabledBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_hNode` field */
		_ALIGN(at, 8);

		/* Add `_hNode` bit array field's size */
		at += 64;

		/* Align for `_isEnabled` field */
		_ALIGN(at, 8);

		/* Add `_isEnabled` bit array field's size */
		at += 64;

		/* Align for `_isEnabled__val` field */
		_ALIGN(at, 8);

		/* Add `_isEnabled__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphNodeGetEnabledEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphNodeGetTypeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_node` field */
		_ALIGN(at, 8);

		/* Add `_node` bit array field's size */
		at += 64;

		/* Align for `_pType` field */
		_ALIGN(at, 8);

		/* Add `_pType` bit array field's size */
		at += 64;

		/* Align for `_pType__val` field */
		_ALIGN(at, 8);

		/* Add `_pType__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphNodeGetTypeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphNodeSetEnabledBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hGraphExec` field */
		_ALIGN(at, 8);

		/* Add `_hGraphExec` bit array field's size */
		at += 64;

		/* Align for `_hNode` field */
		_ALIGN(at, 8);

		/* Add `_hNode` bit array field's size */
		at += 64;

		/* Align for `_isEnabled` field */
		_ALIGN(at, 8);

		/* Add `_isEnabled` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphNodeSetEnabledEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphReleaseUserObjectBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_object` field */
		_ALIGN(at, 8);

		/* Add `_object` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphReleaseUserObjectEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphRemoveDependenciesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_from` field */
		_ALIGN(at, 8);

		/* Add `_from` bit array field's size */
		at += 64;

		/* Align for `_from__val` field */
		_ALIGN(at, 8);

		/* Add `_from__val` bit array field's size */
		at += 64;

		/* Align for `_to` field */
		_ALIGN(at, 8);

		/* Add `_to` bit array field's size */
		at += 64;

		/* Align for `_to__val` field */
		_ALIGN(at, 8);

		/* Add `_to__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphRemoveDependenciesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphRetainUserObjectBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_object` field */
		_ALIGN(at, 8);

		/* Add `_object` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphRetainUserObjectEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphUploadBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_graphExec` field */
		_ALIGN(at, 8);

		/* Add `_graphExec` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphUploadEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsGLRegisterBufferBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_resource` field */
		_ALIGN(at, 8);

		/* Add `_resource` bit array field's size */
		at += 64;

		/* Align for `_resource__val` field */
		_ALIGN(at, 8);

		/* Add `_resource__val` bit array field's size */
		at += 64;

		/* Align for `_buffer` field */
		_ALIGN(at, 8);

		/* Add `_buffer` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsGLRegisterBufferEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsGLRegisterImageBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_resource` field */
		_ALIGN(at, 8);

		/* Add `_resource` bit array field's size */
		at += 64;

		/* Align for `_resource__val` field */
		_ALIGN(at, 8);

		/* Add `_resource__val` bit array field's size */
		at += 64;

		/* Align for `_image` field */
		_ALIGN(at, 8);

		/* Add `_image` bit array field's size */
		at += 32;

		/* Align for `_target` field */
		_ALIGN(at, 8);

		/* Add `_target` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsGLRegisterImageEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsMapResourcesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 32;

		/* Align for `_resources` field */
		_ALIGN(at, 8);

		/* Add `_resources` bit array field's size */
		at += 64;

		/* Align for `_resources__val` field */
		_ALIGN(at, 8);

		/* Add `_resources__val` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsMapResourcesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsResourceGetMappedPointerBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(at, 8);

		/* Add `_devPtr` bit array field's size */
		at += 64;

		/* Align for `_devPtr__val` field */
		_ALIGN(at, 8);

		/* Add `_devPtr__val` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_size__val` field */
		_ALIGN(at, 8);

		/* Add `_size__val` bit array field's size */
		at += 64;

		/* Align for `_resource` field */
		_ALIGN(at, 8);

		/* Add `_resource` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsResourceGetMappedPointerEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsSubResourceGetMappedArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_array` field */
		_ALIGN(at, 8);

		/* Add `_array` bit array field's size */
		at += 64;

		/* Align for `_array__val` field */
		_ALIGN(at, 8);

		/* Add `_array__val` bit array field's size */
		at += 64;

		/* Align for `_resource` field */
		_ALIGN(at, 8);

		/* Add `_resource` bit array field's size */
		at += 64;

		/* Align for `_arrayIndex` field */
		_ALIGN(at, 8);

		/* Add `_arrayIndex` bit array field's size */
		at += 32;

		/* Align for `_mipLevel` field */
		_ALIGN(at, 8);

		/* Add `_mipLevel` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsSubResourceGetMappedArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsUnmapResourcesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 32;

		/* Align for `_resources` field */
		_ALIGN(at, 8);

		/* Add `_resources` bit array field's size */
		at += 64;

		/* Align for `_resources__val` field */
		_ALIGN(at, 8);

		/* Add `_resources__val` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsUnmapResourcesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsUnregisterResourceBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_resource` field */
		_ALIGN(at, 8);

		/* Add `_resource` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipGraphicsUnregisterResourceEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHccModuleLaunchKernelBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_f` field */
		_ALIGN(at, 8);

		/* Add `_f` bit array field's size */
		at += 64;

		/* Align for `_globalWorkSizeX` field */
		_ALIGN(at, 8);

		/* Add `_globalWorkSizeX` bit array field's size */
		at += 32;

		/* Align for `_globalWorkSizeY` field */
		_ALIGN(at, 8);

		/* Add `_globalWorkSizeY` bit array field's size */
		at += 32;

		/* Align for `_globalWorkSizeZ` field */
		_ALIGN(at, 8);

		/* Add `_globalWorkSizeZ` bit array field's size */
		at += 32;

		/* Align for `_blockDimX` field */
		_ALIGN(at, 8);

		/* Add `_blockDimX` bit array field's size */
		at += 32;

		/* Align for `_blockDimY` field */
		_ALIGN(at, 8);

		/* Add `_blockDimY` bit array field's size */
		at += 32;

		/* Align for `_blockDimZ` field */
		_ALIGN(at, 8);

		/* Add `_blockDimZ` bit array field's size */
		at += 32;

		/* Align for `_sharedMemBytes` field */
		_ALIGN(at, 8);

		/* Add `_sharedMemBytes` bit array field's size */
		at += 64;

		/* Align for `_hStream` field */
		_ALIGN(at, 8);

		/* Add `_hStream` bit array field's size */
		at += 64;

		/* Align for `_kernelParams` field */
		_ALIGN(at, 8);

		/* Add `_kernelParams` bit array field's size */
		at += 64;

		/* Align for `_kernelParams__val` field */
		_ALIGN(at, 8);

		/* Add `_kernelParams__val` bit array field's size */
		at += 64;

		/* Align for `_extra` field */
		_ALIGN(at, 8);

		/* Add `_extra` bit array field's size */
		at += 64;

		/* Align for `_extra__val` field */
		_ALIGN(at, 8);

		/* Add `_extra__val` bit array field's size */
		at += 64;

		/* Align for `_startEvent` field */
		_ALIGN(at, 8);

		/* Add `_startEvent` bit array field's size */
		at += 64;

		/* Align for `_stopEvent` field */
		_ALIGN(at, 8);

		/* Add `_stopEvent` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHccModuleLaunchKernelEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostAllocBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_ptr__val` field */
		_ALIGN(at, 8);

		/* Add `_ptr__val` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostAllocEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostFreeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostFreeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostGetDevicePointerBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(at, 8);

		/* Add `_devPtr` bit array field's size */
		at += 64;

		/* Align for `_devPtr__val` field */
		_ALIGN(at, 8);

		/* Add `_devPtr__val` bit array field's size */
		at += 64;

		/* Align for `_hstPtr` field */
		_ALIGN(at, 8);

		/* Add `_hstPtr` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostGetDevicePointerEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostGetFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_flagsPtr` field */
		_ALIGN(at, 8);

		/* Add `_flagsPtr` bit array field's size */
		at += 64;

		/* Align for `_flagsPtr__val` field */
		_ALIGN(at, 8);

		/* Add `_flagsPtr__val` bit array field's size */
		at += 32;

		/* Align for `_hostPtr` field */
		_ALIGN(at, 8);

		/* Add `_hostPtr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostGetFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostMallocBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_ptr__val` field */
		_ALIGN(at, 8);

		/* Add `_ptr__val` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostMallocEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostRegisterBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hostPtr` field */
		_ALIGN(at, 8);

		/* Add `_hostPtr` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostRegisterEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostUnregisterBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hostPtr` field */
		_ALIGN(at, 8);

		/* Add `_hostPtr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipHostUnregisterEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipImportExternalMemoryBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_extMem_out` field */
		_ALIGN(at, 8);

		/* Add `_extMem_out` bit array field's size */
		at += 64;

		/* Align for `_extMem_out__val` field */
		_ALIGN(at, 8);

		/* Add `_extMem_out__val` bit array field's size */
		at += 64;

		/* Align for `_memHandleDesc` field */
		_ALIGN(at, 8);

		/* Add `_memHandleDesc` bit array field's size */
		at += 64;

		/* Align for `_memHandleDesc__val__type` field */
		_ALIGN(at, 8);

		/* Add `_memHandleDesc__val__type` bit array field's size */
		at += 32;

		/* Align for `_memHandleDesc__val__size` field */
		_ALIGN(at, 8);

		/* Add `_memHandleDesc__val__size` bit array field's size */
		at += 64;

		/* Align for `_memHandleDesc__val__flags` field */
		_ALIGN(at, 8);

		/* Add `_memHandleDesc__val__flags` bit array field's size */
		at += 32;

		/* Align for `_memHandleDesc__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_memHandleDesc__val__reserved` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipImportExternalMemoryEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipImportExternalSemaphoreBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_extSem_out` field */
		_ALIGN(at, 8);

		/* Add `_extSem_out` bit array field's size */
		at += 64;

		/* Align for `_extSem_out__val` field */
		_ALIGN(at, 8);

		/* Add `_extSem_out__val` bit array field's size */
		at += 64;

		/* Align for `_semHandleDesc` field */
		_ALIGN(at, 8);

		/* Add `_semHandleDesc` bit array field's size */
		at += 64;

		/* Align for `_semHandleDesc__val__type` field */
		_ALIGN(at, 8);

		/* Add `_semHandleDesc__val__type` bit array field's size */
		at += 32;

		/* Align for `_semHandleDesc__val__flags` field */
		_ALIGN(at, 8);

		/* Add `_semHandleDesc__val__flags` bit array field's size */
		at += 32;

		/* Align for `_semHandleDesc__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_semHandleDesc__val__reserved` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipImportExternalSemaphoreEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipInitBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipInitEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipIpcCloseMemHandleBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(at, 8);

		/* Add `_devPtr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipIpcCloseMemHandleEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipIpcGetEventHandleBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;

		/* Align for `_handle__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_handle__val__reserved` bit array field's size */
		at += 64;

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipIpcGetEventHandleEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipIpcGetMemHandleBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;

		/* Align for `_handle__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_handle__val__reserved` bit array field's size */
		at += 64;

		/* Align for `_devPtr` field */
		_ALIGN(at, 8);

		/* Add `_devPtr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipIpcGetMemHandleEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipIpcOpenEventHandleBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;

		/* Align for `_event__val` field */
		_ALIGN(at, 8);

		/* Add `_event__val` bit array field's size */
		at += 64;

		/* Align for `_handle__reserved` field */
		_ALIGN(at, 8);

		/* Add `_handle__reserved` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipIpcOpenEventHandleEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipIpcOpenMemHandleBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(at, 8);

		/* Add `_devPtr` bit array field's size */
		at += 64;

		/* Align for `_devPtr__val` field */
		_ALIGN(at, 8);

		/* Add `_devPtr__val` bit array field's size */
		at += 64;

		/* Align for `_handle__reserved` field */
		_ALIGN(at, 8);

		/* Add `_handle__reserved` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipIpcOpenMemHandleEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipLaunchByPtrBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hostFunction` field */
		_ALIGN(at, 8);

		/* Add `_hostFunction` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipLaunchByPtrEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipLaunchCooperativeKernelBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_f` field */
		_ALIGN(at, 8);

		/* Add `_f` bit array field's size */
		at += 64;

		/* Align for `_gridDim__x` field */
		_ALIGN(at, 8);

		/* Add `_gridDim__x` bit array field's size */
		at += 32;

		/* Align for `_gridDim__y` field */
		_ALIGN(at, 8);

		/* Add `_gridDim__y` bit array field's size */
		at += 32;

		/* Align for `_gridDim__z` field */
		_ALIGN(at, 8);

		/* Add `_gridDim__z` bit array field's size */
		at += 32;

		/* Align for `_blockDimX__x` field */
		_ALIGN(at, 8);

		/* Add `_blockDimX__x` bit array field's size */
		at += 32;

		/* Align for `_blockDimX__y` field */
		_ALIGN(at, 8);

		/* Add `_blockDimX__y` bit array field's size */
		at += 32;

		/* Align for `_blockDimX__z` field */
		_ALIGN(at, 8);

		/* Add `_blockDimX__z` bit array field's size */
		at += 32;

		/* Align for `_kernelParams` field */
		_ALIGN(at, 8);

		/* Add `_kernelParams` bit array field's size */
		at += 64;

		/* Align for `_kernelParams__val` field */
		_ALIGN(at, 8);

		/* Add `_kernelParams__val` bit array field's size */
		at += 64;

		/* Align for `_sharedMemBytes` field */
		_ALIGN(at, 8);

		/* Add `_sharedMemBytes` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipLaunchCooperativeKernelEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipLaunchCooperativeKernelMultiDeviceBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_launchParamsList` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList` bit array field's size */
		at += 64;

		/* Align for `_launchParamsList__val__func` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__func` bit array field's size */
		at += 64;

		/* Align for `_launchParamsList__val__gridDim__x` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__gridDim__x` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__gridDim__y` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__gridDim__y` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__gridDim__z` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__gridDim__z` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__blockDim__x` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__blockDim__x` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__blockDim__y` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__blockDim__y` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__blockDim__z` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__blockDim__z` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__args` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__args` bit array field's size */
		at += 64;

		/* Align for `_launchParamsList__val__sharedMem` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__sharedMem` bit array field's size */
		at += 64;

		/* Align for `_launchParamsList__val__stream` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__stream` bit array field's size */
		at += 64;

		/* Align for `_numDevices` field */
		_ALIGN(at, 8);

		/* Add `_numDevices` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipLaunchCooperativeKernelMultiDeviceEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipLaunchHostFuncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_fn` field */
		_ALIGN(at, 8);

		/* Add `_fn` bit array field's size */
		at += 64;

		/* Align for `_userData` field */
		_ALIGN(at, 8);

		/* Add `_userData` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipLaunchHostFuncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipLaunchKernelBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_function_address` field */
		_ALIGN(at, 8);

		/* Add `_function_address` bit array field's size */
		at += 64;

		/* Align for `_numBlocks__x` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks__x` bit array field's size */
		at += 32;

		/* Align for `_numBlocks__y` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks__y` bit array field's size */
		at += 32;

		/* Align for `_numBlocks__z` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks__z` bit array field's size */
		at += 32;

		/* Align for `_dimBlocks__x` field */
		_ALIGN(at, 8);

		/* Add `_dimBlocks__x` bit array field's size */
		at += 32;

		/* Align for `_dimBlocks__y` field */
		_ALIGN(at, 8);

		/* Add `_dimBlocks__y` bit array field's size */
		at += 32;

		/* Align for `_dimBlocks__z` field */
		_ALIGN(at, 8);

		/* Add `_dimBlocks__z` bit array field's size */
		at += 32;

		/* Align for `_args` field */
		_ALIGN(at, 8);

		/* Add `_args` bit array field's size */
		at += 64;

		/* Align for `_args__val` field */
		_ALIGN(at, 8);

		/* Add `_args__val` bit array field's size */
		at += 64;

		/* Align for `_sharedMemBytes` field */
		_ALIGN(at, 8);

		/* Add `_sharedMemBytes` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipLaunchKernelEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMalloc3DArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_array` field */
		_ALIGN(at, 8);

		/* Add `_array` bit array field's size */
		at += 64;

		/* Align for `_array__val` field */
		_ALIGN(at, 8);

		/* Add `_array__val` bit array field's size */
		at += 64;

		/* Align for `_desc` field */
		_ALIGN(at, 8);

		/* Add `_desc` bit array field's size */
		at += 64;

		/* Align for `_desc__val__x` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__x` bit array field's size */
		at += 32;

		/* Align for `_desc__val__y` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__y` bit array field's size */
		at += 32;

		/* Align for `_desc__val__z` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__z` bit array field's size */
		at += 32;

		/* Align for `_desc__val__w` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__w` bit array field's size */
		at += 32;

		/* Align for `_desc__val__f` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__f` bit array field's size */
		at += 32;

		/* Align for `_extent__width` field */
		_ALIGN(at, 8);

		/* Add `_extent__width` bit array field's size */
		at += 64;

		/* Align for `_extent__height` field */
		_ALIGN(at, 8);

		/* Add `_extent__height` bit array field's size */
		at += 64;

		/* Align for `_extent__depth` field */
		_ALIGN(at, 8);

		/* Add `_extent__depth` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMalloc3DArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMalloc3DBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pitchedDevPtr` field */
		_ALIGN(at, 8);

		/* Add `_pitchedDevPtr` bit array field's size */
		at += 64;

		/* Align for `_pitchedDevPtr__val__ptr` field */
		_ALIGN(at, 8);

		/* Add `_pitchedDevPtr__val__ptr` bit array field's size */
		at += 64;

		/* Align for `_pitchedDevPtr__val__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pitchedDevPtr__val__pitch` bit array field's size */
		at += 64;

		/* Align for `_pitchedDevPtr__val__xsize` field */
		_ALIGN(at, 8);

		/* Add `_pitchedDevPtr__val__xsize` bit array field's size */
		at += 64;

		/* Align for `_pitchedDevPtr__val__ysize` field */
		_ALIGN(at, 8);

		/* Add `_pitchedDevPtr__val__ysize` bit array field's size */
		at += 64;

		/* Align for `_extent__width` field */
		_ALIGN(at, 8);

		/* Add `_extent__width` bit array field's size */
		at += 64;

		/* Align for `_extent__height` field */
		_ALIGN(at, 8);

		/* Add `_extent__height` bit array field's size */
		at += 64;

		/* Align for `_extent__depth` field */
		_ALIGN(at, 8);

		/* Add `_extent__depth` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMalloc3DEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_array` field */
		_ALIGN(at, 8);

		/* Add `_array` bit array field's size */
		at += 64;

		/* Align for `_array__val` field */
		_ALIGN(at, 8);

		/* Add `_array__val` bit array field's size */
		at += 64;

		/* Align for `_desc` field */
		_ALIGN(at, 8);

		/* Add `_desc` bit array field's size */
		at += 64;

		/* Align for `_desc__val__x` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__x` bit array field's size */
		at += 32;

		/* Align for `_desc__val__y` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__y` bit array field's size */
		at += 32;

		/* Align for `_desc__val__z` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__z` bit array field's size */
		at += 32;

		/* Align for `_desc__val__w` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__w` bit array field's size */
		at += 32;

		/* Align for `_desc__val__f` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__f` bit array field's size */
		at += 32;

		/* Align for `_width` field */
		_ALIGN(at, 8);

		/* Add `_width` bit array field's size */
		at += 64;

		/* Align for `_height` field */
		_ALIGN(at, 8);

		/* Add `_height` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;

		/* Align for `_dev_ptr__val` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr__val` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_ptr__val` field */
		_ALIGN(at, 8);

		/* Add `_ptr__val` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocFromPoolAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;

		/* Align for `_dev_ptr__val` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr__val` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocFromPoolAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocHostBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_ptr__val` field */
		_ALIGN(at, 8);

		/* Add `_ptr__val` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocHostEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocManagedBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;

		/* Align for `_dev_ptr__val` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr__val` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocManagedEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocMipmappedArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mipmappedArray` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray` bit array field's size */
		at += 64;

		/* Align for `_mipmappedArray__val` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val` bit array field's size */
		at += 64;

		/* Align for `_desc` field */
		_ALIGN(at, 8);

		/* Add `_desc` bit array field's size */
		at += 64;

		/* Align for `_desc__val__x` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__x` bit array field's size */
		at += 32;

		/* Align for `_desc__val__y` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__y` bit array field's size */
		at += 32;

		/* Align for `_desc__val__z` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__z` bit array field's size */
		at += 32;

		/* Align for `_desc__val__w` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__w` bit array field's size */
		at += 32;

		/* Align for `_desc__val__f` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__f` bit array field's size */
		at += 32;

		/* Align for `_extent__width` field */
		_ALIGN(at, 8);

		/* Add `_extent__width` bit array field's size */
		at += 64;

		/* Align for `_extent__height` field */
		_ALIGN(at, 8);

		/* Add `_extent__height` bit array field's size */
		at += 64;

		/* Align for `_extent__depth` field */
		_ALIGN(at, 8);

		/* Add `_extent__depth` bit array field's size */
		at += 64;

		/* Align for `_numLevels` field */
		_ALIGN(at, 8);

		/* Add `_numLevels` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocMipmappedArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocPitchBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_ptr__val` field */
		_ALIGN(at, 8);

		/* Add `_ptr__val` bit array field's size */
		at += 64;

		/* Align for `_pitch` field */
		_ALIGN(at, 8);

		/* Add `_pitch` bit array field's size */
		at += 64;

		/* Align for `_pitch__val` field */
		_ALIGN(at, 8);

		/* Add `_pitch__val` bit array field's size */
		at += 64;

		/* Align for `_width` field */
		_ALIGN(at, 8);

		/* Add `_width` bit array field's size */
		at += 64;

		/* Align for `_height` field */
		_ALIGN(at, 8);

		/* Add `_height` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMallocPitchEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemAddressFreeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_devPtr` field */
		_ALIGN(at, 8);

		/* Add `_devPtr` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemAddressFreeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemAddressReserveBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_ptr__val` field */
		_ALIGN(at, 8);

		/* Add `_ptr__val` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_alignment` field */
		_ALIGN(at, 8);

		/* Add `_alignment` bit array field's size */
		at += 64;

		/* Align for `_addr` field */
		_ALIGN(at, 8);

		/* Add `_addr` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemAddressReserveEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemAdviseBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_advice` field */
		_ALIGN(at, 8);

		/* Add `_advice` bit array field's size */
		at += 32;

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemAdviseEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemAllocHostBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_ptr__val` field */
		_ALIGN(at, 8);

		/* Add `_ptr__val` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemAllocHostEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemAllocPitchBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dptr` field */
		_ALIGN(at, 8);

		/* Add `_dptr` bit array field's size */
		at += 64;

		/* Align for `_dptr__val` field */
		_ALIGN(at, 8);

		/* Add `_dptr__val` bit array field's size */
		at += 64;

		/* Align for `_pitch` field */
		_ALIGN(at, 8);

		/* Add `_pitch` bit array field's size */
		at += 64;

		/* Align for `_pitch__val` field */
		_ALIGN(at, 8);

		/* Add `_pitch__val` bit array field's size */
		at += 64;

		/* Align for `_widthInBytes` field */
		_ALIGN(at, 8);

		/* Add `_widthInBytes` bit array field's size */
		at += 64;

		/* Align for `_height` field */
		_ALIGN(at, 8);

		/* Add `_height` bit array field's size */
		at += 64;

		/* Align for `_elementSizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_elementSizeBytes` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemAllocPitchEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemCreateBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;

		/* Align for `_handle__val` field */
		_ALIGN(at, 8);

		/* Add `_handle__val` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_prop` field */
		_ALIGN(at, 8);

		/* Add `_prop` bit array field's size */
		at += 64;

		/* Align for `_prop__val__type` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__type` bit array field's size */
		at += 32;

		/* Align for `_prop__val__requestedHandleType` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__requestedHandleType` bit array field's size */
		at += 32;

		/* Align for `_prop__val__location__type` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__location__type` bit array field's size */
		at += 32;

		/* Align for `_prop__val__location__id` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__location__id` bit array field's size */
		at += 32;

		/* Align for `_prop__val__win32HandleMetaData` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__win32HandleMetaData` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemCreateEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemExportToShareableHandleBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_shareableHandle` field */
		_ALIGN(at, 8);

		/* Add `_shareableHandle` bit array field's size */
		at += 64;

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;

		/* Align for `_handleType` field */
		_ALIGN(at, 8);

		/* Add `_handleType` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemExportToShareableHandleEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemGetAccessBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;

		/* Align for `_flags__val` field */
		_ALIGN(at, 8);

		/* Add `_flags__val` bit array field's size */
		at += 64;

		/* Align for `_location` field */
		_ALIGN(at, 8);

		/* Add `_location` bit array field's size */
		at += 64;

		/* Align for `_location__val__type` field */
		_ALIGN(at, 8);

		/* Add `_location__val__type` bit array field's size */
		at += 32;

		/* Align for `_location__val__id` field */
		_ALIGN(at, 8);

		/* Add `_location__val__id` bit array field's size */
		at += 32;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemGetAccessEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemGetAddressRangeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pbase` field */
		_ALIGN(at, 8);

		/* Add `_pbase` bit array field's size */
		at += 64;

		/* Align for `_pbase__val` field */
		_ALIGN(at, 8);

		/* Add `_pbase__val` bit array field's size */
		at += 64;

		/* Align for `_psize` field */
		_ALIGN(at, 8);

		/* Add `_psize` bit array field's size */
		at += 64;

		/* Align for `_psize__val` field */
		_ALIGN(at, 8);

		/* Add `_psize__val` bit array field's size */
		at += 64;

		/* Align for `_dptr` field */
		_ALIGN(at, 8);

		/* Add `_dptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemGetAddressRangeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemGetAllocationGranularityBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_granularity` field */
		_ALIGN(at, 8);

		/* Add `_granularity` bit array field's size */
		at += 64;

		/* Align for `_granularity__val` field */
		_ALIGN(at, 8);

		/* Add `_granularity__val` bit array field's size */
		at += 64;

		/* Align for `_prop` field */
		_ALIGN(at, 8);

		/* Add `_prop` bit array field's size */
		at += 64;

		/* Align for `_prop__val__type` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__type` bit array field's size */
		at += 32;

		/* Align for `_prop__val__requestedHandleType` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__requestedHandleType` bit array field's size */
		at += 32;

		/* Align for `_prop__val__location__type` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__location__type` bit array field's size */
		at += 32;

		/* Align for `_prop__val__location__id` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__location__id` bit array field's size */
		at += 32;

		/* Align for `_prop__val__win32HandleMetaData` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__win32HandleMetaData` bit array field's size */
		at += 64;

		/* Align for `_option` field */
		_ALIGN(at, 8);

		/* Add `_option` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemGetAllocationGranularityEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemGetAllocationPropertiesFromHandleBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_prop` field */
		_ALIGN(at, 8);

		/* Add `_prop` bit array field's size */
		at += 64;

		/* Align for `_prop__val__type` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__type` bit array field's size */
		at += 32;

		/* Align for `_prop__val__requestedHandleType` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__requestedHandleType` bit array field's size */
		at += 32;

		/* Align for `_prop__val__location__type` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__location__type` bit array field's size */
		at += 32;

		/* Align for `_prop__val__location__id` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__location__id` bit array field's size */
		at += 32;

		/* Align for `_prop__val__win32HandleMetaData` field */
		_ALIGN(at, 8);

		/* Add `_prop__val__win32HandleMetaData` bit array field's size */
		at += 64;

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemGetAllocationPropertiesFromHandleEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemGetInfoBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_free` field */
		_ALIGN(at, 8);

		/* Add `_free` bit array field's size */
		at += 64;

		/* Align for `_free__val` field */
		_ALIGN(at, 8);

		/* Add `_free__val` bit array field's size */
		at += 64;

		/* Align for `_total` field */
		_ALIGN(at, 8);

		/* Add `_total` bit array field's size */
		at += 64;

		/* Align for `_total__val` field */
		_ALIGN(at, 8);

		/* Add `_total__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemGetInfoEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemImportFromShareableHandleBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;

		/* Align for `_handle__val` field */
		_ALIGN(at, 8);

		/* Add `_handle__val` bit array field's size */
		at += 64;

		/* Align for `_osHandle` field */
		_ALIGN(at, 8);

		/* Add `_osHandle` bit array field's size */
		at += 64;

		/* Align for `_shHandleType` field */
		_ALIGN(at, 8);

		/* Add `_shHandleType` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemImportFromShareableHandleEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemMapArrayAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mapInfoList` field */
		_ALIGN(at, 8);

		/* Add `_mapInfoList` bit array field's size */
		at += 64;

		/* Align for `_mapInfoList__val__resourceType` field */
		_ALIGN(at, 8);

		/* Add `_mapInfoList__val__resourceType` bit array field's size */
		at += 32;

		/* Align for `_mapInfoList__val__subresourceType` field */
		_ALIGN(at, 8);

		/* Add `_mapInfoList__val__subresourceType` bit array field's size */
		at += 32;

		/* Align for `_mapInfoList__val__memOperationType` field */
		_ALIGN(at, 8);

		/* Add `_mapInfoList__val__memOperationType` bit array field's size */
		at += 32;

		/* Align for `_mapInfoList__val__memHandleType` field */
		_ALIGN(at, 8);

		/* Add `_mapInfoList__val__memHandleType` bit array field's size */
		at += 32;

		/* Align for `_mapInfoList__val__offset` field */
		_ALIGN(at, 8);

		/* Add `_mapInfoList__val__offset` bit array field's size */
		at += 64;

		/* Align for `_mapInfoList__val__deviceBitMask` field */
		_ALIGN(at, 8);

		/* Add `_mapInfoList__val__deviceBitMask` bit array field's size */
		at += 32;

		/* Align for `_mapInfoList__val__flags` field */
		_ALIGN(at, 8);

		/* Add `_mapInfoList__val__flags` bit array field's size */
		at += 32;

		/* Align for `_mapInfoList__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_mapInfoList__val__reserved` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemMapArrayAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemMapBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_offset` field */
		_ALIGN(at, 8);

		/* Add `_offset` bit array field's size */
		at += 64;

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemMapEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolCreateBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;

		/* Align for `_mem_pool__val` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool__val` bit array field's size */
		at += 64;

		/* Align for `_pool_props` field */
		_ALIGN(at, 8);

		/* Add `_pool_props` bit array field's size */
		at += 64;

		/* Align for `_pool_props__val__allocType` field */
		_ALIGN(at, 8);

		/* Add `_pool_props__val__allocType` bit array field's size */
		at += 32;

		/* Align for `_pool_props__val__handleTypes` field */
		_ALIGN(at, 8);

		/* Add `_pool_props__val__handleTypes` bit array field's size */
		at += 32;

		/* Align for `_pool_props__val__location__type` field */
		_ALIGN(at, 8);

		/* Add `_pool_props__val__location__type` bit array field's size */
		at += 32;

		/* Align for `_pool_props__val__location__id` field */
		_ALIGN(at, 8);

		/* Add `_pool_props__val__location__id` bit array field's size */
		at += 32;

		/* Align for `_pool_props__val__win32SecurityAttributes` field */
		_ALIGN(at, 8);

		/* Add `_pool_props__val__win32SecurityAttributes` bit array field's size */
		at += 64;

		/* Align for `_pool_props__val__maxSize` field */
		_ALIGN(at, 8);

		/* Add `_pool_props__val__maxSize` bit array field's size */
		at += 64;

		/* Align for `_pool_props__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_pool_props__val__reserved` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolCreateEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolDestroyBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolDestroyEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolExportPointerBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_export_data` field */
		_ALIGN(at, 8);

		/* Add `_export_data` bit array field's size */
		at += 64;

		/* Align for `_export_data__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_export_data__val__reserved` bit array field's size */
		at += 64;

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolExportPointerEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolExportToShareableHandleBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_shared_handle` field */
		_ALIGN(at, 8);

		/* Add `_shared_handle` bit array field's size */
		at += 64;

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;

		/* Align for `_handle_type` field */
		_ALIGN(at, 8);

		/* Add `_handle_type` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolExportToShareableHandleEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolGetAccessBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;

		/* Align for `_flags__val` field */
		_ALIGN(at, 8);

		/* Add `_flags__val` bit array field's size */
		at += 32;

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;

		/* Align for `_location` field */
		_ALIGN(at, 8);

		/* Add `_location` bit array field's size */
		at += 64;

		/* Align for `_location__val__type` field */
		_ALIGN(at, 8);

		/* Add `_location__val__type` bit array field's size */
		at += 32;

		/* Align for `_location__val__id` field */
		_ALIGN(at, 8);

		/* Add `_location__val__id` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolGetAccessEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolGetAttributeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;

		/* Align for `_attr` field */
		_ALIGN(at, 8);

		/* Add `_attr` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolGetAttributeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolImportFromShareableHandleBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;

		/* Align for `_mem_pool__val` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool__val` bit array field's size */
		at += 64;

		/* Align for `_shared_handle` field */
		_ALIGN(at, 8);

		/* Add `_shared_handle` bit array field's size */
		at += 64;

		/* Align for `_handle_type` field */
		_ALIGN(at, 8);

		/* Add `_handle_type` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolImportFromShareableHandleEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolImportPointerBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;

		/* Align for `_dev_ptr__val` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr__val` bit array field's size */
		at += 64;

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;

		/* Align for `_export_data` field */
		_ALIGN(at, 8);

		/* Add `_export_data` bit array field's size */
		at += 64;

		/* Align for `_export_data__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_export_data__val__reserved` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolImportPointerEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolSetAccessBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;

		/* Align for `_desc_list` field */
		_ALIGN(at, 8);

		/* Add `_desc_list` bit array field's size */
		at += 64;

		/* Align for `_desc_list__val__location__type` field */
		_ALIGN(at, 8);

		/* Add `_desc_list__val__location__type` bit array field's size */
		at += 32;

		/* Align for `_desc_list__val__location__id` field */
		_ALIGN(at, 8);

		/* Add `_desc_list__val__location__id` bit array field's size */
		at += 32;

		/* Align for `_desc_list__val__flags` field */
		_ALIGN(at, 8);

		/* Add `_desc_list__val__flags` bit array field's size */
		at += 32;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolSetAccessEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolSetAttributeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;

		/* Align for `_attr` field */
		_ALIGN(at, 8);

		/* Add `_attr` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolSetAttributeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolTrimToBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mem_pool` field */
		_ALIGN(at, 8);

		/* Add `_mem_pool` bit array field's size */
		at += 64;

		/* Align for `_min_bytes_to_hold` field */
		_ALIGN(at, 8);

		/* Add `_min_bytes_to_hold` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPoolTrimToEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPrefetchAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPrefetchAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPtrGetInfoBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_size__val` field */
		_ALIGN(at, 8);

		/* Add `_size__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemPtrGetInfoEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemRangeGetAttributeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;

		/* Align for `_data_size` field */
		_ALIGN(at, 8);

		/* Add `_data_size` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemRangeGetAttributeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemRangeGetAttributesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;

		/* Align for `_data__val` field */
		_ALIGN(at, 8);

		/* Add `_data__val` bit array field's size */
		at += 64;

		/* Align for `_data_sizes` field */
		_ALIGN(at, 8);

		/* Add `_data_sizes` bit array field's size */
		at += 64;

		/* Align for `_data_sizes__val` field */
		_ALIGN(at, 8);

		/* Add `_data_sizes__val` bit array field's size */
		at += 64;

		/* Align for `_attributes` field */
		_ALIGN(at, 8);

		/* Add `_attributes` bit array field's size */
		at += 64;

		/* Align for `_attributes__val` field */
		_ALIGN(at, 8);

		/* Add `_attributes__val` bit array field's size */
		at += 32;

		/* Align for `_num_attributes` field */
		_ALIGN(at, 8);

		/* Add `_num_attributes` bit array field's size */
		at += 64;

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemRangeGetAttributesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemReleaseBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemReleaseEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemRetainAllocationHandleBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;

		/* Align for `_handle__val` field */
		_ALIGN(at, 8);

		/* Add `_handle__val` bit array field's size */
		at += 64;

		/* Align for `_addr` field */
		_ALIGN(at, 8);

		/* Add `_addr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemRetainAllocationHandleEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemSetAccessBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_desc` field */
		_ALIGN(at, 8);

		/* Add `_desc` bit array field's size */
		at += 64;

		/* Align for `_desc__val__location__type` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__location__type` bit array field's size */
		at += 32;

		/* Align for `_desc__val__location__id` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__location__id` bit array field's size */
		at += 32;

		/* Align for `_desc__val__flags` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__flags` bit array field's size */
		at += 32;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemSetAccessEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemUnmapBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemUnmapEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DArrayToArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_wOffsetDst` field */
		_ALIGN(at, 8);

		/* Add `_wOffsetDst` bit array field's size */
		at += 64;

		/* Align for `_hOffsetDst` field */
		_ALIGN(at, 8);

		/* Add `_hOffsetDst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_wOffsetSrc` field */
		_ALIGN(at, 8);

		/* Add `_wOffsetSrc` bit array field's size */
		at += 64;

		/* Align for `_hOffsetSrc` field */
		_ALIGN(at, 8);

		/* Add `_hOffsetSrc` bit array field's size */
		at += 64;

		/* Align for `_width` field */
		_ALIGN(at, 8);

		/* Add `_width` bit array field's size */
		at += 64;

		/* Align for `_height` field */
		_ALIGN(at, 8);

		/* Add `_height` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DArrayToArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_dpitch` field */
		_ALIGN(at, 8);

		/* Add `_dpitch` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_spitch` field */
		_ALIGN(at, 8);

		/* Add `_spitch` bit array field's size */
		at += 64;

		/* Align for `_width` field */
		_ALIGN(at, 8);

		/* Add `_width` bit array field's size */
		at += 64;

		/* Align for `_height` field */
		_ALIGN(at, 8);

		/* Add `_height` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_dpitch` field */
		_ALIGN(at, 8);

		/* Add `_dpitch` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_spitch` field */
		_ALIGN(at, 8);

		/* Add `_spitch` bit array field's size */
		at += 64;

		/* Align for `_width` field */
		_ALIGN(at, 8);

		/* Add `_width` bit array field's size */
		at += 64;

		/* Align for `_height` field */
		_ALIGN(at, 8);

		/* Add `_height` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DFromArrayAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_dpitch` field */
		_ALIGN(at, 8);

		/* Add `_dpitch` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_wOffset` field */
		_ALIGN(at, 8);

		/* Add `_wOffset` bit array field's size */
		at += 64;

		/* Align for `_hOffset` field */
		_ALIGN(at, 8);

		/* Add `_hOffset` bit array field's size */
		at += 64;

		/* Align for `_width` field */
		_ALIGN(at, 8);

		/* Add `_width` bit array field's size */
		at += 64;

		/* Align for `_height` field */
		_ALIGN(at, 8);

		/* Add `_height` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DFromArrayAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DFromArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_dpitch` field */
		_ALIGN(at, 8);

		/* Add `_dpitch` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_wOffset` field */
		_ALIGN(at, 8);

		/* Add `_wOffset` bit array field's size */
		at += 64;

		/* Align for `_hOffset` field */
		_ALIGN(at, 8);

		/* Add `_hOffset` bit array field's size */
		at += 64;

		/* Align for `_width` field */
		_ALIGN(at, 8);

		/* Add `_width` bit array field's size */
		at += 64;

		/* Align for `_height` field */
		_ALIGN(at, 8);

		/* Add `_height` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DFromArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DToArrayAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_wOffset` field */
		_ALIGN(at, 8);

		/* Add `_wOffset` bit array field's size */
		at += 64;

		/* Align for `_hOffset` field */
		_ALIGN(at, 8);

		/* Add `_hOffset` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_spitch` field */
		_ALIGN(at, 8);

		/* Add `_spitch` bit array field's size */
		at += 64;

		/* Align for `_width` field */
		_ALIGN(at, 8);

		/* Add `_width` bit array field's size */
		at += 64;

		/* Align for `_height` field */
		_ALIGN(at, 8);

		/* Add `_height` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DToArrayAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DToArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_wOffset` field */
		_ALIGN(at, 8);

		/* Add `_wOffset` bit array field's size */
		at += 64;

		/* Align for `_hOffset` field */
		_ALIGN(at, 8);

		/* Add `_hOffset` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_spitch` field */
		_ALIGN(at, 8);

		/* Add `_spitch` bit array field's size */
		at += 64;

		/* Align for `_width` field */
		_ALIGN(at, 8);

		/* Add `_width` bit array field's size */
		at += 64;

		/* Align for `_height` field */
		_ALIGN(at, 8);

		/* Add `_height` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy2DToArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy3DAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_p` field */
		_ALIGN(at, 8);

		/* Add `_p` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcArray` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcArray` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPos__x` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPos__x` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPos__y` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPos__y` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPos__z` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPos__z` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstArray` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstArray` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPos__x` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPos__x` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPos__y` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPos__y` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPos__z` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPos__z` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_p__val__extent__width` field */
		_ALIGN(at, 8);

		/* Add `_p__val__extent__width` bit array field's size */
		at += 64;

		/* Align for `_p__val__extent__height` field */
		_ALIGN(at, 8);

		/* Add `_p__val__extent__height` bit array field's size */
		at += 64;

		/* Align for `_p__val__extent__depth` field */
		_ALIGN(at, 8);

		/* Add `_p__val__extent__depth` bit array field's size */
		at += 64;

		/* Align for `_p__val__kind` field */
		_ALIGN(at, 8);

		/* Add `_p__val__kind` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy3DAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy3DBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_p` field */
		_ALIGN(at, 8);

		/* Add `_p` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcArray` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcArray` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPos__x` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPos__x` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPos__y` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPos__y` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPos__z` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPos__z` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_p__val__srcPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_p__val__srcPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstArray` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstArray` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPos__x` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPos__x` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPos__y` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPos__y` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPos__z` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPos__z` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_p__val__dstPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_p__val__dstPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_p__val__extent__width` field */
		_ALIGN(at, 8);

		/* Add `_p__val__extent__width` bit array field's size */
		at += 64;

		/* Align for `_p__val__extent__height` field */
		_ALIGN(at, 8);

		/* Add `_p__val__extent__height` bit array field's size */
		at += 64;

		/* Align for `_p__val__extent__depth` field */
		_ALIGN(at, 8);

		/* Add `_p__val__extent__depth` bit array field's size */
		at += 64;

		/* Align for `_p__val__kind` field */
		_ALIGN(at, 8);

		/* Add `_p__val__kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpy3DEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyAtoABegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dstArray` field */
		_ALIGN(at, 8);

		/* Add `_dstArray` bit array field's size */
		at += 64;

		/* Align for `_dstOffset` field */
		_ALIGN(at, 8);

		/* Add `_dstOffset` bit array field's size */
		at += 64;

		/* Align for `_srcArray` field */
		_ALIGN(at, 8);

		/* Add `_srcArray` bit array field's size */
		at += 64;

		/* Align for `_srcOffset` field */
		_ALIGN(at, 8);

		/* Add `_srcOffset` bit array field's size */
		at += 64;

		/* Align for `_ByteCount` field */
		_ALIGN(at, 8);

		/* Add `_ByteCount` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyAtoAEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyAtoDBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dstDevice` field */
		_ALIGN(at, 8);

		/* Add `_dstDevice` bit array field's size */
		at += 64;

		/* Align for `_srcArray` field */
		_ALIGN(at, 8);

		/* Add `_srcArray` bit array field's size */
		at += 64;

		/* Align for `_srcOffset` field */
		_ALIGN(at, 8);

		/* Add `_srcOffset` bit array field's size */
		at += 64;

		/* Align for `_ByteCount` field */
		_ALIGN(at, 8);

		/* Add `_ByteCount` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyAtoDEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyAtoHAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dstHost` field */
		_ALIGN(at, 8);

		/* Add `_dstHost` bit array field's size */
		at += 64;

		/* Align for `_srcArray` field */
		_ALIGN(at, 8);

		/* Add `_srcArray` bit array field's size */
		at += 64;

		/* Align for `_srcOffset` field */
		_ALIGN(at, 8);

		/* Add `_srcOffset` bit array field's size */
		at += 64;

		/* Align for `_ByteCount` field */
		_ALIGN(at, 8);

		/* Add `_ByteCount` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyAtoHAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyAtoHBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_srcArray` field */
		_ALIGN(at, 8);

		/* Add `_srcArray` bit array field's size */
		at += 64;

		/* Align for `_srcOffset` field */
		_ALIGN(at, 8);

		/* Add `_srcOffset` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyAtoHEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyDtoABegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dstArray` field */
		_ALIGN(at, 8);

		/* Add `_dstArray` bit array field's size */
		at += 64;

		/* Align for `_dstOffset` field */
		_ALIGN(at, 8);

		/* Add `_dstOffset` bit array field's size */
		at += 64;

		/* Align for `_srcDevice` field */
		_ALIGN(at, 8);

		/* Add `_srcDevice` bit array field's size */
		at += 64;

		/* Align for `_ByteCount` field */
		_ALIGN(at, 8);

		/* Add `_ByteCount` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyDtoAEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyDtoDAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyDtoDAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyDtoDBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyDtoDEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyDtoHAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyDtoHAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyDtoHBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyDtoHEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyFromArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_srcArray` field */
		_ALIGN(at, 8);

		/* Add `_srcArray` bit array field's size */
		at += 64;

		/* Align for `_wOffset` field */
		_ALIGN(at, 8);

		/* Add `_wOffset` bit array field's size */
		at += 64;

		/* Align for `_hOffset` field */
		_ALIGN(at, 8);

		/* Add `_hOffset` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyFromArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyFromSymbolAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_offset` field */
		_ALIGN(at, 8);

		/* Add `_offset` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyFromSymbolAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyFromSymbolBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_offset` field */
		_ALIGN(at, 8);

		/* Add `_offset` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyFromSymbolEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyHtoAAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dstArray` field */
		_ALIGN(at, 8);

		/* Add `_dstArray` bit array field's size */
		at += 64;

		/* Align for `_dstOffset` field */
		_ALIGN(at, 8);

		/* Add `_dstOffset` bit array field's size */
		at += 64;

		/* Align for `_srcHost` field */
		_ALIGN(at, 8);

		/* Add `_srcHost` bit array field's size */
		at += 64;

		/* Align for `_ByteCount` field */
		_ALIGN(at, 8);

		/* Add `_ByteCount` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyHtoAAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyHtoABegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dstArray` field */
		_ALIGN(at, 8);

		/* Add `_dstArray` bit array field's size */
		at += 64;

		/* Align for `_dstOffset` field */
		_ALIGN(at, 8);

		/* Add `_dstOffset` bit array field's size */
		at += 64;

		/* Align for `_srcHost` field */
		_ALIGN(at, 8);

		/* Add `_srcHost` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyHtoAEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyHtoDAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyHtoDAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyHtoDBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyHtoDEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyParam2DAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pCopy` field */
		_ALIGN(at, 8);

		/* Add `_pCopy` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcXInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcXInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcY` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcY` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcMemoryType` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcMemoryType` bit array field's size */
		at += 32;

		/* Align for `_pCopy__val__srcHost` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcHost` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcDevice` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcDevice` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcArray` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcArray` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcPitch` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcPitch` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstXInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstXInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstY` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstY` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstMemoryType` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstMemoryType` bit array field's size */
		at += 32;

		/* Align for `_pCopy__val__dstHost` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstHost` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstDevice` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstDevice` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstArray` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstArray` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstPitch` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstPitch` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__WidthInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__WidthInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__Height` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__Height` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyParam2DAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyParam2DBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pCopy` field */
		_ALIGN(at, 8);

		/* Add `_pCopy` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcXInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcXInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcY` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcY` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcMemoryType` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcMemoryType` bit array field's size */
		at += 32;

		/* Align for `_pCopy__val__srcHost` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcHost` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcDevice` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcDevice` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcArray` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcArray` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__srcPitch` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__srcPitch` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstXInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstXInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstY` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstY` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstMemoryType` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstMemoryType` bit array field's size */
		at += 32;

		/* Align for `_pCopy__val__dstHost` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstHost` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstDevice` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstDevice` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstArray` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstArray` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__dstPitch` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__dstPitch` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__WidthInBytes` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__WidthInBytes` bit array field's size */
		at += 64;

		/* Align for `_pCopy__val__Height` field */
		_ALIGN(at, 8);

		/* Add `_pCopy__val__Height` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyParam2DEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyPeerAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_dstDeviceId` field */
		_ALIGN(at, 8);

		/* Add `_dstDeviceId` bit array field's size */
		at += 32;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_srcDevice` field */
		_ALIGN(at, 8);

		/* Add `_srcDevice` bit array field's size */
		at += 32;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyPeerAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyPeerBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_dstDeviceId` field */
		_ALIGN(at, 8);

		/* Add `_dstDeviceId` bit array field's size */
		at += 32;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_srcDeviceId` field */
		_ALIGN(at, 8);

		/* Add `_srcDeviceId` bit array field's size */
		at += 32;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyPeerEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyToArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_wOffset` field */
		_ALIGN(at, 8);

		/* Add `_wOffset` bit array field's size */
		at += 64;

		/* Align for `_hOffset` field */
		_ALIGN(at, 8);

		/* Add `_hOffset` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyToArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyToSymbolAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_offset` field */
		_ALIGN(at, 8);

		/* Add `_offset` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyToSymbolAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyToSymbolBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_offset` field */
		_ALIGN(at, 8);

		/* Add `_offset` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyToSymbolEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyWithStreamBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_kind` field */
		_ALIGN(at, 8);

		/* Add `_kind` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemcpyWithStreamEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemset2DAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_pitch` field */
		_ALIGN(at, 8);

		/* Add `_pitch` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 32;

		/* Align for `_width` field */
		_ALIGN(at, 8);

		/* Add `_width` bit array field's size */
		at += 64;

		/* Align for `_height` field */
		_ALIGN(at, 8);

		/* Add `_height` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemset2DAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemset2DBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_pitch` field */
		_ALIGN(at, 8);

		/* Add `_pitch` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 32;

		/* Align for `_width` field */
		_ALIGN(at, 8);

		/* Add `_width` bit array field's size */
		at += 64;

		/* Align for `_height` field */
		_ALIGN(at, 8);

		/* Add `_height` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemset2DEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemset3DAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pitchedDevPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_pitchedDevPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_pitchedDevPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pitchedDevPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_pitchedDevPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_pitchedDevPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_pitchedDevPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_pitchedDevPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 32;

		/* Align for `_extent__width` field */
		_ALIGN(at, 8);

		/* Add `_extent__width` bit array field's size */
		at += 64;

		/* Align for `_extent__height` field */
		_ALIGN(at, 8);

		/* Add `_extent__height` bit array field's size */
		at += 64;

		/* Align for `_extent__depth` field */
		_ALIGN(at, 8);

		/* Add `_extent__depth` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemset3DAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemset3DBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pitchedDevPtr__ptr` field */
		_ALIGN(at, 8);

		/* Add `_pitchedDevPtr__ptr` bit array field's size */
		at += 64;

		/* Align for `_pitchedDevPtr__pitch` field */
		_ALIGN(at, 8);

		/* Add `_pitchedDevPtr__pitch` bit array field's size */
		at += 64;

		/* Align for `_pitchedDevPtr__xsize` field */
		_ALIGN(at, 8);

		/* Add `_pitchedDevPtr__xsize` bit array field's size */
		at += 64;

		/* Align for `_pitchedDevPtr__ysize` field */
		_ALIGN(at, 8);

		/* Add `_pitchedDevPtr__ysize` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 32;

		/* Align for `_extent__width` field */
		_ALIGN(at, 8);

		/* Add `_extent__width` bit array field's size */
		at += 64;

		/* Align for `_extent__height` field */
		_ALIGN(at, 8);

		/* Add `_extent__height` bit array field's size */
		at += 64;

		/* Align for `_extent__depth` field */
		_ALIGN(at, 8);

		/* Add `_extent__depth` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemset3DEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 32;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 32;

		/* Align for `_sizeBytes` field */
		_ALIGN(at, 8);

		/* Add `_sizeBytes` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetD16AsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dest` field */
		_ALIGN(at, 8);

		/* Add `_dest` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 16;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetD16AsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetD16Begin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dest` field */
		_ALIGN(at, 8);

		/* Add `_dest` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 16;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetD16End(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetD32AsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 32;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetD32AsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetD32Begin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dest` field */
		_ALIGN(at, 8);

		/* Add `_dest` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 32;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetD32End(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetD8AsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dest` field */
		_ALIGN(at, 8);

		/* Add `_dest` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 8;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetD8AsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetD8Begin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dest` field */
		_ALIGN(at, 8);

		/* Add `_dest` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 8;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetD8End(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMemsetEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMipmappedArrayCreateBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pHandle` field */
		_ALIGN(at, 8);

		/* Add `_pHandle` bit array field's size */
		at += 64;

		/* Align for `_pHandle__val` field */
		_ALIGN(at, 8);

		/* Add `_pHandle__val` bit array field's size */
		at += 64;

		/* Align for `_pMipmappedArrayDesc` field */
		_ALIGN(at, 8);

		/* Add `_pMipmappedArrayDesc` bit array field's size */
		at += 64;

		/* Align for `_pMipmappedArrayDesc__val__Width` field */
		_ALIGN(at, 8);

		/* Add `_pMipmappedArrayDesc__val__Width` bit array field's size */
		at += 64;

		/* Align for `_pMipmappedArrayDesc__val__Height` field */
		_ALIGN(at, 8);

		/* Add `_pMipmappedArrayDesc__val__Height` bit array field's size */
		at += 64;

		/* Align for `_pMipmappedArrayDesc__val__Depth` field */
		_ALIGN(at, 8);

		/* Add `_pMipmappedArrayDesc__val__Depth` bit array field's size */
		at += 64;

		/* Align for `_pMipmappedArrayDesc__val__Format` field */
		_ALIGN(at, 8);

		/* Add `_pMipmappedArrayDesc__val__Format` bit array field's size */
		at += 32;

		/* Align for `_pMipmappedArrayDesc__val__NumChannels` field */
		_ALIGN(at, 8);

		/* Add `_pMipmappedArrayDesc__val__NumChannels` bit array field's size */
		at += 32;

		/* Align for `_pMipmappedArrayDesc__val__Flags` field */
		_ALIGN(at, 8);

		/* Add `_pMipmappedArrayDesc__val__Flags` bit array field's size */
		at += 32;

		/* Align for `_numMipmapLevels` field */
		_ALIGN(at, 8);

		/* Add `_numMipmapLevels` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMipmappedArrayCreateEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMipmappedArrayDestroyBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_hMipmappedArray` field */
		_ALIGN(at, 8);

		/* Add `_hMipmappedArray` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMipmappedArrayDestroyEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMipmappedArrayGetLevelBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pLevelArray` field */
		_ALIGN(at, 8);

		/* Add `_pLevelArray` bit array field's size */
		at += 64;

		/* Align for `_pLevelArray__val` field */
		_ALIGN(at, 8);

		/* Add `_pLevelArray__val` bit array field's size */
		at += 64;

		/* Align for `_hMipMappedArray` field */
		_ALIGN(at, 8);

		/* Add `_hMipMappedArray` bit array field's size */
		at += 64;

		/* Align for `_level` field */
		_ALIGN(at, 8);

		/* Add `_level` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipMipmappedArrayGetLevelEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleGetFunctionBegin(void * const vctx,
	const char * const cc__kernel_name,
	const char * const p__kname)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_function` field */
		_ALIGN(at, 8);

		/* Add `_function` bit array field's size */
		at += 64;

		/* Align for `_function__val` field */
		_ALIGN(at, 8);

		/* Add `_function__val` bit array field's size */
		at += 64;

		/* Align for `_module` field */
		_ALIGN(at, 8);

		/* Add `_module` bit array field's size */
		at += 64;

		/* Align for `_kname` field */
		_ALIGN(at, 8);

		/* Add `_kname` string field's size */
		at += _BYTES_TO_BITS(strlen(p__kname) + 1);

		/* Align for `_kname__val` field */
		_ALIGN(at, 8);

		/* Add `_kname__val` bit array field's size */
		at += 8;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleGetFunctionEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleGetGlobalBegin(void * const vctx,
	const char * const cc__kernel_name,
	const char * const p__name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dptr` field */
		_ALIGN(at, 8);

		/* Add `_dptr` bit array field's size */
		at += 64;

		/* Align for `_dptr__val` field */
		_ALIGN(at, 8);

		/* Add `_dptr__val` bit array field's size */
		at += 64;

		/* Align for `_bytes` field */
		_ALIGN(at, 8);

		/* Add `_bytes` bit array field's size */
		at += 64;

		/* Align for `_bytes__val` field */
		_ALIGN(at, 8);

		/* Add `_bytes__val` bit array field's size */
		at += 64;

		/* Align for `_hmod` field */
		_ALIGN(at, 8);

		/* Add `_hmod` bit array field's size */
		at += 64;

		/* Align for `_name` field */
		_ALIGN(at, 8);

		/* Add `_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__name) + 1);

		/* Align for `_name__val` field */
		_ALIGN(at, 8);

		/* Add `_name__val` bit array field's size */
		at += 8;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleGetGlobalEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleGetTexRefBegin(void * const vctx,
	const char * const cc__kernel_name,
	const char * const p__name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val` bit array field's size */
		at += 64;

		/* Align for `_hmod` field */
		_ALIGN(at, 8);

		/* Add `_hmod` bit array field's size */
		at += 64;

		/* Align for `_name` field */
		_ALIGN(at, 8);

		/* Add `_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__name) + 1);

		/* Align for `_name__val` field */
		_ALIGN(at, 8);

		/* Add `_name__val` bit array field's size */
		at += 8;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleGetTexRefEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleLaunchCooperativeKernelBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_f` field */
		_ALIGN(at, 8);

		/* Add `_f` bit array field's size */
		at += 64;

		/* Align for `_gridDimX` field */
		_ALIGN(at, 8);

		/* Add `_gridDimX` bit array field's size */
		at += 32;

		/* Align for `_gridDimY` field */
		_ALIGN(at, 8);

		/* Add `_gridDimY` bit array field's size */
		at += 32;

		/* Align for `_gridDimZ` field */
		_ALIGN(at, 8);

		/* Add `_gridDimZ` bit array field's size */
		at += 32;

		/* Align for `_blockDimX` field */
		_ALIGN(at, 8);

		/* Add `_blockDimX` bit array field's size */
		at += 32;

		/* Align for `_blockDimY` field */
		_ALIGN(at, 8);

		/* Add `_blockDimY` bit array field's size */
		at += 32;

		/* Align for `_blockDimZ` field */
		_ALIGN(at, 8);

		/* Add `_blockDimZ` bit array field's size */
		at += 32;

		/* Align for `_sharedMemBytes` field */
		_ALIGN(at, 8);

		/* Add `_sharedMemBytes` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_kernelParams` field */
		_ALIGN(at, 8);

		/* Add `_kernelParams` bit array field's size */
		at += 64;

		/* Align for `_kernelParams__val` field */
		_ALIGN(at, 8);

		/* Add `_kernelParams__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleLaunchCooperativeKernelEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleLaunchCooperativeKernelMultiDeviceBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_launchParamsList` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList` bit array field's size */
		at += 64;

		/* Align for `_launchParamsList__val__function` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__function` bit array field's size */
		at += 64;

		/* Align for `_launchParamsList__val__gridDimX` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__gridDimX` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__gridDimY` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__gridDimY` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__gridDimZ` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__gridDimZ` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__blockDimX` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__blockDimX` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__blockDimY` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__blockDimY` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__blockDimZ` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__blockDimZ` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__sharedMemBytes` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__sharedMemBytes` bit array field's size */
		at += 32;

		/* Align for `_launchParamsList__val__hStream` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__hStream` bit array field's size */
		at += 64;

		/* Align for `_launchParamsList__val__kernelParams` field */
		_ALIGN(at, 8);

		/* Add `_launchParamsList__val__kernelParams` bit array field's size */
		at += 64;

		/* Align for `_numDevices` field */
		_ALIGN(at, 8);

		/* Add `_numDevices` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleLaunchCooperativeKernelMultiDeviceEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleLaunchKernelBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_f` field */
		_ALIGN(at, 8);

		/* Add `_f` bit array field's size */
		at += 64;

		/* Align for `_gridDimX` field */
		_ALIGN(at, 8);

		/* Add `_gridDimX` bit array field's size */
		at += 32;

		/* Align for `_gridDimY` field */
		_ALIGN(at, 8);

		/* Add `_gridDimY` bit array field's size */
		at += 32;

		/* Align for `_gridDimZ` field */
		_ALIGN(at, 8);

		/* Add `_gridDimZ` bit array field's size */
		at += 32;

		/* Align for `_blockDimX` field */
		_ALIGN(at, 8);

		/* Add `_blockDimX` bit array field's size */
		at += 32;

		/* Align for `_blockDimY` field */
		_ALIGN(at, 8);

		/* Add `_blockDimY` bit array field's size */
		at += 32;

		/* Align for `_blockDimZ` field */
		_ALIGN(at, 8);

		/* Add `_blockDimZ` bit array field's size */
		at += 32;

		/* Align for `_sharedMemBytes` field */
		_ALIGN(at, 8);

		/* Add `_sharedMemBytes` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_kernelParams` field */
		_ALIGN(at, 8);

		/* Add `_kernelParams` bit array field's size */
		at += 64;

		/* Align for `_kernelParams__val` field */
		_ALIGN(at, 8);

		/* Add `_kernelParams__val` bit array field's size */
		at += 64;

		/* Align for `_extra` field */
		_ALIGN(at, 8);

		/* Add `_extra` bit array field's size */
		at += 64;

		/* Align for `_extra__val` field */
		_ALIGN(at, 8);

		/* Add `_extra__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleLaunchKernelEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleLoadBegin(void * const vctx,
	const char * const cc__kernel_name,
	const char * const p__fname)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_module` field */
		_ALIGN(at, 8);

		/* Add `_module` bit array field's size */
		at += 64;

		/* Align for `_module__val` field */
		_ALIGN(at, 8);

		/* Add `_module__val` bit array field's size */
		at += 64;

		/* Align for `_fname` field */
		_ALIGN(at, 8);

		/* Add `_fname` string field's size */
		at += _BYTES_TO_BITS(strlen(p__fname) + 1);

		/* Align for `_fname__val` field */
		_ALIGN(at, 8);

		/* Add `_fname__val` bit array field's size */
		at += 8;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleLoadDataBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_module` field */
		_ALIGN(at, 8);

		/* Add `_module` bit array field's size */
		at += 64;

		/* Align for `_module__val` field */
		_ALIGN(at, 8);

		/* Add `_module__val` bit array field's size */
		at += 64;

		/* Align for `_image` field */
		_ALIGN(at, 8);

		/* Add `_image` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleLoadDataEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleLoadDataExBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_module` field */
		_ALIGN(at, 8);

		/* Add `_module` bit array field's size */
		at += 64;

		/* Align for `_module__val` field */
		_ALIGN(at, 8);

		/* Add `_module__val` bit array field's size */
		at += 64;

		/* Align for `_image` field */
		_ALIGN(at, 8);

		/* Add `_image` bit array field's size */
		at += 64;

		/* Align for `_numOptions` field */
		_ALIGN(at, 8);

		/* Add `_numOptions` bit array field's size */
		at += 32;

		/* Align for `_options` field */
		_ALIGN(at, 8);

		/* Add `_options` bit array field's size */
		at += 64;

		/* Align for `_options__val` field */
		_ALIGN(at, 8);

		/* Add `_options__val` bit array field's size */
		at += 32;

		/* Align for `_optionsValues` field */
		_ALIGN(at, 8);

		/* Add `_optionsValues` bit array field's size */
		at += 64;

		/* Align for `_optionsValues__val` field */
		_ALIGN(at, 8);

		/* Add `_optionsValues__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleLoadDataExEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleLoadEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_numBlocks` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks` bit array field's size */
		at += 64;

		/* Align for `_numBlocks__val` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks__val` bit array field's size */
		at += 32;

		/* Align for `_f` field */
		_ALIGN(at, 8);

		/* Add `_f` bit array field's size */
		at += 64;

		/* Align for `_blockSize` field */
		_ALIGN(at, 8);

		/* Add `_blockSize` bit array field's size */
		at += 32;

		/* Align for `_dynSharedMemPerBlk` field */
		_ALIGN(at, 8);

		/* Add `_dynSharedMemPerBlk` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_numBlocks` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks` bit array field's size */
		at += 64;

		/* Align for `_numBlocks__val` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks__val` bit array field's size */
		at += 32;

		/* Align for `_f` field */
		_ALIGN(at, 8);

		/* Add `_f` bit array field's size */
		at += 64;

		/* Align for `_blockSize` field */
		_ALIGN(at, 8);

		/* Add `_blockSize` bit array field's size */
		at += 32;

		/* Align for `_dynSharedMemPerBlk` field */
		_ALIGN(at, 8);

		/* Add `_dynSharedMemPerBlk` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleOccupancyMaxPotentialBlockSizeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_gridSize` field */
		_ALIGN(at, 8);

		/* Add `_gridSize` bit array field's size */
		at += 64;

		/* Align for `_gridSize__val` field */
		_ALIGN(at, 8);

		/* Add `_gridSize__val` bit array field's size */
		at += 32;

		/* Align for `_blockSize` field */
		_ALIGN(at, 8);

		/* Add `_blockSize` bit array field's size */
		at += 64;

		/* Align for `_blockSize__val` field */
		_ALIGN(at, 8);

		/* Add `_blockSize__val` bit array field's size */
		at += 32;

		/* Align for `_f` field */
		_ALIGN(at, 8);

		/* Add `_f` bit array field's size */
		at += 64;

		/* Align for `_dynSharedMemPerBlk` field */
		_ALIGN(at, 8);

		/* Add `_dynSharedMemPerBlk` bit array field's size */
		at += 64;

		/* Align for `_blockSizeLimit` field */
		_ALIGN(at, 8);

		/* Add `_blockSizeLimit` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleOccupancyMaxPotentialBlockSizeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleOccupancyMaxPotentialBlockSizeWithFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_gridSize` field */
		_ALIGN(at, 8);

		/* Add `_gridSize` bit array field's size */
		at += 64;

		/* Align for `_gridSize__val` field */
		_ALIGN(at, 8);

		/* Add `_gridSize__val` bit array field's size */
		at += 32;

		/* Align for `_blockSize` field */
		_ALIGN(at, 8);

		/* Add `_blockSize` bit array field's size */
		at += 64;

		/* Align for `_blockSize__val` field */
		_ALIGN(at, 8);

		/* Add `_blockSize__val` bit array field's size */
		at += 32;

		/* Align for `_f` field */
		_ALIGN(at, 8);

		/* Add `_f` bit array field's size */
		at += 64;

		/* Align for `_dynSharedMemPerBlk` field */
		_ALIGN(at, 8);

		/* Add `_dynSharedMemPerBlk` bit array field's size */
		at += 64;

		/* Align for `_blockSizeLimit` field */
		_ALIGN(at, 8);

		/* Add `_blockSizeLimit` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleOccupancyMaxPotentialBlockSizeWithFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleUnloadBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_module` field */
		_ALIGN(at, 8);

		/* Add `_module` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipModuleUnloadEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_numBlocks` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks` bit array field's size */
		at += 64;

		/* Align for `_numBlocks__val` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks__val` bit array field's size */
		at += 32;

		/* Align for `_f` field */
		_ALIGN(at, 8);

		/* Add `_f` bit array field's size */
		at += 64;

		/* Align for `_blockSize` field */
		_ALIGN(at, 8);

		/* Add `_blockSize` bit array field's size */
		at += 32;

		/* Align for `_dynamicSMemSize` field */
		_ALIGN(at, 8);

		/* Add `_dynamicSMemSize` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_numBlocks` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks` bit array field's size */
		at += 64;

		/* Align for `_numBlocks__val` field */
		_ALIGN(at, 8);

		/* Add `_numBlocks__val` bit array field's size */
		at += 32;

		/* Align for `_f` field */
		_ALIGN(at, 8);

		/* Add `_f` bit array field's size */
		at += 64;

		/* Align for `_blockSize` field */
		_ALIGN(at, 8);

		/* Add `_blockSize` bit array field's size */
		at += 32;

		/* Align for `_dynamicSMemSize` field */
		_ALIGN(at, 8);

		/* Add `_dynamicSMemSize` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipOccupancyMaxPotentialBlockSizeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_gridSize` field */
		_ALIGN(at, 8);

		/* Add `_gridSize` bit array field's size */
		at += 64;

		/* Align for `_gridSize__val` field */
		_ALIGN(at, 8);

		/* Add `_gridSize__val` bit array field's size */
		at += 32;

		/* Align for `_blockSize` field */
		_ALIGN(at, 8);

		/* Add `_blockSize` bit array field's size */
		at += 64;

		/* Align for `_blockSize__val` field */
		_ALIGN(at, 8);

		/* Add `_blockSize__val` bit array field's size */
		at += 32;

		/* Align for `_f` field */
		_ALIGN(at, 8);

		/* Add `_f` bit array field's size */
		at += 64;

		/* Align for `_dynSharedMemPerBlk` field */
		_ALIGN(at, 8);

		/* Add `_dynSharedMemPerBlk` bit array field's size */
		at += 64;

		/* Align for `_blockSizeLimit` field */
		_ALIGN(at, 8);

		/* Add `_blockSizeLimit` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipOccupancyMaxPotentialBlockSizeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipPeekAtLastErrorBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipPeekAtLastErrorEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipPointerGetAttributeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipPointerGetAttributeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipPointerGetAttributesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_attributes` field */
		_ALIGN(at, 8);

		/* Add `_attributes` bit array field's size */
		at += 64;

		/* Align for `_attributes__val__type` field */
		_ALIGN(at, 8);

		/* Add `_attributes__val__type` bit array field's size */
		at += 32;

		/* Align for `_attributes__val__device` field */
		_ALIGN(at, 8);

		/* Add `_attributes__val__device` bit array field's size */
		at += 32;

		/* Align for `_attributes__val__devicePointer` field */
		_ALIGN(at, 8);

		/* Add `_attributes__val__devicePointer` bit array field's size */
		at += 64;

		/* Align for `_attributes__val__hostPointer` field */
		_ALIGN(at, 8);

		/* Add `_attributes__val__hostPointer` bit array field's size */
		at += 64;

		/* Align for `_attributes__val__isManaged` field */
		_ALIGN(at, 8);

		/* Add `_attributes__val__isManaged` bit array field's size */
		at += 32;

		/* Align for `_attributes__val__allocationFlags` field */
		_ALIGN(at, 8);

		/* Add `_attributes__val__allocationFlags` bit array field's size */
		at += 32;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipPointerGetAttributesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipPointerSetAttributeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipPointerSetAttributeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipProfilerStartBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipProfilerStartEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipProfilerStopBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipProfilerStopEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipRuntimeGetVersionBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_runtimeVersion` field */
		_ALIGN(at, 8);

		/* Add `_runtimeVersion` bit array field's size */
		at += 64;

		/* Align for `_runtimeVersion__val` field */
		_ALIGN(at, 8);

		/* Add `_runtimeVersion__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipRuntimeGetVersionEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipSetDeviceBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_deviceId` field */
		_ALIGN(at, 8);

		/* Add `_deviceId` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipSetDeviceEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipSetDeviceFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipSetDeviceFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipSetValidDevicesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_device_arr` field */
		_ALIGN(at, 8);

		/* Add `_device_arr` bit array field's size */
		at += 64;

		/* Align for `_device_arr__val` field */
		_ALIGN(at, 8);

		/* Add `_device_arr__val` bit array field's size */
		at += 32;

		/* Align for `_len` field */
		_ALIGN(at, 8);

		/* Add `_len` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipSetValidDevicesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipSetupArgumentBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_arg` field */
		_ALIGN(at, 8);

		/* Add `_arg` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_offset` field */
		_ALIGN(at, 8);

		/* Add `_offset` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipSetupArgumentEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipSignalExternalSemaphoresAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_extSemArray` field */
		_ALIGN(at, 8);

		/* Add `_extSemArray` bit array field's size */
		at += 64;

		/* Align for `_extSemArray__val` field */
		_ALIGN(at, 8);

		/* Add `_extSemArray__val` bit array field's size */
		at += 64;

		/* Align for `_paramsArray` field */
		_ALIGN(at, 8);

		/* Add `_paramsArray` bit array field's size */
		at += 64;

		/* Align for `_paramsArray__val__flags` field */
		_ALIGN(at, 8);

		/* Add `_paramsArray__val__flags` bit array field's size */
		at += 32;

		/* Align for `_paramsArray__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_paramsArray__val__reserved` bit array field's size */
		at += 64;

		/* Align for `_numExtSems` field */
		_ALIGN(at, 8);

		/* Add `_numExtSems` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipSignalExternalSemaphoresAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamAddCallbackBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_userData` field */
		_ALIGN(at, 8);

		/* Add `_userData` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamAddCallbackEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamAttachMemAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;

		/* Align for `_length` field */
		_ALIGN(at, 8);

		/* Add `_length` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamAttachMemAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamBeginCaptureBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_mode` field */
		_ALIGN(at, 8);

		/* Add `_mode` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamBeginCaptureEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamBeginCaptureToGraphBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_graph` field */
		_ALIGN(at, 8);

		/* Add `_graph` bit array field's size */
		at += 64;

		/* Align for `_dependencies` field */
		_ALIGN(at, 8);

		/* Add `_dependencies` bit array field's size */
		at += 64;

		/* Align for `_dependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_dependencies__val` bit array field's size */
		at += 64;

		/* Align for `_dependencyData` field */
		_ALIGN(at, 8);

		/* Add `_dependencyData` bit array field's size */
		at += 64;

		/* Align for `_dependencyData__val__from_port` field */
		_ALIGN(at, 8);

		/* Add `_dependencyData__val__from_port` bit array field's size */
		at += 8;

		/* Align for `_dependencyData__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_dependencyData__val__reserved` bit array field's size */
		at += 64;

		/* Align for `_dependencyData__val__to_port` field */
		_ALIGN(at, 8);

		/* Add `_dependencyData__val__to_port` bit array field's size */
		at += 8;

		/* Align for `_dependencyData__val__type` field */
		_ALIGN(at, 8);

		/* Add `_dependencyData__val__type` bit array field's size */
		at += 8;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_mode` field */
		_ALIGN(at, 8);

		/* Add `_mode` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamBeginCaptureToGraphEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamCreateBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_stream__val` field */
		_ALIGN(at, 8);

		/* Add `_stream__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamCreateEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamCreateWithFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_stream__val` field */
		_ALIGN(at, 8);

		/* Add `_stream__val` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamCreateWithFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamCreateWithPriorityBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_stream__val` field */
		_ALIGN(at, 8);

		/* Add `_stream__val` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;

		/* Align for `_priority` field */
		_ALIGN(at, 8);

		/* Add `_priority` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamCreateWithPriorityEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamDestroyBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamDestroyEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamEndCaptureBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_pGraph` field */
		_ALIGN(at, 8);

		/* Add `_pGraph` bit array field's size */
		at += 64;

		/* Align for `_pGraph__val` field */
		_ALIGN(at, 8);

		/* Add `_pGraph__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamEndCaptureEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamGetCaptureInfoBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_pCaptureStatus` field */
		_ALIGN(at, 8);

		/* Add `_pCaptureStatus` bit array field's size */
		at += 64;

		/* Align for `_pCaptureStatus__val` field */
		_ALIGN(at, 8);

		/* Add `_pCaptureStatus__val` bit array field's size */
		at += 32;

		/* Align for `_pId` field */
		_ALIGN(at, 8);

		/* Add `_pId` bit array field's size */
		at += 64;

		/* Align for `_pId__val` field */
		_ALIGN(at, 8);

		/* Add `_pId__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamGetCaptureInfoEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamGetCaptureInfo_v2Begin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_captureStatus_out` field */
		_ALIGN(at, 8);

		/* Add `_captureStatus_out` bit array field's size */
		at += 64;

		/* Align for `_captureStatus_out__val` field */
		_ALIGN(at, 8);

		/* Add `_captureStatus_out__val` bit array field's size */
		at += 32;

		/* Align for `_id_out` field */
		_ALIGN(at, 8);

		/* Add `_id_out` bit array field's size */
		at += 64;

		/* Align for `_id_out__val` field */
		_ALIGN(at, 8);

		/* Add `_id_out__val` bit array field's size */
		at += 64;

		/* Align for `_graph_out` field */
		_ALIGN(at, 8);

		/* Add `_graph_out` bit array field's size */
		at += 64;

		/* Align for `_graph_out__val` field */
		_ALIGN(at, 8);

		/* Add `_graph_out__val` bit array field's size */
		at += 64;

		/* Align for `_dependencies_out` field */
		_ALIGN(at, 8);

		/* Add `_dependencies_out` bit array field's size */
		at += 64;

		/* Align for `_dependencies_out__val` field */
		_ALIGN(at, 8);

		/* Add `_dependencies_out__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies_out` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies_out` bit array field's size */
		at += 64;

		/* Align for `_numDependencies_out__val` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies_out__val` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamGetCaptureInfo_v2End(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamGetDeviceBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_device` field */
		_ALIGN(at, 8);

		/* Add `_device` bit array field's size */
		at += 64;

		/* Align for `_device__val` field */
		_ALIGN(at, 8);

		/* Add `_device__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamGetDeviceEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamGetFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;

		/* Align for `_flags__val` field */
		_ALIGN(at, 8);

		/* Add `_flags__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamGetFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamGetPriorityBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_priority` field */
		_ALIGN(at, 8);

		/* Add `_priority` bit array field's size */
		at += 64;

		/* Align for `_priority__val` field */
		_ALIGN(at, 8);

		/* Add `_priority__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamGetPriorityEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamIsCapturingBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_pCaptureStatus` field */
		_ALIGN(at, 8);

		/* Add `_pCaptureStatus` bit array field's size */
		at += 64;

		/* Align for `_pCaptureStatus__val` field */
		_ALIGN(at, 8);

		/* Add `_pCaptureStatus__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamIsCapturingEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamQueryBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamQueryEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamSynchronizeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamSynchronizeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamUpdateCaptureDependenciesBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_dependencies` field */
		_ALIGN(at, 8);

		/* Add `_dependencies` bit array field's size */
		at += 64;

		/* Align for `_dependencies__val` field */
		_ALIGN(at, 8);

		/* Add `_dependencies__val` bit array field's size */
		at += 64;

		/* Align for `_numDependencies` field */
		_ALIGN(at, 8);

		/* Add `_numDependencies` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamUpdateCaptureDependenciesEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamWaitEventBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_event` field */
		_ALIGN(at, 8);

		/* Add `_event` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamWaitEventEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamWaitValue32Begin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;

		/* Align for `_mask` field */
		_ALIGN(at, 8);

		/* Add `_mask` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamWaitValue32End(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamWaitValue64Begin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;

		/* Align for `_mask` field */
		_ALIGN(at, 8);

		/* Add `_mask` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamWaitValue64End(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamWriteValue32Begin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamWriteValue32End(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamWriteValue64Begin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipStreamWriteValue64End(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetAddressBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dev_ptr` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr` bit array field's size */
		at += 64;

		/* Align for `_dev_ptr__val` field */
		_ALIGN(at, 8);

		/* Add `_dev_ptr__val` bit array field's size */
		at += 64;

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetAddressEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pArray` field */
		_ALIGN(at, 8);

		/* Add `_pArray` bit array field's size */
		at += 64;

		/* Align for `_pArray__val` field */
		_ALIGN(at, 8);

		/* Add `_pArray__val` bit array field's size */
		at += 64;

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetBorderColorBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pBorderColor` field */
		_ALIGN(at, 8);

		/* Add `_pBorderColor` bit array field's size */
		at += 64;

		/* Align for `_pBorderColor__val` field */
		_ALIGN(at, 8);

		/* Add `_pBorderColor__val` bit array field's size */
		at += 32;

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetBorderColorEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pFlags` field */
		_ALIGN(at, 8);

		/* Add `_pFlags` bit array field's size */
		at += 64;

		/* Align for `_pFlags__val` field */
		_ALIGN(at, 8);

		/* Add `_pFlags__val` bit array field's size */
		at += 32;

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetFormatBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pFormat` field */
		_ALIGN(at, 8);

		/* Add `_pFormat` bit array field's size */
		at += 64;

		/* Align for `_pFormat__val` field */
		_ALIGN(at, 8);

		/* Add `_pFormat__val` bit array field's size */
		at += 32;

		/* Align for `_pNumChannels` field */
		_ALIGN(at, 8);

		/* Add `_pNumChannels` bit array field's size */
		at += 64;

		/* Align for `_pNumChannels__val` field */
		_ALIGN(at, 8);

		/* Add `_pNumChannels__val` bit array field's size */
		at += 32;

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetFormatEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetMaxAnisotropyBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pmaxAnsio` field */
		_ALIGN(at, 8);

		/* Add `_pmaxAnsio` bit array field's size */
		at += 64;

		/* Align for `_pmaxAnsio__val` field */
		_ALIGN(at, 8);

		/* Add `_pmaxAnsio__val` bit array field's size */
		at += 32;

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetMaxAnisotropyEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetMipMappedArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pArray` field */
		_ALIGN(at, 8);

		/* Add `_pArray` bit array field's size */
		at += 64;

		/* Align for `_pArray__val` field */
		_ALIGN(at, 8);

		/* Add `_pArray__val` bit array field's size */
		at += 64;

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetMipMappedArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetMipmapLevelBiasBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pbias` field */
		_ALIGN(at, 8);

		/* Add `_pbias` bit array field's size */
		at += 64;

		/* Align for `_pbias__val` field */
		_ALIGN(at, 8);

		/* Add `_pbias__val` bit array field's size */
		at += 32;

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetMipmapLevelBiasEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetMipmapLevelClampBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pminMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_pminMipmapLevelClamp` bit array field's size */
		at += 64;

		/* Align for `_pminMipmapLevelClamp__val` field */
		_ALIGN(at, 8);

		/* Add `_pminMipmapLevelClamp__val` bit array field's size */
		at += 32;

		/* Align for `_pmaxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_pmaxMipmapLevelClamp` bit array field's size */
		at += 64;

		/* Align for `_pmaxMipmapLevelClamp__val` field */
		_ALIGN(at, 8);

		/* Add `_pmaxMipmapLevelClamp__val` bit array field's size */
		at += 32;

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefGetMipmapLevelClampEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetAddress2DBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;

		/* Align for `_desc` field */
		_ALIGN(at, 8);

		/* Add `_desc` bit array field's size */
		at += 64;

		/* Align for `_desc__val__Width` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__Width` bit array field's size */
		at += 64;

		/* Align for `_desc__val__Height` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__Height` bit array field's size */
		at += 64;

		/* Align for `_desc__val__Format` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__Format` bit array field's size */
		at += 32;

		/* Align for `_desc__val__NumChannels` field */
		_ALIGN(at, 8);

		/* Add `_desc__val__NumChannels` bit array field's size */
		at += 32;

		/* Align for `_dptr` field */
		_ALIGN(at, 8);

		/* Add `_dptr` bit array field's size */
		at += 64;

		/* Align for `_Pitch` field */
		_ALIGN(at, 8);

		/* Add `_Pitch` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetAddress2DEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetAddressBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ByteOffset` field */
		_ALIGN(at, 8);

		/* Add `_ByteOffset` bit array field's size */
		at += 64;

		/* Align for `_ByteOffset__val` field */
		_ALIGN(at, 8);

		/* Add `_ByteOffset__val` bit array field's size */
		at += 64;

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;

		/* Align for `_dptr` field */
		_ALIGN(at, 8);

		/* Add `_dptr` bit array field's size */
		at += 64;

		/* Align for `_bytes` field */
		_ALIGN(at, 8);

		/* Add `_bytes` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetAddressEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_tex` field */
		_ALIGN(at, 8);

		/* Add `_tex` bit array field's size */
		at += 64;

		/* Align for `_tex__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_tex__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_tex__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_tex__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_tex__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_tex__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_tex__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_tex__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_tex__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_tex__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_tex__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_tex__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_tex__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_tex__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_tex__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_tex__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_tex__val__format` field */
		_ALIGN(at, 8);

		/* Add `_tex__val__format` bit array field's size */
		at += 32;

		/* Align for `_array` field */
		_ALIGN(at, 8);

		/* Add `_array` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetBorderColorBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;

		/* Align for `_pBorderColor` field */
		_ALIGN(at, 8);

		/* Add `_pBorderColor` bit array field's size */
		at += 64;

		/* Align for `_pBorderColor__val` field */
		_ALIGN(at, 8);

		/* Add `_pBorderColor__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetBorderColorEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetFlagsBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;

		/* Align for `_Flags` field */
		_ALIGN(at, 8);

		/* Add `_Flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetFlagsEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetFormatBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;

		/* Align for `_fmt` field */
		_ALIGN(at, 8);

		/* Add `_fmt` bit array field's size */
		at += 32;

		/* Align for `_NumPackedComponents` field */
		_ALIGN(at, 8);

		/* Add `_NumPackedComponents` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetFormatEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetMaxAnisotropyBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;

		/* Align for `_maxAniso` field */
		_ALIGN(at, 8);

		/* Add `_maxAniso` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetMaxAnisotropyEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetMipmapLevelBiasBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;

		/* Align for `_bias` field */
		_ALIGN(at, 8);

		/* Add `_bias` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetMipmapLevelBiasEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetMipmapLevelClampBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;

		/* Align for `_minMipMapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_minMipMapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_maxMipMapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_maxMipMapLevelClamp` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetMipmapLevelClampEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetMipmappedArrayBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_texRef` field */
		_ALIGN(at, 8);

		/* Add `_texRef` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__normalized` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__normalized` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__readMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__readMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__filterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__filterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__x` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__x` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__y` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__y` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__z` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__z` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__w` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__w` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__channelDesc__f` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__channelDesc__f` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__sRGB` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__sRGB` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxAnisotropy` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxAnisotropy` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapFilterMode` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapFilterMode` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__mipmapLevelBias` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__mipmapLevelBias` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__minMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__minMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__maxMipmapLevelClamp` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__maxMipmapLevelClamp` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__textureObject` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__textureObject` bit array field's size */
		at += 64;

		/* Align for `_texRef__val__numChannels` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__numChannels` bit array field's size */
		at += 32;

		/* Align for `_texRef__val__format` field */
		_ALIGN(at, 8);

		/* Add `_texRef__val__format` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray` bit array field's size */
		at += 64;

		/* Align for `_mipmappedArray__val__data` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__data` bit array field's size */
		at += 64;

		/* Align for `_mipmappedArray__val__desc__x` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__desc__x` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray__val__desc__y` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__desc__y` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray__val__desc__z` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__desc__z` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray__val__desc__w` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__desc__w` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray__val__desc__f` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__desc__f` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray__val__type` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__type` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray__val__width` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__width` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray__val__height` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__height` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray__val__depth` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__depth` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray__val__min_mipmap_level` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__min_mipmap_level` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray__val__max_mipmap_level` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__max_mipmap_level` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray__val__flags` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__flags` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray__val__format` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__format` bit array field's size */
		at += 32;

		/* Align for `_mipmappedArray__val__num_channels` field */
		_ALIGN(at, 8);

		/* Add `_mipmappedArray__val__num_channels` bit array field's size */
		at += 32;

		/* Align for `_Flags` field */
		_ALIGN(at, 8);

		/* Add `_Flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipTexRefSetMipmappedArrayEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipThreadExchangeStreamCaptureModeBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mode` field */
		_ALIGN(at, 8);

		/* Add `_mode` bit array field's size */
		at += 64;

		/* Align for `_mode__val` field */
		_ALIGN(at, 8);

		/* Add `_mode__val` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipThreadExchangeStreamCaptureModeEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipUserObjectCreateBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_object_out` field */
		_ALIGN(at, 8);

		/* Add `_object_out` bit array field's size */
		at += 64;

		/* Align for `_object_out__val` field */
		_ALIGN(at, 8);

		/* Add `_object_out__val` bit array field's size */
		at += 64;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_destroy` field */
		_ALIGN(at, 8);

		/* Add `_destroy` bit array field's size */
		at += 64;

		/* Align for `_initialRefcount` field */
		_ALIGN(at, 8);

		/* Add `_initialRefcount` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipUserObjectCreateEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipUserObjectReleaseBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_object` field */
		_ALIGN(at, 8);

		/* Add `_object` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipUserObjectReleaseEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipUserObjectRetainBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_object` field */
		_ALIGN(at, 8);

		/* Add `_object` bit array field's size */
		at += 64;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipUserObjectRetainEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipWaitExternalSemaphoresAsyncBegin(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_extSemArray` field */
		_ALIGN(at, 8);

		/* Add `_extSemArray` bit array field's size */
		at += 64;

		/* Align for `_extSemArray__val` field */
		_ALIGN(at, 8);

		/* Add `_extSemArray__val` bit array field's size */
		at += 64;

		/* Align for `_paramsArray` field */
		_ALIGN(at, 8);

		/* Add `_paramsArray` bit array field's size */
		at += 64;

		/* Align for `_paramsArray__val__flags` field */
		_ALIGN(at, 8);

		/* Add `_paramsArray__val__flags` bit array field's size */
		at += 32;

		/* Align for `_paramsArray__val__reserved` field */
		_ALIGN(at, 8);

		/* Add `_paramsArray__val__reserved` bit array field's size */
		at += 64;

		/* Align for `_numExtSems` field */
		_ALIGN(at, 8);

		/* Add `_numExtSems` bit array field's size */
		at += 32;

		/* Align for `_stream` field */
		_ALIGN(at, 8);

		/* Add `_stream` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hip_api_hipWaitExternalSemaphoresAsyncEnd(void * const vctx,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

/* Trace (data stream type `hip_api`, event record type `__hipPopCallConfigurationBegin`) */
void barectf_hip_api_trace___hipPopCallConfigurationBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__gridDim,
	const uint32_t p__gridDim__val__x,
	const uint32_t p__gridDim__val__y,
	const uint32_t p__gridDim__val__z,
	const uint64_t p__blockDim,
	const uint32_t p__blockDim__val__x,
	const uint32_t p__blockDim__val__y,
	const uint32_t p__blockDim__val__z,
	const uint64_t p__sharedMem,
	const uint64_t p__sharedMem__val,
	const uint64_t p__stream,
	const uint64_t p__stream__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api___hipPopCallConfigurationBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api___hipPopCallConfigurationBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__gridDim, p__gridDim__val__x, p__gridDim__val__y, p__gridDim__val__z, p__blockDim, p__blockDim__val__x, p__blockDim__val__y, p__blockDim__val__z, p__sharedMem, p__sharedMem__val, p__stream, p__stream__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `__hipPopCallConfigurationEnd`) */
void barectf_hip_api_trace___hipPopCallConfigurationEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api___hipPopCallConfigurationEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api___hipPopCallConfigurationEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `__hipPushCallConfigurationBegin`) */
void barectf_hip_api_trace___hipPushCallConfigurationBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint32_t p__gridDim__x,
	const uint32_t p__gridDim__y,
	const uint32_t p__gridDim__z,
	const uint32_t p__blockDim__x,
	const uint32_t p__blockDim__y,
	const uint32_t p__blockDim__z,
	const uint64_t p__sharedMem,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api___hipPushCallConfigurationBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api___hipPushCallConfigurationBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__gridDim__x, p__gridDim__y, p__gridDim__z, p__blockDim__x, p__blockDim__y, p__blockDim__z, p__sharedMem, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `__hipPushCallConfigurationEnd`) */
void barectf_hip_api_trace___hipPushCallConfigurationEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api___hipPushCallConfigurationEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api___hipPushCallConfigurationEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipArray3DCreateBegin`) */
void barectf_hip_api_trace_hipArray3DCreateBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__array,
	const uint64_t p__array__val,
	const uint64_t p__pAllocateArray,
	const uint64_t p__pAllocateArray__val__Width,
	const uint64_t p__pAllocateArray__val__Height,
	const uint64_t p__pAllocateArray__val__Depth,
	const int32_t p__pAllocateArray__val__Format,
	const uint32_t p__pAllocateArray__val__NumChannels,
	const uint32_t p__pAllocateArray__val__Flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipArray3DCreateBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipArray3DCreateBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__array, p__array__val, p__pAllocateArray, p__pAllocateArray__val__Width, p__pAllocateArray__val__Height, p__pAllocateArray__val__Depth, p__pAllocateArray__val__Format, p__pAllocateArray__val__NumChannels, p__pAllocateArray__val__Flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipArray3DCreateEnd`) */
void barectf_hip_api_trace_hipArray3DCreateEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipArray3DCreateEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipArray3DCreateEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipArray3DGetDescriptorBegin`) */
void barectf_hip_api_trace_hipArray3DGetDescriptorBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pArrayDescriptor,
	const uint64_t p__pArrayDescriptor__val__Width,
	const uint64_t p__pArrayDescriptor__val__Height,
	const uint64_t p__pArrayDescriptor__val__Depth,
	const int32_t p__pArrayDescriptor__val__Format,
	const uint32_t p__pArrayDescriptor__val__NumChannels,
	const uint32_t p__pArrayDescriptor__val__Flags,
	const uint64_t p__array)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipArray3DGetDescriptorBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipArray3DGetDescriptorBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pArrayDescriptor, p__pArrayDescriptor__val__Width, p__pArrayDescriptor__val__Height, p__pArrayDescriptor__val__Depth, p__pArrayDescriptor__val__Format, p__pArrayDescriptor__val__NumChannels, p__pArrayDescriptor__val__Flags, p__array);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipArray3DGetDescriptorEnd`) */
void barectf_hip_api_trace_hipArray3DGetDescriptorEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipArray3DGetDescriptorEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipArray3DGetDescriptorEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipArrayCreateBegin`) */
void barectf_hip_api_trace_hipArrayCreateBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pHandle,
	const uint64_t p__pHandle__val,
	const uint64_t p__pAllocateArray,
	const uint64_t p__pAllocateArray__val__Width,
	const uint64_t p__pAllocateArray__val__Height,
	const int32_t p__pAllocateArray__val__Format,
	const uint32_t p__pAllocateArray__val__NumChannels)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipArrayCreateBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipArrayCreateBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pHandle, p__pHandle__val, p__pAllocateArray, p__pAllocateArray__val__Width, p__pAllocateArray__val__Height, p__pAllocateArray__val__Format, p__pAllocateArray__val__NumChannels);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipArrayCreateEnd`) */
void barectf_hip_api_trace_hipArrayCreateEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipArrayCreateEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipArrayCreateEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipArrayDestroyBegin`) */
void barectf_hip_api_trace_hipArrayDestroyBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__array)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipArrayDestroyBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipArrayDestroyBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__array);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipArrayDestroyEnd`) */
void barectf_hip_api_trace_hipArrayDestroyEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipArrayDestroyEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipArrayDestroyEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipArrayGetDescriptorBegin`) */
void barectf_hip_api_trace_hipArrayGetDescriptorBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pArrayDescriptor,
	const uint64_t p__pArrayDescriptor__val__Width,
	const uint64_t p__pArrayDescriptor__val__Height,
	const int32_t p__pArrayDescriptor__val__Format,
	const uint32_t p__pArrayDescriptor__val__NumChannels,
	const uint64_t p__array)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipArrayGetDescriptorBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipArrayGetDescriptorBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pArrayDescriptor, p__pArrayDescriptor__val__Width, p__pArrayDescriptor__val__Height, p__pArrayDescriptor__val__Format, p__pArrayDescriptor__val__NumChannels, p__array);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipArrayGetDescriptorEnd`) */
void barectf_hip_api_trace_hipArrayGetDescriptorEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipArrayGetDescriptorEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipArrayGetDescriptorEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipArrayGetInfoBegin`) */
void barectf_hip_api_trace_hipArrayGetInfoBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__desc,
	const int32_t p__desc__val__x,
	const int32_t p__desc__val__y,
	const int32_t p__desc__val__z,
	const int32_t p__desc__val__w,
	const int32_t p__desc__val__f,
	const uint64_t p__extent,
	const uint64_t p__extent__val__width,
	const uint64_t p__extent__val__height,
	const uint64_t p__extent__val__depth,
	const uint64_t p__flags,
	const uint32_t p__flags__val,
	const uint64_t p__array)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipArrayGetInfoBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipArrayGetInfoBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__desc, p__desc__val__x, p__desc__val__y, p__desc__val__z, p__desc__val__w, p__desc__val__f, p__extent, p__extent__val__width, p__extent__val__height, p__extent__val__depth, p__flags, p__flags__val, p__array);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipArrayGetInfoEnd`) */
void barectf_hip_api_trace_hipArrayGetInfoEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipArrayGetInfoEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipArrayGetInfoEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipChooseDeviceBegin`) */
void barectf_hip_api_trace_hipChooseDeviceBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipChooseDeviceBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipChooseDeviceBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipChooseDeviceEnd`) */
void barectf_hip_api_trace_hipChooseDeviceEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipChooseDeviceEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipChooseDeviceEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipChooseDeviceR0000Begin`) */
void barectf_hip_api_trace_hipChooseDeviceR0000Begin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__device,
	const int32_t p__device__val,
	const uint64_t p__prop,
	const uint64_t p__prop__val__name,
	const uint64_t p__prop__val__totalGlobalMem,
	const uint64_t p__prop__val__sharedMemPerBlock,
	const int32_t p__prop__val__regsPerBlock,
	const int32_t p__prop__val__warpSize,
	const int32_t p__prop__val__maxThreadsPerBlock,
	const uint64_t p__prop__val__maxThreadsDim,
	const uint64_t p__prop__val__maxGridSize,
	const int32_t p__prop__val__clockRate,
	const int32_t p__prop__val__memoryClockRate,
	const int32_t p__prop__val__memoryBusWidth,
	const uint64_t p__prop__val__totalConstMem,
	const int32_t p__prop__val__major,
	const int32_t p__prop__val__minor,
	const int32_t p__prop__val__multiProcessorCount,
	const int32_t p__prop__val__l2CacheSize,
	const int32_t p__prop__val__maxThreadsPerMultiProcessor,
	const int32_t p__prop__val__computeMode,
	const int32_t p__prop__val__clockInstructionRate,
	const uint32_t p__prop__val__arch__hasGlobalInt32Atomics,
	const uint32_t p__prop__val__arch__hasGlobalFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasSharedInt32Atomics,
	const uint32_t p__prop__val__arch__hasSharedFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasFloatAtomicAdd,
	const uint32_t p__prop__val__arch__hasGlobalInt64Atomics,
	const uint32_t p__prop__val__arch__hasSharedInt64Atomics,
	const uint32_t p__prop__val__arch__hasDoubles,
	const uint32_t p__prop__val__arch__hasWarpVote,
	const uint32_t p__prop__val__arch__hasWarpBallot,
	const uint32_t p__prop__val__arch__hasWarpShuffle,
	const uint32_t p__prop__val__arch__hasFunnelShift,
	const uint32_t p__prop__val__arch__hasThreadFenceSystem,
	const uint32_t p__prop__val__arch__hasSyncThreadsExt,
	const uint32_t p__prop__val__arch__hasSurfaceFuncs,
	const uint32_t p__prop__val__arch__has3dGrid,
	const uint32_t p__prop__val__arch__hasDynamicParallelism,
	const int32_t p__prop__val__concurrentKernels,
	const int32_t p__prop__val__pciDomainID,
	const int32_t p__prop__val__pciBusID,
	const int32_t p__prop__val__pciDeviceID,
	const uint64_t p__prop__val__maxSharedMemoryPerMultiProcessor,
	const int32_t p__prop__val__isMultiGpuBoard,
	const int32_t p__prop__val__canMapHostMemory,
	const int32_t p__prop__val__gcnArch,
	const uint64_t p__prop__val__gcnArchName,
	const int32_t p__prop__val__integrated,
	const int32_t p__prop__val__cooperativeLaunch,
	const int32_t p__prop__val__cooperativeMultiDeviceLaunch,
	const int32_t p__prop__val__maxTexture1DLinear,
	const int32_t p__prop__val__maxTexture1D,
	const uint64_t p__prop__val__maxTexture2D,
	const uint64_t p__prop__val__maxTexture3D,
	const uint64_t p__prop__val__hdpMemFlushCntl,
	const uint64_t p__prop__val__hdpRegFlushCntl,
	const uint64_t p__prop__val__memPitch,
	const uint64_t p__prop__val__textureAlignment,
	const uint64_t p__prop__val__texturePitchAlignment,
	const int32_t p__prop__val__kernelExecTimeoutEnabled,
	const int32_t p__prop__val__ECCEnabled,
	const int32_t p__prop__val__tccDriver,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedFunc,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedGridDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem,
	const int32_t p__prop__val__isLargeBar,
	const int32_t p__prop__val__asicRevision,
	const int32_t p__prop__val__managedMemory,
	const int32_t p__prop__val__directManagedMemAccessFromHost,
	const int32_t p__prop__val__concurrentManagedAccess,
	const int32_t p__prop__val__pageableMemoryAccess,
	const int32_t p__prop__val__pageableMemoryAccessUsesHostPageTables)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipChooseDeviceR0000Begin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipChooseDeviceR0000Begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__device, p__device__val, p__prop, p__prop__val__name, p__prop__val__totalGlobalMem, p__prop__val__sharedMemPerBlock, p__prop__val__regsPerBlock, p__prop__val__warpSize, p__prop__val__maxThreadsPerBlock, p__prop__val__maxThreadsDim, p__prop__val__maxGridSize, p__prop__val__clockRate, p__prop__val__memoryClockRate, p__prop__val__memoryBusWidth, p__prop__val__totalConstMem, p__prop__val__major, p__prop__val__minor, p__prop__val__multiProcessorCount, p__prop__val__l2CacheSize, p__prop__val__maxThreadsPerMultiProcessor, p__prop__val__computeMode, p__prop__val__clockInstructionRate, p__prop__val__arch__hasGlobalInt32Atomics, p__prop__val__arch__hasGlobalFloatAtomicExch, p__prop__val__arch__hasSharedInt32Atomics, p__prop__val__arch__hasSharedFloatAtomicExch, p__prop__val__arch__hasFloatAtomicAdd, p__prop__val__arch__hasGlobalInt64Atomics, p__prop__val__arch__hasSharedInt64Atomics, p__prop__val__arch__hasDoubles, p__prop__val__arch__hasWarpVote, p__prop__val__arch__hasWarpBallot, p__prop__val__arch__hasWarpShuffle, p__prop__val__arch__hasFunnelShift, p__prop__val__arch__hasThreadFenceSystem, p__prop__val__arch__hasSyncThreadsExt, p__prop__val__arch__hasSurfaceFuncs, p__prop__val__arch__has3dGrid, p__prop__val__arch__hasDynamicParallelism, p__prop__val__concurrentKernels, p__prop__val__pciDomainID, p__prop__val__pciBusID, p__prop__val__pciDeviceID, p__prop__val__maxSharedMemoryPerMultiProcessor, p__prop__val__isMultiGpuBoard, p__prop__val__canMapHostMemory, p__prop__val__gcnArch, p__prop__val__gcnArchName, p__prop__val__integrated, p__prop__val__cooperativeLaunch, p__prop__val__cooperativeMultiDeviceLaunch, p__prop__val__maxTexture1DLinear, p__prop__val__maxTexture1D, p__prop__val__maxTexture2D, p__prop__val__maxTexture3D, p__prop__val__hdpMemFlushCntl, p__prop__val__hdpRegFlushCntl, p__prop__val__memPitch, p__prop__val__textureAlignment, p__prop__val__texturePitchAlignment, p__prop__val__kernelExecTimeoutEnabled, p__prop__val__ECCEnabled, p__prop__val__tccDriver, p__prop__val__cooperativeMultiDeviceUnmatchedFunc, p__prop__val__cooperativeMultiDeviceUnmatchedGridDim, p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim, p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem, p__prop__val__isLargeBar, p__prop__val__asicRevision, p__prop__val__managedMemory, p__prop__val__directManagedMemAccessFromHost, p__prop__val__concurrentManagedAccess, p__prop__val__pageableMemoryAccess, p__prop__val__pageableMemoryAccessUsesHostPageTables);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipChooseDeviceR0000End`) */
void barectf_hip_api_trace_hipChooseDeviceR0000End(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipChooseDeviceR0000End(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipChooseDeviceR0000End(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipChooseDeviceR0600Begin`) */
void barectf_hip_api_trace_hipChooseDeviceR0600Begin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__device,
	const int32_t p__device__val,
	const uint64_t p__prop,
	const uint64_t p__prop__val__name,
	const uint64_t p__prop__val__uuid__bytes,
	const uint64_t p__prop__val__luid,
	const uint32_t p__prop__val__luidDeviceNodeMask,
	const uint64_t p__prop__val__totalGlobalMem,
	const uint64_t p__prop__val__sharedMemPerBlock,
	const int32_t p__prop__val__regsPerBlock,
	const int32_t p__prop__val__warpSize,
	const uint64_t p__prop__val__memPitch,
	const int32_t p__prop__val__maxThreadsPerBlock,
	const uint64_t p__prop__val__maxThreadsDim,
	const uint64_t p__prop__val__maxGridSize,
	const int32_t p__prop__val__clockRate,
	const uint64_t p__prop__val__totalConstMem,
	const int32_t p__prop__val__major,
	const int32_t p__prop__val__minor,
	const uint64_t p__prop__val__textureAlignment,
	const uint64_t p__prop__val__texturePitchAlignment,
	const int32_t p__prop__val__deviceOverlap,
	const int32_t p__prop__val__multiProcessorCount,
	const int32_t p__prop__val__kernelExecTimeoutEnabled,
	const int32_t p__prop__val__integrated,
	const int32_t p__prop__val__canMapHostMemory,
	const int32_t p__prop__val__computeMode,
	const int32_t p__prop__val__maxTexture1D,
	const int32_t p__prop__val__maxTexture1DMipmap,
	const int32_t p__prop__val__maxTexture1DLinear,
	const uint64_t p__prop__val__maxTexture2D,
	const uint64_t p__prop__val__maxTexture2DMipmap,
	const uint64_t p__prop__val__maxTexture2DLinear,
	const uint64_t p__prop__val__maxTexture2DGather,
	const uint64_t p__prop__val__maxTexture3D,
	const uint64_t p__prop__val__maxTexture3DAlt,
	const int32_t p__prop__val__maxTextureCubemap,
	const uint64_t p__prop__val__maxTexture1DLayered,
	const uint64_t p__prop__val__maxTexture2DLayered,
	const uint64_t p__prop__val__maxTextureCubemapLayered,
	const int32_t p__prop__val__maxSurface1D,
	const uint64_t p__prop__val__maxSurface2D,
	const uint64_t p__prop__val__maxSurface3D,
	const uint64_t p__prop__val__maxSurface1DLayered,
	const uint64_t p__prop__val__maxSurface2DLayered,
	const int32_t p__prop__val__maxSurfaceCubemap,
	const uint64_t p__prop__val__maxSurfaceCubemapLayered,
	const uint64_t p__prop__val__surfaceAlignment,
	const int32_t p__prop__val__concurrentKernels,
	const int32_t p__prop__val__ECCEnabled,
	const int32_t p__prop__val__pciBusID,
	const int32_t p__prop__val__pciDeviceID,
	const int32_t p__prop__val__pciDomainID,
	const int32_t p__prop__val__tccDriver,
	const int32_t p__prop__val__asyncEngineCount,
	const int32_t p__prop__val__unifiedAddressing,
	const int32_t p__prop__val__memoryClockRate,
	const int32_t p__prop__val__memoryBusWidth,
	const int32_t p__prop__val__l2CacheSize,
	const int32_t p__prop__val__persistingL2CacheMaxSize,
	const int32_t p__prop__val__maxThreadsPerMultiProcessor,
	const int32_t p__prop__val__streamPrioritiesSupported,
	const int32_t p__prop__val__globalL1CacheSupported,
	const int32_t p__prop__val__localL1CacheSupported,
	const uint64_t p__prop__val__sharedMemPerMultiprocessor,
	const int32_t p__prop__val__regsPerMultiprocessor,
	const int32_t p__prop__val__managedMemory,
	const int32_t p__prop__val__isMultiGpuBoard,
	const int32_t p__prop__val__multiGpuBoardGroupID,
	const int32_t p__prop__val__hostNativeAtomicSupported,
	const int32_t p__prop__val__singleToDoublePrecisionPerfRatio,
	const int32_t p__prop__val__pageableMemoryAccess,
	const int32_t p__prop__val__concurrentManagedAccess,
	const int32_t p__prop__val__computePreemptionSupported,
	const int32_t p__prop__val__canUseHostPointerForRegisteredMem,
	const int32_t p__prop__val__cooperativeLaunch,
	const int32_t p__prop__val__cooperativeMultiDeviceLaunch,
	const uint64_t p__prop__val__sharedMemPerBlockOptin,
	const int32_t p__prop__val__pageableMemoryAccessUsesHostPageTables,
	const int32_t p__prop__val__directManagedMemAccessFromHost,
	const int32_t p__prop__val__maxBlocksPerMultiProcessor,
	const int32_t p__prop__val__accessPolicyMaxWindowSize,
	const uint64_t p__prop__val__reservedSharedMemPerBlock,
	const int32_t p__prop__val__hostRegisterSupported,
	const int32_t p__prop__val__sparseHipArraySupported,
	const int32_t p__prop__val__hostRegisterReadOnlySupported,
	const int32_t p__prop__val__timelineSemaphoreInteropSupported,
	const int32_t p__prop__val__memoryPoolsSupported,
	const int32_t p__prop__val__gpuDirectRDMASupported,
	const uint32_t p__prop__val__gpuDirectRDMAFlushWritesOptions,
	const int32_t p__prop__val__gpuDirectRDMAWritesOrdering,
	const uint32_t p__prop__val__memoryPoolSupportedHandleTypes,
	const int32_t p__prop__val__deferredMappingHipArraySupported,
	const int32_t p__prop__val__ipcEventSupported,
	const int32_t p__prop__val__clusterLaunch,
	const int32_t p__prop__val__unifiedFunctionPointers,
	const uint64_t p__prop__val__reserved,
	const uint64_t p__prop__val__hipReserved,
	const uint64_t p__prop__val__gcnArchName,
	const uint64_t p__prop__val__maxSharedMemoryPerMultiProcessor,
	const int32_t p__prop__val__clockInstructionRate,
	const uint32_t p__prop__val__arch__hasGlobalInt32Atomics,
	const uint32_t p__prop__val__arch__hasGlobalFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasSharedInt32Atomics,
	const uint32_t p__prop__val__arch__hasSharedFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasFloatAtomicAdd,
	const uint32_t p__prop__val__arch__hasGlobalInt64Atomics,
	const uint32_t p__prop__val__arch__hasSharedInt64Atomics,
	const uint32_t p__prop__val__arch__hasDoubles,
	const uint32_t p__prop__val__arch__hasWarpVote,
	const uint32_t p__prop__val__arch__hasWarpBallot,
	const uint32_t p__prop__val__arch__hasWarpShuffle,
	const uint32_t p__prop__val__arch__hasFunnelShift,
	const uint32_t p__prop__val__arch__hasThreadFenceSystem,
	const uint32_t p__prop__val__arch__hasSyncThreadsExt,
	const uint32_t p__prop__val__arch__hasSurfaceFuncs,
	const uint32_t p__prop__val__arch__has3dGrid,
	const uint32_t p__prop__val__arch__hasDynamicParallelism,
	const uint64_t p__prop__val__hdpMemFlushCntl,
	const uint64_t p__prop__val__hdpRegFlushCntl,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedFunc,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedGridDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem,
	const int32_t p__prop__val__isLargeBar,
	const int32_t p__prop__val__asicRevision)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipChooseDeviceR0600Begin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipChooseDeviceR0600Begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__device, p__device__val, p__prop, p__prop__val__name, p__prop__val__uuid__bytes, p__prop__val__luid, p__prop__val__luidDeviceNodeMask, p__prop__val__totalGlobalMem, p__prop__val__sharedMemPerBlock, p__prop__val__regsPerBlock, p__prop__val__warpSize, p__prop__val__memPitch, p__prop__val__maxThreadsPerBlock, p__prop__val__maxThreadsDim, p__prop__val__maxGridSize, p__prop__val__clockRate, p__prop__val__totalConstMem, p__prop__val__major, p__prop__val__minor, p__prop__val__textureAlignment, p__prop__val__texturePitchAlignment, p__prop__val__deviceOverlap, p__prop__val__multiProcessorCount, p__prop__val__kernelExecTimeoutEnabled, p__prop__val__integrated, p__prop__val__canMapHostMemory, p__prop__val__computeMode, p__prop__val__maxTexture1D, p__prop__val__maxTexture1DMipmap, p__prop__val__maxTexture1DLinear, p__prop__val__maxTexture2D, p__prop__val__maxTexture2DMipmap, p__prop__val__maxTexture2DLinear, p__prop__val__maxTexture2DGather, p__prop__val__maxTexture3D, p__prop__val__maxTexture3DAlt, p__prop__val__maxTextureCubemap, p__prop__val__maxTexture1DLayered, p__prop__val__maxTexture2DLayered, p__prop__val__maxTextureCubemapLayered, p__prop__val__maxSurface1D, p__prop__val__maxSurface2D, p__prop__val__maxSurface3D, p__prop__val__maxSurface1DLayered, p__prop__val__maxSurface2DLayered, p__prop__val__maxSurfaceCubemap, p__prop__val__maxSurfaceCubemapLayered, p__prop__val__surfaceAlignment, p__prop__val__concurrentKernels, p__prop__val__ECCEnabled, p__prop__val__pciBusID, p__prop__val__pciDeviceID, p__prop__val__pciDomainID, p__prop__val__tccDriver, p__prop__val__asyncEngineCount, p__prop__val__unifiedAddressing, p__prop__val__memoryClockRate, p__prop__val__memoryBusWidth, p__prop__val__l2CacheSize, p__prop__val__persistingL2CacheMaxSize, p__prop__val__maxThreadsPerMultiProcessor, p__prop__val__streamPrioritiesSupported, p__prop__val__globalL1CacheSupported, p__prop__val__localL1CacheSupported, p__prop__val__sharedMemPerMultiprocessor, p__prop__val__regsPerMultiprocessor, p__prop__val__managedMemory, p__prop__val__isMultiGpuBoard, p__prop__val__multiGpuBoardGroupID, p__prop__val__hostNativeAtomicSupported, p__prop__val__singleToDoublePrecisionPerfRatio, p__prop__val__pageableMemoryAccess, p__prop__val__concurrentManagedAccess, p__prop__val__computePreemptionSupported, p__prop__val__canUseHostPointerForRegisteredMem, p__prop__val__cooperativeLaunch, p__prop__val__cooperativeMultiDeviceLaunch, p__prop__val__sharedMemPerBlockOptin, p__prop__val__pageableMemoryAccessUsesHostPageTables, p__prop__val__directManagedMemAccessFromHost, p__prop__val__maxBlocksPerMultiProcessor, p__prop__val__accessPolicyMaxWindowSize, p__prop__val__reservedSharedMemPerBlock, p__prop__val__hostRegisterSupported, p__prop__val__sparseHipArraySupported, p__prop__val__hostRegisterReadOnlySupported, p__prop__val__timelineSemaphoreInteropSupported, p__prop__val__memoryPoolsSupported, p__prop__val__gpuDirectRDMASupported, p__prop__val__gpuDirectRDMAFlushWritesOptions, p__prop__val__gpuDirectRDMAWritesOrdering, p__prop__val__memoryPoolSupportedHandleTypes, p__prop__val__deferredMappingHipArraySupported, p__prop__val__ipcEventSupported, p__prop__val__clusterLaunch, p__prop__val__unifiedFunctionPointers, p__prop__val__reserved, p__prop__val__hipReserved, p__prop__val__gcnArchName, p__prop__val__maxSharedMemoryPerMultiProcessor, p__prop__val__clockInstructionRate, p__prop__val__arch__hasGlobalInt32Atomics, p__prop__val__arch__hasGlobalFloatAtomicExch, p__prop__val__arch__hasSharedInt32Atomics, p__prop__val__arch__hasSharedFloatAtomicExch, p__prop__val__arch__hasFloatAtomicAdd, p__prop__val__arch__hasGlobalInt64Atomics, p__prop__val__arch__hasSharedInt64Atomics, p__prop__val__arch__hasDoubles, p__prop__val__arch__hasWarpVote, p__prop__val__arch__hasWarpBallot, p__prop__val__arch__hasWarpShuffle, p__prop__val__arch__hasFunnelShift, p__prop__val__arch__hasThreadFenceSystem, p__prop__val__arch__hasSyncThreadsExt, p__prop__val__arch__hasSurfaceFuncs, p__prop__val__arch__has3dGrid, p__prop__val__arch__hasDynamicParallelism, p__prop__val__hdpMemFlushCntl, p__prop__val__hdpRegFlushCntl, p__prop__val__cooperativeMultiDeviceUnmatchedFunc, p__prop__val__cooperativeMultiDeviceUnmatchedGridDim, p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim, p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem, p__prop__val__isLargeBar, p__prop__val__asicRevision);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipChooseDeviceR0600End`) */
void barectf_hip_api_trace_hipChooseDeviceR0600End(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipChooseDeviceR0600End(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipChooseDeviceR0600End(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipConfigureCallBegin`) */
void barectf_hip_api_trace_hipConfigureCallBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint32_t p__gridDim__x,
	const uint32_t p__gridDim__y,
	const uint32_t p__gridDim__z,
	const uint32_t p__blockDim__x,
	const uint32_t p__blockDim__y,
	const uint32_t p__blockDim__z,
	const uint64_t p__sharedMem,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipConfigureCallBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipConfigureCallBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__gridDim__x, p__gridDim__y, p__gridDim__z, p__blockDim__x, p__blockDim__y, p__blockDim__z, p__sharedMem, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipConfigureCallEnd`) */
void barectf_hip_api_trace_hipConfigureCallEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipConfigureCallEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipConfigureCallEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCreateSurfaceObjectBegin`) */
void barectf_hip_api_trace_hipCreateSurfaceObjectBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pSurfObject,
	const uint64_t p__pSurfObject__val,
	const uint64_t p__pResDesc,
	const int32_t p__pResDesc__val__resType)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCreateSurfaceObjectBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCreateSurfaceObjectBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pSurfObject, p__pSurfObject__val, p__pResDesc, p__pResDesc__val__resType);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCreateSurfaceObjectEnd`) */
void barectf_hip_api_trace_hipCreateSurfaceObjectEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCreateSurfaceObjectEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCreateSurfaceObjectEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxCreateBegin`) */
void barectf_hip_api_trace_hipCtxCreateBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx,
	const uint64_t p__ctx__val,
	const uint32_t p__flags,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxCreateBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxCreateBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ctx, p__ctx__val, p__flags, p__device);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxCreateEnd`) */
void barectf_hip_api_trace_hipCtxCreateEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxCreateEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxCreateEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxDestroyBegin`) */
void barectf_hip_api_trace_hipCtxDestroyBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxDestroyBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxDestroyBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ctx);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxDestroyEnd`) */
void barectf_hip_api_trace_hipCtxDestroyEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxDestroyEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxDestroyEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxDisablePeerAccessBegin`) */
void barectf_hip_api_trace_hipCtxDisablePeerAccessBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__peerCtx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxDisablePeerAccessBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxDisablePeerAccessBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__peerCtx);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxDisablePeerAccessEnd`) */
void barectf_hip_api_trace_hipCtxDisablePeerAccessEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxDisablePeerAccessEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxDisablePeerAccessEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxEnablePeerAccessBegin`) */
void barectf_hip_api_trace_hipCtxEnablePeerAccessBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__peerCtx,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxEnablePeerAccessBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxEnablePeerAccessBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__peerCtx, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxEnablePeerAccessEnd`) */
void barectf_hip_api_trace_hipCtxEnablePeerAccessEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxEnablePeerAccessEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxEnablePeerAccessEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxGetApiVersionBegin`) */
void barectf_hip_api_trace_hipCtxGetApiVersionBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx,
	const uint64_t p__apiVersion,
	const int32_t p__apiVersion__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxGetApiVersionBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxGetApiVersionBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ctx, p__apiVersion, p__apiVersion__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxGetApiVersionEnd`) */
void barectf_hip_api_trace_hipCtxGetApiVersionEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxGetApiVersionEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxGetApiVersionEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxGetCacheConfigBegin`) */
void barectf_hip_api_trace_hipCtxGetCacheConfigBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__cacheConfig,
	const int32_t p__cacheConfig__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxGetCacheConfigBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxGetCacheConfigBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__cacheConfig, p__cacheConfig__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxGetCacheConfigEnd`) */
void barectf_hip_api_trace_hipCtxGetCacheConfigEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxGetCacheConfigEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxGetCacheConfigEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxGetCurrentBegin`) */
void barectf_hip_api_trace_hipCtxGetCurrentBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx,
	const uint64_t p__ctx__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxGetCurrentBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxGetCurrentBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ctx, p__ctx__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxGetCurrentEnd`) */
void barectf_hip_api_trace_hipCtxGetCurrentEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxGetCurrentEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxGetCurrentEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxGetDeviceBegin`) */
void barectf_hip_api_trace_hipCtxGetDeviceBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__device,
	const int32_t p__device__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxGetDeviceBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxGetDeviceBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__device, p__device__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxGetDeviceEnd`) */
void barectf_hip_api_trace_hipCtxGetDeviceEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxGetDeviceEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxGetDeviceEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxGetFlagsBegin`) */
void barectf_hip_api_trace_hipCtxGetFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__flags,
	const uint32_t p__flags__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxGetFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxGetFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__flags, p__flags__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxGetFlagsEnd`) */
void barectf_hip_api_trace_hipCtxGetFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxGetFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxGetFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxGetSharedMemConfigBegin`) */
void barectf_hip_api_trace_hipCtxGetSharedMemConfigBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pConfig,
	const int32_t p__pConfig__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxGetSharedMemConfigBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxGetSharedMemConfigBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pConfig, p__pConfig__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxGetSharedMemConfigEnd`) */
void barectf_hip_api_trace_hipCtxGetSharedMemConfigEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxGetSharedMemConfigEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxGetSharedMemConfigEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxPopCurrentBegin`) */
void barectf_hip_api_trace_hipCtxPopCurrentBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx,
	const uint64_t p__ctx__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxPopCurrentBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxPopCurrentBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ctx, p__ctx__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxPopCurrentEnd`) */
void barectf_hip_api_trace_hipCtxPopCurrentEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxPopCurrentEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxPopCurrentEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxPushCurrentBegin`) */
void barectf_hip_api_trace_hipCtxPushCurrentBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxPushCurrentBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxPushCurrentBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ctx);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxPushCurrentEnd`) */
void barectf_hip_api_trace_hipCtxPushCurrentEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxPushCurrentEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxPushCurrentEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxSetCacheConfigBegin`) */
void barectf_hip_api_trace_hipCtxSetCacheConfigBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__cacheConfig)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxSetCacheConfigBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxSetCacheConfigBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__cacheConfig);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxSetCacheConfigEnd`) */
void barectf_hip_api_trace_hipCtxSetCacheConfigEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxSetCacheConfigEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxSetCacheConfigEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxSetCurrentBegin`) */
void barectf_hip_api_trace_hipCtxSetCurrentBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxSetCurrentBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxSetCurrentBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ctx);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxSetCurrentEnd`) */
void barectf_hip_api_trace_hipCtxSetCurrentEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxSetCurrentEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxSetCurrentEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxSetSharedMemConfigBegin`) */
void barectf_hip_api_trace_hipCtxSetSharedMemConfigBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__config)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxSetSharedMemConfigBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxSetSharedMemConfigBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__config);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxSetSharedMemConfigEnd`) */
void barectf_hip_api_trace_hipCtxSetSharedMemConfigEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxSetSharedMemConfigEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxSetSharedMemConfigEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxSynchronizeBegin`) */
void barectf_hip_api_trace_hipCtxSynchronizeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxSynchronizeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxSynchronizeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipCtxSynchronizeEnd`) */
void barectf_hip_api_trace_hipCtxSynchronizeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipCtxSynchronizeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipCtxSynchronizeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDestroyExternalMemoryBegin`) */
void barectf_hip_api_trace_hipDestroyExternalMemoryBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__extMem)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDestroyExternalMemoryBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDestroyExternalMemoryBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__extMem);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDestroyExternalMemoryEnd`) */
void barectf_hip_api_trace_hipDestroyExternalMemoryEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDestroyExternalMemoryEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDestroyExternalMemoryEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDestroyExternalSemaphoreBegin`) */
void barectf_hip_api_trace_hipDestroyExternalSemaphoreBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__extSem)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDestroyExternalSemaphoreBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDestroyExternalSemaphoreBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__extSem);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDestroyExternalSemaphoreEnd`) */
void barectf_hip_api_trace_hipDestroyExternalSemaphoreEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDestroyExternalSemaphoreEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDestroyExternalSemaphoreEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDestroySurfaceObjectBegin`) */
void barectf_hip_api_trace_hipDestroySurfaceObjectBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__surfaceObject)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDestroySurfaceObjectBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDestroySurfaceObjectBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__surfaceObject);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDestroySurfaceObjectEnd`) */
void barectf_hip_api_trace_hipDestroySurfaceObjectEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDestroySurfaceObjectEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDestroySurfaceObjectEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceCanAccessPeerBegin`) */
void barectf_hip_api_trace_hipDeviceCanAccessPeerBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__canAccessPeer,
	const int32_t p__canAccessPeer__val,
	const int32_t p__deviceId,
	const int32_t p__peerDeviceId)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceCanAccessPeerBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceCanAccessPeerBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__canAccessPeer, p__canAccessPeer__val, p__deviceId, p__peerDeviceId);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceCanAccessPeerEnd`) */
void barectf_hip_api_trace_hipDeviceCanAccessPeerEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceCanAccessPeerEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceCanAccessPeerEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceComputeCapabilityBegin`) */
void barectf_hip_api_trace_hipDeviceComputeCapabilityBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__major,
	const int32_t p__major__val,
	const uint64_t p__minor,
	const int32_t p__minor__val,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceComputeCapabilityBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceComputeCapabilityBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__major, p__major__val, p__minor, p__minor__val, p__device);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceComputeCapabilityEnd`) */
void barectf_hip_api_trace_hipDeviceComputeCapabilityEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceComputeCapabilityEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceComputeCapabilityEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceDisablePeerAccessBegin`) */
void barectf_hip_api_trace_hipDeviceDisablePeerAccessBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__peerDeviceId)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceDisablePeerAccessBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceDisablePeerAccessBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__peerDeviceId);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceDisablePeerAccessEnd`) */
void barectf_hip_api_trace_hipDeviceDisablePeerAccessEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceDisablePeerAccessEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceDisablePeerAccessEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceEnablePeerAccessBegin`) */
void barectf_hip_api_trace_hipDeviceEnablePeerAccessBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__peerDeviceId,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceEnablePeerAccessBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceEnablePeerAccessBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__peerDeviceId, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceEnablePeerAccessEnd`) */
void barectf_hip_api_trace_hipDeviceEnablePeerAccessEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceEnablePeerAccessEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceEnablePeerAccessEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetAttributeBegin`) */
void barectf_hip_api_trace_hipDeviceGetAttributeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pi,
	const int32_t p__pi__val,
	const int32_t p__attr,
	const int32_t p__deviceId)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetAttributeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetAttributeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pi, p__pi__val, p__attr, p__deviceId);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetAttributeEnd`) */
void barectf_hip_api_trace_hipDeviceGetAttributeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetAttributeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetAttributeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetBegin`) */
void barectf_hip_api_trace_hipDeviceGetBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__device,
	const int32_t p__device__val,
	const int32_t p__ordinal)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__device, p__device__val, p__ordinal);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetByPCIBusIdBegin`) */
void barectf_hip_api_trace_hipDeviceGetByPCIBusIdBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__device,
	const int32_t p__device__val,
	const char * const p__pciBusId,
	const int8_t p__pciBusId__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetByPCIBusIdBegin(_TO_VOID_PTR(ctx), cc__kernel_name, p__pciBusId);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetByPCIBusIdBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__device, p__device__val, p__pciBusId, p__pciBusId__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetByPCIBusIdEnd`) */
void barectf_hip_api_trace_hipDeviceGetByPCIBusIdEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetByPCIBusIdEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetByPCIBusIdEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetCacheConfigBegin`) */
void barectf_hip_api_trace_hipDeviceGetCacheConfigBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__cacheConfig,
	const int32_t p__cacheConfig__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetCacheConfigBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetCacheConfigBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__cacheConfig, p__cacheConfig__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetCacheConfigEnd`) */
void barectf_hip_api_trace_hipDeviceGetCacheConfigEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetCacheConfigEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetCacheConfigEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetDefaultMemPoolBegin`) */
void barectf_hip_api_trace_hipDeviceGetDefaultMemPoolBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const uint64_t p__mem_pool__val,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetDefaultMemPoolBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetDefaultMemPoolBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mem_pool, p__mem_pool__val, p__device);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetDefaultMemPoolEnd`) */
void barectf_hip_api_trace_hipDeviceGetDefaultMemPoolEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetDefaultMemPoolEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetDefaultMemPoolEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetEnd`) */
void barectf_hip_api_trace_hipDeviceGetEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetGraphMemAttributeBegin`) */
void barectf_hip_api_trace_hipDeviceGetGraphMemAttributeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__device,
	const int32_t p__attr,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetGraphMemAttributeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetGraphMemAttributeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__device, p__attr, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetGraphMemAttributeEnd`) */
void barectf_hip_api_trace_hipDeviceGetGraphMemAttributeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetGraphMemAttributeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetGraphMemAttributeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetLimitBegin`) */
void barectf_hip_api_trace_hipDeviceGetLimitBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pValue,
	const uint64_t p__pValue__val,
	const int32_t p__limit)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetLimitBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetLimitBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pValue, p__pValue__val, p__limit);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetLimitEnd`) */
void barectf_hip_api_trace_hipDeviceGetLimitEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetLimitEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetLimitEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetMemPoolBegin`) */
void barectf_hip_api_trace_hipDeviceGetMemPoolBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const uint64_t p__mem_pool__val,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetMemPoolBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetMemPoolBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mem_pool, p__mem_pool__val, p__device);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetMemPoolEnd`) */
void barectf_hip_api_trace_hipDeviceGetMemPoolEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetMemPoolEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetMemPoolEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetNameBegin`) */
void barectf_hip_api_trace_hipDeviceGetNameBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const char * const p__name,
	const int8_t p__name__val,
	const int32_t p__len,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetNameBegin(_TO_VOID_PTR(ctx), cc__kernel_name, p__name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetNameBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__name, p__name__val, p__len, p__device);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetNameEnd`) */
void barectf_hip_api_trace_hipDeviceGetNameEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetNameEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetNameEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetP2PAttributeBegin`) */
void barectf_hip_api_trace_hipDeviceGetP2PAttributeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__value,
	const int32_t p__value__val,
	const int32_t p__attr,
	const int32_t p__srcDevice,
	const int32_t p__dstDevice)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetP2PAttributeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetP2PAttributeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__value, p__value__val, p__attr, p__srcDevice, p__dstDevice);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetP2PAttributeEnd`) */
void barectf_hip_api_trace_hipDeviceGetP2PAttributeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetP2PAttributeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetP2PAttributeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetPCIBusIdBegin`) */
void barectf_hip_api_trace_hipDeviceGetPCIBusIdBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const char * const p__pciBusId,
	const int8_t p__pciBusId__val,
	const int32_t p__len,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetPCIBusIdBegin(_TO_VOID_PTR(ctx), cc__kernel_name, p__pciBusId);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetPCIBusIdBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pciBusId, p__pciBusId__val, p__len, p__device);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetPCIBusIdEnd`) */
void barectf_hip_api_trace_hipDeviceGetPCIBusIdEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetPCIBusIdEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetPCIBusIdEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetSharedMemConfigBegin`) */
void barectf_hip_api_trace_hipDeviceGetSharedMemConfigBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pConfig,
	const int32_t p__pConfig__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetSharedMemConfigBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetSharedMemConfigBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pConfig, p__pConfig__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetSharedMemConfigEnd`) */
void barectf_hip_api_trace_hipDeviceGetSharedMemConfigEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetSharedMemConfigEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetSharedMemConfigEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetStreamPriorityRangeBegin`) */
void barectf_hip_api_trace_hipDeviceGetStreamPriorityRangeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__leastPriority,
	const int32_t p__leastPriority__val,
	const uint64_t p__greatestPriority,
	const int32_t p__greatestPriority__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetStreamPriorityRangeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetStreamPriorityRangeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__leastPriority, p__leastPriority__val, p__greatestPriority, p__greatestPriority__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetStreamPriorityRangeEnd`) */
void barectf_hip_api_trace_hipDeviceGetStreamPriorityRangeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetStreamPriorityRangeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetStreamPriorityRangeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetUuidBegin`) */
void barectf_hip_api_trace_hipDeviceGetUuidBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__uuid,
	const uint64_t p__uuid__val__bytes,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetUuidBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetUuidBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__uuid, p__uuid__val__bytes, p__device);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGetUuidEnd`) */
void barectf_hip_api_trace_hipDeviceGetUuidEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGetUuidEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGetUuidEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGraphMemTrimBegin`) */
void barectf_hip_api_trace_hipDeviceGraphMemTrimBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGraphMemTrimBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGraphMemTrimBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__device);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceGraphMemTrimEnd`) */
void barectf_hip_api_trace_hipDeviceGraphMemTrimEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceGraphMemTrimEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceGraphMemTrimEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDevicePrimaryCtxGetStateBegin`) */
void barectf_hip_api_trace_hipDevicePrimaryCtxGetStateBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__dev,
	const uint64_t p__flags,
	const uint32_t p__flags__val,
	const uint64_t p__active,
	const int32_t p__active__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDevicePrimaryCtxGetStateBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDevicePrimaryCtxGetStateBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dev, p__flags, p__flags__val, p__active, p__active__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDevicePrimaryCtxGetStateEnd`) */
void barectf_hip_api_trace_hipDevicePrimaryCtxGetStateEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDevicePrimaryCtxGetStateEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDevicePrimaryCtxGetStateEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDevicePrimaryCtxReleaseBegin`) */
void barectf_hip_api_trace_hipDevicePrimaryCtxReleaseBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__dev)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDevicePrimaryCtxReleaseBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDevicePrimaryCtxReleaseBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dev);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDevicePrimaryCtxReleaseEnd`) */
void barectf_hip_api_trace_hipDevicePrimaryCtxReleaseEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDevicePrimaryCtxReleaseEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDevicePrimaryCtxReleaseEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDevicePrimaryCtxResetBegin`) */
void barectf_hip_api_trace_hipDevicePrimaryCtxResetBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__dev)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDevicePrimaryCtxResetBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDevicePrimaryCtxResetBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dev);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDevicePrimaryCtxResetEnd`) */
void barectf_hip_api_trace_hipDevicePrimaryCtxResetEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDevicePrimaryCtxResetEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDevicePrimaryCtxResetEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDevicePrimaryCtxRetainBegin`) */
void barectf_hip_api_trace_hipDevicePrimaryCtxRetainBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pctx,
	const uint64_t p__pctx__val,
	const int32_t p__dev)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDevicePrimaryCtxRetainBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDevicePrimaryCtxRetainBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pctx, p__pctx__val, p__dev);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDevicePrimaryCtxRetainEnd`) */
void barectf_hip_api_trace_hipDevicePrimaryCtxRetainEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDevicePrimaryCtxRetainEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDevicePrimaryCtxRetainEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDevicePrimaryCtxSetFlagsBegin`) */
void barectf_hip_api_trace_hipDevicePrimaryCtxSetFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__dev,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDevicePrimaryCtxSetFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDevicePrimaryCtxSetFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dev, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDevicePrimaryCtxSetFlagsEnd`) */
void barectf_hip_api_trace_hipDevicePrimaryCtxSetFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDevicePrimaryCtxSetFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDevicePrimaryCtxSetFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceResetBegin`) */
void barectf_hip_api_trace_hipDeviceResetBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceResetBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceResetBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceResetEnd`) */
void barectf_hip_api_trace_hipDeviceResetEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceResetEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceResetEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceSetCacheConfigBegin`) */
void barectf_hip_api_trace_hipDeviceSetCacheConfigBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__cacheConfig)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceSetCacheConfigBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceSetCacheConfigBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__cacheConfig);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceSetCacheConfigEnd`) */
void barectf_hip_api_trace_hipDeviceSetCacheConfigEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceSetCacheConfigEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceSetCacheConfigEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceSetGraphMemAttributeBegin`) */
void barectf_hip_api_trace_hipDeviceSetGraphMemAttributeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__device,
	const int32_t p__attr,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceSetGraphMemAttributeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceSetGraphMemAttributeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__device, p__attr, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceSetGraphMemAttributeEnd`) */
void barectf_hip_api_trace_hipDeviceSetGraphMemAttributeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceSetGraphMemAttributeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceSetGraphMemAttributeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceSetLimitBegin`) */
void barectf_hip_api_trace_hipDeviceSetLimitBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__limit,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceSetLimitBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceSetLimitBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__limit, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceSetLimitEnd`) */
void barectf_hip_api_trace_hipDeviceSetLimitEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceSetLimitEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceSetLimitEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceSetMemPoolBegin`) */
void barectf_hip_api_trace_hipDeviceSetMemPoolBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__device,
	const uint64_t p__mem_pool)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceSetMemPoolBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceSetMemPoolBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__device, p__mem_pool);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceSetMemPoolEnd`) */
void barectf_hip_api_trace_hipDeviceSetMemPoolEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceSetMemPoolEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceSetMemPoolEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceSetSharedMemConfigBegin`) */
void barectf_hip_api_trace_hipDeviceSetSharedMemConfigBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__config)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceSetSharedMemConfigBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceSetSharedMemConfigBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__config);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceSetSharedMemConfigEnd`) */
void barectf_hip_api_trace_hipDeviceSetSharedMemConfigEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceSetSharedMemConfigEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceSetSharedMemConfigEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceSynchronizeBegin`) */
void barectf_hip_api_trace_hipDeviceSynchronizeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceSynchronizeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceSynchronizeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceSynchronizeEnd`) */
void barectf_hip_api_trace_hipDeviceSynchronizeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceSynchronizeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceSynchronizeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceTotalMemBegin`) */
void barectf_hip_api_trace_hipDeviceTotalMemBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__bytes,
	const uint64_t p__bytes__val,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceTotalMemBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceTotalMemBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__bytes, p__bytes__val, p__device);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDeviceTotalMemEnd`) */
void barectf_hip_api_trace_hipDeviceTotalMemEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDeviceTotalMemEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDeviceTotalMemEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDriverGetVersionBegin`) */
void barectf_hip_api_trace_hipDriverGetVersionBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__driverVersion,
	const int32_t p__driverVersion__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDriverGetVersionBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDriverGetVersionBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__driverVersion, p__driverVersion__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDriverGetVersionEnd`) */
void barectf_hip_api_trace_hipDriverGetVersionEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDriverGetVersionEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDriverGetVersionEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDrvGraphAddMemcpyNodeBegin`) */
void barectf_hip_api_trace_hipDrvGraphAddMemcpyNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__phGraphNode,
	const uint64_t p__phGraphNode__val,
	const uint64_t p__hGraph,
	const uint64_t p__dependencies,
	const uint64_t p__dependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__copyParams,
	const uint64_t p__copyParams__val__srcXInBytes,
	const uint64_t p__copyParams__val__srcY,
	const uint64_t p__copyParams__val__srcZ,
	const uint64_t p__copyParams__val__srcLOD,
	const int32_t p__copyParams__val__srcMemoryType,
	const uint64_t p__copyParams__val__srcHost,
	const uint64_t p__copyParams__val__srcDevice,
	const uint64_t p__copyParams__val__srcArray,
	const uint64_t p__copyParams__val__srcPitch,
	const uint64_t p__copyParams__val__srcHeight,
	const uint64_t p__copyParams__val__dstXInBytes,
	const uint64_t p__copyParams__val__dstY,
	const uint64_t p__copyParams__val__dstZ,
	const uint64_t p__copyParams__val__dstLOD,
	const int32_t p__copyParams__val__dstMemoryType,
	const uint64_t p__copyParams__val__dstHost,
	const uint64_t p__copyParams__val__dstDevice,
	const uint64_t p__copyParams__val__dstArray,
	const uint64_t p__copyParams__val__dstPitch,
	const uint64_t p__copyParams__val__dstHeight,
	const uint64_t p__copyParams__val__WidthInBytes,
	const uint64_t p__copyParams__val__Height,
	const uint64_t p__copyParams__val__Depth,
	const uint64_t p__ctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDrvGraphAddMemcpyNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDrvGraphAddMemcpyNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__phGraphNode, p__phGraphNode__val, p__hGraph, p__dependencies, p__dependencies__val, p__numDependencies, p__copyParams, p__copyParams__val__srcXInBytes, p__copyParams__val__srcY, p__copyParams__val__srcZ, p__copyParams__val__srcLOD, p__copyParams__val__srcMemoryType, p__copyParams__val__srcHost, p__copyParams__val__srcDevice, p__copyParams__val__srcArray, p__copyParams__val__srcPitch, p__copyParams__val__srcHeight, p__copyParams__val__dstXInBytes, p__copyParams__val__dstY, p__copyParams__val__dstZ, p__copyParams__val__dstLOD, p__copyParams__val__dstMemoryType, p__copyParams__val__dstHost, p__copyParams__val__dstDevice, p__copyParams__val__dstArray, p__copyParams__val__dstPitch, p__copyParams__val__dstHeight, p__copyParams__val__WidthInBytes, p__copyParams__val__Height, p__copyParams__val__Depth, p__ctx);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDrvGraphAddMemcpyNodeEnd`) */
void barectf_hip_api_trace_hipDrvGraphAddMemcpyNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDrvGraphAddMemcpyNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDrvGraphAddMemcpyNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDrvGraphAddMemsetNodeBegin`) */
void barectf_hip_api_trace_hipDrvGraphAddMemsetNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__phGraphNode,
	const uint64_t p__phGraphNode__val,
	const uint64_t p__hGraph,
	const uint64_t p__dependencies,
	const uint64_t p__dependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__memsetParams,
	const uint64_t p__memsetParams__val__dst,
	const uint64_t p__memsetParams__val__pitch,
	const uint32_t p__memsetParams__val__value,
	const uint32_t p__memsetParams__val__elementSize,
	const uint64_t p__memsetParams__val__width,
	const uint64_t p__memsetParams__val__height,
	const uint64_t p__ctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDrvGraphAddMemsetNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDrvGraphAddMemsetNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__phGraphNode, p__phGraphNode__val, p__hGraph, p__dependencies, p__dependencies__val, p__numDependencies, p__memsetParams, p__memsetParams__val__dst, p__memsetParams__val__pitch, p__memsetParams__val__value, p__memsetParams__val__elementSize, p__memsetParams__val__width, p__memsetParams__val__height, p__ctx);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDrvGraphAddMemsetNodeEnd`) */
void barectf_hip_api_trace_hipDrvGraphAddMemsetNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDrvGraphAddMemsetNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDrvGraphAddMemsetNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDrvMemcpy2DUnalignedBegin`) */
void barectf_hip_api_trace_hipDrvMemcpy2DUnalignedBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pCopy,
	const uint64_t p__pCopy__val__srcXInBytes,
	const uint64_t p__pCopy__val__srcY,
	const int32_t p__pCopy__val__srcMemoryType,
	const uint64_t p__pCopy__val__srcHost,
	const uint64_t p__pCopy__val__srcDevice,
	const uint64_t p__pCopy__val__srcArray,
	const uint64_t p__pCopy__val__srcPitch,
	const uint64_t p__pCopy__val__dstXInBytes,
	const uint64_t p__pCopy__val__dstY,
	const int32_t p__pCopy__val__dstMemoryType,
	const uint64_t p__pCopy__val__dstHost,
	const uint64_t p__pCopy__val__dstDevice,
	const uint64_t p__pCopy__val__dstArray,
	const uint64_t p__pCopy__val__dstPitch,
	const uint64_t p__pCopy__val__WidthInBytes,
	const uint64_t p__pCopy__val__Height)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDrvMemcpy2DUnalignedBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDrvMemcpy2DUnalignedBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pCopy, p__pCopy__val__srcXInBytes, p__pCopy__val__srcY, p__pCopy__val__srcMemoryType, p__pCopy__val__srcHost, p__pCopy__val__srcDevice, p__pCopy__val__srcArray, p__pCopy__val__srcPitch, p__pCopy__val__dstXInBytes, p__pCopy__val__dstY, p__pCopy__val__dstMemoryType, p__pCopy__val__dstHost, p__pCopy__val__dstDevice, p__pCopy__val__dstArray, p__pCopy__val__dstPitch, p__pCopy__val__WidthInBytes, p__pCopy__val__Height);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDrvMemcpy2DUnalignedEnd`) */
void barectf_hip_api_trace_hipDrvMemcpy2DUnalignedEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDrvMemcpy2DUnalignedEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDrvMemcpy2DUnalignedEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDrvMemcpy3DAsyncBegin`) */
void barectf_hip_api_trace_hipDrvMemcpy3DAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pCopy,
	const uint64_t p__pCopy__val__srcXInBytes,
	const uint64_t p__pCopy__val__srcY,
	const uint64_t p__pCopy__val__srcZ,
	const uint64_t p__pCopy__val__srcLOD,
	const int32_t p__pCopy__val__srcMemoryType,
	const uint64_t p__pCopy__val__srcHost,
	const uint64_t p__pCopy__val__srcDevice,
	const uint64_t p__pCopy__val__srcArray,
	const uint64_t p__pCopy__val__srcPitch,
	const uint64_t p__pCopy__val__srcHeight,
	const uint64_t p__pCopy__val__dstXInBytes,
	const uint64_t p__pCopy__val__dstY,
	const uint64_t p__pCopy__val__dstZ,
	const uint64_t p__pCopy__val__dstLOD,
	const int32_t p__pCopy__val__dstMemoryType,
	const uint64_t p__pCopy__val__dstHost,
	const uint64_t p__pCopy__val__dstDevice,
	const uint64_t p__pCopy__val__dstArray,
	const uint64_t p__pCopy__val__dstPitch,
	const uint64_t p__pCopy__val__dstHeight,
	const uint64_t p__pCopy__val__WidthInBytes,
	const uint64_t p__pCopy__val__Height,
	const uint64_t p__pCopy__val__Depth,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDrvMemcpy3DAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDrvMemcpy3DAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pCopy, p__pCopy__val__srcXInBytes, p__pCopy__val__srcY, p__pCopy__val__srcZ, p__pCopy__val__srcLOD, p__pCopy__val__srcMemoryType, p__pCopy__val__srcHost, p__pCopy__val__srcDevice, p__pCopy__val__srcArray, p__pCopy__val__srcPitch, p__pCopy__val__srcHeight, p__pCopy__val__dstXInBytes, p__pCopy__val__dstY, p__pCopy__val__dstZ, p__pCopy__val__dstLOD, p__pCopy__val__dstMemoryType, p__pCopy__val__dstHost, p__pCopy__val__dstDevice, p__pCopy__val__dstArray, p__pCopy__val__dstPitch, p__pCopy__val__dstHeight, p__pCopy__val__WidthInBytes, p__pCopy__val__Height, p__pCopy__val__Depth, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDrvMemcpy3DAsyncEnd`) */
void barectf_hip_api_trace_hipDrvMemcpy3DAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDrvMemcpy3DAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDrvMemcpy3DAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDrvMemcpy3DBegin`) */
void barectf_hip_api_trace_hipDrvMemcpy3DBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pCopy,
	const uint64_t p__pCopy__val__srcXInBytes,
	const uint64_t p__pCopy__val__srcY,
	const uint64_t p__pCopy__val__srcZ,
	const uint64_t p__pCopy__val__srcLOD,
	const int32_t p__pCopy__val__srcMemoryType,
	const uint64_t p__pCopy__val__srcHost,
	const uint64_t p__pCopy__val__srcDevice,
	const uint64_t p__pCopy__val__srcArray,
	const uint64_t p__pCopy__val__srcPitch,
	const uint64_t p__pCopy__val__srcHeight,
	const uint64_t p__pCopy__val__dstXInBytes,
	const uint64_t p__pCopy__val__dstY,
	const uint64_t p__pCopy__val__dstZ,
	const uint64_t p__pCopy__val__dstLOD,
	const int32_t p__pCopy__val__dstMemoryType,
	const uint64_t p__pCopy__val__dstHost,
	const uint64_t p__pCopy__val__dstDevice,
	const uint64_t p__pCopy__val__dstArray,
	const uint64_t p__pCopy__val__dstPitch,
	const uint64_t p__pCopy__val__dstHeight,
	const uint64_t p__pCopy__val__WidthInBytes,
	const uint64_t p__pCopy__val__Height,
	const uint64_t p__pCopy__val__Depth)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDrvMemcpy3DBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDrvMemcpy3DBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pCopy, p__pCopy__val__srcXInBytes, p__pCopy__val__srcY, p__pCopy__val__srcZ, p__pCopy__val__srcLOD, p__pCopy__val__srcMemoryType, p__pCopy__val__srcHost, p__pCopy__val__srcDevice, p__pCopy__val__srcArray, p__pCopy__val__srcPitch, p__pCopy__val__srcHeight, p__pCopy__val__dstXInBytes, p__pCopy__val__dstY, p__pCopy__val__dstZ, p__pCopy__val__dstLOD, p__pCopy__val__dstMemoryType, p__pCopy__val__dstHost, p__pCopy__val__dstDevice, p__pCopy__val__dstArray, p__pCopy__val__dstPitch, p__pCopy__val__dstHeight, p__pCopy__val__WidthInBytes, p__pCopy__val__Height, p__pCopy__val__Depth);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDrvMemcpy3DEnd`) */
void barectf_hip_api_trace_hipDrvMemcpy3DEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDrvMemcpy3DEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDrvMemcpy3DEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDrvPointerGetAttributesBegin`) */
void barectf_hip_api_trace_hipDrvPointerGetAttributesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint32_t p__numAttributes,
	const uint64_t p__attributes,
	const int32_t p__attributes__val,
	const uint64_t p__data,
	const uint64_t p__data__val,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDrvPointerGetAttributesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDrvPointerGetAttributesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__numAttributes, p__attributes, p__attributes__val, p__data, p__data__val, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipDrvPointerGetAttributesEnd`) */
void barectf_hip_api_trace_hipDrvPointerGetAttributesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipDrvPointerGetAttributesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipDrvPointerGetAttributesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventCreateBegin`) */
void barectf_hip_api_trace_hipEventCreateBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event,
	const uint64_t p__event__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventCreateBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventCreateBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__event, p__event__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventCreateEnd`) */
void barectf_hip_api_trace_hipEventCreateEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventCreateEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventCreateEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventCreateWithFlagsBegin`) */
void barectf_hip_api_trace_hipEventCreateWithFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event,
	const uint64_t p__event__val,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventCreateWithFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventCreateWithFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__event, p__event__val, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventCreateWithFlagsEnd`) */
void barectf_hip_api_trace_hipEventCreateWithFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventCreateWithFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventCreateWithFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventDestroyBegin`) */
void barectf_hip_api_trace_hipEventDestroyBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventDestroyBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventDestroyBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__event);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventDestroyEnd`) */
void barectf_hip_api_trace_hipEventDestroyEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventDestroyEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventDestroyEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventElapsedTimeBegin`) */
void barectf_hip_api_trace_hipEventElapsedTimeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ms,
	const uint64_t p__ms__val,
	const uint64_t p__start,
	const uint64_t p__stop)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventElapsedTimeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventElapsedTimeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ms, p__ms__val, p__start, p__stop);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventElapsedTimeEnd`) */
void barectf_hip_api_trace_hipEventElapsedTimeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventElapsedTimeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventElapsedTimeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventQueryBegin`) */
void barectf_hip_api_trace_hipEventQueryBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventQueryBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventQueryBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__event);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventQueryEnd`) */
void barectf_hip_api_trace_hipEventQueryEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventQueryEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventQueryEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventRecordBegin`) */
void barectf_hip_api_trace_hipEventRecordBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventRecordBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventRecordBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__event, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventRecordEnd`) */
void barectf_hip_api_trace_hipEventRecordEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventRecordEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventRecordEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventSynchronizeBegin`) */
void barectf_hip_api_trace_hipEventSynchronizeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventSynchronizeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventSynchronizeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__event);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipEventSynchronizeEnd`) */
void barectf_hip_api_trace_hipEventSynchronizeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipEventSynchronizeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipEventSynchronizeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtGetLastErrorBegin`) */
void barectf_hip_api_trace_hipExtGetLastErrorBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtGetLastErrorBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtGetLastErrorBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtGetLastErrorEnd`) */
void barectf_hip_api_trace_hipExtGetLastErrorEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtGetLastErrorEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtGetLastErrorEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtGetLinkTypeAndHopCountBegin`) */
void barectf_hip_api_trace_hipExtGetLinkTypeAndHopCountBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__device1,
	const int32_t p__device2,
	const uint64_t p__linktype,
	const uint32_t p__linktype__val,
	const uint64_t p__hopcount,
	const uint32_t p__hopcount__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtGetLinkTypeAndHopCountBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtGetLinkTypeAndHopCountBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__device1, p__device2, p__linktype, p__linktype__val, p__hopcount, p__hopcount__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtGetLinkTypeAndHopCountEnd`) */
void barectf_hip_api_trace_hipExtGetLinkTypeAndHopCountEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtGetLinkTypeAndHopCountEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtGetLinkTypeAndHopCountEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtLaunchKernelBegin`) */
void barectf_hip_api_trace_hipExtLaunchKernelBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__function_address,
	const uint32_t p__numBlocks__x,
	const uint32_t p__numBlocks__y,
	const uint32_t p__numBlocks__z,
	const uint32_t p__dimBlocks__x,
	const uint32_t p__dimBlocks__y,
	const uint32_t p__dimBlocks__z,
	const uint64_t p__args,
	const uint64_t p__args__val,
	const uint64_t p__sharedMemBytes,
	const uint64_t p__stream,
	const uint64_t p__startEvent,
	const uint64_t p__stopEvent,
	const int32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtLaunchKernelBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtLaunchKernelBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__function_address, p__numBlocks__x, p__numBlocks__y, p__numBlocks__z, p__dimBlocks__x, p__dimBlocks__y, p__dimBlocks__z, p__args, p__args__val, p__sharedMemBytes, p__stream, p__startEvent, p__stopEvent, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtLaunchKernelEnd`) */
void barectf_hip_api_trace_hipExtLaunchKernelEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtLaunchKernelEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtLaunchKernelEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtLaunchMultiKernelMultiDeviceBegin`) */
void barectf_hip_api_trace_hipExtLaunchMultiKernelMultiDeviceBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__launchParamsList,
	const uint64_t p__launchParamsList__val__func,
	const uint32_t p__launchParamsList__val__gridDim__x,
	const uint32_t p__launchParamsList__val__gridDim__y,
	const uint32_t p__launchParamsList__val__gridDim__z,
	const uint32_t p__launchParamsList__val__blockDim__x,
	const uint32_t p__launchParamsList__val__blockDim__y,
	const uint32_t p__launchParamsList__val__blockDim__z,
	const uint64_t p__launchParamsList__val__args,
	const uint64_t p__launchParamsList__val__sharedMem,
	const uint64_t p__launchParamsList__val__stream,
	const int32_t p__numDevices,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtLaunchMultiKernelMultiDeviceBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtLaunchMultiKernelMultiDeviceBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__launchParamsList, p__launchParamsList__val__func, p__launchParamsList__val__gridDim__x, p__launchParamsList__val__gridDim__y, p__launchParamsList__val__gridDim__z, p__launchParamsList__val__blockDim__x, p__launchParamsList__val__blockDim__y, p__launchParamsList__val__blockDim__z, p__launchParamsList__val__args, p__launchParamsList__val__sharedMem, p__launchParamsList__val__stream, p__numDevices, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtLaunchMultiKernelMultiDeviceEnd`) */
void barectf_hip_api_trace_hipExtLaunchMultiKernelMultiDeviceEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtLaunchMultiKernelMultiDeviceEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtLaunchMultiKernelMultiDeviceEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtMallocWithFlagsBegin`) */
void barectf_hip_api_trace_hipExtMallocWithFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__sizeBytes,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtMallocWithFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtMallocWithFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr, p__ptr__val, p__sizeBytes, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtMallocWithFlagsEnd`) */
void barectf_hip_api_trace_hipExtMallocWithFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtMallocWithFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtMallocWithFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtModuleLaunchKernelBegin`) */
void barectf_hip_api_trace_hipExtModuleLaunchKernelBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__f,
	const uint32_t p__globalWorkSizeX,
	const uint32_t p__globalWorkSizeY,
	const uint32_t p__globalWorkSizeZ,
	const uint32_t p__localWorkSizeX,
	const uint32_t p__localWorkSizeY,
	const uint32_t p__localWorkSizeZ,
	const uint64_t p__sharedMemBytes,
	const uint64_t p__hStream,
	const uint64_t p__kernelParams,
	const uint64_t p__kernelParams__val,
	const uint64_t p__extra,
	const uint64_t p__extra__val,
	const uint64_t p__startEvent,
	const uint64_t p__stopEvent,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtModuleLaunchKernelBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtModuleLaunchKernelBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__f, p__globalWorkSizeX, p__globalWorkSizeY, p__globalWorkSizeZ, p__localWorkSizeX, p__localWorkSizeY, p__localWorkSizeZ, p__sharedMemBytes, p__hStream, p__kernelParams, p__kernelParams__val, p__extra, p__extra__val, p__startEvent, p__stopEvent, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtModuleLaunchKernelEnd`) */
void barectf_hip_api_trace_hipExtModuleLaunchKernelEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtModuleLaunchKernelEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtModuleLaunchKernelEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtStreamCreateWithCUMaskBegin`) */
void barectf_hip_api_trace_hipExtStreamCreateWithCUMaskBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__stream__val,
	const uint32_t p__cuMaskSize,
	const uint64_t p__cuMask,
	const uint32_t p__cuMask__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtStreamCreateWithCUMaskBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtStreamCreateWithCUMaskBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__stream__val, p__cuMaskSize, p__cuMask, p__cuMask__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtStreamCreateWithCUMaskEnd`) */
void barectf_hip_api_trace_hipExtStreamCreateWithCUMaskEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtStreamCreateWithCUMaskEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtStreamCreateWithCUMaskEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtStreamGetCUMaskBegin`) */
void barectf_hip_api_trace_hipExtStreamGetCUMaskBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint32_t p__cuMaskSize,
	const uint64_t p__cuMask,
	const uint32_t p__cuMask__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtStreamGetCUMaskBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtStreamGetCUMaskBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__cuMaskSize, p__cuMask, p__cuMask__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExtStreamGetCUMaskEnd`) */
void barectf_hip_api_trace_hipExtStreamGetCUMaskEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExtStreamGetCUMaskEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExtStreamGetCUMaskEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExternalMemoryGetMappedBufferBegin`) */
void barectf_hip_api_trace_hipExternalMemoryGetMappedBufferBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr,
	const uint64_t p__devPtr__val,
	const uint64_t p__extMem,
	const uint64_t p__bufferDesc,
	const uint64_t p__bufferDesc__val__offset,
	const uint64_t p__bufferDesc__val__size,
	const uint32_t p__bufferDesc__val__flags,
	const uint64_t p__bufferDesc__val__reserved)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExternalMemoryGetMappedBufferBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExternalMemoryGetMappedBufferBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__devPtr, p__devPtr__val, p__extMem, p__bufferDesc, p__bufferDesc__val__offset, p__bufferDesc__val__size, p__bufferDesc__val__flags, p__bufferDesc__val__reserved);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExternalMemoryGetMappedBufferEnd`) */
void barectf_hip_api_trace_hipExternalMemoryGetMappedBufferEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExternalMemoryGetMappedBufferEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExternalMemoryGetMappedBufferEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExternalMemoryGetMappedMipmappedArrayBegin`) */
void barectf_hip_api_trace_hipExternalMemoryGetMappedMipmappedArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mipmap,
	const uint64_t p__mipmap__val,
	const uint64_t p__extMem,
	const uint64_t p__mipmapDesc,
	const uint64_t p__mipmapDesc__val__offset,
	const int32_t p__mipmapDesc__val__formatDesc__x,
	const int32_t p__mipmapDesc__val__formatDesc__y,
	const int32_t p__mipmapDesc__val__formatDesc__z,
	const int32_t p__mipmapDesc__val__formatDesc__w,
	const int32_t p__mipmapDesc__val__formatDesc__f,
	const uint64_t p__mipmapDesc__val__extent__width,
	const uint64_t p__mipmapDesc__val__extent__height,
	const uint64_t p__mipmapDesc__val__extent__depth,
	const uint32_t p__mipmapDesc__val__flags,
	const uint32_t p__mipmapDesc__val__numLevels)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExternalMemoryGetMappedMipmappedArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExternalMemoryGetMappedMipmappedArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mipmap, p__mipmap__val, p__extMem, p__mipmapDesc, p__mipmapDesc__val__offset, p__mipmapDesc__val__formatDesc__x, p__mipmapDesc__val__formatDesc__y, p__mipmapDesc__val__formatDesc__z, p__mipmapDesc__val__formatDesc__w, p__mipmapDesc__val__formatDesc__f, p__mipmapDesc__val__extent__width, p__mipmapDesc__val__extent__height, p__mipmapDesc__val__extent__depth, p__mipmapDesc__val__flags, p__mipmapDesc__val__numLevels);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipExternalMemoryGetMappedMipmappedArrayEnd`) */
void barectf_hip_api_trace_hipExternalMemoryGetMappedMipmappedArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipExternalMemoryGetMappedMipmappedArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipExternalMemoryGetMappedMipmappedArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFreeArrayBegin`) */
void barectf_hip_api_trace_hipFreeArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__array)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFreeArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFreeArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__array);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFreeArrayEnd`) */
void barectf_hip_api_trace_hipFreeArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFreeArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFreeArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFreeAsyncBegin`) */
void barectf_hip_api_trace_hipFreeAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFreeAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFreeAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dev_ptr, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFreeAsyncEnd`) */
void barectf_hip_api_trace_hipFreeAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFreeAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFreeAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFreeBegin`) */
void barectf_hip_api_trace_hipFreeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFreeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFreeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFreeEnd`) */
void barectf_hip_api_trace_hipFreeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFreeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFreeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFreeHostBegin`) */
void barectf_hip_api_trace_hipFreeHostBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFreeHostBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFreeHostBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFreeHostEnd`) */
void barectf_hip_api_trace_hipFreeHostEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFreeHostEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFreeHostEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFreeMipmappedArrayBegin`) */
void barectf_hip_api_trace_hipFreeMipmappedArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mipmappedArray)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFreeMipmappedArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFreeMipmappedArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mipmappedArray);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFreeMipmappedArrayEnd`) */
void barectf_hip_api_trace_hipFreeMipmappedArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFreeMipmappedArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFreeMipmappedArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFuncGetAttributeBegin`) */
void barectf_hip_api_trace_hipFuncGetAttributeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__value,
	const int32_t p__value__val,
	const int32_t p__attrib,
	const uint64_t p__hfunc)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFuncGetAttributeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFuncGetAttributeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__value, p__value__val, p__attrib, p__hfunc);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFuncGetAttributeEnd`) */
void barectf_hip_api_trace_hipFuncGetAttributeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFuncGetAttributeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFuncGetAttributeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFuncGetAttributesBegin`) */
void barectf_hip_api_trace_hipFuncGetAttributesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__attr,
	const int32_t p__attr__val__binaryVersion,
	const int32_t p__attr__val__cacheModeCA,
	const uint64_t p__attr__val__constSizeBytes,
	const uint64_t p__attr__val__localSizeBytes,
	const int32_t p__attr__val__maxDynamicSharedSizeBytes,
	const int32_t p__attr__val__maxThreadsPerBlock,
	const int32_t p__attr__val__numRegs,
	const int32_t p__attr__val__preferredShmemCarveout,
	const int32_t p__attr__val__ptxVersion,
	const uint64_t p__attr__val__sharedSizeBytes,
	const uint64_t p__func)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFuncGetAttributesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFuncGetAttributesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__attr, p__attr__val__binaryVersion, p__attr__val__cacheModeCA, p__attr__val__constSizeBytes, p__attr__val__localSizeBytes, p__attr__val__maxDynamicSharedSizeBytes, p__attr__val__maxThreadsPerBlock, p__attr__val__numRegs, p__attr__val__preferredShmemCarveout, p__attr__val__ptxVersion, p__attr__val__sharedSizeBytes, p__func);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFuncGetAttributesEnd`) */
void barectf_hip_api_trace_hipFuncGetAttributesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFuncGetAttributesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFuncGetAttributesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFuncSetAttributeBegin`) */
void barectf_hip_api_trace_hipFuncSetAttributeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__func,
	const int32_t p__attr,
	const int32_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFuncSetAttributeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFuncSetAttributeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__func, p__attr, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFuncSetAttributeEnd`) */
void barectf_hip_api_trace_hipFuncSetAttributeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFuncSetAttributeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFuncSetAttributeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFuncSetCacheConfigBegin`) */
void barectf_hip_api_trace_hipFuncSetCacheConfigBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__func,
	const int32_t p__config)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFuncSetCacheConfigBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFuncSetCacheConfigBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__func, p__config);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFuncSetCacheConfigEnd`) */
void barectf_hip_api_trace_hipFuncSetCacheConfigEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFuncSetCacheConfigEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFuncSetCacheConfigEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFuncSetSharedMemConfigBegin`) */
void barectf_hip_api_trace_hipFuncSetSharedMemConfigBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__func,
	const int32_t p__config)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFuncSetSharedMemConfigBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFuncSetSharedMemConfigBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__func, p__config);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipFuncSetSharedMemConfigEnd`) */
void barectf_hip_api_trace_hipFuncSetSharedMemConfigEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipFuncSetSharedMemConfigEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipFuncSetSharedMemConfigEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGLGetDevicesBegin`) */
void barectf_hip_api_trace_hipGLGetDevicesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pHipDeviceCount,
	const uint32_t p__pHipDeviceCount__val,
	const uint64_t p__pHipDevices,
	const int32_t p__pHipDevices__val,
	const uint32_t p__hipDeviceCount,
	const int32_t p__deviceList)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGLGetDevicesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGLGetDevicesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pHipDeviceCount, p__pHipDeviceCount__val, p__pHipDevices, p__pHipDevices__val, p__hipDeviceCount, p__deviceList);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGLGetDevicesEnd`) */
void barectf_hip_api_trace_hipGLGetDevicesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGLGetDevicesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGLGetDevicesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetChannelDescBegin`) */
void barectf_hip_api_trace_hipGetChannelDescBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__desc,
	const int32_t p__desc__val__x,
	const int32_t p__desc__val__y,
	const int32_t p__desc__val__z,
	const int32_t p__desc__val__w,
	const int32_t p__desc__val__f,
	const uint64_t p__array)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetChannelDescBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetChannelDescBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__desc, p__desc__val__x, p__desc__val__y, p__desc__val__z, p__desc__val__w, p__desc__val__f, p__array);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetChannelDescEnd`) */
void barectf_hip_api_trace_hipGetChannelDescEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetChannelDescEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetChannelDescEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetDeviceBegin`) */
void barectf_hip_api_trace_hipGetDeviceBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__deviceId,
	const int32_t p__deviceId__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetDeviceBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetDeviceBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__deviceId, p__deviceId__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetDeviceCountBegin`) */
void barectf_hip_api_trace_hipGetDeviceCountBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__count,
	const int32_t p__count__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetDeviceCountBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetDeviceCountBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__count, p__count__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetDeviceCountEnd`) */
void barectf_hip_api_trace_hipGetDeviceCountEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetDeviceCountEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetDeviceCountEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetDeviceEnd`) */
void barectf_hip_api_trace_hipGetDeviceEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetDeviceEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetDeviceEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetDeviceFlagsBegin`) */
void barectf_hip_api_trace_hipGetDeviceFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__flags,
	const uint32_t p__flags__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetDeviceFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetDeviceFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__flags, p__flags__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetDeviceFlagsEnd`) */
void barectf_hip_api_trace_hipGetDeviceFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetDeviceFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetDeviceFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetDevicePropertiesBegin`) */
void barectf_hip_api_trace_hipGetDevicePropertiesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetDevicePropertiesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetDevicePropertiesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetDevicePropertiesEnd`) */
void barectf_hip_api_trace_hipGetDevicePropertiesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetDevicePropertiesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetDevicePropertiesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetDevicePropertiesR0000Begin`) */
void barectf_hip_api_trace_hipGetDevicePropertiesR0000Begin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__prop,
	const uint64_t p__prop__val__name,
	const uint64_t p__prop__val__totalGlobalMem,
	const uint64_t p__prop__val__sharedMemPerBlock,
	const int32_t p__prop__val__regsPerBlock,
	const int32_t p__prop__val__warpSize,
	const int32_t p__prop__val__maxThreadsPerBlock,
	const uint64_t p__prop__val__maxThreadsDim,
	const uint64_t p__prop__val__maxGridSize,
	const int32_t p__prop__val__clockRate,
	const int32_t p__prop__val__memoryClockRate,
	const int32_t p__prop__val__memoryBusWidth,
	const uint64_t p__prop__val__totalConstMem,
	const int32_t p__prop__val__major,
	const int32_t p__prop__val__minor,
	const int32_t p__prop__val__multiProcessorCount,
	const int32_t p__prop__val__l2CacheSize,
	const int32_t p__prop__val__maxThreadsPerMultiProcessor,
	const int32_t p__prop__val__computeMode,
	const int32_t p__prop__val__clockInstructionRate,
	const uint32_t p__prop__val__arch__hasGlobalInt32Atomics,
	const uint32_t p__prop__val__arch__hasGlobalFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasSharedInt32Atomics,
	const uint32_t p__prop__val__arch__hasSharedFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasFloatAtomicAdd,
	const uint32_t p__prop__val__arch__hasGlobalInt64Atomics,
	const uint32_t p__prop__val__arch__hasSharedInt64Atomics,
	const uint32_t p__prop__val__arch__hasDoubles,
	const uint32_t p__prop__val__arch__hasWarpVote,
	const uint32_t p__prop__val__arch__hasWarpBallot,
	const uint32_t p__prop__val__arch__hasWarpShuffle,
	const uint32_t p__prop__val__arch__hasFunnelShift,
	const uint32_t p__prop__val__arch__hasThreadFenceSystem,
	const uint32_t p__prop__val__arch__hasSyncThreadsExt,
	const uint32_t p__prop__val__arch__hasSurfaceFuncs,
	const uint32_t p__prop__val__arch__has3dGrid,
	const uint32_t p__prop__val__arch__hasDynamicParallelism,
	const int32_t p__prop__val__concurrentKernels,
	const int32_t p__prop__val__pciDomainID,
	const int32_t p__prop__val__pciBusID,
	const int32_t p__prop__val__pciDeviceID,
	const uint64_t p__prop__val__maxSharedMemoryPerMultiProcessor,
	const int32_t p__prop__val__isMultiGpuBoard,
	const int32_t p__prop__val__canMapHostMemory,
	const int32_t p__prop__val__gcnArch,
	const uint64_t p__prop__val__gcnArchName,
	const int32_t p__prop__val__integrated,
	const int32_t p__prop__val__cooperativeLaunch,
	const int32_t p__prop__val__cooperativeMultiDeviceLaunch,
	const int32_t p__prop__val__maxTexture1DLinear,
	const int32_t p__prop__val__maxTexture1D,
	const uint64_t p__prop__val__maxTexture2D,
	const uint64_t p__prop__val__maxTexture3D,
	const uint64_t p__prop__val__hdpMemFlushCntl,
	const uint64_t p__prop__val__hdpRegFlushCntl,
	const uint64_t p__prop__val__memPitch,
	const uint64_t p__prop__val__textureAlignment,
	const uint64_t p__prop__val__texturePitchAlignment,
	const int32_t p__prop__val__kernelExecTimeoutEnabled,
	const int32_t p__prop__val__ECCEnabled,
	const int32_t p__prop__val__tccDriver,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedFunc,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedGridDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem,
	const int32_t p__prop__val__isLargeBar,
	const int32_t p__prop__val__asicRevision,
	const int32_t p__prop__val__managedMemory,
	const int32_t p__prop__val__directManagedMemAccessFromHost,
	const int32_t p__prop__val__concurrentManagedAccess,
	const int32_t p__prop__val__pageableMemoryAccess,
	const int32_t p__prop__val__pageableMemoryAccessUsesHostPageTables,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetDevicePropertiesR0000Begin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetDevicePropertiesR0000Begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__prop, p__prop__val__name, p__prop__val__totalGlobalMem, p__prop__val__sharedMemPerBlock, p__prop__val__regsPerBlock, p__prop__val__warpSize, p__prop__val__maxThreadsPerBlock, p__prop__val__maxThreadsDim, p__prop__val__maxGridSize, p__prop__val__clockRate, p__prop__val__memoryClockRate, p__prop__val__memoryBusWidth, p__prop__val__totalConstMem, p__prop__val__major, p__prop__val__minor, p__prop__val__multiProcessorCount, p__prop__val__l2CacheSize, p__prop__val__maxThreadsPerMultiProcessor, p__prop__val__computeMode, p__prop__val__clockInstructionRate, p__prop__val__arch__hasGlobalInt32Atomics, p__prop__val__arch__hasGlobalFloatAtomicExch, p__prop__val__arch__hasSharedInt32Atomics, p__prop__val__arch__hasSharedFloatAtomicExch, p__prop__val__arch__hasFloatAtomicAdd, p__prop__val__arch__hasGlobalInt64Atomics, p__prop__val__arch__hasSharedInt64Atomics, p__prop__val__arch__hasDoubles, p__prop__val__arch__hasWarpVote, p__prop__val__arch__hasWarpBallot, p__prop__val__arch__hasWarpShuffle, p__prop__val__arch__hasFunnelShift, p__prop__val__arch__hasThreadFenceSystem, p__prop__val__arch__hasSyncThreadsExt, p__prop__val__arch__hasSurfaceFuncs, p__prop__val__arch__has3dGrid, p__prop__val__arch__hasDynamicParallelism, p__prop__val__concurrentKernels, p__prop__val__pciDomainID, p__prop__val__pciBusID, p__prop__val__pciDeviceID, p__prop__val__maxSharedMemoryPerMultiProcessor, p__prop__val__isMultiGpuBoard, p__prop__val__canMapHostMemory, p__prop__val__gcnArch, p__prop__val__gcnArchName, p__prop__val__integrated, p__prop__val__cooperativeLaunch, p__prop__val__cooperativeMultiDeviceLaunch, p__prop__val__maxTexture1DLinear, p__prop__val__maxTexture1D, p__prop__val__maxTexture2D, p__prop__val__maxTexture3D, p__prop__val__hdpMemFlushCntl, p__prop__val__hdpRegFlushCntl, p__prop__val__memPitch, p__prop__val__textureAlignment, p__prop__val__texturePitchAlignment, p__prop__val__kernelExecTimeoutEnabled, p__prop__val__ECCEnabled, p__prop__val__tccDriver, p__prop__val__cooperativeMultiDeviceUnmatchedFunc, p__prop__val__cooperativeMultiDeviceUnmatchedGridDim, p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim, p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem, p__prop__val__isLargeBar, p__prop__val__asicRevision, p__prop__val__managedMemory, p__prop__val__directManagedMemAccessFromHost, p__prop__val__concurrentManagedAccess, p__prop__val__pageableMemoryAccess, p__prop__val__pageableMemoryAccessUsesHostPageTables, p__device);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetDevicePropertiesR0000End`) */
void barectf_hip_api_trace_hipGetDevicePropertiesR0000End(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetDevicePropertiesR0000End(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetDevicePropertiesR0000End(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetDevicePropertiesR0600Begin`) */
void barectf_hip_api_trace_hipGetDevicePropertiesR0600Begin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__prop,
	const uint64_t p__prop__val__name,
	const uint64_t p__prop__val__uuid__bytes,
	const uint64_t p__prop__val__luid,
	const uint32_t p__prop__val__luidDeviceNodeMask,
	const uint64_t p__prop__val__totalGlobalMem,
	const uint64_t p__prop__val__sharedMemPerBlock,
	const int32_t p__prop__val__regsPerBlock,
	const int32_t p__prop__val__warpSize,
	const uint64_t p__prop__val__memPitch,
	const int32_t p__prop__val__maxThreadsPerBlock,
	const uint64_t p__prop__val__maxThreadsDim,
	const uint64_t p__prop__val__maxGridSize,
	const int32_t p__prop__val__clockRate,
	const uint64_t p__prop__val__totalConstMem,
	const int32_t p__prop__val__major,
	const int32_t p__prop__val__minor,
	const uint64_t p__prop__val__textureAlignment,
	const uint64_t p__prop__val__texturePitchAlignment,
	const int32_t p__prop__val__deviceOverlap,
	const int32_t p__prop__val__multiProcessorCount,
	const int32_t p__prop__val__kernelExecTimeoutEnabled,
	const int32_t p__prop__val__integrated,
	const int32_t p__prop__val__canMapHostMemory,
	const int32_t p__prop__val__computeMode,
	const int32_t p__prop__val__maxTexture1D,
	const int32_t p__prop__val__maxTexture1DMipmap,
	const int32_t p__prop__val__maxTexture1DLinear,
	const uint64_t p__prop__val__maxTexture2D,
	const uint64_t p__prop__val__maxTexture2DMipmap,
	const uint64_t p__prop__val__maxTexture2DLinear,
	const uint64_t p__prop__val__maxTexture2DGather,
	const uint64_t p__prop__val__maxTexture3D,
	const uint64_t p__prop__val__maxTexture3DAlt,
	const int32_t p__prop__val__maxTextureCubemap,
	const uint64_t p__prop__val__maxTexture1DLayered,
	const uint64_t p__prop__val__maxTexture2DLayered,
	const uint64_t p__prop__val__maxTextureCubemapLayered,
	const int32_t p__prop__val__maxSurface1D,
	const uint64_t p__prop__val__maxSurface2D,
	const uint64_t p__prop__val__maxSurface3D,
	const uint64_t p__prop__val__maxSurface1DLayered,
	const uint64_t p__prop__val__maxSurface2DLayered,
	const int32_t p__prop__val__maxSurfaceCubemap,
	const uint64_t p__prop__val__maxSurfaceCubemapLayered,
	const uint64_t p__prop__val__surfaceAlignment,
	const int32_t p__prop__val__concurrentKernels,
	const int32_t p__prop__val__ECCEnabled,
	const int32_t p__prop__val__pciBusID,
	const int32_t p__prop__val__pciDeviceID,
	const int32_t p__prop__val__pciDomainID,
	const int32_t p__prop__val__tccDriver,
	const int32_t p__prop__val__asyncEngineCount,
	const int32_t p__prop__val__unifiedAddressing,
	const int32_t p__prop__val__memoryClockRate,
	const int32_t p__prop__val__memoryBusWidth,
	const int32_t p__prop__val__l2CacheSize,
	const int32_t p__prop__val__persistingL2CacheMaxSize,
	const int32_t p__prop__val__maxThreadsPerMultiProcessor,
	const int32_t p__prop__val__streamPrioritiesSupported,
	const int32_t p__prop__val__globalL1CacheSupported,
	const int32_t p__prop__val__localL1CacheSupported,
	const uint64_t p__prop__val__sharedMemPerMultiprocessor,
	const int32_t p__prop__val__regsPerMultiprocessor,
	const int32_t p__prop__val__managedMemory,
	const int32_t p__prop__val__isMultiGpuBoard,
	const int32_t p__prop__val__multiGpuBoardGroupID,
	const int32_t p__prop__val__hostNativeAtomicSupported,
	const int32_t p__prop__val__singleToDoublePrecisionPerfRatio,
	const int32_t p__prop__val__pageableMemoryAccess,
	const int32_t p__prop__val__concurrentManagedAccess,
	const int32_t p__prop__val__computePreemptionSupported,
	const int32_t p__prop__val__canUseHostPointerForRegisteredMem,
	const int32_t p__prop__val__cooperativeLaunch,
	const int32_t p__prop__val__cooperativeMultiDeviceLaunch,
	const uint64_t p__prop__val__sharedMemPerBlockOptin,
	const int32_t p__prop__val__pageableMemoryAccessUsesHostPageTables,
	const int32_t p__prop__val__directManagedMemAccessFromHost,
	const int32_t p__prop__val__maxBlocksPerMultiProcessor,
	const int32_t p__prop__val__accessPolicyMaxWindowSize,
	const uint64_t p__prop__val__reservedSharedMemPerBlock,
	const int32_t p__prop__val__hostRegisterSupported,
	const int32_t p__prop__val__sparseHipArraySupported,
	const int32_t p__prop__val__hostRegisterReadOnlySupported,
	const int32_t p__prop__val__timelineSemaphoreInteropSupported,
	const int32_t p__prop__val__memoryPoolsSupported,
	const int32_t p__prop__val__gpuDirectRDMASupported,
	const uint32_t p__prop__val__gpuDirectRDMAFlushWritesOptions,
	const int32_t p__prop__val__gpuDirectRDMAWritesOrdering,
	const uint32_t p__prop__val__memoryPoolSupportedHandleTypes,
	const int32_t p__prop__val__deferredMappingHipArraySupported,
	const int32_t p__prop__val__ipcEventSupported,
	const int32_t p__prop__val__clusterLaunch,
	const int32_t p__prop__val__unifiedFunctionPointers,
	const uint64_t p__prop__val__reserved,
	const uint64_t p__prop__val__hipReserved,
	const uint64_t p__prop__val__gcnArchName,
	const uint64_t p__prop__val__maxSharedMemoryPerMultiProcessor,
	const int32_t p__prop__val__clockInstructionRate,
	const uint32_t p__prop__val__arch__hasGlobalInt32Atomics,
	const uint32_t p__prop__val__arch__hasGlobalFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasSharedInt32Atomics,
	const uint32_t p__prop__val__arch__hasSharedFloatAtomicExch,
	const uint32_t p__prop__val__arch__hasFloatAtomicAdd,
	const uint32_t p__prop__val__arch__hasGlobalInt64Atomics,
	const uint32_t p__prop__val__arch__hasSharedInt64Atomics,
	const uint32_t p__prop__val__arch__hasDoubles,
	const uint32_t p__prop__val__arch__hasWarpVote,
	const uint32_t p__prop__val__arch__hasWarpBallot,
	const uint32_t p__prop__val__arch__hasWarpShuffle,
	const uint32_t p__prop__val__arch__hasFunnelShift,
	const uint32_t p__prop__val__arch__hasThreadFenceSystem,
	const uint32_t p__prop__val__arch__hasSyncThreadsExt,
	const uint32_t p__prop__val__arch__hasSurfaceFuncs,
	const uint32_t p__prop__val__arch__has3dGrid,
	const uint32_t p__prop__val__arch__hasDynamicParallelism,
	const uint64_t p__prop__val__hdpMemFlushCntl,
	const uint64_t p__prop__val__hdpRegFlushCntl,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedFunc,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedGridDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim,
	const int32_t p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem,
	const int32_t p__prop__val__isLargeBar,
	const int32_t p__prop__val__asicRevision,
	const int32_t p__deviceId)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetDevicePropertiesR0600Begin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetDevicePropertiesR0600Begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__prop, p__prop__val__name, p__prop__val__uuid__bytes, p__prop__val__luid, p__prop__val__luidDeviceNodeMask, p__prop__val__totalGlobalMem, p__prop__val__sharedMemPerBlock, p__prop__val__regsPerBlock, p__prop__val__warpSize, p__prop__val__memPitch, p__prop__val__maxThreadsPerBlock, p__prop__val__maxThreadsDim, p__prop__val__maxGridSize, p__prop__val__clockRate, p__prop__val__totalConstMem, p__prop__val__major, p__prop__val__minor, p__prop__val__textureAlignment, p__prop__val__texturePitchAlignment, p__prop__val__deviceOverlap, p__prop__val__multiProcessorCount, p__prop__val__kernelExecTimeoutEnabled, p__prop__val__integrated, p__prop__val__canMapHostMemory, p__prop__val__computeMode, p__prop__val__maxTexture1D, p__prop__val__maxTexture1DMipmap, p__prop__val__maxTexture1DLinear, p__prop__val__maxTexture2D, p__prop__val__maxTexture2DMipmap, p__prop__val__maxTexture2DLinear, p__prop__val__maxTexture2DGather, p__prop__val__maxTexture3D, p__prop__val__maxTexture3DAlt, p__prop__val__maxTextureCubemap, p__prop__val__maxTexture1DLayered, p__prop__val__maxTexture2DLayered, p__prop__val__maxTextureCubemapLayered, p__prop__val__maxSurface1D, p__prop__val__maxSurface2D, p__prop__val__maxSurface3D, p__prop__val__maxSurface1DLayered, p__prop__val__maxSurface2DLayered, p__prop__val__maxSurfaceCubemap, p__prop__val__maxSurfaceCubemapLayered, p__prop__val__surfaceAlignment, p__prop__val__concurrentKernels, p__prop__val__ECCEnabled, p__prop__val__pciBusID, p__prop__val__pciDeviceID, p__prop__val__pciDomainID, p__prop__val__tccDriver, p__prop__val__asyncEngineCount, p__prop__val__unifiedAddressing, p__prop__val__memoryClockRate, p__prop__val__memoryBusWidth, p__prop__val__l2CacheSize, p__prop__val__persistingL2CacheMaxSize, p__prop__val__maxThreadsPerMultiProcessor, p__prop__val__streamPrioritiesSupported, p__prop__val__globalL1CacheSupported, p__prop__val__localL1CacheSupported, p__prop__val__sharedMemPerMultiprocessor, p__prop__val__regsPerMultiprocessor, p__prop__val__managedMemory, p__prop__val__isMultiGpuBoard, p__prop__val__multiGpuBoardGroupID, p__prop__val__hostNativeAtomicSupported, p__prop__val__singleToDoublePrecisionPerfRatio, p__prop__val__pageableMemoryAccess, p__prop__val__concurrentManagedAccess, p__prop__val__computePreemptionSupported, p__prop__val__canUseHostPointerForRegisteredMem, p__prop__val__cooperativeLaunch, p__prop__val__cooperativeMultiDeviceLaunch, p__prop__val__sharedMemPerBlockOptin, p__prop__val__pageableMemoryAccessUsesHostPageTables, p__prop__val__directManagedMemAccessFromHost, p__prop__val__maxBlocksPerMultiProcessor, p__prop__val__accessPolicyMaxWindowSize, p__prop__val__reservedSharedMemPerBlock, p__prop__val__hostRegisterSupported, p__prop__val__sparseHipArraySupported, p__prop__val__hostRegisterReadOnlySupported, p__prop__val__timelineSemaphoreInteropSupported, p__prop__val__memoryPoolsSupported, p__prop__val__gpuDirectRDMASupported, p__prop__val__gpuDirectRDMAFlushWritesOptions, p__prop__val__gpuDirectRDMAWritesOrdering, p__prop__val__memoryPoolSupportedHandleTypes, p__prop__val__deferredMappingHipArraySupported, p__prop__val__ipcEventSupported, p__prop__val__clusterLaunch, p__prop__val__unifiedFunctionPointers, p__prop__val__reserved, p__prop__val__hipReserved, p__prop__val__gcnArchName, p__prop__val__maxSharedMemoryPerMultiProcessor, p__prop__val__clockInstructionRate, p__prop__val__arch__hasGlobalInt32Atomics, p__prop__val__arch__hasGlobalFloatAtomicExch, p__prop__val__arch__hasSharedInt32Atomics, p__prop__val__arch__hasSharedFloatAtomicExch, p__prop__val__arch__hasFloatAtomicAdd, p__prop__val__arch__hasGlobalInt64Atomics, p__prop__val__arch__hasSharedInt64Atomics, p__prop__val__arch__hasDoubles, p__prop__val__arch__hasWarpVote, p__prop__val__arch__hasWarpBallot, p__prop__val__arch__hasWarpShuffle, p__prop__val__arch__hasFunnelShift, p__prop__val__arch__hasThreadFenceSystem, p__prop__val__arch__hasSyncThreadsExt, p__prop__val__arch__hasSurfaceFuncs, p__prop__val__arch__has3dGrid, p__prop__val__arch__hasDynamicParallelism, p__prop__val__hdpMemFlushCntl, p__prop__val__hdpRegFlushCntl, p__prop__val__cooperativeMultiDeviceUnmatchedFunc, p__prop__val__cooperativeMultiDeviceUnmatchedGridDim, p__prop__val__cooperativeMultiDeviceUnmatchedBlockDim, p__prop__val__cooperativeMultiDeviceUnmatchedSharedMem, p__prop__val__isLargeBar, p__prop__val__asicRevision, p__deviceId);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetDevicePropertiesR0600End`) */
void barectf_hip_api_trace_hipGetDevicePropertiesR0600End(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetDevicePropertiesR0600End(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetDevicePropertiesR0600End(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetErrorStringBegin`) */
void barectf_hip_api_trace_hipGetErrorStringBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetErrorStringBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetErrorStringBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetErrorStringEnd`) */
void barectf_hip_api_trace_hipGetErrorStringEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetErrorStringEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetErrorStringEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetFuncBySymbolBegin`) */
void barectf_hip_api_trace_hipGetFuncBySymbolBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__functionPtr,
	const uint64_t p__functionPtr__val,
	const uint64_t p__symbolPtr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetFuncBySymbolBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetFuncBySymbolBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__functionPtr, p__functionPtr__val, p__symbolPtr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetFuncBySymbolEnd`) */
void barectf_hip_api_trace_hipGetFuncBySymbolEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetFuncBySymbolEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetFuncBySymbolEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetLastErrorBegin`) */
void barectf_hip_api_trace_hipGetLastErrorBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetLastErrorBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetLastErrorBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetLastErrorEnd`) */
void barectf_hip_api_trace_hipGetLastErrorEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetLastErrorEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetLastErrorEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetMipmappedArrayLevelBegin`) */
void barectf_hip_api_trace_hipGetMipmappedArrayLevelBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__levelArray,
	const uint64_t p__levelArray__val,
	const uint64_t p__mipmappedArray,
	const uint32_t p__level)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetMipmappedArrayLevelBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetMipmappedArrayLevelBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__levelArray, p__levelArray__val, p__mipmappedArray, p__level);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetMipmappedArrayLevelEnd`) */
void barectf_hip_api_trace_hipGetMipmappedArrayLevelEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetMipmappedArrayLevelEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetMipmappedArrayLevelEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetProcAddressBegin`) */
void barectf_hip_api_trace_hipGetProcAddressBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const char * const p__symbol,
	const int8_t p__symbol__val,
	const uint64_t p__pfn,
	const uint64_t p__pfn__val,
	const int32_t p__hipVersion,
	const uint64_t p__flags,
	const uint64_t p__symbolStatus,
	const int32_t p__symbolStatus__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetProcAddressBegin(_TO_VOID_PTR(ctx), cc__kernel_name, p__symbol);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetProcAddressBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__symbol, p__symbol__val, p__pfn, p__pfn__val, p__hipVersion, p__flags, p__symbolStatus, p__symbolStatus__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetProcAddressEnd`) */
void barectf_hip_api_trace_hipGetProcAddressEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetProcAddressEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetProcAddressEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetSymbolAddressBegin`) */
void barectf_hip_api_trace_hipGetSymbolAddressBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr,
	const uint64_t p__devPtr__val,
	const uint64_t p__symbol)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetSymbolAddressBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetSymbolAddressBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__devPtr, p__devPtr__val, p__symbol);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetSymbolAddressEnd`) */
void barectf_hip_api_trace_hipGetSymbolAddressEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetSymbolAddressEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetSymbolAddressEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetSymbolSizeBegin`) */
void barectf_hip_api_trace_hipGetSymbolSizeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__size,
	const uint64_t p__size__val,
	const uint64_t p__symbol)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetSymbolSizeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetSymbolSizeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__size, p__size__val, p__symbol);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGetSymbolSizeEnd`) */
void barectf_hip_api_trace_hipGetSymbolSizeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGetSymbolSizeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGetSymbolSizeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddChildGraphNodeBegin`) */
void barectf_hip_api_trace_hipGraphAddChildGraphNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__childGraph)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddChildGraphNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddChildGraphNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__childGraph);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddChildGraphNodeEnd`) */
void barectf_hip_api_trace_hipGraphAddChildGraphNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddChildGraphNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddChildGraphNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddDependenciesBegin`) */
void barectf_hip_api_trace_hipGraphAddDependenciesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__from,
	const uint64_t p__from__val,
	const uint64_t p__to,
	const uint64_t p__to__val,
	const uint64_t p__numDependencies)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddDependenciesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddDependenciesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__graph, p__from, p__from__val, p__to, p__to__val, p__numDependencies);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddDependenciesEnd`) */
void barectf_hip_api_trace_hipGraphAddDependenciesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddDependenciesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddDependenciesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddEmptyNodeBegin`) */
void barectf_hip_api_trace_hipGraphAddEmptyNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddEmptyNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddEmptyNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddEmptyNodeEnd`) */
void barectf_hip_api_trace_hipGraphAddEmptyNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddEmptyNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddEmptyNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddEventRecordNodeBegin`) */
void barectf_hip_api_trace_hipGraphAddEventRecordNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddEventRecordNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddEventRecordNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__event);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddEventRecordNodeEnd`) */
void barectf_hip_api_trace_hipGraphAddEventRecordNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddEventRecordNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddEventRecordNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddEventWaitNodeBegin`) */
void barectf_hip_api_trace_hipGraphAddEventWaitNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddEventWaitNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddEventWaitNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__event);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddEventWaitNodeEnd`) */
void barectf_hip_api_trace_hipGraphAddEventWaitNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddEventWaitNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddEventWaitNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddExternalSemaphoresSignalNodeBegin`) */
void barectf_hip_api_trace_hipGraphAddExternalSemaphoresSignalNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__nodeParams,
	const uint64_t p__nodeParams__val__extSemArray,
	const uint64_t p__nodeParams__val__paramsArray,
	const uint32_t p__nodeParams__val__numExtSems)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddExternalSemaphoresSignalNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddExternalSemaphoresSignalNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__nodeParams, p__nodeParams__val__extSemArray, p__nodeParams__val__paramsArray, p__nodeParams__val__numExtSems);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddExternalSemaphoresSignalNodeEnd`) */
void barectf_hip_api_trace_hipGraphAddExternalSemaphoresSignalNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddExternalSemaphoresSignalNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddExternalSemaphoresSignalNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddExternalSemaphoresWaitNodeBegin`) */
void barectf_hip_api_trace_hipGraphAddExternalSemaphoresWaitNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__nodeParams,
	const uint64_t p__nodeParams__val__extSemArray,
	const uint64_t p__nodeParams__val__paramsArray,
	const uint32_t p__nodeParams__val__numExtSems)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddExternalSemaphoresWaitNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddExternalSemaphoresWaitNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__nodeParams, p__nodeParams__val__extSemArray, p__nodeParams__val__paramsArray, p__nodeParams__val__numExtSems);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddExternalSemaphoresWaitNodeEnd`) */
void barectf_hip_api_trace_hipGraphAddExternalSemaphoresWaitNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddExternalSemaphoresWaitNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddExternalSemaphoresWaitNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddHostNodeBegin`) */
void barectf_hip_api_trace_hipGraphAddHostNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__fn,
	const uint64_t p__pNodeParams__val__userData)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddHostNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddHostNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__pNodeParams, p__pNodeParams__val__fn, p__pNodeParams__val__userData);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddHostNodeEnd`) */
void barectf_hip_api_trace_hipGraphAddHostNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddHostNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddHostNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddKernelNodeBegin`) */
void barectf_hip_api_trace_hipGraphAddKernelNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__pNodeParams,
	const uint32_t p__pNodeParams__val__blockDim__x,
	const uint32_t p__pNodeParams__val__blockDim__y,
	const uint32_t p__pNodeParams__val__blockDim__z,
	const uint64_t p__pNodeParams__val__extra,
	const uint64_t p__pNodeParams__val__func,
	const uint32_t p__pNodeParams__val__gridDim__x,
	const uint32_t p__pNodeParams__val__gridDim__y,
	const uint32_t p__pNodeParams__val__gridDim__z,
	const uint64_t p__pNodeParams__val__kernelParams,
	const uint32_t p__pNodeParams__val__sharedMemBytes)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddKernelNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddKernelNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__pNodeParams, p__pNodeParams__val__blockDim__x, p__pNodeParams__val__blockDim__y, p__pNodeParams__val__blockDim__z, p__pNodeParams__val__extra, p__pNodeParams__val__func, p__pNodeParams__val__gridDim__x, p__pNodeParams__val__gridDim__y, p__pNodeParams__val__gridDim__z, p__pNodeParams__val__kernelParams, p__pNodeParams__val__sharedMemBytes);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddKernelNodeEnd`) */
void barectf_hip_api_trace_hipGraphAddKernelNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddKernelNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddKernelNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemAllocNodeBegin`) */
void barectf_hip_api_trace_hipGraphAddMemAllocNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__pNodeParams,
	const int32_t p__pNodeParams__val__poolProps__allocType,
	const int32_t p__pNodeParams__val__poolProps__handleTypes,
	const int32_t p__pNodeParams__val__poolProps__location__type,
	const int32_t p__pNodeParams__val__poolProps__location__id,
	const uint64_t p__pNodeParams__val__poolProps__win32SecurityAttributes,
	const uint64_t p__pNodeParams__val__poolProps__maxSize,
	const uint64_t p__pNodeParams__val__poolProps__reserved,
	const uint64_t p__pNodeParams__val__accessDescs,
	const uint64_t p__pNodeParams__val__accessDescCount,
	const uint64_t p__pNodeParams__val__bytesize,
	const uint64_t p__pNodeParams__val__dptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemAllocNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemAllocNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__pNodeParams, p__pNodeParams__val__poolProps__allocType, p__pNodeParams__val__poolProps__handleTypes, p__pNodeParams__val__poolProps__location__type, p__pNodeParams__val__poolProps__location__id, p__pNodeParams__val__poolProps__win32SecurityAttributes, p__pNodeParams__val__poolProps__maxSize, p__pNodeParams__val__poolProps__reserved, p__pNodeParams__val__accessDescs, p__pNodeParams__val__accessDescCount, p__pNodeParams__val__bytesize, p__pNodeParams__val__dptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemAllocNodeEnd`) */
void barectf_hip_api_trace_hipGraphAddMemAllocNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemAllocNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemAllocNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemFreeNodeBegin`) */
void barectf_hip_api_trace_hipGraphAddMemFreeNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__dev_ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemFreeNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemFreeNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__dev_ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemFreeNodeEnd`) */
void barectf_hip_api_trace_hipGraphAddMemFreeNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemFreeNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemFreeNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemcpyNode1DBegin`) */
void barectf_hip_api_trace_hipGraphAddMemcpyNode1DBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__count,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemcpyNode1DBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemcpyNode1DBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__dst, p__src, p__count, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemcpyNode1DEnd`) */
void barectf_hip_api_trace_hipGraphAddMemcpyNode1DEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemcpyNode1DEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemcpyNode1DEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemcpyNodeBegin`) */
void barectf_hip_api_trace_hipGraphAddMemcpyNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__pCopyParams,
	const uint64_t p__pCopyParams__val__srcArray,
	const uint64_t p__pCopyParams__val__srcPos__x,
	const uint64_t p__pCopyParams__val__srcPos__y,
	const uint64_t p__pCopyParams__val__srcPos__z,
	const uint64_t p__pCopyParams__val__srcPtr__ptr,
	const uint64_t p__pCopyParams__val__srcPtr__pitch,
	const uint64_t p__pCopyParams__val__srcPtr__xsize,
	const uint64_t p__pCopyParams__val__srcPtr__ysize,
	const uint64_t p__pCopyParams__val__dstArray,
	const uint64_t p__pCopyParams__val__dstPos__x,
	const uint64_t p__pCopyParams__val__dstPos__y,
	const uint64_t p__pCopyParams__val__dstPos__z,
	const uint64_t p__pCopyParams__val__dstPtr__ptr,
	const uint64_t p__pCopyParams__val__dstPtr__pitch,
	const uint64_t p__pCopyParams__val__dstPtr__xsize,
	const uint64_t p__pCopyParams__val__dstPtr__ysize,
	const uint64_t p__pCopyParams__val__extent__width,
	const uint64_t p__pCopyParams__val__extent__height,
	const uint64_t p__pCopyParams__val__extent__depth,
	const int32_t p__pCopyParams__val__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemcpyNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemcpyNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__pCopyParams, p__pCopyParams__val__srcArray, p__pCopyParams__val__srcPos__x, p__pCopyParams__val__srcPos__y, p__pCopyParams__val__srcPos__z, p__pCopyParams__val__srcPtr__ptr, p__pCopyParams__val__srcPtr__pitch, p__pCopyParams__val__srcPtr__xsize, p__pCopyParams__val__srcPtr__ysize, p__pCopyParams__val__dstArray, p__pCopyParams__val__dstPos__x, p__pCopyParams__val__dstPos__y, p__pCopyParams__val__dstPos__z, p__pCopyParams__val__dstPtr__ptr, p__pCopyParams__val__dstPtr__pitch, p__pCopyParams__val__dstPtr__xsize, p__pCopyParams__val__dstPtr__ysize, p__pCopyParams__val__extent__width, p__pCopyParams__val__extent__height, p__pCopyParams__val__extent__depth, p__pCopyParams__val__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemcpyNodeEnd`) */
void barectf_hip_api_trace_hipGraphAddMemcpyNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemcpyNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemcpyNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemcpyNodeFromSymbolBegin`) */
void barectf_hip_api_trace_hipGraphAddMemcpyNodeFromSymbolBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__dst,
	const uint64_t p__symbol,
	const uint64_t p__count,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemcpyNodeFromSymbolBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemcpyNodeFromSymbolBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__dst, p__symbol, p__count, p__offset, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemcpyNodeFromSymbolEnd`) */
void barectf_hip_api_trace_hipGraphAddMemcpyNodeFromSymbolEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemcpyNodeFromSymbolEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemcpyNodeFromSymbolEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemcpyNodeToSymbolBegin`) */
void barectf_hip_api_trace_hipGraphAddMemcpyNodeToSymbolBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__symbol,
	const uint64_t p__src,
	const uint64_t p__count,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemcpyNodeToSymbolBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemcpyNodeToSymbolBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__symbol, p__src, p__count, p__offset, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemcpyNodeToSymbolEnd`) */
void barectf_hip_api_trace_hipGraphAddMemcpyNodeToSymbolEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemcpyNodeToSymbolEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemcpyNodeToSymbolEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemsetNodeBegin`) */
void barectf_hip_api_trace_hipGraphAddMemsetNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__pMemsetParams,
	const uint64_t p__pMemsetParams__val__dst,
	const uint32_t p__pMemsetParams__val__elementSize,
	const uint64_t p__pMemsetParams__val__height,
	const uint64_t p__pMemsetParams__val__pitch,
	const uint32_t p__pMemsetParams__val__value,
	const uint64_t p__pMemsetParams__val__width)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemsetNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemsetNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__pMemsetParams, p__pMemsetParams__val__dst, p__pMemsetParams__val__elementSize, p__pMemsetParams__val__height, p__pMemsetParams__val__pitch, p__pMemsetParams__val__value, p__pMemsetParams__val__width);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddMemsetNodeEnd`) */
void barectf_hip_api_trace_hipGraphAddMemsetNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddMemsetNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddMemsetNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddNodeBegin`) */
void barectf_hip_api_trace_hipGraphAddNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphNode,
	const uint64_t p__pGraphNode__val,
	const uint64_t p__graph,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__numDependencies,
	const uint64_t p__nodeParams,
	const int32_t p__nodeParams__val__type,
	const uint64_t p__nodeParams__val__reserved0,
	const int64_t p__nodeParams__val__reserved2)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphNode, p__pGraphNode__val, p__graph, p__pDependencies, p__pDependencies__val, p__numDependencies, p__nodeParams, p__nodeParams__val__type, p__nodeParams__val__reserved0, p__nodeParams__val__reserved2);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphAddNodeEnd`) */
void barectf_hip_api_trace_hipGraphAddNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphAddNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphAddNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphChildGraphNodeGetGraphBegin`) */
void barectf_hip_api_trace_hipGraphChildGraphNodeGetGraphBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pGraph,
	const uint64_t p__pGraph__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphChildGraphNodeGetGraphBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphChildGraphNodeGetGraphBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__pGraph, p__pGraph__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphChildGraphNodeGetGraphEnd`) */
void barectf_hip_api_trace_hipGraphChildGraphNodeGetGraphEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphChildGraphNodeGetGraphEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphChildGraphNodeGetGraphEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphCloneBegin`) */
void barectf_hip_api_trace_hipGraphCloneBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphClone,
	const uint64_t p__pGraphClone__val,
	const uint64_t p__originalGraph)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphCloneBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphCloneBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphClone, p__pGraphClone__val, p__originalGraph);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphCloneEnd`) */
void barectf_hip_api_trace_hipGraphCloneEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphCloneEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphCloneEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphCreateBegin`) */
void barectf_hip_api_trace_hipGraphCreateBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraph,
	const uint64_t p__pGraph__val,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphCreateBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphCreateBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraph, p__pGraph__val, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphCreateEnd`) */
void barectf_hip_api_trace_hipGraphCreateEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphCreateEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphCreateEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphDebugDotPrintBegin`) */
void barectf_hip_api_trace_hipGraphDebugDotPrintBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const char * const p__path,
	const int8_t p__path__val,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphDebugDotPrintBegin(_TO_VOID_PTR(ctx), cc__kernel_name, p__path);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphDebugDotPrintBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__graph, p__path, p__path__val, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphDebugDotPrintEnd`) */
void barectf_hip_api_trace_hipGraphDebugDotPrintEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphDebugDotPrintEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphDebugDotPrintEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphDestroyBegin`) */
void barectf_hip_api_trace_hipGraphDestroyBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphDestroyBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphDestroyBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__graph);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphDestroyEnd`) */
void barectf_hip_api_trace_hipGraphDestroyEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphDestroyEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphDestroyEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphDestroyNodeBegin`) */
void barectf_hip_api_trace_hipGraphDestroyNodeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphDestroyNodeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphDestroyNodeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphDestroyNodeEnd`) */
void barectf_hip_api_trace_hipGraphDestroyNodeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphDestroyNodeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphDestroyNodeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphEventRecordNodeGetEventBegin`) */
void barectf_hip_api_trace_hipGraphEventRecordNodeGetEventBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__event_out,
	const uint64_t p__event_out__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphEventRecordNodeGetEventBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphEventRecordNodeGetEventBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__event_out, p__event_out__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphEventRecordNodeGetEventEnd`) */
void barectf_hip_api_trace_hipGraphEventRecordNodeGetEventEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphEventRecordNodeGetEventEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphEventRecordNodeGetEventEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphEventRecordNodeSetEventBegin`) */
void barectf_hip_api_trace_hipGraphEventRecordNodeSetEventBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphEventRecordNodeSetEventBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphEventRecordNodeSetEventBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__event);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphEventRecordNodeSetEventEnd`) */
void barectf_hip_api_trace_hipGraphEventRecordNodeSetEventEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphEventRecordNodeSetEventEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphEventRecordNodeSetEventEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphEventWaitNodeGetEventBegin`) */
void barectf_hip_api_trace_hipGraphEventWaitNodeGetEventBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__event_out,
	const uint64_t p__event_out__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphEventWaitNodeGetEventBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphEventWaitNodeGetEventBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__event_out, p__event_out__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphEventWaitNodeGetEventEnd`) */
void barectf_hip_api_trace_hipGraphEventWaitNodeGetEventEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphEventWaitNodeGetEventEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphEventWaitNodeGetEventEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphEventWaitNodeSetEventBegin`) */
void barectf_hip_api_trace_hipGraphEventWaitNodeSetEventBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphEventWaitNodeSetEventBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphEventWaitNodeSetEventBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__event);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphEventWaitNodeSetEventEnd`) */
void barectf_hip_api_trace_hipGraphEventWaitNodeSetEventEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphEventWaitNodeSetEventEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphEventWaitNodeSetEventEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecChildGraphNodeSetParamsBegin`) */
void barectf_hip_api_trace_hipGraphExecChildGraphNodeSetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__childGraph)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecChildGraphNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecChildGraphNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__node, p__childGraph);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecChildGraphNodeSetParamsEnd`) */
void barectf_hip_api_trace_hipGraphExecChildGraphNodeSetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecChildGraphNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecChildGraphNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecDestroyBegin`) */
void barectf_hip_api_trace_hipGraphExecDestroyBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graphExec)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecDestroyBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecDestroyBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__graphExec);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecDestroyEnd`) */
void barectf_hip_api_trace_hipGraphExecDestroyEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecDestroyEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecDestroyEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecEventRecordNodeSetEventBegin`) */
void barectf_hip_api_trace_hipGraphExecEventRecordNodeSetEventBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hNode,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecEventRecordNodeSetEventBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecEventRecordNodeSetEventBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__hNode, p__event);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecEventRecordNodeSetEventEnd`) */
void barectf_hip_api_trace_hipGraphExecEventRecordNodeSetEventEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecEventRecordNodeSetEventEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecEventRecordNodeSetEventEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecEventWaitNodeSetEventBegin`) */
void barectf_hip_api_trace_hipGraphExecEventWaitNodeSetEventBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hNode,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecEventWaitNodeSetEventBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecEventWaitNodeSetEventBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__hNode, p__event);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecEventWaitNodeSetEventEnd`) */
void barectf_hip_api_trace_hipGraphExecEventWaitNodeSetEventEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecEventWaitNodeSetEventEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecEventWaitNodeSetEventEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecExternalSemaphoresSignalNodeSetParamsBegin`) */
void barectf_hip_api_trace_hipGraphExecExternalSemaphoresSignalNodeSetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hNode,
	const uint64_t p__nodeParams,
	const uint64_t p__nodeParams__val__extSemArray,
	const uint64_t p__nodeParams__val__paramsArray,
	const uint32_t p__nodeParams__val__numExtSems)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecExternalSemaphoresSignalNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecExternalSemaphoresSignalNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__hNode, p__nodeParams, p__nodeParams__val__extSemArray, p__nodeParams__val__paramsArray, p__nodeParams__val__numExtSems);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecExternalSemaphoresSignalNodeSetParamsEnd`) */
void barectf_hip_api_trace_hipGraphExecExternalSemaphoresSignalNodeSetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecExternalSemaphoresSignalNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecExternalSemaphoresSignalNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecExternalSemaphoresWaitNodeSetParamsBegin`) */
void barectf_hip_api_trace_hipGraphExecExternalSemaphoresWaitNodeSetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hNode,
	const uint64_t p__nodeParams,
	const uint64_t p__nodeParams__val__extSemArray,
	const uint64_t p__nodeParams__val__paramsArray,
	const uint32_t p__nodeParams__val__numExtSems)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecExternalSemaphoresWaitNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecExternalSemaphoresWaitNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__hNode, p__nodeParams, p__nodeParams__val__extSemArray, p__nodeParams__val__paramsArray, p__nodeParams__val__numExtSems);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecExternalSemaphoresWaitNodeSetParamsEnd`) */
void barectf_hip_api_trace_hipGraphExecExternalSemaphoresWaitNodeSetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecExternalSemaphoresWaitNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecExternalSemaphoresWaitNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecHostNodeSetParamsBegin`) */
void barectf_hip_api_trace_hipGraphExecHostNodeSetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__fn,
	const uint64_t p__pNodeParams__val__userData)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecHostNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecHostNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__node, p__pNodeParams, p__pNodeParams__val__fn, p__pNodeParams__val__userData);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecHostNodeSetParamsEnd`) */
void barectf_hip_api_trace_hipGraphExecHostNodeSetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecHostNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecHostNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecKernelNodeSetParamsBegin`) */
void barectf_hip_api_trace_hipGraphExecKernelNodeSetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint32_t p__pNodeParams__val__blockDim__x,
	const uint32_t p__pNodeParams__val__blockDim__y,
	const uint32_t p__pNodeParams__val__blockDim__z,
	const uint64_t p__pNodeParams__val__extra,
	const uint64_t p__pNodeParams__val__func,
	const uint32_t p__pNodeParams__val__gridDim__x,
	const uint32_t p__pNodeParams__val__gridDim__y,
	const uint32_t p__pNodeParams__val__gridDim__z,
	const uint64_t p__pNodeParams__val__kernelParams,
	const uint32_t p__pNodeParams__val__sharedMemBytes)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecKernelNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecKernelNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__node, p__pNodeParams, p__pNodeParams__val__blockDim__x, p__pNodeParams__val__blockDim__y, p__pNodeParams__val__blockDim__z, p__pNodeParams__val__extra, p__pNodeParams__val__func, p__pNodeParams__val__gridDim__x, p__pNodeParams__val__gridDim__y, p__pNodeParams__val__gridDim__z, p__pNodeParams__val__kernelParams, p__pNodeParams__val__sharedMemBytes);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecKernelNodeSetParamsEnd`) */
void barectf_hip_api_trace_hipGraphExecKernelNodeSetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecKernelNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecKernelNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecMemcpyNodeSetParams1DBegin`) */
void barectf_hip_api_trace_hipGraphExecMemcpyNodeSetParams1DBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__count,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecMemcpyNodeSetParams1DBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecMemcpyNodeSetParams1DBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__node, p__dst, p__src, p__count, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecMemcpyNodeSetParams1DEnd`) */
void barectf_hip_api_trace_hipGraphExecMemcpyNodeSetParams1DEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecMemcpyNodeSetParams1DEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecMemcpyNodeSetParams1DEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecMemcpyNodeSetParamsBegin`) */
void barectf_hip_api_trace_hipGraphExecMemcpyNodeSetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__srcArray,
	const uint64_t p__pNodeParams__val__srcPos__x,
	const uint64_t p__pNodeParams__val__srcPos__y,
	const uint64_t p__pNodeParams__val__srcPos__z,
	const uint64_t p__pNodeParams__val__srcPtr__ptr,
	const uint64_t p__pNodeParams__val__srcPtr__pitch,
	const uint64_t p__pNodeParams__val__srcPtr__xsize,
	const uint64_t p__pNodeParams__val__srcPtr__ysize,
	const uint64_t p__pNodeParams__val__dstArray,
	const uint64_t p__pNodeParams__val__dstPos__x,
	const uint64_t p__pNodeParams__val__dstPos__y,
	const uint64_t p__pNodeParams__val__dstPos__z,
	const uint64_t p__pNodeParams__val__dstPtr__ptr,
	const uint64_t p__pNodeParams__val__dstPtr__pitch,
	const uint64_t p__pNodeParams__val__dstPtr__xsize,
	const uint64_t p__pNodeParams__val__dstPtr__ysize,
	const uint64_t p__pNodeParams__val__extent__width,
	const uint64_t p__pNodeParams__val__extent__height,
	const uint64_t p__pNodeParams__val__extent__depth,
	const int32_t p__pNodeParams__val__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecMemcpyNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecMemcpyNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__node, p__pNodeParams, p__pNodeParams__val__srcArray, p__pNodeParams__val__srcPos__x, p__pNodeParams__val__srcPos__y, p__pNodeParams__val__srcPos__z, p__pNodeParams__val__srcPtr__ptr, p__pNodeParams__val__srcPtr__pitch, p__pNodeParams__val__srcPtr__xsize, p__pNodeParams__val__srcPtr__ysize, p__pNodeParams__val__dstArray, p__pNodeParams__val__dstPos__x, p__pNodeParams__val__dstPos__y, p__pNodeParams__val__dstPos__z, p__pNodeParams__val__dstPtr__ptr, p__pNodeParams__val__dstPtr__pitch, p__pNodeParams__val__dstPtr__xsize, p__pNodeParams__val__dstPtr__ysize, p__pNodeParams__val__extent__width, p__pNodeParams__val__extent__height, p__pNodeParams__val__extent__depth, p__pNodeParams__val__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecMemcpyNodeSetParamsEnd`) */
void barectf_hip_api_trace_hipGraphExecMemcpyNodeSetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecMemcpyNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecMemcpyNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecMemcpyNodeSetParamsFromSymbolBegin`) */
void barectf_hip_api_trace_hipGraphExecMemcpyNodeSetParamsFromSymbolBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__dst,
	const uint64_t p__symbol,
	const uint64_t p__count,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecMemcpyNodeSetParamsFromSymbolBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecMemcpyNodeSetParamsFromSymbolBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__node, p__dst, p__symbol, p__count, p__offset, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecMemcpyNodeSetParamsFromSymbolEnd`) */
void barectf_hip_api_trace_hipGraphExecMemcpyNodeSetParamsFromSymbolEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecMemcpyNodeSetParamsFromSymbolEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecMemcpyNodeSetParamsFromSymbolEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecMemcpyNodeSetParamsToSymbolBegin`) */
void barectf_hip_api_trace_hipGraphExecMemcpyNodeSetParamsToSymbolBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__symbol,
	const uint64_t p__src,
	const uint64_t p__count,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecMemcpyNodeSetParamsToSymbolBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecMemcpyNodeSetParamsToSymbolBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__node, p__symbol, p__src, p__count, p__offset, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecMemcpyNodeSetParamsToSymbolEnd`) */
void barectf_hip_api_trace_hipGraphExecMemcpyNodeSetParamsToSymbolEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecMemcpyNodeSetParamsToSymbolEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecMemcpyNodeSetParamsToSymbolEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecMemsetNodeSetParamsBegin`) */
void barectf_hip_api_trace_hipGraphExecMemsetNodeSetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__dst,
	const uint32_t p__pNodeParams__val__elementSize,
	const uint64_t p__pNodeParams__val__height,
	const uint64_t p__pNodeParams__val__pitch,
	const uint32_t p__pNodeParams__val__value,
	const uint64_t p__pNodeParams__val__width)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecMemsetNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecMemsetNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__node, p__pNodeParams, p__pNodeParams__val__dst, p__pNodeParams__val__elementSize, p__pNodeParams__val__height, p__pNodeParams__val__pitch, p__pNodeParams__val__value, p__pNodeParams__val__width);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecMemsetNodeSetParamsEnd`) */
void barectf_hip_api_trace_hipGraphExecMemsetNodeSetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecMemsetNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecMemsetNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecUpdateBegin`) */
void barectf_hip_api_trace_hipGraphExecUpdateBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hGraph,
	const uint64_t p__hErrorNode_out,
	const uint64_t p__hErrorNode_out__val,
	const uint64_t p__updateResult_out,
	const int32_t p__updateResult_out__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecUpdateBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecUpdateBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__hGraph, p__hErrorNode_out, p__hErrorNode_out__val, p__updateResult_out, p__updateResult_out__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExecUpdateEnd`) */
void barectf_hip_api_trace_hipGraphExecUpdateEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExecUpdateEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExecUpdateEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExternalSemaphoresSignalNodeGetParamsBegin`) */
void barectf_hip_api_trace_hipGraphExternalSemaphoresSignalNodeGetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hNode,
	const uint64_t p__params_out,
	const uint64_t p__params_out__val__extSemArray,
	const uint64_t p__params_out__val__paramsArray,
	const uint32_t p__params_out__val__numExtSems)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExternalSemaphoresSignalNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExternalSemaphoresSignalNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hNode, p__params_out, p__params_out__val__extSemArray, p__params_out__val__paramsArray, p__params_out__val__numExtSems);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExternalSemaphoresSignalNodeGetParamsEnd`) */
void barectf_hip_api_trace_hipGraphExternalSemaphoresSignalNodeGetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExternalSemaphoresSignalNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExternalSemaphoresSignalNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExternalSemaphoresSignalNodeSetParamsBegin`) */
void barectf_hip_api_trace_hipGraphExternalSemaphoresSignalNodeSetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hNode,
	const uint64_t p__nodeParams,
	const uint64_t p__nodeParams__val__extSemArray,
	const uint64_t p__nodeParams__val__paramsArray,
	const uint32_t p__nodeParams__val__numExtSems)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExternalSemaphoresSignalNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExternalSemaphoresSignalNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hNode, p__nodeParams, p__nodeParams__val__extSemArray, p__nodeParams__val__paramsArray, p__nodeParams__val__numExtSems);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExternalSemaphoresSignalNodeSetParamsEnd`) */
void barectf_hip_api_trace_hipGraphExternalSemaphoresSignalNodeSetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExternalSemaphoresSignalNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExternalSemaphoresSignalNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExternalSemaphoresWaitNodeGetParamsBegin`) */
void barectf_hip_api_trace_hipGraphExternalSemaphoresWaitNodeGetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hNode,
	const uint64_t p__params_out,
	const uint64_t p__params_out__val__extSemArray,
	const uint64_t p__params_out__val__paramsArray,
	const uint32_t p__params_out__val__numExtSems)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExternalSemaphoresWaitNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExternalSemaphoresWaitNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hNode, p__params_out, p__params_out__val__extSemArray, p__params_out__val__paramsArray, p__params_out__val__numExtSems);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExternalSemaphoresWaitNodeGetParamsEnd`) */
void barectf_hip_api_trace_hipGraphExternalSemaphoresWaitNodeGetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExternalSemaphoresWaitNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExternalSemaphoresWaitNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExternalSemaphoresWaitNodeSetParamsBegin`) */
void barectf_hip_api_trace_hipGraphExternalSemaphoresWaitNodeSetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hNode,
	const uint64_t p__nodeParams,
	const uint64_t p__nodeParams__val__extSemArray,
	const uint64_t p__nodeParams__val__paramsArray,
	const uint32_t p__nodeParams__val__numExtSems)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExternalSemaphoresWaitNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExternalSemaphoresWaitNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hNode, p__nodeParams, p__nodeParams__val__extSemArray, p__nodeParams__val__paramsArray, p__nodeParams__val__numExtSems);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphExternalSemaphoresWaitNodeSetParamsEnd`) */
void barectf_hip_api_trace_hipGraphExternalSemaphoresWaitNodeSetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphExternalSemaphoresWaitNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphExternalSemaphoresWaitNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphGetEdgesBegin`) */
void barectf_hip_api_trace_hipGraphGetEdgesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__from,
	const uint64_t p__from__val,
	const uint64_t p__to,
	const uint64_t p__to__val,
	const uint64_t p__numEdges,
	const uint64_t p__numEdges__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphGetEdgesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphGetEdgesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__graph, p__from, p__from__val, p__to, p__to__val, p__numEdges, p__numEdges__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphGetEdgesEnd`) */
void barectf_hip_api_trace_hipGraphGetEdgesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphGetEdgesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphGetEdgesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphGetNodesBegin`) */
void barectf_hip_api_trace_hipGraphGetNodesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__nodes,
	const uint64_t p__nodes__val,
	const uint64_t p__numNodes,
	const uint64_t p__numNodes__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphGetNodesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphGetNodesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__graph, p__nodes, p__nodes__val, p__numNodes, p__numNodes__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphGetNodesEnd`) */
void barectf_hip_api_trace_hipGraphGetNodesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphGetNodesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphGetNodesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphGetRootNodesBegin`) */
void barectf_hip_api_trace_hipGraphGetRootNodesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__pRootNodes,
	const uint64_t p__pRootNodes__val,
	const uint64_t p__pNumRootNodes,
	const uint64_t p__pNumRootNodes__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphGetRootNodesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphGetRootNodesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__graph, p__pRootNodes, p__pRootNodes__val, p__pNumRootNodes, p__pNumRootNodes__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphGetRootNodesEnd`) */
void barectf_hip_api_trace_hipGraphGetRootNodesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphGetRootNodesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphGetRootNodesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphHostNodeGetParamsBegin`) */
void barectf_hip_api_trace_hipGraphHostNodeGetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__fn,
	const uint64_t p__pNodeParams__val__userData)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphHostNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphHostNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__pNodeParams, p__pNodeParams__val__fn, p__pNodeParams__val__userData);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphHostNodeGetParamsEnd`) */
void barectf_hip_api_trace_hipGraphHostNodeGetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphHostNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphHostNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphHostNodeSetParamsBegin`) */
void barectf_hip_api_trace_hipGraphHostNodeSetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__fn,
	const uint64_t p__pNodeParams__val__userData)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphHostNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphHostNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__pNodeParams, p__pNodeParams__val__fn, p__pNodeParams__val__userData);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphHostNodeSetParamsEnd`) */
void barectf_hip_api_trace_hipGraphHostNodeSetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphHostNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphHostNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphInstantiateBegin`) */
void barectf_hip_api_trace_hipGraphInstantiateBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphExec,
	const uint64_t p__pGraphExec__val,
	const uint64_t p__graph,
	const uint64_t p__pErrorNode,
	const uint64_t p__pErrorNode__val,
	const char * const p__pLogBuffer,
	const int8_t p__pLogBuffer__val,
	const uint64_t p__bufferSize)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphInstantiateBegin(_TO_VOID_PTR(ctx), cc__kernel_name, p__pLogBuffer);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphInstantiateBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphExec, p__pGraphExec__val, p__graph, p__pErrorNode, p__pErrorNode__val, p__pLogBuffer, p__pLogBuffer__val, p__bufferSize);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphInstantiateEnd`) */
void barectf_hip_api_trace_hipGraphInstantiateEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphInstantiateEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphInstantiateEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphInstantiateWithFlagsBegin`) */
void barectf_hip_api_trace_hipGraphInstantiateWithFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphExec,
	const uint64_t p__pGraphExec__val,
	const uint64_t p__graph,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphInstantiateWithFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphInstantiateWithFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphExec, p__pGraphExec__val, p__graph, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphInstantiateWithFlagsEnd`) */
void barectf_hip_api_trace_hipGraphInstantiateWithFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphInstantiateWithFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphInstantiateWithFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphInstantiateWithParamsBegin`) */
void barectf_hip_api_trace_hipGraphInstantiateWithParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pGraphExec,
	const uint64_t p__pGraphExec__val,
	const uint64_t p__graph,
	const uint64_t p__instantiateParams,
	const uint64_t p__instantiateParams__val__errNode_out,
	const uint64_t p__instantiateParams__val__flags,
	const int32_t p__instantiateParams__val__result_out,
	const uint64_t p__instantiateParams__val__uploadStream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphInstantiateWithParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphInstantiateWithParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pGraphExec, p__pGraphExec__val, p__graph, p__instantiateParams, p__instantiateParams__val__errNode_out, p__instantiateParams__val__flags, p__instantiateParams__val__result_out, p__instantiateParams__val__uploadStream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphInstantiateWithParamsEnd`) */
void barectf_hip_api_trace_hipGraphInstantiateWithParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphInstantiateWithParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphInstantiateWithParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphKernelNodeCopyAttributesBegin`) */
void barectf_hip_api_trace_hipGraphKernelNodeCopyAttributesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hSrc,
	const uint64_t p__hDst)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphKernelNodeCopyAttributesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphKernelNodeCopyAttributesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hSrc, p__hDst);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphKernelNodeCopyAttributesEnd`) */
void barectf_hip_api_trace_hipGraphKernelNodeCopyAttributesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphKernelNodeCopyAttributesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphKernelNodeCopyAttributesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphKernelNodeGetAttributeBegin`) */
void barectf_hip_api_trace_hipGraphKernelNodeGetAttributeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hNode,
	const int32_t p__attr,
	const uint64_t p__value,
	const uint64_t p__value__val__accessPolicyWindow__base_ptr,
	const int32_t p__value__val__accessPolicyWindow__hitProp,
	const uint64_t p__value__val__accessPolicyWindow__hitRatio,
	const int32_t p__value__val__accessPolicyWindow__missProp,
	const uint64_t p__value__val__accessPolicyWindow__num_bytes,
	const int32_t p__value__val__cooperative,
	const int32_t p__value__val__priority)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphKernelNodeGetAttributeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphKernelNodeGetAttributeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hNode, p__attr, p__value, p__value__val__accessPolicyWindow__base_ptr, p__value__val__accessPolicyWindow__hitProp, p__value__val__accessPolicyWindow__hitRatio, p__value__val__accessPolicyWindow__missProp, p__value__val__accessPolicyWindow__num_bytes, p__value__val__cooperative, p__value__val__priority);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphKernelNodeGetAttributeEnd`) */
void barectf_hip_api_trace_hipGraphKernelNodeGetAttributeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphKernelNodeGetAttributeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphKernelNodeGetAttributeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphKernelNodeGetParamsBegin`) */
void barectf_hip_api_trace_hipGraphKernelNodeGetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint32_t p__pNodeParams__val__blockDim__x,
	const uint32_t p__pNodeParams__val__blockDim__y,
	const uint32_t p__pNodeParams__val__blockDim__z,
	const uint64_t p__pNodeParams__val__extra,
	const uint64_t p__pNodeParams__val__func,
	const uint32_t p__pNodeParams__val__gridDim__x,
	const uint32_t p__pNodeParams__val__gridDim__y,
	const uint32_t p__pNodeParams__val__gridDim__z,
	const uint64_t p__pNodeParams__val__kernelParams,
	const uint32_t p__pNodeParams__val__sharedMemBytes)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphKernelNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphKernelNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__pNodeParams, p__pNodeParams__val__blockDim__x, p__pNodeParams__val__blockDim__y, p__pNodeParams__val__blockDim__z, p__pNodeParams__val__extra, p__pNodeParams__val__func, p__pNodeParams__val__gridDim__x, p__pNodeParams__val__gridDim__y, p__pNodeParams__val__gridDim__z, p__pNodeParams__val__kernelParams, p__pNodeParams__val__sharedMemBytes);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphKernelNodeGetParamsEnd`) */
void barectf_hip_api_trace_hipGraphKernelNodeGetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphKernelNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphKernelNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphKernelNodeSetAttributeBegin`) */
void barectf_hip_api_trace_hipGraphKernelNodeSetAttributeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hNode,
	const int32_t p__attr,
	const uint64_t p__value,
	const uint64_t p__value__val__accessPolicyWindow__base_ptr,
	const int32_t p__value__val__accessPolicyWindow__hitProp,
	const uint64_t p__value__val__accessPolicyWindow__hitRatio,
	const int32_t p__value__val__accessPolicyWindow__missProp,
	const uint64_t p__value__val__accessPolicyWindow__num_bytes,
	const int32_t p__value__val__cooperative,
	const int32_t p__value__val__priority)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphKernelNodeSetAttributeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphKernelNodeSetAttributeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hNode, p__attr, p__value, p__value__val__accessPolicyWindow__base_ptr, p__value__val__accessPolicyWindow__hitProp, p__value__val__accessPolicyWindow__hitRatio, p__value__val__accessPolicyWindow__missProp, p__value__val__accessPolicyWindow__num_bytes, p__value__val__cooperative, p__value__val__priority);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphKernelNodeSetAttributeEnd`) */
void barectf_hip_api_trace_hipGraphKernelNodeSetAttributeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphKernelNodeSetAttributeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphKernelNodeSetAttributeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphKernelNodeSetParamsBegin`) */
void barectf_hip_api_trace_hipGraphKernelNodeSetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint32_t p__pNodeParams__val__blockDim__x,
	const uint32_t p__pNodeParams__val__blockDim__y,
	const uint32_t p__pNodeParams__val__blockDim__z,
	const uint64_t p__pNodeParams__val__extra,
	const uint64_t p__pNodeParams__val__func,
	const uint32_t p__pNodeParams__val__gridDim__x,
	const uint32_t p__pNodeParams__val__gridDim__y,
	const uint32_t p__pNodeParams__val__gridDim__z,
	const uint64_t p__pNodeParams__val__kernelParams,
	const uint32_t p__pNodeParams__val__sharedMemBytes)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphKernelNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphKernelNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__pNodeParams, p__pNodeParams__val__blockDim__x, p__pNodeParams__val__blockDim__y, p__pNodeParams__val__blockDim__z, p__pNodeParams__val__extra, p__pNodeParams__val__func, p__pNodeParams__val__gridDim__x, p__pNodeParams__val__gridDim__y, p__pNodeParams__val__gridDim__z, p__pNodeParams__val__kernelParams, p__pNodeParams__val__sharedMemBytes);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphKernelNodeSetParamsEnd`) */
void barectf_hip_api_trace_hipGraphKernelNodeSetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphKernelNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphKernelNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphLaunchBegin`) */
void barectf_hip_api_trace_hipGraphLaunchBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graphExec,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphLaunchBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphLaunchBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__graphExec, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphLaunchEnd`) */
void barectf_hip_api_trace_hipGraphLaunchEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphLaunchEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphLaunchEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemAllocNodeGetParamsBegin`) */
void barectf_hip_api_trace_hipGraphMemAllocNodeGetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const int32_t p__pNodeParams__val__poolProps__allocType,
	const int32_t p__pNodeParams__val__poolProps__handleTypes,
	const int32_t p__pNodeParams__val__poolProps__location__type,
	const int32_t p__pNodeParams__val__poolProps__location__id,
	const uint64_t p__pNodeParams__val__poolProps__win32SecurityAttributes,
	const uint64_t p__pNodeParams__val__poolProps__maxSize,
	const uint64_t p__pNodeParams__val__poolProps__reserved,
	const uint64_t p__pNodeParams__val__accessDescs,
	const uint64_t p__pNodeParams__val__accessDescCount,
	const uint64_t p__pNodeParams__val__bytesize,
	const uint64_t p__pNodeParams__val__dptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemAllocNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemAllocNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__pNodeParams, p__pNodeParams__val__poolProps__allocType, p__pNodeParams__val__poolProps__handleTypes, p__pNodeParams__val__poolProps__location__type, p__pNodeParams__val__poolProps__location__id, p__pNodeParams__val__poolProps__win32SecurityAttributes, p__pNodeParams__val__poolProps__maxSize, p__pNodeParams__val__poolProps__reserved, p__pNodeParams__val__accessDescs, p__pNodeParams__val__accessDescCount, p__pNodeParams__val__bytesize, p__pNodeParams__val__dptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemAllocNodeGetParamsEnd`) */
void barectf_hip_api_trace_hipGraphMemAllocNodeGetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemAllocNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemAllocNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemFreeNodeGetParamsBegin`) */
void barectf_hip_api_trace_hipGraphMemFreeNodeGetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__dev_ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemFreeNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemFreeNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__dev_ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemFreeNodeGetParamsEnd`) */
void barectf_hip_api_trace_hipGraphMemFreeNodeGetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemFreeNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemFreeNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemcpyNodeGetParamsBegin`) */
void barectf_hip_api_trace_hipGraphMemcpyNodeGetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__srcArray,
	const uint64_t p__pNodeParams__val__srcPos__x,
	const uint64_t p__pNodeParams__val__srcPos__y,
	const uint64_t p__pNodeParams__val__srcPos__z,
	const uint64_t p__pNodeParams__val__srcPtr__ptr,
	const uint64_t p__pNodeParams__val__srcPtr__pitch,
	const uint64_t p__pNodeParams__val__srcPtr__xsize,
	const uint64_t p__pNodeParams__val__srcPtr__ysize,
	const uint64_t p__pNodeParams__val__dstArray,
	const uint64_t p__pNodeParams__val__dstPos__x,
	const uint64_t p__pNodeParams__val__dstPos__y,
	const uint64_t p__pNodeParams__val__dstPos__z,
	const uint64_t p__pNodeParams__val__dstPtr__ptr,
	const uint64_t p__pNodeParams__val__dstPtr__pitch,
	const uint64_t p__pNodeParams__val__dstPtr__xsize,
	const uint64_t p__pNodeParams__val__dstPtr__ysize,
	const uint64_t p__pNodeParams__val__extent__width,
	const uint64_t p__pNodeParams__val__extent__height,
	const uint64_t p__pNodeParams__val__extent__depth,
	const int32_t p__pNodeParams__val__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemcpyNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemcpyNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__pNodeParams, p__pNodeParams__val__srcArray, p__pNodeParams__val__srcPos__x, p__pNodeParams__val__srcPos__y, p__pNodeParams__val__srcPos__z, p__pNodeParams__val__srcPtr__ptr, p__pNodeParams__val__srcPtr__pitch, p__pNodeParams__val__srcPtr__xsize, p__pNodeParams__val__srcPtr__ysize, p__pNodeParams__val__dstArray, p__pNodeParams__val__dstPos__x, p__pNodeParams__val__dstPos__y, p__pNodeParams__val__dstPos__z, p__pNodeParams__val__dstPtr__ptr, p__pNodeParams__val__dstPtr__pitch, p__pNodeParams__val__dstPtr__xsize, p__pNodeParams__val__dstPtr__ysize, p__pNodeParams__val__extent__width, p__pNodeParams__val__extent__height, p__pNodeParams__val__extent__depth, p__pNodeParams__val__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemcpyNodeGetParamsEnd`) */
void barectf_hip_api_trace_hipGraphMemcpyNodeGetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemcpyNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemcpyNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemcpyNodeSetParams1DBegin`) */
void barectf_hip_api_trace_hipGraphMemcpyNodeSetParams1DBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__count,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemcpyNodeSetParams1DBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemcpyNodeSetParams1DBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__dst, p__src, p__count, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemcpyNodeSetParams1DEnd`) */
void barectf_hip_api_trace_hipGraphMemcpyNodeSetParams1DEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemcpyNodeSetParams1DEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemcpyNodeSetParams1DEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemcpyNodeSetParamsBegin`) */
void barectf_hip_api_trace_hipGraphMemcpyNodeSetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__srcArray,
	const uint64_t p__pNodeParams__val__srcPos__x,
	const uint64_t p__pNodeParams__val__srcPos__y,
	const uint64_t p__pNodeParams__val__srcPos__z,
	const uint64_t p__pNodeParams__val__srcPtr__ptr,
	const uint64_t p__pNodeParams__val__srcPtr__pitch,
	const uint64_t p__pNodeParams__val__srcPtr__xsize,
	const uint64_t p__pNodeParams__val__srcPtr__ysize,
	const uint64_t p__pNodeParams__val__dstArray,
	const uint64_t p__pNodeParams__val__dstPos__x,
	const uint64_t p__pNodeParams__val__dstPos__y,
	const uint64_t p__pNodeParams__val__dstPos__z,
	const uint64_t p__pNodeParams__val__dstPtr__ptr,
	const uint64_t p__pNodeParams__val__dstPtr__pitch,
	const uint64_t p__pNodeParams__val__dstPtr__xsize,
	const uint64_t p__pNodeParams__val__dstPtr__ysize,
	const uint64_t p__pNodeParams__val__extent__width,
	const uint64_t p__pNodeParams__val__extent__height,
	const uint64_t p__pNodeParams__val__extent__depth,
	const int32_t p__pNodeParams__val__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemcpyNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemcpyNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__pNodeParams, p__pNodeParams__val__srcArray, p__pNodeParams__val__srcPos__x, p__pNodeParams__val__srcPos__y, p__pNodeParams__val__srcPos__z, p__pNodeParams__val__srcPtr__ptr, p__pNodeParams__val__srcPtr__pitch, p__pNodeParams__val__srcPtr__xsize, p__pNodeParams__val__srcPtr__ysize, p__pNodeParams__val__dstArray, p__pNodeParams__val__dstPos__x, p__pNodeParams__val__dstPos__y, p__pNodeParams__val__dstPos__z, p__pNodeParams__val__dstPtr__ptr, p__pNodeParams__val__dstPtr__pitch, p__pNodeParams__val__dstPtr__xsize, p__pNodeParams__val__dstPtr__ysize, p__pNodeParams__val__extent__width, p__pNodeParams__val__extent__height, p__pNodeParams__val__extent__depth, p__pNodeParams__val__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemcpyNodeSetParamsEnd`) */
void barectf_hip_api_trace_hipGraphMemcpyNodeSetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemcpyNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemcpyNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemcpyNodeSetParamsFromSymbolBegin`) */
void barectf_hip_api_trace_hipGraphMemcpyNodeSetParamsFromSymbolBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__dst,
	const uint64_t p__symbol,
	const uint64_t p__count,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemcpyNodeSetParamsFromSymbolBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemcpyNodeSetParamsFromSymbolBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__dst, p__symbol, p__count, p__offset, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemcpyNodeSetParamsFromSymbolEnd`) */
void barectf_hip_api_trace_hipGraphMemcpyNodeSetParamsFromSymbolEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemcpyNodeSetParamsFromSymbolEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemcpyNodeSetParamsFromSymbolEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemcpyNodeSetParamsToSymbolBegin`) */
void barectf_hip_api_trace_hipGraphMemcpyNodeSetParamsToSymbolBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__symbol,
	const uint64_t p__src,
	const uint64_t p__count,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemcpyNodeSetParamsToSymbolBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemcpyNodeSetParamsToSymbolBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__symbol, p__src, p__count, p__offset, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemcpyNodeSetParamsToSymbolEnd`) */
void barectf_hip_api_trace_hipGraphMemcpyNodeSetParamsToSymbolEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemcpyNodeSetParamsToSymbolEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemcpyNodeSetParamsToSymbolEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemsetNodeGetParamsBegin`) */
void barectf_hip_api_trace_hipGraphMemsetNodeGetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__dst,
	const uint32_t p__pNodeParams__val__elementSize,
	const uint64_t p__pNodeParams__val__height,
	const uint64_t p__pNodeParams__val__pitch,
	const uint32_t p__pNodeParams__val__value,
	const uint64_t p__pNodeParams__val__width)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemsetNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemsetNodeGetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__pNodeParams, p__pNodeParams__val__dst, p__pNodeParams__val__elementSize, p__pNodeParams__val__height, p__pNodeParams__val__pitch, p__pNodeParams__val__value, p__pNodeParams__val__width);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemsetNodeGetParamsEnd`) */
void barectf_hip_api_trace_hipGraphMemsetNodeGetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemsetNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemsetNodeGetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemsetNodeSetParamsBegin`) */
void barectf_hip_api_trace_hipGraphMemsetNodeSetParamsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pNodeParams,
	const uint64_t p__pNodeParams__val__dst,
	const uint32_t p__pNodeParams__val__elementSize,
	const uint64_t p__pNodeParams__val__height,
	const uint64_t p__pNodeParams__val__pitch,
	const uint32_t p__pNodeParams__val__value,
	const uint64_t p__pNodeParams__val__width)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemsetNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemsetNodeSetParamsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__pNodeParams, p__pNodeParams__val__dst, p__pNodeParams__val__elementSize, p__pNodeParams__val__height, p__pNodeParams__val__pitch, p__pNodeParams__val__value, p__pNodeParams__val__width);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphMemsetNodeSetParamsEnd`) */
void barectf_hip_api_trace_hipGraphMemsetNodeSetParamsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphMemsetNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphMemsetNodeSetParamsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphNodeFindInCloneBegin`) */
void barectf_hip_api_trace_hipGraphNodeFindInCloneBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pNode,
	const uint64_t p__pNode__val,
	const uint64_t p__originalNode,
	const uint64_t p__clonedGraph)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphNodeFindInCloneBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphNodeFindInCloneBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pNode, p__pNode__val, p__originalNode, p__clonedGraph);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphNodeFindInCloneEnd`) */
void barectf_hip_api_trace_hipGraphNodeFindInCloneEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphNodeFindInCloneEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphNodeFindInCloneEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphNodeGetDependenciesBegin`) */
void barectf_hip_api_trace_hipGraphNodeGetDependenciesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pDependencies,
	const uint64_t p__pDependencies__val,
	const uint64_t p__pNumDependencies,
	const uint64_t p__pNumDependencies__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphNodeGetDependenciesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphNodeGetDependenciesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__pDependencies, p__pDependencies__val, p__pNumDependencies, p__pNumDependencies__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphNodeGetDependenciesEnd`) */
void barectf_hip_api_trace_hipGraphNodeGetDependenciesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphNodeGetDependenciesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphNodeGetDependenciesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphNodeGetDependentNodesBegin`) */
void barectf_hip_api_trace_hipGraphNodeGetDependentNodesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pDependentNodes,
	const uint64_t p__pDependentNodes__val,
	const uint64_t p__pNumDependentNodes,
	const uint64_t p__pNumDependentNodes__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphNodeGetDependentNodesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphNodeGetDependentNodesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__pDependentNodes, p__pDependentNodes__val, p__pNumDependentNodes, p__pNumDependentNodes__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphNodeGetDependentNodesEnd`) */
void barectf_hip_api_trace_hipGraphNodeGetDependentNodesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphNodeGetDependentNodesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphNodeGetDependentNodesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphNodeGetEnabledBegin`) */
void barectf_hip_api_trace_hipGraphNodeGetEnabledBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hNode,
	const uint64_t p__isEnabled,
	const uint32_t p__isEnabled__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphNodeGetEnabledBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphNodeGetEnabledBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__hNode, p__isEnabled, p__isEnabled__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphNodeGetEnabledEnd`) */
void barectf_hip_api_trace_hipGraphNodeGetEnabledEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphNodeGetEnabledEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphNodeGetEnabledEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphNodeGetTypeBegin`) */
void barectf_hip_api_trace_hipGraphNodeGetTypeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__node,
	const uint64_t p__pType,
	const int32_t p__pType__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphNodeGetTypeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphNodeGetTypeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__node, p__pType, p__pType__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphNodeGetTypeEnd`) */
void barectf_hip_api_trace_hipGraphNodeGetTypeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphNodeGetTypeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphNodeGetTypeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphNodeSetEnabledBegin`) */
void barectf_hip_api_trace_hipGraphNodeSetEnabledBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hGraphExec,
	const uint64_t p__hNode,
	const uint32_t p__isEnabled)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphNodeSetEnabledBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphNodeSetEnabledBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hGraphExec, p__hNode, p__isEnabled);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphNodeSetEnabledEnd`) */
void barectf_hip_api_trace_hipGraphNodeSetEnabledEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphNodeSetEnabledEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphNodeSetEnabledEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphReleaseUserObjectBegin`) */
void barectf_hip_api_trace_hipGraphReleaseUserObjectBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__object,
	const uint32_t p__count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphReleaseUserObjectBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphReleaseUserObjectBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__graph, p__object, p__count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphReleaseUserObjectEnd`) */
void barectf_hip_api_trace_hipGraphReleaseUserObjectEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphReleaseUserObjectEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphReleaseUserObjectEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphRemoveDependenciesBegin`) */
void barectf_hip_api_trace_hipGraphRemoveDependenciesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__from,
	const uint64_t p__from__val,
	const uint64_t p__to,
	const uint64_t p__to__val,
	const uint64_t p__numDependencies)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphRemoveDependenciesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphRemoveDependenciesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__graph, p__from, p__from__val, p__to, p__to__val, p__numDependencies);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphRemoveDependenciesEnd`) */
void barectf_hip_api_trace_hipGraphRemoveDependenciesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphRemoveDependenciesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphRemoveDependenciesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphRetainUserObjectBegin`) */
void barectf_hip_api_trace_hipGraphRetainUserObjectBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graph,
	const uint64_t p__object,
	const uint32_t p__count,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphRetainUserObjectBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphRetainUserObjectBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__graph, p__object, p__count, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphRetainUserObjectEnd`) */
void barectf_hip_api_trace_hipGraphRetainUserObjectEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphRetainUserObjectEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphRetainUserObjectEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphUploadBegin`) */
void barectf_hip_api_trace_hipGraphUploadBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__graphExec,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphUploadBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphUploadBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__graphExec, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphUploadEnd`) */
void barectf_hip_api_trace_hipGraphUploadEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphUploadEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphUploadEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsGLRegisterBufferBegin`) */
void barectf_hip_api_trace_hipGraphicsGLRegisterBufferBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__resource,
	const uint64_t p__resource__val,
	const int32_t p__buffer,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsGLRegisterBufferBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsGLRegisterBufferBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__resource, p__resource__val, p__buffer, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsGLRegisterBufferEnd`) */
void barectf_hip_api_trace_hipGraphicsGLRegisterBufferEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsGLRegisterBufferEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsGLRegisterBufferEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsGLRegisterImageBegin`) */
void barectf_hip_api_trace_hipGraphicsGLRegisterImageBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__resource,
	const uint64_t p__resource__val,
	const int32_t p__image,
	const int32_t p__target,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsGLRegisterImageBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsGLRegisterImageBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__resource, p__resource__val, p__image, p__target, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsGLRegisterImageEnd`) */
void barectf_hip_api_trace_hipGraphicsGLRegisterImageEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsGLRegisterImageEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsGLRegisterImageEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsMapResourcesBegin`) */
void barectf_hip_api_trace_hipGraphicsMapResourcesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__count,
	const uint64_t p__resources,
	const uint64_t p__resources__val,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsMapResourcesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsMapResourcesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__count, p__resources, p__resources__val, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsMapResourcesEnd`) */
void barectf_hip_api_trace_hipGraphicsMapResourcesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsMapResourcesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsMapResourcesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsResourceGetMappedPointerBegin`) */
void barectf_hip_api_trace_hipGraphicsResourceGetMappedPointerBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr,
	const uint64_t p__devPtr__val,
	const uint64_t p__size,
	const uint64_t p__size__val,
	const uint64_t p__resource)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsResourceGetMappedPointerBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsResourceGetMappedPointerBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__devPtr, p__devPtr__val, p__size, p__size__val, p__resource);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsResourceGetMappedPointerEnd`) */
void barectf_hip_api_trace_hipGraphicsResourceGetMappedPointerEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsResourceGetMappedPointerEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsResourceGetMappedPointerEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsSubResourceGetMappedArrayBegin`) */
void barectf_hip_api_trace_hipGraphicsSubResourceGetMappedArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__array,
	const uint64_t p__array__val,
	const uint64_t p__resource,
	const uint32_t p__arrayIndex,
	const uint32_t p__mipLevel)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsSubResourceGetMappedArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsSubResourceGetMappedArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__array, p__array__val, p__resource, p__arrayIndex, p__mipLevel);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsSubResourceGetMappedArrayEnd`) */
void barectf_hip_api_trace_hipGraphicsSubResourceGetMappedArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsSubResourceGetMappedArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsSubResourceGetMappedArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsUnmapResourcesBegin`) */
void barectf_hip_api_trace_hipGraphicsUnmapResourcesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__count,
	const uint64_t p__resources,
	const uint64_t p__resources__val,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsUnmapResourcesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsUnmapResourcesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__count, p__resources, p__resources__val, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsUnmapResourcesEnd`) */
void barectf_hip_api_trace_hipGraphicsUnmapResourcesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsUnmapResourcesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsUnmapResourcesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsUnregisterResourceBegin`) */
void barectf_hip_api_trace_hipGraphicsUnregisterResourceBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__resource)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsUnregisterResourceBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsUnregisterResourceBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__resource);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipGraphicsUnregisterResourceEnd`) */
void barectf_hip_api_trace_hipGraphicsUnregisterResourceEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipGraphicsUnregisterResourceEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipGraphicsUnregisterResourceEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHccModuleLaunchKernelBegin`) */
void barectf_hip_api_trace_hipHccModuleLaunchKernelBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__f,
	const uint32_t p__globalWorkSizeX,
	const uint32_t p__globalWorkSizeY,
	const uint32_t p__globalWorkSizeZ,
	const uint32_t p__blockDimX,
	const uint32_t p__blockDimY,
	const uint32_t p__blockDimZ,
	const uint64_t p__sharedMemBytes,
	const uint64_t p__hStream,
	const uint64_t p__kernelParams,
	const uint64_t p__kernelParams__val,
	const uint64_t p__extra,
	const uint64_t p__extra__val,
	const uint64_t p__startEvent,
	const uint64_t p__stopEvent)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHccModuleLaunchKernelBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHccModuleLaunchKernelBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__f, p__globalWorkSizeX, p__globalWorkSizeY, p__globalWorkSizeZ, p__blockDimX, p__blockDimY, p__blockDimZ, p__sharedMemBytes, p__hStream, p__kernelParams, p__kernelParams__val, p__extra, p__extra__val, p__startEvent, p__stopEvent);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHccModuleLaunchKernelEnd`) */
void barectf_hip_api_trace_hipHccModuleLaunchKernelEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHccModuleLaunchKernelEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHccModuleLaunchKernelEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostAllocBegin`) */
void barectf_hip_api_trace_hipHostAllocBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__size,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostAllocBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostAllocBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr, p__ptr__val, p__size, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostAllocEnd`) */
void barectf_hip_api_trace_hipHostAllocEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostAllocEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostAllocEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostFreeBegin`) */
void barectf_hip_api_trace_hipHostFreeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostFreeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostFreeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostFreeEnd`) */
void barectf_hip_api_trace_hipHostFreeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostFreeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostFreeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostGetDevicePointerBegin`) */
void barectf_hip_api_trace_hipHostGetDevicePointerBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr,
	const uint64_t p__devPtr__val,
	const uint64_t p__hstPtr,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostGetDevicePointerBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostGetDevicePointerBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__devPtr, p__devPtr__val, p__hstPtr, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostGetDevicePointerEnd`) */
void barectf_hip_api_trace_hipHostGetDevicePointerEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostGetDevicePointerEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostGetDevicePointerEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostGetFlagsBegin`) */
void barectf_hip_api_trace_hipHostGetFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__flagsPtr,
	const uint32_t p__flagsPtr__val,
	const uint64_t p__hostPtr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostGetFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostGetFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__flagsPtr, p__flagsPtr__val, p__hostPtr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostGetFlagsEnd`) */
void barectf_hip_api_trace_hipHostGetFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostGetFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostGetFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostMallocBegin`) */
void barectf_hip_api_trace_hipHostMallocBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__size,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostMallocBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostMallocBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr, p__ptr__val, p__size, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostMallocEnd`) */
void barectf_hip_api_trace_hipHostMallocEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostMallocEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostMallocEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostRegisterBegin`) */
void barectf_hip_api_trace_hipHostRegisterBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hostPtr,
	const uint64_t p__sizeBytes,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostRegisterBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostRegisterBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hostPtr, p__sizeBytes, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostRegisterEnd`) */
void barectf_hip_api_trace_hipHostRegisterEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostRegisterEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostRegisterEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostUnregisterBegin`) */
void barectf_hip_api_trace_hipHostUnregisterBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hostPtr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostUnregisterBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostUnregisterBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hostPtr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipHostUnregisterEnd`) */
void barectf_hip_api_trace_hipHostUnregisterEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipHostUnregisterEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipHostUnregisterEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipImportExternalMemoryBegin`) */
void barectf_hip_api_trace_hipImportExternalMemoryBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__extMem_out,
	const uint64_t p__extMem_out__val,
	const uint64_t p__memHandleDesc,
	const int32_t p__memHandleDesc__val__type,
	const uint64_t p__memHandleDesc__val__size,
	const uint32_t p__memHandleDesc__val__flags,
	const uint64_t p__memHandleDesc__val__reserved)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipImportExternalMemoryBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipImportExternalMemoryBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__extMem_out, p__extMem_out__val, p__memHandleDesc, p__memHandleDesc__val__type, p__memHandleDesc__val__size, p__memHandleDesc__val__flags, p__memHandleDesc__val__reserved);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipImportExternalMemoryEnd`) */
void barectf_hip_api_trace_hipImportExternalMemoryEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipImportExternalMemoryEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipImportExternalMemoryEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipImportExternalSemaphoreBegin`) */
void barectf_hip_api_trace_hipImportExternalSemaphoreBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__extSem_out,
	const uint64_t p__extSem_out__val,
	const uint64_t p__semHandleDesc,
	const int32_t p__semHandleDesc__val__type,
	const uint32_t p__semHandleDesc__val__flags,
	const uint64_t p__semHandleDesc__val__reserved)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipImportExternalSemaphoreBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipImportExternalSemaphoreBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__extSem_out, p__extSem_out__val, p__semHandleDesc, p__semHandleDesc__val__type, p__semHandleDesc__val__flags, p__semHandleDesc__val__reserved);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipImportExternalSemaphoreEnd`) */
void barectf_hip_api_trace_hipImportExternalSemaphoreEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipImportExternalSemaphoreEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipImportExternalSemaphoreEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipInitBegin`) */
void barectf_hip_api_trace_hipInitBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipInitBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipInitBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipInitEnd`) */
void barectf_hip_api_trace_hipInitEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipInitEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipInitEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipIpcCloseMemHandleBegin`) */
void barectf_hip_api_trace_hipIpcCloseMemHandleBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipIpcCloseMemHandleBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipIpcCloseMemHandleBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__devPtr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipIpcCloseMemHandleEnd`) */
void barectf_hip_api_trace_hipIpcCloseMemHandleEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipIpcCloseMemHandleEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipIpcCloseMemHandleEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipIpcGetEventHandleBegin`) */
void barectf_hip_api_trace_hipIpcGetEventHandleBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__handle,
	const uint64_t p__handle__val__reserved,
	const uint64_t p__event)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipIpcGetEventHandleBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipIpcGetEventHandleBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__handle, p__handle__val__reserved, p__event);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipIpcGetEventHandleEnd`) */
void barectf_hip_api_trace_hipIpcGetEventHandleEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipIpcGetEventHandleEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipIpcGetEventHandleEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipIpcGetMemHandleBegin`) */
void barectf_hip_api_trace_hipIpcGetMemHandleBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__handle,
	const uint64_t p__handle__val__reserved,
	const uint64_t p__devPtr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipIpcGetMemHandleBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipIpcGetMemHandleBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__handle, p__handle__val__reserved, p__devPtr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipIpcGetMemHandleEnd`) */
void barectf_hip_api_trace_hipIpcGetMemHandleEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipIpcGetMemHandleEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipIpcGetMemHandleEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipIpcOpenEventHandleBegin`) */
void barectf_hip_api_trace_hipIpcOpenEventHandleBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__event,
	const uint64_t p__event__val,
	const uint64_t p__handle__reserved)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipIpcOpenEventHandleBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipIpcOpenEventHandleBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__event, p__event__val, p__handle__reserved);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipIpcOpenEventHandleEnd`) */
void barectf_hip_api_trace_hipIpcOpenEventHandleEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipIpcOpenEventHandleEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipIpcOpenEventHandleEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipIpcOpenMemHandleBegin`) */
void barectf_hip_api_trace_hipIpcOpenMemHandleBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr,
	const uint64_t p__devPtr__val,
	const uint64_t p__handle__reserved,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipIpcOpenMemHandleBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipIpcOpenMemHandleBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__devPtr, p__devPtr__val, p__handle__reserved, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipIpcOpenMemHandleEnd`) */
void barectf_hip_api_trace_hipIpcOpenMemHandleEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipIpcOpenMemHandleEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipIpcOpenMemHandleEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipLaunchByPtrBegin`) */
void barectf_hip_api_trace_hipLaunchByPtrBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hostFunction)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipLaunchByPtrBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipLaunchByPtrBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hostFunction);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipLaunchByPtrEnd`) */
void barectf_hip_api_trace_hipLaunchByPtrEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipLaunchByPtrEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipLaunchByPtrEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipLaunchCooperativeKernelBegin`) */
void barectf_hip_api_trace_hipLaunchCooperativeKernelBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__f,
	const uint32_t p__gridDim__x,
	const uint32_t p__gridDim__y,
	const uint32_t p__gridDim__z,
	const uint32_t p__blockDimX__x,
	const uint32_t p__blockDimX__y,
	const uint32_t p__blockDimX__z,
	const uint64_t p__kernelParams,
	const uint64_t p__kernelParams__val,
	const uint32_t p__sharedMemBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipLaunchCooperativeKernelBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipLaunchCooperativeKernelBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__f, p__gridDim__x, p__gridDim__y, p__gridDim__z, p__blockDimX__x, p__blockDimX__y, p__blockDimX__z, p__kernelParams, p__kernelParams__val, p__sharedMemBytes, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipLaunchCooperativeKernelEnd`) */
void barectf_hip_api_trace_hipLaunchCooperativeKernelEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipLaunchCooperativeKernelEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipLaunchCooperativeKernelEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipLaunchCooperativeKernelMultiDeviceBegin`) */
void barectf_hip_api_trace_hipLaunchCooperativeKernelMultiDeviceBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__launchParamsList,
	const uint64_t p__launchParamsList__val__func,
	const uint32_t p__launchParamsList__val__gridDim__x,
	const uint32_t p__launchParamsList__val__gridDim__y,
	const uint32_t p__launchParamsList__val__gridDim__z,
	const uint32_t p__launchParamsList__val__blockDim__x,
	const uint32_t p__launchParamsList__val__blockDim__y,
	const uint32_t p__launchParamsList__val__blockDim__z,
	const uint64_t p__launchParamsList__val__args,
	const uint64_t p__launchParamsList__val__sharedMem,
	const uint64_t p__launchParamsList__val__stream,
	const int32_t p__numDevices,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipLaunchCooperativeKernelMultiDeviceBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipLaunchCooperativeKernelMultiDeviceBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__launchParamsList, p__launchParamsList__val__func, p__launchParamsList__val__gridDim__x, p__launchParamsList__val__gridDim__y, p__launchParamsList__val__gridDim__z, p__launchParamsList__val__blockDim__x, p__launchParamsList__val__blockDim__y, p__launchParamsList__val__blockDim__z, p__launchParamsList__val__args, p__launchParamsList__val__sharedMem, p__launchParamsList__val__stream, p__numDevices, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipLaunchCooperativeKernelMultiDeviceEnd`) */
void barectf_hip_api_trace_hipLaunchCooperativeKernelMultiDeviceEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipLaunchCooperativeKernelMultiDeviceEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipLaunchCooperativeKernelMultiDeviceEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipLaunchHostFuncBegin`) */
void barectf_hip_api_trace_hipLaunchHostFuncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__fn,
	const uint64_t p__userData)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipLaunchHostFuncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipLaunchHostFuncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__fn, p__userData);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipLaunchHostFuncEnd`) */
void barectf_hip_api_trace_hipLaunchHostFuncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipLaunchHostFuncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipLaunchHostFuncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipLaunchKernelBegin`) */
void barectf_hip_api_trace_hipLaunchKernelBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__function_address,
	const uint32_t p__numBlocks__x,
	const uint32_t p__numBlocks__y,
	const uint32_t p__numBlocks__z,
	const uint32_t p__dimBlocks__x,
	const uint32_t p__dimBlocks__y,
	const uint32_t p__dimBlocks__z,
	const uint64_t p__args,
	const uint64_t p__args__val,
	const uint64_t p__sharedMemBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipLaunchKernelBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipLaunchKernelBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__function_address, p__numBlocks__x, p__numBlocks__y, p__numBlocks__z, p__dimBlocks__x, p__dimBlocks__y, p__dimBlocks__z, p__args, p__args__val, p__sharedMemBytes, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipLaunchKernelEnd`) */
void barectf_hip_api_trace_hipLaunchKernelEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipLaunchKernelEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipLaunchKernelEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMalloc3DArrayBegin`) */
void barectf_hip_api_trace_hipMalloc3DArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__array,
	const uint64_t p__array__val,
	const uint64_t p__desc,
	const int32_t p__desc__val__x,
	const int32_t p__desc__val__y,
	const int32_t p__desc__val__z,
	const int32_t p__desc__val__w,
	const int32_t p__desc__val__f,
	const uint64_t p__extent__width,
	const uint64_t p__extent__height,
	const uint64_t p__extent__depth,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMalloc3DArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMalloc3DArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__array, p__array__val, p__desc, p__desc__val__x, p__desc__val__y, p__desc__val__z, p__desc__val__w, p__desc__val__f, p__extent__width, p__extent__height, p__extent__depth, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMalloc3DArrayEnd`) */
void barectf_hip_api_trace_hipMalloc3DArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMalloc3DArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMalloc3DArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMalloc3DBegin`) */
void barectf_hip_api_trace_hipMalloc3DBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pitchedDevPtr,
	const uint64_t p__pitchedDevPtr__val__ptr,
	const uint64_t p__pitchedDevPtr__val__pitch,
	const uint64_t p__pitchedDevPtr__val__xsize,
	const uint64_t p__pitchedDevPtr__val__ysize,
	const uint64_t p__extent__width,
	const uint64_t p__extent__height,
	const uint64_t p__extent__depth)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMalloc3DBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMalloc3DBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pitchedDevPtr, p__pitchedDevPtr__val__ptr, p__pitchedDevPtr__val__pitch, p__pitchedDevPtr__val__xsize, p__pitchedDevPtr__val__ysize, p__extent__width, p__extent__height, p__extent__depth);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMalloc3DEnd`) */
void barectf_hip_api_trace_hipMalloc3DEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMalloc3DEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMalloc3DEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocArrayBegin`) */
void barectf_hip_api_trace_hipMallocArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__array,
	const uint64_t p__array__val,
	const uint64_t p__desc,
	const int32_t p__desc__val__x,
	const int32_t p__desc__val__y,
	const int32_t p__desc__val__z,
	const int32_t p__desc__val__w,
	const int32_t p__desc__val__f,
	const uint64_t p__width,
	const uint64_t p__height,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__array, p__array__val, p__desc, p__desc__val__x, p__desc__val__y, p__desc__val__z, p__desc__val__w, p__desc__val__f, p__width, p__height, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocArrayEnd`) */
void barectf_hip_api_trace_hipMallocArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocAsyncBegin`) */
void barectf_hip_api_trace_hipMallocAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__dev_ptr__val,
	const uint64_t p__size,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dev_ptr, p__dev_ptr__val, p__size, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocAsyncEnd`) */
void barectf_hip_api_trace_hipMallocAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocBegin`) */
void barectf_hip_api_trace_hipMallocBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr, p__ptr__val, p__size);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocEnd`) */
void barectf_hip_api_trace_hipMallocEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocFromPoolAsyncBegin`) */
void barectf_hip_api_trace_hipMallocFromPoolAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__dev_ptr__val,
	const uint64_t p__size,
	const uint64_t p__mem_pool,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocFromPoolAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocFromPoolAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dev_ptr, p__dev_ptr__val, p__size, p__mem_pool, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocFromPoolAsyncEnd`) */
void barectf_hip_api_trace_hipMallocFromPoolAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocFromPoolAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocFromPoolAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocHostBegin`) */
void barectf_hip_api_trace_hipMallocHostBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocHostBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocHostBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr, p__ptr__val, p__size);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocHostEnd`) */
void barectf_hip_api_trace_hipMallocHostEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocHostEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocHostEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocManagedBegin`) */
void barectf_hip_api_trace_hipMallocManagedBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__dev_ptr__val,
	const uint64_t p__size,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocManagedBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocManagedBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dev_ptr, p__dev_ptr__val, p__size, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocManagedEnd`) */
void barectf_hip_api_trace_hipMallocManagedEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocManagedEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocManagedEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocMipmappedArrayBegin`) */
void barectf_hip_api_trace_hipMallocMipmappedArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mipmappedArray,
	const uint64_t p__mipmappedArray__val,
	const uint64_t p__desc,
	const int32_t p__desc__val__x,
	const int32_t p__desc__val__y,
	const int32_t p__desc__val__z,
	const int32_t p__desc__val__w,
	const int32_t p__desc__val__f,
	const uint64_t p__extent__width,
	const uint64_t p__extent__height,
	const uint64_t p__extent__depth,
	const uint32_t p__numLevels,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocMipmappedArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocMipmappedArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mipmappedArray, p__mipmappedArray__val, p__desc, p__desc__val__x, p__desc__val__y, p__desc__val__z, p__desc__val__w, p__desc__val__f, p__extent__width, p__extent__height, p__extent__depth, p__numLevels, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocMipmappedArrayEnd`) */
void barectf_hip_api_trace_hipMallocMipmappedArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocMipmappedArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocMipmappedArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocPitchBegin`) */
void barectf_hip_api_trace_hipMallocPitchBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__pitch,
	const uint64_t p__pitch__val,
	const uint64_t p__width,
	const uint64_t p__height)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocPitchBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocPitchBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr, p__ptr__val, p__pitch, p__pitch__val, p__width, p__height);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMallocPitchEnd`) */
void barectf_hip_api_trace_hipMallocPitchEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMallocPitchEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMallocPitchEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemAddressFreeBegin`) */
void barectf_hip_api_trace_hipMemAddressFreeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__devPtr,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemAddressFreeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemAddressFreeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__devPtr, p__size);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemAddressFreeEnd`) */
void barectf_hip_api_trace_hipMemAddressFreeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemAddressFreeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemAddressFreeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemAddressReserveBegin`) */
void barectf_hip_api_trace_hipMemAddressReserveBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__size,
	const uint64_t p__alignment,
	const uint64_t p__addr,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemAddressReserveBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemAddressReserveBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr, p__ptr__val, p__size, p__alignment, p__addr, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemAddressReserveEnd`) */
void barectf_hip_api_trace_hipMemAddressReserveEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemAddressReserveEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemAddressReserveEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemAdviseBegin`) */
void barectf_hip_api_trace_hipMemAdviseBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__count,
	const int32_t p__advice,
	const int32_t p__device)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemAdviseBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemAdviseBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dev_ptr, p__count, p__advice, p__device);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemAdviseEnd`) */
void barectf_hip_api_trace_hipMemAdviseEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemAdviseEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemAdviseEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemAllocHostBegin`) */
void barectf_hip_api_trace_hipMemAllocHostBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__ptr__val,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemAllocHostBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemAllocHostBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr, p__ptr__val, p__size);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemAllocHostEnd`) */
void barectf_hip_api_trace_hipMemAllocHostEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemAllocHostEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemAllocHostEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemAllocPitchBegin`) */
void barectf_hip_api_trace_hipMemAllocPitchBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dptr,
	const uint64_t p__dptr__val,
	const uint64_t p__pitch,
	const uint64_t p__pitch__val,
	const uint64_t p__widthInBytes,
	const uint64_t p__height,
	const uint32_t p__elementSizeBytes)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemAllocPitchBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemAllocPitchBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dptr, p__dptr__val, p__pitch, p__pitch__val, p__widthInBytes, p__height, p__elementSizeBytes);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemAllocPitchEnd`) */
void barectf_hip_api_trace_hipMemAllocPitchEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemAllocPitchEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemAllocPitchEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemCreateBegin`) */
void barectf_hip_api_trace_hipMemCreateBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__handle,
	const uint64_t p__handle__val,
	const uint64_t p__size,
	const uint64_t p__prop,
	const int32_t p__prop__val__type,
	const int32_t p__prop__val__requestedHandleType,
	const int32_t p__prop__val__location__type,
	const int32_t p__prop__val__location__id,
	const uint64_t p__prop__val__win32HandleMetaData,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemCreateBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemCreateBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__handle, p__handle__val, p__size, p__prop, p__prop__val__type, p__prop__val__requestedHandleType, p__prop__val__location__type, p__prop__val__location__id, p__prop__val__win32HandleMetaData, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemCreateEnd`) */
void barectf_hip_api_trace_hipMemCreateEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemCreateEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemCreateEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemExportToShareableHandleBegin`) */
void barectf_hip_api_trace_hipMemExportToShareableHandleBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__shareableHandle,
	const uint64_t p__handle,
	const int32_t p__handleType,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemExportToShareableHandleBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemExportToShareableHandleBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__shareableHandle, p__handle, p__handleType, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemExportToShareableHandleEnd`) */
void barectf_hip_api_trace_hipMemExportToShareableHandleEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemExportToShareableHandleEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemExportToShareableHandleEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemGetAccessBegin`) */
void barectf_hip_api_trace_hipMemGetAccessBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__flags,
	const uint64_t p__flags__val,
	const uint64_t p__location,
	const int32_t p__location__val__type,
	const int32_t p__location__val__id,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemGetAccessBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemGetAccessBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__flags, p__flags__val, p__location, p__location__val__type, p__location__val__id, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemGetAccessEnd`) */
void barectf_hip_api_trace_hipMemGetAccessEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemGetAccessEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemGetAccessEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemGetAddressRangeBegin`) */
void barectf_hip_api_trace_hipMemGetAddressRangeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pbase,
	const uint64_t p__pbase__val,
	const uint64_t p__psize,
	const uint64_t p__psize__val,
	const uint64_t p__dptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemGetAddressRangeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemGetAddressRangeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pbase, p__pbase__val, p__psize, p__psize__val, p__dptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemGetAddressRangeEnd`) */
void barectf_hip_api_trace_hipMemGetAddressRangeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemGetAddressRangeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemGetAddressRangeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemGetAllocationGranularityBegin`) */
void barectf_hip_api_trace_hipMemGetAllocationGranularityBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__granularity,
	const uint64_t p__granularity__val,
	const uint64_t p__prop,
	const int32_t p__prop__val__type,
	const int32_t p__prop__val__requestedHandleType,
	const int32_t p__prop__val__location__type,
	const int32_t p__prop__val__location__id,
	const uint64_t p__prop__val__win32HandleMetaData,
	const int32_t p__option)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemGetAllocationGranularityBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemGetAllocationGranularityBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__granularity, p__granularity__val, p__prop, p__prop__val__type, p__prop__val__requestedHandleType, p__prop__val__location__type, p__prop__val__location__id, p__prop__val__win32HandleMetaData, p__option);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemGetAllocationGranularityEnd`) */
void barectf_hip_api_trace_hipMemGetAllocationGranularityEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemGetAllocationGranularityEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemGetAllocationGranularityEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemGetAllocationPropertiesFromHandleBegin`) */
void barectf_hip_api_trace_hipMemGetAllocationPropertiesFromHandleBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__prop,
	const int32_t p__prop__val__type,
	const int32_t p__prop__val__requestedHandleType,
	const int32_t p__prop__val__location__type,
	const int32_t p__prop__val__location__id,
	const uint64_t p__prop__val__win32HandleMetaData,
	const uint64_t p__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemGetAllocationPropertiesFromHandleBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemGetAllocationPropertiesFromHandleBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__prop, p__prop__val__type, p__prop__val__requestedHandleType, p__prop__val__location__type, p__prop__val__location__id, p__prop__val__win32HandleMetaData, p__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemGetAllocationPropertiesFromHandleEnd`) */
void barectf_hip_api_trace_hipMemGetAllocationPropertiesFromHandleEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemGetAllocationPropertiesFromHandleEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemGetAllocationPropertiesFromHandleEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemGetInfoBegin`) */
void barectf_hip_api_trace_hipMemGetInfoBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__free,
	const uint64_t p__free__val,
	const uint64_t p__total,
	const uint64_t p__total__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemGetInfoBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemGetInfoBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__free, p__free__val, p__total, p__total__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemGetInfoEnd`) */
void barectf_hip_api_trace_hipMemGetInfoEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemGetInfoEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemGetInfoEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemImportFromShareableHandleBegin`) */
void barectf_hip_api_trace_hipMemImportFromShareableHandleBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__handle,
	const uint64_t p__handle__val,
	const uint64_t p__osHandle,
	const int32_t p__shHandleType)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemImportFromShareableHandleBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemImportFromShareableHandleBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__handle, p__handle__val, p__osHandle, p__shHandleType);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemImportFromShareableHandleEnd`) */
void barectf_hip_api_trace_hipMemImportFromShareableHandleEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemImportFromShareableHandleEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemImportFromShareableHandleEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemMapArrayAsyncBegin`) */
void barectf_hip_api_trace_hipMemMapArrayAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mapInfoList,
	const int32_t p__mapInfoList__val__resourceType,
	const int32_t p__mapInfoList__val__subresourceType,
	const int32_t p__mapInfoList__val__memOperationType,
	const int32_t p__mapInfoList__val__memHandleType,
	const uint64_t p__mapInfoList__val__offset,
	const uint32_t p__mapInfoList__val__deviceBitMask,
	const uint32_t p__mapInfoList__val__flags,
	const uint64_t p__mapInfoList__val__reserved,
	const uint32_t p__count,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemMapArrayAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemMapArrayAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mapInfoList, p__mapInfoList__val__resourceType, p__mapInfoList__val__subresourceType, p__mapInfoList__val__memOperationType, p__mapInfoList__val__memHandleType, p__mapInfoList__val__offset, p__mapInfoList__val__deviceBitMask, p__mapInfoList__val__flags, p__mapInfoList__val__reserved, p__count, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemMapArrayAsyncEnd`) */
void barectf_hip_api_trace_hipMemMapArrayAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemMapArrayAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemMapArrayAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemMapBegin`) */
void barectf_hip_api_trace_hipMemMapBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__offset,
	const uint64_t p__handle,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemMapBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemMapBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr, p__size, p__offset, p__handle, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemMapEnd`) */
void barectf_hip_api_trace_hipMemMapEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemMapEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemMapEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolCreateBegin`) */
void barectf_hip_api_trace_hipMemPoolCreateBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const uint64_t p__mem_pool__val,
	const uint64_t p__pool_props,
	const int32_t p__pool_props__val__allocType,
	const int32_t p__pool_props__val__handleTypes,
	const int32_t p__pool_props__val__location__type,
	const int32_t p__pool_props__val__location__id,
	const uint64_t p__pool_props__val__win32SecurityAttributes,
	const uint64_t p__pool_props__val__maxSize,
	const uint64_t p__pool_props__val__reserved)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolCreateBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolCreateBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mem_pool, p__mem_pool__val, p__pool_props, p__pool_props__val__allocType, p__pool_props__val__handleTypes, p__pool_props__val__location__type, p__pool_props__val__location__id, p__pool_props__val__win32SecurityAttributes, p__pool_props__val__maxSize, p__pool_props__val__reserved);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolCreateEnd`) */
void barectf_hip_api_trace_hipMemPoolCreateEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolCreateEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolCreateEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolDestroyBegin`) */
void barectf_hip_api_trace_hipMemPoolDestroyBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolDestroyBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolDestroyBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mem_pool);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolDestroyEnd`) */
void barectf_hip_api_trace_hipMemPoolDestroyEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolDestroyEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolDestroyEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolExportPointerBegin`) */
void barectf_hip_api_trace_hipMemPoolExportPointerBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__export_data,
	const uint64_t p__export_data__val__reserved,
	const uint64_t p__dev_ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolExportPointerBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolExportPointerBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__export_data, p__export_data__val__reserved, p__dev_ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolExportPointerEnd`) */
void barectf_hip_api_trace_hipMemPoolExportPointerEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolExportPointerEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolExportPointerEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolExportToShareableHandleBegin`) */
void barectf_hip_api_trace_hipMemPoolExportToShareableHandleBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__shared_handle,
	const uint64_t p__mem_pool,
	const int32_t p__handle_type,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolExportToShareableHandleBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolExportToShareableHandleBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__shared_handle, p__mem_pool, p__handle_type, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolExportToShareableHandleEnd`) */
void barectf_hip_api_trace_hipMemPoolExportToShareableHandleEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolExportToShareableHandleEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolExportToShareableHandleEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolGetAccessBegin`) */
void barectf_hip_api_trace_hipMemPoolGetAccessBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__flags,
	const int32_t p__flags__val,
	const uint64_t p__mem_pool,
	const uint64_t p__location,
	const int32_t p__location__val__type,
	const int32_t p__location__val__id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolGetAccessBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolGetAccessBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__flags, p__flags__val, p__mem_pool, p__location, p__location__val__type, p__location__val__id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolGetAccessEnd`) */
void barectf_hip_api_trace_hipMemPoolGetAccessEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolGetAccessEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolGetAccessEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolGetAttributeBegin`) */
void barectf_hip_api_trace_hipMemPoolGetAttributeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const int32_t p__attr,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolGetAttributeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolGetAttributeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mem_pool, p__attr, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolGetAttributeEnd`) */
void barectf_hip_api_trace_hipMemPoolGetAttributeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolGetAttributeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolGetAttributeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolImportFromShareableHandleBegin`) */
void barectf_hip_api_trace_hipMemPoolImportFromShareableHandleBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const uint64_t p__mem_pool__val,
	const uint64_t p__shared_handle,
	const int32_t p__handle_type,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolImportFromShareableHandleBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolImportFromShareableHandleBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mem_pool, p__mem_pool__val, p__shared_handle, p__handle_type, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolImportFromShareableHandleEnd`) */
void barectf_hip_api_trace_hipMemPoolImportFromShareableHandleEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolImportFromShareableHandleEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolImportFromShareableHandleEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolImportPointerBegin`) */
void barectf_hip_api_trace_hipMemPoolImportPointerBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__dev_ptr__val,
	const uint64_t p__mem_pool,
	const uint64_t p__export_data,
	const uint64_t p__export_data__val__reserved)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolImportPointerBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolImportPointerBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dev_ptr, p__dev_ptr__val, p__mem_pool, p__export_data, p__export_data__val__reserved);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolImportPointerEnd`) */
void barectf_hip_api_trace_hipMemPoolImportPointerEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolImportPointerEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolImportPointerEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolSetAccessBegin`) */
void barectf_hip_api_trace_hipMemPoolSetAccessBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const uint64_t p__desc_list,
	const int32_t p__desc_list__val__location__type,
	const int32_t p__desc_list__val__location__id,
	const int32_t p__desc_list__val__flags,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolSetAccessBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolSetAccessBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mem_pool, p__desc_list, p__desc_list__val__location__type, p__desc_list__val__location__id, p__desc_list__val__flags, p__count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolSetAccessEnd`) */
void barectf_hip_api_trace_hipMemPoolSetAccessEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolSetAccessEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolSetAccessEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolSetAttributeBegin`) */
void barectf_hip_api_trace_hipMemPoolSetAttributeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const int32_t p__attr,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolSetAttributeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolSetAttributeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mem_pool, p__attr, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolSetAttributeEnd`) */
void barectf_hip_api_trace_hipMemPoolSetAttributeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolSetAttributeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolSetAttributeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolTrimToBegin`) */
void barectf_hip_api_trace_hipMemPoolTrimToBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mem_pool,
	const uint64_t p__min_bytes_to_hold)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolTrimToBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolTrimToBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mem_pool, p__min_bytes_to_hold);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPoolTrimToEnd`) */
void barectf_hip_api_trace_hipMemPoolTrimToEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPoolTrimToEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPoolTrimToEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPrefetchAsyncBegin`) */
void barectf_hip_api_trace_hipMemPrefetchAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__count,
	const int32_t p__device,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPrefetchAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPrefetchAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dev_ptr, p__count, p__device, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPrefetchAsyncEnd`) */
void barectf_hip_api_trace_hipMemPrefetchAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPrefetchAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPrefetchAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPtrGetInfoBegin`) */
void barectf_hip_api_trace_hipMemPtrGetInfoBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__size__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPtrGetInfoBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPtrGetInfoBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr, p__size, p__size__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemPtrGetInfoEnd`) */
void barectf_hip_api_trace_hipMemPtrGetInfoEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemPtrGetInfoEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemPtrGetInfoEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemRangeGetAttributeBegin`) */
void barectf_hip_api_trace_hipMemRangeGetAttributeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__data,
	const uint64_t p__data_size,
	const int32_t p__attribute,
	const uint64_t p__dev_ptr,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemRangeGetAttributeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemRangeGetAttributeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__data, p__data_size, p__attribute, p__dev_ptr, p__count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemRangeGetAttributeEnd`) */
void barectf_hip_api_trace_hipMemRangeGetAttributeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemRangeGetAttributeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemRangeGetAttributeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemRangeGetAttributesBegin`) */
void barectf_hip_api_trace_hipMemRangeGetAttributesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__data,
	const uint64_t p__data__val,
	const uint64_t p__data_sizes,
	const uint64_t p__data_sizes__val,
	const uint64_t p__attributes,
	const int32_t p__attributes__val,
	const uint64_t p__num_attributes,
	const uint64_t p__dev_ptr,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemRangeGetAttributesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemRangeGetAttributesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__data, p__data__val, p__data_sizes, p__data_sizes__val, p__attributes, p__attributes__val, p__num_attributes, p__dev_ptr, p__count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemRangeGetAttributesEnd`) */
void barectf_hip_api_trace_hipMemRangeGetAttributesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemRangeGetAttributesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemRangeGetAttributesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemReleaseBegin`) */
void barectf_hip_api_trace_hipMemReleaseBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemReleaseBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemReleaseBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemReleaseEnd`) */
void barectf_hip_api_trace_hipMemReleaseEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemReleaseEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemReleaseEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemRetainAllocationHandleBegin`) */
void barectf_hip_api_trace_hipMemRetainAllocationHandleBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__handle,
	const uint64_t p__handle__val,
	const uint64_t p__addr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemRetainAllocationHandleBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemRetainAllocationHandleBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__handle, p__handle__val, p__addr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemRetainAllocationHandleEnd`) */
void barectf_hip_api_trace_hipMemRetainAllocationHandleEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemRetainAllocationHandleEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemRetainAllocationHandleEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemSetAccessBegin`) */
void barectf_hip_api_trace_hipMemSetAccessBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__desc,
	const int32_t p__desc__val__location__type,
	const int32_t p__desc__val__location__id,
	const int32_t p__desc__val__flags,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemSetAccessBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemSetAccessBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr, p__size, p__desc, p__desc__val__location__type, p__desc__val__location__id, p__desc__val__flags, p__count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemSetAccessEnd`) */
void barectf_hip_api_trace_hipMemSetAccessEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemSetAccessEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemSetAccessEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemUnmapBegin`) */
void barectf_hip_api_trace_hipMemUnmapBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ptr,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemUnmapBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemUnmapBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ptr, p__size);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemUnmapEnd`) */
void barectf_hip_api_trace_hipMemUnmapEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemUnmapEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemUnmapEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DArrayToArrayBegin`) */
void barectf_hip_api_trace_hipMemcpy2DArrayToArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__wOffsetDst,
	const uint64_t p__hOffsetDst,
	const uint64_t p__src,
	const uint64_t p__wOffsetSrc,
	const uint64_t p__hOffsetSrc,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DArrayToArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DArrayToArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__wOffsetDst, p__hOffsetDst, p__src, p__wOffsetSrc, p__hOffsetSrc, p__width, p__height, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DArrayToArrayEnd`) */
void barectf_hip_api_trace_hipMemcpy2DArrayToArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DArrayToArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DArrayToArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpy2DAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__dpitch,
	const uint64_t p__src,
	const uint64_t p__spitch,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__dpitch, p__src, p__spitch, p__width, p__height, p__kind, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpy2DAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DBegin`) */
void barectf_hip_api_trace_hipMemcpy2DBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__dpitch,
	const uint64_t p__src,
	const uint64_t p__spitch,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__dpitch, p__src, p__spitch, p__width, p__height, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DEnd`) */
void barectf_hip_api_trace_hipMemcpy2DEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DFromArrayAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpy2DFromArrayAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__dpitch,
	const uint64_t p__src,
	const uint64_t p__wOffset,
	const uint64_t p__hOffset,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DFromArrayAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DFromArrayAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__dpitch, p__src, p__wOffset, p__hOffset, p__width, p__height, p__kind, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DFromArrayAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpy2DFromArrayAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DFromArrayAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DFromArrayAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DFromArrayBegin`) */
void barectf_hip_api_trace_hipMemcpy2DFromArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__dpitch,
	const uint64_t p__src,
	const uint64_t p__wOffset,
	const uint64_t p__hOffset,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DFromArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DFromArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__dpitch, p__src, p__wOffset, p__hOffset, p__width, p__height, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DFromArrayEnd`) */
void barectf_hip_api_trace_hipMemcpy2DFromArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DFromArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DFromArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DToArrayAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpy2DToArrayAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__wOffset,
	const uint64_t p__hOffset,
	const uint64_t p__src,
	const uint64_t p__spitch,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DToArrayAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DToArrayAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__wOffset, p__hOffset, p__src, p__spitch, p__width, p__height, p__kind, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DToArrayAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpy2DToArrayAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DToArrayAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DToArrayAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DToArrayBegin`) */
void barectf_hip_api_trace_hipMemcpy2DToArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__wOffset,
	const uint64_t p__hOffset,
	const uint64_t p__src,
	const uint64_t p__spitch,
	const uint64_t p__width,
	const uint64_t p__height,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DToArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DToArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__wOffset, p__hOffset, p__src, p__spitch, p__width, p__height, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy2DToArrayEnd`) */
void barectf_hip_api_trace_hipMemcpy2DToArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy2DToArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy2DToArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy3DAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpy3DAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__p,
	const uint64_t p__p__val__srcArray,
	const uint64_t p__p__val__srcPos__x,
	const uint64_t p__p__val__srcPos__y,
	const uint64_t p__p__val__srcPos__z,
	const uint64_t p__p__val__srcPtr__ptr,
	const uint64_t p__p__val__srcPtr__pitch,
	const uint64_t p__p__val__srcPtr__xsize,
	const uint64_t p__p__val__srcPtr__ysize,
	const uint64_t p__p__val__dstArray,
	const uint64_t p__p__val__dstPos__x,
	const uint64_t p__p__val__dstPos__y,
	const uint64_t p__p__val__dstPos__z,
	const uint64_t p__p__val__dstPtr__ptr,
	const uint64_t p__p__val__dstPtr__pitch,
	const uint64_t p__p__val__dstPtr__xsize,
	const uint64_t p__p__val__dstPtr__ysize,
	const uint64_t p__p__val__extent__width,
	const uint64_t p__p__val__extent__height,
	const uint64_t p__p__val__extent__depth,
	const int32_t p__p__val__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy3DAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy3DAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__p, p__p__val__srcArray, p__p__val__srcPos__x, p__p__val__srcPos__y, p__p__val__srcPos__z, p__p__val__srcPtr__ptr, p__p__val__srcPtr__pitch, p__p__val__srcPtr__xsize, p__p__val__srcPtr__ysize, p__p__val__dstArray, p__p__val__dstPos__x, p__p__val__dstPos__y, p__p__val__dstPos__z, p__p__val__dstPtr__ptr, p__p__val__dstPtr__pitch, p__p__val__dstPtr__xsize, p__p__val__dstPtr__ysize, p__p__val__extent__width, p__p__val__extent__height, p__p__val__extent__depth, p__p__val__kind, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy3DAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpy3DAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy3DAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy3DAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy3DBegin`) */
void barectf_hip_api_trace_hipMemcpy3DBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__p,
	const uint64_t p__p__val__srcArray,
	const uint64_t p__p__val__srcPos__x,
	const uint64_t p__p__val__srcPos__y,
	const uint64_t p__p__val__srcPos__z,
	const uint64_t p__p__val__srcPtr__ptr,
	const uint64_t p__p__val__srcPtr__pitch,
	const uint64_t p__p__val__srcPtr__xsize,
	const uint64_t p__p__val__srcPtr__ysize,
	const uint64_t p__p__val__dstArray,
	const uint64_t p__p__val__dstPos__x,
	const uint64_t p__p__val__dstPos__y,
	const uint64_t p__p__val__dstPos__z,
	const uint64_t p__p__val__dstPtr__ptr,
	const uint64_t p__p__val__dstPtr__pitch,
	const uint64_t p__p__val__dstPtr__xsize,
	const uint64_t p__p__val__dstPtr__ysize,
	const uint64_t p__p__val__extent__width,
	const uint64_t p__p__val__extent__height,
	const uint64_t p__p__val__extent__depth,
	const int32_t p__p__val__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy3DBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy3DBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__p, p__p__val__srcArray, p__p__val__srcPos__x, p__p__val__srcPos__y, p__p__val__srcPos__z, p__p__val__srcPtr__ptr, p__p__val__srcPtr__pitch, p__p__val__srcPtr__xsize, p__p__val__srcPtr__ysize, p__p__val__dstArray, p__p__val__dstPos__x, p__p__val__dstPos__y, p__p__val__dstPos__z, p__p__val__dstPtr__ptr, p__p__val__dstPtr__pitch, p__p__val__dstPtr__xsize, p__p__val__dstPtr__ysize, p__p__val__extent__width, p__p__val__extent__height, p__p__val__extent__depth, p__p__val__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpy3DEnd`) */
void barectf_hip_api_trace_hipMemcpy3DEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpy3DEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpy3DEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpyAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__src, p__sizeBytes, p__kind, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpyAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyAtoABegin`) */
void barectf_hip_api_trace_hipMemcpyAtoABegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dstArray,
	const uint64_t p__dstOffset,
	const uint64_t p__srcArray,
	const uint64_t p__srcOffset,
	const uint64_t p__ByteCount)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyAtoABegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyAtoABegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dstArray, p__dstOffset, p__srcArray, p__srcOffset, p__ByteCount);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyAtoAEnd`) */
void barectf_hip_api_trace_hipMemcpyAtoAEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyAtoAEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyAtoAEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyAtoDBegin`) */
void barectf_hip_api_trace_hipMemcpyAtoDBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dstDevice,
	const uint64_t p__srcArray,
	const uint64_t p__srcOffset,
	const uint64_t p__ByteCount)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyAtoDBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyAtoDBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dstDevice, p__srcArray, p__srcOffset, p__ByteCount);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyAtoDEnd`) */
void barectf_hip_api_trace_hipMemcpyAtoDEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyAtoDEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyAtoDEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyAtoHAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpyAtoHAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dstHost,
	const uint64_t p__srcArray,
	const uint64_t p__srcOffset,
	const uint64_t p__ByteCount,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyAtoHAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyAtoHAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dstHost, p__srcArray, p__srcOffset, p__ByteCount, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyAtoHAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpyAtoHAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyAtoHAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyAtoHAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyAtoHBegin`) */
void barectf_hip_api_trace_hipMemcpyAtoHBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__srcArray,
	const uint64_t p__srcOffset,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyAtoHBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyAtoHBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__srcArray, p__srcOffset, p__count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyAtoHEnd`) */
void barectf_hip_api_trace_hipMemcpyAtoHEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyAtoHEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyAtoHEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyBegin`) */
void barectf_hip_api_trace_hipMemcpyBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__src, p__sizeBytes, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyDtoABegin`) */
void barectf_hip_api_trace_hipMemcpyDtoABegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dstArray,
	const uint64_t p__dstOffset,
	const uint64_t p__srcDevice,
	const uint64_t p__ByteCount)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyDtoABegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyDtoABegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dstArray, p__dstOffset, p__srcDevice, p__ByteCount);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyDtoAEnd`) */
void barectf_hip_api_trace_hipMemcpyDtoAEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyDtoAEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyDtoAEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyDtoDAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpyDtoDAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyDtoDAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyDtoDAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__src, p__sizeBytes, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyDtoDAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpyDtoDAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyDtoDAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyDtoDAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyDtoDBegin`) */
void barectf_hip_api_trace_hipMemcpyDtoDBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyDtoDBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyDtoDBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__src, p__sizeBytes);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyDtoDEnd`) */
void barectf_hip_api_trace_hipMemcpyDtoDEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyDtoDEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyDtoDEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyDtoHAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpyDtoHAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyDtoHAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyDtoHAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__src, p__sizeBytes, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyDtoHAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpyDtoHAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyDtoHAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyDtoHAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyDtoHBegin`) */
void barectf_hip_api_trace_hipMemcpyDtoHBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyDtoHBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyDtoHBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__src, p__sizeBytes);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyDtoHEnd`) */
void barectf_hip_api_trace_hipMemcpyDtoHEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyDtoHEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyDtoHEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyEnd`) */
void barectf_hip_api_trace_hipMemcpyEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyFromArrayBegin`) */
void barectf_hip_api_trace_hipMemcpyFromArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__srcArray,
	const uint64_t p__wOffset,
	const uint64_t p__hOffset,
	const uint64_t p__count,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyFromArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyFromArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__srcArray, p__wOffset, p__hOffset, p__count, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyFromArrayEnd`) */
void barectf_hip_api_trace_hipMemcpyFromArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyFromArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyFromArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyFromSymbolAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpyFromSymbolAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__symbol,
	const uint64_t p__sizeBytes,
	const uint64_t p__offset,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyFromSymbolAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyFromSymbolAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__symbol, p__sizeBytes, p__offset, p__kind, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyFromSymbolAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpyFromSymbolAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyFromSymbolAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyFromSymbolAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyFromSymbolBegin`) */
void barectf_hip_api_trace_hipMemcpyFromSymbolBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__symbol,
	const uint64_t p__sizeBytes,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyFromSymbolBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyFromSymbolBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__symbol, p__sizeBytes, p__offset, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyFromSymbolEnd`) */
void barectf_hip_api_trace_hipMemcpyFromSymbolEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyFromSymbolEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyFromSymbolEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyHtoAAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpyHtoAAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dstArray,
	const uint64_t p__dstOffset,
	const uint64_t p__srcHost,
	const uint64_t p__ByteCount,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyHtoAAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyHtoAAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dstArray, p__dstOffset, p__srcHost, p__ByteCount, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyHtoAAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpyHtoAAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyHtoAAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyHtoAAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyHtoABegin`) */
void barectf_hip_api_trace_hipMemcpyHtoABegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dstArray,
	const uint64_t p__dstOffset,
	const uint64_t p__srcHost,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyHtoABegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyHtoABegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dstArray, p__dstOffset, p__srcHost, p__count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyHtoAEnd`) */
void barectf_hip_api_trace_hipMemcpyHtoAEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyHtoAEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyHtoAEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyHtoDAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpyHtoDAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyHtoDAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyHtoDAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__src, p__sizeBytes, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyHtoDAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpyHtoDAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyHtoDAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyHtoDAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyHtoDBegin`) */
void barectf_hip_api_trace_hipMemcpyHtoDBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyHtoDBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyHtoDBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__src, p__sizeBytes);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyHtoDEnd`) */
void barectf_hip_api_trace_hipMemcpyHtoDEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyHtoDEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyHtoDEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyParam2DAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpyParam2DAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pCopy,
	const uint64_t p__pCopy__val__srcXInBytes,
	const uint64_t p__pCopy__val__srcY,
	const int32_t p__pCopy__val__srcMemoryType,
	const uint64_t p__pCopy__val__srcHost,
	const uint64_t p__pCopy__val__srcDevice,
	const uint64_t p__pCopy__val__srcArray,
	const uint64_t p__pCopy__val__srcPitch,
	const uint64_t p__pCopy__val__dstXInBytes,
	const uint64_t p__pCopy__val__dstY,
	const int32_t p__pCopy__val__dstMemoryType,
	const uint64_t p__pCopy__val__dstHost,
	const uint64_t p__pCopy__val__dstDevice,
	const uint64_t p__pCopy__val__dstArray,
	const uint64_t p__pCopy__val__dstPitch,
	const uint64_t p__pCopy__val__WidthInBytes,
	const uint64_t p__pCopy__val__Height,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyParam2DAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyParam2DAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pCopy, p__pCopy__val__srcXInBytes, p__pCopy__val__srcY, p__pCopy__val__srcMemoryType, p__pCopy__val__srcHost, p__pCopy__val__srcDevice, p__pCopy__val__srcArray, p__pCopy__val__srcPitch, p__pCopy__val__dstXInBytes, p__pCopy__val__dstY, p__pCopy__val__dstMemoryType, p__pCopy__val__dstHost, p__pCopy__val__dstDevice, p__pCopy__val__dstArray, p__pCopy__val__dstPitch, p__pCopy__val__WidthInBytes, p__pCopy__val__Height, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyParam2DAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpyParam2DAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyParam2DAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyParam2DAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyParam2DBegin`) */
void barectf_hip_api_trace_hipMemcpyParam2DBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pCopy,
	const uint64_t p__pCopy__val__srcXInBytes,
	const uint64_t p__pCopy__val__srcY,
	const int32_t p__pCopy__val__srcMemoryType,
	const uint64_t p__pCopy__val__srcHost,
	const uint64_t p__pCopy__val__srcDevice,
	const uint64_t p__pCopy__val__srcArray,
	const uint64_t p__pCopy__val__srcPitch,
	const uint64_t p__pCopy__val__dstXInBytes,
	const uint64_t p__pCopy__val__dstY,
	const int32_t p__pCopy__val__dstMemoryType,
	const uint64_t p__pCopy__val__dstHost,
	const uint64_t p__pCopy__val__dstDevice,
	const uint64_t p__pCopy__val__dstArray,
	const uint64_t p__pCopy__val__dstPitch,
	const uint64_t p__pCopy__val__WidthInBytes,
	const uint64_t p__pCopy__val__Height)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyParam2DBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyParam2DBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pCopy, p__pCopy__val__srcXInBytes, p__pCopy__val__srcY, p__pCopy__val__srcMemoryType, p__pCopy__val__srcHost, p__pCopy__val__srcDevice, p__pCopy__val__srcArray, p__pCopy__val__srcPitch, p__pCopy__val__dstXInBytes, p__pCopy__val__dstY, p__pCopy__val__dstMemoryType, p__pCopy__val__dstHost, p__pCopy__val__dstDevice, p__pCopy__val__dstArray, p__pCopy__val__dstPitch, p__pCopy__val__WidthInBytes, p__pCopy__val__Height);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyParam2DEnd`) */
void barectf_hip_api_trace_hipMemcpyParam2DEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyParam2DEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyParam2DEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyPeerAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpyPeerAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const int32_t p__dstDeviceId,
	const uint64_t p__src,
	const int32_t p__srcDevice,
	const uint64_t p__sizeBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyPeerAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyPeerAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__dstDeviceId, p__src, p__srcDevice, p__sizeBytes, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyPeerAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpyPeerAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyPeerAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyPeerAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyPeerBegin`) */
void barectf_hip_api_trace_hipMemcpyPeerBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const int32_t p__dstDeviceId,
	const uint64_t p__src,
	const int32_t p__srcDeviceId,
	const uint64_t p__sizeBytes)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyPeerBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyPeerBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__dstDeviceId, p__src, p__srcDeviceId, p__sizeBytes);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyPeerEnd`) */
void barectf_hip_api_trace_hipMemcpyPeerEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyPeerEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyPeerEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyToArrayBegin`) */
void barectf_hip_api_trace_hipMemcpyToArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__wOffset,
	const uint64_t p__hOffset,
	const uint64_t p__src,
	const uint64_t p__count,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyToArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyToArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__wOffset, p__hOffset, p__src, p__count, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyToArrayEnd`) */
void barectf_hip_api_trace_hipMemcpyToArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyToArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyToArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyToSymbolAsyncBegin`) */
void barectf_hip_api_trace_hipMemcpyToSymbolAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__symbol,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const uint64_t p__offset,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyToSymbolAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyToSymbolAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__symbol, p__src, p__sizeBytes, p__offset, p__kind, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyToSymbolAsyncEnd`) */
void barectf_hip_api_trace_hipMemcpyToSymbolAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyToSymbolAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyToSymbolAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyToSymbolBegin`) */
void barectf_hip_api_trace_hipMemcpyToSymbolBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__symbol,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const uint64_t p__offset,
	const int32_t p__kind)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyToSymbolBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyToSymbolBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__symbol, p__src, p__sizeBytes, p__offset, p__kind);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyToSymbolEnd`) */
void barectf_hip_api_trace_hipMemcpyToSymbolEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyToSymbolEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyToSymbolEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyWithStreamBegin`) */
void barectf_hip_api_trace_hipMemcpyWithStreamBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__sizeBytes,
	const int32_t p__kind,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyWithStreamBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyWithStreamBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__src, p__sizeBytes, p__kind, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemcpyWithStreamEnd`) */
void barectf_hip_api_trace_hipMemcpyWithStreamEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemcpyWithStreamEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemcpyWithStreamEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemset2DAsyncBegin`) */
void barectf_hip_api_trace_hipMemset2DAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__pitch,
	const int32_t p__value,
	const uint64_t p__width,
	const uint64_t p__height,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemset2DAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemset2DAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__pitch, p__value, p__width, p__height, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemset2DAsyncEnd`) */
void barectf_hip_api_trace_hipMemset2DAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemset2DAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemset2DAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemset2DBegin`) */
void barectf_hip_api_trace_hipMemset2DBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const uint64_t p__pitch,
	const int32_t p__value,
	const uint64_t p__width,
	const uint64_t p__height)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemset2DBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemset2DBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__pitch, p__value, p__width, p__height);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemset2DEnd`) */
void barectf_hip_api_trace_hipMemset2DEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemset2DEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemset2DEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemset3DAsyncBegin`) */
void barectf_hip_api_trace_hipMemset3DAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pitchedDevPtr__ptr,
	const uint64_t p__pitchedDevPtr__pitch,
	const uint64_t p__pitchedDevPtr__xsize,
	const uint64_t p__pitchedDevPtr__ysize,
	const int32_t p__value,
	const uint64_t p__extent__width,
	const uint64_t p__extent__height,
	const uint64_t p__extent__depth,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemset3DAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemset3DAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pitchedDevPtr__ptr, p__pitchedDevPtr__pitch, p__pitchedDevPtr__xsize, p__pitchedDevPtr__ysize, p__value, p__extent__width, p__extent__height, p__extent__depth, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemset3DAsyncEnd`) */
void barectf_hip_api_trace_hipMemset3DAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemset3DAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemset3DAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemset3DBegin`) */
void barectf_hip_api_trace_hipMemset3DBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pitchedDevPtr__ptr,
	const uint64_t p__pitchedDevPtr__pitch,
	const uint64_t p__pitchedDevPtr__xsize,
	const uint64_t p__pitchedDevPtr__ysize,
	const int32_t p__value,
	const uint64_t p__extent__width,
	const uint64_t p__extent__height,
	const uint64_t p__extent__depth)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemset3DBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemset3DBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pitchedDevPtr__ptr, p__pitchedDevPtr__pitch, p__pitchedDevPtr__xsize, p__pitchedDevPtr__ysize, p__value, p__extent__width, p__extent__height, p__extent__depth);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemset3DEnd`) */
void barectf_hip_api_trace_hipMemset3DEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemset3DEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemset3DEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetAsyncBegin`) */
void barectf_hip_api_trace_hipMemsetAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const int32_t p__value,
	const uint64_t p__sizeBytes,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__value, p__sizeBytes, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetAsyncEnd`) */
void barectf_hip_api_trace_hipMemsetAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetBegin`) */
void barectf_hip_api_trace_hipMemsetBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const int32_t p__value,
	const uint64_t p__sizeBytes)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__value, p__sizeBytes);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetD16AsyncBegin`) */
void barectf_hip_api_trace_hipMemsetD16AsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dest,
	const uint16_t p__value,
	const uint64_t p__count,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetD16AsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetD16AsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dest, p__value, p__count, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetD16AsyncEnd`) */
void barectf_hip_api_trace_hipMemsetD16AsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetD16AsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetD16AsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetD16Begin`) */
void barectf_hip_api_trace_hipMemsetD16Begin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dest,
	const uint16_t p__value,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetD16Begin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetD16Begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dest, p__value, p__count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetD16End`) */
void barectf_hip_api_trace_hipMemsetD16End(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetD16End(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetD16End(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetD32AsyncBegin`) */
void barectf_hip_api_trace_hipMemsetD32AsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dst,
	const int32_t p__value,
	const uint64_t p__count,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetD32AsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetD32AsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dst, p__value, p__count, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetD32AsyncEnd`) */
void barectf_hip_api_trace_hipMemsetD32AsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetD32AsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetD32AsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetD32Begin`) */
void barectf_hip_api_trace_hipMemsetD32Begin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dest,
	const int32_t p__value,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetD32Begin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetD32Begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dest, p__value, p__count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetD32End`) */
void barectf_hip_api_trace_hipMemsetD32End(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetD32End(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetD32End(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetD8AsyncBegin`) */
void barectf_hip_api_trace_hipMemsetD8AsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dest,
	const uint8_t p__value,
	const uint64_t p__count,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetD8AsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetD8AsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dest, p__value, p__count, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetD8AsyncEnd`) */
void barectf_hip_api_trace_hipMemsetD8AsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetD8AsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetD8AsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetD8Begin`) */
void barectf_hip_api_trace_hipMemsetD8Begin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dest,
	const uint8_t p__value,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetD8Begin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetD8Begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dest, p__value, p__count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetD8End`) */
void barectf_hip_api_trace_hipMemsetD8End(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetD8End(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetD8End(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMemsetEnd`) */
void barectf_hip_api_trace_hipMemsetEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMemsetEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMemsetEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMipmappedArrayCreateBegin`) */
void barectf_hip_api_trace_hipMipmappedArrayCreateBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pHandle,
	const uint64_t p__pHandle__val,
	const uint64_t p__pMipmappedArrayDesc,
	const uint64_t p__pMipmappedArrayDesc__val__Width,
	const uint64_t p__pMipmappedArrayDesc__val__Height,
	const uint64_t p__pMipmappedArrayDesc__val__Depth,
	const int32_t p__pMipmappedArrayDesc__val__Format,
	const uint32_t p__pMipmappedArrayDesc__val__NumChannels,
	const uint32_t p__pMipmappedArrayDesc__val__Flags,
	const uint32_t p__numMipmapLevels)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMipmappedArrayCreateBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMipmappedArrayCreateBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pHandle, p__pHandle__val, p__pMipmappedArrayDesc, p__pMipmappedArrayDesc__val__Width, p__pMipmappedArrayDesc__val__Height, p__pMipmappedArrayDesc__val__Depth, p__pMipmappedArrayDesc__val__Format, p__pMipmappedArrayDesc__val__NumChannels, p__pMipmappedArrayDesc__val__Flags, p__numMipmapLevels);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMipmappedArrayCreateEnd`) */
void barectf_hip_api_trace_hipMipmappedArrayCreateEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMipmappedArrayCreateEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMipmappedArrayCreateEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMipmappedArrayDestroyBegin`) */
void barectf_hip_api_trace_hipMipmappedArrayDestroyBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__hMipmappedArray)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMipmappedArrayDestroyBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMipmappedArrayDestroyBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__hMipmappedArray);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMipmappedArrayDestroyEnd`) */
void barectf_hip_api_trace_hipMipmappedArrayDestroyEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMipmappedArrayDestroyEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMipmappedArrayDestroyEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMipmappedArrayGetLevelBegin`) */
void barectf_hip_api_trace_hipMipmappedArrayGetLevelBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pLevelArray,
	const uint64_t p__pLevelArray__val,
	const uint64_t p__hMipMappedArray,
	const uint32_t p__level)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMipmappedArrayGetLevelBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMipmappedArrayGetLevelBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pLevelArray, p__pLevelArray__val, p__hMipMappedArray, p__level);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipMipmappedArrayGetLevelEnd`) */
void barectf_hip_api_trace_hipMipmappedArrayGetLevelEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipMipmappedArrayGetLevelEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipMipmappedArrayGetLevelEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleGetFunctionBegin`) */
void barectf_hip_api_trace_hipModuleGetFunctionBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__function,
	const uint64_t p__function__val,
	const uint64_t p__module,
	const char * const p__kname,
	const int8_t p__kname__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleGetFunctionBegin(_TO_VOID_PTR(ctx), cc__kernel_name, p__kname);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleGetFunctionBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__function, p__function__val, p__module, p__kname, p__kname__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleGetFunctionEnd`) */
void barectf_hip_api_trace_hipModuleGetFunctionEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleGetFunctionEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleGetFunctionEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleGetGlobalBegin`) */
void barectf_hip_api_trace_hipModuleGetGlobalBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dptr,
	const uint64_t p__dptr__val,
	const uint64_t p__bytes,
	const uint64_t p__bytes__val,
	const uint64_t p__hmod,
	const char * const p__name,
	const int8_t p__name__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleGetGlobalBegin(_TO_VOID_PTR(ctx), cc__kernel_name, p__name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleGetGlobalBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dptr, p__dptr__val, p__bytes, p__bytes__val, p__hmod, p__name, p__name__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleGetGlobalEnd`) */
void barectf_hip_api_trace_hipModuleGetGlobalEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleGetGlobalEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleGetGlobalEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleGetTexRefBegin`) */
void barectf_hip_api_trace_hipModuleGetTexRefBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const uint64_t p__texRef__val,
	const uint64_t p__hmod,
	const char * const p__name,
	const int8_t p__name__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleGetTexRefBegin(_TO_VOID_PTR(ctx), cc__kernel_name, p__name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleGetTexRefBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__texRef, p__texRef__val, p__hmod, p__name, p__name__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleGetTexRefEnd`) */
void barectf_hip_api_trace_hipModuleGetTexRefEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleGetTexRefEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleGetTexRefEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleLaunchCooperativeKernelBegin`) */
void barectf_hip_api_trace_hipModuleLaunchCooperativeKernelBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__f,
	const uint32_t p__gridDimX,
	const uint32_t p__gridDimY,
	const uint32_t p__gridDimZ,
	const uint32_t p__blockDimX,
	const uint32_t p__blockDimY,
	const uint32_t p__blockDimZ,
	const uint32_t p__sharedMemBytes,
	const uint64_t p__stream,
	const uint64_t p__kernelParams,
	const uint64_t p__kernelParams__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleLaunchCooperativeKernelBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleLaunchCooperativeKernelBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__f, p__gridDimX, p__gridDimY, p__gridDimZ, p__blockDimX, p__blockDimY, p__blockDimZ, p__sharedMemBytes, p__stream, p__kernelParams, p__kernelParams__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleLaunchCooperativeKernelEnd`) */
void barectf_hip_api_trace_hipModuleLaunchCooperativeKernelEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleLaunchCooperativeKernelEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleLaunchCooperativeKernelEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleLaunchCooperativeKernelMultiDeviceBegin`) */
void barectf_hip_api_trace_hipModuleLaunchCooperativeKernelMultiDeviceBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__launchParamsList,
	const uint64_t p__launchParamsList__val__function,
	const uint32_t p__launchParamsList__val__gridDimX,
	const uint32_t p__launchParamsList__val__gridDimY,
	const uint32_t p__launchParamsList__val__gridDimZ,
	const uint32_t p__launchParamsList__val__blockDimX,
	const uint32_t p__launchParamsList__val__blockDimY,
	const uint32_t p__launchParamsList__val__blockDimZ,
	const uint32_t p__launchParamsList__val__sharedMemBytes,
	const uint64_t p__launchParamsList__val__hStream,
	const uint64_t p__launchParamsList__val__kernelParams,
	const uint32_t p__numDevices,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleLaunchCooperativeKernelMultiDeviceBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleLaunchCooperativeKernelMultiDeviceBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__launchParamsList, p__launchParamsList__val__function, p__launchParamsList__val__gridDimX, p__launchParamsList__val__gridDimY, p__launchParamsList__val__gridDimZ, p__launchParamsList__val__blockDimX, p__launchParamsList__val__blockDimY, p__launchParamsList__val__blockDimZ, p__launchParamsList__val__sharedMemBytes, p__launchParamsList__val__hStream, p__launchParamsList__val__kernelParams, p__numDevices, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleLaunchCooperativeKernelMultiDeviceEnd`) */
void barectf_hip_api_trace_hipModuleLaunchCooperativeKernelMultiDeviceEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleLaunchCooperativeKernelMultiDeviceEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleLaunchCooperativeKernelMultiDeviceEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleLaunchKernelBegin`) */
void barectf_hip_api_trace_hipModuleLaunchKernelBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__f,
	const uint32_t p__gridDimX,
	const uint32_t p__gridDimY,
	const uint32_t p__gridDimZ,
	const uint32_t p__blockDimX,
	const uint32_t p__blockDimY,
	const uint32_t p__blockDimZ,
	const uint32_t p__sharedMemBytes,
	const uint64_t p__stream,
	const uint64_t p__kernelParams,
	const uint64_t p__kernelParams__val,
	const uint64_t p__extra,
	const uint64_t p__extra__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleLaunchKernelBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleLaunchKernelBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__f, p__gridDimX, p__gridDimY, p__gridDimZ, p__blockDimX, p__blockDimY, p__blockDimZ, p__sharedMemBytes, p__stream, p__kernelParams, p__kernelParams__val, p__extra, p__extra__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleLaunchKernelEnd`) */
void barectf_hip_api_trace_hipModuleLaunchKernelEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleLaunchKernelEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleLaunchKernelEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleLoadBegin`) */
void barectf_hip_api_trace_hipModuleLoadBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__module,
	const uint64_t p__module__val,
	const char * const p__fname,
	const int8_t p__fname__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleLoadBegin(_TO_VOID_PTR(ctx), cc__kernel_name, p__fname);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleLoadBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__module, p__module__val, p__fname, p__fname__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleLoadDataBegin`) */
void barectf_hip_api_trace_hipModuleLoadDataBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__module,
	const uint64_t p__module__val,
	const uint64_t p__image)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleLoadDataBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleLoadDataBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__module, p__module__val, p__image);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleLoadDataEnd`) */
void barectf_hip_api_trace_hipModuleLoadDataEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleLoadDataEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleLoadDataEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleLoadDataExBegin`) */
void barectf_hip_api_trace_hipModuleLoadDataExBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__module,
	const uint64_t p__module__val,
	const uint64_t p__image,
	const uint32_t p__numOptions,
	const uint64_t p__options,
	const int32_t p__options__val,
	const uint64_t p__optionsValues,
	const uint64_t p__optionsValues__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleLoadDataExBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleLoadDataExBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__module, p__module__val, p__image, p__numOptions, p__options, p__options__val, p__optionsValues, p__optionsValues__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleLoadDataExEnd`) */
void barectf_hip_api_trace_hipModuleLoadDataExEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleLoadDataExEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleLoadDataExEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleLoadEnd`) */
void barectf_hip_api_trace_hipModuleLoadEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleLoadEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleLoadEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleOccupancyMaxActiveBlocksPerMultiprocessorBegin`) */
void barectf_hip_api_trace_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__numBlocks,
	const int32_t p__numBlocks__val,
	const uint64_t p__f,
	const int32_t p__blockSize,
	const uint64_t p__dynSharedMemPerBlk)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__numBlocks, p__numBlocks__val, p__f, p__blockSize, p__dynSharedMemPerBlk);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleOccupancyMaxActiveBlocksPerMultiprocessorEnd`) */
void barectf_hip_api_trace_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsBegin`) */
void barectf_hip_api_trace_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__numBlocks,
	const int32_t p__numBlocks__val,
	const uint64_t p__f,
	const int32_t p__blockSize,
	const uint64_t p__dynSharedMemPerBlk,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__numBlocks, p__numBlocks__val, p__f, p__blockSize, p__dynSharedMemPerBlk, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsEnd`) */
void barectf_hip_api_trace_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleOccupancyMaxPotentialBlockSizeBegin`) */
void barectf_hip_api_trace_hipModuleOccupancyMaxPotentialBlockSizeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__gridSize,
	const int32_t p__gridSize__val,
	const uint64_t p__blockSize,
	const int32_t p__blockSize__val,
	const uint64_t p__f,
	const uint64_t p__dynSharedMemPerBlk,
	const int32_t p__blockSizeLimit)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleOccupancyMaxPotentialBlockSizeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleOccupancyMaxPotentialBlockSizeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__gridSize, p__gridSize__val, p__blockSize, p__blockSize__val, p__f, p__dynSharedMemPerBlk, p__blockSizeLimit);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleOccupancyMaxPotentialBlockSizeEnd`) */
void barectf_hip_api_trace_hipModuleOccupancyMaxPotentialBlockSizeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleOccupancyMaxPotentialBlockSizeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleOccupancyMaxPotentialBlockSizeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleOccupancyMaxPotentialBlockSizeWithFlagsBegin`) */
void barectf_hip_api_trace_hipModuleOccupancyMaxPotentialBlockSizeWithFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__gridSize,
	const int32_t p__gridSize__val,
	const uint64_t p__blockSize,
	const int32_t p__blockSize__val,
	const uint64_t p__f,
	const uint64_t p__dynSharedMemPerBlk,
	const int32_t p__blockSizeLimit,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleOccupancyMaxPotentialBlockSizeWithFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleOccupancyMaxPotentialBlockSizeWithFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__gridSize, p__gridSize__val, p__blockSize, p__blockSize__val, p__f, p__dynSharedMemPerBlk, p__blockSizeLimit, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleOccupancyMaxPotentialBlockSizeWithFlagsEnd`) */
void barectf_hip_api_trace_hipModuleOccupancyMaxPotentialBlockSizeWithFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleOccupancyMaxPotentialBlockSizeWithFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleOccupancyMaxPotentialBlockSizeWithFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleUnloadBegin`) */
void barectf_hip_api_trace_hipModuleUnloadBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__module)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleUnloadBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleUnloadBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__module);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipModuleUnloadEnd`) */
void barectf_hip_api_trace_hipModuleUnloadEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipModuleUnloadEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipModuleUnloadEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipOccupancyMaxActiveBlocksPerMultiprocessorBegin`) */
void barectf_hip_api_trace_hipOccupancyMaxActiveBlocksPerMultiprocessorBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__numBlocks,
	const int32_t p__numBlocks__val,
	const uint64_t p__f,
	const int32_t p__blockSize,
	const uint64_t p__dynamicSMemSize)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__numBlocks, p__numBlocks__val, p__f, p__blockSize, p__dynamicSMemSize);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipOccupancyMaxActiveBlocksPerMultiprocessorEnd`) */
void barectf_hip_api_trace_hipOccupancyMaxActiveBlocksPerMultiprocessorEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsBegin`) */
void barectf_hip_api_trace_hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__numBlocks,
	const int32_t p__numBlocks__val,
	const uint64_t p__f,
	const int32_t p__blockSize,
	const uint64_t p__dynamicSMemSize,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__numBlocks, p__numBlocks__val, p__f, p__blockSize, p__dynamicSMemSize, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsEnd`) */
void barectf_hip_api_trace_hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipOccupancyMaxPotentialBlockSizeBegin`) */
void barectf_hip_api_trace_hipOccupancyMaxPotentialBlockSizeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__gridSize,
	const int32_t p__gridSize__val,
	const uint64_t p__blockSize,
	const int32_t p__blockSize__val,
	const uint64_t p__f,
	const uint64_t p__dynSharedMemPerBlk,
	const int32_t p__blockSizeLimit)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipOccupancyMaxPotentialBlockSizeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipOccupancyMaxPotentialBlockSizeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__gridSize, p__gridSize__val, p__blockSize, p__blockSize__val, p__f, p__dynSharedMemPerBlk, p__blockSizeLimit);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipOccupancyMaxPotentialBlockSizeEnd`) */
void barectf_hip_api_trace_hipOccupancyMaxPotentialBlockSizeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipOccupancyMaxPotentialBlockSizeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipOccupancyMaxPotentialBlockSizeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipPeekAtLastErrorBegin`) */
void barectf_hip_api_trace_hipPeekAtLastErrorBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipPeekAtLastErrorBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipPeekAtLastErrorBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipPeekAtLastErrorEnd`) */
void barectf_hip_api_trace_hipPeekAtLastErrorEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipPeekAtLastErrorEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipPeekAtLastErrorEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipPointerGetAttributeBegin`) */
void barectf_hip_api_trace_hipPointerGetAttributeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__data,
	const int32_t p__attribute,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipPointerGetAttributeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipPointerGetAttributeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__data, p__attribute, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipPointerGetAttributeEnd`) */
void barectf_hip_api_trace_hipPointerGetAttributeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipPointerGetAttributeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipPointerGetAttributeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipPointerGetAttributesBegin`) */
void barectf_hip_api_trace_hipPointerGetAttributesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__attributes,
	const int32_t p__attributes__val__type,
	const int32_t p__attributes__val__device,
	const uint64_t p__attributes__val__devicePointer,
	const uint64_t p__attributes__val__hostPointer,
	const int32_t p__attributes__val__isManaged,
	const uint32_t p__attributes__val__allocationFlags,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipPointerGetAttributesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipPointerGetAttributesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__attributes, p__attributes__val__type, p__attributes__val__device, p__attributes__val__devicePointer, p__attributes__val__hostPointer, p__attributes__val__isManaged, p__attributes__val__allocationFlags, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipPointerGetAttributesEnd`) */
void barectf_hip_api_trace_hipPointerGetAttributesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipPointerGetAttributesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipPointerGetAttributesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipPointerSetAttributeBegin`) */
void barectf_hip_api_trace_hipPointerSetAttributeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__value,
	const int32_t p__attribute,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipPointerSetAttributeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipPointerSetAttributeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__value, p__attribute, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipPointerSetAttributeEnd`) */
void barectf_hip_api_trace_hipPointerSetAttributeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipPointerSetAttributeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipPointerSetAttributeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipProfilerStartBegin`) */
void barectf_hip_api_trace_hipProfilerStartBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipProfilerStartBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipProfilerStartBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipProfilerStartEnd`) */
void barectf_hip_api_trace_hipProfilerStartEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipProfilerStartEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipProfilerStartEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipProfilerStopBegin`) */
void barectf_hip_api_trace_hipProfilerStopBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipProfilerStopBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipProfilerStopBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipProfilerStopEnd`) */
void barectf_hip_api_trace_hipProfilerStopEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipProfilerStopEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipProfilerStopEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipRuntimeGetVersionBegin`) */
void barectf_hip_api_trace_hipRuntimeGetVersionBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__runtimeVersion,
	const int32_t p__runtimeVersion__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipRuntimeGetVersionBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipRuntimeGetVersionBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__runtimeVersion, p__runtimeVersion__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipRuntimeGetVersionEnd`) */
void barectf_hip_api_trace_hipRuntimeGetVersionEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipRuntimeGetVersionEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipRuntimeGetVersionEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipSetDeviceBegin`) */
void barectf_hip_api_trace_hipSetDeviceBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const int32_t p__deviceId)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipSetDeviceBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipSetDeviceBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__deviceId);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipSetDeviceEnd`) */
void barectf_hip_api_trace_hipSetDeviceEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipSetDeviceEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipSetDeviceEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipSetDeviceFlagsBegin`) */
void barectf_hip_api_trace_hipSetDeviceFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipSetDeviceFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipSetDeviceFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipSetDeviceFlagsEnd`) */
void barectf_hip_api_trace_hipSetDeviceFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipSetDeviceFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipSetDeviceFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipSetValidDevicesBegin`) */
void barectf_hip_api_trace_hipSetValidDevicesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__device_arr,
	const int32_t p__device_arr__val,
	const int32_t p__len)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipSetValidDevicesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipSetValidDevicesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__device_arr, p__device_arr__val, p__len);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipSetValidDevicesEnd`) */
void barectf_hip_api_trace_hipSetValidDevicesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipSetValidDevicesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipSetValidDevicesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipSetupArgumentBegin`) */
void barectf_hip_api_trace_hipSetupArgumentBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__arg,
	const uint64_t p__size,
	const uint64_t p__offset)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipSetupArgumentBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipSetupArgumentBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__arg, p__size, p__offset);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipSetupArgumentEnd`) */
void barectf_hip_api_trace_hipSetupArgumentEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipSetupArgumentEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipSetupArgumentEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipSignalExternalSemaphoresAsyncBegin`) */
void barectf_hip_api_trace_hipSignalExternalSemaphoresAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__extSemArray,
	const uint64_t p__extSemArray__val,
	const uint64_t p__paramsArray,
	const uint32_t p__paramsArray__val__flags,
	const uint64_t p__paramsArray__val__reserved,
	const uint32_t p__numExtSems,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipSignalExternalSemaphoresAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipSignalExternalSemaphoresAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__extSemArray, p__extSemArray__val, p__paramsArray, p__paramsArray__val__flags, p__paramsArray__val__reserved, p__numExtSems, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipSignalExternalSemaphoresAsyncEnd`) */
void barectf_hip_api_trace_hipSignalExternalSemaphoresAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipSignalExternalSemaphoresAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipSignalExternalSemaphoresAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamAddCallbackBegin`) */
void barectf_hip_api_trace_hipStreamAddCallbackBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__callback,
	const uint64_t p__userData,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamAddCallbackBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamAddCallbackBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__callback, p__userData, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamAddCallbackEnd`) */
void barectf_hip_api_trace_hipStreamAddCallbackEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamAddCallbackEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamAddCallbackEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamAttachMemAsyncBegin`) */
void barectf_hip_api_trace_hipStreamAttachMemAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__dev_ptr,
	const uint64_t p__length,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamAttachMemAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamAttachMemAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__dev_ptr, p__length, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamAttachMemAsyncEnd`) */
void barectf_hip_api_trace_hipStreamAttachMemAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamAttachMemAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamAttachMemAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamBeginCaptureBegin`) */
void barectf_hip_api_trace_hipStreamBeginCaptureBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const int32_t p__mode)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamBeginCaptureBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamBeginCaptureBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__mode);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamBeginCaptureEnd`) */
void barectf_hip_api_trace_hipStreamBeginCaptureEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamBeginCaptureEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamBeginCaptureEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamBeginCaptureToGraphBegin`) */
void barectf_hip_api_trace_hipStreamBeginCaptureToGraphBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__graph,
	const uint64_t p__dependencies,
	const uint64_t p__dependencies__val,
	const uint64_t p__dependencyData,
	const uint8_t p__dependencyData__val__from_port,
	const uint64_t p__dependencyData__val__reserved,
	const uint8_t p__dependencyData__val__to_port,
	const uint8_t p__dependencyData__val__type,
	const uint64_t p__numDependencies,
	const int32_t p__mode)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamBeginCaptureToGraphBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamBeginCaptureToGraphBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__graph, p__dependencies, p__dependencies__val, p__dependencyData, p__dependencyData__val__from_port, p__dependencyData__val__reserved, p__dependencyData__val__to_port, p__dependencyData__val__type, p__numDependencies, p__mode);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamBeginCaptureToGraphEnd`) */
void barectf_hip_api_trace_hipStreamBeginCaptureToGraphEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamBeginCaptureToGraphEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamBeginCaptureToGraphEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamCreateBegin`) */
void barectf_hip_api_trace_hipStreamCreateBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__stream__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamCreateBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamCreateBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__stream__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamCreateEnd`) */
void barectf_hip_api_trace_hipStreamCreateEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamCreateEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamCreateEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamCreateWithFlagsBegin`) */
void barectf_hip_api_trace_hipStreamCreateWithFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__stream__val,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamCreateWithFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamCreateWithFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__stream__val, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamCreateWithFlagsEnd`) */
void barectf_hip_api_trace_hipStreamCreateWithFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamCreateWithFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamCreateWithFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamCreateWithPriorityBegin`) */
void barectf_hip_api_trace_hipStreamCreateWithPriorityBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__stream__val,
	const uint32_t p__flags,
	const int32_t p__priority)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamCreateWithPriorityBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamCreateWithPriorityBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__stream__val, p__flags, p__priority);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamCreateWithPriorityEnd`) */
void barectf_hip_api_trace_hipStreamCreateWithPriorityEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamCreateWithPriorityEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamCreateWithPriorityEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamDestroyBegin`) */
void barectf_hip_api_trace_hipStreamDestroyBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamDestroyBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamDestroyBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamDestroyEnd`) */
void barectf_hip_api_trace_hipStreamDestroyEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamDestroyEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamDestroyEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamEndCaptureBegin`) */
void barectf_hip_api_trace_hipStreamEndCaptureBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__pGraph,
	const uint64_t p__pGraph__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamEndCaptureBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamEndCaptureBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__pGraph, p__pGraph__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamEndCaptureEnd`) */
void barectf_hip_api_trace_hipStreamEndCaptureEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamEndCaptureEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamEndCaptureEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamGetCaptureInfoBegin`) */
void barectf_hip_api_trace_hipStreamGetCaptureInfoBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__pCaptureStatus,
	const int32_t p__pCaptureStatus__val,
	const uint64_t p__pId,
	const uint64_t p__pId__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamGetCaptureInfoBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamGetCaptureInfoBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__pCaptureStatus, p__pCaptureStatus__val, p__pId, p__pId__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamGetCaptureInfoEnd`) */
void barectf_hip_api_trace_hipStreamGetCaptureInfoEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamGetCaptureInfoEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamGetCaptureInfoEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamGetCaptureInfo_v2Begin`) */
void barectf_hip_api_trace_hipStreamGetCaptureInfo_v2Begin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__captureStatus_out,
	const int32_t p__captureStatus_out__val,
	const uint64_t p__id_out,
	const uint64_t p__id_out__val,
	const uint64_t p__graph_out,
	const uint64_t p__graph_out__val,
	const uint64_t p__dependencies_out,
	const uint64_t p__dependencies_out__val,
	const uint64_t p__numDependencies_out,
	const uint64_t p__numDependencies_out__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamGetCaptureInfo_v2Begin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamGetCaptureInfo_v2Begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__captureStatus_out, p__captureStatus_out__val, p__id_out, p__id_out__val, p__graph_out, p__graph_out__val, p__dependencies_out, p__dependencies_out__val, p__numDependencies_out, p__numDependencies_out__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamGetCaptureInfo_v2End`) */
void barectf_hip_api_trace_hipStreamGetCaptureInfo_v2End(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamGetCaptureInfo_v2End(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamGetCaptureInfo_v2End(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamGetDeviceBegin`) */
void barectf_hip_api_trace_hipStreamGetDeviceBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__device,
	const int32_t p__device__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamGetDeviceBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamGetDeviceBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__device, p__device__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamGetDeviceEnd`) */
void barectf_hip_api_trace_hipStreamGetDeviceEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamGetDeviceEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamGetDeviceEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamGetFlagsBegin`) */
void barectf_hip_api_trace_hipStreamGetFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__flags,
	const uint32_t p__flags__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamGetFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamGetFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__flags, p__flags__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamGetFlagsEnd`) */
void barectf_hip_api_trace_hipStreamGetFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamGetFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamGetFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamGetPriorityBegin`) */
void barectf_hip_api_trace_hipStreamGetPriorityBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__priority,
	const int32_t p__priority__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamGetPriorityBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamGetPriorityBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__priority, p__priority__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamGetPriorityEnd`) */
void barectf_hip_api_trace_hipStreamGetPriorityEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamGetPriorityEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamGetPriorityEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamIsCapturingBegin`) */
void barectf_hip_api_trace_hipStreamIsCapturingBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__pCaptureStatus,
	const int32_t p__pCaptureStatus__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamIsCapturingBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamIsCapturingBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__pCaptureStatus, p__pCaptureStatus__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamIsCapturingEnd`) */
void barectf_hip_api_trace_hipStreamIsCapturingEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamIsCapturingEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamIsCapturingEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamQueryBegin`) */
void barectf_hip_api_trace_hipStreamQueryBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamQueryBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamQueryBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamQueryEnd`) */
void barectf_hip_api_trace_hipStreamQueryEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamQueryEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamQueryEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamSynchronizeBegin`) */
void barectf_hip_api_trace_hipStreamSynchronizeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamSynchronizeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamSynchronizeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamSynchronizeEnd`) */
void barectf_hip_api_trace_hipStreamSynchronizeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamSynchronizeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamSynchronizeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamUpdateCaptureDependenciesBegin`) */
void barectf_hip_api_trace_hipStreamUpdateCaptureDependenciesBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__dependencies,
	const uint64_t p__dependencies__val,
	const uint64_t p__numDependencies,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamUpdateCaptureDependenciesBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamUpdateCaptureDependenciesBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__dependencies, p__dependencies__val, p__numDependencies, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamUpdateCaptureDependenciesEnd`) */
void barectf_hip_api_trace_hipStreamUpdateCaptureDependenciesEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamUpdateCaptureDependenciesEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamUpdateCaptureDependenciesEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamWaitEventBegin`) */
void barectf_hip_api_trace_hipStreamWaitEventBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__event,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamWaitEventBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamWaitEventBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__event, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamWaitEventEnd`) */
void barectf_hip_api_trace_hipStreamWaitEventEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamWaitEventEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamWaitEventEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamWaitValue32Begin`) */
void barectf_hip_api_trace_hipStreamWaitValue32Begin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__ptr,
	const uint32_t p__value,
	const uint32_t p__flags,
	const uint32_t p__mask)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamWaitValue32Begin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamWaitValue32Begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__ptr, p__value, p__flags, p__mask);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamWaitValue32End`) */
void barectf_hip_api_trace_hipStreamWaitValue32End(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamWaitValue32End(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamWaitValue32End(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamWaitValue64Begin`) */
void barectf_hip_api_trace_hipStreamWaitValue64Begin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__ptr,
	const uint64_t p__value,
	const uint32_t p__flags,
	const uint64_t p__mask)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamWaitValue64Begin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamWaitValue64Begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__ptr, p__value, p__flags, p__mask);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamWaitValue64End`) */
void barectf_hip_api_trace_hipStreamWaitValue64End(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamWaitValue64End(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamWaitValue64End(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamWriteValue32Begin`) */
void barectf_hip_api_trace_hipStreamWriteValue32Begin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__ptr,
	const uint32_t p__value,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamWriteValue32Begin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamWriteValue32Begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__ptr, p__value, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamWriteValue32End`) */
void barectf_hip_api_trace_hipStreamWriteValue32End(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamWriteValue32End(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamWriteValue32End(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamWriteValue64Begin`) */
void barectf_hip_api_trace_hipStreamWriteValue64Begin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__stream,
	const uint64_t p__ptr,
	const uint64_t p__value,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamWriteValue64Begin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamWriteValue64Begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__stream, p__ptr, p__value, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipStreamWriteValue64End`) */
void barectf_hip_api_trace_hipStreamWriteValue64End(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipStreamWriteValue64End(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipStreamWriteValue64End(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetAddressBegin`) */
void barectf_hip_api_trace_hipTexRefGetAddressBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__dev_ptr,
	const uint64_t p__dev_ptr__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetAddressBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetAddressBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__dev_ptr, p__dev_ptr__val, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetAddressEnd`) */
void barectf_hip_api_trace_hipTexRefGetAddressEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetAddressEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetAddressEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetArrayBegin`) */
void barectf_hip_api_trace_hipTexRefGetArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pArray,
	const uint64_t p__pArray__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pArray, p__pArray__val, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetArrayEnd`) */
void barectf_hip_api_trace_hipTexRefGetArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetBorderColorBegin`) */
void barectf_hip_api_trace_hipTexRefGetBorderColorBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pBorderColor,
	const uint64_t p__pBorderColor__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetBorderColorBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetBorderColorBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pBorderColor, p__pBorderColor__val, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetBorderColorEnd`) */
void barectf_hip_api_trace_hipTexRefGetBorderColorEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetBorderColorEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetBorderColorEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetFlagsBegin`) */
void barectf_hip_api_trace_hipTexRefGetFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pFlags,
	const uint32_t p__pFlags__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pFlags, p__pFlags__val, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetFlagsEnd`) */
void barectf_hip_api_trace_hipTexRefGetFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetFormatBegin`) */
void barectf_hip_api_trace_hipTexRefGetFormatBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pFormat,
	const int32_t p__pFormat__val,
	const uint64_t p__pNumChannels,
	const int32_t p__pNumChannels__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetFormatBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetFormatBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pFormat, p__pFormat__val, p__pNumChannels, p__pNumChannels__val, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetFormatEnd`) */
void barectf_hip_api_trace_hipTexRefGetFormatEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetFormatEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetFormatEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetMaxAnisotropyBegin`) */
void barectf_hip_api_trace_hipTexRefGetMaxAnisotropyBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pmaxAnsio,
	const int32_t p__pmaxAnsio__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetMaxAnisotropyBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetMaxAnisotropyBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pmaxAnsio, p__pmaxAnsio__val, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetMaxAnisotropyEnd`) */
void barectf_hip_api_trace_hipTexRefGetMaxAnisotropyEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetMaxAnisotropyEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetMaxAnisotropyEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetMipMappedArrayBegin`) */
void barectf_hip_api_trace_hipTexRefGetMipMappedArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pArray,
	const uint64_t p__pArray__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetMipMappedArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetMipMappedArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pArray, p__pArray__val, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetMipMappedArrayEnd`) */
void barectf_hip_api_trace_hipTexRefGetMipMappedArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetMipMappedArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetMipMappedArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetMipmapLevelBiasBegin`) */
void barectf_hip_api_trace_hipTexRefGetMipmapLevelBiasBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pbias,
	const uint64_t p__pbias__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetMipmapLevelBiasBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetMipmapLevelBiasBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pbias, p__pbias__val, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetMipmapLevelBiasEnd`) */
void barectf_hip_api_trace_hipTexRefGetMipmapLevelBiasEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetMipmapLevelBiasEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetMipmapLevelBiasEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetMipmapLevelClampBegin`) */
void barectf_hip_api_trace_hipTexRefGetMipmapLevelClampBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__pminMipmapLevelClamp,
	const uint64_t p__pminMipmapLevelClamp__val,
	const uint64_t p__pmaxMipmapLevelClamp,
	const uint64_t p__pmaxMipmapLevelClamp__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetMipmapLevelClampBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetMipmapLevelClampBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__pminMipmapLevelClamp, p__pminMipmapLevelClamp__val, p__pmaxMipmapLevelClamp, p__pmaxMipmapLevelClamp__val, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefGetMipmapLevelClampEnd`) */
void barectf_hip_api_trace_hipTexRefGetMipmapLevelClampEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefGetMipmapLevelClampEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefGetMipmapLevelClampEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetAddress2DBegin`) */
void barectf_hip_api_trace_hipTexRefSetAddress2DBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint64_t p__desc,
	const uint64_t p__desc__val__Width,
	const uint64_t p__desc__val__Height,
	const int32_t p__desc__val__Format,
	const uint32_t p__desc__val__NumChannels,
	const uint64_t p__dptr,
	const uint64_t p__Pitch)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetAddress2DBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetAddress2DBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format, p__desc, p__desc__val__Width, p__desc__val__Height, p__desc__val__Format, p__desc__val__NumChannels, p__dptr, p__Pitch);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetAddress2DEnd`) */
void barectf_hip_api_trace_hipTexRefSetAddress2DEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetAddress2DEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetAddress2DEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetAddressBegin`) */
void barectf_hip_api_trace_hipTexRefSetAddressBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__ByteOffset,
	const uint64_t p__ByteOffset__val,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint64_t p__dptr,
	const uint64_t p__bytes)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetAddressBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetAddressBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__ByteOffset, p__ByteOffset__val, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format, p__dptr, p__bytes);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetAddressEnd`) */
void barectf_hip_api_trace_hipTexRefSetAddressEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetAddressEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetAddressEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetArrayBegin`) */
void barectf_hip_api_trace_hipTexRefSetArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__tex,
	const int32_t p__tex__val__normalized,
	const int32_t p__tex__val__readMode,
	const int32_t p__tex__val__filterMode,
	const int32_t p__tex__val__channelDesc__x,
	const int32_t p__tex__val__channelDesc__y,
	const int32_t p__tex__val__channelDesc__z,
	const int32_t p__tex__val__channelDesc__w,
	const int32_t p__tex__val__channelDesc__f,
	const int32_t p__tex__val__sRGB,
	const uint32_t p__tex__val__maxAnisotropy,
	const int32_t p__tex__val__mipmapFilterMode,
	const uint64_t p__tex__val__mipmapLevelBias,
	const uint64_t p__tex__val__minMipmapLevelClamp,
	const uint64_t p__tex__val__maxMipmapLevelClamp,
	const uint64_t p__tex__val__textureObject,
	const int32_t p__tex__val__numChannels,
	const int32_t p__tex__val__format,
	const uint64_t p__array,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__tex, p__tex__val__normalized, p__tex__val__readMode, p__tex__val__filterMode, p__tex__val__channelDesc__x, p__tex__val__channelDesc__y, p__tex__val__channelDesc__z, p__tex__val__channelDesc__w, p__tex__val__channelDesc__f, p__tex__val__sRGB, p__tex__val__maxAnisotropy, p__tex__val__mipmapFilterMode, p__tex__val__mipmapLevelBias, p__tex__val__minMipmapLevelClamp, p__tex__val__maxMipmapLevelClamp, p__tex__val__textureObject, p__tex__val__numChannels, p__tex__val__format, p__array, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetArrayEnd`) */
void barectf_hip_api_trace_hipTexRefSetArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetBorderColorBegin`) */
void barectf_hip_api_trace_hipTexRefSetBorderColorBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint64_t p__pBorderColor,
	const uint64_t p__pBorderColor__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetBorderColorBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetBorderColorBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format, p__pBorderColor, p__pBorderColor__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetBorderColorEnd`) */
void barectf_hip_api_trace_hipTexRefSetBorderColorEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetBorderColorEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetBorderColorEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetFlagsBegin`) */
void barectf_hip_api_trace_hipTexRefSetFlagsBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint32_t p__Flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetFlagsBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetFlagsBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format, p__Flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetFlagsEnd`) */
void barectf_hip_api_trace_hipTexRefSetFlagsEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetFlagsEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetFlagsEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetFormatBegin`) */
void barectf_hip_api_trace_hipTexRefSetFormatBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const int32_t p__fmt,
	const int32_t p__NumPackedComponents)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetFormatBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetFormatBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format, p__fmt, p__NumPackedComponents);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetFormatEnd`) */
void barectf_hip_api_trace_hipTexRefSetFormatEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetFormatEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetFormatEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetMaxAnisotropyBegin`) */
void barectf_hip_api_trace_hipTexRefSetMaxAnisotropyBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint32_t p__maxAniso)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetMaxAnisotropyBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetMaxAnisotropyBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format, p__maxAniso);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetMaxAnisotropyEnd`) */
void barectf_hip_api_trace_hipTexRefSetMaxAnisotropyEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetMaxAnisotropyEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetMaxAnisotropyEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetMipmapLevelBiasBegin`) */
void barectf_hip_api_trace_hipTexRefSetMipmapLevelBiasBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint64_t p__bias)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetMipmapLevelBiasBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetMipmapLevelBiasBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format, p__bias);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetMipmapLevelBiasEnd`) */
void barectf_hip_api_trace_hipTexRefSetMipmapLevelBiasEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetMipmapLevelBiasEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetMipmapLevelBiasEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetMipmapLevelClampBegin`) */
void barectf_hip_api_trace_hipTexRefSetMipmapLevelClampBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint64_t p__minMipMapLevelClamp,
	const uint64_t p__maxMipMapLevelClamp)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetMipmapLevelClampBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetMipmapLevelClampBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format, p__minMipMapLevelClamp, p__maxMipMapLevelClamp);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetMipmapLevelClampEnd`) */
void barectf_hip_api_trace_hipTexRefSetMipmapLevelClampEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetMipmapLevelClampEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetMipmapLevelClampEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetMipmappedArrayBegin`) */
void barectf_hip_api_trace_hipTexRefSetMipmappedArrayBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__texRef,
	const int32_t p__texRef__val__normalized,
	const int32_t p__texRef__val__readMode,
	const int32_t p__texRef__val__filterMode,
	const int32_t p__texRef__val__channelDesc__x,
	const int32_t p__texRef__val__channelDesc__y,
	const int32_t p__texRef__val__channelDesc__z,
	const int32_t p__texRef__val__channelDesc__w,
	const int32_t p__texRef__val__channelDesc__f,
	const int32_t p__texRef__val__sRGB,
	const uint32_t p__texRef__val__maxAnisotropy,
	const int32_t p__texRef__val__mipmapFilterMode,
	const uint64_t p__texRef__val__mipmapLevelBias,
	const uint64_t p__texRef__val__minMipmapLevelClamp,
	const uint64_t p__texRef__val__maxMipmapLevelClamp,
	const uint64_t p__texRef__val__textureObject,
	const int32_t p__texRef__val__numChannels,
	const int32_t p__texRef__val__format,
	const uint64_t p__mipmappedArray,
	const uint64_t p__mipmappedArray__val__data,
	const int32_t p__mipmappedArray__val__desc__x,
	const int32_t p__mipmappedArray__val__desc__y,
	const int32_t p__mipmappedArray__val__desc__z,
	const int32_t p__mipmappedArray__val__desc__w,
	const int32_t p__mipmappedArray__val__desc__f,
	const uint32_t p__mipmappedArray__val__type,
	const uint32_t p__mipmappedArray__val__width,
	const uint32_t p__mipmappedArray__val__height,
	const uint32_t p__mipmappedArray__val__depth,
	const uint32_t p__mipmappedArray__val__min_mipmap_level,
	const uint32_t p__mipmappedArray__val__max_mipmap_level,
	const uint32_t p__mipmappedArray__val__flags,
	const int32_t p__mipmappedArray__val__format,
	const uint32_t p__mipmappedArray__val__num_channels,
	const uint32_t p__Flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetMipmappedArrayBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetMipmappedArrayBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__texRef, p__texRef__val__normalized, p__texRef__val__readMode, p__texRef__val__filterMode, p__texRef__val__channelDesc__x, p__texRef__val__channelDesc__y, p__texRef__val__channelDesc__z, p__texRef__val__channelDesc__w, p__texRef__val__channelDesc__f, p__texRef__val__sRGB, p__texRef__val__maxAnisotropy, p__texRef__val__mipmapFilterMode, p__texRef__val__mipmapLevelBias, p__texRef__val__minMipmapLevelClamp, p__texRef__val__maxMipmapLevelClamp, p__texRef__val__textureObject, p__texRef__val__numChannels, p__texRef__val__format, p__mipmappedArray, p__mipmappedArray__val__data, p__mipmappedArray__val__desc__x, p__mipmappedArray__val__desc__y, p__mipmappedArray__val__desc__z, p__mipmappedArray__val__desc__w, p__mipmappedArray__val__desc__f, p__mipmappedArray__val__type, p__mipmappedArray__val__width, p__mipmappedArray__val__height, p__mipmappedArray__val__depth, p__mipmappedArray__val__min_mipmap_level, p__mipmappedArray__val__max_mipmap_level, p__mipmappedArray__val__flags, p__mipmappedArray__val__format, p__mipmappedArray__val__num_channels, p__Flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipTexRefSetMipmappedArrayEnd`) */
void barectf_hip_api_trace_hipTexRefSetMipmappedArrayEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipTexRefSetMipmappedArrayEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipTexRefSetMipmappedArrayEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipThreadExchangeStreamCaptureModeBegin`) */
void barectf_hip_api_trace_hipThreadExchangeStreamCaptureModeBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__mode,
	const int32_t p__mode__val)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipThreadExchangeStreamCaptureModeBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipThreadExchangeStreamCaptureModeBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__mode, p__mode__val);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipThreadExchangeStreamCaptureModeEnd`) */
void barectf_hip_api_trace_hipThreadExchangeStreamCaptureModeEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipThreadExchangeStreamCaptureModeEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipThreadExchangeStreamCaptureModeEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipUserObjectCreateBegin`) */
void barectf_hip_api_trace_hipUserObjectCreateBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__object_out,
	const uint64_t p__object_out__val,
	const uint64_t p__ptr,
	const uint64_t p__destroy,
	const uint32_t p__initialRefcount,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipUserObjectCreateBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipUserObjectCreateBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__object_out, p__object_out__val, p__ptr, p__destroy, p__initialRefcount, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipUserObjectCreateEnd`) */
void barectf_hip_api_trace_hipUserObjectCreateEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipUserObjectCreateEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipUserObjectCreateEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipUserObjectReleaseBegin`) */
void barectf_hip_api_trace_hipUserObjectReleaseBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__object,
	const uint32_t p__count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipUserObjectReleaseBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipUserObjectReleaseBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__object, p__count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipUserObjectReleaseEnd`) */
void barectf_hip_api_trace_hipUserObjectReleaseEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipUserObjectReleaseEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipUserObjectReleaseEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipUserObjectRetainBegin`) */
void barectf_hip_api_trace_hipUserObjectRetainBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__object,
	const uint32_t p__count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipUserObjectRetainBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipUserObjectRetainBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__object, p__count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipUserObjectRetainEnd`) */
void barectf_hip_api_trace_hipUserObjectRetainEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipUserObjectRetainEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipUserObjectRetainEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipWaitExternalSemaphoresAsyncBegin`) */
void barectf_hip_api_trace_hipWaitExternalSemaphoresAsyncBegin(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name,
	const uint64_t p__extSemArray,
	const uint64_t p__extSemArray__val,
	const uint64_t p__paramsArray,
	const uint32_t p__paramsArray__val__flags,
	const uint64_t p__paramsArray__val__reserved,
	const uint32_t p__numExtSems,
	const uint64_t p__stream)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipWaitExternalSemaphoresAsyncBegin(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipWaitExternalSemaphoresAsyncBegin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name, p__extSemArray, p__extSemArray__val, p__paramsArray, p__paramsArray__val__flags, p__paramsArray__val__reserved, p__numExtSems, p__stream);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hip_api`, event record type `hipWaitExternalSemaphoresAsyncEnd`) */
void barectf_hip_api_trace_hipWaitExternalSemaphoresAsyncEnd(struct barectf_hip_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const cc__kernel_name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hip_api_hipWaitExternalSemaphoresAsyncEnd(_TO_VOID_PTR(ctx), cc__kernel_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hip_api_hipWaitExternalSemaphoresAsyncEnd(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, cc__kernel_name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}
/* Open packet for data stream type `hsa_api` */
void barectf_hsa_api_open_packet(
	struct barectf_hsa_api_ctx * const sctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	const int saved_in_tracing_section = ctx->in_tracing_section;

	/*
	 * This function is either called by a tracing function, or
	 * directly by the platform.
	 *
	 * If it's called by a tracing function, then
	 * `ctx->in_tracing_section` is 1, so it's safe to open
	 * the packet here (alter the packet), even if tracing was
	 * disabled in the meantime because we're already in a tracing
	 * section (which finishes at the end of the tracing function
	 * call).
	 *
	 * If it's called directly by the platform, then if tracing is
	 * disabled, we don't want to alter the packet, and return
	 * immediately.
	 */
	if (!ctx->is_tracing_enabled && !saved_in_tracing_section) {
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Do not open a packet that is already open */
	if (ctx->packet_is_open) {
		ctx->in_tracing_section = saved_in_tracing_section;
		goto end;
	}

	ctx->at = 0;

	/* Write packet header structure */
	{
		/* Align for packet header structure */
		_ALIGN(ctx->at, 8);

		/* Align for `magic` field */
		_ALIGN(ctx->at, 8);

		/* Write magic number field */
		{
			const uint32_t tmp_val = (uint32_t) 0xc1fc1fc1UL;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `stream_id` field */
		_ALIGN(ctx->at, 8);

		/* Write data stream type ID field */
		{
			const uint64_t tmp_val = (uint64_t) 2;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}

	/* Write packet context structure */
	{
		/* Align for packet context structure */
		_ALIGN(ctx->at, 8);

		/* Align for `packet_size` field */
		_ALIGN(ctx->at, 8);

		/* Write packet total size field */
		{
			const uint64_t tmp_val = (uint64_t) ctx->packet_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `content_size` field */
		_ALIGN(ctx->at, 8);

		/* Do not write `content_size` field; save its offset */
		sctx->off_pc_content_size = ctx->at;
		ctx->at += 64;
	}

	/* Save content beginning's offset */
	ctx->off_content = ctx->at;

	/* Mark current packet as open */
	ctx->packet_is_open = 1;

	/* Not tracing anymore */
	ctx->in_tracing_section = saved_in_tracing_section;

end:
	return;
}

/* Close packet for data stream type `hsa_api` */
void barectf_hsa_api_close_packet(struct barectf_hsa_api_ctx * const sctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	const int saved_in_tracing_section = ctx->in_tracing_section;

	/*
	 * This function is either called by a tracing function, or
	 * directly by the platform.
	 *
	 * If it's called by a tracing function, then
	 * `ctx->in_tracing_section` is 1, so it's safe to close
	 * the packet here (alter the packet), even if tracing was
	 * disabled in the meantime, because we're already in a tracing
	 * section (which finishes at the end of the tracing function
	 * call).
	 *
	 * If it's called directly by the platform, then if tracing is
	 * disabled, we don't want to alter the packet, and return
	 * immediately.
	 */
	if (!ctx->is_tracing_enabled && !saved_in_tracing_section) {
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Do not close a packet that is not open */
	if (!ctx->packet_is_open) {
		ctx->in_tracing_section = saved_in_tracing_section;
		goto end;
	}

	/* Save content size */
	ctx->content_size = ctx->at;

	/* Go back to `content_size` field offset */
	ctx->at = sctx->off_pc_content_size;

	/* Write `content_size` field */
	{
		const uint64_t tmp_val = (uint64_t) ctx->content_size;

		memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
		ctx->at += 64;
	}

	/* Go back to end of packet */
	ctx->at = ctx->packet_size;

	/* Mark packet as closed */
	ctx->packet_is_open = 0;

	/* Not tracing anymore */
	ctx->in_tracing_section = saved_in_tracing_section;

end:
	return;
}

static void _serialize_er_header_hsa_api(void * const vctx,
	const uint32_t ert_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	struct barectf_hsa_api_ctx * const sctx = _FROM_VOID_PTR(struct barectf_hsa_api_ctx, vctx);
	const uint64_t ts = sctx->cur_last_event_ts;

	/* Write header structure */
	{
		/* Align for header structure */
		_ALIGN(ctx->at, 8);

		/* Align for `id` field */
		_ALIGN(ctx->at, 8);

		/* Write event record type ID field */
		{
			const uint64_t tmp_val = (uint64_t) ert_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `timestamp` field */
		_ALIGN(ctx->at, 8);

		/* Write timestamp field */
		{
			const uint64_t tmp_val = (uint64_t) ts;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_common_ctx_hsa_api(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Write common context structure */
	{
		/* Align for common context structure */
		_ALIGN(ctx->at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(ctx->at, 32);

		/* Write `_thread_id` field */
		{
			const uint32_t tmp_val = (uint32_t) cc__thread_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_queue_id` field */
		_ALIGN(ctx->at, 32);

		/* Write `_queue_id` field */
		{
			const uint32_t tmp_val = (uint32_t) cc__queue_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_agent_id` field */
		_ALIGN(ctx->at, 32);

		/* Write `_agent_id` field */
		{
			const uint32_t tmp_val = (uint32_t) cc__agent_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_correlation_id` field */
		_ALIGN(ctx->at, 64);

		/* Write `_correlation_id` field */
		{
			const uint64_t tmp_val = (uint64_t) cc__correlation_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_extension_supported_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint64_t p__agent__handle,
	const uint16_t p__version_major,
	const uint16_t p__version_minor,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 0);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_extension` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extension` field */
		{
			const uint16_t tmp_val = (uint16_t) p__extension;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_version_major` field */
		_ALIGN(ctx->at, 8);

		/* Write `_version_major` field */
		{
			const uint16_t tmp_val = (uint16_t) p__version_major;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_version_minor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_version_minor` field */
		{
			const uint16_t tmp_val = (uint16_t) p__version_minor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_result` field */
		_ALIGN(ctx->at, 8);

		/* Write `_result` field */
		{
			const uint64_t tmp_val = (uint64_t) p__result;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_extension_supported_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 1);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_get_exception_policies_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const int32_t p__profile,
	const uint64_t p__mask)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 2);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_profile` field */
		_ALIGN(ctx->at, 8);

		/* Write `_profile` field */
		{
			const uint32_t tmp_val = (uint32_t) p__profile;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mask` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_get_exception_policies_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 3);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 4);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 5);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_iterate_caches_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 6);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_iterate_caches_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 7);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_iterate_isas_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 8);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_iterate_isas_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 9);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_iterate_regions_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 10);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_iterate_regions_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 11);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_major_extension_supported_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint64_t p__agent__handle,
	const uint16_t p__version_major,
	const uint64_t p__version_minor,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 12);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_extension` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extension` field */
		{
			const uint16_t tmp_val = (uint16_t) p__extension;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_version_major` field */
		_ALIGN(ctx->at, 8);

		/* Write `_version_major` field */
		{
			const uint16_t tmp_val = (uint16_t) p__version_major;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_version_minor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_version_minor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__version_minor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_result` field */
		_ALIGN(ctx->at, 8);

		/* Write `_result` field */
		{
			const uint64_t tmp_val = (uint64_t) p__result;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_agent_major_extension_supported_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 13);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_agent_iterate_memory_pools_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 14);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_agent_iterate_memory_pools_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 15);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_agent_memory_pool_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__memory_pool__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 16);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_memory_pool__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memory_pool__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memory_pool__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_agent_memory_pool_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 17);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_agent_set_async_scratch_limit_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__threshold)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 18);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_threshold` field */
		_ALIGN(ctx->at, 8);

		/* Write `_threshold` field */
		{
			const uint64_t tmp_val = (uint64_t) p__threshold;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_agent_set_async_scratch_limit_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 19);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_agents_allow_access_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint32_t p__num_agents,
	const uint64_t p__agents,
	const uint64_t p__flags,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 20);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_num_agents` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_agents` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_agents;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_agents` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agents` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agents;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_agents_allow_access_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 21);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_async_function_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__callback,
	const uint64_t p__arg)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 22);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_arg` field */
		_ALIGN(ctx->at, 8);

		/* Write `_arg` field */
		{
			const uint64_t tmp_val = (uint64_t) p__arg;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_async_function_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 23);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_coherency_get_type_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__type)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 24);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_type` field */
		{
			const uint64_t tmp_val = (uint64_t) p__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_coherency_get_type_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 25);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_coherency_set_type_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const int32_t p__type)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 26);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_coherency_set_type_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 27);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_deregister_deallocation_callback_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__callback)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 28);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_deregister_deallocation_callback_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 29);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_image_create_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image_descriptor,
	const uint64_t p__image_layout,
	const uint64_t p__image_data,
	const int32_t p__access_permission,
	const uint64_t p__image)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 30);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_descriptor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_descriptor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_descriptor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_layout` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_layout` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_layout;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_access_permission` field */
		_ALIGN(ctx->at, 8);

		/* Write `_access_permission` field */
		{
			const uint32_t tmp_val = (uint32_t) p__access_permission;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_image` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_image_create_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 31);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_interop_map_buffer_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint32_t p__num_agents,
	const uint64_t p__agents,
	const int32_t p__interop_handle,
	const uint32_t p__flags,
	const uint64_t p__size,
	const uint64_t p__ptr,
	const uint64_t p__metadata_size,
	const uint64_t p__metadata)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 32);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_num_agents` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_agents` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_agents;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_agents` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agents` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agents;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_interop_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_interop_handle` field */
		{
			const uint32_t tmp_val = (uint32_t) p__interop_handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_metadata_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_metadata_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__metadata_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_metadata` field */
		_ALIGN(ctx->at, 8);

		/* Write `_metadata` field */
		{
			const uint64_t tmp_val = (uint64_t) p__metadata;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_interop_map_buffer_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 33);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_interop_unmap_buffer_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 34);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_interop_unmap_buffer_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 35);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_ipc_memory_attach_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__handle,
	const uint64_t p__len,
	const uint32_t p__num_agents,
	const uint64_t p__mapping_agents,
	const uint64_t p__mapped_ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 36);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_len` field */
		_ALIGN(ctx->at, 8);

		/* Write `_len` field */
		{
			const uint64_t tmp_val = (uint64_t) p__len;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_num_agents` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_agents` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_agents;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mapping_agents` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mapping_agents` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mapping_agents;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_mapped_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mapped_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mapped_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_ipc_memory_attach_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 37);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_ipc_memory_create_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__len,
	const uint64_t p__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 38);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_len` field */
		_ALIGN(ctx->at, 8);

		/* Write `_len` field */
		{
			const uint64_t tmp_val = (uint64_t) p__len;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_ipc_memory_create_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 39);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_ipc_memory_detach_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__mapped_ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 40);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_mapped_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mapped_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mapped_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_ipc_memory_detach_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 41);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_ipc_signal_attach_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__handle,
	const uint64_t p__signal)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 42);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_signal` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_ipc_signal_attach_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 43);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_ipc_signal_create_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const uint64_t p__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 44);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_ipc_signal_create_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 45);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_async_copy_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__dst,
	const uint64_t p__dst_agent__handle,
	const uint64_t p__src,
	const uint64_t p__src_agent__handle,
	const uint64_t p__size,
	const uint32_t p__num_dep_signals,
	const uint64_t p__dep_signals,
	const uint64_t p__completion_signal__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 46);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst_agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src_agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_num_dep_signals` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_dep_signals` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_dep_signals;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dep_signals` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dep_signals` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dep_signals;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_completion_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_completion_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__completion_signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_async_copy_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 47);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_async_copy_on_engine_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__dst,
	const uint64_t p__dst_agent__handle,
	const uint64_t p__src,
	const uint64_t p__src_agent__handle,
	const uint64_t p__size,
	const uint32_t p__num_dep_signals,
	const uint64_t p__dep_signals,
	const uint64_t p__completion_signal__handle,
	const int32_t p__engine_id,
	const int32_t p__force_copy_on_sdma)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 48);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst_agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src_agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_num_dep_signals` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_dep_signals` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_dep_signals;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dep_signals` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dep_signals` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dep_signals;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_completion_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_completion_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__completion_signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_engine_id` field */
		_ALIGN(ctx->at, 8);

		/* Write `_engine_id` field */
		{
			const uint32_t tmp_val = (uint32_t) p__engine_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_force_copy_on_sdma` field */
		_ALIGN(ctx->at, 8);

		/* Write `_force_copy_on_sdma` field */
		{
			const uint32_t tmp_val = (uint32_t) p__force_copy_on_sdma;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_async_copy_on_engine_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 49);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_async_copy_rect_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__dst,
	const uint64_t p__dst_offset,
	const uint64_t p__src,
	const uint64_t p__src_offset,
	const uint64_t p__range,
	const uint32_t p__range__val__x,
	const uint32_t p__range__val__y,
	const uint32_t p__range__val__z,
	const uint64_t p__copy_agent__handle,
	const int32_t p__dir,
	const uint32_t p__num_dep_signals,
	const uint64_t p__dep_signals,
	const uint64_t p__completion_signal__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 50);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst_offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src_offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_range` field */
		_ALIGN(ctx->at, 8);

		/* Write `_range` field */
		{
			const uint64_t tmp_val = (uint64_t) p__range;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_range__val__x` field */
		_ALIGN(ctx->at, 8);

		/* Write `_range__val__x` field */
		{
			const uint32_t tmp_val = (uint32_t) p__range__val__x;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_range__val__y` field */
		_ALIGN(ctx->at, 8);

		/* Write `_range__val__y` field */
		{
			const uint32_t tmp_val = (uint32_t) p__range__val__y;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_range__val__z` field */
		_ALIGN(ctx->at, 8);

		/* Write `_range__val__z` field */
		{
			const uint32_t tmp_val = (uint32_t) p__range__val__z;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_copy_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_copy_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__copy_agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dir` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dir` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dir;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_num_dep_signals` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_dep_signals` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_dep_signals;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dep_signals` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dep_signals` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dep_signals;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_completion_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_completion_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__completion_signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_async_copy_rect_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 51);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_copy_engine_status_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__dst_agent__handle,
	const uint64_t p__src_agent__handle,
	const uint64_t p__engine_ids_mask)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 52);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst_agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src_agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_engine_ids_mask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_engine_ids_mask` field */
		{
			const uint64_t tmp_val = (uint64_t) p__engine_ids_mask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_copy_engine_status_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 53);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_fill_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint32_t p__value,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 54);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint32_t tmp_val = (uint32_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_fill_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 55);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_lock_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__host_ptr,
	const uint64_t p__size,
	const uint64_t p__agents,
	const int32_t p__num_agent,
	const uint64_t p__agent_ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 56);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_host_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_host_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__host_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_agents` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agents` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agents;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_num_agent` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_agent` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_agent;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_agent_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_lock_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 57);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_lock_to_pool_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__host_ptr,
	const uint64_t p__size,
	const uint64_t p__agents,
	const int32_t p__num_agent,
	const uint64_t p__pool__handle,
	const uint32_t p__flags,
	const uint64_t p__agent_ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 58);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_host_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_host_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__host_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_agents` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agents` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agents;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_num_agent` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_agent` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_agent;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_pool__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pool__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pool__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_agent_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_lock_to_pool_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 59);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_migrate_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__memory_pool__handle,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 60);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_memory_pool__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memory_pool__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memory_pool__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_migrate_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 61);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_pool_allocate_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__memory_pool__handle,
	const uint64_t p__size,
	const uint32_t p__flags,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 62);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_memory_pool__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memory_pool__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memory_pool__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_pool_allocate_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 63);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_pool_can_migrate_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__src_memory_pool__handle,
	const uint64_t p__dst_memory_pool__handle,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 64);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_src_memory_pool__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src_memory_pool__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src_memory_pool__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst_memory_pool__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst_memory_pool__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst_memory_pool__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_result` field */
		_ALIGN(ctx->at, 8);

		/* Write `_result` field */
		{
			const uint64_t tmp_val = (uint64_t) p__result;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_pool_can_migrate_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 65);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_pool_free_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 66);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_pool_free_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 67);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_pool_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__memory_pool__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 68);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_memory_pool__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memory_pool__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memory_pool__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_pool_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 69);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_unlock_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__host_ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 70);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_host_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_host_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__host_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_memory_unlock_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 71);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_pointer_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__info,
	const uint64_t p__alloc,
	const uint64_t p__num_agents_accessible,
	const uint64_t p__accessible)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 72);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_info` field */
		_ALIGN(ctx->at, 8);

		/* Write `_info` field */
		{
			const uint64_t tmp_val = (uint64_t) p__info;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_alloc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_alloc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__alloc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_num_agents_accessible` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_agents_accessible` field */
		{
			const uint64_t tmp_val = (uint64_t) p__num_agents_accessible;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_accessible` field */
		_ALIGN(ctx->at, 8);

		/* Write `_accessible` field */
		{
			const uint64_t tmp_val = (uint64_t) p__accessible;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_pointer_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 73);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_pointer_info_set_userdata_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__userdata)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 74);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_userdata` field */
		_ALIGN(ctx->at, 8);

		/* Write `_userdata` field */
		{
			const uint64_t tmp_val = (uint64_t) p__userdata;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_pointer_info_set_userdata_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 75);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_portable_close_dmabuf_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__dmabuf)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 76);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dmabuf` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dmabuf` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dmabuf;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_portable_close_dmabuf_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 77);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_portable_export_dmabuf_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__dmabuf,
	const uint64_t p__offset)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 78);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dmabuf` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dmabuf` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dmabuf;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_portable_export_dmabuf_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 79);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_profiling_async_copy_enable_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__enable)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 80);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_enable` field */
		_ALIGN(ctx->at, 8);

		/* Write `_enable` field */
		{
			const uint32_t tmp_val = (uint32_t) p__enable;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_profiling_async_copy_enable_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 81);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_profiling_convert_tick_to_system_domain_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__agent_tick,
	const uint64_t p__system_tick)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 82);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_agent_tick` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent_tick` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent_tick;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_system_tick` field */
		_ALIGN(ctx->at, 8);

		/* Write `_system_tick` field */
		{
			const uint64_t tmp_val = (uint64_t) p__system_tick;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_profiling_convert_tick_to_system_domain_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 83);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_profiling_get_async_copy_time_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const uint64_t p__time)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 84);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_time` field */
		_ALIGN(ctx->at, 8);

		/* Write `_time` field */
		{
			const uint64_t tmp_val = (uint64_t) p__time;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_profiling_get_async_copy_time_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 85);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_profiling_get_dispatch_time_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__signal__handle,
	const uint64_t p__time)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 86);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_time` field */
		_ALIGN(ctx->at, 8);

		/* Write `_time` field */
		{
			const uint64_t tmp_val = (uint64_t) p__time;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_profiling_get_dispatch_time_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 87);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_profiling_set_profiler_enabled_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const int32_t p__enable)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 88);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_enable` field */
		_ALIGN(ctx->at, 8);

		/* Write `_enable` field */
		{
			const uint32_t tmp_val = (uint32_t) p__enable;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_profiling_set_profiler_enabled_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 89);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_queue_cu_get_mask_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint32_t p__num_cu_mask_count,
	const uint64_t p__cu_mask)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 90);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_num_cu_mask_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_cu_mask_count` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_cu_mask_count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_cu_mask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cu_mask` field */
		{
			const uint64_t tmp_val = (uint64_t) p__cu_mask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_queue_cu_get_mask_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 91);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_queue_cu_set_mask_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint32_t p__num_cu_mask_count,
	const uint64_t p__cu_mask)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 92);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_num_cu_mask_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_cu_mask_count` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_cu_mask_count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_cu_mask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cu_mask` field */
		{
			const uint64_t tmp_val = (uint64_t) p__cu_mask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_queue_cu_set_mask_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 93);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_queue_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 94);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_queue_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 95);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_queue_intercept_create_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent_handle__handle,
	const uint32_t p__size,
	const uint32_t p__type,
	const uint64_t p__callback,
	const uint64_t p__data,
	const uint32_t p__private_segment_size,
	const uint32_t p__group_segment_size,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 96);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent_handle__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent_handle__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent_handle__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint32_t tmp_val = (uint32_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_private_segment_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_private_segment_size` field */
		{
			const uint32_t tmp_val = (uint32_t) p__private_segment_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_group_segment_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_group_segment_size` field */
		{
			const uint32_t tmp_val = (uint32_t) p__group_segment_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_queue_intercept_create_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 97);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_queue_intercept_register_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__callback,
	const uint64_t p__user_data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 98);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_user_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_user_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__user_data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_queue_intercept_register_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 99);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_queue_set_priority_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const int32_t p__priority)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 100);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_priority` field */
		_ALIGN(ctx->at, 8);

		/* Write `_priority` field */
		{
			const uint32_t tmp_val = (uint32_t) p__priority;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_queue_set_priority_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 101);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_register_deallocation_callback_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__callback,
	const uint64_t p__user_data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 102);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_user_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_user_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__user_data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_register_deallocation_callback_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 103);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_register_system_event_handler_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 104);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_register_system_event_handler_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 105);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_runtime_queue_create_register_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__callback,
	const uint64_t p__user_data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 106);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_user_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_user_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__user_data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_runtime_queue_create_register_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 107);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_signal_async_handler_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int32_t p__cond,
	const int64_t p__value,
	const uint64_t p__handler,
	const uint64_t p__arg)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 108);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_cond` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cond` field */
		{
			const uint32_t tmp_val = (uint32_t) p__cond;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handler` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handler` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handler;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_arg` field */
		_ALIGN(ctx->at, 8);

		/* Write `_arg` field */
		{
			const uint64_t tmp_val = (uint64_t) p__arg;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_signal_async_handler_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 109);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_signal_create_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int64_t p__initial_value,
	const uint32_t p__num_consumers,
	const uint64_t p__consumers,
	const uint64_t p__attributes,
	const uint64_t p__signal)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 110);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_initial_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_initial_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__initial_value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_num_consumers` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_consumers` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_consumers;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_consumers` field */
		_ALIGN(ctx->at, 8);

		/* Write `_consumers` field */
		{
			const uint64_t tmp_val = (uint64_t) p__consumers;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attributes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attributes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attributes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_signal` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_signal_create_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 111);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_signal_value_pointer_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const uint64_t p__value_ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 112);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value_ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_signal_value_pointer_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 113);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_signal_wait_any_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint32_t p__signal_count,
	const uint64_t p__signals,
	const uint64_t p__conds,
	const uint64_t p__values,
	const uint64_t p__timeout_hint,
	const int32_t p__wait_hint,
	const uint64_t p__satisfying_value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 114);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal_count` field */
		{
			const uint32_t tmp_val = (uint32_t) p__signal_count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_signals` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signals` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signals;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_conds` field */
		_ALIGN(ctx->at, 8);

		/* Write `_conds` field */
		{
			const uint64_t tmp_val = (uint64_t) p__conds;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_values` field */
		_ALIGN(ctx->at, 8);

		/* Write `_values` field */
		{
			const uint64_t tmp_val = (uint64_t) p__values;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_timeout_hint` field */
		_ALIGN(ctx->at, 8);

		/* Write `_timeout_hint` field */
		{
			const uint64_t tmp_val = (uint64_t) p__timeout_hint;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wait_hint` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wait_hint` field */
		{
			const uint32_t tmp_val = (uint32_t) p__wait_hint;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_satisfying_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_satisfying_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__satisfying_value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_signal_wait_any_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 115);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_spm_acquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__preferred_agent__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 116);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_preferred_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_preferred_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__preferred_agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_spm_acquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 117);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_spm_release_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__preferred_agent__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 118);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_preferred_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_preferred_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__preferred_agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_spm_release_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 119);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_spm_set_dest_buffer_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__preferred_agent__handle,
	const uint64_t p__size_in_bytes,
	const uint64_t p__timeout,
	const uint64_t p__size_copied,
	const uint64_t p__dest,
	const uint64_t p__is_data_loss)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 120);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_preferred_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_preferred_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__preferred_agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size_in_bytes` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size_in_bytes` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size_in_bytes;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_timeout` field */
		_ALIGN(ctx->at, 8);

		/* Write `_timeout` field */
		{
			const uint64_t tmp_val = (uint64_t) p__timeout;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size_copied` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size_copied` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size_copied;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dest` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dest` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dest;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_is_data_loss` field */
		_ALIGN(ctx->at, 8);

		/* Write `_is_data_loss` field */
		{
			const uint64_t tmp_val = (uint64_t) p__is_data_loss;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_spm_set_dest_buffer_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 121);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_svm_attributes_get_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__attribute_list,
	const uint64_t p__attribute_count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 122);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute_list` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute_list` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attribute_list;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attribute_count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_svm_attributes_get_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 123);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_svm_attributes_set_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__attribute_list,
	const uint64_t p__attribute_count)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 124);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute_list` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute_list` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attribute_list;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute_count` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__attribute_count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_svm_attributes_set_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 125);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_svm_prefetch_async_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__agent__handle,
	const uint32_t p__num_dep_signals,
	const uint64_t p__dep_signals,
	const uint64_t p__completion_signal__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 126);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_num_dep_signals` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_dep_signals` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_dep_signals;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_dep_signals` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dep_signals` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dep_signals;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_completion_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_completion_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__completion_signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_svm_prefetch_async_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 127);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_address_free_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 128);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_va` field */
		_ALIGN(ctx->at, 8);

		/* Write `_va` field */
		{
			const uint64_t tmp_val = (uint64_t) p__va;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_address_free_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 129);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_address_reserve_align_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__size,
	const uint64_t p__address,
	const uint64_t p__alignment,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 130);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_va` field */
		_ALIGN(ctx->at, 8);

		/* Write `_va` field */
		{
			const uint64_t tmp_val = (uint64_t) p__va;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_address` field */
		_ALIGN(ctx->at, 8);

		/* Write `_address` field */
		{
			const uint64_t tmp_val = (uint64_t) p__address;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_alignment` field */
		_ALIGN(ctx->at, 8);

		/* Write `_alignment` field */
		{
			const uint64_t tmp_val = (uint64_t) p__alignment;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_address_reserve_align_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 131);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_address_reserve_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__size,
	const uint64_t p__address,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 132);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_va` field */
		_ALIGN(ctx->at, 8);

		/* Write `_va` field */
		{
			const uint64_t tmp_val = (uint64_t) p__va;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_address` field */
		_ALIGN(ctx->at, 8);

		/* Write `_address` field */
		{
			const uint64_t tmp_val = (uint64_t) p__address;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_address_reserve_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 133);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_export_shareable_handle_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__dmabuf_fd,
	const uint64_t p__handle__handle,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 134);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dmabuf_fd` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dmabuf_fd` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dmabuf_fd;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_handle__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_export_shareable_handle_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 135);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_get_access_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__perms,
	const uint64_t p__agent_handle__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 136);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_va` field */
		_ALIGN(ctx->at, 8);

		/* Write `_va` field */
		{
			const uint64_t tmp_val = (uint64_t) p__va;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_perms` field */
		_ALIGN(ctx->at, 8);

		/* Write `_perms` field */
		{
			const uint64_t tmp_val = (uint64_t) p__perms;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_agent_handle__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent_handle__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent_handle__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_get_access_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 137);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_get_alloc_properties_from_handle_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__memory_handle__handle,
	const uint64_t p__pool,
	const uint64_t p__type)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 138);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_memory_handle__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memory_handle__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memory_handle__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_pool` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pool` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pool;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_type` field */
		{
			const uint64_t tmp_val = (uint64_t) p__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_get_alloc_properties_from_handle_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 139);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_handle_create_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__pool__handle,
	const uint64_t p__size,
	const int32_t p__type,
	const uint64_t p__flags,
	const uint64_t p__memory_handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 140);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_pool__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_pool__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__pool__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_memory_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memory_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memory_handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_handle_create_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 141);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_handle_release_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__memory_handle__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 142);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_memory_handle__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memory_handle__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memory_handle__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_handle_release_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 143);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_import_shareable_handle_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__dmabuf_fd,
	const uint64_t p__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 144);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dmabuf_fd` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dmabuf_fd` field */
		{
			const uint32_t tmp_val = (uint32_t) p__dmabuf_fd;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_import_shareable_handle_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 145);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_map_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__size,
	const uint64_t p__in_offset,
	const uint64_t p__memory_handle__handle,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 146);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_va` field */
		_ALIGN(ctx->at, 8);

		/* Write `_va` field */
		{
			const uint64_t tmp_val = (uint64_t) p__va;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_in_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_in_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__in_offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_memory_handle__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memory_handle__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memory_handle__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_flags` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flags` field */
		{
			const uint64_t tmp_val = (uint64_t) p__flags;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_map_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 147);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_retain_alloc_handle_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__memory_handle,
	const uint64_t p__addr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 148);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_memory_handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_memory_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__memory_handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_addr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_addr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__addr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_retain_alloc_handle_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 149);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_set_access_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__size,
	const uint64_t p__desc,
	const uint64_t p__desc_cnt)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 150);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_va` field */
		_ALIGN(ctx->at, 8);

		/* Write `_va` field */
		{
			const uint64_t tmp_val = (uint64_t) p__va;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc` field */
		{
			const uint64_t tmp_val = (uint64_t) p__desc;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_desc_cnt` field */
		_ALIGN(ctx->at, 8);

		/* Write `_desc_cnt` field */
		{
			const uint64_t tmp_val = (uint64_t) p__desc_cnt;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_set_access_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 151);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_unmap_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 152);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_va` field */
		_ALIGN(ctx->at, 8);

		/* Write `_va` field */
		{
			const uint64_t tmp_val = (uint64_t) p__va;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_amd_vmem_unmap_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 153);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_cache_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__cache__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 154);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_cache__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_cache__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__cache__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_cache_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 155);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_deserialize_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__serialized_code_object,
	const uint64_t p__serialized_code_object_size,
	const char * const p__options,
	const uint64_t p__code_object)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 156);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_serialized_code_object` field */
		_ALIGN(ctx->at, 8);

		/* Write `_serialized_code_object` field */
		{
			const uint64_t tmp_val = (uint64_t) p__serialized_code_object;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_serialized_code_object_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_serialized_code_object_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__serialized_code_object_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_options` field */
		_ALIGN(ctx->at, 8);

		/* Write `_options` field */
		_write_c_str(ctx, p__options);

		/* Align for `_code_object` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_deserialize_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 157);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_destroy_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 158);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_code_object__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_destroy_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 159);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 160);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_code_object__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 161);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_get_symbol_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object__handle,
	const char * const p__symbol_name,
	const uint64_t p__symbol)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 162);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_code_object__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbol_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol_name` field */
		_write_c_str(ctx, p__symbol_name);

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_get_symbol_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 163);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_get_symbol_from_name_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object__handle,
	const char * const p__module_name,
	const char * const p__symbol_name,
	const uint64_t p__symbol)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 164);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_code_object__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_module_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_module_name` field */
		_write_c_str(ctx, p__module_name);

		/* Align for `_symbol_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol_name` field */
		_write_c_str(ctx, p__symbol_name);

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_get_symbol_from_name_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 165);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_iterate_symbols_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 166);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_code_object__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_iterate_symbols_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 167);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_reader_create_from_file_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__file,
	const uint64_t p__code_object_reader)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 168);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_file` field */
		_ALIGN(ctx->at, 8);

		/* Write `_file` field */
		{
			const uint32_t tmp_val = (uint32_t) p__file;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_code_object_reader` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object_reader` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object_reader;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_reader_create_from_file_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 169);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_reader_create_from_memory_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object,
	const uint64_t p__size,
	const uint64_t p__code_object_reader)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 170);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_code_object` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_code_object_reader` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object_reader` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object_reader;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_reader_create_from_memory_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 171);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_reader_destroy_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object_reader__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 172);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_code_object_reader__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object_reader__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object_reader__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_reader_destroy_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 173);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_serialize_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object__handle,
	const uint64_t p__alloc_callback,
	const uint64_t p__callback_data__handle,
	const char * const p__options,
	const uint64_t p__serialized_code_object,
	const uint64_t p__serialized_code_object_size)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 174);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_code_object__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_alloc_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_alloc_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__alloc_callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback_data__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback_data__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback_data__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_options` field */
		_ALIGN(ctx->at, 8);

		/* Write `_options` field */
		_write_c_str(ctx, p__options);

		/* Align for `_serialized_code_object` field */
		_ALIGN(ctx->at, 8);

		/* Write `_serialized_code_object` field */
		{
			const uint64_t tmp_val = (uint64_t) p__serialized_code_object;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_serialized_code_object_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_serialized_code_object_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__serialized_code_object_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_object_serialize_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 175);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_symbol_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_symbol__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 176);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_code_symbol__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_symbol__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_symbol__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_code_symbol_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 177);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_agent_global_variable_define_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__agent__handle,
	const char * const p__variable_name,
	const uint64_t p__address)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 178);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_variable_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_variable_name` field */
		_write_c_str(ctx, p__variable_name);

		/* Align for `_address` field */
		_ALIGN(ctx->at, 8);

		/* Write `_address` field */
		{
			const uint64_t tmp_val = (uint64_t) p__address;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_agent_global_variable_define_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 179);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_create_alt_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__profile,
	const int32_t p__default_float_rounding_mode,
	const char * const p__options,
	const uint64_t p__executable)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 180);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_profile` field */
		_ALIGN(ctx->at, 8);

		/* Write `_profile` field */
		{
			const uint32_t tmp_val = (uint32_t) p__profile;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_default_float_rounding_mode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_default_float_rounding_mode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__default_float_rounding_mode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_options` field */
		_ALIGN(ctx->at, 8);

		/* Write `_options` field */
		_write_c_str(ctx, p__options);

		/* Align for `_executable` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_create_alt_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 181);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_create_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__profile,
	const int32_t p__executable_state,
	const char * const p__options,
	const uint64_t p__executable)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 182);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_profile` field */
		_ALIGN(ctx->at, 8);

		/* Write `_profile` field */
		{
			const uint32_t tmp_val = (uint32_t) p__profile;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_executable_state` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable_state` field */
		{
			const uint32_t tmp_val = (uint32_t) p__executable_state;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_options` field */
		_ALIGN(ctx->at, 8);

		/* Write `_options` field */
		_write_c_str(ctx, p__options);

		/* Align for `_executable` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_create_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 183);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_destroy_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 184);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_destroy_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 185);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_freeze_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const char * const p__options)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 186);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_options` field */
		_ALIGN(ctx->at, 8);

		/* Write `_options` field */
		_write_c_str(ctx, p__options);
	}
}

static void _serialize_er_hsa_api_hsa_executable_freeze_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 187);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 188);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 189);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_get_symbol_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const char * const p__module_name,
	const char * const p__symbol_name,
	const uint64_t p__agent__handle,
	const int32_t p__call_convention,
	const uint64_t p__symbol)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 190);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_module_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_module_name` field */
		_write_c_str(ctx, p__module_name);

		/* Align for `_symbol_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol_name` field */
		_write_c_str(ctx, p__symbol_name);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_call_convention` field */
		_ALIGN(ctx->at, 8);

		/* Write `_call_convention` field */
		{
			const uint32_t tmp_val = (uint32_t) p__call_convention;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_get_symbol_by_name_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const char * const p__symbol_name,
	const uint64_t p__agent,
	const uint64_t p__symbol)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 191);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbol_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol_name` field */
		_write_c_str(ctx, p__symbol_name);

		/* Align for `_agent` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_symbol` field */
		_ALIGN(ctx->at, 8);

		/* Write `_symbol` field */
		{
			const uint64_t tmp_val = (uint64_t) p__symbol;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_get_symbol_by_name_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 192);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_get_symbol_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 193);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_global_variable_define_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const char * const p__variable_name,
	const uint64_t p__address)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 194);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_variable_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_variable_name` field */
		_write_c_str(ctx, p__variable_name);

		/* Align for `_address` field */
		_ALIGN(ctx->at, 8);

		/* Write `_address` field */
		{
			const uint64_t tmp_val = (uint64_t) p__address;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_global_variable_define_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 195);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_iterate_agent_symbols_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__agent__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 196);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_iterate_agent_symbols_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 197);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_iterate_program_symbols_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 198);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_iterate_program_symbols_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 199);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_iterate_symbols_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 200);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_iterate_symbols_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 201);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_load_agent_code_object_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__agent__handle,
	const uint64_t p__code_object_reader__handle,
	const char * const p__options,
	const uint64_t p__loaded_code_object)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 202);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_code_object_reader__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object_reader__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object_reader__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_options` field */
		_ALIGN(ctx->at, 8);

		/* Write `_options` field */
		_write_c_str(ctx, p__options);

		/* Align for `_loaded_code_object` field */
		_ALIGN(ctx->at, 8);

		/* Write `_loaded_code_object` field */
		{
			const uint64_t tmp_val = (uint64_t) p__loaded_code_object;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_load_agent_code_object_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 203);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_load_code_object_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__agent__handle,
	const uint64_t p__code_object__handle,
	const char * const p__options)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 204);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_code_object__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_options` field */
		_ALIGN(ctx->at, 8);

		/* Write `_options` field */
		_write_c_str(ctx, p__options);
	}
}

static void _serialize_er_hsa_api_hsa_executable_load_code_object_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 205);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_load_program_code_object_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__code_object_reader__handle,
	const char * const p__options,
	const uint64_t p__loaded_code_object)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 206);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_code_object_reader__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object_reader__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object_reader__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_options` field */
		_ALIGN(ctx->at, 8);

		/* Write `_options` field */
		_write_c_str(ctx, p__options);

		/* Align for `_loaded_code_object` field */
		_ALIGN(ctx->at, 8);

		/* Write `_loaded_code_object` field */
		{
			const uint64_t tmp_val = (uint64_t) p__loaded_code_object;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_load_program_code_object_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 207);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_readonly_variable_define_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__agent__handle,
	const char * const p__variable_name,
	const uint64_t p__address)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 208);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_variable_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_variable_name` field */
		_write_c_str(ctx, p__variable_name);

		/* Align for `_address` field */
		_ALIGN(ctx->at, 8);

		/* Write `_address` field */
		{
			const uint64_t tmp_val = (uint64_t) p__address;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_readonly_variable_define_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 209);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_symbol_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable_symbol__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 210);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable_symbol__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable_symbol__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable_symbol__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_symbol_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 211);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_validate_alt_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const char * const p__options,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 212);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_options` field */
		_ALIGN(ctx->at, 8);

		/* Write `_options` field */
		_write_c_str(ctx, p__options);

		/* Align for `_result` field */
		_ALIGN(ctx->at, 8);

		/* Write `_result` field */
		{
			const uint64_t tmp_val = (uint64_t) p__result;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_validate_alt_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 213);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_validate_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 214);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_executable__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__executable__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_result` field */
		_ALIGN(ctx->at, 8);

		/* Write `_result` field */
		{
			const uint64_t tmp_val = (uint64_t) p__result;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_executable_validate_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 215);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_clear_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image__handle,
	const uint64_t p__data,
	const uint64_t p__image_region)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 216);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_region` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_region` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_region;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_clear_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 217);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_copy_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__src_image__handle,
	const uint64_t p__src_offset,
	const uint64_t p__dst_image__handle,
	const uint64_t p__dst_offset,
	const uint64_t p__range)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 218);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src_image__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src_image__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src_image__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src_offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst_image__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst_image__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst_image__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst_offset` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst_offset` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst_offset;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_range` field */
		_ALIGN(ctx->at, 8);

		/* Write `_range` field */
		{
			const uint64_t tmp_val = (uint64_t) p__range;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_copy_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 219);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_create_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image_descriptor,
	const uint64_t p__image_data,
	const int32_t p__access_permission,
	const uint64_t p__image)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 220);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_descriptor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_descriptor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_descriptor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_access_permission` field */
		_ALIGN(ctx->at, 8);

		/* Write `_access_permission` field */
		{
			const uint32_t tmp_val = (uint32_t) p__access_permission;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_image` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_create_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 221);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_create_with_layout_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image_descriptor,
	const uint64_t p__image_data,
	const int32_t p__access_permission,
	const int32_t p__image_data_layout,
	const uint64_t p__image_data_row_pitch,
	const uint64_t p__image_data_slice_pitch,
	const uint64_t p__image)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 222);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_descriptor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_descriptor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_descriptor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_access_permission` field */
		_ALIGN(ctx->at, 8);

		/* Write `_access_permission` field */
		{
			const uint32_t tmp_val = (uint32_t) p__access_permission;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_image_data_layout` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_data_layout` field */
		{
			const uint32_t tmp_val = (uint32_t) p__image_data_layout;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_image_data_row_pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_data_row_pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_data_row_pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_data_slice_pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_data_slice_pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_data_slice_pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_create_with_layout_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 223);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_data_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image_descriptor,
	const int32_t p__access_permission,
	const uint64_t p__image_data_info)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 224);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_descriptor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_descriptor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_descriptor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_access_permission` field */
		_ALIGN(ctx->at, 8);

		/* Write `_access_permission` field */
		{
			const uint32_t tmp_val = (uint32_t) p__access_permission;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_image_data_info` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_data_info` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_data_info;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_data_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 225);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_data_get_info_with_layout_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image_descriptor,
	const int32_t p__access_permission,
	const int32_t p__image_data_layout,
	const uint64_t p__image_data_row_pitch,
	const uint64_t p__image_data_slice_pitch,
	const uint64_t p__image_data_info)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 226);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_descriptor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_descriptor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_descriptor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_access_permission` field */
		_ALIGN(ctx->at, 8);

		/* Write `_access_permission` field */
		{
			const uint32_t tmp_val = (uint32_t) p__access_permission;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_image_data_layout` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_data_layout` field */
		{
			const uint32_t tmp_val = (uint32_t) p__image_data_layout;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_image_data_row_pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_data_row_pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_data_row_pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_data_slice_pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_data_slice_pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_data_slice_pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_data_info` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_data_info` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_data_info;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_data_get_info_with_layout_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 227);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_destroy_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 228);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_destroy_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 229);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_export_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__src_image__handle,
	const uint64_t p__dst_memory,
	const uint64_t p__dst_row_pitch,
	const uint64_t p__dst_slice_pitch,
	const uint64_t p__image_region)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 230);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src_image__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src_image__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src_image__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst_memory` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst_memory` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst_memory;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst_row_pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst_row_pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst_row_pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst_slice_pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst_slice_pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst_slice_pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_region` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_region` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_region;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_export_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 231);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_get_capability_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const int32_t p__geometry,
	const uint64_t p__image_format,
	const uint64_t p__capability_mask)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 232);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_geometry` field */
		_ALIGN(ctx->at, 8);

		/* Write `_geometry` field */
		{
			const uint32_t tmp_val = (uint32_t) p__geometry;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_image_format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_format` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_capability_mask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_capability_mask` field */
		{
			const uint64_t tmp_val = (uint64_t) p__capability_mask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_get_capability_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 233);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_get_capability_with_layout_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const int32_t p__geometry,
	const uint64_t p__image_format,
	const int32_t p__image_data_layout,
	const uint64_t p__capability_mask)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 234);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_geometry` field */
		_ALIGN(ctx->at, 8);

		/* Write `_geometry` field */
		{
			const uint32_t tmp_val = (uint32_t) p__geometry;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_image_format` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_format` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_format;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_data_layout` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_data_layout` field */
		{
			const uint32_t tmp_val = (uint32_t) p__image_data_layout;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_capability_mask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_capability_mask` field */
		{
			const uint64_t tmp_val = (uint64_t) p__capability_mask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_get_capability_with_layout_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 235);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_import_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__src_memory,
	const uint64_t p__src_row_pitch,
	const uint64_t p__src_slice_pitch,
	const uint64_t p__dst_image__handle,
	const uint64_t p__image_region)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 236);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src_memory` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src_memory` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src_memory;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src_row_pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src_row_pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src_row_pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src_slice_pitch` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src_slice_pitch` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src_slice_pitch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_dst_image__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst_image__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst_image__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_image_region` field */
		_ALIGN(ctx->at, 8);

		/* Write `_image_region` field */
		{
			const uint64_t tmp_val = (uint64_t) p__image_region;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_image_import_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 237);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_sampler_create_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__sampler_descriptor,
	const uint64_t p__sampler)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 238);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sampler_descriptor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sampler_descriptor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sampler_descriptor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sampler` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sampler` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sampler;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_sampler_create_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 239);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_sampler_destroy_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__sampler__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 240);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sampler__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_sampler__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sampler__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_ext_sampler_destroy_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 241);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_extension_get_name_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint64_t p__name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 242);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_extension` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extension` field */
		{
			const uint16_t tmp_val = (uint16_t) p__extension;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_name` field */
		{
			const uint64_t tmp_val = (uint64_t) p__name;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_extension_get_name_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 243);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_init_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 244);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_init_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 245);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_compatible_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object_isa__handle,
	const uint64_t p__agent_isa__handle,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 246);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_code_object_isa__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_code_object_isa__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__code_object_isa__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_agent_isa__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent_isa__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent_isa__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_result` field */
		_ALIGN(ctx->at, 8);

		/* Write `_result` field */
		{
			const uint64_t tmp_val = (uint64_t) p__result;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_compatible_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 247);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_from_name_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const p__name,
	const uint64_t p__isa)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 248);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_name` field */
		_write_c_str(ctx, p__name);

		/* Align for `_isa` field */
		_ALIGN(ctx->at, 8);

		/* Write `_isa` field */
		{
			const uint64_t tmp_val = (uint64_t) p__isa;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_from_name_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 249);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_get_exception_policies_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__isa__handle,
	const int32_t p__profile,
	const uint64_t p__mask)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 250);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_isa__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_isa__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__isa__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_profile` field */
		_ALIGN(ctx->at, 8);

		/* Write `_profile` field */
		{
			const uint32_t tmp_val = (uint32_t) p__profile;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_mask` field */
		_ALIGN(ctx->at, 8);

		/* Write `_mask` field */
		{
			const uint64_t tmp_val = (uint64_t) p__mask;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_get_exception_policies_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 251);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_get_info_alt_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__isa__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 252);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_isa__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_isa__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__isa__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_get_info_alt_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 253);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__isa__handle,
	const int32_t p__attribute,
	const uint32_t p__index,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 254);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_isa__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_isa__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__isa__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_index` field */
		_ALIGN(ctx->at, 8);

		/* Write `_index` field */
		{
			const uint32_t tmp_val = (uint32_t) p__index;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 255);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_get_round_method_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__isa__handle,
	const int32_t p__fp_type,
	const int32_t p__flush_mode,
	const uint64_t p__round_method)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 256);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_isa__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_isa__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__isa__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_fp_type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_fp_type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__fp_type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_flush_mode` field */
		_ALIGN(ctx->at, 8);

		/* Write `_flush_mode` field */
		{
			const uint32_t tmp_val = (uint32_t) p__flush_mode;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_round_method` field */
		_ALIGN(ctx->at, 8);

		/* Write `_round_method` field */
		{
			const uint64_t tmp_val = (uint64_t) p__round_method;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_get_round_method_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 257);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_iterate_wavefronts_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__isa__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 258);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_isa__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_isa__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__isa__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_isa_iterate_wavefronts_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 259);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_iterate_agents_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 260);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_iterate_agents_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 261);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_memory_allocate_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__region__handle,
	const uint64_t p__size,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 262);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_region__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_region__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__region__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_memory_allocate_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 263);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_memory_assign_agent_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__agent__handle,
	const int32_t p__access)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 264);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_access` field */
		_ALIGN(ctx->at, 8);

		/* Write `_access` field */
		{
			const uint32_t tmp_val = (uint32_t) p__access;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_memory_assign_agent_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 265);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_memory_copy_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 266);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_dst` field */
		_ALIGN(ctx->at, 8);

		/* Write `_dst` field */
		{
			const uint64_t tmp_val = (uint64_t) p__dst;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_src` field */
		_ALIGN(ctx->at, 8);

		/* Write `_src` field */
		{
			const uint64_t tmp_val = (uint64_t) p__src;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_memory_copy_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 267);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_memory_deregister_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 268);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_memory_deregister_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 269);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_memory_free_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 270);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_memory_free_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 271);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_memory_register_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 272);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_ptr` field */
		_ALIGN(ctx->at, 8);

		/* Write `_ptr` field */
		{
			const uint64_t tmp_val = (uint64_t) p__ptr;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_memory_register_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 273);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_add_write_index_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 274);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_add_write_index_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 275);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_add_write_index_scacq_screl_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 276);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_add_write_index_scacq_screl_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 277);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_add_write_index_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 278);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_add_write_index_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 279);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_add_write_index_screlease_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 280);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_add_write_index_screlease_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 281);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_cas_write_index_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__expected,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 282);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_expected` field */
		_ALIGN(ctx->at, 8);

		/* Write `_expected` field */
		{
			const uint64_t tmp_val = (uint64_t) p__expected;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_cas_write_index_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 283);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_cas_write_index_scacq_screl_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__expected,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 284);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_expected` field */
		_ALIGN(ctx->at, 8);

		/* Write `_expected` field */
		{
			const uint64_t tmp_val = (uint64_t) p__expected;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_cas_write_index_scacq_screl_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 285);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_cas_write_index_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__expected,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 286);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_expected` field */
		_ALIGN(ctx->at, 8);

		/* Write `_expected` field */
		{
			const uint64_t tmp_val = (uint64_t) p__expected;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_cas_write_index_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 287);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_cas_write_index_screlease_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__expected,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 288);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_expected` field */
		_ALIGN(ctx->at, 8);

		/* Write `_expected` field */
		{
			const uint64_t tmp_val = (uint64_t) p__expected;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_cas_write_index_screlease_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 289);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_create_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint32_t p__size,
	const uint32_t p__type,
	const uint64_t p__callback,
	const uint64_t p__data,
	const uint32_t p__private_segment_size,
	const uint32_t p__group_segment_size,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 290);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_agent__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__agent__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint32_t tmp_val = (uint32_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_callback` field */
		_ALIGN(ctx->at, 8);

		/* Write `_callback` field */
		{
			const uint64_t tmp_val = (uint64_t) p__callback;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_data` field */
		_ALIGN(ctx->at, 8);

		/* Write `_data` field */
		{
			const uint64_t tmp_val = (uint64_t) p__data;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_private_segment_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_private_segment_size` field */
		{
			const uint32_t tmp_val = (uint32_t) p__private_segment_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_group_segment_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_group_segment_size` field */
		{
			const uint32_t tmp_val = (uint32_t) p__group_segment_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_create_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 291);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_destroy_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 292);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_destroy_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 293);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_inactivate_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 294);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_inactivate_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 295);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_load_read_index_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 296);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_load_read_index_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 297);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_load_read_index_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 298);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_load_read_index_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 299);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_load_write_index_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 300);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_load_write_index_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 301);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_load_write_index_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 302);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_load_write_index_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 303);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_store_read_index_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 304);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_store_read_index_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 305);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_queue_store_read_index_screlease_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 306);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_store_read_index_screlease_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 307);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_queue_store_write_index_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 308);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_store_write_index_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 309);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_queue_store_write_index_screlease_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 310);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_queue_store_write_index_screlease_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 311);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_region_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__region__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 312);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_region__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_region__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__region__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_region_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 313);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_shut_down_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 314);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_shut_down_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 315);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_add_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 316);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_add_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 317);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_add_scacq_screl_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 318);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_add_scacq_screl_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 319);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_add_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 320);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_add_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 321);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_add_screlease_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 322);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_add_screlease_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 323);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_and_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 324);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_and_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 325);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_and_scacq_screl_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 326);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_and_scacq_screl_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 327);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_and_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 328);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_and_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 329);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_and_screlease_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 330);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_and_screlease_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 331);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_cas_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__expected,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 332);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_expected` field */
		_ALIGN(ctx->at, 8);

		/* Write `_expected` field */
		{
			const uint64_t tmp_val = (uint64_t) p__expected;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_cas_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 333);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_cas_scacq_screl_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__expected,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 334);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_expected` field */
		_ALIGN(ctx->at, 8);

		/* Write `_expected` field */
		{
			const uint64_t tmp_val = (uint64_t) p__expected;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_cas_scacq_screl_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 335);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_cas_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__expected,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 336);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_expected` field */
		_ALIGN(ctx->at, 8);

		/* Write `_expected` field */
		{
			const uint64_t tmp_val = (uint64_t) p__expected;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_cas_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 337);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_cas_screlease_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__expected,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 338);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_expected` field */
		_ALIGN(ctx->at, 8);

		/* Write `_expected` field */
		{
			const uint64_t tmp_val = (uint64_t) p__expected;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_cas_screlease_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 339);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_create_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int64_t p__initial_value,
	const uint32_t p__num_consumers,
	const uint64_t p__consumers,
	const uint64_t p__signal)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 340);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_initial_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_initial_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__initial_value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_num_consumers` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_consumers` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_consumers;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_consumers` field */
		_ALIGN(ctx->at, 8);

		/* Write `_consumers` field */
		{
			const uint64_t tmp_val = (uint64_t) p__consumers;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_signal` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_create_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 341);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_destroy_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 342);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_destroy_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 343);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_exchange_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 344);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_exchange_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 345);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_exchange_scacq_screl_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 346);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_exchange_scacq_screl_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 347);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_exchange_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 348);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_exchange_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 349);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_exchange_screlease_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 350);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_exchange_screlease_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 351);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_group_create_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint32_t p__num_signals,
	const uint64_t p__signals,
	const uint32_t p__num_consumers,
	const uint64_t p__consumers,
	const uint64_t p__signal_group)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 352);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_num_signals` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_signals` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_signals;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_signals` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signals` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signals;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_num_consumers` field */
		_ALIGN(ctx->at, 8);

		/* Write `_num_consumers` field */
		{
			const uint32_t tmp_val = (uint32_t) p__num_consumers;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_consumers` field */
		_ALIGN(ctx->at, 8);

		/* Write `_consumers` field */
		{
			const uint64_t tmp_val = (uint64_t) p__consumers;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_signal_group` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal_group` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal_group;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_group_create_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 353);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_group_destroy_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal_group__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 354);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal_group__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal_group__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal_group__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_group_destroy_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 355);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_group_wait_any_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal_group__handle,
	const uint64_t p__conditions,
	const uint64_t p__compare_values,
	const int32_t p__wait_state_hint,
	const uint64_t p__signal,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 356);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal_group__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal_group__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal_group__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_conditions` field */
		_ALIGN(ctx->at, 8);

		/* Write `_conditions` field */
		{
			const uint64_t tmp_val = (uint64_t) p__conditions;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_compare_values` field */
		_ALIGN(ctx->at, 8);

		/* Write `_compare_values` field */
		{
			const uint64_t tmp_val = (uint64_t) p__compare_values;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wait_state_hint` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wait_state_hint` field */
		{
			const uint32_t tmp_val = (uint32_t) p__wait_state_hint;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_signal` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_group_wait_any_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 357);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_group_wait_any_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal_group__handle,
	const uint64_t p__conditions,
	const uint64_t p__compare_values,
	const int32_t p__wait_state_hint,
	const uint64_t p__signal,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 358);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal_group__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal_group__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal_group__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_conditions` field */
		_ALIGN(ctx->at, 8);

		/* Write `_conditions` field */
		{
			const uint64_t tmp_val = (uint64_t) p__conditions;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_compare_values` field */
		_ALIGN(ctx->at, 8);

		/* Write `_compare_values` field */
		{
			const uint64_t tmp_val = (uint64_t) p__compare_values;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wait_state_hint` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wait_state_hint` field */
		{
			const uint32_t tmp_val = (uint32_t) p__wait_state_hint;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_signal` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_group_wait_any_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 359);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_load_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 360);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_load_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 361);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_load_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 362);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_load_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 363);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_or_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 364);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_or_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 365);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_or_scacq_screl_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 366);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_or_scacq_screl_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 367);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_or_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 368);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_or_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 369);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_or_screlease_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 370);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_or_screlease_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 371);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_silent_store_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 372);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_silent_store_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 373);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_silent_store_screlease_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 374);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_silent_store_screlease_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 375);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_store_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 376);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_store_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 377);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_store_screlease_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 378);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_store_screlease_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 379);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_subtract_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 380);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_subtract_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 381);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_subtract_scacq_screl_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 382);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_subtract_scacq_screl_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 383);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_subtract_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 384);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_subtract_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 385);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_subtract_screlease_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 386);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_subtract_screlease_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 387);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_wait_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int32_t p__condition,
	const int64_t p__compare_value,
	const uint64_t p__timeout_hint,
	const int32_t p__wait_state_hint)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 388);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_condition` field */
		_ALIGN(ctx->at, 8);

		/* Write `_condition` field */
		{
			const uint32_t tmp_val = (uint32_t) p__condition;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_compare_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_compare_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__compare_value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_timeout_hint` field */
		_ALIGN(ctx->at, 8);

		/* Write `_timeout_hint` field */
		{
			const uint64_t tmp_val = (uint64_t) p__timeout_hint;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wait_state_hint` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wait_state_hint` field */
		{
			const uint32_t tmp_val = (uint32_t) p__wait_state_hint;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_wait_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 389);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_wait_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int32_t p__condition,
	const int64_t p__compare_value,
	const uint64_t p__timeout_hint,
	const int32_t p__wait_state_hint)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 390);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_condition` field */
		_ALIGN(ctx->at, 8);

		/* Write `_condition` field */
		{
			const uint32_t tmp_val = (uint32_t) p__condition;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_compare_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_compare_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__compare_value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_timeout_hint` field */
		_ALIGN(ctx->at, 8);

		/* Write `_timeout_hint` field */
		{
			const uint64_t tmp_val = (uint64_t) p__timeout_hint;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wait_state_hint` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wait_state_hint` field */
		{
			const uint32_t tmp_val = (uint32_t) p__wait_state_hint;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_wait_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 391);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_xor_relaxed_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 392);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_xor_relaxed_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 393);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_xor_scacq_screl_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 394);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_xor_scacq_screl_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 395);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_xor_scacquire_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 396);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_xor_scacquire_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 397);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_signal_xor_screlease_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 398);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_signal_xor_screlease_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 399);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_hsa_api_hsa_soft_queue_create_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__region__handle,
	const uint32_t p__size,
	const uint32_t p__type,
	const uint32_t p__features,
	const uint64_t p__doorbell_signal__handle,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 400);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_region__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_region__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__region__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_size` field */
		_ALIGN(ctx->at, 8);

		/* Write `_size` field */
		{
			const uint32_t tmp_val = (uint32_t) p__size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_type` field */
		{
			const uint32_t tmp_val = (uint32_t) p__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_features` field */
		_ALIGN(ctx->at, 8);

		/* Write `_features` field */
		{
			const uint32_t tmp_val = (uint32_t) p__features;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_doorbell_signal__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_doorbell_signal__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__doorbell_signal__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_queue` field */
		_ALIGN(ctx->at, 8);

		/* Write `_queue` field */
		{
			const uint64_t tmp_val = (uint64_t) p__queue;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_soft_queue_create_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 401);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_status_string_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__status,
	const uint64_t p__status_string)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 402);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_status` field */
		_ALIGN(ctx->at, 8);

		/* Write `_status` field */
		{
			const uint32_t tmp_val = (uint32_t) p__status;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_status_string` field */
		_ALIGN(ctx->at, 8);

		/* Write `_status_string` field */
		{
			const uint64_t tmp_val = (uint64_t) p__status_string;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_status_string_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 403);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_system_extension_supported_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint16_t p__version_major,
	const uint16_t p__version_minor,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 404);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_extension` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extension` field */
		{
			const uint16_t tmp_val = (uint16_t) p__extension;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_version_major` field */
		_ALIGN(ctx->at, 8);

		/* Write `_version_major` field */
		{
			const uint16_t tmp_val = (uint16_t) p__version_major;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_version_minor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_version_minor` field */
		{
			const uint16_t tmp_val = (uint16_t) p__version_minor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_result` field */
		_ALIGN(ctx->at, 8);

		/* Write `_result` field */
		{
			const uint64_t tmp_val = (uint64_t) p__result;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_system_extension_supported_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 405);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_system_get_extension_table_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint16_t p__version_major,
	const uint16_t p__version_minor,
	const uint64_t p__table)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 406);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_extension` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extension` field */
		{
			const uint16_t tmp_val = (uint16_t) p__extension;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_version_major` field */
		_ALIGN(ctx->at, 8);

		/* Write `_version_major` field */
		{
			const uint16_t tmp_val = (uint16_t) p__version_major;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_version_minor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_version_minor` field */
		{
			const uint16_t tmp_val = (uint16_t) p__version_minor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_table` field */
		_ALIGN(ctx->at, 8);

		/* Write `_table` field */
		{
			const uint64_t tmp_val = (uint64_t) p__table;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_system_get_extension_table_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 407);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_system_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 408);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_system_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 409);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_system_get_major_extension_table_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint16_t p__version_major,
	const uint64_t p__table_length,
	const uint64_t p__table)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 410);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_extension` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extension` field */
		{
			const uint16_t tmp_val = (uint16_t) p__extension;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_version_major` field */
		_ALIGN(ctx->at, 8);

		/* Write `_version_major` field */
		{
			const uint16_t tmp_val = (uint16_t) p__version_major;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_table_length` field */
		_ALIGN(ctx->at, 8);

		/* Write `_table_length` field */
		{
			const uint64_t tmp_val = (uint64_t) p__table_length;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_table` field */
		_ALIGN(ctx->at, 8);

		/* Write `_table` field */
		{
			const uint64_t tmp_val = (uint64_t) p__table;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_system_get_major_extension_table_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 411);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_system_major_extension_supported_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint16_t p__version_major,
	const uint64_t p__version_minor,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 412);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_extension` field */
		_ALIGN(ctx->at, 8);

		/* Write `_extension` field */
		{
			const uint16_t tmp_val = (uint16_t) p__extension;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_version_major` field */
		_ALIGN(ctx->at, 8);

		/* Write `_version_major` field */
		{
			const uint16_t tmp_val = (uint16_t) p__version_major;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 16;
		}

		/* Align for `_version_minor` field */
		_ALIGN(ctx->at, 8);

		/* Write `_version_minor` field */
		{
			const uint64_t tmp_val = (uint64_t) p__version_minor;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_result` field */
		_ALIGN(ctx->at, 8);

		/* Write `_result` field */
		{
			const uint64_t tmp_val = (uint64_t) p__result;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_system_major_extension_supported_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 413);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_hsa_api_hsa_wavefront_get_info_begin(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__wavefront__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 414);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_wavefront__handle` field */
		_ALIGN(ctx->at, 8);

		/* Write `_wavefront__handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__wavefront__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_attribute` field */
		_ALIGN(ctx->at, 8);

		/* Write `_attribute` field */
		{
			const uint32_t tmp_val = (uint32_t) p__attribute;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_value` field */
		_ALIGN(ctx->at, 8);

		/* Write `_value` field */
		{
			const uint64_t tmp_val = (uint64_t) p__value;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_api_hsa_wavefront_get_info_end(void * const vctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_api(ctx, 415);

	/* Serialize common context */
	_serialize_er_common_ctx_hsa_api(ctx, cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 8);

		/* Align for `_retval` field */
		_ALIGN(ctx->at, 8);

		/* Write `_retval` field */
		{
			const uint32_t tmp_val = (uint32_t) p__retval;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static uint32_t _er_size_hsa_api_hsa_agent_extension_supported_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_extension` field */
		_ALIGN(at, 8);

		/* Add `_extension` bit array field's size */
		at += 16;

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_version_major` field */
		_ALIGN(at, 8);

		/* Add `_version_major` bit array field's size */
		at += 16;

		/* Align for `_version_minor` field */
		_ALIGN(at, 8);

		/* Add `_version_minor` bit array field's size */
		at += 16;

		/* Align for `_result` field */
		_ALIGN(at, 8);

		/* Add `_result` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_agent_extension_supported_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_agent_get_exception_policies_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_profile` field */
		_ALIGN(at, 8);

		/* Add `_profile` bit array field's size */
		at += 32;

		/* Align for `_mask` field */
		_ALIGN(at, 8);

		/* Add `_mask` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_agent_get_exception_policies_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_agent_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_agent_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_agent_iterate_caches_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_agent_iterate_caches_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_agent_iterate_isas_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_agent_iterate_isas_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_agent_iterate_regions_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_agent_iterate_regions_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_agent_major_extension_supported_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_extension` field */
		_ALIGN(at, 8);

		/* Add `_extension` bit array field's size */
		at += 16;

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_version_major` field */
		_ALIGN(at, 8);

		/* Add `_version_major` bit array field's size */
		at += 16;

		/* Align for `_version_minor` field */
		_ALIGN(at, 8);

		/* Add `_version_minor` bit array field's size */
		at += 64;

		/* Align for `_result` field */
		_ALIGN(at, 8);

		/* Add `_result` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_agent_major_extension_supported_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_agent_iterate_memory_pools_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_agent_iterate_memory_pools_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_agent_memory_pool_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_memory_pool__handle` field */
		_ALIGN(at, 8);

		/* Add `_memory_pool__handle` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_agent_memory_pool_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_agent_set_async_scratch_limit_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_threshold` field */
		_ALIGN(at, 8);

		/* Add `_threshold` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_agent_set_async_scratch_limit_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_agents_allow_access_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_num_agents` field */
		_ALIGN(at, 8);

		/* Add `_num_agents` bit array field's size */
		at += 32;

		/* Align for `_agents` field */
		_ALIGN(at, 8);

		/* Add `_agents` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_agents_allow_access_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_async_function_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_arg` field */
		_ALIGN(at, 8);

		/* Add `_arg` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_async_function_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_coherency_get_type_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_type` field */
		_ALIGN(at, 8);

		/* Add `_type` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_coherency_get_type_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_coherency_set_type_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_type` field */
		_ALIGN(at, 8);

		/* Add `_type` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_coherency_set_type_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_deregister_deallocation_callback_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_deregister_deallocation_callback_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_image_create_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_image_descriptor` field */
		_ALIGN(at, 8);

		/* Add `_image_descriptor` bit array field's size */
		at += 64;

		/* Align for `_image_layout` field */
		_ALIGN(at, 8);

		/* Add `_image_layout` bit array field's size */
		at += 64;

		/* Align for `_image_data` field */
		_ALIGN(at, 8);

		/* Add `_image_data` bit array field's size */
		at += 64;

		/* Align for `_access_permission` field */
		_ALIGN(at, 8);

		/* Add `_access_permission` bit array field's size */
		at += 32;

		/* Align for `_image` field */
		_ALIGN(at, 8);

		/* Add `_image` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_image_create_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_interop_map_buffer_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_num_agents` field */
		_ALIGN(at, 8);

		/* Add `_num_agents` bit array field's size */
		at += 32;

		/* Align for `_agents` field */
		_ALIGN(at, 8);

		/* Add `_agents` bit array field's size */
		at += 64;

		/* Align for `_interop_handle` field */
		_ALIGN(at, 8);

		/* Add `_interop_handle` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_metadata_size` field */
		_ALIGN(at, 8);

		/* Add `_metadata_size` bit array field's size */
		at += 64;

		/* Align for `_metadata` field */
		_ALIGN(at, 8);

		/* Add `_metadata` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_interop_map_buffer_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_interop_unmap_buffer_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_interop_unmap_buffer_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_ipc_memory_attach_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;

		/* Align for `_len` field */
		_ALIGN(at, 8);

		/* Add `_len` bit array field's size */
		at += 64;

		/* Align for `_num_agents` field */
		_ALIGN(at, 8);

		/* Add `_num_agents` bit array field's size */
		at += 32;

		/* Align for `_mapping_agents` field */
		_ALIGN(at, 8);

		/* Add `_mapping_agents` bit array field's size */
		at += 64;

		/* Align for `_mapped_ptr` field */
		_ALIGN(at, 8);

		/* Add `_mapped_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_ipc_memory_attach_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_ipc_memory_create_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_len` field */
		_ALIGN(at, 8);

		/* Add `_len` bit array field's size */
		at += 64;

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_ipc_memory_create_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_ipc_memory_detach_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_mapped_ptr` field */
		_ALIGN(at, 8);

		/* Add `_mapped_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_ipc_memory_detach_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_ipc_signal_attach_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;

		/* Align for `_signal` field */
		_ALIGN(at, 8);

		/* Add `_signal` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_ipc_signal_attach_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_ipc_signal_create_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_ipc_signal_create_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_async_copy_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_dst_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_dst_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_src_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_src_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_num_dep_signals` field */
		_ALIGN(at, 8);

		/* Add `_num_dep_signals` bit array field's size */
		at += 32;

		/* Align for `_dep_signals` field */
		_ALIGN(at, 8);

		/* Add `_dep_signals` bit array field's size */
		at += 64;

		/* Align for `_completion_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_completion_signal__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_async_copy_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_async_copy_on_engine_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_dst_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_dst_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_src_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_src_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_num_dep_signals` field */
		_ALIGN(at, 8);

		/* Add `_num_dep_signals` bit array field's size */
		at += 32;

		/* Align for `_dep_signals` field */
		_ALIGN(at, 8);

		/* Add `_dep_signals` bit array field's size */
		at += 64;

		/* Align for `_completion_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_completion_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_engine_id` field */
		_ALIGN(at, 8);

		/* Add `_engine_id` bit array field's size */
		at += 32;

		/* Align for `_force_copy_on_sdma` field */
		_ALIGN(at, 8);

		/* Add `_force_copy_on_sdma` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_async_copy_on_engine_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_async_copy_rect_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_dst_offset` field */
		_ALIGN(at, 8);

		/* Add `_dst_offset` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_src_offset` field */
		_ALIGN(at, 8);

		/* Add `_src_offset` bit array field's size */
		at += 64;

		/* Align for `_range` field */
		_ALIGN(at, 8);

		/* Add `_range` bit array field's size */
		at += 64;

		/* Align for `_range__val__x` field */
		_ALIGN(at, 8);

		/* Add `_range__val__x` bit array field's size */
		at += 32;

		/* Align for `_range__val__y` field */
		_ALIGN(at, 8);

		/* Add `_range__val__y` bit array field's size */
		at += 32;

		/* Align for `_range__val__z` field */
		_ALIGN(at, 8);

		/* Add `_range__val__z` bit array field's size */
		at += 32;

		/* Align for `_copy_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_copy_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_dir` field */
		_ALIGN(at, 8);

		/* Add `_dir` bit array field's size */
		at += 32;

		/* Align for `_num_dep_signals` field */
		_ALIGN(at, 8);

		/* Add `_num_dep_signals` bit array field's size */
		at += 32;

		/* Align for `_dep_signals` field */
		_ALIGN(at, 8);

		/* Add `_dep_signals` bit array field's size */
		at += 64;

		/* Align for `_completion_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_completion_signal__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_async_copy_rect_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_copy_engine_status_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_dst_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_src_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_src_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_engine_ids_mask` field */
		_ALIGN(at, 8);

		/* Add `_engine_ids_mask` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_copy_engine_status_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_fill_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 32;

		/* Align for `_count` field */
		_ALIGN(at, 8);

		/* Add `_count` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_fill_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_lock_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_host_ptr` field */
		_ALIGN(at, 8);

		/* Add `_host_ptr` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_agents` field */
		_ALIGN(at, 8);

		/* Add `_agents` bit array field's size */
		at += 64;

		/* Align for `_num_agent` field */
		_ALIGN(at, 8);

		/* Add `_num_agent` bit array field's size */
		at += 32;

		/* Align for `_agent_ptr` field */
		_ALIGN(at, 8);

		/* Add `_agent_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_lock_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_lock_to_pool_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_host_ptr` field */
		_ALIGN(at, 8);

		/* Add `_host_ptr` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_agents` field */
		_ALIGN(at, 8);

		/* Add `_agents` bit array field's size */
		at += 64;

		/* Align for `_num_agent` field */
		_ALIGN(at, 8);

		/* Add `_num_agent` bit array field's size */
		at += 32;

		/* Align for `_pool__handle` field */
		_ALIGN(at, 8);

		/* Add `_pool__handle` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;

		/* Align for `_agent_ptr` field */
		_ALIGN(at, 8);

		/* Add `_agent_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_lock_to_pool_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_migrate_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_memory_pool__handle` field */
		_ALIGN(at, 8);

		/* Add `_memory_pool__handle` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_migrate_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_pool_allocate_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_memory_pool__handle` field */
		_ALIGN(at, 8);

		/* Add `_memory_pool__handle` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 32;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_pool_allocate_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_pool_can_migrate_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_src_memory_pool__handle` field */
		_ALIGN(at, 8);

		/* Add `_src_memory_pool__handle` bit array field's size */
		at += 64;

		/* Align for `_dst_memory_pool__handle` field */
		_ALIGN(at, 8);

		/* Add `_dst_memory_pool__handle` bit array field's size */
		at += 64;

		/* Align for `_result` field */
		_ALIGN(at, 8);

		/* Add `_result` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_pool_can_migrate_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_pool_free_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_pool_free_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_pool_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_memory_pool__handle` field */
		_ALIGN(at, 8);

		/* Add `_memory_pool__handle` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_pool_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_unlock_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_host_ptr` field */
		_ALIGN(at, 8);

		/* Add `_host_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_memory_unlock_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_pointer_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_info` field */
		_ALIGN(at, 8);

		/* Add `_info` bit array field's size */
		at += 64;

		/* Align for `_alloc` field */
		_ALIGN(at, 8);

		/* Add `_alloc` bit array field's size */
		at += 64;

		/* Align for `_num_agents_accessible` field */
		_ALIGN(at, 8);

		/* Add `_num_agents_accessible` bit array field's size */
		at += 64;

		/* Align for `_accessible` field */
		_ALIGN(at, 8);

		/* Add `_accessible` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_pointer_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_pointer_info_set_userdata_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_userdata` field */
		_ALIGN(at, 8);

		/* Add `_userdata` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_pointer_info_set_userdata_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_portable_close_dmabuf_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dmabuf` field */
		_ALIGN(at, 8);

		/* Add `_dmabuf` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_portable_close_dmabuf_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_portable_export_dmabuf_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_dmabuf` field */
		_ALIGN(at, 8);

		/* Add `_dmabuf` bit array field's size */
		at += 64;

		/* Align for `_offset` field */
		_ALIGN(at, 8);

		/* Add `_offset` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_portable_export_dmabuf_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_profiling_async_copy_enable_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_enable` field */
		_ALIGN(at, 8);

		/* Add `_enable` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_profiling_async_copy_enable_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_profiling_convert_tick_to_system_domain_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_agent_tick` field */
		_ALIGN(at, 8);

		/* Add `_agent_tick` bit array field's size */
		at += 64;

		/* Align for `_system_tick` field */
		_ALIGN(at, 8);

		/* Add `_system_tick` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_profiling_convert_tick_to_system_domain_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_profiling_get_async_copy_time_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_time` field */
		_ALIGN(at, 8);

		/* Add `_time` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_profiling_get_async_copy_time_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_profiling_get_dispatch_time_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_time` field */
		_ALIGN(at, 8);

		/* Add `_time` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_profiling_get_dispatch_time_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_profiling_set_profiler_enabled_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_enable` field */
		_ALIGN(at, 8);

		/* Add `_enable` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_profiling_set_profiler_enabled_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_queue_cu_get_mask_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_num_cu_mask_count` field */
		_ALIGN(at, 8);

		/* Add `_num_cu_mask_count` bit array field's size */
		at += 32;

		/* Align for `_cu_mask` field */
		_ALIGN(at, 8);

		/* Add `_cu_mask` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_queue_cu_get_mask_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_queue_cu_set_mask_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_num_cu_mask_count` field */
		_ALIGN(at, 8);

		/* Add `_num_cu_mask_count` bit array field's size */
		at += 32;

		/* Align for `_cu_mask` field */
		_ALIGN(at, 8);

		/* Add `_cu_mask` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_queue_cu_set_mask_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_queue_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_queue_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_queue_intercept_create_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent_handle__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent_handle__handle` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 32;

		/* Align for `_type` field */
		_ALIGN(at, 8);

		/* Add `_type` bit array field's size */
		at += 32;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;

		/* Align for `_private_segment_size` field */
		_ALIGN(at, 8);

		/* Add `_private_segment_size` bit array field's size */
		at += 32;

		/* Align for `_group_segment_size` field */
		_ALIGN(at, 8);

		/* Add `_group_segment_size` bit array field's size */
		at += 32;

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_queue_intercept_create_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_queue_intercept_register_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_user_data` field */
		_ALIGN(at, 8);

		/* Add `_user_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_queue_intercept_register_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_queue_set_priority_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_priority` field */
		_ALIGN(at, 8);

		/* Add `_priority` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_queue_set_priority_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_register_deallocation_callback_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_user_data` field */
		_ALIGN(at, 8);

		/* Add `_user_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_register_deallocation_callback_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_register_system_event_handler_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_register_system_event_handler_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_runtime_queue_create_register_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_user_data` field */
		_ALIGN(at, 8);

		/* Add `_user_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_runtime_queue_create_register_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_signal_async_handler_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_cond` field */
		_ALIGN(at, 8);

		/* Add `_cond` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;

		/* Align for `_handler` field */
		_ALIGN(at, 8);

		/* Add `_handler` bit array field's size */
		at += 64;

		/* Align for `_arg` field */
		_ALIGN(at, 8);

		/* Add `_arg` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_signal_async_handler_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_signal_create_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_initial_value` field */
		_ALIGN(at, 8);

		/* Add `_initial_value` bit array field's size */
		at += 64;

		/* Align for `_num_consumers` field */
		_ALIGN(at, 8);

		/* Add `_num_consumers` bit array field's size */
		at += 32;

		/* Align for `_consumers` field */
		_ALIGN(at, 8);

		/* Add `_consumers` bit array field's size */
		at += 64;

		/* Align for `_attributes` field */
		_ALIGN(at, 8);

		/* Add `_attributes` bit array field's size */
		at += 64;

		/* Align for `_signal` field */
		_ALIGN(at, 8);

		/* Add `_signal` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_signal_create_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_signal_value_pointer_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value_ptr` field */
		_ALIGN(at, 8);

		/* Add `_value_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_signal_value_pointer_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_signal_wait_any_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal_count` field */
		_ALIGN(at, 8);

		/* Add `_signal_count` bit array field's size */
		at += 32;

		/* Align for `_signals` field */
		_ALIGN(at, 8);

		/* Add `_signals` bit array field's size */
		at += 64;

		/* Align for `_conds` field */
		_ALIGN(at, 8);

		/* Add `_conds` bit array field's size */
		at += 64;

		/* Align for `_values` field */
		_ALIGN(at, 8);

		/* Add `_values` bit array field's size */
		at += 64;

		/* Align for `_timeout_hint` field */
		_ALIGN(at, 8);

		/* Add `_timeout_hint` bit array field's size */
		at += 64;

		/* Align for `_wait_hint` field */
		_ALIGN(at, 8);

		/* Add `_wait_hint` bit array field's size */
		at += 32;

		/* Align for `_satisfying_value` field */
		_ALIGN(at, 8);

		/* Add `_satisfying_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_signal_wait_any_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_spm_acquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_preferred_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_preferred_agent__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_spm_acquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_spm_release_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_preferred_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_preferred_agent__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_spm_release_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_spm_set_dest_buffer_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_preferred_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_preferred_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_size_in_bytes` field */
		_ALIGN(at, 8);

		/* Add `_size_in_bytes` bit array field's size */
		at += 64;

		/* Align for `_timeout` field */
		_ALIGN(at, 8);

		/* Add `_timeout` bit array field's size */
		at += 64;

		/* Align for `_size_copied` field */
		_ALIGN(at, 8);

		/* Add `_size_copied` bit array field's size */
		at += 64;

		/* Align for `_dest` field */
		_ALIGN(at, 8);

		/* Add `_dest` bit array field's size */
		at += 64;

		/* Align for `_is_data_loss` field */
		_ALIGN(at, 8);

		/* Add `_is_data_loss` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_spm_set_dest_buffer_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_svm_attributes_get_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_attribute_list` field */
		_ALIGN(at, 8);

		/* Add `_attribute_list` bit array field's size */
		at += 64;

		/* Align for `_attribute_count` field */
		_ALIGN(at, 8);

		/* Add `_attribute_count` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_svm_attributes_get_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_svm_attributes_set_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_attribute_list` field */
		_ALIGN(at, 8);

		/* Add `_attribute_list` bit array field's size */
		at += 64;

		/* Align for `_attribute_count` field */
		_ALIGN(at, 8);

		/* Add `_attribute_count` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_svm_attributes_set_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_svm_prefetch_async_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_num_dep_signals` field */
		_ALIGN(at, 8);

		/* Add `_num_dep_signals` bit array field's size */
		at += 32;

		/* Align for `_dep_signals` field */
		_ALIGN(at, 8);

		/* Add `_dep_signals` bit array field's size */
		at += 64;

		/* Align for `_completion_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_completion_signal__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_svm_prefetch_async_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_address_free_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_va` field */
		_ALIGN(at, 8);

		/* Add `_va` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_address_free_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_address_reserve_align_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_va` field */
		_ALIGN(at, 8);

		/* Add `_va` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_address` field */
		_ALIGN(at, 8);

		/* Add `_address` bit array field's size */
		at += 64;

		/* Align for `_alignment` field */
		_ALIGN(at, 8);

		/* Add `_alignment` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_address_reserve_align_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_address_reserve_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_va` field */
		_ALIGN(at, 8);

		/* Add `_va` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_address` field */
		_ALIGN(at, 8);

		/* Add `_address` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_address_reserve_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_export_shareable_handle_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dmabuf_fd` field */
		_ALIGN(at, 8);

		/* Add `_dmabuf_fd` bit array field's size */
		at += 64;

		/* Align for `_handle__handle` field */
		_ALIGN(at, 8);

		/* Add `_handle__handle` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_export_shareable_handle_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_get_access_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_va` field */
		_ALIGN(at, 8);

		/* Add `_va` bit array field's size */
		at += 64;

		/* Align for `_perms` field */
		_ALIGN(at, 8);

		/* Add `_perms` bit array field's size */
		at += 64;

		/* Align for `_agent_handle__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent_handle__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_get_access_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_get_alloc_properties_from_handle_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_memory_handle__handle` field */
		_ALIGN(at, 8);

		/* Add `_memory_handle__handle` bit array field's size */
		at += 64;

		/* Align for `_pool` field */
		_ALIGN(at, 8);

		/* Add `_pool` bit array field's size */
		at += 64;

		/* Align for `_type` field */
		_ALIGN(at, 8);

		/* Add `_type` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_get_alloc_properties_from_handle_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_handle_create_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_pool__handle` field */
		_ALIGN(at, 8);

		/* Add `_pool__handle` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_type` field */
		_ALIGN(at, 8);

		/* Add `_type` bit array field's size */
		at += 32;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;

		/* Align for `_memory_handle` field */
		_ALIGN(at, 8);

		/* Add `_memory_handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_handle_create_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_handle_release_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_memory_handle__handle` field */
		_ALIGN(at, 8);

		/* Add `_memory_handle__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_handle_release_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_import_shareable_handle_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dmabuf_fd` field */
		_ALIGN(at, 8);

		/* Add `_dmabuf_fd` bit array field's size */
		at += 32;

		/* Align for `_handle` field */
		_ALIGN(at, 8);

		/* Add `_handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_import_shareable_handle_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_map_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_va` field */
		_ALIGN(at, 8);

		/* Add `_va` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_in_offset` field */
		_ALIGN(at, 8);

		/* Add `_in_offset` bit array field's size */
		at += 64;

		/* Align for `_memory_handle__handle` field */
		_ALIGN(at, 8);

		/* Add `_memory_handle__handle` bit array field's size */
		at += 64;

		/* Align for `_flags` field */
		_ALIGN(at, 8);

		/* Add `_flags` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_map_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_retain_alloc_handle_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_memory_handle` field */
		_ALIGN(at, 8);

		/* Add `_memory_handle` bit array field's size */
		at += 64;

		/* Align for `_addr` field */
		_ALIGN(at, 8);

		/* Add `_addr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_retain_alloc_handle_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_set_access_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_va` field */
		_ALIGN(at, 8);

		/* Add `_va` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_desc` field */
		_ALIGN(at, 8);

		/* Add `_desc` bit array field's size */
		at += 64;

		/* Align for `_desc_cnt` field */
		_ALIGN(at, 8);

		/* Add `_desc_cnt` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_set_access_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_unmap_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_va` field */
		_ALIGN(at, 8);

		/* Add `_va` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_amd_vmem_unmap_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_cache_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_cache__handle` field */
		_ALIGN(at, 8);

		/* Add `_cache__handle` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_cache_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_deserialize_begin(void * const vctx,
	const char * const p__options)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_serialized_code_object` field */
		_ALIGN(at, 8);

		/* Add `_serialized_code_object` bit array field's size */
		at += 64;

		/* Align for `_serialized_code_object_size` field */
		_ALIGN(at, 8);

		/* Add `_serialized_code_object_size` bit array field's size */
		at += 64;

		/* Align for `_options` field */
		_ALIGN(at, 8);

		/* Add `_options` string field's size */
		at += _BYTES_TO_BITS(strlen(p__options) + 1);

		/* Align for `_code_object` field */
		_ALIGN(at, 8);

		/* Add `_code_object` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_deserialize_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_destroy_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_code_object__handle` field */
		_ALIGN(at, 8);

		/* Add `_code_object__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_destroy_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_code_object__handle` field */
		_ALIGN(at, 8);

		/* Add `_code_object__handle` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_get_symbol_begin(void * const vctx,
	const char * const p__symbol_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_code_object__handle` field */
		_ALIGN(at, 8);

		/* Add `_code_object__handle` bit array field's size */
		at += 64;

		/* Align for `_symbol_name` field */
		_ALIGN(at, 8);

		/* Add `_symbol_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__symbol_name) + 1);

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_get_symbol_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_get_symbol_from_name_begin(void * const vctx,
	const char * const p__module_name,
	const char * const p__symbol_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_code_object__handle` field */
		_ALIGN(at, 8);

		/* Add `_code_object__handle` bit array field's size */
		at += 64;

		/* Align for `_module_name` field */
		_ALIGN(at, 8);

		/* Add `_module_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__module_name) + 1);

		/* Align for `_symbol_name` field */
		_ALIGN(at, 8);

		/* Add `_symbol_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__symbol_name) + 1);

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_get_symbol_from_name_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_iterate_symbols_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_code_object__handle` field */
		_ALIGN(at, 8);

		/* Add `_code_object__handle` bit array field's size */
		at += 64;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_iterate_symbols_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_reader_create_from_file_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_file` field */
		_ALIGN(at, 8);

		/* Add `_file` bit array field's size */
		at += 32;

		/* Align for `_code_object_reader` field */
		_ALIGN(at, 8);

		/* Add `_code_object_reader` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_reader_create_from_file_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_reader_create_from_memory_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_code_object` field */
		_ALIGN(at, 8);

		/* Add `_code_object` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_code_object_reader` field */
		_ALIGN(at, 8);

		/* Add `_code_object_reader` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_reader_create_from_memory_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_reader_destroy_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_code_object_reader__handle` field */
		_ALIGN(at, 8);

		/* Add `_code_object_reader__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_reader_destroy_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_serialize_begin(void * const vctx,
	const char * const p__options)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_code_object__handle` field */
		_ALIGN(at, 8);

		/* Add `_code_object__handle` bit array field's size */
		at += 64;

		/* Align for `_alloc_callback` field */
		_ALIGN(at, 8);

		/* Add `_alloc_callback` bit array field's size */
		at += 64;

		/* Align for `_callback_data__handle` field */
		_ALIGN(at, 8);

		/* Add `_callback_data__handle` bit array field's size */
		at += 64;

		/* Align for `_options` field */
		_ALIGN(at, 8);

		/* Add `_options` string field's size */
		at += _BYTES_TO_BITS(strlen(p__options) + 1);

		/* Align for `_serialized_code_object` field */
		_ALIGN(at, 8);

		/* Add `_serialized_code_object` bit array field's size */
		at += 64;

		/* Align for `_serialized_code_object_size` field */
		_ALIGN(at, 8);

		/* Add `_serialized_code_object_size` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_object_serialize_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_symbol_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_code_symbol__handle` field */
		_ALIGN(at, 8);

		/* Add `_code_symbol__handle` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_code_symbol_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_agent_global_variable_define_begin(void * const vctx,
	const char * const p__variable_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_variable_name` field */
		_ALIGN(at, 8);

		/* Add `_variable_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__variable_name) + 1);

		/* Align for `_address` field */
		_ALIGN(at, 8);

		/* Add `_address` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_agent_global_variable_define_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_create_alt_begin(void * const vctx,
	const char * const p__options)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_profile` field */
		_ALIGN(at, 8);

		/* Add `_profile` bit array field's size */
		at += 32;

		/* Align for `_default_float_rounding_mode` field */
		_ALIGN(at, 8);

		/* Add `_default_float_rounding_mode` bit array field's size */
		at += 32;

		/* Align for `_options` field */
		_ALIGN(at, 8);

		/* Add `_options` string field's size */
		at += _BYTES_TO_BITS(strlen(p__options) + 1);

		/* Align for `_executable` field */
		_ALIGN(at, 8);

		/* Add `_executable` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_create_alt_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_create_begin(void * const vctx,
	const char * const p__options)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_profile` field */
		_ALIGN(at, 8);

		/* Add `_profile` bit array field's size */
		at += 32;

		/* Align for `_executable_state` field */
		_ALIGN(at, 8);

		/* Add `_executable_state` bit array field's size */
		at += 32;

		/* Align for `_options` field */
		_ALIGN(at, 8);

		/* Add `_options` string field's size */
		at += _BYTES_TO_BITS(strlen(p__options) + 1);

		/* Align for `_executable` field */
		_ALIGN(at, 8);

		/* Add `_executable` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_create_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_destroy_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_destroy_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_freeze_begin(void * const vctx,
	const char * const p__options)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_options` field */
		_ALIGN(at, 8);

		/* Add `_options` string field's size */
		at += _BYTES_TO_BITS(strlen(p__options) + 1);
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_freeze_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_get_symbol_begin(void * const vctx,
	const char * const p__module_name,
	const char * const p__symbol_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_module_name` field */
		_ALIGN(at, 8);

		/* Add `_module_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__module_name) + 1);

		/* Align for `_symbol_name` field */
		_ALIGN(at, 8);

		/* Add `_symbol_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__symbol_name) + 1);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_call_convention` field */
		_ALIGN(at, 8);

		/* Add `_call_convention` bit array field's size */
		at += 32;

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_get_symbol_by_name_begin(void * const vctx,
	const char * const p__symbol_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_symbol_name` field */
		_ALIGN(at, 8);

		/* Add `_symbol_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__symbol_name) + 1);

		/* Align for `_agent` field */
		_ALIGN(at, 8);

		/* Add `_agent` bit array field's size */
		at += 64;

		/* Align for `_symbol` field */
		_ALIGN(at, 8);

		/* Add `_symbol` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_get_symbol_by_name_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_get_symbol_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_global_variable_define_begin(void * const vctx,
	const char * const p__variable_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_variable_name` field */
		_ALIGN(at, 8);

		/* Add `_variable_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__variable_name) + 1);

		/* Align for `_address` field */
		_ALIGN(at, 8);

		/* Add `_address` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_global_variable_define_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_iterate_agent_symbols_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_iterate_agent_symbols_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_iterate_program_symbols_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_iterate_program_symbols_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_iterate_symbols_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_iterate_symbols_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_load_agent_code_object_begin(void * const vctx,
	const char * const p__options)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_code_object_reader__handle` field */
		_ALIGN(at, 8);

		/* Add `_code_object_reader__handle` bit array field's size */
		at += 64;

		/* Align for `_options` field */
		_ALIGN(at, 8);

		/* Add `_options` string field's size */
		at += _BYTES_TO_BITS(strlen(p__options) + 1);

		/* Align for `_loaded_code_object` field */
		_ALIGN(at, 8);

		/* Add `_loaded_code_object` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_load_agent_code_object_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_load_code_object_begin(void * const vctx,
	const char * const p__options)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_code_object__handle` field */
		_ALIGN(at, 8);

		/* Add `_code_object__handle` bit array field's size */
		at += 64;

		/* Align for `_options` field */
		_ALIGN(at, 8);

		/* Add `_options` string field's size */
		at += _BYTES_TO_BITS(strlen(p__options) + 1);
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_load_code_object_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_load_program_code_object_begin(void * const vctx,
	const char * const p__options)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_code_object_reader__handle` field */
		_ALIGN(at, 8);

		/* Add `_code_object_reader__handle` bit array field's size */
		at += 64;

		/* Align for `_options` field */
		_ALIGN(at, 8);

		/* Add `_options` string field's size */
		at += _BYTES_TO_BITS(strlen(p__options) + 1);

		/* Align for `_loaded_code_object` field */
		_ALIGN(at, 8);

		/* Add `_loaded_code_object` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_load_program_code_object_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_readonly_variable_define_begin(void * const vctx,
	const char * const p__variable_name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_variable_name` field */
		_ALIGN(at, 8);

		/* Add `_variable_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__variable_name) + 1);

		/* Align for `_address` field */
		_ALIGN(at, 8);

		/* Add `_address` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_readonly_variable_define_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_symbol_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable_symbol__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable_symbol__handle` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_symbol_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_validate_alt_begin(void * const vctx,
	const char * const p__options)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_options` field */
		_ALIGN(at, 8);

		/* Add `_options` string field's size */
		at += _BYTES_TO_BITS(strlen(p__options) + 1);

		/* Align for `_result` field */
		_ALIGN(at, 8);

		/* Add `_result` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_validate_alt_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_validate_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_executable__handle` field */
		_ALIGN(at, 8);

		/* Add `_executable__handle` bit array field's size */
		at += 64;

		/* Align for `_result` field */
		_ALIGN(at, 8);

		/* Add `_result` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_executable_validate_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_clear_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_image__handle` field */
		_ALIGN(at, 8);

		/* Add `_image__handle` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;

		/* Align for `_image_region` field */
		_ALIGN(at, 8);

		/* Add `_image_region` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_clear_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_copy_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_src_image__handle` field */
		_ALIGN(at, 8);

		/* Add `_src_image__handle` bit array field's size */
		at += 64;

		/* Align for `_src_offset` field */
		_ALIGN(at, 8);

		/* Add `_src_offset` bit array field's size */
		at += 64;

		/* Align for `_dst_image__handle` field */
		_ALIGN(at, 8);

		/* Add `_dst_image__handle` bit array field's size */
		at += 64;

		/* Align for `_dst_offset` field */
		_ALIGN(at, 8);

		/* Add `_dst_offset` bit array field's size */
		at += 64;

		/* Align for `_range` field */
		_ALIGN(at, 8);

		/* Add `_range` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_copy_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_create_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_image_descriptor` field */
		_ALIGN(at, 8);

		/* Add `_image_descriptor` bit array field's size */
		at += 64;

		/* Align for `_image_data` field */
		_ALIGN(at, 8);

		/* Add `_image_data` bit array field's size */
		at += 64;

		/* Align for `_access_permission` field */
		_ALIGN(at, 8);

		/* Add `_access_permission` bit array field's size */
		at += 32;

		/* Align for `_image` field */
		_ALIGN(at, 8);

		/* Add `_image` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_create_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_create_with_layout_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_image_descriptor` field */
		_ALIGN(at, 8);

		/* Add `_image_descriptor` bit array field's size */
		at += 64;

		/* Align for `_image_data` field */
		_ALIGN(at, 8);

		/* Add `_image_data` bit array field's size */
		at += 64;

		/* Align for `_access_permission` field */
		_ALIGN(at, 8);

		/* Add `_access_permission` bit array field's size */
		at += 32;

		/* Align for `_image_data_layout` field */
		_ALIGN(at, 8);

		/* Add `_image_data_layout` bit array field's size */
		at += 32;

		/* Align for `_image_data_row_pitch` field */
		_ALIGN(at, 8);

		/* Add `_image_data_row_pitch` bit array field's size */
		at += 64;

		/* Align for `_image_data_slice_pitch` field */
		_ALIGN(at, 8);

		/* Add `_image_data_slice_pitch` bit array field's size */
		at += 64;

		/* Align for `_image` field */
		_ALIGN(at, 8);

		/* Add `_image` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_create_with_layout_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_data_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_image_descriptor` field */
		_ALIGN(at, 8);

		/* Add `_image_descriptor` bit array field's size */
		at += 64;

		/* Align for `_access_permission` field */
		_ALIGN(at, 8);

		/* Add `_access_permission` bit array field's size */
		at += 32;

		/* Align for `_image_data_info` field */
		_ALIGN(at, 8);

		/* Add `_image_data_info` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_data_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_data_get_info_with_layout_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_image_descriptor` field */
		_ALIGN(at, 8);

		/* Add `_image_descriptor` bit array field's size */
		at += 64;

		/* Align for `_access_permission` field */
		_ALIGN(at, 8);

		/* Add `_access_permission` bit array field's size */
		at += 32;

		/* Align for `_image_data_layout` field */
		_ALIGN(at, 8);

		/* Add `_image_data_layout` bit array field's size */
		at += 32;

		/* Align for `_image_data_row_pitch` field */
		_ALIGN(at, 8);

		/* Add `_image_data_row_pitch` bit array field's size */
		at += 64;

		/* Align for `_image_data_slice_pitch` field */
		_ALIGN(at, 8);

		/* Add `_image_data_slice_pitch` bit array field's size */
		at += 64;

		/* Align for `_image_data_info` field */
		_ALIGN(at, 8);

		/* Add `_image_data_info` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_data_get_info_with_layout_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_destroy_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_image__handle` field */
		_ALIGN(at, 8);

		/* Add `_image__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_destroy_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_export_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_src_image__handle` field */
		_ALIGN(at, 8);

		/* Add `_src_image__handle` bit array field's size */
		at += 64;

		/* Align for `_dst_memory` field */
		_ALIGN(at, 8);

		/* Add `_dst_memory` bit array field's size */
		at += 64;

		/* Align for `_dst_row_pitch` field */
		_ALIGN(at, 8);

		/* Add `_dst_row_pitch` bit array field's size */
		at += 64;

		/* Align for `_dst_slice_pitch` field */
		_ALIGN(at, 8);

		/* Add `_dst_slice_pitch` bit array field's size */
		at += 64;

		/* Align for `_image_region` field */
		_ALIGN(at, 8);

		/* Add `_image_region` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_export_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_get_capability_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_geometry` field */
		_ALIGN(at, 8);

		/* Add `_geometry` bit array field's size */
		at += 32;

		/* Align for `_image_format` field */
		_ALIGN(at, 8);

		/* Add `_image_format` bit array field's size */
		at += 64;

		/* Align for `_capability_mask` field */
		_ALIGN(at, 8);

		/* Add `_capability_mask` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_get_capability_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_get_capability_with_layout_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_geometry` field */
		_ALIGN(at, 8);

		/* Add `_geometry` bit array field's size */
		at += 32;

		/* Align for `_image_format` field */
		_ALIGN(at, 8);

		/* Add `_image_format` bit array field's size */
		at += 64;

		/* Align for `_image_data_layout` field */
		_ALIGN(at, 8);

		/* Add `_image_data_layout` bit array field's size */
		at += 32;

		/* Align for `_capability_mask` field */
		_ALIGN(at, 8);

		/* Add `_capability_mask` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_get_capability_with_layout_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_import_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_src_memory` field */
		_ALIGN(at, 8);

		/* Add `_src_memory` bit array field's size */
		at += 64;

		/* Align for `_src_row_pitch` field */
		_ALIGN(at, 8);

		/* Add `_src_row_pitch` bit array field's size */
		at += 64;

		/* Align for `_src_slice_pitch` field */
		_ALIGN(at, 8);

		/* Add `_src_slice_pitch` bit array field's size */
		at += 64;

		/* Align for `_dst_image__handle` field */
		_ALIGN(at, 8);

		/* Add `_dst_image__handle` bit array field's size */
		at += 64;

		/* Align for `_image_region` field */
		_ALIGN(at, 8);

		/* Add `_image_region` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_image_import_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_sampler_create_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_sampler_descriptor` field */
		_ALIGN(at, 8);

		/* Add `_sampler_descriptor` bit array field's size */
		at += 64;

		/* Align for `_sampler` field */
		_ALIGN(at, 8);

		/* Add `_sampler` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_sampler_create_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_sampler_destroy_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_sampler__handle` field */
		_ALIGN(at, 8);

		/* Add `_sampler__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_ext_sampler_destroy_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_extension_get_name_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_extension` field */
		_ALIGN(at, 8);

		/* Add `_extension` bit array field's size */
		at += 16;

		/* Align for `_name` field */
		_ALIGN(at, 8);

		/* Add `_name` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_extension_get_name_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_init_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_init_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_compatible_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_code_object_isa__handle` field */
		_ALIGN(at, 8);

		/* Add `_code_object_isa__handle` bit array field's size */
		at += 64;

		/* Align for `_agent_isa__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent_isa__handle` bit array field's size */
		at += 64;

		/* Align for `_result` field */
		_ALIGN(at, 8);

		/* Add `_result` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_compatible_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_from_name_begin(void * const vctx,
	const char * const p__name)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_name` field */
		_ALIGN(at, 8);

		/* Add `_name` string field's size */
		at += _BYTES_TO_BITS(strlen(p__name) + 1);

		/* Align for `_isa` field */
		_ALIGN(at, 8);

		/* Add `_isa` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_from_name_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_get_exception_policies_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_isa__handle` field */
		_ALIGN(at, 8);

		/* Add `_isa__handle` bit array field's size */
		at += 64;

		/* Align for `_profile` field */
		_ALIGN(at, 8);

		/* Add `_profile` bit array field's size */
		at += 32;

		/* Align for `_mask` field */
		_ALIGN(at, 8);

		/* Add `_mask` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_get_exception_policies_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_get_info_alt_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_isa__handle` field */
		_ALIGN(at, 8);

		/* Add `_isa__handle` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_get_info_alt_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_isa__handle` field */
		_ALIGN(at, 8);

		/* Add `_isa__handle` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_index` field */
		_ALIGN(at, 8);

		/* Add `_index` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_get_round_method_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_isa__handle` field */
		_ALIGN(at, 8);

		/* Add `_isa__handle` bit array field's size */
		at += 64;

		/* Align for `_fp_type` field */
		_ALIGN(at, 8);

		/* Add `_fp_type` bit array field's size */
		at += 32;

		/* Align for `_flush_mode` field */
		_ALIGN(at, 8);

		/* Add `_flush_mode` bit array field's size */
		at += 32;

		/* Align for `_round_method` field */
		_ALIGN(at, 8);

		/* Add `_round_method` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_get_round_method_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_iterate_wavefronts_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_isa__handle` field */
		_ALIGN(at, 8);

		/* Add `_isa__handle` bit array field's size */
		at += 64;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_isa_iterate_wavefronts_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_iterate_agents_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_iterate_agents_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_memory_allocate_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_region__handle` field */
		_ALIGN(at, 8);

		/* Add `_region__handle` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_memory_allocate_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_memory_assign_agent_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_access` field */
		_ALIGN(at, 8);

		/* Add `_access` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_memory_assign_agent_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_memory_copy_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_dst` field */
		_ALIGN(at, 8);

		/* Add `_dst` bit array field's size */
		at += 64;

		/* Align for `_src` field */
		_ALIGN(at, 8);

		/* Add `_src` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_memory_copy_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_memory_deregister_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_memory_deregister_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_memory_free_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_memory_free_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_memory_register_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_ptr` field */
		_ALIGN(at, 8);

		/* Add `_ptr` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_memory_register_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_add_write_index_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_add_write_index_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_add_write_index_scacq_screl_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_add_write_index_scacq_screl_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_add_write_index_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_add_write_index_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_add_write_index_screlease_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_add_write_index_screlease_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_cas_write_index_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_expected` field */
		_ALIGN(at, 8);

		/* Add `_expected` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_cas_write_index_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_cas_write_index_scacq_screl_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_expected` field */
		_ALIGN(at, 8);

		/* Add `_expected` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_cas_write_index_scacq_screl_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_cas_write_index_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_expected` field */
		_ALIGN(at, 8);

		/* Add `_expected` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_cas_write_index_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_cas_write_index_screlease_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_expected` field */
		_ALIGN(at, 8);

		/* Add `_expected` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_cas_write_index_screlease_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_create_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_agent__handle` field */
		_ALIGN(at, 8);

		/* Add `_agent__handle` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 32;

		/* Align for `_type` field */
		_ALIGN(at, 8);

		/* Add `_type` bit array field's size */
		at += 32;

		/* Align for `_callback` field */
		_ALIGN(at, 8);

		/* Add `_callback` bit array field's size */
		at += 64;

		/* Align for `_data` field */
		_ALIGN(at, 8);

		/* Add `_data` bit array field's size */
		at += 64;

		/* Align for `_private_segment_size` field */
		_ALIGN(at, 8);

		/* Add `_private_segment_size` bit array field's size */
		at += 32;

		/* Align for `_group_segment_size` field */
		_ALIGN(at, 8);

		/* Add `_group_segment_size` bit array field's size */
		at += 32;

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_create_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_destroy_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_destroy_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_inactivate_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_inactivate_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_load_read_index_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_load_read_index_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_load_read_index_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_load_read_index_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_load_write_index_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_load_write_index_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_load_write_index_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_load_write_index_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_store_read_index_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_store_read_index_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_store_read_index_screlease_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_store_read_index_screlease_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_store_write_index_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_store_write_index_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_store_write_index_screlease_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_queue_store_write_index_screlease_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_region_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_region__handle` field */
		_ALIGN(at, 8);

		/* Add `_region__handle` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_region_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_shut_down_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_shut_down_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_add_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_add_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_add_scacq_screl_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_add_scacq_screl_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_add_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_add_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_add_screlease_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_add_screlease_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_and_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_and_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_and_scacq_screl_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_and_scacq_screl_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_and_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_and_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_and_screlease_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_and_screlease_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_cas_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_expected` field */
		_ALIGN(at, 8);

		/* Add `_expected` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_cas_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_cas_scacq_screl_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_expected` field */
		_ALIGN(at, 8);

		/* Add `_expected` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_cas_scacq_screl_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_cas_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_expected` field */
		_ALIGN(at, 8);

		/* Add `_expected` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_cas_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_cas_screlease_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_expected` field */
		_ALIGN(at, 8);

		/* Add `_expected` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_cas_screlease_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_create_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_initial_value` field */
		_ALIGN(at, 8);

		/* Add `_initial_value` bit array field's size */
		at += 64;

		/* Align for `_num_consumers` field */
		_ALIGN(at, 8);

		/* Add `_num_consumers` bit array field's size */
		at += 32;

		/* Align for `_consumers` field */
		_ALIGN(at, 8);

		/* Add `_consumers` bit array field's size */
		at += 64;

		/* Align for `_signal` field */
		_ALIGN(at, 8);

		/* Add `_signal` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_create_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_destroy_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_destroy_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_exchange_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_exchange_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_exchange_scacq_screl_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_exchange_scacq_screl_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_exchange_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_exchange_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_exchange_screlease_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_exchange_screlease_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_group_create_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_num_signals` field */
		_ALIGN(at, 8);

		/* Add `_num_signals` bit array field's size */
		at += 32;

		/* Align for `_signals` field */
		_ALIGN(at, 8);

		/* Add `_signals` bit array field's size */
		at += 64;

		/* Align for `_num_consumers` field */
		_ALIGN(at, 8);

		/* Add `_num_consumers` bit array field's size */
		at += 32;

		/* Align for `_consumers` field */
		_ALIGN(at, 8);

		/* Add `_consumers` bit array field's size */
		at += 64;

		/* Align for `_signal_group` field */
		_ALIGN(at, 8);

		/* Add `_signal_group` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_group_create_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_group_destroy_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal_group__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal_group__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_group_destroy_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_group_wait_any_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal_group__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal_group__handle` bit array field's size */
		at += 64;

		/* Align for `_conditions` field */
		_ALIGN(at, 8);

		/* Add `_conditions` bit array field's size */
		at += 64;

		/* Align for `_compare_values` field */
		_ALIGN(at, 8);

		/* Add `_compare_values` bit array field's size */
		at += 64;

		/* Align for `_wait_state_hint` field */
		_ALIGN(at, 8);

		/* Add `_wait_state_hint` bit array field's size */
		at += 32;

		/* Align for `_signal` field */
		_ALIGN(at, 8);

		/* Add `_signal` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_group_wait_any_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_group_wait_any_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal_group__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal_group__handle` bit array field's size */
		at += 64;

		/* Align for `_conditions` field */
		_ALIGN(at, 8);

		/* Add `_conditions` bit array field's size */
		at += 64;

		/* Align for `_compare_values` field */
		_ALIGN(at, 8);

		/* Add `_compare_values` bit array field's size */
		at += 64;

		/* Align for `_wait_state_hint` field */
		_ALIGN(at, 8);

		/* Add `_wait_state_hint` bit array field's size */
		at += 32;

		/* Align for `_signal` field */
		_ALIGN(at, 8);

		/* Add `_signal` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_group_wait_any_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_load_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_load_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_load_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_load_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_or_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_or_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_or_scacq_screl_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_or_scacq_screl_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_or_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_or_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_or_screlease_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_or_screlease_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_silent_store_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_silent_store_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_silent_store_screlease_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_silent_store_screlease_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_store_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_store_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_store_screlease_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_store_screlease_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_subtract_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_subtract_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_subtract_scacq_screl_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_subtract_scacq_screl_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_subtract_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_subtract_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_subtract_screlease_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_subtract_screlease_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_wait_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_condition` field */
		_ALIGN(at, 8);

		/* Add `_condition` bit array field's size */
		at += 32;

		/* Align for `_compare_value` field */
		_ALIGN(at, 8);

		/* Add `_compare_value` bit array field's size */
		at += 64;

		/* Align for `_timeout_hint` field */
		_ALIGN(at, 8);

		/* Add `_timeout_hint` bit array field's size */
		at += 64;

		/* Align for `_wait_state_hint` field */
		_ALIGN(at, 8);

		/* Add `_wait_state_hint` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_wait_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_wait_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_condition` field */
		_ALIGN(at, 8);

		/* Add `_condition` bit array field's size */
		at += 32;

		/* Align for `_compare_value` field */
		_ALIGN(at, 8);

		/* Add `_compare_value` bit array field's size */
		at += 64;

		/* Align for `_timeout_hint` field */
		_ALIGN(at, 8);

		/* Add `_timeout_hint` bit array field's size */
		at += 64;

		/* Align for `_wait_state_hint` field */
		_ALIGN(at, 8);

		/* Add `_wait_state_hint` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_wait_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_xor_relaxed_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_xor_relaxed_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_xor_scacq_screl_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_xor_scacq_screl_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_xor_scacquire_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_xor_scacquire_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_xor_screlease_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_signal_xor_screlease_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_soft_queue_create_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_region__handle` field */
		_ALIGN(at, 8);

		/* Add `_region__handle` bit array field's size */
		at += 64;

		/* Align for `_size` field */
		_ALIGN(at, 8);

		/* Add `_size` bit array field's size */
		at += 32;

		/* Align for `_type` field */
		_ALIGN(at, 8);

		/* Add `_type` bit array field's size */
		at += 32;

		/* Align for `_features` field */
		_ALIGN(at, 8);

		/* Add `_features` bit array field's size */
		at += 32;

		/* Align for `_doorbell_signal__handle` field */
		_ALIGN(at, 8);

		/* Add `_doorbell_signal__handle` bit array field's size */
		at += 64;

		/* Align for `_queue` field */
		_ALIGN(at, 8);

		/* Add `_queue` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_soft_queue_create_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_status_string_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_status` field */
		_ALIGN(at, 8);

		/* Add `_status` bit array field's size */
		at += 32;

		/* Align for `_status_string` field */
		_ALIGN(at, 8);

		/* Add `_status_string` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_status_string_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_system_extension_supported_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_extension` field */
		_ALIGN(at, 8);

		/* Add `_extension` bit array field's size */
		at += 16;

		/* Align for `_version_major` field */
		_ALIGN(at, 8);

		/* Add `_version_major` bit array field's size */
		at += 16;

		/* Align for `_version_minor` field */
		_ALIGN(at, 8);

		/* Add `_version_minor` bit array field's size */
		at += 16;

		/* Align for `_result` field */
		_ALIGN(at, 8);

		/* Add `_result` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_system_extension_supported_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_system_get_extension_table_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_extension` field */
		_ALIGN(at, 8);

		/* Add `_extension` bit array field's size */
		at += 16;

		/* Align for `_version_major` field */
		_ALIGN(at, 8);

		/* Add `_version_major` bit array field's size */
		at += 16;

		/* Align for `_version_minor` field */
		_ALIGN(at, 8);

		/* Add `_version_minor` bit array field's size */
		at += 16;

		/* Align for `_table` field */
		_ALIGN(at, 8);

		/* Add `_table` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_system_get_extension_table_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_system_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_system_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_system_get_major_extension_table_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_extension` field */
		_ALIGN(at, 8);

		/* Add `_extension` bit array field's size */
		at += 16;

		/* Align for `_version_major` field */
		_ALIGN(at, 8);

		/* Add `_version_major` bit array field's size */
		at += 16;

		/* Align for `_table_length` field */
		_ALIGN(at, 8);

		/* Add `_table_length` bit array field's size */
		at += 64;

		/* Align for `_table` field */
		_ALIGN(at, 8);

		/* Add `_table` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_system_get_major_extension_table_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_system_major_extension_supported_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_extension` field */
		_ALIGN(at, 8);

		/* Add `_extension` bit array field's size */
		at += 16;

		/* Align for `_version_major` field */
		_ALIGN(at, 8);

		/* Add `_version_major` bit array field's size */
		at += 16;

		/* Align for `_version_minor` field */
		_ALIGN(at, 8);

		/* Add `_version_minor` bit array field's size */
		at += 64;

		/* Align for `_result` field */
		_ALIGN(at, 8);

		/* Add `_result` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_system_major_extension_supported_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_wavefront_get_info_begin(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_wavefront__handle` field */
		_ALIGN(at, 8);

		/* Add `_wavefront__handle` bit array field's size */
		at += 64;

		/* Align for `_attribute` field */
		_ALIGN(at, 8);

		/* Add `_attribute` bit array field's size */
		at += 32;

		/* Align for `_value` field */
		_ALIGN(at, 8);

		/* Add `_value` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

static uint32_t _er_size_hsa_api_hsa_wavefront_get_info_end(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_queue_id` field */
		_ALIGN(at, 32);

		/* Add `_queue_id` bit array field's size */
		at += 32;

		/* Align for `_agent_id` field */
		_ALIGN(at, 32);

		/* Add `_agent_id` bit array field's size */
		at += 32;

		/* Align for `_correlation_id` field */
		_ALIGN(at, 64);

		/* Add `_correlation_id` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 8);

		/* Align for `_retval` field */
		_ALIGN(at, 8);

		/* Add `_retval` bit array field's size */
		at += 32;
	}

	return at - ctx->at;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_extension_supported_begin`) */
void barectf_hsa_api_trace_hsa_agent_extension_supported_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint64_t p__agent__handle,
	const uint16_t p__version_major,
	const uint16_t p__version_minor,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_extension_supported_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_extension_supported_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__extension, p__agent__handle, p__version_major, p__version_minor, p__result);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_extension_supported_end`) */
void barectf_hsa_api_trace_hsa_agent_extension_supported_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_extension_supported_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_extension_supported_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_get_exception_policies_begin`) */
void barectf_hsa_api_trace_hsa_agent_get_exception_policies_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const int32_t p__profile,
	const uint64_t p__mask)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_get_exception_policies_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_get_exception_policies_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__profile, p__mask);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_get_exception_policies_end`) */
void barectf_hsa_api_trace_hsa_agent_get_exception_policies_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_get_exception_policies_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_get_exception_policies_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_get_info_begin`) */
void barectf_hsa_api_trace_hsa_agent_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__attribute, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_get_info_end`) */
void barectf_hsa_api_trace_hsa_agent_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_iterate_caches_begin`) */
void barectf_hsa_api_trace_hsa_agent_iterate_caches_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_iterate_caches_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_iterate_caches_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__callback, p__data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_iterate_caches_end`) */
void barectf_hsa_api_trace_hsa_agent_iterate_caches_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_iterate_caches_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_iterate_caches_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_iterate_isas_begin`) */
void barectf_hsa_api_trace_hsa_agent_iterate_isas_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_iterate_isas_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_iterate_isas_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__callback, p__data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_iterate_isas_end`) */
void barectf_hsa_api_trace_hsa_agent_iterate_isas_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_iterate_isas_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_iterate_isas_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_iterate_regions_begin`) */
void barectf_hsa_api_trace_hsa_agent_iterate_regions_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_iterate_regions_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_iterate_regions_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__callback, p__data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_iterate_regions_end`) */
void barectf_hsa_api_trace_hsa_agent_iterate_regions_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_iterate_regions_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_iterate_regions_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_major_extension_supported_begin`) */
void barectf_hsa_api_trace_hsa_agent_major_extension_supported_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint64_t p__agent__handle,
	const uint16_t p__version_major,
	const uint64_t p__version_minor,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_major_extension_supported_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_major_extension_supported_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__extension, p__agent__handle, p__version_major, p__version_minor, p__result);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_agent_major_extension_supported_end`) */
void barectf_hsa_api_trace_hsa_agent_major_extension_supported_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_agent_major_extension_supported_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_agent_major_extension_supported_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_agent_iterate_memory_pools_begin`) */
void barectf_hsa_api_trace_hsa_amd_agent_iterate_memory_pools_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_agent_iterate_memory_pools_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_agent_iterate_memory_pools_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__callback, p__data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_agent_iterate_memory_pools_end`) */
void barectf_hsa_api_trace_hsa_amd_agent_iterate_memory_pools_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_agent_iterate_memory_pools_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_agent_iterate_memory_pools_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_agent_memory_pool_get_info_begin`) */
void barectf_hsa_api_trace_hsa_amd_agent_memory_pool_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__memory_pool__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_agent_memory_pool_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_agent_memory_pool_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__memory_pool__handle, p__attribute, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_agent_memory_pool_get_info_end`) */
void barectf_hsa_api_trace_hsa_amd_agent_memory_pool_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_agent_memory_pool_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_agent_memory_pool_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_agent_set_async_scratch_limit_begin`) */
void barectf_hsa_api_trace_hsa_amd_agent_set_async_scratch_limit_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__threshold)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_agent_set_async_scratch_limit_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_agent_set_async_scratch_limit_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__threshold);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_agent_set_async_scratch_limit_end`) */
void barectf_hsa_api_trace_hsa_amd_agent_set_async_scratch_limit_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_agent_set_async_scratch_limit_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_agent_set_async_scratch_limit_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_agents_allow_access_begin`) */
void barectf_hsa_api_trace_hsa_amd_agents_allow_access_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint32_t p__num_agents,
	const uint64_t p__agents,
	const uint64_t p__flags,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_agents_allow_access_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_agents_allow_access_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__num_agents, p__agents, p__flags, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_agents_allow_access_end`) */
void barectf_hsa_api_trace_hsa_amd_agents_allow_access_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_agents_allow_access_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_agents_allow_access_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_async_function_begin`) */
void barectf_hsa_api_trace_hsa_amd_async_function_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__callback,
	const uint64_t p__arg)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_async_function_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_async_function_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__callback, p__arg);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_async_function_end`) */
void barectf_hsa_api_trace_hsa_amd_async_function_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_async_function_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_async_function_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_coherency_get_type_begin`) */
void barectf_hsa_api_trace_hsa_amd_coherency_get_type_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__type)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_coherency_get_type_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_coherency_get_type_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__type);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_coherency_get_type_end`) */
void barectf_hsa_api_trace_hsa_amd_coherency_get_type_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_coherency_get_type_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_coherency_get_type_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_coherency_set_type_begin`) */
void barectf_hsa_api_trace_hsa_amd_coherency_set_type_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const int32_t p__type)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_coherency_set_type_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_coherency_set_type_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__type);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_coherency_set_type_end`) */
void barectf_hsa_api_trace_hsa_amd_coherency_set_type_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_coherency_set_type_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_coherency_set_type_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_deregister_deallocation_callback_begin`) */
void barectf_hsa_api_trace_hsa_amd_deregister_deallocation_callback_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__callback)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_deregister_deallocation_callback_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_deregister_deallocation_callback_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__callback);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_deregister_deallocation_callback_end`) */
void barectf_hsa_api_trace_hsa_amd_deregister_deallocation_callback_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_deregister_deallocation_callback_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_deregister_deallocation_callback_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_image_create_begin`) */
void barectf_hsa_api_trace_hsa_amd_image_create_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image_descriptor,
	const uint64_t p__image_layout,
	const uint64_t p__image_data,
	const int32_t p__access_permission,
	const uint64_t p__image)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_image_create_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_image_create_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__image_descriptor, p__image_layout, p__image_data, p__access_permission, p__image);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_image_create_end`) */
void barectf_hsa_api_trace_hsa_amd_image_create_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_image_create_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_image_create_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_interop_map_buffer_begin`) */
void barectf_hsa_api_trace_hsa_amd_interop_map_buffer_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint32_t p__num_agents,
	const uint64_t p__agents,
	const int32_t p__interop_handle,
	const uint32_t p__flags,
	const uint64_t p__size,
	const uint64_t p__ptr,
	const uint64_t p__metadata_size,
	const uint64_t p__metadata)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_interop_map_buffer_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_interop_map_buffer_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__num_agents, p__agents, p__interop_handle, p__flags, p__size, p__ptr, p__metadata_size, p__metadata);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_interop_map_buffer_end`) */
void barectf_hsa_api_trace_hsa_amd_interop_map_buffer_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_interop_map_buffer_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_interop_map_buffer_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_interop_unmap_buffer_begin`) */
void barectf_hsa_api_trace_hsa_amd_interop_unmap_buffer_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_interop_unmap_buffer_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_interop_unmap_buffer_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_interop_unmap_buffer_end`) */
void barectf_hsa_api_trace_hsa_amd_interop_unmap_buffer_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_interop_unmap_buffer_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_interop_unmap_buffer_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_ipc_memory_attach_begin`) */
void barectf_hsa_api_trace_hsa_amd_ipc_memory_attach_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__handle,
	const uint64_t p__len,
	const uint32_t p__num_agents,
	const uint64_t p__mapping_agents,
	const uint64_t p__mapped_ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_ipc_memory_attach_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_ipc_memory_attach_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__handle, p__len, p__num_agents, p__mapping_agents, p__mapped_ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_ipc_memory_attach_end`) */
void barectf_hsa_api_trace_hsa_amd_ipc_memory_attach_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_ipc_memory_attach_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_ipc_memory_attach_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_ipc_memory_create_begin`) */
void barectf_hsa_api_trace_hsa_amd_ipc_memory_create_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__len,
	const uint64_t p__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_ipc_memory_create_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_ipc_memory_create_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__len, p__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_ipc_memory_create_end`) */
void barectf_hsa_api_trace_hsa_amd_ipc_memory_create_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_ipc_memory_create_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_ipc_memory_create_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_ipc_memory_detach_begin`) */
void barectf_hsa_api_trace_hsa_amd_ipc_memory_detach_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__mapped_ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_ipc_memory_detach_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_ipc_memory_detach_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__mapped_ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_ipc_memory_detach_end`) */
void barectf_hsa_api_trace_hsa_amd_ipc_memory_detach_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_ipc_memory_detach_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_ipc_memory_detach_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_ipc_signal_attach_begin`) */
void barectf_hsa_api_trace_hsa_amd_ipc_signal_attach_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__handle,
	const uint64_t p__signal)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_ipc_signal_attach_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_ipc_signal_attach_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__handle, p__signal);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_ipc_signal_attach_end`) */
void barectf_hsa_api_trace_hsa_amd_ipc_signal_attach_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_ipc_signal_attach_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_ipc_signal_attach_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_ipc_signal_create_begin`) */
void barectf_hsa_api_trace_hsa_amd_ipc_signal_create_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const uint64_t p__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_ipc_signal_create_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_ipc_signal_create_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_ipc_signal_create_end`) */
void barectf_hsa_api_trace_hsa_amd_ipc_signal_create_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_ipc_signal_create_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_ipc_signal_create_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_async_copy_begin`) */
void barectf_hsa_api_trace_hsa_amd_memory_async_copy_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__dst,
	const uint64_t p__dst_agent__handle,
	const uint64_t p__src,
	const uint64_t p__src_agent__handle,
	const uint64_t p__size,
	const uint32_t p__num_dep_signals,
	const uint64_t p__dep_signals,
	const uint64_t p__completion_signal__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_async_copy_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_async_copy_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__dst, p__dst_agent__handle, p__src, p__src_agent__handle, p__size, p__num_dep_signals, p__dep_signals, p__completion_signal__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_async_copy_end`) */
void barectf_hsa_api_trace_hsa_amd_memory_async_copy_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_async_copy_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_async_copy_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_async_copy_on_engine_begin`) */
void barectf_hsa_api_trace_hsa_amd_memory_async_copy_on_engine_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__dst,
	const uint64_t p__dst_agent__handle,
	const uint64_t p__src,
	const uint64_t p__src_agent__handle,
	const uint64_t p__size,
	const uint32_t p__num_dep_signals,
	const uint64_t p__dep_signals,
	const uint64_t p__completion_signal__handle,
	const int32_t p__engine_id,
	const int32_t p__force_copy_on_sdma)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_async_copy_on_engine_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_async_copy_on_engine_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__dst, p__dst_agent__handle, p__src, p__src_agent__handle, p__size, p__num_dep_signals, p__dep_signals, p__completion_signal__handle, p__engine_id, p__force_copy_on_sdma);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_async_copy_on_engine_end`) */
void barectf_hsa_api_trace_hsa_amd_memory_async_copy_on_engine_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_async_copy_on_engine_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_async_copy_on_engine_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_async_copy_rect_begin`) */
void barectf_hsa_api_trace_hsa_amd_memory_async_copy_rect_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__dst,
	const uint64_t p__dst_offset,
	const uint64_t p__src,
	const uint64_t p__src_offset,
	const uint64_t p__range,
	const uint32_t p__range__val__x,
	const uint32_t p__range__val__y,
	const uint32_t p__range__val__z,
	const uint64_t p__copy_agent__handle,
	const int32_t p__dir,
	const uint32_t p__num_dep_signals,
	const uint64_t p__dep_signals,
	const uint64_t p__completion_signal__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_async_copy_rect_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_async_copy_rect_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__dst, p__dst_offset, p__src, p__src_offset, p__range, p__range__val__x, p__range__val__y, p__range__val__z, p__copy_agent__handle, p__dir, p__num_dep_signals, p__dep_signals, p__completion_signal__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_async_copy_rect_end`) */
void barectf_hsa_api_trace_hsa_amd_memory_async_copy_rect_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_async_copy_rect_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_async_copy_rect_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_copy_engine_status_begin`) */
void barectf_hsa_api_trace_hsa_amd_memory_copy_engine_status_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__dst_agent__handle,
	const uint64_t p__src_agent__handle,
	const uint64_t p__engine_ids_mask)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_copy_engine_status_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_copy_engine_status_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__dst_agent__handle, p__src_agent__handle, p__engine_ids_mask);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_copy_engine_status_end`) */
void barectf_hsa_api_trace_hsa_amd_memory_copy_engine_status_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_copy_engine_status_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_copy_engine_status_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_fill_begin`) */
void barectf_hsa_api_trace_hsa_amd_memory_fill_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint32_t p__value,
	const uint64_t p__count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_fill_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_fill_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__value, p__count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_fill_end`) */
void barectf_hsa_api_trace_hsa_amd_memory_fill_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_fill_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_fill_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_lock_begin`) */
void barectf_hsa_api_trace_hsa_amd_memory_lock_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__host_ptr,
	const uint64_t p__size,
	const uint64_t p__agents,
	const int32_t p__num_agent,
	const uint64_t p__agent_ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_lock_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_lock_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__host_ptr, p__size, p__agents, p__num_agent, p__agent_ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_lock_end`) */
void barectf_hsa_api_trace_hsa_amd_memory_lock_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_lock_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_lock_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_lock_to_pool_begin`) */
void barectf_hsa_api_trace_hsa_amd_memory_lock_to_pool_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__host_ptr,
	const uint64_t p__size,
	const uint64_t p__agents,
	const int32_t p__num_agent,
	const uint64_t p__pool__handle,
	const uint32_t p__flags,
	const uint64_t p__agent_ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_lock_to_pool_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_lock_to_pool_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__host_ptr, p__size, p__agents, p__num_agent, p__pool__handle, p__flags, p__agent_ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_lock_to_pool_end`) */
void barectf_hsa_api_trace_hsa_amd_memory_lock_to_pool_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_lock_to_pool_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_lock_to_pool_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_migrate_begin`) */
void barectf_hsa_api_trace_hsa_amd_memory_migrate_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__memory_pool__handle,
	const uint32_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_migrate_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_migrate_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__memory_pool__handle, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_migrate_end`) */
void barectf_hsa_api_trace_hsa_amd_memory_migrate_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_migrate_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_migrate_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_pool_allocate_begin`) */
void barectf_hsa_api_trace_hsa_amd_memory_pool_allocate_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__memory_pool__handle,
	const uint64_t p__size,
	const uint32_t p__flags,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_pool_allocate_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_pool_allocate_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__memory_pool__handle, p__size, p__flags, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_pool_allocate_end`) */
void barectf_hsa_api_trace_hsa_amd_memory_pool_allocate_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_pool_allocate_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_pool_allocate_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_pool_can_migrate_begin`) */
void barectf_hsa_api_trace_hsa_amd_memory_pool_can_migrate_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__src_memory_pool__handle,
	const uint64_t p__dst_memory_pool__handle,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_pool_can_migrate_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_pool_can_migrate_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__src_memory_pool__handle, p__dst_memory_pool__handle, p__result);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_pool_can_migrate_end`) */
void barectf_hsa_api_trace_hsa_amd_memory_pool_can_migrate_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_pool_can_migrate_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_pool_can_migrate_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_pool_free_begin`) */
void barectf_hsa_api_trace_hsa_amd_memory_pool_free_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_pool_free_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_pool_free_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_pool_free_end`) */
void barectf_hsa_api_trace_hsa_amd_memory_pool_free_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_pool_free_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_pool_free_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_pool_get_info_begin`) */
void barectf_hsa_api_trace_hsa_amd_memory_pool_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__memory_pool__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_pool_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_pool_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__memory_pool__handle, p__attribute, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_pool_get_info_end`) */
void barectf_hsa_api_trace_hsa_amd_memory_pool_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_pool_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_pool_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_unlock_begin`) */
void barectf_hsa_api_trace_hsa_amd_memory_unlock_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__host_ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_unlock_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_unlock_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__host_ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_memory_unlock_end`) */
void barectf_hsa_api_trace_hsa_amd_memory_unlock_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_memory_unlock_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_memory_unlock_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_pointer_info_begin`) */
void barectf_hsa_api_trace_hsa_amd_pointer_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__info,
	const uint64_t p__alloc,
	const uint64_t p__num_agents_accessible,
	const uint64_t p__accessible)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_pointer_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_pointer_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__info, p__alloc, p__num_agents_accessible, p__accessible);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_pointer_info_end`) */
void barectf_hsa_api_trace_hsa_amd_pointer_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_pointer_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_pointer_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_pointer_info_set_userdata_begin`) */
void barectf_hsa_api_trace_hsa_amd_pointer_info_set_userdata_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__userdata)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_pointer_info_set_userdata_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_pointer_info_set_userdata_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__userdata);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_pointer_info_set_userdata_end`) */
void barectf_hsa_api_trace_hsa_amd_pointer_info_set_userdata_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_pointer_info_set_userdata_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_pointer_info_set_userdata_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_portable_close_dmabuf_begin`) */
void barectf_hsa_api_trace_hsa_amd_portable_close_dmabuf_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__dmabuf)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_portable_close_dmabuf_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_portable_close_dmabuf_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__dmabuf);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_portable_close_dmabuf_end`) */
void barectf_hsa_api_trace_hsa_amd_portable_close_dmabuf_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_portable_close_dmabuf_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_portable_close_dmabuf_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_portable_export_dmabuf_begin`) */
void barectf_hsa_api_trace_hsa_amd_portable_export_dmabuf_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__dmabuf,
	const uint64_t p__offset)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_portable_export_dmabuf_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_portable_export_dmabuf_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__size, p__dmabuf, p__offset);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_portable_export_dmabuf_end`) */
void barectf_hsa_api_trace_hsa_amd_portable_export_dmabuf_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_portable_export_dmabuf_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_portable_export_dmabuf_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_profiling_async_copy_enable_begin`) */
void barectf_hsa_api_trace_hsa_amd_profiling_async_copy_enable_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__enable)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_profiling_async_copy_enable_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_profiling_async_copy_enable_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__enable);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_profiling_async_copy_enable_end`) */
void barectf_hsa_api_trace_hsa_amd_profiling_async_copy_enable_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_profiling_async_copy_enable_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_profiling_async_copy_enable_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_profiling_convert_tick_to_system_domain_begin`) */
void barectf_hsa_api_trace_hsa_amd_profiling_convert_tick_to_system_domain_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__agent_tick,
	const uint64_t p__system_tick)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_profiling_convert_tick_to_system_domain_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_profiling_convert_tick_to_system_domain_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__agent_tick, p__system_tick);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_profiling_convert_tick_to_system_domain_end`) */
void barectf_hsa_api_trace_hsa_amd_profiling_convert_tick_to_system_domain_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_profiling_convert_tick_to_system_domain_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_profiling_convert_tick_to_system_domain_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_profiling_get_async_copy_time_begin`) */
void barectf_hsa_api_trace_hsa_amd_profiling_get_async_copy_time_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const uint64_t p__time)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_profiling_get_async_copy_time_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_profiling_get_async_copy_time_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__time);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_profiling_get_async_copy_time_end`) */
void barectf_hsa_api_trace_hsa_amd_profiling_get_async_copy_time_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_profiling_get_async_copy_time_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_profiling_get_async_copy_time_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_profiling_get_dispatch_time_begin`) */
void barectf_hsa_api_trace_hsa_amd_profiling_get_dispatch_time_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__signal__handle,
	const uint64_t p__time)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_profiling_get_dispatch_time_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_profiling_get_dispatch_time_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__signal__handle, p__time);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_profiling_get_dispatch_time_end`) */
void barectf_hsa_api_trace_hsa_amd_profiling_get_dispatch_time_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_profiling_get_dispatch_time_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_profiling_get_dispatch_time_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_profiling_set_profiler_enabled_begin`) */
void barectf_hsa_api_trace_hsa_amd_profiling_set_profiler_enabled_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const int32_t p__enable)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_profiling_set_profiler_enabled_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_profiling_set_profiler_enabled_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__enable);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_profiling_set_profiler_enabled_end`) */
void barectf_hsa_api_trace_hsa_amd_profiling_set_profiler_enabled_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_profiling_set_profiler_enabled_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_profiling_set_profiler_enabled_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_queue_cu_get_mask_begin`) */
void barectf_hsa_api_trace_hsa_amd_queue_cu_get_mask_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint32_t p__num_cu_mask_count,
	const uint64_t p__cu_mask)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_queue_cu_get_mask_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_queue_cu_get_mask_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__num_cu_mask_count, p__cu_mask);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_queue_cu_get_mask_end`) */
void barectf_hsa_api_trace_hsa_amd_queue_cu_get_mask_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_queue_cu_get_mask_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_queue_cu_get_mask_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_queue_cu_set_mask_begin`) */
void barectf_hsa_api_trace_hsa_amd_queue_cu_set_mask_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint32_t p__num_cu_mask_count,
	const uint64_t p__cu_mask)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_queue_cu_set_mask_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_queue_cu_set_mask_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__num_cu_mask_count, p__cu_mask);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_queue_cu_set_mask_end`) */
void barectf_hsa_api_trace_hsa_amd_queue_cu_set_mask_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_queue_cu_set_mask_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_queue_cu_set_mask_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_queue_get_info_begin`) */
void barectf_hsa_api_trace_hsa_amd_queue_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_queue_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_queue_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__attribute, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_queue_get_info_end`) */
void barectf_hsa_api_trace_hsa_amd_queue_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_queue_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_queue_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_queue_intercept_create_begin`) */
void barectf_hsa_api_trace_hsa_amd_queue_intercept_create_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent_handle__handle,
	const uint32_t p__size,
	const uint32_t p__type,
	const uint64_t p__callback,
	const uint64_t p__data,
	const uint32_t p__private_segment_size,
	const uint32_t p__group_segment_size,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_queue_intercept_create_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_queue_intercept_create_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent_handle__handle, p__size, p__type, p__callback, p__data, p__private_segment_size, p__group_segment_size, p__queue);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_queue_intercept_create_end`) */
void barectf_hsa_api_trace_hsa_amd_queue_intercept_create_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_queue_intercept_create_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_queue_intercept_create_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_queue_intercept_register_begin`) */
void barectf_hsa_api_trace_hsa_amd_queue_intercept_register_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__callback,
	const uint64_t p__user_data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_queue_intercept_register_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_queue_intercept_register_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__callback, p__user_data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_queue_intercept_register_end`) */
void barectf_hsa_api_trace_hsa_amd_queue_intercept_register_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_queue_intercept_register_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_queue_intercept_register_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_queue_set_priority_begin`) */
void barectf_hsa_api_trace_hsa_amd_queue_set_priority_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const int32_t p__priority)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_queue_set_priority_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_queue_set_priority_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__priority);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_queue_set_priority_end`) */
void barectf_hsa_api_trace_hsa_amd_queue_set_priority_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_queue_set_priority_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_queue_set_priority_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_register_deallocation_callback_begin`) */
void barectf_hsa_api_trace_hsa_amd_register_deallocation_callback_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__callback,
	const uint64_t p__user_data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_register_deallocation_callback_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_register_deallocation_callback_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__callback, p__user_data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_register_deallocation_callback_end`) */
void barectf_hsa_api_trace_hsa_amd_register_deallocation_callback_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_register_deallocation_callback_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_register_deallocation_callback_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_register_system_event_handler_begin`) */
void barectf_hsa_api_trace_hsa_amd_register_system_event_handler_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_register_system_event_handler_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_register_system_event_handler_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__callback, p__data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_register_system_event_handler_end`) */
void barectf_hsa_api_trace_hsa_amd_register_system_event_handler_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_register_system_event_handler_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_register_system_event_handler_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_runtime_queue_create_register_begin`) */
void barectf_hsa_api_trace_hsa_amd_runtime_queue_create_register_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__callback,
	const uint64_t p__user_data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_runtime_queue_create_register_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_runtime_queue_create_register_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__callback, p__user_data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_runtime_queue_create_register_end`) */
void barectf_hsa_api_trace_hsa_amd_runtime_queue_create_register_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_runtime_queue_create_register_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_runtime_queue_create_register_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_signal_async_handler_begin`) */
void barectf_hsa_api_trace_hsa_amd_signal_async_handler_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int32_t p__cond,
	const int64_t p__value,
	const uint64_t p__handler,
	const uint64_t p__arg)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_signal_async_handler_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_signal_async_handler_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__cond, p__value, p__handler, p__arg);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_signal_async_handler_end`) */
void barectf_hsa_api_trace_hsa_amd_signal_async_handler_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_signal_async_handler_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_signal_async_handler_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_signal_create_begin`) */
void barectf_hsa_api_trace_hsa_amd_signal_create_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int64_t p__initial_value,
	const uint32_t p__num_consumers,
	const uint64_t p__consumers,
	const uint64_t p__attributes,
	const uint64_t p__signal)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_signal_create_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_signal_create_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__initial_value, p__num_consumers, p__consumers, p__attributes, p__signal);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_signal_create_end`) */
void barectf_hsa_api_trace_hsa_amd_signal_create_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_signal_create_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_signal_create_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_signal_value_pointer_begin`) */
void barectf_hsa_api_trace_hsa_amd_signal_value_pointer_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const uint64_t p__value_ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_signal_value_pointer_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_signal_value_pointer_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value_ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_signal_value_pointer_end`) */
void barectf_hsa_api_trace_hsa_amd_signal_value_pointer_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_signal_value_pointer_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_signal_value_pointer_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_signal_wait_any_begin`) */
void barectf_hsa_api_trace_hsa_amd_signal_wait_any_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint32_t p__signal_count,
	const uint64_t p__signals,
	const uint64_t p__conds,
	const uint64_t p__values,
	const uint64_t p__timeout_hint,
	const int32_t p__wait_hint,
	const uint64_t p__satisfying_value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_signal_wait_any_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_signal_wait_any_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal_count, p__signals, p__conds, p__values, p__timeout_hint, p__wait_hint, p__satisfying_value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_signal_wait_any_end`) */
void barectf_hsa_api_trace_hsa_amd_signal_wait_any_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_signal_wait_any_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_signal_wait_any_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_spm_acquire_begin`) */
void barectf_hsa_api_trace_hsa_amd_spm_acquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__preferred_agent__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_spm_acquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_spm_acquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__preferred_agent__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_spm_acquire_end`) */
void barectf_hsa_api_trace_hsa_amd_spm_acquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_spm_acquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_spm_acquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_spm_release_begin`) */
void barectf_hsa_api_trace_hsa_amd_spm_release_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__preferred_agent__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_spm_release_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_spm_release_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__preferred_agent__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_spm_release_end`) */
void barectf_hsa_api_trace_hsa_amd_spm_release_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_spm_release_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_spm_release_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_spm_set_dest_buffer_begin`) */
void barectf_hsa_api_trace_hsa_amd_spm_set_dest_buffer_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__preferred_agent__handle,
	const uint64_t p__size_in_bytes,
	const uint64_t p__timeout,
	const uint64_t p__size_copied,
	const uint64_t p__dest,
	const uint64_t p__is_data_loss)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_spm_set_dest_buffer_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_spm_set_dest_buffer_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__preferred_agent__handle, p__size_in_bytes, p__timeout, p__size_copied, p__dest, p__is_data_loss);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_spm_set_dest_buffer_end`) */
void barectf_hsa_api_trace_hsa_amd_spm_set_dest_buffer_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_spm_set_dest_buffer_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_spm_set_dest_buffer_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_svm_attributes_get_begin`) */
void barectf_hsa_api_trace_hsa_amd_svm_attributes_get_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__attribute_list,
	const uint64_t p__attribute_count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_svm_attributes_get_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_svm_attributes_get_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__size, p__attribute_list, p__attribute_count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_svm_attributes_get_end`) */
void barectf_hsa_api_trace_hsa_amd_svm_attributes_get_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_svm_attributes_get_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_svm_attributes_get_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_svm_attributes_set_begin`) */
void barectf_hsa_api_trace_hsa_amd_svm_attributes_set_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__attribute_list,
	const uint64_t p__attribute_count)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_svm_attributes_set_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_svm_attributes_set_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__size, p__attribute_list, p__attribute_count);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_svm_attributes_set_end`) */
void barectf_hsa_api_trace_hsa_amd_svm_attributes_set_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_svm_attributes_set_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_svm_attributes_set_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_svm_prefetch_async_begin`) */
void barectf_hsa_api_trace_hsa_amd_svm_prefetch_async_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__size,
	const uint64_t p__agent__handle,
	const uint32_t p__num_dep_signals,
	const uint64_t p__dep_signals,
	const uint64_t p__completion_signal__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_svm_prefetch_async_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_svm_prefetch_async_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__size, p__agent__handle, p__num_dep_signals, p__dep_signals, p__completion_signal__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_svm_prefetch_async_end`) */
void barectf_hsa_api_trace_hsa_amd_svm_prefetch_async_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_svm_prefetch_async_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_svm_prefetch_async_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_address_free_begin`) */
void barectf_hsa_api_trace_hsa_amd_vmem_address_free_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_address_free_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_address_free_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__va, p__size);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_address_free_end`) */
void barectf_hsa_api_trace_hsa_amd_vmem_address_free_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_address_free_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_address_free_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_address_reserve_align_begin`) */
void barectf_hsa_api_trace_hsa_amd_vmem_address_reserve_align_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__size,
	const uint64_t p__address,
	const uint64_t p__alignment,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_address_reserve_align_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_address_reserve_align_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__va, p__size, p__address, p__alignment, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_address_reserve_align_end`) */
void barectf_hsa_api_trace_hsa_amd_vmem_address_reserve_align_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_address_reserve_align_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_address_reserve_align_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_address_reserve_begin`) */
void barectf_hsa_api_trace_hsa_amd_vmem_address_reserve_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__size,
	const uint64_t p__address,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_address_reserve_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_address_reserve_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__va, p__size, p__address, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_address_reserve_end`) */
void barectf_hsa_api_trace_hsa_amd_vmem_address_reserve_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_address_reserve_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_address_reserve_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_export_shareable_handle_begin`) */
void barectf_hsa_api_trace_hsa_amd_vmem_export_shareable_handle_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__dmabuf_fd,
	const uint64_t p__handle__handle,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_export_shareable_handle_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_export_shareable_handle_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__dmabuf_fd, p__handle__handle, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_export_shareable_handle_end`) */
void barectf_hsa_api_trace_hsa_amd_vmem_export_shareable_handle_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_export_shareable_handle_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_export_shareable_handle_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_get_access_begin`) */
void barectf_hsa_api_trace_hsa_amd_vmem_get_access_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__perms,
	const uint64_t p__agent_handle__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_get_access_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_get_access_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__va, p__perms, p__agent_handle__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_get_access_end`) */
void barectf_hsa_api_trace_hsa_amd_vmem_get_access_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_get_access_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_get_access_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_get_alloc_properties_from_handle_begin`) */
void barectf_hsa_api_trace_hsa_amd_vmem_get_alloc_properties_from_handle_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__memory_handle__handle,
	const uint64_t p__pool,
	const uint64_t p__type)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_get_alloc_properties_from_handle_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_get_alloc_properties_from_handle_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__memory_handle__handle, p__pool, p__type);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_get_alloc_properties_from_handle_end`) */
void barectf_hsa_api_trace_hsa_amd_vmem_get_alloc_properties_from_handle_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_get_alloc_properties_from_handle_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_get_alloc_properties_from_handle_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_handle_create_begin`) */
void barectf_hsa_api_trace_hsa_amd_vmem_handle_create_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__pool__handle,
	const uint64_t p__size,
	const int32_t p__type,
	const uint64_t p__flags,
	const uint64_t p__memory_handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_handle_create_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_handle_create_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__pool__handle, p__size, p__type, p__flags, p__memory_handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_handle_create_end`) */
void barectf_hsa_api_trace_hsa_amd_vmem_handle_create_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_handle_create_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_handle_create_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_handle_release_begin`) */
void barectf_hsa_api_trace_hsa_amd_vmem_handle_release_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__memory_handle__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_handle_release_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_handle_release_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__memory_handle__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_handle_release_end`) */
void barectf_hsa_api_trace_hsa_amd_vmem_handle_release_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_handle_release_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_handle_release_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_import_shareable_handle_begin`) */
void barectf_hsa_api_trace_hsa_amd_vmem_import_shareable_handle_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__dmabuf_fd,
	const uint64_t p__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_import_shareable_handle_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_import_shareable_handle_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__dmabuf_fd, p__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_import_shareable_handle_end`) */
void barectf_hsa_api_trace_hsa_amd_vmem_import_shareable_handle_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_import_shareable_handle_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_import_shareable_handle_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_map_begin`) */
void barectf_hsa_api_trace_hsa_amd_vmem_map_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__size,
	const uint64_t p__in_offset,
	const uint64_t p__memory_handle__handle,
	const uint64_t p__flags)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_map_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_map_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__va, p__size, p__in_offset, p__memory_handle__handle, p__flags);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_map_end`) */
void barectf_hsa_api_trace_hsa_amd_vmem_map_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_map_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_map_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_retain_alloc_handle_begin`) */
void barectf_hsa_api_trace_hsa_amd_vmem_retain_alloc_handle_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__memory_handle,
	const uint64_t p__addr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_retain_alloc_handle_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_retain_alloc_handle_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__memory_handle, p__addr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_retain_alloc_handle_end`) */
void barectf_hsa_api_trace_hsa_amd_vmem_retain_alloc_handle_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_retain_alloc_handle_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_retain_alloc_handle_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_set_access_begin`) */
void barectf_hsa_api_trace_hsa_amd_vmem_set_access_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__size,
	const uint64_t p__desc,
	const uint64_t p__desc_cnt)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_set_access_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_set_access_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__va, p__size, p__desc, p__desc_cnt);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_set_access_end`) */
void barectf_hsa_api_trace_hsa_amd_vmem_set_access_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_set_access_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_set_access_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_unmap_begin`) */
void barectf_hsa_api_trace_hsa_amd_vmem_unmap_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__va,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_unmap_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_unmap_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__va, p__size);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_amd_vmem_unmap_end`) */
void barectf_hsa_api_trace_hsa_amd_vmem_unmap_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_amd_vmem_unmap_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_amd_vmem_unmap_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_cache_get_info_begin`) */
void barectf_hsa_api_trace_hsa_cache_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__cache__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_cache_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_cache_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__cache__handle, p__attribute, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_cache_get_info_end`) */
void barectf_hsa_api_trace_hsa_cache_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_cache_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_cache_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_deserialize_begin`) */
void barectf_hsa_api_trace_hsa_code_object_deserialize_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__serialized_code_object,
	const uint64_t p__serialized_code_object_size,
	const char * const p__options,
	const uint64_t p__code_object)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_deserialize_begin(_TO_VOID_PTR(ctx), p__options);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_deserialize_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__serialized_code_object, p__serialized_code_object_size, p__options, p__code_object);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_deserialize_end`) */
void barectf_hsa_api_trace_hsa_code_object_deserialize_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_deserialize_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_deserialize_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_destroy_begin`) */
void barectf_hsa_api_trace_hsa_code_object_destroy_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_destroy_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_destroy_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__code_object__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_destroy_end`) */
void barectf_hsa_api_trace_hsa_code_object_destroy_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_destroy_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_destroy_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_get_info_begin`) */
void barectf_hsa_api_trace_hsa_code_object_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__code_object__handle, p__attribute, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_get_info_end`) */
void barectf_hsa_api_trace_hsa_code_object_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_get_symbol_begin`) */
void barectf_hsa_api_trace_hsa_code_object_get_symbol_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object__handle,
	const char * const p__symbol_name,
	const uint64_t p__symbol)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_get_symbol_begin(_TO_VOID_PTR(ctx), p__symbol_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_get_symbol_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__code_object__handle, p__symbol_name, p__symbol);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_get_symbol_end`) */
void barectf_hsa_api_trace_hsa_code_object_get_symbol_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_get_symbol_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_get_symbol_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_get_symbol_from_name_begin`) */
void barectf_hsa_api_trace_hsa_code_object_get_symbol_from_name_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object__handle,
	const char * const p__module_name,
	const char * const p__symbol_name,
	const uint64_t p__symbol)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_get_symbol_from_name_begin(_TO_VOID_PTR(ctx), p__module_name, p__symbol_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_get_symbol_from_name_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__code_object__handle, p__module_name, p__symbol_name, p__symbol);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_get_symbol_from_name_end`) */
void barectf_hsa_api_trace_hsa_code_object_get_symbol_from_name_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_get_symbol_from_name_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_get_symbol_from_name_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_iterate_symbols_begin`) */
void barectf_hsa_api_trace_hsa_code_object_iterate_symbols_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_iterate_symbols_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_iterate_symbols_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__code_object__handle, p__callback, p__data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_iterate_symbols_end`) */
void barectf_hsa_api_trace_hsa_code_object_iterate_symbols_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_iterate_symbols_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_iterate_symbols_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_reader_create_from_file_begin`) */
void barectf_hsa_api_trace_hsa_code_object_reader_create_from_file_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__file,
	const uint64_t p__code_object_reader)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_reader_create_from_file_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_reader_create_from_file_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__file, p__code_object_reader);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_reader_create_from_file_end`) */
void barectf_hsa_api_trace_hsa_code_object_reader_create_from_file_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_reader_create_from_file_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_reader_create_from_file_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_reader_create_from_memory_begin`) */
void barectf_hsa_api_trace_hsa_code_object_reader_create_from_memory_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object,
	const uint64_t p__size,
	const uint64_t p__code_object_reader)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_reader_create_from_memory_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_reader_create_from_memory_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__code_object, p__size, p__code_object_reader);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_reader_create_from_memory_end`) */
void barectf_hsa_api_trace_hsa_code_object_reader_create_from_memory_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_reader_create_from_memory_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_reader_create_from_memory_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_reader_destroy_begin`) */
void barectf_hsa_api_trace_hsa_code_object_reader_destroy_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object_reader__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_reader_destroy_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_reader_destroy_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__code_object_reader__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_reader_destroy_end`) */
void barectf_hsa_api_trace_hsa_code_object_reader_destroy_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_reader_destroy_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_reader_destroy_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_serialize_begin`) */
void barectf_hsa_api_trace_hsa_code_object_serialize_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object__handle,
	const uint64_t p__alloc_callback,
	const uint64_t p__callback_data__handle,
	const char * const p__options,
	const uint64_t p__serialized_code_object,
	const uint64_t p__serialized_code_object_size)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_serialize_begin(_TO_VOID_PTR(ctx), p__options);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_serialize_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__code_object__handle, p__alloc_callback, p__callback_data__handle, p__options, p__serialized_code_object, p__serialized_code_object_size);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_object_serialize_end`) */
void barectf_hsa_api_trace_hsa_code_object_serialize_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_object_serialize_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_object_serialize_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_symbol_get_info_begin`) */
void barectf_hsa_api_trace_hsa_code_symbol_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_symbol__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_symbol_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_symbol_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__code_symbol__handle, p__attribute, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_code_symbol_get_info_end`) */
void barectf_hsa_api_trace_hsa_code_symbol_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_code_symbol_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_code_symbol_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_agent_global_variable_define_begin`) */
void barectf_hsa_api_trace_hsa_executable_agent_global_variable_define_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__agent__handle,
	const char * const p__variable_name,
	const uint64_t p__address)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_agent_global_variable_define_begin(_TO_VOID_PTR(ctx), p__variable_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_agent_global_variable_define_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__agent__handle, p__variable_name, p__address);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_agent_global_variable_define_end`) */
void barectf_hsa_api_trace_hsa_executable_agent_global_variable_define_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_agent_global_variable_define_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_agent_global_variable_define_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_create_alt_begin`) */
void barectf_hsa_api_trace_hsa_executable_create_alt_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__profile,
	const int32_t p__default_float_rounding_mode,
	const char * const p__options,
	const uint64_t p__executable)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_create_alt_begin(_TO_VOID_PTR(ctx), p__options);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_create_alt_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__profile, p__default_float_rounding_mode, p__options, p__executable);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_create_alt_end`) */
void barectf_hsa_api_trace_hsa_executable_create_alt_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_create_alt_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_create_alt_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_create_begin`) */
void barectf_hsa_api_trace_hsa_executable_create_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__profile,
	const int32_t p__executable_state,
	const char * const p__options,
	const uint64_t p__executable)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_create_begin(_TO_VOID_PTR(ctx), p__options);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_create_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__profile, p__executable_state, p__options, p__executable);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_create_end`) */
void barectf_hsa_api_trace_hsa_executable_create_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_create_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_create_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_destroy_begin`) */
void barectf_hsa_api_trace_hsa_executable_destroy_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_destroy_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_destroy_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_destroy_end`) */
void barectf_hsa_api_trace_hsa_executable_destroy_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_destroy_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_destroy_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_freeze_begin`) */
void barectf_hsa_api_trace_hsa_executable_freeze_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const char * const p__options)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_freeze_begin(_TO_VOID_PTR(ctx), p__options);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_freeze_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__options);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_freeze_end`) */
void barectf_hsa_api_trace_hsa_executable_freeze_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_freeze_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_freeze_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_get_info_begin`) */
void barectf_hsa_api_trace_hsa_executable_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__attribute, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_get_info_end`) */
void barectf_hsa_api_trace_hsa_executable_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_get_symbol_begin`) */
void barectf_hsa_api_trace_hsa_executable_get_symbol_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const char * const p__module_name,
	const char * const p__symbol_name,
	const uint64_t p__agent__handle,
	const int32_t p__call_convention,
	const uint64_t p__symbol)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_get_symbol_begin(_TO_VOID_PTR(ctx), p__module_name, p__symbol_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_get_symbol_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__module_name, p__symbol_name, p__agent__handle, p__call_convention, p__symbol);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_get_symbol_by_name_begin`) */
void barectf_hsa_api_trace_hsa_executable_get_symbol_by_name_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const char * const p__symbol_name,
	const uint64_t p__agent,
	const uint64_t p__symbol)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_get_symbol_by_name_begin(_TO_VOID_PTR(ctx), p__symbol_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_get_symbol_by_name_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__symbol_name, p__agent, p__symbol);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_get_symbol_by_name_end`) */
void barectf_hsa_api_trace_hsa_executable_get_symbol_by_name_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_get_symbol_by_name_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_get_symbol_by_name_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_get_symbol_end`) */
void barectf_hsa_api_trace_hsa_executable_get_symbol_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_get_symbol_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_get_symbol_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_global_variable_define_begin`) */
void barectf_hsa_api_trace_hsa_executable_global_variable_define_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const char * const p__variable_name,
	const uint64_t p__address)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_global_variable_define_begin(_TO_VOID_PTR(ctx), p__variable_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_global_variable_define_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__variable_name, p__address);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_global_variable_define_end`) */
void barectf_hsa_api_trace_hsa_executable_global_variable_define_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_global_variable_define_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_global_variable_define_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_iterate_agent_symbols_begin`) */
void barectf_hsa_api_trace_hsa_executable_iterate_agent_symbols_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__agent__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_iterate_agent_symbols_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_iterate_agent_symbols_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__agent__handle, p__callback, p__data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_iterate_agent_symbols_end`) */
void barectf_hsa_api_trace_hsa_executable_iterate_agent_symbols_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_iterate_agent_symbols_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_iterate_agent_symbols_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_iterate_program_symbols_begin`) */
void barectf_hsa_api_trace_hsa_executable_iterate_program_symbols_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_iterate_program_symbols_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_iterate_program_symbols_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__callback, p__data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_iterate_program_symbols_end`) */
void barectf_hsa_api_trace_hsa_executable_iterate_program_symbols_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_iterate_program_symbols_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_iterate_program_symbols_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_iterate_symbols_begin`) */
void barectf_hsa_api_trace_hsa_executable_iterate_symbols_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_iterate_symbols_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_iterate_symbols_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__callback, p__data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_iterate_symbols_end`) */
void barectf_hsa_api_trace_hsa_executable_iterate_symbols_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_iterate_symbols_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_iterate_symbols_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_load_agent_code_object_begin`) */
void barectf_hsa_api_trace_hsa_executable_load_agent_code_object_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__agent__handle,
	const uint64_t p__code_object_reader__handle,
	const char * const p__options,
	const uint64_t p__loaded_code_object)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_load_agent_code_object_begin(_TO_VOID_PTR(ctx), p__options);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_load_agent_code_object_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__agent__handle, p__code_object_reader__handle, p__options, p__loaded_code_object);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_load_agent_code_object_end`) */
void barectf_hsa_api_trace_hsa_executable_load_agent_code_object_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_load_agent_code_object_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_load_agent_code_object_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_load_code_object_begin`) */
void barectf_hsa_api_trace_hsa_executable_load_code_object_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__agent__handle,
	const uint64_t p__code_object__handle,
	const char * const p__options)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_load_code_object_begin(_TO_VOID_PTR(ctx), p__options);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_load_code_object_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__agent__handle, p__code_object__handle, p__options);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_load_code_object_end`) */
void barectf_hsa_api_trace_hsa_executable_load_code_object_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_load_code_object_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_load_code_object_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_load_program_code_object_begin`) */
void barectf_hsa_api_trace_hsa_executable_load_program_code_object_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__code_object_reader__handle,
	const char * const p__options,
	const uint64_t p__loaded_code_object)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_load_program_code_object_begin(_TO_VOID_PTR(ctx), p__options);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_load_program_code_object_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__code_object_reader__handle, p__options, p__loaded_code_object);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_load_program_code_object_end`) */
void barectf_hsa_api_trace_hsa_executable_load_program_code_object_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_load_program_code_object_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_load_program_code_object_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_readonly_variable_define_begin`) */
void barectf_hsa_api_trace_hsa_executable_readonly_variable_define_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__agent__handle,
	const char * const p__variable_name,
	const uint64_t p__address)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_readonly_variable_define_begin(_TO_VOID_PTR(ctx), p__variable_name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_readonly_variable_define_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__agent__handle, p__variable_name, p__address);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_readonly_variable_define_end`) */
void barectf_hsa_api_trace_hsa_executable_readonly_variable_define_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_readonly_variable_define_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_readonly_variable_define_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_symbol_get_info_begin`) */
void barectf_hsa_api_trace_hsa_executable_symbol_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable_symbol__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_symbol_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_symbol_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable_symbol__handle, p__attribute, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_symbol_get_info_end`) */
void barectf_hsa_api_trace_hsa_executable_symbol_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_symbol_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_symbol_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_validate_alt_begin`) */
void barectf_hsa_api_trace_hsa_executable_validate_alt_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const char * const p__options,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_validate_alt_begin(_TO_VOID_PTR(ctx), p__options);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_validate_alt_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__options, p__result);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_validate_alt_end`) */
void barectf_hsa_api_trace_hsa_executable_validate_alt_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_validate_alt_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_validate_alt_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_validate_begin`) */
void barectf_hsa_api_trace_hsa_executable_validate_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__executable__handle,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_validate_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_validate_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__executable__handle, p__result);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_executable_validate_end`) */
void barectf_hsa_api_trace_hsa_executable_validate_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_executable_validate_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_executable_validate_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_clear_begin`) */
void barectf_hsa_api_trace_hsa_ext_image_clear_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image__handle,
	const uint64_t p__data,
	const uint64_t p__image_region)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_clear_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_clear_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__image__handle, p__data, p__image_region);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_clear_end`) */
void barectf_hsa_api_trace_hsa_ext_image_clear_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_clear_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_clear_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_copy_begin`) */
void barectf_hsa_api_trace_hsa_ext_image_copy_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__src_image__handle,
	const uint64_t p__src_offset,
	const uint64_t p__dst_image__handle,
	const uint64_t p__dst_offset,
	const uint64_t p__range)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_copy_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_copy_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__src_image__handle, p__src_offset, p__dst_image__handle, p__dst_offset, p__range);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_copy_end`) */
void barectf_hsa_api_trace_hsa_ext_image_copy_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_copy_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_copy_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_create_begin`) */
void barectf_hsa_api_trace_hsa_ext_image_create_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image_descriptor,
	const uint64_t p__image_data,
	const int32_t p__access_permission,
	const uint64_t p__image)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_create_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_create_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__image_descriptor, p__image_data, p__access_permission, p__image);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_create_end`) */
void barectf_hsa_api_trace_hsa_ext_image_create_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_create_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_create_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_create_with_layout_begin`) */
void barectf_hsa_api_trace_hsa_ext_image_create_with_layout_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image_descriptor,
	const uint64_t p__image_data,
	const int32_t p__access_permission,
	const int32_t p__image_data_layout,
	const uint64_t p__image_data_row_pitch,
	const uint64_t p__image_data_slice_pitch,
	const uint64_t p__image)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_create_with_layout_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_create_with_layout_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__image_descriptor, p__image_data, p__access_permission, p__image_data_layout, p__image_data_row_pitch, p__image_data_slice_pitch, p__image);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_create_with_layout_end`) */
void barectf_hsa_api_trace_hsa_ext_image_create_with_layout_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_create_with_layout_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_create_with_layout_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_data_get_info_begin`) */
void barectf_hsa_api_trace_hsa_ext_image_data_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image_descriptor,
	const int32_t p__access_permission,
	const uint64_t p__image_data_info)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_data_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_data_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__image_descriptor, p__access_permission, p__image_data_info);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_data_get_info_end`) */
void barectf_hsa_api_trace_hsa_ext_image_data_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_data_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_data_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_data_get_info_with_layout_begin`) */
void barectf_hsa_api_trace_hsa_ext_image_data_get_info_with_layout_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image_descriptor,
	const int32_t p__access_permission,
	const int32_t p__image_data_layout,
	const uint64_t p__image_data_row_pitch,
	const uint64_t p__image_data_slice_pitch,
	const uint64_t p__image_data_info)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_data_get_info_with_layout_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_data_get_info_with_layout_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__image_descriptor, p__access_permission, p__image_data_layout, p__image_data_row_pitch, p__image_data_slice_pitch, p__image_data_info);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_data_get_info_with_layout_end`) */
void barectf_hsa_api_trace_hsa_ext_image_data_get_info_with_layout_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_data_get_info_with_layout_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_data_get_info_with_layout_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_destroy_begin`) */
void barectf_hsa_api_trace_hsa_ext_image_destroy_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__image__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_destroy_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_destroy_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__image__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_destroy_end`) */
void barectf_hsa_api_trace_hsa_ext_image_destroy_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_destroy_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_destroy_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_export_begin`) */
void barectf_hsa_api_trace_hsa_ext_image_export_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__src_image__handle,
	const uint64_t p__dst_memory,
	const uint64_t p__dst_row_pitch,
	const uint64_t p__dst_slice_pitch,
	const uint64_t p__image_region)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_export_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_export_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__src_image__handle, p__dst_memory, p__dst_row_pitch, p__dst_slice_pitch, p__image_region);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_export_end`) */
void barectf_hsa_api_trace_hsa_ext_image_export_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_export_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_export_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_get_capability_begin`) */
void barectf_hsa_api_trace_hsa_ext_image_get_capability_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const int32_t p__geometry,
	const uint64_t p__image_format,
	const uint64_t p__capability_mask)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_get_capability_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_get_capability_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__geometry, p__image_format, p__capability_mask);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_get_capability_end`) */
void barectf_hsa_api_trace_hsa_ext_image_get_capability_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_get_capability_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_get_capability_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_get_capability_with_layout_begin`) */
void barectf_hsa_api_trace_hsa_ext_image_get_capability_with_layout_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const int32_t p__geometry,
	const uint64_t p__image_format,
	const int32_t p__image_data_layout,
	const uint64_t p__capability_mask)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_get_capability_with_layout_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_get_capability_with_layout_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__geometry, p__image_format, p__image_data_layout, p__capability_mask);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_get_capability_with_layout_end`) */
void barectf_hsa_api_trace_hsa_ext_image_get_capability_with_layout_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_get_capability_with_layout_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_get_capability_with_layout_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_import_begin`) */
void barectf_hsa_api_trace_hsa_ext_image_import_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__src_memory,
	const uint64_t p__src_row_pitch,
	const uint64_t p__src_slice_pitch,
	const uint64_t p__dst_image__handle,
	const uint64_t p__image_region)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_import_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_import_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__src_memory, p__src_row_pitch, p__src_slice_pitch, p__dst_image__handle, p__image_region);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_image_import_end`) */
void barectf_hsa_api_trace_hsa_ext_image_import_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_image_import_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_image_import_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_sampler_create_begin`) */
void barectf_hsa_api_trace_hsa_ext_sampler_create_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__sampler_descriptor,
	const uint64_t p__sampler)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_sampler_create_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_sampler_create_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__sampler_descriptor, p__sampler);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_sampler_create_end`) */
void barectf_hsa_api_trace_hsa_ext_sampler_create_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_sampler_create_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_sampler_create_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_sampler_destroy_begin`) */
void barectf_hsa_api_trace_hsa_ext_sampler_destroy_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint64_t p__sampler__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_sampler_destroy_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_sampler_destroy_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__sampler__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_ext_sampler_destroy_end`) */
void barectf_hsa_api_trace_hsa_ext_sampler_destroy_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_ext_sampler_destroy_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_ext_sampler_destroy_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_extension_get_name_begin`) */
void barectf_hsa_api_trace_hsa_extension_get_name_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint64_t p__name)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_extension_get_name_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_extension_get_name_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__extension, p__name);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_extension_get_name_end`) */
void barectf_hsa_api_trace_hsa_extension_get_name_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_extension_get_name_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_extension_get_name_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_init_begin`) */
void barectf_hsa_api_trace_hsa_init_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_init_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_init_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_init_end`) */
void barectf_hsa_api_trace_hsa_init_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_init_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_init_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_compatible_begin`) */
void barectf_hsa_api_trace_hsa_isa_compatible_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__code_object_isa__handle,
	const uint64_t p__agent_isa__handle,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_compatible_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_compatible_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__code_object_isa__handle, p__agent_isa__handle, p__result);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_compatible_end`) */
void barectf_hsa_api_trace_hsa_isa_compatible_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_compatible_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_compatible_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_from_name_begin`) */
void barectf_hsa_api_trace_hsa_isa_from_name_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const char * const p__name,
	const uint64_t p__isa)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_from_name_begin(_TO_VOID_PTR(ctx), p__name);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_from_name_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__name, p__isa);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_from_name_end`) */
void barectf_hsa_api_trace_hsa_isa_from_name_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_from_name_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_from_name_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_get_exception_policies_begin`) */
void barectf_hsa_api_trace_hsa_isa_get_exception_policies_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__isa__handle,
	const int32_t p__profile,
	const uint64_t p__mask)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_get_exception_policies_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_get_exception_policies_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__isa__handle, p__profile, p__mask);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_get_exception_policies_end`) */
void barectf_hsa_api_trace_hsa_isa_get_exception_policies_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_get_exception_policies_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_get_exception_policies_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_get_info_alt_begin`) */
void barectf_hsa_api_trace_hsa_isa_get_info_alt_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__isa__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_get_info_alt_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_get_info_alt_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__isa__handle, p__attribute, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_get_info_alt_end`) */
void barectf_hsa_api_trace_hsa_isa_get_info_alt_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_get_info_alt_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_get_info_alt_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_get_info_begin`) */
void barectf_hsa_api_trace_hsa_isa_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__isa__handle,
	const int32_t p__attribute,
	const uint32_t p__index,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__isa__handle, p__attribute, p__index, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_get_info_end`) */
void barectf_hsa_api_trace_hsa_isa_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_get_round_method_begin`) */
void barectf_hsa_api_trace_hsa_isa_get_round_method_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__isa__handle,
	const int32_t p__fp_type,
	const int32_t p__flush_mode,
	const uint64_t p__round_method)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_get_round_method_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_get_round_method_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__isa__handle, p__fp_type, p__flush_mode, p__round_method);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_get_round_method_end`) */
void barectf_hsa_api_trace_hsa_isa_get_round_method_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_get_round_method_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_get_round_method_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_iterate_wavefronts_begin`) */
void barectf_hsa_api_trace_hsa_isa_iterate_wavefronts_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__isa__handle,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_iterate_wavefronts_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_iterate_wavefronts_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__isa__handle, p__callback, p__data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_isa_iterate_wavefronts_end`) */
void barectf_hsa_api_trace_hsa_isa_iterate_wavefronts_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_isa_iterate_wavefronts_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_isa_iterate_wavefronts_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_iterate_agents_begin`) */
void barectf_hsa_api_trace_hsa_iterate_agents_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__callback,
	const uint64_t p__data)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_iterate_agents_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_iterate_agents_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__callback, p__data);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_iterate_agents_end`) */
void barectf_hsa_api_trace_hsa_iterate_agents_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_iterate_agents_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_iterate_agents_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_memory_allocate_begin`) */
void barectf_hsa_api_trace_hsa_memory_allocate_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__region__handle,
	const uint64_t p__size,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_memory_allocate_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_memory_allocate_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__region__handle, p__size, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_memory_allocate_end`) */
void barectf_hsa_api_trace_hsa_memory_allocate_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_memory_allocate_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_memory_allocate_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_memory_assign_agent_begin`) */
void barectf_hsa_api_trace_hsa_memory_assign_agent_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__agent__handle,
	const int32_t p__access)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_memory_assign_agent_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_memory_assign_agent_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__agent__handle, p__access);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_memory_assign_agent_end`) */
void barectf_hsa_api_trace_hsa_memory_assign_agent_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_memory_assign_agent_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_memory_assign_agent_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_memory_copy_begin`) */
void barectf_hsa_api_trace_hsa_memory_copy_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__dst,
	const uint64_t p__src,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_memory_copy_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_memory_copy_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__dst, p__src, p__size);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_memory_copy_end`) */
void barectf_hsa_api_trace_hsa_memory_copy_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_memory_copy_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_memory_copy_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_memory_deregister_begin`) */
void barectf_hsa_api_trace_hsa_memory_deregister_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_memory_deregister_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_memory_deregister_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__size);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_memory_deregister_end`) */
void barectf_hsa_api_trace_hsa_memory_deregister_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_memory_deregister_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_memory_deregister_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_memory_free_begin`) */
void barectf_hsa_api_trace_hsa_memory_free_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_memory_free_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_memory_free_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_memory_free_end`) */
void barectf_hsa_api_trace_hsa_memory_free_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_memory_free_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_memory_free_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_memory_register_begin`) */
void barectf_hsa_api_trace_hsa_memory_register_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__ptr,
	const uint64_t p__size)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_memory_register_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_memory_register_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__ptr, p__size);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_memory_register_end`) */
void barectf_hsa_api_trace_hsa_memory_register_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_memory_register_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_memory_register_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_add_write_index_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_queue_add_write_index_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_add_write_index_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_add_write_index_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_add_write_index_relaxed_end`) */
void barectf_hsa_api_trace_hsa_queue_add_write_index_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_add_write_index_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_add_write_index_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_add_write_index_scacq_screl_begin`) */
void barectf_hsa_api_trace_hsa_queue_add_write_index_scacq_screl_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_add_write_index_scacq_screl_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_add_write_index_scacq_screl_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_add_write_index_scacq_screl_end`) */
void barectf_hsa_api_trace_hsa_queue_add_write_index_scacq_screl_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_add_write_index_scacq_screl_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_add_write_index_scacq_screl_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_add_write_index_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_queue_add_write_index_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_add_write_index_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_add_write_index_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_add_write_index_scacquire_end`) */
void barectf_hsa_api_trace_hsa_queue_add_write_index_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_add_write_index_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_add_write_index_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_add_write_index_screlease_begin`) */
void barectf_hsa_api_trace_hsa_queue_add_write_index_screlease_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_add_write_index_screlease_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_add_write_index_screlease_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_add_write_index_screlease_end`) */
void barectf_hsa_api_trace_hsa_queue_add_write_index_screlease_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_add_write_index_screlease_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_add_write_index_screlease_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_cas_write_index_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_queue_cas_write_index_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__expected,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_cas_write_index_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_cas_write_index_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__expected, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_cas_write_index_relaxed_end`) */
void barectf_hsa_api_trace_hsa_queue_cas_write_index_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_cas_write_index_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_cas_write_index_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_cas_write_index_scacq_screl_begin`) */
void barectf_hsa_api_trace_hsa_queue_cas_write_index_scacq_screl_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__expected,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_cas_write_index_scacq_screl_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_cas_write_index_scacq_screl_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__expected, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_cas_write_index_scacq_screl_end`) */
void barectf_hsa_api_trace_hsa_queue_cas_write_index_scacq_screl_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_cas_write_index_scacq_screl_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_cas_write_index_scacq_screl_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_cas_write_index_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_queue_cas_write_index_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__expected,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_cas_write_index_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_cas_write_index_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__expected, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_cas_write_index_scacquire_end`) */
void barectf_hsa_api_trace_hsa_queue_cas_write_index_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_cas_write_index_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_cas_write_index_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_cas_write_index_screlease_begin`) */
void barectf_hsa_api_trace_hsa_queue_cas_write_index_screlease_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__expected,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_cas_write_index_screlease_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_cas_write_index_screlease_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__expected, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_cas_write_index_screlease_end`) */
void barectf_hsa_api_trace_hsa_queue_cas_write_index_screlease_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_cas_write_index_screlease_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_cas_write_index_screlease_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_create_begin`) */
void barectf_hsa_api_trace_hsa_queue_create_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__agent__handle,
	const uint32_t p__size,
	const uint32_t p__type,
	const uint64_t p__callback,
	const uint64_t p__data,
	const uint32_t p__private_segment_size,
	const uint32_t p__group_segment_size,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_create_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_create_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__agent__handle, p__size, p__type, p__callback, p__data, p__private_segment_size, p__group_segment_size, p__queue);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_create_end`) */
void barectf_hsa_api_trace_hsa_queue_create_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_create_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_create_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_destroy_begin`) */
void barectf_hsa_api_trace_hsa_queue_destroy_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_destroy_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_destroy_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_destroy_end`) */
void barectf_hsa_api_trace_hsa_queue_destroy_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_destroy_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_destroy_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_inactivate_begin`) */
void barectf_hsa_api_trace_hsa_queue_inactivate_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_inactivate_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_inactivate_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_inactivate_end`) */
void barectf_hsa_api_trace_hsa_queue_inactivate_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_inactivate_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_inactivate_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_load_read_index_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_queue_load_read_index_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_load_read_index_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_load_read_index_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_load_read_index_relaxed_end`) */
void barectf_hsa_api_trace_hsa_queue_load_read_index_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_load_read_index_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_load_read_index_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_load_read_index_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_queue_load_read_index_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_load_read_index_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_load_read_index_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_load_read_index_scacquire_end`) */
void barectf_hsa_api_trace_hsa_queue_load_read_index_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_load_read_index_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_load_read_index_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_load_write_index_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_queue_load_write_index_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_load_write_index_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_load_write_index_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_load_write_index_relaxed_end`) */
void barectf_hsa_api_trace_hsa_queue_load_write_index_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_load_write_index_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_load_write_index_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_load_write_index_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_queue_load_write_index_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_load_write_index_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_load_write_index_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_load_write_index_scacquire_end`) */
void barectf_hsa_api_trace_hsa_queue_load_write_index_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_load_write_index_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_load_write_index_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_store_read_index_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_queue_store_read_index_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_store_read_index_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_store_read_index_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_store_read_index_relaxed_end`) */
void barectf_hsa_api_trace_hsa_queue_store_read_index_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_store_read_index_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_store_read_index_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_store_read_index_screlease_begin`) */
void barectf_hsa_api_trace_hsa_queue_store_read_index_screlease_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_store_read_index_screlease_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_store_read_index_screlease_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_store_read_index_screlease_end`) */
void barectf_hsa_api_trace_hsa_queue_store_read_index_screlease_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_store_read_index_screlease_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_store_read_index_screlease_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_store_write_index_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_queue_store_write_index_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_store_write_index_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_store_write_index_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_store_write_index_relaxed_end`) */
void barectf_hsa_api_trace_hsa_queue_store_write_index_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_store_write_index_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_store_write_index_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_store_write_index_screlease_begin`) */
void barectf_hsa_api_trace_hsa_queue_store_write_index_screlease_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__queue,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_store_write_index_screlease_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_store_write_index_screlease_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__queue, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_queue_store_write_index_screlease_end`) */
void barectf_hsa_api_trace_hsa_queue_store_write_index_screlease_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_queue_store_write_index_screlease_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_queue_store_write_index_screlease_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_region_get_info_begin`) */
void barectf_hsa_api_trace_hsa_region_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__region__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_region_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_region_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__region__handle, p__attribute, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_region_get_info_end`) */
void barectf_hsa_api_trace_hsa_region_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_region_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_region_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_shut_down_begin`) */
void barectf_hsa_api_trace_hsa_shut_down_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_shut_down_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_shut_down_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_shut_down_end`) */
void barectf_hsa_api_trace_hsa_shut_down_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_shut_down_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_shut_down_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_add_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_signal_add_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_add_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_add_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_add_relaxed_end`) */
void barectf_hsa_api_trace_hsa_signal_add_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_add_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_add_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_add_scacq_screl_begin`) */
void barectf_hsa_api_trace_hsa_signal_add_scacq_screl_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_add_scacq_screl_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_add_scacq_screl_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_add_scacq_screl_end`) */
void barectf_hsa_api_trace_hsa_signal_add_scacq_screl_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_add_scacq_screl_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_add_scacq_screl_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_add_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_signal_add_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_add_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_add_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_add_scacquire_end`) */
void barectf_hsa_api_trace_hsa_signal_add_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_add_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_add_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_add_screlease_begin`) */
void barectf_hsa_api_trace_hsa_signal_add_screlease_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_add_screlease_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_add_screlease_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_add_screlease_end`) */
void barectf_hsa_api_trace_hsa_signal_add_screlease_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_add_screlease_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_add_screlease_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_and_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_signal_and_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_and_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_and_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_and_relaxed_end`) */
void barectf_hsa_api_trace_hsa_signal_and_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_and_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_and_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_and_scacq_screl_begin`) */
void barectf_hsa_api_trace_hsa_signal_and_scacq_screl_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_and_scacq_screl_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_and_scacq_screl_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_and_scacq_screl_end`) */
void barectf_hsa_api_trace_hsa_signal_and_scacq_screl_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_and_scacq_screl_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_and_scacq_screl_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_and_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_signal_and_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_and_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_and_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_and_scacquire_end`) */
void barectf_hsa_api_trace_hsa_signal_and_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_and_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_and_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_and_screlease_begin`) */
void barectf_hsa_api_trace_hsa_signal_and_screlease_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_and_screlease_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_and_screlease_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_and_screlease_end`) */
void barectf_hsa_api_trace_hsa_signal_and_screlease_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_and_screlease_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_and_screlease_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_cas_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_signal_cas_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__expected,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_cas_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_cas_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__expected, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_cas_relaxed_end`) */
void barectf_hsa_api_trace_hsa_signal_cas_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_cas_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_cas_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_cas_scacq_screl_begin`) */
void barectf_hsa_api_trace_hsa_signal_cas_scacq_screl_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__expected,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_cas_scacq_screl_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_cas_scacq_screl_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__expected, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_cas_scacq_screl_end`) */
void barectf_hsa_api_trace_hsa_signal_cas_scacq_screl_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_cas_scacq_screl_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_cas_scacq_screl_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_cas_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_signal_cas_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__expected,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_cas_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_cas_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__expected, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_cas_scacquire_end`) */
void barectf_hsa_api_trace_hsa_signal_cas_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_cas_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_cas_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_cas_screlease_begin`) */
void barectf_hsa_api_trace_hsa_signal_cas_screlease_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__expected,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_cas_screlease_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_cas_screlease_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__expected, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_cas_screlease_end`) */
void barectf_hsa_api_trace_hsa_signal_cas_screlease_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_cas_screlease_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_cas_screlease_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_create_begin`) */
void barectf_hsa_api_trace_hsa_signal_create_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int64_t p__initial_value,
	const uint32_t p__num_consumers,
	const uint64_t p__consumers,
	const uint64_t p__signal)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_create_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_create_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__initial_value, p__num_consumers, p__consumers, p__signal);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_create_end`) */
void barectf_hsa_api_trace_hsa_signal_create_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_create_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_create_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_destroy_begin`) */
void barectf_hsa_api_trace_hsa_signal_destroy_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_destroy_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_destroy_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_destroy_end`) */
void barectf_hsa_api_trace_hsa_signal_destroy_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_destroy_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_destroy_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_exchange_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_signal_exchange_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_exchange_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_exchange_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_exchange_relaxed_end`) */
void barectf_hsa_api_trace_hsa_signal_exchange_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_exchange_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_exchange_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_exchange_scacq_screl_begin`) */
void barectf_hsa_api_trace_hsa_signal_exchange_scacq_screl_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_exchange_scacq_screl_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_exchange_scacq_screl_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_exchange_scacq_screl_end`) */
void barectf_hsa_api_trace_hsa_signal_exchange_scacq_screl_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_exchange_scacq_screl_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_exchange_scacq_screl_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_exchange_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_signal_exchange_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_exchange_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_exchange_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_exchange_scacquire_end`) */
void barectf_hsa_api_trace_hsa_signal_exchange_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_exchange_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_exchange_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_exchange_screlease_begin`) */
void barectf_hsa_api_trace_hsa_signal_exchange_screlease_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_exchange_screlease_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_exchange_screlease_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_exchange_screlease_end`) */
void barectf_hsa_api_trace_hsa_signal_exchange_screlease_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_exchange_screlease_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_exchange_screlease_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_group_create_begin`) */
void barectf_hsa_api_trace_hsa_signal_group_create_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint32_t p__num_signals,
	const uint64_t p__signals,
	const uint32_t p__num_consumers,
	const uint64_t p__consumers,
	const uint64_t p__signal_group)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_group_create_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_group_create_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__num_signals, p__signals, p__num_consumers, p__consumers, p__signal_group);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_group_create_end`) */
void barectf_hsa_api_trace_hsa_signal_group_create_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_group_create_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_group_create_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_group_destroy_begin`) */
void barectf_hsa_api_trace_hsa_signal_group_destroy_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal_group__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_group_destroy_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_group_destroy_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal_group__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_group_destroy_end`) */
void barectf_hsa_api_trace_hsa_signal_group_destroy_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_group_destroy_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_group_destroy_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_group_wait_any_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_signal_group_wait_any_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal_group__handle,
	const uint64_t p__conditions,
	const uint64_t p__compare_values,
	const int32_t p__wait_state_hint,
	const uint64_t p__signal,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_group_wait_any_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_group_wait_any_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal_group__handle, p__conditions, p__compare_values, p__wait_state_hint, p__signal, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_group_wait_any_relaxed_end`) */
void barectf_hsa_api_trace_hsa_signal_group_wait_any_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_group_wait_any_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_group_wait_any_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_group_wait_any_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_signal_group_wait_any_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal_group__handle,
	const uint64_t p__conditions,
	const uint64_t p__compare_values,
	const int32_t p__wait_state_hint,
	const uint64_t p__signal,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_group_wait_any_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_group_wait_any_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal_group__handle, p__conditions, p__compare_values, p__wait_state_hint, p__signal, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_group_wait_any_scacquire_end`) */
void barectf_hsa_api_trace_hsa_signal_group_wait_any_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_group_wait_any_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_group_wait_any_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_load_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_signal_load_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_load_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_load_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_load_relaxed_end`) */
void barectf_hsa_api_trace_hsa_signal_load_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_load_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_load_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_load_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_signal_load_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_load_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_load_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_load_scacquire_end`) */
void barectf_hsa_api_trace_hsa_signal_load_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_load_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_load_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_or_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_signal_or_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_or_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_or_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_or_relaxed_end`) */
void barectf_hsa_api_trace_hsa_signal_or_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_or_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_or_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_or_scacq_screl_begin`) */
void barectf_hsa_api_trace_hsa_signal_or_scacq_screl_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_or_scacq_screl_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_or_scacq_screl_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_or_scacq_screl_end`) */
void barectf_hsa_api_trace_hsa_signal_or_scacq_screl_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_or_scacq_screl_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_or_scacq_screl_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_or_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_signal_or_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_or_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_or_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_or_scacquire_end`) */
void barectf_hsa_api_trace_hsa_signal_or_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_or_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_or_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_or_screlease_begin`) */
void barectf_hsa_api_trace_hsa_signal_or_screlease_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_or_screlease_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_or_screlease_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_or_screlease_end`) */
void barectf_hsa_api_trace_hsa_signal_or_screlease_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_or_screlease_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_or_screlease_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_silent_store_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_signal_silent_store_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_silent_store_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_silent_store_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_silent_store_relaxed_end`) */
void barectf_hsa_api_trace_hsa_signal_silent_store_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_silent_store_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_silent_store_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_silent_store_screlease_begin`) */
void barectf_hsa_api_trace_hsa_signal_silent_store_screlease_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_silent_store_screlease_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_silent_store_screlease_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_silent_store_screlease_end`) */
void barectf_hsa_api_trace_hsa_signal_silent_store_screlease_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_silent_store_screlease_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_silent_store_screlease_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_store_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_signal_store_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_store_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_store_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_store_relaxed_end`) */
void barectf_hsa_api_trace_hsa_signal_store_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_store_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_store_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_store_screlease_begin`) */
void barectf_hsa_api_trace_hsa_signal_store_screlease_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_store_screlease_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_store_screlease_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_store_screlease_end`) */
void barectf_hsa_api_trace_hsa_signal_store_screlease_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_store_screlease_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_store_screlease_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_subtract_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_signal_subtract_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_subtract_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_subtract_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_subtract_relaxed_end`) */
void barectf_hsa_api_trace_hsa_signal_subtract_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_subtract_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_subtract_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_subtract_scacq_screl_begin`) */
void barectf_hsa_api_trace_hsa_signal_subtract_scacq_screl_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_subtract_scacq_screl_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_subtract_scacq_screl_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_subtract_scacq_screl_end`) */
void barectf_hsa_api_trace_hsa_signal_subtract_scacq_screl_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_subtract_scacq_screl_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_subtract_scacq_screl_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_subtract_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_signal_subtract_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_subtract_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_subtract_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_subtract_scacquire_end`) */
void barectf_hsa_api_trace_hsa_signal_subtract_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_subtract_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_subtract_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_subtract_screlease_begin`) */
void barectf_hsa_api_trace_hsa_signal_subtract_screlease_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_subtract_screlease_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_subtract_screlease_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_subtract_screlease_end`) */
void barectf_hsa_api_trace_hsa_signal_subtract_screlease_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_subtract_screlease_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_subtract_screlease_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_wait_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_signal_wait_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int32_t p__condition,
	const int64_t p__compare_value,
	const uint64_t p__timeout_hint,
	const int32_t p__wait_state_hint)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_wait_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_wait_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__condition, p__compare_value, p__timeout_hint, p__wait_state_hint);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_wait_relaxed_end`) */
void barectf_hsa_api_trace_hsa_signal_wait_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_wait_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_wait_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_wait_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_signal_wait_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int32_t p__condition,
	const int64_t p__compare_value,
	const uint64_t p__timeout_hint,
	const int32_t p__wait_state_hint)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_wait_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_wait_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__condition, p__compare_value, p__timeout_hint, p__wait_state_hint);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_wait_scacquire_end`) */
void barectf_hsa_api_trace_hsa_signal_wait_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_wait_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_wait_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_xor_relaxed_begin`) */
void barectf_hsa_api_trace_hsa_signal_xor_relaxed_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_xor_relaxed_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_xor_relaxed_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_xor_relaxed_end`) */
void barectf_hsa_api_trace_hsa_signal_xor_relaxed_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_xor_relaxed_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_xor_relaxed_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_xor_scacq_screl_begin`) */
void barectf_hsa_api_trace_hsa_signal_xor_scacq_screl_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_xor_scacq_screl_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_xor_scacq_screl_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_xor_scacq_screl_end`) */
void barectf_hsa_api_trace_hsa_signal_xor_scacq_screl_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_xor_scacq_screl_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_xor_scacq_screl_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_xor_scacquire_begin`) */
void barectf_hsa_api_trace_hsa_signal_xor_scacquire_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_xor_scacquire_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_xor_scacquire_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_xor_scacquire_end`) */
void barectf_hsa_api_trace_hsa_signal_xor_scacquire_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_xor_scacquire_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_xor_scacquire_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_xor_screlease_begin`) */
void barectf_hsa_api_trace_hsa_signal_xor_screlease_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__signal__handle,
	const int64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_xor_screlease_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_xor_screlease_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__signal__handle, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_signal_xor_screlease_end`) */
void barectf_hsa_api_trace_hsa_signal_xor_screlease_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_signal_xor_screlease_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_signal_xor_screlease_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_soft_queue_create_begin`) */
void barectf_hsa_api_trace_hsa_soft_queue_create_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__region__handle,
	const uint32_t p__size,
	const uint32_t p__type,
	const uint32_t p__features,
	const uint64_t p__doorbell_signal__handle,
	const uint64_t p__queue)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_soft_queue_create_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_soft_queue_create_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__region__handle, p__size, p__type, p__features, p__doorbell_signal__handle, p__queue);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_soft_queue_create_end`) */
void barectf_hsa_api_trace_hsa_soft_queue_create_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_soft_queue_create_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_soft_queue_create_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_status_string_begin`) */
void barectf_hsa_api_trace_hsa_status_string_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__status,
	const uint64_t p__status_string)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_status_string_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_status_string_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__status, p__status_string);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_status_string_end`) */
void barectf_hsa_api_trace_hsa_status_string_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_status_string_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_status_string_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_system_extension_supported_begin`) */
void barectf_hsa_api_trace_hsa_system_extension_supported_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint16_t p__version_major,
	const uint16_t p__version_minor,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_system_extension_supported_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_system_extension_supported_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__extension, p__version_major, p__version_minor, p__result);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_system_extension_supported_end`) */
void barectf_hsa_api_trace_hsa_system_extension_supported_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_system_extension_supported_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_system_extension_supported_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_system_get_extension_table_begin`) */
void barectf_hsa_api_trace_hsa_system_get_extension_table_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint16_t p__version_major,
	const uint16_t p__version_minor,
	const uint64_t p__table)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_system_get_extension_table_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_system_get_extension_table_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__extension, p__version_major, p__version_minor, p__table);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_system_get_extension_table_end`) */
void barectf_hsa_api_trace_hsa_system_get_extension_table_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_system_get_extension_table_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_system_get_extension_table_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_system_get_info_begin`) */
void barectf_hsa_api_trace_hsa_system_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_system_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_system_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__attribute, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_system_get_info_end`) */
void barectf_hsa_api_trace_hsa_system_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_system_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_system_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_system_get_major_extension_table_begin`) */
void barectf_hsa_api_trace_hsa_system_get_major_extension_table_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint16_t p__version_major,
	const uint64_t p__table_length,
	const uint64_t p__table)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_system_get_major_extension_table_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_system_get_major_extension_table_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__extension, p__version_major, p__table_length, p__table);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_system_get_major_extension_table_end`) */
void barectf_hsa_api_trace_hsa_system_get_major_extension_table_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_system_get_major_extension_table_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_system_get_major_extension_table_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_system_major_extension_supported_begin`) */
void barectf_hsa_api_trace_hsa_system_major_extension_supported_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint16_t p__extension,
	const uint16_t p__version_major,
	const uint64_t p__version_minor,
	const uint64_t p__result)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_system_major_extension_supported_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_system_major_extension_supported_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__extension, p__version_major, p__version_minor, p__result);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_system_major_extension_supported_end`) */
void barectf_hsa_api_trace_hsa_system_major_extension_supported_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_system_major_extension_supported_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_system_major_extension_supported_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_wavefront_get_info_begin`) */
void barectf_hsa_api_trace_hsa_wavefront_get_info_begin(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const uint64_t p__wavefront__handle,
	const int32_t p__attribute,
	const uint64_t p__value)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_wavefront_get_info_begin(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_wavefront_get_info_begin(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__wavefront__handle, p__attribute, p__value);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `hsa_api`, event record type `hsa_wavefront_get_info_end`) */
void barectf_hsa_api_trace_hsa_wavefront_get_info_end(struct barectf_hsa_api_ctx * const sctx,
	const uint32_t cc__thread_id,
	const uint32_t cc__queue_id,
	const uint32_t cc__agent_id,
	const uint64_t cc__correlation_id,
	const int32_t p__retval)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_api_hsa_wavefront_get_info_end(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_api_hsa_wavefront_get_info_end(_TO_VOID_PTR(ctx), cc__thread_id, cc__queue_id, cc__agent_id, cc__correlation_id, p__retval);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}
/* Open packet for data stream type `hsa_handles` */
void barectf_hsa_handles_open_packet(
	struct barectf_hsa_handles_ctx * const sctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	const int saved_in_tracing_section = ctx->in_tracing_section;

	/*
	 * This function is either called by a tracing function, or
	 * directly by the platform.
	 *
	 * If it's called by a tracing function, then
	 * `ctx->in_tracing_section` is 1, so it's safe to open
	 * the packet here (alter the packet), even if tracing was
	 * disabled in the meantime because we're already in a tracing
	 * section (which finishes at the end of the tracing function
	 * call).
	 *
	 * If it's called directly by the platform, then if tracing is
	 * disabled, we don't want to alter the packet, and return
	 * immediately.
	 */
	if (!ctx->is_tracing_enabled && !saved_in_tracing_section) {
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Do not open a packet that is already open */
	if (ctx->packet_is_open) {
		ctx->in_tracing_section = saved_in_tracing_section;
		goto end;
	}

	ctx->at = 0;

	/* Write packet header structure */
	{
		/* Align for packet header structure */
		_ALIGN(ctx->at, 8);

		/* Align for `magic` field */
		_ALIGN(ctx->at, 8);

		/* Write magic number field */
		{
			const uint32_t tmp_val = (uint32_t) 0xc1fc1fc1UL;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `stream_id` field */
		_ALIGN(ctx->at, 8);

		/* Write data stream type ID field */
		{
			const uint64_t tmp_val = (uint64_t) 3;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}

	/* Write packet context structure */
	{
		/* Align for packet context structure */
		_ALIGN(ctx->at, 8);

		/* Align for `packet_size` field */
		_ALIGN(ctx->at, 8);

		/* Write packet total size field */
		{
			const uint64_t tmp_val = (uint64_t) ctx->packet_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `content_size` field */
		_ALIGN(ctx->at, 8);

		/* Do not write `content_size` field; save its offset */
		sctx->off_pc_content_size = ctx->at;
		ctx->at += 64;
	}

	/* Save content beginning's offset */
	ctx->off_content = ctx->at;

	/* Mark current packet as open */
	ctx->packet_is_open = 1;

	/* Not tracing anymore */
	ctx->in_tracing_section = saved_in_tracing_section;

end:
	return;
}

/* Close packet for data stream type `hsa_handles` */
void barectf_hsa_handles_close_packet(struct barectf_hsa_handles_ctx * const sctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	const int saved_in_tracing_section = ctx->in_tracing_section;

	/*
	 * This function is either called by a tracing function, or
	 * directly by the platform.
	 *
	 * If it's called by a tracing function, then
	 * `ctx->in_tracing_section` is 1, so it's safe to close
	 * the packet here (alter the packet), even if tracing was
	 * disabled in the meantime, because we're already in a tracing
	 * section (which finishes at the end of the tracing function
	 * call).
	 *
	 * If it's called directly by the platform, then if tracing is
	 * disabled, we don't want to alter the packet, and return
	 * immediately.
	 */
	if (!ctx->is_tracing_enabled && !saved_in_tracing_section) {
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Do not close a packet that is not open */
	if (!ctx->packet_is_open) {
		ctx->in_tracing_section = saved_in_tracing_section;
		goto end;
	}

	/* Save content size */
	ctx->content_size = ctx->at;

	/* Go back to `content_size` field offset */
	ctx->at = sctx->off_pc_content_size;

	/* Write `content_size` field */
	{
		const uint64_t tmp_val = (uint64_t) ctx->content_size;

		memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
		ctx->at += 64;
	}

	/* Go back to end of packet */
	ctx->at = ctx->packet_size;

	/* Mark packet as closed */
	ctx->packet_is_open = 0;

	/* Not tracing anymore */
	ctx->in_tracing_section = saved_in_tracing_section;

end:
	return;
}

static void _serialize_er_header_hsa_handles(void * const vctx,
	const uint32_t ert_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	struct barectf_hsa_handles_ctx * const sctx = _FROM_VOID_PTR(struct barectf_hsa_handles_ctx, vctx);
	const uint64_t ts = sctx->cur_last_event_ts;

	/* Write header structure */
	{
		/* Align for header structure */
		_ALIGN(ctx->at, 8);

		/* Align for `id` field */
		_ALIGN(ctx->at, 8);

		/* Write event record type ID field */
		{
			const uint64_t tmp_val = (uint64_t) ert_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `timestamp` field */
		_ALIGN(ctx->at, 8);

		/* Write timestamp field */
		{
			const uint64_t tmp_val = (uint64_t) ts;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_hsa_handles_hsa_handle_type(void * const vctx,
	const uint64_t p__handle,
	const uint8_t p__type)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_hsa_handles(ctx, 0);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 64);

		/* Align for `_handle` field */
		_ALIGN(ctx->at, 64);

		/* Write `_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_type` field */
		_ALIGN(ctx->at, 8);

		/* Write `_type` field */
		{
			const uint8_t tmp_val = (uint8_t) p__type;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 8;
		}
	}
}

static uint32_t _er_size_hsa_handles_hsa_handle_type(void * const vctx)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 64);

		/* Align for `_handle` field */
		_ALIGN(at, 64);

		/* Add `_handle` bit array field's size */
		at += 64;

		/* Align for `_type` field */
		_ALIGN(at, 8);

		/* Add `_type` bit array field's size */
		at += 8;
	}

	return at - ctx->at;
}

/* Trace (data stream type `hsa_handles`, event record type `hsa_handle_type`) */
void barectf_hsa_handles_trace_hsa_handle_type(struct barectf_hsa_handles_ctx * const sctx,
	const uint64_t p__handle,
	const uint8_t p__type)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_hsa_handles_hsa_handle_type(_TO_VOID_PTR(ctx));

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_hsa_handles_hsa_handle_type(_TO_VOID_PTR(ctx), p__handle, p__type);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}
/* Open packet for data stream type `profiler` */
void barectf_profiler_open_packet(
	struct barectf_profiler_ctx * const sctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	const int saved_in_tracing_section = ctx->in_tracing_section;

	/*
	 * This function is either called by a tracing function, or
	 * directly by the platform.
	 *
	 * If it's called by a tracing function, then
	 * `ctx->in_tracing_section` is 1, so it's safe to open
	 * the packet here (alter the packet), even if tracing was
	 * disabled in the meantime because we're already in a tracing
	 * section (which finishes at the end of the tracing function
	 * call).
	 *
	 * If it's called directly by the platform, then if tracing is
	 * disabled, we don't want to alter the packet, and return
	 * immediately.
	 */
	if (!ctx->is_tracing_enabled && !saved_in_tracing_section) {
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Do not open a packet that is already open */
	if (ctx->packet_is_open) {
		ctx->in_tracing_section = saved_in_tracing_section;
		goto end;
	}

	ctx->at = 0;

	/* Write packet header structure */
	{
		/* Align for packet header structure */
		_ALIGN(ctx->at, 8);

		/* Align for `magic` field */
		_ALIGN(ctx->at, 8);

		/* Write magic number field */
		{
			const uint32_t tmp_val = (uint32_t) 0xc1fc1fc1UL;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `stream_id` field */
		_ALIGN(ctx->at, 8);

		/* Write data stream type ID field */
		{
			const uint64_t tmp_val = (uint64_t) 4;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}

	/* Write packet context structure */
	{
		/* Align for packet context structure */
		_ALIGN(ctx->at, 8);

		/* Align for `packet_size` field */
		_ALIGN(ctx->at, 8);

		/* Write packet total size field */
		{
			const uint64_t tmp_val = (uint64_t) ctx->packet_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `content_size` field */
		_ALIGN(ctx->at, 8);

		/* Do not write `content_size` field; save its offset */
		sctx->off_pc_content_size = ctx->at;
		ctx->at += 64;
	}

	/* Save content beginning's offset */
	ctx->off_content = ctx->at;

	/* Mark current packet as open */
	ctx->packet_is_open = 1;

	/* Not tracing anymore */
	ctx->in_tracing_section = saved_in_tracing_section;

end:
	return;
}

/* Close packet for data stream type `profiler` */
void barectf_profiler_close_packet(struct barectf_profiler_ctx * const sctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	const int saved_in_tracing_section = ctx->in_tracing_section;

	/*
	 * This function is either called by a tracing function, or
	 * directly by the platform.
	 *
	 * If it's called by a tracing function, then
	 * `ctx->in_tracing_section` is 1, so it's safe to close
	 * the packet here (alter the packet), even if tracing was
	 * disabled in the meantime, because we're already in a tracing
	 * section (which finishes at the end of the tracing function
	 * call).
	 *
	 * If it's called directly by the platform, then if tracing is
	 * disabled, we don't want to alter the packet, and return
	 * immediately.
	 */
	if (!ctx->is_tracing_enabled && !saved_in_tracing_section) {
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Do not close a packet that is not open */
	if (!ctx->packet_is_open) {
		ctx->in_tracing_section = saved_in_tracing_section;
		goto end;
	}

	/* Save content size */
	ctx->content_size = ctx->at;

	/* Go back to `content_size` field offset */
	ctx->at = sctx->off_pc_content_size;

	/* Write `content_size` field */
	{
		const uint64_t tmp_val = (uint64_t) ctx->content_size;

		memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
		ctx->at += 64;
	}

	/* Go back to end of packet */
	ctx->at = ctx->packet_size;

	/* Mark packet as closed */
	ctx->packet_is_open = 0;

	/* Not tracing anymore */
	ctx->in_tracing_section = saved_in_tracing_section;

end:
	return;
}

static void _serialize_er_header_profiler(void * const vctx,
	const uint32_t ert_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	struct barectf_profiler_ctx * const sctx = _FROM_VOID_PTR(struct barectf_profiler_ctx, vctx);
	const uint64_t ts = sctx->cur_last_event_ts;

	/* Write header structure */
	{
		/* Align for header structure */
		_ALIGN(ctx->at, 8);

		/* Align for `id` field */
		_ALIGN(ctx->at, 8);

		/* Write event record type ID field */
		{
			const uint64_t tmp_val = (uint64_t) ert_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `timestamp` field */
		_ALIGN(ctx->at, 8);

		/* Write timestamp field */
		{
			const uint64_t tmp_val = (uint64_t) ts;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_common_ctx_profiler(void * const vctx,
	const uint64_t cc__dispatch,
	const uint64_t cc__gpu_id,
	const uint64_t cc__queue_id,
	const uint64_t cc__queue_index,
	const uint32_t cc__process_id,
	const uint32_t cc__thread_id,
	const uint64_t cc__kernel_id,
	const char * const cc__kernel_name,
	const uint32_t cc____counter_names_len,
	const char * const * const cc__counter_names,
	const uint32_t cc____counter_values_len,
	const uint64_t * const cc__counter_values)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Write common context structure */
	{
		/* Align for common context structure */
		_ALIGN(ctx->at, 64);

		/* Align for `_dispatch` field */
		_ALIGN(ctx->at, 64);

		/* Write `_dispatch` field */
		{
			const uint64_t tmp_val = (uint64_t) cc__dispatch;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_gpu_id` field */
		_ALIGN(ctx->at, 64);

		/* Write `_gpu_id` field */
		{
			const uint64_t tmp_val = (uint64_t) cc__gpu_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_queue_id` field */
		_ALIGN(ctx->at, 64);

		/* Write `_queue_id` field */
		{
			const uint64_t tmp_val = (uint64_t) cc__queue_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_queue_index` field */
		_ALIGN(ctx->at, 64);

		/* Write `_queue_index` field */
		{
			const uint64_t tmp_val = (uint64_t) cc__queue_index;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_process_id` field */
		_ALIGN(ctx->at, 32);

		/* Write `_process_id` field */
		{
			const uint32_t tmp_val = (uint32_t) cc__process_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_thread_id` field */
		_ALIGN(ctx->at, 32);

		/* Write `_thread_id` field */
		{
			const uint32_t tmp_val = (uint32_t) cc__thread_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_kernel_id` field */
		_ALIGN(ctx->at, 64);

		/* Write `_kernel_id` field */
		{
			const uint64_t tmp_val = (uint64_t) cc__kernel_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_kernel_name` field */
		_ALIGN(ctx->at, 8);

		/* Write `_kernel_name` field */
		_write_c_str(ctx, cc__kernel_name);

		/* Align for `___counter_names_len` field */
		_ALIGN(ctx->at, 8);

		/* Write `___counter_names_len` field */
		{
			const uint32_t tmp_val = (uint32_t) cc____counter_names_len;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_counter_names` field */
		_ALIGN(ctx->at, 8);

		/* Write `_counter_names` field */
		{
			uint32_t i;

			for (i = 0; i < (uint32_t) cc____counter_names_len; ++i) {
				/* Align for `[i]` field */
				_ALIGN(ctx->at, 8);

				/* Write `[i]` field */
				_write_c_str(ctx, cc__counter_names[i]);
			}
		}

		/* Align for `___counter_values_len` field */
		_ALIGN(ctx->at, 8);

		/* Write `___counter_values_len` field */
		{
			const uint32_t tmp_val = (uint32_t) cc____counter_values_len;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `_counter_values` field */
		_ALIGN(ctx->at, 64);

		/* Write `_counter_values` field */
		{
			uint32_t i;

			for (i = 0; i < (uint32_t) cc____counter_values_len; ++i) {
				/* Align for `[i]` field */
				_ALIGN(ctx->at, 64);

				/* Write `[i]` field */
				{
					const uint64_t tmp_val = (uint64_t) cc__counter_values[i];

					memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
					ctx->at += 64;
				}
			}
		}
	}
}

static void _serialize_er_profiler_profiler_record(void * const vctx,
	const uint64_t cc__dispatch,
	const uint64_t cc__gpu_id,
	const uint64_t cc__queue_id,
	const uint64_t cc__queue_index,
	const uint32_t cc__process_id,
	const uint32_t cc__thread_id,
	const uint64_t cc__kernel_id,
	const char * const cc__kernel_name,
	const uint32_t cc____counter_names_len,
	const char * const * const cc__counter_names,
	const uint32_t cc____counter_values_len,
	const uint64_t * const cc__counter_values)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_profiler(ctx, 0);

	/* Serialize common context */
	_serialize_er_common_ctx_profiler(ctx, cc__dispatch, cc__gpu_id, cc__queue_id, cc__queue_index, cc__process_id, cc__thread_id, cc__kernel_id, cc__kernel_name, cc____counter_names_len, cc__counter_names, cc____counter_values_len, cc__counter_values);

	/* Write payload structure */
	{
	}
}

static void _serialize_er_profiler_profiler_record_with_kernel_properties(void * const vctx,
	const uint64_t cc__dispatch,
	const uint64_t cc__gpu_id,
	const uint64_t cc__queue_id,
	const uint64_t cc__queue_index,
	const uint32_t cc__process_id,
	const uint32_t cc__thread_id,
	const uint64_t cc__kernel_id,
	const char * const cc__kernel_name,
	const uint32_t cc____counter_names_len,
	const char * const * const cc__counter_names,
	const uint32_t cc____counter_values_len,
	const uint64_t * const cc__counter_values,
	const uint64_t p__grid_size,
	const uint64_t p__workgroup_size,
	const uint64_t p__lds_size,
	const uint64_t p__scratch_size,
	const uint64_t p__arch_vgpr_count,
	const uint64_t p__accum_vgpr_count,
	const uint64_t p__sgpr_count,
	const uint64_t p__wave_size,
	const uint64_t p__signal_handle)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_profiler(ctx, 1);

	/* Serialize common context */
	_serialize_er_common_ctx_profiler(ctx, cc__dispatch, cc__gpu_id, cc__queue_id, cc__queue_index, cc__process_id, cc__thread_id, cc__kernel_id, cc__kernel_name, cc____counter_names_len, cc__counter_names, cc____counter_values_len, cc__counter_values);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 64);

		/* Align for `_grid_size` field */
		_ALIGN(ctx->at, 64);

		/* Write `_grid_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__grid_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_workgroup_size` field */
		_ALIGN(ctx->at, 64);

		/* Write `_workgroup_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__workgroup_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_lds_size` field */
		_ALIGN(ctx->at, 64);

		/* Write `_lds_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__lds_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_scratch_size` field */
		_ALIGN(ctx->at, 64);

		/* Write `_scratch_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__scratch_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_arch_vgpr_count` field */
		_ALIGN(ctx->at, 64);

		/* Write `_arch_vgpr_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__arch_vgpr_count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_accum_vgpr_count` field */
		_ALIGN(ctx->at, 64);

		/* Write `_accum_vgpr_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__accum_vgpr_count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_sgpr_count` field */
		_ALIGN(ctx->at, 64);

		/* Write `_sgpr_count` field */
		{
			const uint64_t tmp_val = (uint64_t) p__sgpr_count;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_wave_size` field */
		_ALIGN(ctx->at, 64);

		/* Write `_wave_size` field */
		{
			const uint64_t tmp_val = (uint64_t) p__wave_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_signal_handle` field */
		_ALIGN(ctx->at, 64);

		/* Write `_signal_handle` field */
		{
			const uint64_t tmp_val = (uint64_t) p__signal_handle;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static uint32_t _er_size_profiler_profiler_record(void * const vctx,
	const char * const cc__kernel_name,
	const uint32_t cc____counter_names_len,
	const char * const * const cc__counter_names,
	const uint32_t cc____counter_values_len,
	const uint64_t * const cc__counter_values)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_dispatch` field */
		_ALIGN(at, 64);

		/* Add `_dispatch` bit array field's size */
		at += 64;

		/* Align for `_gpu_id` field */
		_ALIGN(at, 64);

		/* Add `_gpu_id` bit array field's size */
		at += 64;

		/* Align for `_queue_id` field */
		_ALIGN(at, 64);

		/* Add `_queue_id` bit array field's size */
		at += 64;

		/* Align for `_queue_index` field */
		_ALIGN(at, 64);

		/* Add `_queue_index` bit array field's size */
		at += 64;

		/* Align for `_process_id` field */
		_ALIGN(at, 32);

		/* Add `_process_id` bit array field's size */
		at += 32;

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_kernel_id` field */
		_ALIGN(at, 64);

		/* Add `_kernel_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);

		/* Align for `___counter_names_len` field */
		_ALIGN(at, 8);

		/* Add `___counter_names_len` bit array field's size */
		at += 32;

		/* Align for `_counter_names` field */
		_ALIGN(at, 8);

		/* Write `_counter_names` field */
		{
			uint32_t i;

			for (i = 0; i < (uint32_t) cc____counter_names_len; ++i) {
				/* Align for `[i]` field */
				_ALIGN(at, 8);

				/* Add `[i]` string field's size */
				at += _BYTES_TO_BITS(strlen(cc__counter_names[i]) + 1);
			}
		}

		/* Align for `___counter_values_len` field */
		_ALIGN(at, 8);

		/* Add `___counter_values_len` bit array field's size */
		at += 32;

		/* Align for `_counter_values` field */
		_ALIGN(at, 64);

		/* Write `_counter_values` field */
		{
			uint32_t i;

			for (i = 0; i < (uint32_t) cc____counter_values_len; ++i) {
				/* Align for `[i]` field */
				_ALIGN(at, 64);

				/* Add `[i]` bit array field's size */
				at += 64;
			}
		}
	}

	/* Add payload structure size */
	{
	}

	return at - ctx->at;
}

static uint32_t _er_size_profiler_profiler_record_with_kernel_properties(void * const vctx,
	const char * const cc__kernel_name,
	const uint32_t cc____counter_names_len,
	const char * const * const cc__counter_names,
	const uint32_t cc____counter_values_len,
	const uint64_t * const cc__counter_values)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 64);

		/* Align for `_dispatch` field */
		_ALIGN(at, 64);

		/* Add `_dispatch` bit array field's size */
		at += 64;

		/* Align for `_gpu_id` field */
		_ALIGN(at, 64);

		/* Add `_gpu_id` bit array field's size */
		at += 64;

		/* Align for `_queue_id` field */
		_ALIGN(at, 64);

		/* Add `_queue_id` bit array field's size */
		at += 64;

		/* Align for `_queue_index` field */
		_ALIGN(at, 64);

		/* Add `_queue_index` bit array field's size */
		at += 64;

		/* Align for `_process_id` field */
		_ALIGN(at, 32);

		/* Add `_process_id` bit array field's size */
		at += 32;

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;

		/* Align for `_kernel_id` field */
		_ALIGN(at, 64);

		/* Add `_kernel_id` bit array field's size */
		at += 64;

		/* Align for `_kernel_name` field */
		_ALIGN(at, 8);

		/* Add `_kernel_name` string field's size */
		at += _BYTES_TO_BITS(strlen(cc__kernel_name) + 1);

		/* Align for `___counter_names_len` field */
		_ALIGN(at, 8);

		/* Add `___counter_names_len` bit array field's size */
		at += 32;

		/* Align for `_counter_names` field */
		_ALIGN(at, 8);

		/* Write `_counter_names` field */
		{
			uint32_t i;

			for (i = 0; i < (uint32_t) cc____counter_names_len; ++i) {
				/* Align for `[i]` field */
				_ALIGN(at, 8);

				/* Add `[i]` string field's size */
				at += _BYTES_TO_BITS(strlen(cc__counter_names[i]) + 1);
			}
		}

		/* Align for `___counter_values_len` field */
		_ALIGN(at, 8);

		/* Add `___counter_values_len` bit array field's size */
		at += 32;

		/* Align for `_counter_values` field */
		_ALIGN(at, 64);

		/* Write `_counter_values` field */
		{
			uint32_t i;

			for (i = 0; i < (uint32_t) cc____counter_values_len; ++i) {
				/* Align for `[i]` field */
				_ALIGN(at, 64);

				/* Add `[i]` bit array field's size */
				at += 64;
			}
		}
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 64);

		/* Align for `_grid_size` field */
		_ALIGN(at, 64);

		/* Add `_grid_size` bit array field's size */
		at += 64;

		/* Align for `_workgroup_size` field */
		_ALIGN(at, 64);

		/* Add `_workgroup_size` bit array field's size */
		at += 64;

		/* Align for `_lds_size` field */
		_ALIGN(at, 64);

		/* Add `_lds_size` bit array field's size */
		at += 64;

		/* Align for `_scratch_size` field */
		_ALIGN(at, 64);

		/* Add `_scratch_size` bit array field's size */
		at += 64;

		/* Align for `_arch_vgpr_count` field */
		_ALIGN(at, 64);

		/* Add `_arch_vgpr_count` bit array field's size */
		at += 64;

		/* Align for `_accum_vgpr_count` field */
		_ALIGN(at, 64);

		/* Add `_accum_vgpr_count` bit array field's size */
		at += 64;

		/* Align for `_sgpr_count` field */
		_ALIGN(at, 64);

		/* Add `_sgpr_count` bit array field's size */
		at += 64;

		/* Align for `_wave_size` field */
		_ALIGN(at, 64);

		/* Add `_wave_size` bit array field's size */
		at += 64;

		/* Align for `_signal_handle` field */
		_ALIGN(at, 64);

		/* Add `_signal_handle` bit array field's size */
		at += 64;
	}

	return at - ctx->at;
}

/* Trace (data stream type `profiler`, event record type `profiler_record`) */
void barectf_profiler_trace_profiler_record(struct barectf_profiler_ctx * const sctx,
	const uint64_t cc__dispatch,
	const uint64_t cc__gpu_id,
	const uint64_t cc__queue_id,
	const uint64_t cc__queue_index,
	const uint32_t cc__process_id,
	const uint32_t cc__thread_id,
	const uint64_t cc__kernel_id,
	const char * const cc__kernel_name,
	const uint32_t cc____counter_names_len,
	const char * const * const cc__counter_names,
	const uint32_t cc____counter_values_len,
	const uint64_t * const cc__counter_values)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_profiler_profiler_record(_TO_VOID_PTR(ctx), cc__kernel_name, cc____counter_names_len, cc__counter_names, cc____counter_values_len, cc__counter_values);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_profiler_profiler_record(_TO_VOID_PTR(ctx), cc__dispatch, cc__gpu_id, cc__queue_id, cc__queue_index, cc__process_id, cc__thread_id, cc__kernel_id, cc__kernel_name, cc____counter_names_len, cc__counter_names, cc____counter_values_len, cc__counter_values);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}

/* Trace (data stream type `profiler`, event record type `profiler_record_with_kernel_properties`) */
void barectf_profiler_trace_profiler_record_with_kernel_properties(struct barectf_profiler_ctx * const sctx,
	const uint64_t cc__dispatch,
	const uint64_t cc__gpu_id,
	const uint64_t cc__queue_id,
	const uint64_t cc__queue_index,
	const uint32_t cc__process_id,
	const uint32_t cc__thread_id,
	const uint64_t cc__kernel_id,
	const char * const cc__kernel_name,
	const uint32_t cc____counter_names_len,
	const char * const * const cc__counter_names,
	const uint32_t cc____counter_values_len,
	const uint64_t * const cc__counter_values,
	const uint64_t p__grid_size,
	const uint64_t p__workgroup_size,
	const uint64_t p__lds_size,
	const uint64_t p__scratch_size,
	const uint64_t p__arch_vgpr_count,
	const uint64_t p__accum_vgpr_count,
	const uint64_t p__sgpr_count,
	const uint64_t p__wave_size,
	const uint64_t p__signal_handle)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_profiler_profiler_record_with_kernel_properties(_TO_VOID_PTR(ctx), cc__kernel_name, cc____counter_names_len, cc__counter_names, cc____counter_values_len, cc__counter_values);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_profiler_profiler_record_with_kernel_properties(_TO_VOID_PTR(ctx), cc__dispatch, cc__gpu_id, cc__queue_id, cc__queue_index, cc__process_id, cc__thread_id, cc__kernel_id, cc__kernel_name, cc____counter_names_len, cc__counter_names, cc____counter_values_len, cc__counter_values, p__grid_size, p__workgroup_size, p__lds_size, p__scratch_size, p__arch_vgpr_count, p__accum_vgpr_count, p__sgpr_count, p__wave_size, p__signal_handle);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}
/* Open packet for data stream type `roctx` */
void barectf_roctx_open_packet(
	struct barectf_roctx_ctx * const sctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	const int saved_in_tracing_section = ctx->in_tracing_section;

	/*
	 * This function is either called by a tracing function, or
	 * directly by the platform.
	 *
	 * If it's called by a tracing function, then
	 * `ctx->in_tracing_section` is 1, so it's safe to open
	 * the packet here (alter the packet), even if tracing was
	 * disabled in the meantime because we're already in a tracing
	 * section (which finishes at the end of the tracing function
	 * call).
	 *
	 * If it's called directly by the platform, then if tracing is
	 * disabled, we don't want to alter the packet, and return
	 * immediately.
	 */
	if (!ctx->is_tracing_enabled && !saved_in_tracing_section) {
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Do not open a packet that is already open */
	if (ctx->packet_is_open) {
		ctx->in_tracing_section = saved_in_tracing_section;
		goto end;
	}

	ctx->at = 0;

	/* Write packet header structure */
	{
		/* Align for packet header structure */
		_ALIGN(ctx->at, 8);

		/* Align for `magic` field */
		_ALIGN(ctx->at, 8);

		/* Write magic number field */
		{
			const uint32_t tmp_val = (uint32_t) 0xc1fc1fc1UL;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}

		/* Align for `stream_id` field */
		_ALIGN(ctx->at, 8);

		/* Write data stream type ID field */
		{
			const uint64_t tmp_val = (uint64_t) 5;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}

	/* Write packet context structure */
	{
		/* Align for packet context structure */
		_ALIGN(ctx->at, 8);

		/* Align for `packet_size` field */
		_ALIGN(ctx->at, 8);

		/* Write packet total size field */
		{
			const uint64_t tmp_val = (uint64_t) ctx->packet_size;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `content_size` field */
		_ALIGN(ctx->at, 8);

		/* Do not write `content_size` field; save its offset */
		sctx->off_pc_content_size = ctx->at;
		ctx->at += 64;
	}

	/* Save content beginning's offset */
	ctx->off_content = ctx->at;

	/* Mark current packet as open */
	ctx->packet_is_open = 1;

	/* Not tracing anymore */
	ctx->in_tracing_section = saved_in_tracing_section;

end:
	return;
}

/* Close packet for data stream type `roctx` */
void barectf_roctx_close_packet(struct barectf_roctx_ctx * const sctx)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	const int saved_in_tracing_section = ctx->in_tracing_section;

	/*
	 * This function is either called by a tracing function, or
	 * directly by the platform.
	 *
	 * If it's called by a tracing function, then
	 * `ctx->in_tracing_section` is 1, so it's safe to close
	 * the packet here (alter the packet), even if tracing was
	 * disabled in the meantime, because we're already in a tracing
	 * section (which finishes at the end of the tracing function
	 * call).
	 *
	 * If it's called directly by the platform, then if tracing is
	 * disabled, we don't want to alter the packet, and return
	 * immediately.
	 */
	if (!ctx->is_tracing_enabled && !saved_in_tracing_section) {
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Do not close a packet that is not open */
	if (!ctx->packet_is_open) {
		ctx->in_tracing_section = saved_in_tracing_section;
		goto end;
	}

	/* Save content size */
	ctx->content_size = ctx->at;

	/* Go back to `content_size` field offset */
	ctx->at = sctx->off_pc_content_size;

	/* Write `content_size` field */
	{
		const uint64_t tmp_val = (uint64_t) ctx->content_size;

		memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
		ctx->at += 64;
	}

	/* Go back to end of packet */
	ctx->at = ctx->packet_size;

	/* Mark packet as closed */
	ctx->packet_is_open = 0;

	/* Not tracing anymore */
	ctx->in_tracing_section = saved_in_tracing_section;

end:
	return;
}

static void _serialize_er_header_roctx(void * const vctx,
	const uint32_t ert_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	struct barectf_roctx_ctx * const sctx = _FROM_VOID_PTR(struct barectf_roctx_ctx, vctx);
	const uint64_t ts = sctx->cur_last_event_ts;

	/* Write header structure */
	{
		/* Align for header structure */
		_ALIGN(ctx->at, 8);

		/* Align for `id` field */
		_ALIGN(ctx->at, 8);

		/* Write event record type ID field */
		{
			const uint64_t tmp_val = (uint64_t) ert_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `timestamp` field */
		_ALIGN(ctx->at, 8);

		/* Write timestamp field */
		{
			const uint64_t tmp_val = (uint64_t) ts;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}
	}
}

static void _serialize_er_common_ctx_roctx(void * const vctx,
	const uint32_t cc__thread_id)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Write common context structure */
	{
		/* Align for common context structure */
		_ALIGN(ctx->at, 32);

		/* Align for `_thread_id` field */
		_ALIGN(ctx->at, 32);

		/* Write `_thread_id` field */
		{
			const uint32_t tmp_val = (uint32_t) cc__thread_id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 32;
		}
	}
}

static void _serialize_er_roctx_roctx(void * const vctx,
	const uint32_t cc__thread_id,
	const int64_t p__id,
	const char * const p__msg)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);

	/* Serialize header */
	_serialize_er_header_roctx(ctx, 0);

	/* Serialize common context */
	_serialize_er_common_ctx_roctx(ctx, cc__thread_id);

	/* Write payload structure */
	{
		/* Align for payload structure */
		_ALIGN(ctx->at, 64);

		/* Align for `_id` field */
		_ALIGN(ctx->at, 64);

		/* Write `_id` field */
		{
			const uint64_t tmp_val = (uint64_t) p__id;

			memcpy(&ctx->buf[_BITS_TO_BYTES(ctx->at)], &tmp_val, sizeof(tmp_val));
			ctx->at += 64;
		}

		/* Align for `_msg` field */
		_ALIGN(ctx->at, 8);

		/* Write `_msg` field */
		_write_c_str(ctx, p__msg);
	}
}

static uint32_t _er_size_roctx_roctx(void * const vctx,
	const char * const p__msg)
{
	struct barectf_ctx * const ctx = _FROM_VOID_PTR(struct barectf_ctx, vctx);
	uint32_t at = ctx->at;

	/* Add header structure size */
	{
		/* Align for header structure */
		_ALIGN(at, 8);

		/* Align for `id` field */
		_ALIGN(at, 8);

		/* Add `id` bit array field's size */
		at += 64;

		/* Align for `timestamp` field */
		_ALIGN(at, 8);

		/* Add `timestamp` bit array field's size */
		at += 64;
	}

	/* Add common context structure size */
	{
		/* Align for common context structure */
		_ALIGN(at, 32);

		/* Align for `_thread_id` field */
		_ALIGN(at, 32);

		/* Add `_thread_id` bit array field's size */
		at += 32;
	}

	/* Add payload structure size */
	{
		/* Align for payload structure */
		_ALIGN(at, 64);

		/* Align for `_id` field */
		_ALIGN(at, 64);

		/* Add `_id` bit array field's size */
		at += 64;

		/* Align for `_msg` field */
		_ALIGN(at, 8);

		/* Add `_msg` string field's size */
		at += _BYTES_TO_BITS(strlen(p__msg) + 1);
	}

	return at - ctx->at;
}

/* Trace (data stream type `roctx`, event record type `roctx`) */
void barectf_roctx_trace_roctx(struct barectf_roctx_ctx * const sctx,
	const uint32_t cc__thread_id,
	const int64_t p__id,
	const char * const p__msg)
{
	struct barectf_ctx * const ctx = &sctx->parent;
	uint32_t er_size;

	/* Save timestamp */
	sctx->cur_last_event_ts = ctx->cbs.default_clock_get_value(ctx->data);

	if (!ctx->is_tracing_enabled) {
		goto end;
	}

	/* We can alter the packet */
	ctx->in_tracing_section = 1;

	/* Compute event record size */
	er_size = _er_size_roctx_roctx(_TO_VOID_PTR(ctx), p__msg);

	/* Is there enough space to serialize? */
	if (!_reserve_er_space(_TO_VOID_PTR(ctx), er_size)) {
		/* no: forget this */
		ctx->in_tracing_section = 0;
		goto end;
	}

	/* Serialize event record */
	_serialize_er_roctx_roctx(_TO_VOID_PTR(ctx), cc__thread_id, p__id, p__msg);

	/* Commit event record */
	_commit_er(_TO_VOID_PTR(ctx));

	/* Not tracing anymore */
	ctx->in_tracing_section = 0;

end:
	return;
}
